/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// to do :
//   * java to java mode
//   * host id/address
//   * to connections in parallel
//   * stop testtalk

package com.radview.util.testtalk;

import java.util.Hashtable;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Enumeration;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

// radview classes
import com.radview.util.gui.OKDialog;
//import com.radview.util.general.DebugMessages;

// TestTalkC is the TestTalk engine.
// It may used once in JVM. The Caller must create only one object
// of this class. The class must be passed to each object of
// TestTalkClientC.
// This class in common to all clients in one JVM.
public class TestTalkC extends Thread {

    private int UserLocalPort;    // the local port asked by the user
    private int LocalPort;        // the port used by local testtalk
    private String LocalHost= ""; // local host : saved once for all requests
    private TestTalkClientC TestTalkInternalClient;

    private Hashtable LocalTestTalkConnectedClients = new Hashtable();
                        // dictionary which contains all connected
                        // clients to the current TestTalk
    private Hashtable ConnectedTestTalks = new Hashtable();
                        // dictionary which contains the host and port
                        // of all connected TestTalks to the current
                        // TestTalk.

    private ServerSocket Soc; // The ServerSocket of the local testtalk

		private Log log = LogFactory.getLog(this.getClass());	
	
    // Ctor for using when creating TestTalk object which listen
    // to the default port.
    public TestTalkC(boolean DebugMode)
                    throws ErrorLocatingLocalHostAddressException,
                           CouldNotListenOnPortException {
        TestTalkCInit(TestTalkConstants.DEFAULT_PORT,true,DebugMode,20000);
    }

    // Ctor for using when want to listen to different port then the
    // default, or using TestTalk for communication with client only in
    // current JVM.
    public TestTalkC(int     Port,
                     boolean CommunicationWithExternalClients,
                     boolean DebugMode,
                     int     SendingTimeout)
                    throws ErrorLocatingLocalHostAddressException,
                           CouldNotListenOnPortException {
        TestTalkCInit(Port,
                      CommunicationWithExternalClients,
                      DebugMode,
                      SendingTimeout);
    }

    // init which is used only by the Ctors
    private void TestTalkCInit(
                  int         port,
                  boolean     CommunicationWithExternalClients,
                  boolean     DebugMode,
                  int         SendingTimeout)
                    throws ErrorLocatingLocalHostAddressException,
                           CouldNotListenOnPortException {

        // saving variables
        UserLocalPort = port;
        TestTalkConstants.DEBUG_MODE = DebugMode;
        TestTalkConstants.TIMEOUT_IN_MILISECONDS = SendingTimeout;
/*	try {
          DebugMessages.SetLogFile("c:\\log.txt");
	}
	catch (Exception e) {
	  e.printStackTrace();
	}
*/
        // saving local host
        try {
            InetAddress LocalInetAddress = InetAddress.getLocalHost();
//            LocalHost = LocalInetAddress.getHostName();

            byte[] IPArray = LocalInetAddress.getAddress();
            for (int i = 0; i<4; i++) {
                LocalHost = LocalHost +
                            (short)((short)IPArray[i] & (short)0x00ff) + ".";
            }
            LocalHost = LocalHost.substring(0,LocalHost.length()-1);

            TestTalkInternalClient = new TestTalkClientC(this);
            TestTalkInternalClient.NetInit("TestTalkInternalClient");
            TestTalkInternalClient.start();
        }
        catch (Exception e) {  //Throwable
            throw new ErrorLocatingLocalHostAddressException("" + e);
        }
        // if the user want to communicate with different TestTalk has
        // to start the Thread which listen to the port

        try {
            Soc = new ServerSocket(UserLocalPort);
        }
        catch (Exception e) {
            new CouldNotListenOnPortException("Port : " + UserLocalPort + " " + e);
        }

        // when the user asked for port 0, the system can fetch any
        // available port. Therefore, has to save the local port after
        // creating the socket.
        if (UserLocalPort == 0)
            LocalPort = Soc.getLocalPort();
        else
            LocalPort = UserLocalPort;

        if (CommunicationWithExternalClients) start();

/*
	    // temporary sleep to correct bug.
	    try {
	        (Thread.currentThread()).sleep(1000);
	    }
	    catch (InterruptedException e) {}
*/
    }

    // init method which called once by each client. The Client which
    // communicate with other client must use the NetInit method
    protected void NetInit(String Name) {
        // creates clients object and register in
        // LocalTestTalkConnectedClients.
        ClientC Client = new ClientC();
        RegisterClient(Name,Client);
    }

    // Terminate method for using when client wants to stop
    // communicating with other clients
    protected void NetTerminate(String Name) {
        // when client stop working with test talk just have to erase
        // it from the dictionary of connected clients. The object
        // will be deleted automatically by the garbage collector.
        UnregisterClient(Name);
    }

    // local host name
    protected String NetLocalHost() {
        return LocalHost;
    }

    // local port
    protected int NetLocalPort() {
        return LocalPort;
    }

    // fetch message from messages queue
    protected String NetFetchMessage(String MyClientName)
                                        throws EmptyQueueException {
        return NetFetchMessageInternal(TestTalkConstants.DATAQ,MyClientName);
    }

    // fetch ack message from ack messages queue
    protected String NetFetchAckMessage(String MyClientName)
                                        throws EmptyQueueException {
        return NetFetchMessageInternal(TestTalkConstants.ACKQ,MyClientName);
    }

    // general method which fetch message from queue
    private String NetFetchMessageInternal(
                                    byte QueueType,
                                    String MyClientName)
                                        throws EmptyQueueException {

        // search the client object
        ClientC Client = ClientObjectOf(MyClientName);
        try {
            // fetch the message
            return (Client.GetMessageFromClientQueue(
                                            QueueType,
                                            false,  // no need to wait
                                            0)      // no timeout
                                            ).Data;
        }
        catch (WrongQueueTypeException e) {}
        catch (TimeoutExpiredException e) {}
        return null;
    }

    // wait for message
    protected String NetWaitMessage(String MyClientName,
                                    int    TimeoutInMiliSeconds)
                                    throws TimeoutExpiredException {
        return NetWaitMessageInternal(TestTalkConstants.DATAQ,
                                      MyClientName,
                                      TimeoutInMiliSeconds);
    }

    // wait for ack message
    protected String NetWaitAckMessage(String MyClientName,
                                       int    TimeoutInMiliSeconds)
                                       throws TimeoutExpiredException {
        return NetWaitMessageInternal(TestTalkConstants.ACKQ,
                                      MyClientName,
                                      TimeoutInMiliSeconds);
    }

    // general method which wait for message
    private String NetWaitMessageInternal(
                                    byte   QueueType,
                                    String MyClientName,
                                    int    TimeoutInMiliSeconds)
                                       throws TimeoutExpiredException {

        // search the client object
        ClientC Client = ClientObjectOf(MyClientName);
        try {
            // wait for the message
            return (Client.GetMessageFromClientQueue(
                                            QueueType,
                                            true,   // wait
                                            TimeoutInMiliSeconds)
                                            ).Data;
        }
        catch (WrongQueueTypeException e) {}
        catch (EmptyQueueException e) {}
        return null;
    }

    // send message
    protected void NetSendMessage(
                        String FromClient,
                        String Data,
                        String Address)
                                throws ErrorSendingMessageException,
                                       WrongAddressException {
        NetSendMessageInternal(FromClient,TestTalkConstants.DATAQ,Data,Address);
    }

    // send Ack message
    protected void NetSendAckMessage(
                        String FromClient,
                        String Data,
                        String Address)
                                throws ErrorSendingMessageException,
                                       WrongAddressException {
        NetSendMessageInternal(FromClient,TestTalkConstants.ACKQ,Data,Address);
    }

    // general method for sending message
    protected void NetSendMessageInternal(
                        String FromClient,
                        byte   QueueType,
                        String Data,
                        String Address)
                                throws ErrorSendingMessageException,
                                       WrongAddressException {

        // Create message object
        MessageC Message = new MessageC();

        int TargetPort = -1;
        String TargetHost = "";

        // throw error on empty address
        if ( (Address == null)    ||
             (Address.equals(""))   )
            throw new WrongAddressException(Address);


        // analyze the address

        int SeparatorIndex = Address.indexOf("@");
        if (SeparatorIndex == -1) {
            // the target is client in local JVM
            Message.ToClient = Address;
            Message.ToHost = "";
        }
        else {
            // the target is client of different TestTalk
            Message.ToClient = Address.substring(0,SeparatorIndex);
            Message.ToHost = Address.substring(SeparatorIndex+1);
            SeparatorIndex = (Message.ToHost).indexOf(":");
            if (SeparatorIndex == -1) {
                // port not found : has to use the default port
                TargetPort = TestTalkConstants.DEFAULT_PORT;
                TargetHost = Message.ToHost;
                Message.ToHost = Message.ToHost +
                                 ":" + TestTalkConstants.DEFAULT_PORT;
            }
            else {
                // port found
                TargetPort = Integer.parseInt((Message.ToHost).
                                        substring(SeparatorIndex+1));
                TargetHost = (Message.ToHost).
                                substring(0,SeparatorIndex);
            }

            // Check whether the client is registed in local TestTalk or in
            // different TestTalk
            if ( (TargetHost.equals(LocalHost)) &&
                 (TargetPort == LocalPort) ) {
                Message.ToHost = "";
            }

            // the address must contains client name
            if ( (Message.ToClient == null)      ||
                 ((Message.ToClient).equals(""))    )
                        throw new WrongAddressException(Address);
        }

        // fill rest fields of message
        Message.Data       = Data;
        Message.Type       = TestTalkConstants.SEND;
        Message.SubType    = TestTalkConstants.VERSION;
        Message.QueueType  = QueueType;
        Message.FromClient = FromClient;
        Message.FromHost   = NetLocalHost() + ":" + NetLocalPort();


        if ((Message.ToHost).equals("")) {
            // The target is client in the current TestTalk

            // search for client object
            ClientC Client = ClientObjectOf(Message.ToClient);
            if (Client == null)
                // client does not exist : cannot deliver the message
                throw new ErrorSendingMessageException
                            ("Client " + Message.ToClient +
                             " does not exist in local host");
            else {
                // put the message in client queue
                try {
                    Client.PutMessageInClientQueue(QueueType,Message);
                }
                catch (WrongQueueTypeException e) {}
            }
        }
        else {
            // The client located in different TestTalk

            // Search for TestTalkToTestTalk object
            TestTalkToTestTalkC TestTalkToTestTalk =
                TestTalkToTestTalkObjectOf(TargetHost,TargetPort);

            // If there is no object has to create connection
            if (TestTalkToTestTalk == null) {
                TestTalkToTestTalk = new TestTalkToTestTalkC(
                                                    this,
                                                    TargetHost,
                                                    TargetPort);
            }

            // The following section does :
            //    1) clear control queue
            //    2) send message
            //    3) wait for control message
            // There is a documented bug. But it won't happens, thus,
            // it is not solved. Control Message of earlier message can
            // be received. It can occred due to the fact that we stop
            // waiting to control message after TIMEOUT_IN_MILISECONDS.
            // We assume that after that period the control message
            // won't be received.
            ClientC Client = ClientObjectOf(FromClient);
            try {
                Client.ClearQueue(TestTalkConstants.CONTROLQ);
            }
            catch (WrongQueueTypeException e) {}

            TestTalkToTestTalk.Send(Message);

            // wait for control message
            try {
                MessageC ControlMessage =
                    Client.GetMessageFromClientQueue(
                                TestTalkConstants.CONTROLQ,
                                true, // wait
                                TestTalkConstants.TIMEOUT_IN_MILISECONDS);

                if (ControlMessage.Type == TestTalkConstants.SEND_FAIL)
                    throw new ErrorSendingMessageException
                                            (ControlMessage.Data);
            }
            catch (TimeoutExpiredException e) {
                throw new ErrorSendingMessageException
                                            ("Timeout Expired");
            }
            catch (WrongQueueTypeException e) {}
            catch (EmptyQueueException e) {}

        }
    }

    // save The TestTalkToTestTalk object in hashtable which contains
    // all TestTalks connected to the current TestTalk. The key
    // of the connected TestTalk is Host+Port.
    protected synchronized void RegisterTestTalk(
                            String              TargetComputerName,
                            int                 Port,
                            TestTalkToTestTalkC TestTalkToTestTalk) {

        ConnectedTestTalks.put(TargetComputerName+":"+Port,
                               TestTalkToTestTalk);

    }

    // remove the TestTalkToTestTalk object from the hashtable
    protected synchronized void UnregisterTestTalk(
                            String      TargetComputerName,
                            int         Port) {

        ConnectedTestTalks.remove(TargetComputerName+":"+Port);

    }

    // fetch TestTalkToTestTalk object from hashtable
    protected synchronized TestTalkToTestTalkC TestTalkToTestTalkObjectOf(
                                String TargetComputerName,
                                int    Port) {

        return (TestTalkToTestTalkC)ConnectedTestTalks.
                                    get(TargetComputerName+":"+Port);

    }

    // save client in hashtable which contains clients connected to
    // current TestTalk
    protected synchronized void RegisterClient(
                                String Name,
                                ClientC Client) {
        LocalTestTalkConnectedClients.put(Name,Client);
    }

    // remove client from hashtable
    protected synchronized void UnregisterClient(
                                String Name) {
        LocalTestTalkConnectedClients.remove(Name);
    }

    // get client object frmo hashtable
    protected synchronized ClientC ClientObjectOf(
                                String Client) {
        return (ClientC)LocalTestTalkConnectedClients.get(Client);
    }

    protected String InternalClientWhoAmI() {
        return TestTalkInternalClient.NetWhoAmI();
    }

    protected void NetNotifyOnConnectionFail(String  CheckAddress,
                                             boolean CheckAbnormalExit,
                                             String  DataToBeSent,
                                             String  TargetAddress,
                                             byte    QueueType) {

        String CheckHost;
        int Port;
        int SeparatorIndex = CheckAddress.indexOf("@");
        if (SeparatorIndex == -1)
            CheckHost = CheckAddress;
        else
            CheckHost = CheckAddress.substring(SeparatorIndex+1);
        SeparatorIndex = CheckHost.indexOf(":");
        if (SeparatorIndex == -1)
            Port = TestTalkConstants.DEFAULT_PORT;
        else {
            Port = Integer.parseInt(CheckHost.substring(SeparatorIndex+1));
            CheckHost = CheckHost.substring(0,SeparatorIndex);
        }

        TestTalkToTestTalkC TestTalkToTestTalk = TestTalkToTestTalkObjectOf(CheckHost,Port);

        TestTalkToTestTalk.NetNotifyOnConnectionFail(CheckAbnormalExit,
                                                     DataToBeSent,
                                                     TargetAddress,
                                                     QueueType);
    }

    protected void NetNormalExitTestTalk() {
        String key;
        TestTalkToTestTalkC TestTalkToTestTalk;
        Enumeration Enum = ConnectedTestTalks.keys();
        while (Enum.hasMoreElements()) {
            key = (String)Enum.nextElement();
            TestTalkToTestTalk = (TestTalkToTestTalkC)ConnectedTestTalks.get(key);
            MessageC Message = new MessageC();
            Message.FromHost = LocalHost + ":" + LocalPort;
            Message.FromClient = "TestTalkInternalClient";
            Message.ToHost = key;
            Message.ToClient = "TestTalkInternalClient";
            Message.QueueType = TestTalkConstants.ACKQ;
            Message.Type = TestTalkConstants.SEND;
            Message.SubType = TestTalkConstants.VERSION;
            Message.Data = "Normal Exit : " + Message.FromHost;
            TestTalkToTestTalk.Send(Message);
        }
    }

    protected void NetNormalExitTestTalk(String HostAndPort) {
        int SeparatorIndex = HostAndPort.indexOf(":");
        int Port;
        String Host;
        if (SeparatorIndex == -1) {
            Host = HostAndPort;
            Port = TestTalkConstants.DEFAULT_PORT;
        }
        else {
            Host = HostAndPort.substring(0,SeparatorIndex);
            Port = Integer.parseInt(HostAndPort.substring(SeparatorIndex+1));
        }
    }

    // The thread which waits for other TestTalk to create connection
    // with the current TestTalk
    public void run() {

        setPriority(MAX_PRIORITY);

        try {
            while (true) {
                // Listen to other TestTalk in the network
                Socket TargetSocket = Soc.accept();
                // Create specific connection to that TestTalk
                TestTalkToTestTalkC TestTalkToTestTalk =
                    new TestTalkToTestTalkC(this,TargetSocket);
            }
        }
        catch (Exception e) {  //Throwable
            //DebugMessages.WriteMessage(
            //        "Could not listen on port: " + UserLocalPort + ", " + e);
						if (log.isErrorEnabled()) log.error("Could not listen on port " + UserLocalPort);
            OKDialog.createOKDialog("Could not listen on port " + UserLocalPort);
            System.exit(1);
        }
    }

}
