/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.util.testtalk;

import java.net.Socket;
import java.util.Vector;

class TestTalkToTestTalkC {

    private TestTalkC TestTalk;

    private TestTalkReaderC TestTalkReader;
    private TestTalkWriterC TestTalkWriter;

    private Vector TerminationDataVec = new Vector(0,1);

    // Ctor for using when current computer is the target
    TestTalkToTestTalkC(TestTalkC testtalk,
                        Socket    TargetSocket) {

        // save values
        TestTalk = testtalk;

        // The reader create the writer thread after the connection
        // is created successfully
        TestTalkReader = new TestTalkReaderC(
                                     this,
                                     null,  // computer is not known yet
                                     -1,    // port is not known yet
                                     TargetSocket,
                                     false);
        TestTalkReader.start();
    }

    // Ctor for using when current computer creates the connection
    TestTalkToTestTalkC(TestTalkC testtalk,
                        String    TargetComputerName,
                        int       TargetPort) {

        // save values
        TestTalk = testtalk;

        RegisterTestTalk(TargetComputerName,TargetPort);

        // The Writer create the reader thread after the connection
        // is created successfully
        TestTalkWriter = new TestTalkWriterC(
                                     this,
                                     TargetComputerName,
                                     TargetPort,
                                     null, // socket is not known yet
                                     true);
        TestTalkWriter.start();

    }

    // activated by the Writer after it creates connection
    protected void CreateReaderInSendOption (String TargetComputerName,
                                             int    TargetPort,
                                             Socket TargetSocket){
        TestTalkReader = new TestTalkReaderC(
                                     this,
                                     TargetComputerName,
                                     TargetPort,
                                     TargetSocket,
                                     true);
        TestTalkReader.start();
    }

    // activated by the reader after it get the connection and knows
    // the computer name which asked for connection
    protected void CreateWriter (String TargetComputerName,
                                 int    TargetPort,
                                 Socket TargetSocket){
        TestTalkWriter = new TestTalkWriterC(
                                     this,
                                     TargetComputerName,
                                     TargetPort,
                                     TargetSocket,
                                     false);
        TestTalkWriter.start();
    }

    // just put message in the writer queue. The writer will read
    // the message from the queue and will send it.
    protected void Send(MessageC Message) {
        TestTalkWriter.PutMessageInWriterQueue(Message);
    }

    protected void SendUrgent(MessageC Message) {
        TestTalkWriter.PutMessageInWriterQueueOnTop(Message);
    }

    // search for that client
    protected ClientC ClientObjectOf(String name) {
        return TestTalk.ClientObjectOf(name);
    }

    // search for that host & port
    protected TestTalkToTestTalkC TestTalkToTestTalkObjectOf(
                                                    String name,
                                                    int port) {
        return TestTalk.TestTalkToTestTalkObjectOf(name,port);
    }

    // local host
    protected String NetLocalHost() {
        return TestTalk.NetLocalHost();
    }

    // local port
    protected int NetLocalPort() {
        return TestTalk.NetLocalPort();
    }

    // write the TestTalkToTestTalk object in dictionary
    protected void RegisterTestTalk(String TargetComputerName,
                                    int    TargetPort) {
        TestTalk.RegisterTestTalk(TargetComputerName,TargetPort,this);
    }

    // erase the object from the dictionary
    protected void UnregisterTestTalk(String TargetComputerName,
                                      int    TargetPort) {
        TestTalk.UnregisterTestTalk(TargetComputerName,TargetPort);
        for (int i = 0; i < TerminationDataVec.size(); i++) {
            TerminationData TerminationData = (TerminationData)TerminationDataVec.elementAt(i);
            if (!(TerminationData.CheckAbnormalExit && TerminationData.NormalExit)) {
                try {
                    TestTalk.NetSendMessageInternal(TestTalk.InternalClientWhoAmI(),
                                                    TerminationData.QueueType,
                                                    TerminationData.DataToBeSent,
                                                    TerminationData.TargetAddress);
                }
                catch (WrongAddressException e) {}
                catch (ErrorSendingMessageException e) {}
            }
        }
    }

    protected void NetNotifyOnConnectionFail(boolean CheckAbnormalExit,
                                             String  DataToBeSent,
                                             String  TargetAddress,
                                             byte    QueueType) {

        TerminationData TerminationData = new TerminationData();
        TerminationData.CheckAbnormalExit = CheckAbnormalExit;
        TerminationData.DataToBeSent = DataToBeSent;
        TerminationData.TargetAddress = TargetAddress;
        TerminationData.QueueType = QueueType;
        TerminationData.NormalExit = false;

        TerminationDataVec.addElement(TerminationData);
    }
}
