/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_FRAME_CONTAINER__
#define __WL_FRAME_CONTAINER__

#include <hashdict.h>
#include <RVCString.h>
#include <slistcol.h>
#include <thr.h>
#include "wldocument.h"
#include "wllocation.h"
#include "platform.h"
#include "wlidentlocation.h"

class CWLDocument;
class frameContainer;

enum JSFrameMode{
		JS_None,
		JS_Document,
		JS_Frame,
		JS_Frames
};

enum replaceFrameStatus{
		replaceFrame_REPLACE_OK,
		replaceFrame_INSERT_FRAME_FAILURE,
		replaceFrame_INSERT_QUEUE_FAILURE,
		replaceFrame_DELETE_NAME_MATCH_NUMBER_MISMATCH_WARNING,
		replaceFrame_DELETE_NUMBER_MATCH_NAME_MISMATCH_WARNING,
		replaceFrame_DELETE_FAILURE
};

/******************************************************
  Name    : CWLDomWrapper
  Purpose : encapsulates all dom data.
  Author  : Yossi Nakash
******************************************************/
class DllExport CWLDomWrapper : public RVCollectable
{
  friend class frameContainer ;

private:
	CWLDocument*            m_pDom;
	CWLDocument*            m_pPageTransaction;
	CWLDocument*            m_pErrorFrame;
	JSFrameMode				      m_FrameMode;
  RVCString               m_sDomName ;

	RVOrdered				        m_CloneContainer;
	RVOrdered               m_AllFrames;
	RVOrdered               m_AllForms;
	RVOrdered               m_AllLinks;

public:
  CWLDomWrapper(RVCString& sDomName);
  virtual ~CWLDomWrapper();

	CWLDocument *GetPageTransaction(){return m_pPageTransaction;}
	void        SetPageTransaction(CWLDocument *pFrame){m_pPageTransaction=pFrame;}
  void        cleanUp();

private:
  // avoid copy ctor & asignment operator
  CWLDomWrapper( const CWLDomWrapper& src );
  operator=( const CWLDomWrapper& src );

};


//===============================FRAMECONTAINER=====================
class DllExport frameContainer  
{
private:
  RVHashDictionary        m_DomHash ;

  RVCriticalSection*      m_DeletePageLock;
	CWLHtmlVirtualQueue*	  m_pQueuePtr ;
  CWLDomWrapper*          m_pActiveDom ;
  CWLDomWrapper*          m_pPreviousDom ;

public:
	frameContainer(CWLHtmlVirtualQueue* queuePtr);
	virtual				~frameContainer();

	void          deleteAllPages();
	int						insertNewPage(RVCString&, RVCString& pageUrl,
							   CWLUserParseFlags& parseFlags);
  int           insertNewFrame(RVCString&           pageUrl,
                               CWLUserParseFlags&   parseFlags,
                               const RVCString&     Target,
                               replaceFrameStatus&  retStatus);
	void					deletePage(RVCString& pageName);
  CWLDocument*  deleteOldFrame(const CWLDocument*   startFrame,
                               CWLIdentLocation&    TargetLocation,
                               int&                 retFrameNumber,
                               RVCString&           retFrameName,
                               replaceFrameStatus&  retStatus);
  void          SetInvalidPage(RVCString& pageName);
  void					cleanUp();
	void					insertSubTreeFrames();
	void					parse(char* ,size_t);
	void					append(const char*, size_t);
	CWLDocument*	findPageByName(RVCString& );
  CWLDocument*	getCurrentPage(){return m_pActiveDom ? m_pActiveDom->m_pDom : NULL;}

  bool           SetActiveDom(const char* pcDomName);
  bool           SetActiveDom(CWLDomWrapper* pDom);
  CWLDomWrapper* GetActiveDom();
  void           SetPreviousDomAsActive(CWLDomWrapper* pDom = NULL);
  CWLHtmlVirtualQueue* GetQueuePtr() { return m_pQueuePtr;}

//!================still need these functions to be used by http=======
	void					insertHeader(const char* name,const char* value);
	void					insertStatus(unsigned, int, RVCString&);
	void					insertUri(RVCString&);
	void					insertHost(RVCString&, unsigned);
	void					deReferenceObject(CWLDocument* docPtr);
	void					referenceObject(CWLDocument* docPtr);
	int						GetFrameByIndex(CWLDocument* frm, int, CWLDocument** );
	int						GetFrameByFrameName(CWLDocument* frm ,const char* , CWLDocument**);
	int						GetFrameByUrl(CWLDocument* frm, const char* , CWLDocument** );
	int						GetFrameUrl(CWLDocument* frm, CWLUrl** );
	int						GetHrefByIndex(CWLDocument* frm, int , CWLLink** );
	int						GetHrefByName(CWLDocument* frm, const char* , CWLLink** );
	int						GetHrefByUrl(CWLDocument* frm, const char* , CWLLink** );
	int						GetFormAction(CWLDocument* frm, int, CWLLocation** );
	int						GetFieldValueByPage(CWLDocument* frm,const char*, const char**);
	int						GetFieldValueByForm(CWLDocument* frm, int, const char*, const char** );
	int						GetHeaderValue(CWLDocument* frm, const char*, const char*&);
	int						GetStatusLine(CWLDocument* frm, const char*&);
	int						GetHost(CWLDocument* frm, const char*&);
	int						GetHostName(CWLDocument* frm, const char*&);
	int						GetUri(CWLDocument* frm, const char*& );
	unsigned			GetPortNum(CWLDocument* frm);
	unsigned			GetStatusNumber(CWLDocument* frm);
	CWLDocument*	GetGlobalDoc();
	void					CleanCloneContainer();
	void          AddFrameToArray(CWLDocument* frame);
	void          AddFormToArray(CWLForm* form);
	void          AddLinkToArray(CWLLink* link);
	RVOrdered*	  GetCloneContainer();
  RVOrdered*    GetAllFrames();
	RVOrdered*    GetAllForms();
	RVOrdered*    GetAllLinks();

  JSFrameMode		GetFrameMode(){return m_pActiveDom ? m_pActiveDom->m_FrameMode : JS_None;}
	void			SetFrameMode(JSFrameMode frameMode);

	CWLDocument *GetPageTransaction();
	void        SetPageTransaction(CWLDocument *pFrame);
};

#endif
