/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_LOCATION__
#define __WL_LOCATION__

#include <RVCString.h>
#include <hashdict.h>
#include "platform.h"
#include "wlurl.h"
#include "wlelement.h"
#include "wlurlbase.h"
#include "wlhtmlfuncs.h"
#include "wltransactiondata.h"

class CWLDocument;
class CWLEmbed;

class DllExport CWLLocation : public CWLUrl {

public:
	CWLLocation() {}
	CWLLocation(RVCString& url, RVCString& origUrl, RVCString& referrer, const char* linkName, const char* id, const char* target);
  CWLLocation(const CWLLocation& aCWLl) : CWLUrl(*(CWLUrl*)&aCWLl) {}
  virtual ~CWLLocation() {}
  CWLLocation& operator=(const CWLLocation& aL) { CWLUrl::operator =(*(CWLUrl*)&aL); return *this; }

};

class DllExport CWLLink : public CWLUrl {

private:
	//CWLDocument *m_DocPtr;
	long m_Index;
	CWLEmbed*    m_InnerImg;
	RVCString    m_InnerHTML; //! just for httpload call (ASM new link)
	RVCString*   m_InnerText;

public:
	CWLLink(CWLDocument *docPtr) : CWLUrl(docPtr) { m_InnerImg = NULL; m_Index = -1; m_InnerText = NULL;}
	CWLLink(CWLDocument *docPtr, RVCString& url, RVCString& origUrl, const char* linkName, const char* id, const char* target);
	CWLLink(RVCString& url, RVCString& origUrl, RVCString& referrer, const char* linkName, const char* innerHTML, const char* id, const char* target); // just for ASM (httpload call)
	CWLLink(const CWLLink&);
  virtual ~CWLLink();
    CWLLink& operator=(const CWLLink&) ;
	long GetIndex() { return m_Index; }
	void SetInnerImg(CWLEmbed* img) { m_InnerImg = img; }
	CWLEmbed* GetInnerImg() { return m_InnerImg; }
    virtual RVCSubString GetInnerHTML();
   virtual const char* GetInnerText();
  void SetInnerHTML(RVCString& innerHTML) { m_InnerHTML = innerHTML; }
  	virtual void printAll();
};

class DllExport CWLCss : public CWLUrl {

private:
//	CWLDocument *m_DocPtr;
	long m_Index;

protected:
	CWLTransactionsObject m_TransObj ;
public:
	CWLTransactionsObject* GetTransactionObj(){return &m_TransObj;}

public:
	CWLCss() : CWLUrl(NULL), m_TransObj(NULL) { m_TransObj.SetObj(this); m_TransObj.SetObjType(LINK); m_iUrlOffset = -1; }
	CWLCss(RVCString& url, RVCString& origUrl, CWLDocument *docPtr, const char* linkName, const char* id);
	CWLCss(const CWLCss&);
  virtual ~CWLCss() {}
    CWLCss& operator=(const CWLCss&) ;
	long GetIndex() { return m_Index; }

protected:
  int m_iUrlOffset;
public:
  void setUrlOffset(int uo) { m_iUrlOffset = uo; }
  int  getUrlOffset() { return m_iUrlOffset; }

};

class DllExport CWLEmbed : public CWLUrl {

protected:
  CWLTransactionsObject m_TransObj ;

public:
  CWLTransactionsObject* GetTransactionObj(){return &m_TransObj;}

protected:
  int m_iUrlOffset;

public:
  void setUrlOffset(int uo) { m_iUrlOffset = uo; }
  int  getUrlOffset() { return m_iUrlOffset; }

private:
	RVCString *tagName;
	RVCString *attrName;
	//CWLDocument *m_DocPtr;
	long m_Index;

	RVCString* m_pWidth;
	RVCString* m_pHeight;
	RVCString* m_pAlt;
	RVCString* m_pTitle;
	RVCString* m_pUseMap;
	RVCString* m_pLowsrc;
	RVCString* m_pOrigUseMap;
	RVCString* m_pOrigLowsrc;

public:
	//CWLEmbed(const char* aTagName, const char* anAttrName, RVCString& url, RVCString& origUrl, CWLDocument *docPtr, const char* linkName, const char* id, const char* target);
	CWLEmbed(const char* aTagName, const char* anAttrName, RVCString& url, RVCString& origUrl, CWLDocument *docPtr, const char* linkName, const char* id, const char* target);
	CWLEmbed( const CWLEmbed& aCWLEmbed);
  virtual ~CWLEmbed();
  CWLEmbed& operator=(const CWLEmbed& aL);
	const char* Tag();
	const char* Attr();
	long        GetIndex() { return m_Index; }
	void        SetProperties(RVCString& p_Width, RVCString& p_Height, RVCString& p_Alt,
							  RVCString& p_Title, RVCString& p_UseMap, RVCString& p_Lowsrc,
							  RVCString& p_OrigUseMap, RVCString& p_OrigLowsrc);
	const char* GetWidth() { return (m_pWidth)  ? m_pWidth->data()  : NULL; }
	const char* GetHeight(){ return (m_pHeight) ? m_pHeight->data() : NULL; }
	const char* GetAlt()   { return (m_pAlt)    ? m_pAlt->data()    : NULL; }
	const char* GetTitle() { return (m_pTitle)  ? m_pTitle->data()  : NULL; }
	const char* GetUseMap(){ return (m_pUseMap) ? m_pUseMap->data() : NULL; }
	const char* GetLowsrc(){ return (m_pLowsrc) ? m_pLowsrc->data() : NULL; }
	const char* GetOrigUseMap(){ return (m_pOrigUseMap) ? m_pOrigUseMap->data() : NULL; }
	const char* GetOrigLowsrc(){ return (m_pOrigLowsrc) ? m_pOrigLowsrc->data() : NULL; }
};

/*! This class for APPLET and OBJECT tags */
class DllExport CWLApplet : public RVOrdered {
protected:
  int m_iCodebaseOffset;
  int m_iCodeOffset;
  int m_iClassidOffset;
  int m_iSrcOffset;
  int m_iDataOffset;

public:
  void setCodebaseOffset(int o) { m_iCodebaseOffset = o; }
  int  getCodebaseOffset() { return m_iCodebaseOffset; }
  void setCodeOffset(int o) { m_iCodeOffset = o; }
  int  getCodeOffset() { return m_iCodeOffset; }
  void setClassidOffset(int o) { m_iClassidOffset = o; }
  int  getClassidOffset() { return m_iClassidOffset; }
  void setSrcOffset(int o) { m_iSrcOffset = o; }
  int  getSrcOffset() { return m_iSrcOffset; }
  void setDataOffset(int o) { m_iDataOffset = o; }
  int  getDataOffset() { return m_iDataOffset; }

private:
	urlBase   m_Codebase;
	RVCString m_Code;
	RVCString m_Classid;
	RVCString m_Src;
	RVCString m_Data;
	RVCString m_Codetype;
	RVCString m_Id;
	RVCString m_Type;
	RVCString m_TagName;
	RVCString m_Url;
	RVCString m_Text;
	long	  m_Index;
	RVCString m_OrigUrl;
	RVCString m_Name;

    CWLTransactionsObject m_TransObj ;

	CWLDocument* m_DocPtr;
	CWLApplet& operator=(const CWLApplet& ){ return *this; }
	CWLApplet(const CWLApplet&);

public:
    CWLApplet(CWLDocument* docPtr);
	virtual ~CWLApplet(){ clearAndDestroy(); }
	const char* GetText() { return m_Text.isNull() ? NULL : m_Text.data(); }
	const char* GetId()   { return m_Id.isNull()   ? NULL : m_Id.data(); }
	const char* GetSrc()  { return m_Url.isNull()  ? NULL : m_Url.data(); }
	const char* GetName()  { return m_Name.isNull()  ? NULL : m_Name.data(); }
	elementReadState parse(RVCString& buff, urlBase* urlBasePtr, size_t scriptEndIndex, const char *tag);
      void	printAll();
	RVCString& GetOriginalUrl() { return m_OrigUrl; }  // doesn't clear what's original url for applet ???
	RVCString& GetReferrer();
	long	   GetIndex() { return m_Index; }
    CWLTransactionsObject* GetTransactionObj(){return &m_TransObj;}
};

class DllExport CWLParam : public RVCollectable {

private:
	RVCString m_Name;
	RVCString m_Value;

public:
	CWLParam(){}
	virtual ~CWLParam(){}
	CWLParam(const char* name, const char* value) : m_Name(name), m_Value(value) {}
	void SetName(const char* name)   { m_Name = name; }
	void SetValue(const char* value) { m_Value = value; }
	const char* GetName()  { return m_Name.data(); }
	const char* GetValue() { return m_Value.data(); }
};

#endif

