/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include  "wlurlarray.h"
#include	"tvenv.h"
#include  "wlurlbase.h"
#include  "wlhtmlfuncs.h"
#include "wlhtmlvirtualqueue.h"
#include "wldocument.h"
#include "tracedll.h"

CWLLinkArray::CWLLinkArray(CWLDocument *docPtr) : m_DocPtr(docPtr)
{
	m_hrefAttFound = false;
  stateFlag = FIND_ATT_VALUE;
}

CWLLinkArray& CWLLinkArray::operator=(const CWLLinkArray& aCWLLinkArray)
{
	size_t _entries = aCWLLinkArray.entries();

	if(!aCWLLinkArray.isEmpty())
		for(size_t i = 0; i < _entries; ++i)
		{
			CWLLink* urlPtr = new CWLLink(m_DocPtr);
			(*urlPtr) = *(CWLLink*)aCWLLinkArray[i];
			insert(urlPtr);
		}
	return (*this);
}


elementReadState CWLLinkArray::parse(RVCString& buff, anchorType type, urlBase* urlBasePtr)
{
  size_t buffPtr = 0, attValueStart = RV_NPOS, lt = 0, innerHTMLOffset = -1;
  RVCString attName, attValue;
  while(1)
  {
    switch(stateFlag) 
    {  
			case(FIND_ATT_VALUE):  
				while(1)
				{
					attState hrefState = htmlFindNextAttribute(buff, attName, attValue, buffPtr, attValueStart);
					if(hrefState == HTML_CURRENT_ATT_COMPLETE)
					{
						switch(kindOfAttribute(attName)) {
						case HREF:
							m_hrefAttFound = true;
							origUrl = attValue;
							url = urlBasePtr->calcUrl(attValue);
							break;
						case ID:
							id = attValue.data();
							break;
						case NAME:
							name = attValue.data();
							break;
						case TARGET:					
							target = attValue.data();
							break;
						}
					}										
					else if (hrefState == HTML_FOUND_DELIMITER)
					{
						if(!m_hrefAttFound)//!<a name= 
						{
							stateFlag = FIND_ATT_VALUE;//!reset
							return END_OF_ELEMENT;				
						}
						stateFlag = FIND_NAME_VALUE;
						size_t _lt = buff.index(">");
						m_DocPtr->m_iBuffPos += _lt + 1;
						buff.remove(0, _lt + 1);//!get ready to extract name
						break;//!while
					}
					else//!HTML_READ_MORE_DATA
						return READ_MORE;
				}						

			case (FIND_NAME_VALUE):  
				if(type == A_ANCHOR)
				{
					while((lt = buff.index("<", lt)) != RV_NPOS)
					{
						if(buff.length() <= lt + 3)
							return READ_MORE;
						RVCString tempBuff = buff(lt + 1, 3); 
						if(isElementOfType("/a", 2, tempBuff, '>', m_DocPtr, false)
							|| isElementOfType("a", 1, tempBuff, ' ', m_DocPtr, false))
						{
							//!<a href,,,>..</a> or <a href..>...<a href...>..</a>
							if(m_DocPtr->m_ParseInnerHTML)
								innerHTMLOffset = m_DocPtr->m_iBuffPos;
							else
								innerHTML  = buff(0, lt);					
							break;
						}
						//!different tag
						++lt;//!advance by 1 to avoid infinite loop <a href....><img...>
					}
				} else if (type == AREA_ANCHOR) {//AREA_ANCHOR
					innerHTML = "AREA_ELEMENT";
					// qqqq hrefToArray->SetInnerHTML(innerHTML);
				}

				CWLLink* hrefToArray = new CWLLink(m_DocPtr, url, origUrl, name.data(), id.data(), target.data());
				if (type == A_ANCHOR) {
					if (m_DocPtr->m_CurLink) {
						WlTraceMsg("ERROR: Current link is not null.");
						crash_func();
					}
					m_DocPtr->m_CurLink = hrefToArray;
				}
				if (innerHTMLOffset != -1)
					hrefToArray->SetInnerHTMLOffset(innerHTMLOffset, lt); 
				else
					hrefToArray->SetInnerHTML(innerHTML);

				if(insert(hrefToArray) == NULL) 
					delete hrefToArray;

				stateFlag = FIND_ATT_VALUE;//!reset
				m_hrefAttFound = false;//!reset
				innerHTML.remove(0);
				name.remove(0);
				url.remove(0);
				origUrl.remove(0);
				target.remove(0);
				id.remove(0);
				return END_OF_ELEMENT;				
		}
	}
	return READ_MORE;
}

int CWLLinkArray::findEntryByUrlMatch(const char* regExp, CWLLink** hrefPtrAdr)
{
  int numItems = entries();
  if (numItems < 1)  
    return -3;
  char REarr[100];
  strcpy(REarr,regExp);
  REarr[strlen(regExp)] = '\0';
  RVCRegexp regExpUrl(REarr);
  for(int i=0;i<numItems;++i)
  {  
		CWLLink* hrefPtr = (CWLLink*)at(i);		
    RVCString curUrl(hrefPtr->Href());
    size_t ext;
    size_t regIndex = curUrl.index(regExpUrl, &ext);
    if(regIndex == 0  &&  ext == curUrl.length())          	  
		{
			*hrefPtrAdr = hrefPtr;
      return 0;    
		}
  }
  return -2; //!No entry with same name
}

int CWLLinkArray::findEntryByInnerHTMLMatch(const char* regExp, CWLLink** hrefPtrAdr)
{
  int numItems = entries();
  if (numItems < 1)  
    return -3;
  char REarr[100];
  strcpy(REarr,regExp);
  REarr[strlen(regExp)] = '\0';
  RVCRegexp regExpUrl(REarr);
  for(int i=0;i<numItems;++i)
  {  
		CWLLink* hrefPtr = (CWLLink*)at(i);
    RVCString curName(hrefPtr->GetInnerHTML());
    size_t ext;
    size_t regIndex = curName.index(regExpUrl, &ext);
    if(regIndex == 0  &&  ext == curName.length())
		{
			*hrefPtrAdr = hrefPtr;
      return 0;
		}
  }
  return -2; //!No entry with same name
}


int CWLLinkArray:: getUrlByIndex( int index,CWLLink** hrefPtrAdr)
{

  if (entries() < 1)
    return -3;//!no links in this page    

  if (index >= (int)entries())
    return -2;//!number of links smaller than index
  
  CWLLink* hrefPtr = ((CWLLink*)at(index));
	*hrefPtrAdr = hrefPtr;
  return 0;
    
}

void CWLLinkArray::printAll()
{
	cout<<endl<<"List of links"<<endl;
	cout<<"============="<<endl;
	size_t entries_ = entries();
	if (entries_ != RV_NPOS)
	{
		for(size_t i=0; i < entries_; i++)
		{
			cout<<i<<'\n'<<"==="<<'\n';
			CWLLink url = *((CWLLink*)(*this)[i]);					
			cout<<"href: "<<url.Href()	<<endl<<
				  "protocol: "<<url.Protocol()<<"  "<<
				  "hostname: "<<url.Hostname()<<"  "<<
				  "host: "<<url.Host()	<<"  "<<
				  "port: "<<url.Port()	<<endl<<
				  "uri: "<<url.Uri()<<endl<<
				  "search: "<<url.Search()	<<endl<<
				  "commands:"<<url.Commands()<<endl<<
				  "hash: "<<url.Hash()	<<endl<<
					"target: "<<url.Target()<<endl<<
				  "search pairs:"<<endl;
			url.Array().print();
			cout<<endl;			
		}
	}
}

//=============================CWLCSSArray=========================================
CWLCSSArray& CWLCSSArray::operator=(const CWLCSSArray& aCWLCSSArray)
{
	size_t _entries = aCWLCSSArray.entries();

	if(!aCWLCSSArray.isEmpty())
		for(size_t i = 0; i < _entries; ++i)
		{
			CWLCss* urlPtr = new CWLCss();
			(*urlPtr) = *(CWLCss*)aCWLCSSArray[i];
			insert(urlPtr);
		}
	return (*this);
}


elementReadState CWLCSSArray::parse(RVCString& buff, urlBase* urlBasePtr)
{
  size_t buffPtr = 0, attValueStart = RV_NPOS, attValueStartF = RV_NPOS;
  RVCString attName, attValue, origUrl;
  RVCString url, name, id, rel;

  while(1) {
	attState hrefState = htmlFindNextAttribute(buff, attName, attValue, buffPtr, attValueStart);
	if(hrefState == HTML_CURRENT_ATT_COMPLETE){
		switch(kindOfAttribute(attName)) {
		case HREF:
			attValueStartF = attValueStart;
			origUrl = attValue;
			url = urlBasePtr->calcUrl(attValue);
			break;
		case ID:
			id = attValue.data();
			break;
		case NAME:
			name = attValue.data();
			break;
    case REL:
      rel = attValue.data();
      break;
		}
	} else if (hrefState == HTML_FOUND_DELIMITER) {
    // @@Chaim - fix for rel=alternate
		if(rel.compareTo("alternate", RVCString::ignoreCase) != 0 && m_DocPtr->m_wlAll && isHttpProtocol(url)) { 
			CWLCss* hrefToArray = new CWLCss(url, origUrl, m_DocPtr, name.data(), id.data());
			if(!m_DocPtr->m_wlAll->insert(hrefToArray)) 
				delete hrefToArray;
			else if (!origUrl.isNull()) {
				CWLTransactionsObject* TOptr = hrefToArray->GetTransactionObj();
				if (attValueStartF != RV_NPOS) {
					size_t uo = m_DocPtr->m_iBuffPos + attValueStartF;
					TOptr->SetUrlOffset(uo);
					hrefToArray->setUrlOffset(uo);
				}
				m_DocPtr->insertTransObjToQueue(TOptr);
			}
		}
		size_t _lt = buff.index(">");
		m_DocPtr->m_iBuffPos += _lt + 1;
		buff.remove(0, _lt + 1);
		return END_OF_ELEMENT;	
	} else//!HTML_READ_MORE_DATA
		return READ_MORE;
  }						
  return READ_MORE;
}

void CWLCSSArray::printAll()
{
	cout<<endl<<"List of links"<<endl;
	cout<<"============="<<endl;
	size_t entries_ = entries();
	if (entries_ != RV_NPOS)
	{
		for(size_t i=0; i < entries_; i++)
		{
			cout<<i<<'\n'<<"==="<<'\n';
			CWLCss url = *((CWLCss*)(*this)[i]);					
			cout<<"href: "<<url.Href()	<<endl<<
				  "protocol: "<<url.Protocol()<<"  "<<
				  "hostname: "<<url.Hostname()<<"  "<<
				  "host: "<<url.Host()	<<"  "<<
				  "port: "<<url.Port()	<<endl<<
				  "uri: "<<url.Uri()<<endl<<
				  "search: "<<url.Search()	<<endl<<
				  "commands:"<<url.Commands()<<endl<<
				  "hash: "<<url.Hash()	<<endl<<
					"target: "<<url.Target()<<endl<<
				  "search pairs:"<<endl;
			url.Array().print();
			cout<<endl;			
		}
	}
}

