/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/*! *****************************************************************\
 *	File name : wlHtmlFuncs.cpp
 *
 *	Author : Yuval Luski
 *
 *	Description : General functions extracting strings
 *	              htmlGetStringValue extracts a string, if it finds it's end
 *								& returns a ptr to it's end.
 *
 *	History :
 *
 *	Modifications :
 *		Who				When				What
 *		---				----				----
 *	ToDo :
 *  1)	change "htmlFindNextAttribute" function so it's able to handle non-paired properties
 *			like checked in radio.
 *  2)  change "htmlFindNextAttribute" function so it returns a ptr just beyond right delimiter
 *      when it returns HTML_FOUND_DELIMITER
\******************************************************************/
#include "wlregularexpressions.h"
#include <RVCString.h>
#include <regexp.h>
#include "wlhtmlfuncs.h"
#include "charset.h"
#include "assert.h"
#include "wldocument.h"
#ifdef __TV_UNIX__
#include <ctype.h> //! needed for isalnum()
#endif

extern CWLCharSet wlConvertCharSet;



#ifdef __TV_UNIX__
const char* _itoa(int i, char* buf, int size) {
  sprintf(buf,"%d",i);
  return buf;
}
#endif

inline bool findSpace(char& c)
{
  if(c == ' ' || c == '\n' || c == '\r' || c == '\t')
    return true;

  return false;
}

inline size_t findSpaceIndex(RVCString& buff, int startUrlIndex)
{
	RVCRegexp sp("[\\t\\r\\n ]");
  return buff.index(sp, startUrlIndex);
}


//! Need to change the name because it removes TABs as well..  Uri
void removeNewLine(RVCString& str, bool spaceReplace /*! = true */)
{
//	RVCRegexp REnewline("[\x0a\x0d]+");
	RVCRegexp REnewline("[\n\r\t]+");
	size_t location = 0, len;
//	size_t i,l;
	while((location = str.index(REnewline, &len, location) )!= RV_NPOS)
		if( spaceReplace )
		{
			str.replace(location, len, " ");
			++location;
		}
		else
			str.remove(location, len);

	//! now remove leading and trailing blanks
	str.strip(RVCString::both);
	/*
	l = str.length();
	for(i = 0; i < l; ++i)
		if(str(i) != ' ')
			break;

	if( i > 0 ) // found some
		str.remove(0,i);

	l = str.length();
	for(i = 1; i < l; ++i)
		if(str(l - i) != ' ')
			break;

	if( i > 1)
		str.resize(l-i+1);
*/
}



bool htmlSkipWhiteSpaces(RVCString& buff, size_t buffLength, size_t& beginIndex)
{
	while(++beginIndex < buffLength)
	{
		/*char	c = buff(beginIndex);
		  if(c != ' ' && c != '\n' && c != '\r' && c != '\t')
			break;*/
		switch(buff(beginIndex)) {
		  case ' ':
		  case '\n':
		  case '\r':
		  case '\t': continue;
		  default: return true;
		}
	}
	return false;
}

bool htmlIsWhiteSpace(char c)
{
  switch(c) {
	case ' ':
	case '\t':
	case '\n':
	case '\r':
		return 1;
  }
  return 0;
}

strVal htmlGetStringValue(RVCString& buff, RVCString& str, size_t beginIndex, size_t& startAttributsPtr, size_t& endAttributsPtr, size_t buffLength)  //!get a desired string value(URL, NAME= , VALUE=, ....)
{
  //size_t beginIndex = buff.index("=", strPtr);
  //size_t buffLength = buff.length();

	if(!htmlSkipWhiteSpaces(buff, buffLength, beginIndex))
		return HTML_STRING_NOT_COMPLETE;
	str.remove(0);
	char quote = buff(beginIndex);
	char currentChar;
	startAttributsPtr = beginIndex;

  if(quote =='"' || quote == '\'')
  {
		++startAttributsPtr;
		while(++beginIndex < buffLength)
		{
			currentChar = buff(beginIndex);
			if(currentChar == quote)
			{
				if(beginIndex >	endAttributsPtr)
					endAttributsPtr = beginIndex;
				return HTML_STRING_COMPLETE;
			}

			//if( !htmlIsWhiteSpace(currentChar) )
				str.append(currentChar, 1);
		}
	}
	else
	{
		do{
			currentChar = buff(beginIndex);
			if(htmlIsWhiteSpace(currentChar) || currentChar == '>')
			{
				if(beginIndex >	endAttributsPtr)
					endAttributsPtr = beginIndex;
				return HTML_STRING_COMPLETE;
			}
			str.append(currentChar, 1);
		}
		while(++beginIndex < buffLength);
	}
	return 	HTML_STRING_NOT_COMPLETE;
}


attState htmlFindNextAttribute(RVCString& buff, RVCString& attributeName, RVCString& attributeValue, size_t& buffPtr, size_t& attValueStart)
{
	size_t length = buff.length(), index;
	size_t attributeIndex = RV_NPOS, delimiterIndex = RV_NPOS,
		endAttributeIndex = RV_NPOS, equalSignIndex = RV_NPOS;
	char c;
	static	RVCRegexp sp("[>\\t\\r\\n ]"); // oooo

	//attValueStart = RV_NPOS;

	//! find attribute: RVCRegexp REatt("[-a-zA-Z0-9]+[\\t\\r\\n ]*[>=a-zA-Z0-9]?")
	//! define attribute name by attributeIndex and endAttributeIndex
	//! equalSignIndex points to "=" if any
	//! delimiterIndex points to ">" if any
	//! pay attention that index can be advanced in case of the illegal char
	for(index = buffPtr; index < length; ++index){
		c = buff(index);
		switch(c) {
				case '=':
					if (attributeIndex != RV_NPOS) {
						if (endAttributeIndex == RV_NPOS)
							endAttributeIndex = index;
						equalSignIndex = index;
					} else { //! Illegal char
						size_t i = buff.index(sp, index);
						if (i == RV_NPOS)
							return HTML_READ_MORE_DATA;
						if (buff(i) == '>') {
							delimiterIndex = i;
							break;
						}
						index = i;
					}

					break;

				case '>':
					if (attributeIndex != RV_NPOS && endAttributeIndex == RV_NPOS)
						endAttributeIndex = index;
					delimiterIndex = index;
					break;

				case ' ':
				case '\t':
				case '\n':
				case '\r':
				case '"': // 11/7/02 Sergei : fix for Shahar
					if (attributeIndex != RV_NPOS && endAttributeIndex == RV_NPOS)
						endAttributeIndex = index;
					continue;

				default:
					if (isalnum(c) || c == '-') {
						if (attributeIndex == RV_NPOS)
							attributeIndex = index;
						else if (endAttributeIndex != RV_NPOS)
							break;
					} else { //! Illegal char
						size_t i = buff.index(sp, index);
						if (i == RV_NPOS)
							return HTML_READ_MORE_DATA;
						if (buff(i) == '>') {
							delimiterIndex = i;
							break;
						}
						index = i;
					}
					continue;
		}
		break;
	}

	 if(attributeIndex != RV_NPOS && endAttributeIndex != RV_NPOS)
	{
		attributeName  = buff(attributeIndex, endAttributeIndex - attributeIndex);

		if( equalSignIndex == RV_NPOS )  //! no value !!
		{
			attributeValue = "";
			buffPtr = endAttributeIndex;
			return HTML_CURRENT_ATT_COMPLETE;
		} else {
			size_t endAttributsPtr = 0, startAttributsPtr = 0;
			strVal attVal = htmlGetStringValue(buff, attributeValue, equalSignIndex,
												startAttributsPtr, endAttributsPtr, length);
			if(attVal == HTML_STRING_NOT_COMPLETE)
				return HTML_READ_MORE_DATA;
			else//! attVal == HTML_STRING_COMPLETE
			{
				attValueStart = startAttributsPtr;
				buffPtr = endAttributsPtr;
				return HTML_CURRENT_ATT_COMPLETE;
			}
		}
	} else if(delimiterIndex != RV_NPOS) {
		buffPtr = delimiterIndex+1;
		return  HTML_FOUND_DELIMITER;
	} else
		return HTML_READ_MORE_DATA;
}

bool isElementOfType(const char* elementStr, size_t elementLength, RVCString& buff, char nextChar, CWLDocument *docPtr, bool CutBuffer)
{
	if(buff.length() < elementLength)
		return false;
	//RVCString elementInBuff = buff(0, len);  // Uri
	const char *c_buff = (const char *)buff;
	//if(elementInBuff.compareTo(elementStr, RVCString::ignoreCase) == 0 &&
	//	 (nextChar == buff(len) || nextChar == '~' || findSpace(buff(len))))
#ifdef __TV_UNIX__
	if(strncasecmp(c_buff, elementStr, elementLength) == 0 &&
		 (nextChar == buff(elementLength) || nextChar == '~' || findSpace(buff(elementLength))))
#else
	if(strnicmp(c_buff,elementStr,elementLength) == 0 &&
		 (nextChar == buff(elementLength) || nextChar == '~' || findSpace(buff(elementLength))))
#endif
	{
		if (CutBuffer) {
			docPtr->m_iBuffPos += elementLength;
			buff.remove(0, elementLength);
		}
		return true;
	}
	else
		return false;
	}


elementReadState skipHtmlBlock(RVCString& tempBuff,const char *closingTag, size_t closingTagLength, CWLDocument *docPtr)
{
	size_t tempBuffLength = tempBuff.length();
	size_t closingTagEndIndex = tempBuff.index(closingTag,0,RVCString::ignoreCase);
	if(tempBuffLength >=  closingTagLength && closingTagEndIndex != RV_NPOS)
	{
		docPtr->m_iBuffPos += closingTagEndIndex + closingTagLength;
		tempBuff.remove(0, closingTagEndIndex + closingTagLength);
		return END_OF_ELEMENT;
	}
	//!len < strlen(closingTag)  || no --> found
	return READ_MORE;
}


//!this method removes all mark-up from the innerHTML string
//! Automata states
#define H2TStart    0
#define H2TInTag    1
#define H2TInProp   2
#define H2TEndTag   3
#define H2TSpace    4

void Html2Text(RVCString &buff,int start_ptr,int end_ptr,RVCString &innerText)
{
	const char *c_data = ((const char *)buff);
	C_Html2Text(c_data,start_ptr,end_ptr,innerText);
}

//!this method removes all mark-up from the innerHTML string including extra spaces
void C_Html2Text(const char *c_data,int start_ptr,int end_ptr,RVCString &innerText)
{
	int state = H2TStart;
	bool spaceFound = false;
	size_t location1, location2, startTag = -1;
	innerText = ""; //!initialize
	if(end_ptr < start_ptr)
		return;
	for(location1 = start_ptr , location2 = location1 ; location1 <= size_t(end_ptr) ; ++location1)
	{
		switch(c_data[location1])
		{
		case ' ':
		case '\n':
		case '\r':
		case '\t':
			if( state == H2TStart )
			{
				spaceFound = true;
				startTag = location1;
				state = H2TSpace;
			}
			else if( state == H2TEndTag || state == H2TSpace )
			{
				spaceFound = true;
				state = H2TSpace;
			}
			break;

		case '<':
			if( state == H2TStart )
			{
				startTag = location1;
				state = H2TInTag;
			}
			else if( state == H2TEndTag || state == H2TSpace )
				state = H2TInTag;
			break;

		case '>':
			if( state == H2TEndTag || state == H2TSpace )
			{
				if(startTag > location2)
					innerText.append(c_data+location2,startTag-location2);
				location2 = location1;
				if( spaceFound )
				{
					innerText.append(' ',1);
					spaceFound = false;
				}
				state = H2TStart;
			}
			else if( state == H2TInTag )
				state = H2TEndTag;
			break;

		case '"':
			if( state == H2TEndTag || state == H2TSpace )
			{
				if(startTag > location2)
					innerText.append(c_data+location2,startTag-location2);
				location2 = location1;
				if( spaceFound )
				{
					innerText.append(' ',1);
					spaceFound = false;
				}
				state = H2TStart;
			}
			else if( state == H2TInTag )
				state = H2TInProp;
			else if( state == H2TInProp )
				state = H2TInTag;

			break;

		default:
			if( state == H2TEndTag || state == H2TSpace )
			{
				if(startTag > location2)
					innerText.append(c_data+location2,startTag-location2);
				location2 = location1;
				if( spaceFound )
				{
					innerText.append(' ',1);
					spaceFound = false;
				}
				state = H2TStart;
			}

			break;
		}  //! end of switch
	}  //! end of for
	if( state == H2TEndTag || state == H2TSpace )
	{
		if(startTag > location2)
			innerText.append(c_data+location2,startTag-location2);
	}
	else if( state == H2TStart )
		innerText.append(c_data+location2,location1-location2+1);

	//if( spaceFound )
	//	innerText.append(' ',1);
	wlConvertCharSet.convert(innerText);
}


//! the url(0) should be first letter of the protocol
bool isHttpProtocol(const RVCString &url)
{
	RVCRegexp REprotocol("^[a-zA-Z]+:");
	RVCString protocol = url(REprotocol);

	if (protocol.isNull())
		return true; //! no protocol was found, it's a relative url

	protocol.toLower();
	if (protocol(0,4) == "http")
		return true;
	else if (protocol(0,5) == "clsid")
		return true;
	else
		return false;
}

attribute kindOfAttribute(RVCString& upName)
{
  if(upName.length() > 1) //peter: for now all known table properties has more than 1 letter so if we get only one we'll crash on checking second.
	switch(upName(0)) {
	case 'a': case 'A':
		switch(upName(1)) {
		case 'c': case 'C':
			if (upName.compareTo("action", RVCString::ignoreCase) == 0)
			return ACTION;
			break;
		case 'l': case 'L':
			if (upName.compareTo("alt", RVCString::ignoreCase) == 0)
				return ALT;
			break;
		}
		break;

	case 'b': case 'B':
		switch(upName(1)) {
		case 'a': case 'A':
		if (upName.compareTo("background", RVCString::ignoreCase) == 0)
			return BACKGROUND;
			break;
		}
		break;

	case 'c': case 'C':
		upName.toLower();
		if(upName(1) && upName(2)) {
			switch(upName(2)) {
				case 'a':
					if (upName.compareTo("classid") == 0)
						return CLASSID;
					break;
				case 'e':
					if (upName.compareTo("checked") == 0)
						return CHECKED;
					break;
				case 'd':
					if (upName.compareTo("codebase") == 0)
						return CODEBASE;
					else if (upName.compareTo("codetype") == 0)
						return CODETYPE;
					else if (upName.compareTo("code") == 0)
						return CODE;
					break;
				case 'l':
					if (upName.compareTo("colspan") == 0)
						return COLSPAN;
					else if (upName.compareTo("cols") == 0)
						return COLS;
					break;
				case 'n':
					if (upName.compareTo("content") == 0)
						return CONTENT;
					break;
			}
		}
		break;

	case 'd':
	case 'D':
		switch(upName(1)) {
		case 'a': case 'A':
			if (upName.compareTo("data", RVCString::ignoreCase) == 0)
			return DATA;
			break;
		case 'y': case 'Y':
			if (upName.compareTo("dynsrc", RVCString::ignoreCase) == 0)
			return DYNSRC;
		break;
		}
		break;

	case 'e':
	case 'E':
		switch(upName(1)) {
		case 'n': case 'N':
			if (upName.compareTo("encoding", RVCString::ignoreCase) == 0)
			return ENCODING;
			else if (upName.compareTo("enctype", RVCString::ignoreCase) == 0)
			return ENCTYPE;
			break;
		case 'v': case 'V':
			if (upName.compareTo("event", RVCString::ignoreCase) == 0)
			return EVENT;
		break;
		}
		break;

	case 'h':
	case 'H':
		switch(upName(1)) {
		case 'e': case 'E':
			if (upName.compareTo("height", RVCString::ignoreCase) == 0)
				return HEIGHT;
			break;
		case 'r': case 'R':
			if (upName.compareTo("href", RVCString::ignoreCase) == 0)
			return HREF;
			break;
		case 't': case 'T':
			if (upName.compareTo("http-equiv", RVCString::ignoreCase) == 0)
			return HTTP_EQUIV;
		break;
		}
		break;

	case 'i':
	case 'I':
		switch(upName(1)) {
		case 'd': case 'D':
      // @@Migration
//			if(!upName(2))
			if(upName.length() == 2)
			return ID;
			break;
		}
		break;

	case 'l':
	case 'L':
		switch(upName(1)) {
		case 'a': case 'A':
			if (upName.compareTo("language", RVCString::ignoreCase) == 0)
			return LANGUAGE;
			break;
		case 'o': case 'O':
			if (upName.compareTo("lowsrc", RVCString::ignoreCase) == 0)
			return LOWSRC;
		break;
		}
		break;

	case 'm':
	case 'M':
		switch(upName(1)) {
		case 'e': case 'E':
			if (upName.compareTo("method", RVCString::ignoreCase) == 0)
			return METHOD;
			break;
		}
		break;

	case 'n':
	case 'N':
		switch(upName(1)) {
		case 'a': case 'A':
		if (upName.compareTo("name", RVCString::ignoreCase) == 0)
			return NAME;
			break;
		}
		break;

	case 'r':
	case 'R':
		switch(upName(1)) {
		case 'e': case 'E':
		if (upName.compareTo("rel", RVCString::ignoreCase) == 0)
			return REL;
		case 'o': case 'O':
		if (upName.compareTo("rowspan", RVCString::ignoreCase) == 0)
			return ROWSPAN;
			break;
		}
		break;

	case 's':
	case 'S':
		switch(upName(1)) {
		case 'r': case 'R':
		if (upName.compareTo("src",  RVCString::ignoreCase) == 0)
			return SRC;
			break;
		}
		break;

	case 't':
	case 'T':
		switch(upName(1)) {
		case 'a': case 'A':
			if (upName.compareTo("target",  RVCString::ignoreCase) == 0)
			return TARGET;
		break;
		case 'i': case 'I':
			if (upName.compareTo("title",  RVCString::ignoreCase) == 0)
				return TITLE;
			break;
		case 'y': case 'Y':
			if (upName.compareTo("type",  RVCString::ignoreCase) == 0)
				return TYPE;
			break;
		}
		break;

	case 'u':
	case 'U':
		switch(upName(1)) {
		case 'n': case 'N':
		if (upName.compareTo("unchecked", RVCString::ignoreCase) == 0)
			return UNCHECKED;
			break;
		case 's': case 'S':
			if (upName.compareTo("usemap", RVCString::ignoreCase) == 0)
				return USEMAP;
			break;
		}
		break;

	case 'v':
	case 'V':
		switch(upName(1)) {
		case 'a': case 'A':
		if (upName.compareTo("value", RVCString::ignoreCase) == 0)
			return VALUE;
			break;
		}
		break;

	case 'w':
	case 'W':
		switch(upName(1)) {
		case 'i': case 'I':
			if (upName.compareTo("width", RVCString::ignoreCase) == 0)
				return WIDTH;
			break;
		}
		break;

	}
	return UNKNOWN;
}

attState htmlFindNextAttribute_old(RVCString& buff, RVCString& attributeName, RVCString& attributeValue, size_t& buffPtr)
{
	size_t att_len;
	RVCRegexp REatt("[-a-zA-Z0-9]+[\\t\\r\\n ]*[>=a-zA-Z0-9]?");
	size_t delimiterIndex = buff.index(">", buffPtr);
	size_t attributeIndex = buff.index(REatt, &att_len, buffPtr);
	if(attributeIndex != RV_NPOS && ((delimiterIndex == RV_NPOS) || (attributeIndex < delimiterIndex)))
	{
		//size_t buffLength = buff.length();
		//htmlSkipWhiteSpaces(buff, buffLength, attributeIndex);
		size_t endAttributeIndex = attributeIndex + 1;

/*		while(endAttributeIndex < attributeIndex + att_len &&
			  buff(endAttributeIndex) != '=' &&
			  buff(endAttributeIndex) != '>' &&
			  !htmlIsWhiteSpace(buff(endAttributeIndex)))
				++endAttributeIndex;*/

		for (; endAttributeIndex < attributeIndex + att_len; ++endAttributeIndex) {
			switch(buff(endAttributeIndex)) {
				case '=': break;
				case '>': break;
				case ' ': break;
				case '\t': break;
				case '\n': break;
				case '\r': break;
				default: continue;
			}
			break;
		}


		attributeName  = buff(attributeIndex, endAttributeIndex - attributeIndex);
		if( buff(attributeIndex+att_len-1 ) != '=' )  //! no value !!
		{
			attributeValue = "";
			buffPtr = endAttributeIndex;
			return HTML_CURRENT_ATT_COMPLETE;
		}
		else
		{
			size_t endAttributsPtr = 0;
/* oooo			strVal attVal = htmlGetStringValue(buff, attributeValue, endAttributeIndex, endAttributsPtr);
			if(attVal == HTML_STRING_NOT_COMPLETE)
				return HTML_READ_MORE_DATA;
			else// attVal == HTML_STRING_COMPLETE
			{
				buffPtr = endAttributsPtr;
				return HTML_CURRENT_ATT_COMPLETE;
			}
*/
		}
	}
	else if(delimiterIndex != RV_NPOS)
	{
		buffPtr = delimiterIndex+1;
		return  HTML_FOUND_DELIMITER;
	}
	else return HTML_READ_MORE_DATA;

return HTML_READ_MORE_DATA;
}

void crash_func()
{
	int *ptr = NULL;
	*ptr = 666;
}
