/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include "wllocation.h"
#include "wlhtmlvirtualqueue.h"
#include "wldocument.h"

class CWLApplet;

CWLLocation::CWLLocation(RVCString& url, RVCString& origUrl, RVCString& referrer,
						 const char* linkName, const char* id, const char* target) :
  CWLUrl(NULL, url, origUrl, referrer, linkName, id, target)
{}

CWLLink::CWLLink(CWLDocument *docPtr, RVCString& url, RVCString& origUrl,
			   const char* linkName,  const char* id, const char* target) :
  CWLUrl(docPtr, url, origUrl, docPtr->GetDocUrl(), linkName, id, target)
{
	m_InnerImg = NULL;
	m_InnerText = NULL;
	m_Index = docPtr->m_Links->entries();
}

CWLLink::CWLLink(RVCString& url, RVCString& origUrl, RVCString& referrer,
			   const char* linkName, const char* innerHTML, const char* id, const char* target) :
  m_Index(-1), m_InnerHTML(innerHTML), m_InnerText(NULL),
  CWLUrl(NULL, url, origUrl, referrer, linkName, id, target)
{
	  m_InnerImg = NULL;
}

RVCSubString CWLLink::GetInnerHTML()
{
  if(m_InnerHTML.isNull())
    return CWLUrl::GetInnerHTML();
  else
    return m_InnerHTML(0,m_InnerHTML.length());
}

const char* CWLLink::GetInnerText()
{
  static char empty_str[1] = "";

	if (m_InnerText) {
		return m_InnerText->data();	
	} else if (!m_InnerHTML.isNull()) {
		m_InnerText = new RVCString;
		C_Html2Text(m_InnerHTML,0, m_InnerHTML.length()-1,*m_InnerText);
		return m_InnerText->data();		
	} else if(m_DocPtr && m_DocPtr->m_ParseInnerHTML) {
	    return CWLUrl::GetInnerText();
	} else {
		return empty_str;
	}
}

CWLLink::CWLLink(const CWLLink& aCWLLink)
: CWLUrl(*(CWLUrl*)&aCWLLink)
{
   m_Index = aCWLLink.m_Index;
   m_InnerImg = aCWLLink.m_InnerImg; // it's ok since deletion in the img
    m_InnerHTML = aCWLLink.m_InnerHTML;
    m_InnerText = (aCWLLink.m_InnerText) ? new RVCString(*aCWLLink.m_InnerText) : NULL;
}

CWLLink::~CWLLink()
{
	if (m_InnerText)
		delete m_InnerText;
}

CWLLink&	CWLLink::operator=(const CWLLink& aCWLLink)
{
    CWLUrl::operator =(*(CWLUrl*)&aCWLLink);
	  m_Index = aCWLLink.m_Index;
    m_InnerImg = aCWLLink.m_InnerImg; // it's ok since deletion in the img
   m_InnerHTML = aCWLLink.m_InnerHTML;
    if (m_InnerText)
      delete m_InnerText;
    m_InnerText = (aCWLLink.m_InnerText) ? new RVCString(*aCWLLink.m_InnerText) : NULL;
//	*(CWLUrl*)this = (*(CWLUrl*)&aCWLLink);
	return (*this);
}

 void CWLLink::printAll()
{
	CWLUrl::printAll();
	if (m_InnerImg) {
		cout<<"InnerImg:----------------------"<< endl;
		m_InnerImg->printAll();
	} else
		cout << "Inner Img: no InnerImg" << endl;
}

CWLCss::CWLCss(RVCString& url, RVCString& origUrl, CWLDocument *docPtr,
			   const char* linkName, const char* id) :
  CWLUrl(docPtr, url, origUrl, docPtr->GetDocUrl(), linkName, id, ""), m_TransObj(NULL)
{
	  m_TransObj.SetObj(this);
	  m_TransObj.SetObjType(LINK);
	  m_Index = docPtr->m_StyleLinks->entries();
	  m_TransObj.GetTransaction()->SetName(url);
	  m_iUrlOffset = -1;
}

CWLCss::CWLCss(const CWLCss& aCWLCss)
: CWLUrl(*(CWLUrl*)&aCWLCss), m_TransObj(aCWLCss.m_TransObj)
{
	m_Index = aCWLCss.m_Index;
  m_iUrlOffset = aCWLCss.m_iUrlOffset;
	//*(CWLUrl*)this = (*(CWLUrl*)&aCWLCss);
}

CWLCss&	CWLCss::operator=(const CWLCss& aCWLCss)
{
  CWLUrl::operator =(*(CWLUrl*)&aCWLCss);
	m_Index = aCWLCss.m_Index;
  m_iUrlOffset = aCWLCss.m_iUrlOffset;
  m_TransObj = aCWLCss.m_TransObj;
	//*(CWLUrl*)this = (*(CWLUrl*)&aCWLCss);
	return (*this);
}
CWLEmbed::~CWLEmbed()
{
	if(attrName)
		delete attrName;
	if(tagName)
		delete tagName;

	if (m_pWidth)
		delete m_pWidth;
	if (m_pHeight)
		delete m_pHeight;
	if (m_pAlt)
		delete m_pAlt;
	if (m_pTitle)
		delete m_pTitle;
	if (m_pOrigUseMap) {
		delete m_pUseMap;
		delete m_pOrigUseMap;
	}
	if (m_pOrigLowsrc) {
		delete m_pLowsrc;
		delete m_pOrigLowsrc;
	}
}

CWLEmbed::CWLEmbed(const char* aTagName, const char* anAttrName,
				   RVCString& url, RVCString& origUrl, CWLDocument *docPtr,
				   const char* linkName, const char* id, const char* target) :
  CWLUrl(docPtr, url, origUrl, docPtr->GetDocUrl(), linkName, id, target), m_TransObj(NULL)
{
	  m_TransObj.SetObj(this);
	  m_TransObj.GetTransaction()->SetName(url);

	  m_iUrlOffset = -1;

	  tagName = (aTagName)    ? new RVCString(aTagName)  : NULL;
	  attrName = (anAttrName) ? new RVCString(anAttrName) : NULL;

	  if (tagName) {
		  if (tagName->compareTo("embed", RVCString::ignoreCase) == 0) {
			m_TransObj.SetObjType(EMBED);
			m_Index = m_DocPtr->m_EmbedList->entries();
		  } else {
			m_TransObj.SetObjType(IMAGE);
      m_Index = m_DocPtr->m_SrcList ? m_DocPtr->m_SrcList->entries() : -1 ;
		  }
	  }

	m_pWidth  = NULL;
	m_pHeight = NULL;
	m_pAlt    = NULL;
	m_pTitle  = NULL;
	m_pUseMap = NULL;
	m_pLowsrc = NULL;
	m_pOrigUseMap = NULL;
	m_pOrigLowsrc = NULL;
}

static void CopyRWCString(RVCString*& d, RVCString*const & s)
{
	if (d) {
		if (s)
			*d = *s; // RVCString operator=
		else
			d->remove(0);
	} else {
		if (s)
			d = new RVCString(*s);
	}
}

CWLEmbed::CWLEmbed( const CWLEmbed& aCWLEmbed)
: CWLUrl(*(CWLUrl*)&aCWLEmbed), 	m_TransObj(aCWLEmbed.m_TransObj)
{
	m_Index = aCWLEmbed.m_Index;
    m_iUrlOffset = aCWLEmbed.m_iUrlOffset;

	attrName = (aCWLEmbed.attrName) ? new RVCString(*aCWLEmbed.attrName) : NULL;
	attrName = (aCWLEmbed.tagName) ? new RVCString(*aCWLEmbed.tagName) : NULL;
	attrName = (aCWLEmbed.m_pAlt) ? new RVCString(*aCWLEmbed.m_pAlt) : NULL;
	attrName = (aCWLEmbed.m_pHeight) ? new RVCString(*aCWLEmbed.m_pHeight) : NULL;
	attrName = (aCWLEmbed.m_pLowsrc) ? new RVCString(*aCWLEmbed.m_pLowsrc) : NULL;
	attrName = (aCWLEmbed.m_pOrigLowsrc) ? new RVCString(*aCWLEmbed.m_pOrigLowsrc) : NULL;
	attrName = (aCWLEmbed.m_pOrigUseMap) ? new RVCString(*aCWLEmbed.m_pOrigUseMap) : NULL;
	attrName = (aCWLEmbed.m_pTitle) ? new RVCString(*aCWLEmbed.m_pTitle) : NULL;
	attrName = (aCWLEmbed.m_pUseMap) ? new RVCString(*aCWLEmbed.m_pUseMap) : NULL;
	attrName = (aCWLEmbed.m_pWidth) ? new RVCString(*aCWLEmbed.m_pWidth) : NULL;
}

CWLEmbed&	CWLEmbed::operator=( const CWLEmbed& aCWLEmbed)
{
  CWLUrl::operator =(*(CWLUrl*)&aCWLEmbed);
	m_Index = aCWLEmbed.m_Index;
    m_iUrlOffset = aCWLEmbed.m_iUrlOffset;
	CopyRWCString(attrName, aCWLEmbed.attrName);
	CopyRWCString(tagName, aCWLEmbed.tagName);
	CopyRWCString(m_pAlt, aCWLEmbed.m_pAlt);
	CopyRWCString(m_pHeight, aCWLEmbed.m_pHeight);
	CopyRWCString(m_pLowsrc, aCWLEmbed.m_pLowsrc);
	CopyRWCString(m_pOrigLowsrc, aCWLEmbed.m_pOrigLowsrc);
	CopyRWCString(m_pOrigUseMap, aCWLEmbed.m_pOrigUseMap);
	CopyRWCString(m_pTitle, aCWLEmbed.m_pTitle);
	CopyRWCString(m_pUseMap, aCWLEmbed.m_pUseMap);
	CopyRWCString(m_pWidth, aCWLEmbed.m_pWidth);
	m_TransObj = aCWLEmbed.m_TransObj;	//*(CWLUrl*)this = (*(CWLUrl*)&aCWLEmbed);
  return *this;
}

void CWLEmbed::SetProperties(RVCString& p_Width, RVCString& p_Height, RVCString& p_Alt,
						  RVCString& p_Title, RVCString& p_UseMap, RVCString& p_Lowsrc,
						  RVCString& p_OrigUseMap, RVCString& p_OrigLowsrc)
{
	if (!p_Width.isNull())
		m_pWidth = new RVCString(p_Width);
	if (!p_Height.isNull())
		m_pHeight = new RVCString(p_Height);
	if (!p_Alt.isNull())
		m_pAlt = new RVCString(p_Alt);
	if (!p_Title.isNull())
		m_pTitle = new RVCString(p_Title);
	if (!p_OrigUseMap.isNull()) {
		m_pUseMap = new RVCString(p_UseMap);
		m_pOrigUseMap = new RVCString(p_OrigUseMap);
	}
	if (!p_OrigLowsrc.isNull()) {
		m_pLowsrc = new RVCString(p_Lowsrc);
		m_pOrigLowsrc = new RVCString(p_OrigLowsrc);
	}
}

const char* CWLEmbed::Tag()
{
	if (tagName)
		return tagName->data();
	else
		return NULL;
}

const char* CWLEmbed::Attr()
{
	if (attrName)
		return attrName->data();
	else
		return NULL;
}

/*CWLApplet& operator=(const CWLApplet& aCWLApplet)
{
	if(!aCWLApplet.isEmpty())
		for(size_t i = 0; i < aCWLApplet.entries(); ++i)
		{
			CWLParam *param = new CWLParam((CWLParam*)aCWLApplet.at(i));
			insert(param);
		}
	return (*this);
}*/

CWLApplet::CWLApplet(CWLDocument* docPtr)
	: m_DocPtr(docPtr), m_TransObj(NULL)
{
	m_TransObj.SetObj(this);
	m_TransObj.SetObjType(APPLET);
	m_Index = docPtr->m_AppletList->entries();
	m_iCodebaseOffset = -1;
	m_iCodeOffset = -1;
	m_iClassidOffset = -1;
	m_iSrcOffset = -1;
	m_iDataOffset = -1;

}

RVCString& CWLApplet::GetReferrer()
{
	return m_DocPtr->GetDocUrl();
}

elementReadState CWLApplet::parse(RVCString& buff, urlBase* urlBasePtr, size_t appletEndIndex, const char *tagName)
{
	size_t buffPtr = 0, attValueStart = RV_NPOS, attValueStartF = RV_NPOS;
	RVCString attName, attValue;
	attState state;
	urlBase *base;
	bool noCodeBase;

	if(tagName)
		m_TagName = tagName;

	while(1) {
		state = htmlFindNextAttribute(buff, attName, attValue, buffPtr, attValueStart);
		if(state == HTML_CURRENT_ATT_COMPLETE) {
			switch(kindOfAttribute(attName)) {
			case CLASSID:
				this->setClassidOffset(m_DocPtr->m_iBuffPos + attValueStart);
				m_Classid = attValue;
				break;
			case CODE:
				this->setCodeOffset(m_DocPtr->m_iBuffPos + attValueStart);
				m_Code = attValue;
				break;
			case CODEBASE:
				attValueStartF = attValueStart; // oooo only this attr goes to transaction object
				this->setCodebaseOffset(m_DocPtr->m_iBuffPos + attValueStart);
				if (attValue.length() > 0) {
					if (attValue(attValue.length()-1) != '/')
    					attValue.append("/");
					m_Codebase.setBase(urlBasePtr->calcUrl(attValue));
					m_OrigUrl = attValue;
				}
				break;
			case CODETYPE:
				m_Codetype = attValue;
				break;
			case DATA:
				this->setDataOffset(m_DocPtr->m_iBuffPos + attValueStart);
				m_Data = attValue;
				break;
			case ID:
				m_Id = attValue;
				break;
			case NAME:
				m_Name = attValue;
			case SRC:
				this->setSrcOffset(m_DocPtr->m_iBuffPos + attValueStart);
				m_Src = attValue;
				break;
			case TYPE:
				m_Type = attValue;
				break;
			}

		} else if (state == HTML_FOUND_DELIMITER) {

			noCodeBase = m_Codebase.returnBase().isNull();

			if (noCodeBase)
				base = urlBasePtr;
			else
				base = &m_Codebase;

/*	src attribute doesn't work in IE5.0
			if (!m_Src.isNull())
				m_Url = base->calcUrl(m_Src);
			else */
			if (!m_Code.isNull()) {
				m_Url = base->calcUrl(m_Code);
				m_OrigUrl.append(m_Code);
			} else if (!m_Data.isNull()) {
				m_Url = base->calcUrl(m_Data);
				m_OrigUrl.append(m_Data);
/*			} else if (!m_Classid.isNull()) {
				m_Url = base->calcUrl(m_Classid);*/
			} else if (!noCodeBase)
				m_Url = base->returnBase();


      if(!m_Url.isNull() && m_Classid.isNull()) //! classid - attribute of the activex object (currently no care about)
      {
		m_TransObj.GetTransaction()->SetName(m_Url);
		CWLTransactionsObject* TOptr = this->GetTransactionObj();
		if (attValueStartF != RV_NPOS)
			TOptr->SetUrlOffset(m_DocPtr->m_iBuffPos + attValueStartF);
		m_DocPtr->insertTransObjToQueue(TOptr);
      }

			buffPtr = buff.first('>') + 1;
			m_Text  = buff(buffPtr, appletEndIndex - buffPtr);

			return END_OF_ELEMENT;
		} else
			return READ_MORE; //! Error: the buff should include whole applet body
	}
	return READ_MORE;
}

 void CWLApplet::printAll()
{
	RVCString &source = *(m_DocPtr->m_HtmlPagePtr);

	cout<<"ID:       "<<m_Id.data()<<'\n';
	cout<<"URL:      "<<m_Url.data()<<'\n';
	cout<<"ORIGURL:  "<<m_OrigUrl.data()<<'\n';
	cout<<"CODEBASE: "<<m_Codebase.returnBase()<<'\n';
	if (m_iCodebaseOffset != -1) {
		RVCString url = source(m_iCodebaseOffset, 25);
		cout <<"CODEBASE2: "<< url << endl;
	}
	cout<<"CODE:     "<<m_Code.data()<<'\n';
	cout<<"CLASSID:  "<<m_Classid.data()<<'\n';
	cout<<"SRC:      "<<m_Src.data()<<'\n';
	cout<<"DATA:     "<<m_Data.data()<<'\n';
	cout<<"CODETYPE  "<<m_Codetype.data()<<'\n';
	cout<<"TEXT:     "<<'\n'<<m_Text.data()<<'\n'<<'\n';
}
