/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlobjectcollection.h"
#include "wllocation.h"

//! Perform Unify (search) in the collection.
//! Currently the search is done using a CWLLink objects
RVCollectable* CWLObjectCollection::unifyObject(RVCollectable *comp, int expectedIndex)
{
	CWLLink  *base = (CWLLink *)comp;
	CWLLink  *iterator;
	int      index;
	CWLUrlCmp *results = new CWLUrlCmp[entries()];
	int  grade,propGrade,bestGrade = WORST_GRADE;
	int  bestResult,bestPropGrade = -1;

	//! first look at the Expected Location (Dudu's location)
	if( expectedIndex < int(entries()) )
	{
		iterator = (CWLLink *)comparedObject(expectedIndex);
		if( iterator != NULL )
		{
			if( base->urlCompare(*iterator, results[expectedIndex]) ) //! possible match
				if( results[expectedIndex].exactMatch )  //!exact Match
				{
					delete [] results;
					return at(expectedIndex);
				}
				else
				{  //! Take him as the best so far...
					bestResult = expectedIndex;
					bestGrade = results[expectedIndex].rank();
					//base->propertiesCompare(*iterator, results[expectedIndex]);
					bestPropGrade = results[expectedIndex].propRank();
				}
			else  //! failure. collect properties grade
			{
				bestResult = expectedIndex;
				//base->propertiesCompare(*iterator, results[expectedIndex]);
				bestPropGrade = results[expectedIndex].propRank();
			}
		}
	}

	bool leftSide=true;
	int left_index, right_index, left_border=-1, right_border=entries();

	if (expectedIndex >= right_border-1) {
		right_index = right_border; //! no move righter
		left_index = right_border-1;
	} else {
		//!right_index = expectedIndex+1;
		right_index = expectedIndex;
		left_index = expectedIndex-1;
	}

	//! Loop on all objects and try to find the best
	for(index = left_index;;)
	{
		//!========== CONDITION ======================
		if (left_index == left_border && right_index == right_border)
			break;
		//!-------------------------------------------
		else if (index != left_border && index != right_border) {
			iterator = (CWLLink *)comparedObject(index);
			if( iterator != NULL )
			{
				if( base->urlCompare(*iterator, results[index]) ) //! possible match
					if( results[index].exactMatch )  //!exact Match
					{
						delete [] results;
						return at(index);
					}
					else
					{ //! calculate grades
						grade = results[index].rank();
						//base->propertiesCompare(*iterator, results[index]);
						propGrade = results[index].propRank();
						if( grade < bestGrade || (grade == bestGrade && propGrade > bestPropGrade) )  //! found better one !
						{
							bestResult = index;
							bestGrade = grade;
							bestPropGrade = propGrade;
						}
					}
				else if( bestGrade == WORST_GRADE )  //! still with no REAL candidate
				{
					//base->propertiesCompare(*iterator, results[index]);
					propGrade = results[index].propRank();
					if( propGrade > bestPropGrade ) //! better "POOR" candidate
					{
						bestPropGrade = propGrade;
						bestResult = index;
					}
				}
			}
		}
		//!=========== ITERATION ========================
		if (leftSide) {
			if (right_index == right_border) {
				index = --left_index; //! continue to the left
			} else {
				leftSide = false;	  //! switch to the right
				index = ++right_index; 
			}
		} else {
			if (left_index == left_border) {
				index = ++right_index; //! continue to the right
			} else {
				leftSide = true;       //! switch to the left
				index = --left_index;
			}
		}
	}  //! of loop on all objects

	delete [] results;
	//! if found something, return it.
	if( bestGrade < WORST_GRADE || bestPropGrade > 0 )
		return at(bestResult);
	else
		return NULL;
}
