/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <collint.h>
#include "wlregularexpressions.h"
#include "wlurl.h"
#include "wlhtmlfuncs.h"
#include "charset.h"
#include "htmlvalues.h"
#include <assert.h>
#include "wldocument.h"

extern CWLCharSet wlConvertCharSet;


//================================CWLUrl================================
char *CWLUrl::extenssionCharacters = ";,!@$."; //! if you change this, remember to change the define !!!
#define NUM_EXTENSSION_CHARS 6

CWLUrl::CWLUrl()
{
	m_Name      = NULL;
	m_Target    = NULL;
	m_Id	    = NULL;
//	m_InnerHTML	= NULL;
	m_InnerText = NULL;
	m_HostDirty = m_HrefDirty = false;
	m_UrlAnalyzed = false;
	m_SearchAnalyzed = false;
	m_PortValid  = false;
	m_HrefValid  = true;
	m_Source = NULL;
    m_StartInnerHTML = -1;
    m_LenInnerHTML = -1;
	m_DocPtr = NULL;
}

CWLUrl::CWLUrl(CWLDocument *docPtr)
{
	m_DocPtr = docPtr;
	m_Name      = NULL;
	m_Target    = NULL;
	m_Id	    = NULL;
//	m_InnerHTML	= NULL;
	m_InnerText = NULL;
	m_HostDirty = m_HrefDirty = false;
	m_UrlAnalyzed = false;
	m_SearchAnalyzed = false;
	m_PortValid  = false;
	m_HrefValid  = true;
	m_Source = NULL;
    m_StartInnerHTML = -1;
    m_LenInnerHTML = -1;
}

CWLUrl::CWLUrl(CWLDocument* docPtr, RVCString& href, RVCString& origUrl, RVCString& referrer, const char* name, const char* id, const char* target)
{
	m_DocPtr = docPtr;
	int location = 0;
	m_Port = 80;
	m_Href = href.data(); //!  =  =  protocol://host/uri?search#hash
	//! remove control chars (newline etc.)
	removeNewLine(m_Href,false);  //! actual remove the chars
	//! replace a space with "%20"
	while((location = m_Href.index(" ", location) ) != RV_NPOS)
		m_Href.replace(location, 1, "%20");
	//! expand any HTML macro (like &amp;)
	wlConvertCharSet.convert(m_Href);

	if( name )
		m_Name = strdup(name);
	else
		m_Name = NULL;
	if( target )
		m_Target = strdup(target);
	else
		m_Target = NULL;
	if( id )
		m_Id = strdup(id);
	else
		m_Id = NULL;
	//if( innerHTML )
//		m_InnerHTML = strdup(innerHTML);
//	else
//		m_InnerHTML = NULL;
	m_InnerText = NULL;
	m_HostDirty = m_HrefDirty = false;
	m_UrlAnalyzed = false;
	m_SearchAnalyzed = false;
	m_PortValid  = false;
	m_HrefValid  = true;
	m_Source = strdup(href.data());

  m_OriginalUrl = origUrl;
  m_Referrer = referrer;

  m_StartInnerHTML = -1;
  m_LenInnerHTML = -1;
}

CWLUrl::~CWLUrl()
{
	if( m_Name != NULL )
		free(m_Name);
	if( m_Target != NULL )
		free(m_Target);
	if (m_Source != NULL)
		free(m_Source);
	if( m_Id != NULL )
	  free(m_Id);
//	if( m_InnerHTML != NULL)
//	  free(m_InnerHTML);
	if( m_InnerText != NULL)
	  delete m_InnerText;

	m_Kvps.clearAndDestroy();
}

CWLUrl::CWLUrl( const CWLUrl& aCWLUrl)
{
	m_Port = 80;

	if( aCWLUrl.m_Name )
		m_Name = strdup(aCWLUrl.m_Name);
	else
		m_Name = NULL;

	if( aCWLUrl.m_Target )
		m_Target = strdup(aCWLUrl.m_Target);
	else
		m_Target = NULL;

	if( aCWLUrl.m_Id )
		m_Id = strdup(aCWLUrl.m_Id);
	else
		m_Id = NULL;

	m_StartInnerHTML = aCWLUrl.m_StartInnerHTML;
	m_LenInnerHTML   = aCWLUrl.m_LenInnerHTML;

	if( aCWLUrl.m_InnerText )
		m_InnerText = new RVCString(*aCWLUrl.m_InnerText);
  else
    m_InnerText = NULL;

	m_Href = aCWLUrl.m_Href;
	m_HostDirty = m_HrefDirty = false;
	m_UrlAnalyzed = false;
	m_SearchAnalyzed = false;
	m_PortValid  = false;
	m_HrefValid  = true;

  if(aCWLUrl.m_Source)
	  m_Source = strdup(aCWLUrl.m_Source);
  else
    m_Source = NULL;

  m_DocPtr = aCWLUrl.m_DocPtr;
}

CWLUrl&	CWLUrl::operator=(const CWLUrl& aCWLUrl)
{
	m_Port = 80;
	if (m_Name)
		free(m_Name);
	if( aCWLUrl.m_Name )
		m_Name = strdup(aCWLUrl.m_Name);
	else
		m_Name = NULL;

	if (m_Target)
		free(m_Target);
	if( aCWLUrl.m_Target )
		m_Target = strdup(aCWLUrl.m_Target);
	else
		m_Target = NULL;
	if (m_Id)
		free(m_Id);
	if( aCWLUrl.m_Id )
		m_Id = strdup(aCWLUrl.m_Id);
	else
		m_Id = NULL;

	m_StartInnerHTML = aCWLUrl.m_StartInnerHTML;
	m_LenInnerHTML   = aCWLUrl.m_LenInnerHTML;

  if(m_InnerText)
    delete m_InnerText;
	if( aCWLUrl.m_InnerText )
		m_InnerText = new RVCString(*aCWLUrl.m_InnerText);
  else
    m_InnerText = NULL;

	m_Href = aCWLUrl.m_Href;
	m_HostDirty = m_HrefDirty = false;
	m_UrlAnalyzed = false;
	m_SearchAnalyzed = false;
	m_PortValid  = false;
	m_HrefValid  = true;

  if (m_Source)
		free(m_Source);
  if (aCWLUrl.m_Source)
	  m_Source = strdup(aCWLUrl.m_Source);
  else
    m_Source = NULL;

	m_Kvps.clearAndDestroy();

	 m_DocPtr = aCWLUrl.m_DocPtr;
	return (*this);
}


 int CWLUrl::analyzeHref()
{
	size_t href_length;
    const char *extPtr;

	size_t nonExtLen, extLen, filePart_len;

	m_UrlAnalyzed = true;
	m_SearchAnalyzed = false;
	m_Kvps.clearAndDestroy();
	if(m_Href.isNull())
		return -1;
	bool question = false;
	size_t mailto;
	href_length = m_Href.length();

	if((mailto = m_Href.index("mailto:",0, RVCString::ignoreCase)) != RV_NPOS)
	{
		m_Href = m_Href(mailto, href_length - mailto);
		return 0;
	}
	int diez = 0, slash = 0;
	size_t urlHost      = m_Href.index("://");
	if(urlHost == RV_NPOS)  //!error
	{
		m_HrefValid = false;
		return -1;
	}
	m_Protocol.setPart(m_Href.data(),0, urlHost + 3);
	size_t urlDiez = m_Href.last('#');
	if ((urlDiez != RV_NPOS) && (urlDiez < href_length -1))
	{
		m_Hash.setPart(m_Href.data(),urlDiez + 1, href_length - urlDiez - 1);
		href_length -= m_Hash.length() + 1;
	}
	else
		m_Hash.resetPart();

	size_t urlQuestion = m_Href.first('?');
	if ((urlQuestion != RV_NPOS) && (urlQuestion < href_length))
	{
		question = true;
		m_Search.setPart(m_Href.data(),urlQuestion+1, href_length - urlQuestion - 1);
		href_length -= m_Search.length() + 1;
	}
	else
		m_Search.resetPart();

	size_t urlSlash = m_Href.index("/", urlHost + 3);
	if((urlSlash != RV_NPOS) && (urlSlash < href_length))
	{
		m_Uri.setPart(m_Href.data(),urlSlash, href_length - urlSlash );
		href_length -= m_Uri.length();

		const char *tmpPtr = m_Href.data()+urlSlash;

		for(const char *f_start = tmpPtr ;
			f_start != NULL && f_start < m_Href.data() + urlSlash + m_Uri.length() ;
			f_start = strchr(f_start,'/') )
			{
				tmpPtr = f_start;
				++f_start;
			}
		size_t lastSlash = (size_t) (tmpPtr - m_Href.data());

		if(lastSlash == (urlSlash+m_Uri.length()-1))  //! www.rad.com/path/
		{
			m_Path.setPart(m_Href.data(),urlSlash,m_Uri.length());
			m_File.resetPart();
			m_FileExt.resetPart();
		}
		else
		{
			if(lastSlash == urlSlash)  //!  www.rad.com/file  or  www.rad.com/
			{
				m_Path.setPart(m_Href.data(),urlSlash,1);
				filePart_len = m_Uri.length()-1;
			}
			else    //! www.rad.com/path/file
			{
				m_Path.setPart(m_Href.data(),urlSlash,lastSlash-urlSlash+1);
				filePart_len = m_Uri.length()-m_Path.length();
			}

			for(int chrIdx = 0 ; chrIdx < NUM_EXTENSSION_CHARS ; ++chrIdx)
			{
				char ch = extenssionCharacters[chrIdx];
				extPtr = strchr(m_Href.data()+lastSlash+1,ch);
				nonExtLen = (size_t) (extPtr - (m_Href.data()+lastSlash+1));
				if( extPtr != NULL  && nonExtLen < filePart_len)
					break;
				else
					extPtr = NULL;
			}
			if( extPtr != NULL )
			{
				extLen = filePart_len - nonExtLen;
				filePart_len -= extLen;
				m_File.setPart(m_Href.data(),lastSlash+1,filePart_len);
				m_FileExt.setPart(m_Href.data(),(extPtr - m_Href.data()),extLen);
			}
			else
			{
				m_File.setPart(m_Href.data(),lastSlash+1,filePart_len);
				m_FileExt.resetPart();
			}

		}
	}
	else
	{
		m_Uri.updatePart("/");
		m_File.resetPart();
		m_FileExt.resetPart();
		m_Path.updatePart("/");
	}


	m_Host.setPart(m_Href.data(),urlHost + 3, href_length - urlHost - 3);
	m_Hostname.setPart(m_Href.data(),urlHost + 3, href_length - urlHost - 3);
	size_t urlPort = m_Href.index(":",urlHost + 3 );
	if(urlPort != RV_NPOS && urlPort < href_length)
	{
		RVCString portStr = m_Href(urlPort + 1, href_length - urlPort -1);
		m_Port = (unsigned)atoi(portStr.data());//! convert to unsigned int
		if(portStr.data()[0] && !portStr.isNull())
			m_PortValid = true;
		if(portStr.length() > 4)
		{
			m_PortValid = m_HrefValid = false;
			return -1;
		}
		m_Hostname.setPart(m_Href.data(),urlHost + 3, urlPort - urlHost - 3);
		if(m_Hostname.length() < 1)
		{
			m_HrefValid = false;
			return -1;//!error - no hostname
		}
	}
	else
	{
		m_PortValid = false;
		m_Port = 80;
	}
	if(question)
	{
		size_t nextQptr = m_Href.last('?');
		if(nextQptr != RV_NPOS &&
		   nextQptr != urlQuestion &&
		   nextQptr < (urlQuestion + m_Search.length() - 1))
		{
			m_Commands.setPart(m_Href.data(),urlQuestion, nextQptr - urlQuestion);
			m_Search.setPart(m_Href.data(),nextQptr + 1,m_Search.length()-m_Commands.length());
		}
		else
			m_Commands.resetPart();
	}

	// Calculate InnerText
/*	if( m_InnerHTML )
	{
		if( ! m_InnerText )
			m_InnerText = new RVCString;
		C_Html2Text(m_InnerHTML,0,strlen(m_InnerHTML)-1,*m_InnerText);
	}*/

    return 0;
}


void CWLUrl::rebuild()    //!recalculate m_Href
{
	const char *part;
	int         partLength;

	m_Href = "";  //! clear
	m_Protocol.getPart(&part,&partLength);
	m_Href.append(part,partLength);
	m_Hostname.getPart(&part,&partLength);
	m_Href.append(part,partLength);

	if (m_PortValid)
	{
		char portBuff[6];
		sprintf(portBuff, "%d", (int)m_Port);
		portBuff[5] = '\0';
		m_Href += ":";
		m_Href += portBuff;
	}

	m_Uri.getPart(&part,&partLength);
	m_Href.append(part,partLength);

	if( !m_Commands.isEmpty() )
	{
		m_Commands.getPart(&part,&partLength);
		m_Href.append(part,partLength);
	}

	if( !m_Search.isEmpty() )
	{
		m_Href += "?";
		m_Search.getPart(&part,&partLength);
		m_Href.append(part,partLength);
	}

	if( !m_Hash.isEmpty() )
	{
		m_Href += "#";
		m_Hash.getPart(&part,&partLength);
		m_Href.append(part,partLength);
	}

	m_UrlAnalyzed = false;
}

bool CWLUrl::operator==(CWLUrl& aUrl) const
{
	if( ! strcmp(m_Href.data(),aUrl.Href()) )
		return true;
	else
		return false;
}

//! get & set functions
const char* CWLUrl::Protocol()
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	return m_Protocol.getPart();
}

const char* CWLUrl::Hostname()
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	return m_Hostname.getPart();
}

const char* CWLUrl::Host()
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	return m_Host.getPart();
}

const char* CWLUrl::Uri()
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	return m_Uri.getPart();
}
const char* CWLUrl::Commands()
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	return m_Commands.getPart();
}

const char* CWLUrl::Search()
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	return m_Search.getPart();
}

const char* CWLUrl::Hash()
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	return m_Hash.getPart();
}


const char* CWLUrl::Name()
{
	return m_Name;
}

const char* CWLUrl::GetName()
{
	return m_Name;
}

const char* CWLUrl::Id()
{
	return m_Id;
}

const char* CWLUrl::Target()
{
	return m_Target;
}

/*const char* CWLUrl::InnerHTML()
{
	return m_InnerHTML;
}*/
/*
const char* CWLUrl::InnerText()
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	if( m_InnerText )
		return m_InnerText->data();
	else
		return NULL;
}
*/
const char* CWLUrl::GetInnerText()
{
	static char empty_str[1] = "";

	if (m_InnerText)
		return m_InnerText->data();	
	else if(m_DocPtr && m_DocPtr->m_ParseInnerHTML) {
		m_InnerText = new RVCString;
		RVCString innerHTML = GetInnerHTML();
		C_Html2Text(innerHTML.data(),0,innerHTML.length()-1,*m_InnerText);
		return m_InnerText->data();		
	} else
		return empty_str;
}

RVCString&  CWLUrl::Href_()
{
	if(m_HrefDirty)
	{
		rebuild();
		m_HrefDirty = false;
	}

	return m_Href;
}


unsigned	CWLUrl::Port()
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	return m_Port;
}

bool CWLUrl::Valid()
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	return m_HrefValid;
}

bool	CWLUrl::PortValid()
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	return m_PortValid;
}


//! You are NOT ALLOWED to change the values of this array !!!
CWLSearchArray& CWLUrl::Array()
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	if( !m_SearchAnalyzed )
	{
		m_Kvps.LoadURLEncodedString(m_Search.getPart());
		m_SearchAnalyzed = true;
	}
	return m_Kvps;
}


void CWLUrl::setProtocol(const char* protocol)
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	m_Protocol.updatePart(protocol);
	m_HrefDirty = true;
}

void CWLUrl::setHostname(const char* hostname)
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	m_Hostname.updatePart(hostname);
	m_HrefDirty = true;
}

void CWLUrl::setHost(const char* host)
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	m_Host.updatePart(host);
	m_HrefDirty = true;
}

void CWLUrl::setUri(const char* uri)
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	m_Uri.updatePart(uri);
	m_HrefDirty = true;
}

void CWLUrl::setSearch(const char* search)
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	m_Search.updatePart(search);
	m_HrefDirty = true;
}

void CWLUrl::setHash(const char* hash)
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	m_Hash.updatePart(hash);
	m_HrefDirty = true;
}

void CWLUrl::setHref(const char* href)
{
	m_Href = href;
	m_UrlAnalyzed = false;
}

void CWLUrl::setPort(unsigned port)
{
	if(!m_UrlAnalyzed)
		analyzeHref();
	m_Port = port;
	m_PortValid = true;
	m_HrefDirty = true;
}

bool CWLUrl::isAnalyzed()
{
	return m_UrlAnalyzed;
}

/*
bool CWLUrl::propertiesCompare(CWLUrl& aUrl, CWLUrlCmp& res)
{
	if( ! isAnalyzed() )
		analyzeHref();
	if( ! aUrl.isAnalyzed() )
		aUrl.analyzeHref();

	if( (!(m_Name && aUrl.m_Name)) || !*m_Name ||strcmp(m_Name,aUrl.m_Name) )
		res.nameMatch = false;

	if( (!(m_Id && aUrl.m_Id)) || !*m_Id || strcmp(m_Id,aUrl.m_Id) )
		res.idMatch = false;

	if( (!(m_InnerText && aUrl.m_InnerText)) || m_InnerText->isNull() || strcmp(m_InnerText->data(),aUrl.m_InnerText->data()) )
		res.innerTextMatch = false;

	return res.nameMatch || res.idMatch || res.innerTextMatch;
}*/


bool CWLUrl::urlCompare(CWLUrl& aUrl, CWLUrlCmp& res)
{
	bool result = true; //! initial result
	int  l1,l2;
	const char *h1,*h2;
	const char *pp1=NULL,*pp2=NULL;

	if( ! isAnalyzed() )
		analyzeHref();
	if( ! aUrl.isAnalyzed() )
		aUrl.analyzeHref();

	//! name, id, innerText - properties compare should be before first return
	//! since no call to propertiesCompare exists.
	if( (!(m_Name && aUrl.m_Name)) || strcmp(m_Name,aUrl.m_Name) || !strcmp(m_Name,""))
		res.nameMatch = false;

	if( (!(m_Id && aUrl.m_Id)) || strcmp(m_Id,aUrl.m_Id) || !strcmp(m_Id,""))
		res.idMatch = false;

	if (!(m_InnerText && aUrl.m_InnerText)
		//|| m_InnerText->data() == NULL || aUrl.m_InnerText->data() == NULL
		|| m_InnerText->isNull() || aUrl.m_InnerText->isNull()
		|| strcmp(m_InnerText->data(),aUrl.m_InnerText->data()) )
		res.innerTextMatch = false;


	if ( aUrl.m_Protocol.isEmpty() ) {//! href != http/s (mailto...)
		if (m_Protocol.isEmpty() )     //! both are not http/s
			if (m_Protocol.compare(aUrl.m_Protocol) == 0)
				return true;
		return false;
	}

	if( m_File.compare(aUrl.m_File) != 0 ) //! wrong file part
		return false;

	if( m_FileExt.compare(aUrl.m_FileExt) != 0 ) //! wrong file extenssion
	{
		res.extMatch = false;
		//! consider it like a QS diff with index -1
		++res.numDiffs;
		res.searchDiffs[res.numDiffs-1] = -1;
	}

	if( m_Host.compare(aUrl.m_Host) != 0 ) //! wrong Host
	{
		res.hostMatch = false;
		//! let's check that the domain part is the same. Otherwise it's no match!
		m_Host.getPart(&h1,&l1);
		aUrl.m_Host.getPart(&h2,&l2);
		if (h1 != NULL)
			pp1 = strchr(h1,'.');
		if (h2 != NULL)
			pp2 = strchr(h2,'.');
		if( pp1 == NULL || pp1 >= h1+l1 )  //! no DOT in first path
		{
			if( pp2 != NULL && pp2 < h2+l2 )  //! and a DOT in the second path
				return false; //! entirly different host
		}
		else if( pp2 == NULL || pp2 >= h2+l2 ) //! no DOT is second path
			return false; //! entirly different host
		else //! compare domains
			if( (h1+l1-pp1 != h2+l2-pp2)          || //! different domain length
				(strncmp(pp1,pp2,h1+l1-pp1) )  ) //! different domain
				return false; //! different domain part
	}


	if( m_Path.compare(aUrl.m_Path) != 0 )
		res.uriMatch = false;

	if( m_Search.compare(aUrl.m_Search) != 0 )
		res.searchMatch = false;


	res.exactMatch = res.hostMatch && res.uriMatch && res.extMatch && res.searchMatch
					 && res.nameMatch && res.idMatch && res.innerTextMatch;

	if( res.uriMatch && res.searchMatch && res.extMatch )
		return true;
	else if( !(res.uriMatch || res.searchMatch) ) //! no URI match nor Search match
		return false;

	if( ! res.searchMatch )  //! needs to compute Search "difference"
	{
		CWLSearchArray& s1 = Array();
		CWLSearchArray& s2 = aUrl.Array();
		if( s1.entries() != s2.entries() ) //! Incompatibale SearchArray
			result = false;
		else {
			int s1_entries = s1.entries();

			for(int ind = 0 ; ind < s1_entries ; ++ind) //!loop on all elements
			{
				if( strcmp(s1.Key(ind),s2.Key(ind)) ) //! incompatibles Keys
				{
					//! check to see if it's not a case of value-less fields
					if( !strcmp(s1.Value(ind),VOID_STRING) && !strcmp(s2.Value(ind),VOID_STRING) )
					{ //! it is! - consider it as a QS difference
						++res.numDiffs;
						if(res.numDiffs > MAX_DIFFS)
						{
							result = false;
							break;
						}
						res.searchDiffs[res.numDiffs-1] = ind;
					}

					else
					{
						result = false;
						break;
					}
				}
				else if( strcmp(s1.Value(ind),s2.Value(ind)) ) //! Incompatibale Values
				{
					++res.numDiffs;
					if(res.numDiffs > MAX_DIFFS)
					{
						result = false;
						break;
					}
					res.searchDiffs[res.numDiffs-1] = ind;
				}
			}
		}
	}

	if( ! res.uriMatch ) //! needs to make sure there is only ONE difference!
	{
		bool diffAlreadyFound = false;
		const char *p1,*p2,*lastP1,*lastP2;
		m_Path.getPart(&p1,&l1);
		aUrl.m_Path.getPart(&p2,&l2);
		//if(l1 != l2) // different length
		//	return false;

		lastP1 = p1;
		lastP2 = p2;
		pp1 = p1;
		pp2 = p2;
		while( pp1 != NULL && pp2 != NULL )
		{
			//! check to see if the search finished on BOTH paths - otherwise - Error
			if(pp1 >= p1 + l1)
				if( pp2 < p2 + l2)
				{
					result = false;
					break;
				}
				else
				{
					result = true;
					break;
				}
			else if( pp2 >= p2 + l2 )
			{
				result = false;
				break;
			}

			if( pp1 > lastP1 && pp2 > lastP2 ) //! skip the first '/'
			{
				if( (pp1 - lastP1 != pp2 - lastP2)    ||  //! different length of path component
					(strncmp(lastP1,lastP2,pp1-lastP1)) )  //! or different component
						if( diffAlreadyFound )
						{
							result = false;
							break;
						}
						else
							diffAlreadyFound = true;
			}
			lastP1 = pp1;
			lastP2 = pp2;
			pp1 = strchr(pp1+1,'/');
			pp2 = strchr(pp2+1,'/');
		}

		if( pp1 != pp2 ) //! one is NULL and the other is not.
			result = false;
		else
			result = true;  //! return true (with URI diff)
	}

	return result;
}

RVCSubString CWLUrl::GetInnerHTML()
{ 
	if (m_DocPtr && m_DocPtr->m_HtmlPagePtr) {
		RVCString &temp = *(m_DocPtr->m_HtmlPagePtr);
		return temp(GetStartInnerHTML(), GetLenInnerHTML());
	} else {
		RVCString temp;
		return temp(0,0);
	}
}


 void CWLUrl::printAll()
{
	cout<<endl;
	cout<<"Protocol:		 "<<Protocol()<<endl;
	cout<<"Hostname:		 "<<Hostname()<<endl ;
	cout<<"Host:			 "<<Host()<<endl;
	cout<<"Uri:				 "<<Uri()<<endl;
	cout<<"Uri(PATH):        "<<m_Path.getPart()<<endl;
	cout<<"Uri(FILE):        "<<m_File.getPart()<<endl;
	cout<<"Uri(FILE Ext):    "<<m_FileExt.getPart()<<endl;
	cout<<"Search:			 "<<Search()<<endl;
	cout<<"Hash:		     "<<Hash()<<endl;
	cout<<"Href:			 "<<Href()<<endl;
	cout<<"Commands:		 "<<Commands()<<endl;
	if( Name() )
		cout<<"Name:				 "<<Name()<<endl;
	cout<<"Port:				 "<<Port()<<endl;
	cout<<"Valid:				 "<<Valid()<<endl;
	cout<<"PortValid:		 "<<PortValid()<<endl;
	if( Target() )
		cout<<"Target:			 "<<Target()<<endl;
	cout<<"SearchArray:	 "<<endl;
	if( Id() )
	  cout<<"Id:					 "<<Id()<<endl;
/*	if( InnerHTML() )
	  cout<<"InnerHTML:		 "<<InnerHTML()<<endl;*/
	cout << "StartInnerHTML: " << GetStartInnerHTML() << endl;
	cout << "LenInnerHTML:   " << GetLenInnerHTML()   << endl;
	RVCString iH = GetInnerHTML();
	cout << "InnerHTML:      " << iH.data()      << endl;
	if( GetInnerText() )
	  cout<<"InnerText:		 "<<GetInnerText()<<endl;
	Array().PrintAllPairs();

}
