/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include "wlxmlentityresolver.h"
#include "wlxmlhelperfuncs.h"
#include <string.h>

CWLXmlEntityResolver::CWLXmlEntityResolver(entityType ent, CWLHtmlVirtualQueue* queue, int start, int end, RVCString* url, RVCString* internalStr, CWLDocument *docPtr, bool p_isDtd/*=false*/)
    : CWLDocument()
{
  m_DocPtr              = docPtr;
 	m_TransObj.SetObj(this);
	if (p_isDtd)
		m_TransObj.SetObjType(DTD);
	else
		m_TransObj.SetObjType(XML);
 	m_TransObj.GetTransaction()->SetName(url->data());

	isParamEntityDec			= 	isParsingInclude = false;
	m_QueuePtr						= queue;
	m_EntityType					= ent;
	totalLength						= 0;
	m_StartLocationParent = start;
	m_EndLocationParent		= m_AddToParentEnd = end;
	strPtr								= 0;
	parseState						= PARSE_INIT;
	parseEntityState			= PARSE_ENTITY_INIT;
	if(url)
		m_DocUrl						= url->data();
	m_UrlBase.setBase(m_DocUrl);//!set the urlBase
	switch(ent){
		case(ENTITY_EXTERNAL):
		case(ENTITY_PARAMETER_EXTERNAL):
		case(DOCTYPE_EXTERNAL):
			m_DocPtr->insertTransObjToQueue(&this->m_TransObj); //!insert self into queue
			break;
		case(PARAMETER_REFERENCE):
			m_Data        = (*internalStr).data();
			break;
		case(ENTITY_PARAMETER_INTERNAL):
			m_Data        = (*internalStr).data();
			totalLength		= m_Data.length();
			parse(NULL, 0);
	}
}


CWLXmlEntityResolver:: ~CWLXmlEntityResolver()
{
	m_ExternalEntitiesAndParameterRefs.clearAndDestroy();
}


bool CWLXmlEntityResolver::isEqual(const RVCollectable* c) const
{
	if(*(RVCollectableString*)c == m_DictName)
		return TRUE;
	return FALSE;
}

int CWLXmlEntityResolver::compareTo(const RVCollectable* a) const
{
  return ( (*((RVCollectableString*)a)).compareTo((const RVCollectable*)&m_DictName) );
}

void CWLXmlEntityResolver::parse(const char* data, size_t size, bool* isDone)
{
	size_t newSize = size;
	appendNoNull(m_Data, data, newSize);
	totalLength += newSize;
	//!need tp parse external entities for charRefs
	//!resolveReferencesInString(m_Data);
	if(m_EntityType == ENTITY_EXTERNAL)
		return;
	while(parseState != PARSE_ERR && parseState != PARSE_DONE){
		switch(parseState){
			case(PARSE_INIT):
				 enumParseState nextState;
				 if((nextState = findNextObject(isDone)) == PARSE_READ_MORE)
					 return;
				 else
					 parseState = nextState;
				break;
			case(PARSE_PARAMETER_REF):
				switch(parameterReferenceHandler()){
					case(PARSE_DONE):
						parseState = PARSE_INIT;
						break;
					case(PARSE_READ_MORE):
						return;
					case(PARSE_ERR):
						parseState = PARSE_ERR;
						break;
				}
				break;
			case(PARSE_ENTITY):
				switch(entityDeclarationHandler()){
					case(PARSE_DONE):
						parseEntityState	= PARSE_ENTITY_INIT;
						parseState				= PARSE_INIT;
						break;
					case(PARSE_READ_MORE):
						return;
					case(PARSE_ERR):
						parseState = PARSE_ERR;
						break;
				}
				break;
			case(PARSE_NOTATION):
			case(PARSE_ATTLIST):
			case(PARSE_ELEMENT):
				switch(elementAttlistNotationDeclarationHandler()){
					case(PARSE_DONE):
						parseState = PARSE_INIT;
						break;
					case(PARSE_READ_MORE):
						return;
					case(PARSE_ERR):
						parseState = PARSE_ERR;
				}
				break;
			case(PARSE_IGNORE_INCLUDE):
				switch(findIgnoreInclude()){
					case(PARSE_IGNORE):
						parseState = PARSE_IGNORE;
						break;
					case(PARSE_INCLUDE):
						parseState				= PARSE_INCLUDE;
						break;
					case(PARSE_ERR):
						parseState = PARSE_ERR;
						break;
				}
				break;
			case(PARSE_INCLUDE):
				isParsingInclude  = true;
				switch(includeHandler()){
					case(PARSE_DONE):
						parseState = PARSE_INIT;
						break;
					case(PARSE_READ_MORE):
						return;
					case(PARSE_ERR):
						parseState = PARSE_ERR;
						break;
				}
				break;
			case(PARSE_IGNORE):
				switch(ignoreHandler()){
					case(PARSE_DONE):
						parseState = PARSE_INIT;
						break;
					case(PARSE_READ_MORE):
						return;
					case(PARSE_ERR):
						parseState = PARSE_ERR;
						break;
				}
			case(PARSE_COMMENT):
				switch(commentHandler()){
					case(PARSE_DONE):
						parseState = PARSE_INIT;
						break;
					case(PARSE_READ_MORE):
						return;
					case(PARSE_ERR):
						parseState = PARSE_ERR;
						break;
				}
			default:
				break;
		}
	}


}

enumParseState CWLXmlEntityResolver::findNextObject(bool* isDone)
{
	char firstNoneWhite;
	switch(xmlSkipWhiteSpaces(m_Data, totalLength, strPtr, firstNoneWhite)){
		case(WHITE_NONE_TO_SKIP):
		case(WHITE_SKIPPED_ALL):
			break;
		case(WHITE_READ_MORE):
				return PARSE_READ_MORE;
	}
	switch(firstNoneWhite){
		case('%'):
			return PARSE_PARAMETER_REF;
		case('<'):
			ltPtr	= strPtr;
			return typeOfObject();
		case(']'):
			if(isParsingInclude){
				if(strPtr + 3 >= totalLength)
					return PARSE_READ_MORE;

				if(m_Data.index("]>", ++strPtr) == strPtr ){
					 m_Data.remove(--strPtr, 3);
					 isParsingInclude = false;
					 return PARSE_INIT;
				}
				else{
					if(m_Data(strPtr) = ']')
						++strPtr;
					appendToErrorString("Unexpected character in DTD: ", strPtr - 20, 20, "^");
					return PARSE_ERR;
				}
			}
			else if(m_EntityType == DOCTYPE_INTERNAL){
				size_t tmpPtr = strPtr;
				if(strPtr + 1 >= totalLength)
					return PARSE_READ_MORE;
				switch(xmlSkipWhiteSpaces(m_Data, totalLength, ++strPtr, firstNoneWhite)){
					case(WHITE_NONE_TO_SKIP):
					case(WHITE_SKIPPED_ALL):
							m_EndLocationParent = m_StartLocationParent + strPtr + m_AddToParentEnd;
							m_Data.remove(strPtr + m_AddToParentEnd);
						switch(firstNoneWhite){
							case('>'):
								/*
								m_EndLocationParent = m_StartLocationParent + strPtr ;
								m_Data.remove(strPtr - 1);
								*/

								*isDone							= true;
								return PARSE_DONE;
							default:
								appendToErrorString("Unexpected character in DTD, Expecting '>' : ", strPtr - 20, 20, "^");
								return PARSE_ERR;
						}
					case(WHITE_READ_MORE):
						strPtr = tmpPtr;
						return PARSE_READ_MORE;
					default:
						break;
				}
			}
			else{
				appendToErrorString("Unexpected character in DTD: ", strPtr - 20, 20, "^");
				return PARSE_ERR;
			}

		default:
			return PARSE_DONE;
	}
}

enumParseState CWLXmlEntityResolver::typeOfObject()
{
	enumParseState type;
	if(strPtr + 11 > totalLength)
		return PARSE_READ_MORE;
	else if(m_Data(strPtr + 1, 3) == "!--"){
		strPtr += 4;
		return PARSE_COMMENT;
	}
	else if(m_Data(strPtr + 1, 8) == "!ELEMENT"){
		strPtr += 9;
		type = PARSE_ELEMENT;
	}
	else if(m_Data(strPtr + 1, 8) == "!ATTLIST"){
		strPtr += 9;
		type = PARSE_ATTLIST;
	}
	else if(m_Data(strPtr + 1, 7) == "!ENTITY"){
		strPtr += 8;
		type = PARSE_ENTITY;
	}
	else if(m_Data(strPtr + 1, 9) == "!NOTATION"){
		strPtr += 10;
		type = PARSE_NOTATION;
	}
	else if(m_Data(strPtr + 1, 2) == "!["){
		if(m_EntityType == DOCTYPE_INTERNAL){
			m_DocTypeErrorString = "Conditional Sections are not allowed in Internal Subsets.";
			return PARSE_ERR;
		}
		m_Data.remove(strPtr, 3);
		return PARSE_IGNORE_INCLUDE;
	}
	if(htmlIsWhiteSpace(m_Data(strPtr)))
		return type;
	else{
		appendToErrorString("White Space expected: ", strPtr - 20, 20,"^");
		return PARSE_ERR;
	}
}

enumParseState	CWLXmlEntityResolver::findIgnoreInclude()
{
	size_t tmpPtr = strPtr;
	char firstNoneWhite;
	switch(xmlSkipWhiteSpaces(m_Data, totalLength, strPtr, firstNoneWhite)){
		case(WHITE_NONE_TO_SKIP):
		case(WHITE_SKIPPED_ALL):
			break;
		case(WHITE_READ_MORE):
				return PARSE_READ_MORE;
	}
	if(strPtr + 8 >= totalLength )
		return PARSE_READ_MORE;
	if(m_Data.index("IGNORE", strPtr) == strPtr && htmlIsWhiteSpace(m_Data(strPtr + 6))){
		m_Data.remove(strPtr, 6);
		return PARSE_IGNORE;
	}
	else if(m_Data.index("INCLUDE", strPtr) == strPtr && htmlIsWhiteSpace(m_Data(strPtr + 7))){
		m_Data.remove(strPtr, 7);
		return PARSE_INCLUDE;
	}
	else if(firstNoneWhite != '%'){
		appendToErrorString("Unexpected character in DTD: ", strPtr - 10, 20, "^");
		return PARSE_ERR;
	}

	size_t paramEndPtr = m_Data.index(';', strPtr);
	if(paramEndPtr != RV_NPOS){
		RVCollectableString dictEntry = m_Data(strPtr + 1, paramEndPtr - strPtr - 1);
		CWLXmlEntityResolver* dictionaryEntity	= (CWLXmlEntityResolver*)m_ExternalEntitiesAndParameterRefs.find((RVCollectable*)&dictEntry);
		if(dictionaryEntity){
			if(dictionaryEntity->GetType() == ENTITY_PARAMETER_INTERNAL){
				if(!strcmp(dictionaryEntity->GetData(), "IGNORE")){
					m_Data.remove(strPtr, paramEndPtr - strPtr + 1);
					return PARSE_IGNORE;
				}
				else if(!strcmp(dictionaryEntity->GetData(), "INCLUDE")){
					m_Data.remove(strPtr, paramEndPtr - strPtr + 1);
					return PARSE_INCLUDE;
				}
				else{
					m_DocTypeErrorString = "Unexpected Entity Parameter Reference in Conditional section.";
					return PARSE_ERR;
				}
			}
		}
		m_DocTypeErrorString = "Unresolved Prameter Reference in Conditional section.";
		return PARSE_ERR;
	}
	else
		return PARSE_READ_MORE;
}

enumParseState CWLXmlEntityResolver::ignoreHandler()
{
	size_t tmpPtr;
	if((tmpPtr = m_Data.index("]]>", strPtr)) != RV_NPOS){
		m_Data.remove(strPtr, tmpPtr - strPtr + 3);
		return PARSE_DONE;
	}
	return PARSE_READ_MORE;
}

//!'<![' S? 'INCLUDE' S? '[' extSubsetDecl ']]>'
enumParseState CWLXmlEntityResolver::includeHandler()
{
	char firstNoneWhite;
	switch(xmlSkipWhiteSpaces(m_Data, totalLength, strPtr, firstNoneWhite)){
		case(WHITE_NONE_TO_SKIP):
		case(WHITE_SKIPPED_ALL):
			if(strPtr < totalLength - 1){
				if(firstNoneWhite == '['){//!ok - remove this char and parse the guts of this INCLUDE
					m_Data.remove(strPtr, 1);
					return PARSE_DONE;
				}
				else{
					appendToErrorString("Unexpected character in Conditional Section: ", strPtr - 20, 20, "^");
					return PARSE_ERR;
				}
			}
		case(WHITE_READ_MORE):
				return PARSE_READ_MORE;
	}
	return PARSE_READ_MORE;
}

enumParseState	CWLXmlEntityResolver::parameterReferenceHandler()
{
	RVCString paramName;
	size_t tmpStrPtr = m_Data.index(';', strPtr);
	if(tmpStrPtr != RV_NPOS &&  tmpStrPtr < totalLength - 1){
			paramName = m_Data(strPtr + 1, tmpStrPtr - strPtr - 1);
			CWLXmlEntityResolver* paramRef = new CWLXmlEntityResolver(PARAMETER_REFERENCE, NULL,  strPtr, tmpStrPtr, NULL, &paramName, m_DocPtr);
			m_ExternalEntitiesAndParameterRefs.insert(paramRef);
			strPtr = tmpStrPtr + 1;
			return PARSE_DONE;
	}
	else
		return PARSE_READ_MORE;
}

enumParseState CWLXmlEntityResolver::entityDeclarationHandler()
{
	size_t wordPtr;
	while(1){
		switch(parseEntityState){
			case(PARSE_ENTITY_INIT):{
				size_t tmp = strPtr;
				if(readNextWord(m_Data, totalLength, strPtr, wordPtr)){
					parseEntityState = PARSE_ENTITY_SEARCH_NAME;
					if(wordEquals(m_Data, strPtr, wordPtr, "%"))
						isParamEntityDec = true;
					else
						strPtr = tmp;
				}
				else
					return PARSE_READ_MORE;
			}
			case(PARSE_ENTITY_SEARCH_NAME)://!try skipping the entity name
				if(readNextWord(m_Data, totalLength, strPtr, wordPtr)){
					if(m_Data(wordPtr) == '%' && m_Data(strPtr - 1) == ';'){//!is the name param reference ?
						RVCollectableString dictEntry = m_Data(wordPtr + 1, strPtr - wordPtr - 2);
						CWLXmlEntityResolver* dictionaryEntity	= (CWLXmlEntityResolver*)m_ExternalEntitiesAndParameterRefs.find((RVCollectable*)&dictEntry);
						if(dictionaryEntity){
							if(dictionaryEntity->GetType() == ENTITY_PARAMETER_INTERNAL){
								if(isParamEntityDec)//!get the dictionary key from dictionaryEntity
									parameterEntityName = dictionaryEntity->GetData();
								else{
									CWLXmlEntityResolver* paramRef = new CWLXmlEntityResolver(PARAMETER_REFERENCE, NULL,  wordPtr, strPtr - 1, NULL, (RVCString*)&dictEntry, m_DocPtr);
									m_ExternalEntitiesAndParameterRefs.insert(paramRef);
								}
							}
							else{//!Error - type is wrong
								m_DocTypeErrorString = "Unresolved Entity Parameter Reference in ENTITY declaration.";
								return PARSE_ERR;
							}
						}
						else{
								m_DocTypeErrorString = "Unresolved Entity Parameter Reference in ENTITY declaration.";
								return PARSE_ERR;
						}
					}
					else
						parameterEntityName = m_Data(wordPtr, strPtr - wordPtr);
					startEntityPtr			= ++strPtr;//!for external entities
					parseEntityState		= PARSE_ENTITY_SEARCH_EXTERNAL;
				}
				else //!!readNextWord
					return PARSE_READ_MORE;

			case(PARSE_ENTITY_SEARCH_EXTERNAL):
				if(readNextWord(m_Data, totalLength, strPtr, wordPtr)){
					if(wordEquals(m_Data, strPtr, wordPtr, "PUBLIC")){
						parseEntityState = PARSE_ENTITY_PUBLIC;
					}
					else if(wordEquals(m_Data, strPtr, wordPtr, "SYSTEM")){
						parseEntityState = PARSE_ENTITY_SYSTEM;
					}
					else
						parseEntityState = PARSE_ENTITY_LOCAL;
					break;
				}
				else
					return PARSE_READ_MORE;

			case(PARSE_ENTITY_PUBLIC):
				if(readNextWord(m_Data, totalLength, strPtr, wordPtr))//!skip public name
					parseEntityState = PARSE_ENTITY_SYSTEM;
				else
					return PARSE_READ_MORE;

			case(PARSE_ENTITY_SYSTEM):
				if(readNextWord(m_Data, totalLength, strPtr, wordPtr)){
					if( (m_Data(wordPtr) == '"' && m_Data(strPtr - 1) == '"') || (m_Data(wordPtr) == '\'' && m_Data(strPtr - 1) == '\'')  ){
						if(isParamEntityDec)
							parseEntityType		= ENTITY_PARAMETER_EXTERNAL;
						else{
						//!replace  double quotes//single quotes with single quotes
							m_Data(wordPtr) = m_Data(strPtr - 1) = '\'';
							parseEntityType		= ENTITY_EXTERNAL;
						}

						endEntityPtr			= strPtr - 2;
						RVCString word		= m_Data(wordPtr + 1, strPtr - wordPtr - 2);
						url								= m_UrlBase.calcUrl(word);
						parseEntityState	= PARSE_ENTITY_TO_END;
						break;
					}
					else{//!error
						appendToErrorString("Unexpected character string in DTD: ", wordPtr, strPtr - wordPtr + 1);
						return PARSE_ERR;
					}
				}
				else
					return PARSE_READ_MORE;

			case(PARSE_ENTITY_LOCAL):
				if(  (m_Data(wordPtr) == '"' && m_Data(strPtr - 1) == '"') || (m_Data(wordPtr) == '\'' && m_Data(strPtr - 1) == '\'')  ){
					if(isParamEntityDec)
						parseEntityType		= ENTITY_PARAMETER_INTERNAL;
					else
						parseEntityType		= ENTITY_INTERNAL;
					word = m_Data(wordPtr + 1, strPtr - wordPtr - 2);
					//!need to go over the string and look for parameter refs and charRefs that need to be converted
					//!resolveReferencesInString()

					endEntityPtr = strPtr;
					parseEntityState		= PARSE_ENTITY_TO_END;
					break;
				}
				else {//!error
					appendToErrorString("Unexpected character string in DTD: ", wordPtr, strPtr - wordPtr + 1);
					return PARSE_ERR;
				}

			case(PARSE_ENTITY_TO_END):{

				//!mark (&remove) NDATA ENTITY declarations
				if((gtPtr = m_Data.index(">", strPtr)) != RV_NPOS){
					if(parseEntityType != ENTITY_INTERNAL){
						RVCRegexp REndata("[\\n\\t ]+NDATA[\\n\\t ]+");
						size_t ndataIndex = m_Data.index(REndata, strPtr);
						if(ndataIndex != RV_NPOS && ndataIndex < gtPtr)
							parseEntityType		= ENTITY_NDATA;
					}
					strPtr = gtPtr + 1;
					parseEntityState	= PARSE_ENTITY_FINISH;
					break;
				}
				else
					return PARSE_READ_MORE;
			}
			case(PARSE_ENTITY_FINISH):
				CWLXmlEntityResolver* entity;
				switch(parseEntityType){
					case(ENTITY_PARAMETER_EXTERNAL):
						entity = new CWLXmlEntityResolver(ENTITY_PARAMETER_EXTERNAL, m_QueuePtr, ltPtr, gtPtr, &url, NULL, m_DocPtr);
						//!name = new RVCollectableString();
						entity->setDictionaryName(parameterEntityName.data());
						//m_ParameterEntitiesDictionary.insertKeyAndValue((RVCollectable*)name, (RVCollectable*)entity);
						m_ExternalEntitiesAndParameterRefs.insert(entity);
						break;
					case(ENTITY_PARAMETER_INTERNAL):
						entity = new CWLXmlEntityResolver(ENTITY_PARAMETER_INTERNAL, m_QueuePtr, ltPtr, gtPtr, &m_DocUrl, &word, m_DocPtr);
						//*name = parameterEntityName.data();
						//m_ParameterEntitiesDictionary.insertKeyAndValue(name, entity);
						entity->setDictionaryName(parameterEntityName.data());
						m_ExternalEntitiesAndParameterRefs.insert(entity);
						break;
					/*
					case(ENTITY_NDATA)://have to remove only;
						entity = new CWLXmlEntityResolver(ENTITY_NDATA, m_QueuePtr, ltPtr, gtPtr, &m_DocUrl, &word);
						m_ExternalEntitiesAndParameterRefs.insert(entity);
						break;
						*/
					case(ENTITY_EXTERNAL):
						entity = new CWLXmlEntityResolver(parseEntityType, m_QueuePtr, startEntityPtr, endEntityPtr, &url, NULL, m_DocPtr);
						m_ExternalEntitiesAndParameterRefs.insert(entity);
						break;
				}

				isParamEntityDec = false;
				return PARSE_DONE;
		}//!switch()
	}//!while
	return  PARSE_DONE;
}
//!this is a rough hanling of NOTATION as in principle there could be '>' in the SYSTEm literal
enumParseState CWLXmlEntityResolver::elementAttlistNotationDeclarationHandler()
{
	//!we  are passed  '<!ELEMENT'  -  have to find the  closing '>'
	size_t endElementDec, startEntity, endEntity;
	if((endElementDec = m_Data.index('>', strPtr)) != RV_NPOS){
		while(1){
			startEntity = m_Data.index('%', strPtr);
			if(startEntity != RV_NPOS && startEntity < endElementDec){
				endEntity = m_Data.index(';', startEntity);
				if(endEntity != RV_NPOS  && endEntity < endElementDec){
					RVCollectableString dictEntry = m_Data(startEntity + 1, endEntity - startEntity - 1);
					CWLXmlEntityResolver* dictionaryEntity	= (CWLXmlEntityResolver*)m_ExternalEntitiesAndParameterRefs.find((RVCollectable*)&dictEntry);
					if(dictionaryEntity){
						if(dictionaryEntity->GetType() == ENTITY_PARAMETER_INTERNAL){
							CWLXmlEntityResolver* paramRef = new CWLXmlEntityResolver(PARAMETER_REFERENCE, NULL,  startEntity, endEntity, NULL, (RVCString*)&dictEntry, m_DocPtr);
							m_ExternalEntitiesAndParameterRefs.insert(paramRef);
							strPtr = endEntity + 1;
							continue;
						}
						else
							m_DocTypeErrorString = "Unresolved Or External Parameter Entity Reference in DTD.";
					}
					else
					//!Error - no dict entry or type is wrong
					m_DocTypeErrorString = "Unresolved Parameter Entity Reference in DTD.";
				}
				else//!error - no ';' found
					m_DocTypeErrorString = "Unexpected character in DTD.";
				return PARSE_ERR;
			}
			else{
				strPtr = endElementDec + 1;
				return PARSE_DONE;
			}
		}
	}
	else
		return PARSE_READ_MORE;
}

//!note that "--" is not allowed in comment.
enumParseState CWLXmlEntityResolver::commentHandler()
{
	RVCRegexp REendOfComment("-->");
	size_t end;
	if((end = m_Data.index(REendOfComment,  strPtr)) != RV_NPOS){
		strPtr = end + 3;
		return PARSE_DONE;
	}
	else{
		strPtr = m_Data.length() - 1;
		return PARSE_READ_MORE;
	}
}

const char* CWLXmlEntityResolver::GetData()
{
	return m_Data;
}

bool isCharXMLLegal(char c)
{
	return true;
}

//!this function collapses the sub tree underneath it into a string that is inserted to
//!the appropriate locations in m_Data

const char* CWLXmlEntityResolver::entityFoldSelf(RVCString& value, RVCString& error)
{
	if(!m_DocTypeErrorString.isNull()){
		error = m_DocTypeErrorString.data();
		return NULL;
	}
	size_t leftStrPtr = 0;
	CWLXmlEntityResolver* entity;
	for(size_t i = 0; i < m_ExternalEntitiesAndParameterRefs.entries(); ++i){
		const char* entityChar;
		RVCString entityStr;
		entity = (CWLXmlEntityResolver*)(m_ExternalEntitiesAndParameterRefs.at(i));
		switch(entity->GetType()){
			case(PARAMETER_REFERENCE):{
				RVCollectableString name;
				name = entity->GetData();
//				CWLXmlEntityResolver* dictionaryEntity	= (CWLXmlEntityResolver*)(m_ParameterEntitiesDictionary.findValue(&name));
				CWLXmlEntityResolver* dictionaryEntity	= (CWLXmlEntityResolver*)m_ExternalEntitiesAndParameterRefs.find((RVCollectable*)&name);
				if(!dictionaryEntity){
					error = "unable to resolve parameter reference.";
					return NULL;
				}
				else if(!(entityChar = dictionaryEntity->entityFoldSelf(entityStr, error)))
					return NULL;
				break;
			}
			case(ENTITY_NDATA):
			case(ENTITY_PARAMETER_INTERNAL):
			case(ENTITY_PARAMETER_EXTERNAL):
				entityChar = NULL;
				break;
			case(ENTITY_EXTERNAL):
				if(!(entityChar = entity->entityFoldSelf(entityStr, error))){
					error = "unable to resolve entity declaration";
					return NULL;
				}
		}
		value.append(m_Data(leftStrPtr, entity->GetStartLocationParent() - leftStrPtr));
		if(entity->GetType() == ENTITY_EXTERNAL)
			value.append('\'');
		if(entityChar)
			value.append(entityChar);
		leftStrPtr = entity->GetEndLocationParent() + 1;
	}
	if(GetType() == ENTITY_EXTERNAL){
		//!get rid of possible <?xml..?>
		size_t firstCharLoc = 0;
		char firstNoneWhite;
		xmlSkipWhiteSpaces(m_Data, m_Data.length(), firstCharLoc, firstNoneWhite);
		if(m_Data.index("<?xml ", firstCharLoc) == firstCharLoc)
			leftStrPtr = m_Data.index(">", firstCharLoc) + 1;
		appendStringWithQuotes(value, leftStrPtr, m_Data.length() - leftStrPtr);
	}
	else
		value.append(m_Data(leftStrPtr, m_Data.length() - leftStrPtr));
	return value.data();
}

void CWLXmlEntityResolver::appendStringWithQuotes(RVCString& value, size_t start, size_t length)
{
	enum {
		IN_SEARCH,
		IN_SINGLE,
		IN_DOUBLE
		} QuoteState;

	QuoteState = IN_SEARCH;
	for(size_t i = start; i < start + length; ++i)
		if(m_Data(i) == '\'')
			switch(QuoteState){
				case(IN_SEARCH):
					QuoteState = IN_SINGLE;
					value.append('"', 1);
					break;
				case(IN_DOUBLE):
					value.append('\'');
					break;
				case(IN_SINGLE):
					value.append('"', 1);
					QuoteState = IN_SEARCH;
					break;
		}

		else if(m_Data(i) == '"')
			switch(QuoteState){
				case(IN_SEARCH):
					value.append('"', 1);
					QuoteState = IN_DOUBLE;
					break;
				case(IN_DOUBLE):
					value.append('"', 1);
					QuoteState = IN_SEARCH;
					break;
				case(IN_SINGLE):
					value.append("&quot;");
					break;
		}

		else
			value.append(m_Data(i), 1);
}



size_t CWLXmlEntityResolver::GetStartLocationParent()
{
	return m_StartLocationParent;
}

size_t CWLXmlEntityResolver::GetEndLocationParent()
{
	return m_EndLocationParent;
}

entityType CWLXmlEntityResolver::GetType()
{
	return m_EntityType;
}

void CWLXmlEntityResolver::appendToErrorString(char* description, size_t start, size_t length, char* ptr)
{
	m_DocTypeErrorString.append(description);
	if(start < 0)
		start = 0;
	if(start + length >  m_Data.length())
		length = m_Data.length();
	RVCString tmp = m_Data(start, length);
	m_DocTypeErrorString.append(tmp.data());
	if(ptr)
		m_DocTypeErrorString.append(ptr);
}

const char* CWLXmlEntityResolver::GetError()
{
	if(m_DocTypeErrorString.isNull())
		return NULL;
	else
		return m_DocTypeErrorString.data();
}

void CWLXmlEntityResolver::setDictionaryName(const char* name)
{
	m_DictName = name;
}
