/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

/************************************************************
 *    Include Headers
 ************************************************************/

#include <ace/OS.h>
#include <windows.h>
#include <rpc.h>
#include <lm.h>

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <tchar.h>
#include <fcntl.h>

#define SECURITY_WIN32
#include "sspi.h"
#include "issperr.h"


// declaration for this module

#include "authorization.h"


#define SEC_SUCCESS(Status) ((Status) >= 0)
#define STATUS_SUCCESS 0


// Target name for the security package

#define TOKEN_SOURCE_NAME       "InetSvcs"


/************************************************************
 *    Authentication functions
 ************************************************************/

class CWLProcessAuthentication {
protected:
	HINSTANCE m_hSecLib;
	SEC_FUNC m_sfProcs;

public:
	CWLProcessAuthentication() { m_hSecLib = NULL; }
	virtual ~CWLProcessAuthentication() {}

	SEC_FUNC* GetFuncAddress(){
		return (&m_sfProcs);
	}
	bool Init();
	void Terminate();
};

CWLProcessAuthentication* CWLClientAuthentication::m_procAuth=NULL;
AUTH_STATUS CWLClientAuthentication::m_validity=AUTH_NOT_INIT;

bool CWLProcessAuthentication::Init()
/*++

 Routine Description:

    Initialize the authentication package

 Arguments:

    None

 Return Value:

    Returns TRUE is successful; otherwise FALSE is returned.

--*/
{

    OSVERSIONINFO   VerInfo;
    UCHAR lpszDLL[MAX_PATH];

    //
    //  Find out which security DLL to use, depending on 
    //  whether we are on NT or Win95
    //
    VerInfo.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
    if (!GetVersionEx (&VerInfo))   // If this fails, something has gone wrong
    {
        return false;
    }

    if (VerInfo.dwPlatformId == VER_PLATFORM_WIN32_NT)
    {
        strcpy ((char*)lpszDLL, "security.dll" );
    }
    else if (VerInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
    {
        strcpy ((char*)lpszDLL, "secur32.dll" );
    }
    else
    {
        return false;
    }

    //
    //  Load Security DLL
    //

    m_hSecLib = LoadLibrary ((char*)lpszDLL);
    if (m_hSecLib == NULL)
    {
        return false;
    }

    // Get all entry points we care about

    m_sfProcs.pFreeCredentialsHandle 
            = (FREE_CREDENTIALS_HANDLE_FN) GetProcAddress( 
                    m_hSecLib, 
                    "FreeCredentialsHandle" );

    m_sfProcs.pQuerySecurityPackageInfo 
            = (QUERY_SECURITY_PACKAGE_INFO_FN) GetProcAddress( 
                    m_hSecLib, 
                    "QuerySecurityPackageInfoA" );

    m_sfProcs.pAcquireCredentialsHandle 
            = (ACQUIRE_CREDENTIALS_HANDLE_FN) GetProcAddress(
                    m_hSecLib, 
                    "AcquireCredentialsHandleA" );

    m_sfProcs.pFreeContextBuffer 
            = (FREE_CONTEXT_BUFFER_FN) GetProcAddress( 
                    m_hSecLib, 
                    "FreeContextBuffer" );

    m_sfProcs.pInitializeSecurityContext
            = (INITIALIZE_SECURITY_CONTEXT_FN) GetProcAddress( 
                    m_hSecLib, 
                    "InitializeSecurityContextA" );

    m_sfProcs.pCompleteAuthToken 
            = (COMPLETE_AUTH_TOKEN_FN) GetProcAddress( 
                    m_hSecLib, 
                    "CompleteAuthToken" );

    m_sfProcs.pDeleteSecurityContext 
            = (DELETE_SECURITY_CONTEXT_FN) GetProcAddress( 
                    m_hSecLib, 
                    "DeleteSecurityContext" );

    if ( m_sfProcs.pFreeCredentialsHandle == NULL
            || m_sfProcs.pQuerySecurityPackageInfo == NULL
            || m_sfProcs.pAcquireCredentialsHandle == NULL
            || m_sfProcs.pFreeContextBuffer == NULL
            || m_sfProcs.pInitializeSecurityContext == NULL 
            || m_sfProcs.pDeleteSecurityContext == NULL )
    {
        FreeLibrary( m_hSecLib );
        m_hSecLib = NULL;
        return false;
    }
	 
    return true;
}


void CWLProcessAuthentication::Terminate()
/*++

 Routine Description:

    Terminate the authentication package

 Arguments:

    None

 Return Value:

    None

--*/
{
   if ( m_hSecLib != NULL )
    {
        //The "FreeLibrary" is commented because for some unknown reason
        //when the agenda contains initialization of a com object 
        //(i.e. aaa = new ActiveXObject("xx.yyy");) the 
        //FreeLibrary is stuck.

        //FreeLibrary( m_hSecLib );
        m_hSecLib = NULL;
    }
}

CWLClientAuthentication::CWLClientAuthentication()
{
	if (m_validity != AUTH_SUCCESS)
		throw(m_validity);
	m_fAuth = false;
	m_sfProcs = m_procAuth->GetFuncAddress();
}


AUTH_STATUS CWLClientAuthentication::GlobalInit()
{
	if (m_validity == AUTH_NOT_INIT) // this means m_procAuth is NULL
	{
		m_procAuth = new CWLProcessAuthentication();
		if (m_procAuth != NULL)
			if (m_procAuth->Init())
				m_validity = AUTH_SUCCESS;
			else
				m_validity = AUTH_FAILED;
	}
	return m_validity;
}

void CWLClientAuthentication::GlobalTerminate(){
		m_procAuth->Terminate();
	}

void CWLClientAuthentication::Clean()
/*++

 Routine Description:

    Clean the authentication package at the end of a handshake -
	use when authentication succeeded

 Arguments:

    None

 Return Value:

    None

--*/
{
    if ( m_fAuth )
    {
        AuthTerminate();
        m_fAuth = false;
    }

}



bool CWLClientAuthentication::GetValue(
	char*       pch,
    const char* pchSchemes,
    const char* pchAuthData,
    const char* pchUserName,
    const char* pchPassword,
    bool*       pfNeedMoreData
    )
/*++

 Routine Description:

    Generates an authentication header to be sent to the server.
    An authentication sequence will be initiated if none is in
    use and at least one of the authentication scheme specified
    in pchSchemes is recognized.
    Otherwise the current authentication sequence will proceed.

 Arguments:

    pch                 where to append authentication header
    pchSchemes          list of null-delimited authentication methods
    pchAuthData         incoming blob from server
    pchUserName         user name ( possibly prefixed with domain )
    pchPassword         password
    pfNeedMoreData      Out: TRUE if authentication sequence to continue

 Return Value:

    Returns TRUE is successful; otherwise FALSE is returned.

--*/
{
    BUFFER buff;
    bool   fSt = true;
    DWORD  cbOut;


    BufferInit( &buff );

    while ( *pchSchemes )
    {
        if ( !stricmp( pchSchemes, "NTLM" ))
        {
            // SSPI package ( assuming methods list have been validated )

            if ( !m_fAuth )
            {
                if ( !AuthInit() )
                {
                    fSt = false;
                    goto ex;
                }
						    m_fAuth = true;	// zion 07.11.98
            }
            else if ( pchAuthData == NULL || *pchAuthData == '\0' )
            {
                // no blob while in authentication sequence : it failed

                SetLastError( ERROR_ACCESS_DENIED );
                fSt = false;
                break;
            }

/*
printf("6, authdata:%s, buff:%s, cbout:%x, more:%x, schemes:%s, user:%s, pass:%s\n",
                  pchAuthData,
                  buff.pBuf,
                  cbOut,
                  *pfNeedMoreData,
                  (char*)pchSchemes,
                  (char*)pchUserName,
                  (char*)pchPassword);
*/

            if ( !AuthConverse( 
				  (VOID*) pchAuthData,
                  0,
                  &buff,
                  &cbOut,
                  (bool*)pfNeedMoreData,
                  (char*)pchSchemes,
                  (char*)pchUserName,
                  (char*)pchPassword ) )
            {
                fSt = false;
                goto ex;
            }

            strcat( pch, pchSchemes );
            strcat( pch, " " );
            strcat( pch, (char *) BufferQueryPtr( &buff ) );
            break;
        }

        pchSchemes += strlen(pchSchemes) + 1;
    }
ex:
    BufferTerminate( &buff );

    return fSt;
}


bool CWLClientAuthentication::AuthInit()
/*++

 Routine Description:

    Initialize a SSP authentication sequence

 Arguments:

    None

 Return Value:

    Returns TRUE is successful; otherwise FALSE is returned.

--*/
{
    m_Auth._fNewConversation = TRUE;
    m_Auth._fHaveCredHandle = FALSE;
    m_Auth._fHaveCtxtHandle = FALSE;
    m_Auth._fUUEncodeData = TRUE;

//    m_fAuth = TRUE;	// zion 07.11.98

    return TRUE;
}


void CWLClientAuthentication::AuthTerminate()
/*++

 Routine Description:

    Terminate a SSP authentication sequence

 Arguments:

    None

 Return Value:

    None

--*/
{
	if ( m_Auth._fHaveCtxtHandle )
        m_sfProcs->pDeleteSecurityContext( &m_Auth._hctxt );

    if ( m_Auth._fHaveCredHandle )
        m_sfProcs->pFreeCredentialsHandle( &m_Auth._hcred );

    m_Auth._fHaveCredHandle = FALSE;
    m_Auth._fHaveCtxtHandle = FALSE;
}


bool CWLClientAuthentication::AuthConverse(
    void   * pBuffIn,
    DWORD    cbBuffIn,
    BUFFER * pbuffOut,
    DWORD  * pcbBuffOut,
    bool   * pfNeedMoreData,
    char   * pszPackage,
    char   * pszUser,
    char   * pszPassword
    )
/*++

Routine Description:

    Initiates or continues a previously initiated authentication conversation

    Client calls this first to get the negotiation message which
    it then sends to the server.  The server calls this with the
    client result and sends back the result.  The conversation
    continues until *pfNeedMoreData is FALSE.

    On the first call, pszPackage must point to the zero terminated
    authentication package name to be used and pszUser and pszPassword
    should point to the user name and password to authenticated with
    on the client side (server side will always be NULL).

Arguments:

    pBuffIn - Points to SSP message received from the
        client.  If UUENCODE is used, then this must point to a
        zero terminated uuencoded string (except for the first call).
    cbBuffIn - Number of bytes in pBuffIn or zero if pBuffIn points to a
        zero terminated, uuencoded string.
    pbuffOut - If *pfDone is not set to TRUE, this buffer contains the data
        that should be sent to the other side.  If this is zero, then no
        data needs to be sent.
    pcbBuffOut - Number of bytes in pbuffOut
    pfNeedMoreData - Set to TRUE while this side of the conversation is
        expecting more data from the remote side.
    pszPackage - On the first call points to a zero terminate string indicating
        the security package to use
    pszUser - Specifies user or domain\user the first time the client calls
        this method (client side only)
    pszPassword - Specifies the password for pszUser the first time the
        client calls this method (client side only)

Return Value:

    TRUE if successful, FALSE otherwise (call GetLastError).  Access is
    denied if FALSE is returned and GetLastError is ERROR_ACCESS_DENIED.

--*/
{
    SECURITY_STATUS       ss;
    TimeStamp             Lifetime;
    SecBufferDesc         OutBuffDesc;
    SecBuffer             OutSecBuff;
    SecBufferDesc         InBuffDesc;
    SecBuffer             InSecBuff;
    ULONG                 ContextAttributes;
    BUFFER                buffData;
    BUFFER                buff;
    bool                  fSt;
    bool                  fReply;

    BufferInit( &buffData );
    BufferInit( &buff );

    //
    //  Decode the data if there's something to decode
    //

    if ( m_Auth._fUUEncodeData && pBuffIn )
    {
        if ( !uudecode( (char *) pBuffIn,
                        &buffData,
                        &cbBuffIn ))
        {
            fSt = FALSE;
            goto ex;
        }

        pBuffIn = BufferQueryPtr( &buffData );
    }

    //
    //  If this is a new conversation, then we need to get the credential
    //  handle and find out the maximum token size
    //

    if ( m_Auth._fNewConversation )
    {
        SecPkgInfo *              pspkg;
        SEC_WINNT_AUTH_IDENTITY   AuthIdentity;
        SEC_WINNT_AUTH_IDENTITY * pAuthIdentity;
        char *                    pszDomain = NULL;
        char                      szDomainAndUser[DNLEN+UNLEN+2];


        //
        //  fill out the authentication information
        //

        if ( ((*pszUser != '\0') ||
              (*pszPassword != '\0')) )
        {
            pAuthIdentity = &AuthIdentity;

            //
            //  Break out the domain from the username if one was specified
            //

            if ( pszUser != NULL )
            {
                strcpy( szDomainAndUser, pszUser );
                if ( !CrackUserAndDomain( szDomainAndUser,
                                          &pszUser,
                                          &pszDomain ))
                {
                    return FALSE;  
                }
            }

            memset( &AuthIdentity,
                    0,
                    sizeof( AuthIdentity ));

            if ( pszUser != NULL )
            {
                AuthIdentity.User       = (unsigned char *) pszUser;
                AuthIdentity.UserLength = strlen( pszUser );
            }

            if ( pszPassword != NULL )
            {
                AuthIdentity.Password       = (unsigned char *) pszPassword;
                AuthIdentity.PasswordLength = strlen( pszPassword );
            }

            if ( pszDomain != NULL )
            {
                AuthIdentity.Domain       = (unsigned char *) pszDomain;
                AuthIdentity.DomainLength = strlen( pszDomain );
            }

            AuthIdentity.Flags = SEC_WINNT_AUTH_IDENTITY_ANSI;
        }
        else
        {
            pAuthIdentity = NULL;
        }

        ss = m_sfProcs->pAcquireCredentialsHandle( NULL,    // New principal
                                       pszPackage,       // Package name
                                       SECPKG_CRED_OUTBOUND,
                                       NULL,             // Logon ID
                                       pAuthIdentity,    // Auth Data
                                       NULL,             // Get key func
                                       NULL,             // Get key arg
                                       &m_Auth._hcred,
                                       &Lifetime );

        //
        //  Need to determine the max token size for this package
        //

        if ( ss == STATUS_SUCCESS )
        {
            m_Auth._fHaveCredHandle = TRUE;
            ss = m_sfProcs->pQuerySecurityPackageInfo( 
                        (char *) pszPackage,
                        &pspkg );
        }

        if ( ss != STATUS_SUCCESS )
        {
            SetLastError( ss );
            return FALSE;
        }

        m_Auth._cbMaxToken = pspkg->cbMaxToken;

        m_sfProcs->pFreeContextBuffer( pspkg );

    }

    //
    //  Prepare our output buffer.  We use a temporary buffer because
    //  the real output buffer will most likely need to be uuencoded
    //

    if ( !BufferResize( &buff, m_Auth._cbMaxToken ))
    {
        fSt = FALSE;
        goto ex;
    }

    OutBuffDesc.ulVersion = 0;
    OutBuffDesc.cBuffers  = 1;
    OutBuffDesc.pBuffers  = &OutSecBuff;

    OutSecBuff.cbBuffer   = m_Auth._cbMaxToken;
    OutSecBuff.BufferType = SECBUFFER_TOKEN;
    OutSecBuff.pvBuffer   = BufferQueryPtr( &buff );

    //
    //  Prepare our Input buffer - Note the server is expecting the client's
    //  negotiation packet on the first call
    //

    if ( pBuffIn )
    {
        InBuffDesc.ulVersion = 0;
        InBuffDesc.cBuffers  = 1;
        InBuffDesc.pBuffers  = &InSecBuff;

        InSecBuff.cbBuffer   = cbBuffIn;
        InSecBuff.BufferType = SECBUFFER_TOKEN;
        InSecBuff.pvBuffer   = pBuffIn;
    }

    {
        //
        //  will return success when its done but we still
        //  need to send the out buffer if there are bytes to send
        //

        ss = m_sfProcs->pInitializeSecurityContext( 
                                        &m_Auth._hcred,
                                        m_Auth._fNewConversation ? NULL :
                                                &m_Auth._hctxt,
                                        TOKEN_SOURCE_NAME,
                                        0,
                                        0,
                                        SECURITY_NATIVE_DREP,
                                        m_Auth._fNewConversation ? NULL :
                                                &InBuffDesc,
                                        0,
                                        &m_Auth._hctxt,
                                        &OutBuffDesc,
                                        &ContextAttributes,
                                        &Lifetime );
    }

    if ( !SEC_SUCCESS( ss ) )
    {
        if ( ss == SEC_E_LOGON_DENIED )
            ss = ERROR_LOGON_FAILURE;

        SetLastError( ss );
        fSt = FALSE;
        goto ex;
    }

    m_Auth._fHaveCtxtHandle = TRUE;

    //
    //  Now we just need to complete the token (if requested) and prepare
    //  it for shipping to the other side if needed
    //

    fReply = !!OutSecBuff.cbBuffer;

    if ( (ss == SEC_I_COMPLETE_NEEDED) ||
         (ss == SEC_I_COMPLETE_AND_CONTINUE) )
    {
        if ( m_sfProcs->pCompleteAuthToken != NULL )
        {
            ss = m_sfProcs->pCompleteAuthToken( &m_Auth._hctxt,
                                    &OutBuffDesc );

            if ( !SEC_SUCCESS( ss ))
            {
                fSt = FALSE;
                goto ex;
            }
        }
        else
        {
            // if not supported

            fSt = FALSE;
            goto ex;
        }
    }

    //
    //  Format or copy to the output buffer if we need to reply
    //

    if ( fReply )
    {
        if ( m_Auth._fUUEncodeData )
        {
            if ( !uuencode( (BYTE *) OutSecBuff.pvBuffer,
                            OutSecBuff.cbBuffer,
                            pbuffOut ))
            {
                fSt = FALSE;
                goto ex;
            }

            *pcbBuffOut = strlen( (char *) BufferQueryPtr(pbuffOut) );
        }
        else
        {
            if ( !BufferResize( pbuffOut, OutSecBuff.cbBuffer ))
            {
                fSt = FALSE;
                goto ex;
            }

            memcpy( BufferQueryPtr(pbuffOut),
                    OutSecBuff.pvBuffer,
                    OutSecBuff.cbBuffer );

            *pcbBuffOut = OutSecBuff.cbBuffer;
        }
    }

    if ( m_Auth._fNewConversation )
        m_Auth._fNewConversation = FALSE;

    *pfNeedMoreData = ((ss == SEC_I_CONTINUE_NEEDED) ||
                       (ss == SEC_I_COMPLETE_AND_CONTINUE));

    fSt = TRUE;

ex:
    BufferTerminate( &buffData );
    BufferTerminate( &buff );

    return fSt;
}


bool CWLClientAuthentication::CrackUserAndDomain(char *   pszDomainAndUser,char * * ppszUser,
    char * * ppszDomain)
/*++

Routine Description:

    Given a user name potentially in the form domain\user, zero terminates
    the domain name and returns pointers to the domain name and the user name

Arguments:

    pszDomainAndUser - Pointer to user name or domain and user name
    ppszUser - Receives pointer to user portion of name
    ppszDomain - Receives pointer to domain portion of name

Return Value:

    TRUE if successful, FALSE otherwise (call GetLastError)

--*/
{
    static char szDefaultDomain[MAX_COMPUTERNAME_LENGTH+1];
    DWORD cbN;

    //
    //  Crack the name into domain/user components.
    //

    *ppszDomain = pszDomainAndUser;
    *ppszUser   = strpbrk( pszDomainAndUser, "/\\" );

    if( *ppszUser == NULL )
    {
        //
        //  No domain name specified, just the username so we assume the
        //  user is on the local machine
        //

        if ( !*szDefaultDomain )
        {
            cbN = sizeof(szDefaultDomain);
            GetComputerName( szDefaultDomain, &cbN );
        }

        *ppszDomain = szDefaultDomain;
        *ppszUser   = pszDomainAndUser;
    }
    else
    {
        //
        //  Both domain & user specified, skip delimiter.
        //

        **ppszUser = '\0';
        (*ppszUser)++;

        if( ( **ppszUser == '\0' ) ||
            ( **ppszUser == '\\' ) ||
            ( **ppszUser == '/' ) )
        {
            //
            //  Name is of one of the following (invalid) forms:
            //
            //      "domain\"
            //      "domain\\..."
            //      "domain/..."
            //

            SetLastError( ERROR_INVALID_PARAMETER );
            return FALSE;
        }
    }

    return TRUE;
}


/************************************************************
 *    uuencode/decode functions
 ************************************************************/

//
//  Taken from NCSA HTTP and wwwlib.
//
//  NOTE: These conform to RFC1113, which is slightly different then the Unix
//        uuencode and uudecode!
//

const int pr2six[256]={
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,62,64,64,64,63,
    52,53,54,55,56,57,58,59,60,61,64,64,64,64,64,64,64,0,1,2,3,4,5,6,7,8,9,
    10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,64,64,64,64,64,64,26,27,
    28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
    64,64,64,64,64,64,64,64,64,64,64,64,64
};

char six2pr[64] = {
    'A','B','C','D','E','F','G','H','I','J','K','L','M',
    'N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
    'a','b','c','d','e','f','g','h','i','j','k','l','m',
    'n','o','p','q','r','s','t','u','v','w','x','y','z',
    '0','1','2','3','4','5','6','7','8','9','+','/'
};


bool CWLClientAuthentication::uudecode(
    char   * bufcoded,
    BUFFER * pbuffdecoded,
    DWORD  * pcbDecoded )
/*++

 Routine Description:

    uudecode a string of data

 Arguments:

    bufcoded            pointer to uuencoded data
    pbuffdecoded        pointer to output BUFFER structure
    pcbDecoded          number of decode bytes

 Return Value:

    Returns TRUE is successful; otherwise FALSE is returned.

--*/
{
    int nbytesdecoded;
    char *bufin = bufcoded;
    unsigned char *bufout;
    int nprbytes;

    /* Strip leading whitespace. */

    while(*bufcoded==' ' || *bufcoded == '\t') bufcoded++;

    /* Figure out how many characters are in the input buffer.
     * If this would decode into more bytes than would fit into
     * the output buffer, adjust the number of input bytes downwards.
     */
    bufin = bufcoded;
    while(pr2six[*(bufin++)] <= 63);
    nprbytes = bufin - bufcoded - 1;
    nbytesdecoded = ((nprbytes+3)/4) * 3;

    if ( !BufferResize( pbuffdecoded, nbytesdecoded + 4 ))
        return FALSE;

    if ( pcbDecoded )
        *pcbDecoded = nbytesdecoded;

    bufout = (unsigned char *) BufferQueryPtr(pbuffdecoded);

    bufin = bufcoded;

    while (nprbytes > 0) {
        *(bufout++) =
            (unsigned char) (pr2six[*bufin] << 2 | pr2six[bufin[1]] >> 4);
        *(bufout++) =
            (unsigned char) (pr2six[bufin[1]] << 4 | pr2six[bufin[2]] >> 2);
        *(bufout++) =
            (unsigned char) (pr2six[bufin[2]] << 6 | pr2six[bufin[3]]);
        bufin += 4;
        nprbytes -= 4;
    }

    if(nprbytes & 03) {
        if(pr2six[bufin[-2]] > 63)
            nbytesdecoded -= 2;
        else
            nbytesdecoded -= 1;
    }

    ((char *)BufferQueryPtr(pbuffdecoded))[nbytesdecoded] = '\0';

    return TRUE;
}


bool CWLClientAuthentication::uuencode( BYTE *   bufin,DWORD    nbytes,BUFFER * pbuffEncoded )
/*++

 Routine Description:

    uuencode a string of data

 Arguments:

    bufin           pointer to data to encode
    nbytes          number of bytes to encode
    pbuffEncoded    pointer to output BUFFER structure

 Return Value:

    Returns TRUE is successful; otherwise FALSE is returned.

--*/
{
   unsigned char *outptr;
   unsigned int i;

   //
   //  Resize the buffer to 133% of the incoming data
   //

   if ( !BufferResize( pbuffEncoded, nbytes + ((nbytes + 3) / 3) + 4))
        return FALSE;

   outptr = (unsigned char *) BufferQueryPtr(pbuffEncoded);

   for (i=0; i<nbytes; i += 3) {
      *(outptr++) = six2pr[*bufin >> 2];            /* c1 */
      *(outptr++) = six2pr[((*bufin << 4) & 060) | ((bufin[1] >> 4) & 017)]; /*c2*/
      *(outptr++) = six2pr[((bufin[1] << 2) & 074) | ((bufin[2] >> 6) & 03)];/*c3*/
      *(outptr++) = six2pr[bufin[2] & 077];         /* c4 */

      bufin += 3;
   }

   /* If nbytes was not a multiple of 3, then we have encoded too
    * many characters.  Adjust appropriately.
    */
   if(i == nbytes+1) {
      /* There were only 2 bytes in that last group */
      outptr[-1] = '=';
   } else if(i == nbytes+2) {
      /* There was only 1 byte in that last group */
      outptr[-1] = '=';
      outptr[-2] = '=';
   }

   *outptr = '\0';

   return TRUE;
}


bool CWLClientAuthentication::BufferInit( BUFFER *pB )
{
    pB->pBuf = NULL;
    pB->cLen = 0;

    return TRUE;
}


void CWLClientAuthentication::BufferTerminate( BUFFER *pB )
{
    if ( pB->pBuf != NULL )
    {
        free( pB->pBuf );
        pB->pBuf = NULL;
        pB->cLen = 0;
    }
}


PBYTE CWLClientAuthentication::BufferQueryPtr( BUFFER * pB )
{
    return pB->pBuf;
}


bool CWLClientAuthentication::BufferResize( BUFFER *pB, DWORD cNewL )
{
    PBYTE pN;
    if ( cNewL > pB->cLen )
    {
        pN = (PBYTE)malloc( cNewL );
        if ( pB->pBuf )
        {
            memcpy( pN, pB->pBuf, pB->cLen );
            free( pB->pBuf );
        }
        pB->pBuf = pN;
        pB->cLen = cNewL;
    }

    return TRUE;
}

