/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <windows.h>
#include <rpc.h>
#include <lm.h>

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <tchar.h>
#include <fcntl.h>

#define SECURITY_WIN32
#include "sspi.h"
#include "issperr.h"

#include "platform.h"

// general purpose dynamic buffer structure

typedef struct _BUFFER {
    PBYTE pBuf;
    DWORD cLen;
} BUFFER ;


// structure storing the state of the authentication sequence

typedef struct _AUTH_SEQ {
    BOOL _fNewConversation;
    CredHandle _hcred;
    BOOL _fHaveCredHandle;
    DWORD _cbMaxToken;
    BOOL _fHaveCtxtHandle;
    struct _SecHandle  _hctxt;
    BOOL _fUUEncodeData;
} AUTH_SEQ;

// entry points in the security DLL

typedef struct _SEC_FUNC {
    FREE_CREDENTIALS_HANDLE_FN pFreeCredentialsHandle;
    ACQUIRE_CREDENTIALS_HANDLE_FN pAcquireCredentialsHandle;
    QUERY_SECURITY_PACKAGE_INFO_FN pQuerySecurityPackageInfo;   // A
    FREE_CONTEXT_BUFFER_FN pFreeContextBuffer;
    INITIALIZE_SECURITY_CONTEXT_FN pInitializeSecurityContext;  // A
    COMPLETE_AUTH_TOKEN_FN pCompleteAuthToken;
    DELETE_SECURITY_CONTEXT_FN pDeleteSecurityContext;  // A
} SEC_FUNC;

typedef enum AUTH_STATUS{AUTH_NOT_INIT,AUTH_FAILED,AUTH_SUCCESS};


class CWLProcessAuthentication;

class CWLClientAuthentication {
private:
	bool m_fAuth;
	AUTH_SEQ m_Auth;
	SEC_FUNC* m_sfProcs;
	static AUTH_STATUS m_validity;
	static CWLProcessAuthentication* m_procAuth;

public:
	CWLClientAuthentication();
	static AUTH_STATUS GlobalInit();
	static void GlobalTerminate();
	bool GetValue(char* pch, const char* pchSchemes, const char* pchAuthData, const char* pchUserName, const char* pchPassword, bool* pfNeedMoreData);
	void Clean();


private:
	bool CrackUserAndDomain(char * pszDomainAndUser,char * * ppszUser,char * * ppszDomain);
	bool AuthConverse(void   * pBuffIn,DWORD    cbBuffIn,BUFFER * pbuffOut,
		DWORD  * pcbBuffOut,bool   * pfNeedMoreData,char   * pszPackage,char   * pszUser,
		char   * pszPassword);

	bool AuthInit();
	void AuthTerminate();

	// uuencode/decode routines declaration
	// used to code the authentication blob
	bool uudecode(char   * bufcoded,BUFFER * pbuffdecoded,DWORD  * pcbDecoded );
	bool uuencode( BYTE *   bufin,DWORD    nbytes,BUFFER * pbuffEncoded );

	bool BufferInit( BUFFER *pB );
	void BufferTerminate( BUFFER *pB );
	PBYTE BufferQueryPtr( BUFFER * pB );
	bool BufferResize( BUFFER *pB, DWORD cNewL );

};

