/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLSecureSslConnection.h: interface for the CWLSecureSslConnection class.
//
//////////////////////////////////////////////////////////////////////
#ifdef __OPEN_SSL__

#if !defined(AFX_WLSECURESSLCONNECTION_H__233964F7_08AA_4FEB_B388_7BA55318FD2C__INCLUDED_)
#define AFX_WLSECURESSLCONNECTION_H__233964F7_08AA_4FEB_B388_7BA55318FD2C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <OpenSSL\ssl.h>


#define UNKNOWN_PROTOCOL 0
#define HTTP_PROTOCOL    1
#define SSL_PROTOCOL     2
#define JAVASCRIPT_PROTOCOL    3


#include "httphost.h"
#include "gethostip.h"
#include "WLGenericConnection.h"


#define X509NoCertSupplied -6973
#define X509BadCertificate -6972
#define X509CertificateFile -6971

#define RV_SSL_AUTH_MASK 0x00007F00L
#define RV_SSL_aNULL 0x00000800L

const RVCString SSL_CERT_NO_VERIFICATION = "SSL_CERT_NO_VERIFICATION";
const RVCString SSL_CERT_VALIDATE_ONLY = "SSL_CERT_VALIDATE_ONLY";
const RVCString SSL_CERT_VERIFY_ALL = "SSL_CERT_VERIFY_ALL";


class CWLCommandManager;
class CWLConnectionsPool;
class CWLWorkspace;
//class CWLSlowConnection;
class CWLSSLCiphers;
class CWLTransactionsObject;
class CWLTransactionData;
class CWLHttpRequest;

class CWLSecureSslConnection : public CWLGenericConnection
{
public:

typedef enum
{	SSL_Version_Undetermined = 0,
	SSL_Version_3_0_With_2_0_Hello = 100,
	SSL_Version_3_0_Only = 101,
	TLS_Version_1_0_Only = 102,
	TLS_Version_1_0_With_2_0_Hello = 103,
	SSL_Version_2_0 = 0x0002,
	SSL_Version_3_0 = 0x0300,
	TLS_Version_1_0 = 0x0301
}SSLProtocolVersion;


typedef enum
{	SSL_AllCrypto,
	SSL_StrongCryptoOnly,
	SSL_ExportCryptoOnly,
	SSL_ServerGatedCrypto
} SSLCryptoStrength;

public:
	static void LibraryInit(); // this method must be called before any other action takes place
	static void DestroyCryptoLockers();
	
	CWLSecureSslConnection( CWLHttpRequest* pHttpReq );
	CWLSecureSslConnection( CWLGenericConnection* pConnection );
	
	virtual ~CWLSecureSslConnection();

	virtual bool Init( CWLCommandManager* cm, CWLConnectionsPool* cp, int socketDescriptor, bool createNewSession = false, const char* certFileName = NULL );
	virtual bool Init( CWLCommandManager* cm, CWLConnectionsPool* cp, sockaddr_in& addr, const char* host, int port );
	bool InitSslConnection();

	virtual int Connect();
	virtual int GetAndCheckCertificateFromServer();
	virtual int Send( const char* buffer, int length, int threadNum = 0 );
	virtual int Recv( char* buffer, int* buffsize, int flags, int* err_code, bool* closed, int threadNum = 0 );
	virtual void CloseConnection();
	virtual void CleanUp();

	virtual bool SetCiphers( int strength = SSL_AllCrypto );

	SSL_ERROR_CODE SetSessionId( const unsigned char* buffer, int bufsize );
	SSL_ERROR_CODE StoreSession( unsigned char*& destination );
	SSL_ERROR_CODE RestoreSession( const unsigned char*& source, int length );
	int GetSessionBinarySize( int & err_code );
	virtual int SetCryptographicStrength( int strength );

	bool virtual IsOpen() { return m_TcpConnected && m_SslConnected; }

public:
	static int DetermineVersion(const char* version);
	static int DetermineCryptographicStrength(const char* CryptoStrength);

protected:
	virtual int EstablishConnection( RVCString& host, int port, bool usingProxy, int threadNum );

	static void crypto_lock_cb(int mode, int type, const char *file, int line);
	static unsigned long crypto_thread_id();

private:
	static RVCriticalSection* m_pSslCryptoLock;

	SSL_METHOD* m_Method;
	SSL_CTX* m_Context;
	SSL* m_Ssl;

	RVCString m_CryptoStrength;

	bool m_SslConnected;

	static bool m_LibraryIsInitialized;

#ifdef TRANSPORT_LAYER_TRACE
	FILE* m_File;
	static int m_FileNum;
#endif // TRANSPORT_LAYER_TRACE

};

#endif // !defined(AFX_WLSECURESSLCONNECTION_H__233964F7_08AA_4FEB_B388_7BA55318FD2C__INCLUDED_)

#endif//__OPEN_SSL__
