/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : httpDefines.h
 *
 *	Author : Zion Menahem
 *
 *	Description :
 *
 *	History :
 *
 *	Modifications :
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

#ifndef _httpdefines_h_
#define _httpdefines_h_

/** include files **/

/** local definitions **/
//#define null "(null)"
#define check_null(str) (str ? str : "(null)")

//#ifdef __TV_linux__
//#define max(v1, v2) ((v1) > (v2) ? (v1) : (v2))
//#define min(v1, v2) ((v1) < (v2) ? (v1) : (v2))
//#endif

#ifdef __TV_UNIX__
#define SOCKET_ERROR -1
#endif

#define MAX_FORM_SIZE      1024
#define HTTP_PORT          80
#define SSL_PORT           443
#define PROXY_PORT         8080

#define LF                "\012"
#define CR                "\015"
#define CRandLF           "\015\012"
#define CRLF              CRandLF

#define CONN_REFUSED      1           // Software caused connection abort.

// error files
#define RESPONSE_FILE     "protocol_httpresponse.frm"
#define HTTP_ERROR_FILE   "protocol_httperrors.frm"
#define SSL_ERRORS_FILE   "sslerrors.frm"
#define HTML_ERROR_FILE   "htmlparse.frm"
#define CIPHER_ERROR_FILE "ciphererrors.frm"
// useful defines
#define HTTPPROTOCOL				"HTTP/"
#define VERSION10						10
#define VERSION11						11

#define HTTP_VERSION_10			"1.0"
#define HTTP_VERSION_11			"1.1"
#define HTTP_1_0        "HTTP/1.0"
#define HTTP_1_1        "HTTP/1.1"

#define SRC_URL_TYPE    2
#define HTML_URL_TYPE   1

// return codes
#define RET_ERROR	-1
#define RET_RETRY_FETCH	-5
#define RET_NONE	0
#define RET_OK		1
#define IS_ABORT	-2
#define RET_HEAD_DONE -3

// HTTP headers constants
#define HEADER_SEPARATOR						": "
#define HEADER_CONTENT_LENGTH				"content-length"
#define HEADER_CONTENT_TYPE					"content-type"
#define HEADER_HOST									"host"
#define HEADER_ACCEPT								"accept"
#define HEADER_USER_AGENT						"user-agent"
#define HEADER_CONNECTION						"connection"
#define HEADER_PROXY_CONNECTION			"proxy-connection"
#define HEADER_PROXY_AUTHORIZATION  "proxy-authorization"
#define HEADER_LOCATION							"location"
#define HEADER_COOKIE								"cookie"
#define HEADER_IF_MODIFIED_SINCE		"if-modified-since"
#define HEADER_AUTHENTICATE					"www-authenticate"
#define HEADER_AUTHORIZATION				"authorization"
#define HEADER_PROXY_AUTHENTICATE   "proxy-authenticate"
#define HEADER_REFERER              "referer"
#define HEADER_IF_MODIFIED_SINCE    "if-modified-since"
#define HEADER_IF_MATCH             "if-match"
#define HEADER_IF_NONE_MATCH        "if-none-match"
#define HEADER_EXPIRES              "expires"
#define HEADER_LAST_MODIFIED        "last-modified"
#define HEADER_ETAG                 "etag"
#define HEADER_DATE                 "date"

// HTTP headers constants case sensitive
#define CS_HEADER_CONTENT_LENGTH			 "Content-Length"
#define CS_HEADER_CONTENT_TYPE				 "Content-Type"
#define CS_HEADER_HOST								 "Host"
#define CS_HEADER_ACCEPT							 "Accept"
#define CS_HEADER_ACCEPT_ENCODING			 "Accept-Encoding"
#define CS_HEADER_ACCEPT_LANGUAGE			 "accept-language"
#define CS_HEADER_USER_AGENT					 "User-Agent"
#define CS_HEADER_CONNECTION					 "Connection"
#define CS_HEADER_PROXY_CONNECTION		 "Proxy-Connection"
#define CS_HEADER_PROXY_AUTHORIZATION  "Proxy-Authorization"
#define CS_HEADER_LOCATION						 "Location"
#define CS_HEADER_COOKIE							 "Cookie"
#define CS_HEADER_IF_MODIFIED_SINCE		 "If-Modified-Since"
#define CS_HEADER_AUTHENTICATE				 "WWW-Authenticate"
#define CS_HEADER_AUTHORIZATION				 "Authorization"
#define CS_HEADER_PROXY_AUTHENTICATE   "Proxy-Authenticate"
#define CS_HEADER_REFERER              "Referer"
#define CS_HEADER_IF_MODIFIED_SINCE    "If-Modified-Since"
#define CS_HEADER_IF_MATCH             "If-Match"
#define CS_HEADER_IF_NONE_MATCH        "If-None-Match"
#define CS_HEADER_EXPIRES              "Expires"
#define CS_HEADER_LAST_MODIFIED        "Last-Modified"
#define CS_HEADER_ETAG                 "ETag"
#define CS_HEADER_DATE                 "Date"

// HTTP tokens constants
#define TOKEN_KEEP_ALIVE						"Keep-Alive"
#define TOKEN_CLOSE									"Close"
#define TOKEN_NTLM									"NTLM"
#define TOKEN_BASIC									"Basic"
#define BASIC_AUTHENTICATION				"Basic"
#define NT_AUTHENTICATION						"NT"
#define TOKEN_ENGLISH               "en-us"

#define MIN_THREADS									1
#define DEFAULT_NUM_THREADS					4
#define MULTI_THREADS								"Multiple"
#define SINGLE_THREAD								"Single"
#define DEFAULT_REDIRECTION_LIMIT		10
#define MIN_REDIRECTION_LIMIT				0
#define DEFAULT_USER_AGENT					"Mozilla/4.0 (compatible; MSIE 5.01; Windows NT; DigExt)"
#define YES_STR_VALUE								"Yes"
#define NO_STR_VALUE								"No"
#define POLLTIMEOUT                 200
#define IGNORE_PERSISTANCY          -1

#define DEFAULT_SELECT_TIMEOUT	POLLTIMEOUT	// 200
#define DEFAULT_SELECT_SECOND_TIMEOUT	      0 //20
#define DEFAULT_SELECT_SWITCH_NUM	          100 //3
#define DEFAULT_SELECT_WRITE_TIMEOUT	      200
#define DEFAULT_SELECT_WRITE_SECOND_TIMEOUT	10 //20
#define DEFAULT_SELECT_WRITE_SWITCH_NUM	    100 //1
#define DEFAULT_CONNECT_TIMEOUT	            10000
#define DEFAULT_APPSIGHT_TRANSACTION        "Default Transaction"

#define DEFAULT_DECODE_BINARY_START			-1
#define DEFAULT_DECODE_BINARY_END			-1

#define MIN_SELECT_TIMEOUT			        0
#define MIN_SELECT_SECOND_TIMEOUT			  0
#define MIN_SELECT_SWITCH_NUM   			  1
#define MIN_SELECT_WRITE_TIMEOUT	      0
#define MIN_SELECT_WRITE_SECOND_TIMEOUT	0
#define MIN_SELECT_WRITE_SWITCH_NUM	    1
#define MIN_CONNECT_TIMEOUT	            0
#define DEFAULT_BUFFER_SIZE			        4096
#define MIN_BUFFER_SIZE					        1
#define MIN_SEND_BUFFER_SIZE					  256
#define MAX_SEND_BUFFER_SIZE					  INT_MAX //0x7FFFFFFF for 32 bit integer
#define DEFAULT_SEND_BUFFER_SIZE				-1

#define DEFAULT_REQUEST_MULTI_PART_BUFFER_SIZE	8192
#define MIN_REQUEST_MULTI_PART_BUFFER_SIZE	1

#define RESPONSE_CONTENT_TYPE_HTML						"HTML"
#define RESPONSE_CONTENT_TYPE_XML							"XML"
#define RESPONSE_CONTENT_TYPE_NOT_DEFINED			"NOT_DEFINED"
#define RESPONSE_CONTENT_TYPE_DEFAULT_VALUES	"HTML,XML,NOT_DEFINED"

#define HTTP_CACHE_SCOPE_COMMAND 1
#define HTTP_CACHE_SCOPE_ROUND   2
#define HTTP_CACHE_SCOPE_RUN     3

#define HTTP_CACHE_TYPE_NONE        1 // use old caching mechanism (i.e. not requesting same images in the current command)
#define HTTP_CACHE_TYPE_FRESH       2 // use HTTP caching mechanism but only fresh method is used
#define HTTP_CACHE_TYPE_HEURISTICS  3 // use HTTP caching mechanism with heuristics method

// type to delete at end of round (if scope is entire run)
#define HTTP_CACHE_CLEAR_NONE       1
#define HTTP_CACHE_CLEAR_HTML       2
#define HTTP_CACHE_CLEAR_JS         3
#define HTTP_CACHE_CLEAR_CSS        4
#define HTTP_CACHE_CLEAR_IMAGE      5
#define HTTP_CACHE_CLEAR_APPLET     6
#define HTTP_CACHE_CLEAR_XML        7

#define HTPP_CACHE_CACHE_NONE     0x0000
#define HTTP_CACHE_CACHE_HTML     0x0001
#define HTTP_CACHE_CACHE_JS       0x0002
#define HTTP_CACHE_CACHE_CSS      0x0004
#define HTTP_CACHE_CACHE_XML      0x0008
#define HTTP_CACHE_CACHE_IMAGE    0x0010
#define HTTP_CACHE_CACHE_APPLET   0x0020
#define HTTP_CACHE_CACHE_DYNAMIC  0x0040
#define HTTP_CACHE_CACHE_ALL      0xFFFFFFFF

/** default settings **/

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** classes definitions **/

/** public functions **/

/** private functions **/

#endif _httpdefines_h_
