/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __HTTP_HOST__
#define __HTTP_HOST__

#include <RVCString.h>
#include <collectable.h>
#include <slistcol.h>
#include <hashdict.h>
#include <thr.h>
#include <set.h>

#ifdef __TV_WINDOWS__
#include <winsock.h>
#else
#include <netinet/in.h>
#include <stdlib.h>
#include "windows.h"
#endif


#ifndef INADDR_NONE
#define INADDR_NONE     0xffffffff
#endif /* INADDR_NONE */
#define ADDRESS  sockaddr_in*


class HTTPAddr : public RVCollectable  {
private:
	char*        m_Addresses;
	unsigned     m_AddressCount;
	unsigned     m_CurrentAddress;
	short        m_AddressLength;
	sockaddr_in  sin;
	RVCString    host;
	int          port;

public:
	HTTPAddr() : m_CurrentAddress(0), m_Addresses(NULL) {}
	HTTPAddr(RVCString& h, int p) : host(h), port(p), m_CurrentAddress(0), m_Addresses(NULL) {}
	virtual ~HTTPAddr() { if (m_Addresses) free(m_Addresses); }

	int        convertHost(RVCString& h, int p, int* err);
	bool       getSockAddrIn(sockaddr_in& addr, bool resolveOnly);
	RVCString& getHost() { return (host); }
	int        getPort() { return (port); }

	virtual bool isEqual(const RVCollectable* t) const;
};


class CWLDnsTable : public RVSet {
private:
	RVReadersWriterLock m_ItemLock;

private:
	HTTPAddr* resolveItem(RVCString& host, int port, int* err);
	HTTPAddr* addItem(RVCString& host, int port, int* err);

public:
	CWLDnsTable() {}
	~CWLDnsTable() { clearAndDestroy(); }

	HTTPAddr* getItem(RVCString& host, int port, bool useCache, int* err, bool* actualLookup);
	void clearCache() { clearAndDestroy(); }
};


#endif // on __HTTP_HOST__
