/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : HttpResponse.h
 *
 *	Author : Zion Menahem
 *
 *	Description : 
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/
#ifndef _HttpResponse_h_
#define _HttpResponse_h_

/** include files **/
#include <stdio.h>
#include <string.h>
#include <fstream.h>
#include <ordered.h>
#include <collstr.h>
#include <thr.h>

#include <platform.h>
#include <tvpsync.h>

#include "rvjsapi.h"

#include "httphost.h"
#include "wlerrorlib.h"

#include "outputbuf.h"
#include "jshttpnames.h"
#include "wlgzip.h"

/** local definitions **/

/** default settings **/
#define CWLPage frameContainer

#define INIT_CONTENT_LENGTH -1
#define INIT_STATUS (unsigned)-1

/** external functions **/
extern void  ParseCharForHttp();

/** external data **/
class CWLCommandManager;
class CWLCookies;
class CWLTransactionsObject;
class CWLHttpCacheEntry;
class CWLHttpRequest;

/** internal functions **/

/** public data **/

/** private data **/

/** public functions **/

/** private functions **/

/** classes definitions **/


// chunked data
class CWLChunkedData {
private:
	unsigned long m_DoneSize;
	unsigned long m_LastSize;
	unsigned long m_ChunkSize;
	unsigned long m_TotalSize;
	RVCString     m_CurrData;
	RVCString     m_ParseData;

	RVCString     m_FooterData;
	RVCString     m_LastHeaderName;
	RVCString     m_LastHeaderValue;

	typedef enum  {CHUNK_SIZE, CHUNK_DATA, CHUNK_DATA_END, FOOTER} Status;
	Status        m_CurrState;

public:
	CWLChunkedData();
	virtual ~CWLChunkedData() {}

	void           Init();

	void           addToFooter(const char* buff, int len);
	int            extractHeader();

	bool           getChunkSize(const char* str, unsigned long* size);
	int            extractData(const char* buff, int len);
	int            extractSize(const char* buff, int len);

	int            extractMessage(const char* buff, int len);
	int            extractSize();
	int	           extractData();
	int            extractFooter();


	unsigned long  getLast() { return m_LastSize; }
	size_t         getSize() { return m_CurrData.length(); }
	const char*    getData() { return m_CurrData.data(); }
	unsigned long  getLength() { return m_TotalSize; }

	size_t         getParseSize() { return m_ParseData.length(); }
	const char*    getParseData() { return m_ParseData.data(); }

	RVCString&     lastHeaderName() { return m_LastHeaderName; }
	RVCString&     lastHeaderValue() { return m_LastHeaderValue; }
};

// collect and parse the response from the Get/Post
class CWLHttpResponse : public RVHashDictionary {
private:
	typedef enum ParseState { STATUS_LINE, MESSAGE_HEADER, MESSAGE_BODY, CONTINUE_STATE };
  typedef enum ContentTypes { NOT_DEFINED, HTML_TYPES, XML_TYPES } ;

	int         m_Version;
	unsigned    m_Status;
	RVCString   m_Reason;

	bool        m_KeepAlive;
	int         server_status;
	long        m_ContentLength;
  RVCString   content_type;
	RVCString   m_TransferType;
  ParseState  status;
  RVCString   line;
  long        m_lengthForParseOnly;
  ofstream*   output;

	bool        m_ContResponse;

	CWLChunkedData  m_ChunkedData;
	unsigned long   m_TotalDone;

  // Chaim
  bool      m_UrlFirstTime;
  bool      m_UrlChecked;
  bool      m_QueueRestored;

	bool      m_bIsFirstContentChunk;

	HtmlQueueItemType m_eExpectedTagType;
  // Greg, pointer to the calling class.
  CWLHttpRequest*   m_cpHttpRequest;

public:
#ifdef _DEBUG
	RVCString   buffer;
#endif

public:
  CWLHttpResponse();
  virtual ~CWLHttpResponse();

	void        Init(CWLHttpRequest * p_cHttpRequest);

  int         setOutput(const char* );
	void        resetOutput();

  void        SetServerStatus( int p_nStatus ) { m_Status = p_nStatus; }
  int         GetServerStatus()   { return (m_Status); }
  int         GetContentLen()    { return (m_ContentLength);   }

	bool        GetKeepAlive() { return m_KeepAlive; }
  int         GetHeadersNumEntries()   { return (entries()); }
  const char* GetHeader(const char* );
  void        printHeaderValues();
  
  int         findStatusNumber(RVCString& );
  size_t      findDelimiter(const char* , int* );

	int					ParseResponse(CWLTransactionsObject* p_pTransObj,
                            CWLDocument* frame,
														const char* buffer,
														int len,
														int* done,
                            CWLCookies* CookieTree,
														HttpFuncs method,
														int version,
														CWLUserParseFlags::ParseOptions doParse,
														bool saveSource,
														bool saveHeaders,
                            bool ParseOnce,
                            RVCString& host,
														RVCString& uri,
														HtmlQueueItemType urlType,
														int threadNum,
														CWLCommandManager* cm,
                            bool ExplicitUrl, 
														CWLHttpCacheEntry* p_pCacheEntry,
														int& errCode,
                            RVCString& sBuffer);

	bool        extractVersion();
	bool        extractStatus();
	void        extractReason();

  ContentTypes CheckContentTypes(CWLCommandManager* cm, const RVCString& uri);

  // Added by Chaim, Apr. 16 2001 - These flags are to avoid the need to store/get headers
  // simply to learn the state of the response header parse.
  bool        m_bHeaderContentLength;
  bool        m_bHeaderLocation;
  bool        m_bHeaderAuthenticate;
  bool        m_bHeaderProxyAuthenticate;
  bool        m_bHeaderConnection;

  // Alon, November 2003 - support compared data - GZip
  ZipTypeCode  m_eZipContentCode;

//  RVCString        m_HeaderContentLengthValue;
  RVCString        m_HeaderLocationValue;
  RVCString        m_HeaderAuthenticateValue;
  RVCString        m_HeaderProxyAuthenticateValue;
  RVCString        m_HeaderConnectionValue;

private:
  bool        IsStatusLine(RVCString piToken) ;
	void        addHeader(RVCString& aKey, RVCString& aValue);
  void        AppendHeader(RVCString& aKey, RVCString& aValue);
	int         getVersion(RVCString&);

  // Chaim
  void stripToCrLf();

protected:
  OutputBuf   m_Output ;
  bool        m_bCallback ;
	RVCString   m_sHeaders;
  bool        m_bCheckFileContent;

public:
  void         SetOutputData( RVCString& p_sData ) { m_Output.SetOutputData( p_sData ); }
	void         SetOutputData( const char* p_pData, unsigned p_uLength ) { m_Output.SetOutputData( p_pData, p_uLength ); }
  OutputBuf&   GetOutputBuff() { return m_Output; }
  const char*  GetOutputData(){return m_Output.GetData();}
  int          GetOutputLength(){return m_Output.GetLength();}
  void         SetCheckFileContent(bool bCheck){m_bCheckFileContent=bCheck;}
  bool         IsCheckFileContent(){return m_bCheckFileContent;}
  void         SetCallback(bool bCallback){m_bCallback=bCallback;}
  bool         IsCallbackEnabled(){return m_bCallback;}
  void         SetOutputData(){m_Output.SetOutput();}
  RVCString&   GetResponseHeadersString();
  void         ResetResponseHeadersString();
  RVCString    GetAndResetResponseHeaderString();
  RVCString&   GetContentType(){return content_type;}
  void         SetContentType(RVCString& p_content_type) { content_type = p_content_type; }

  //Galtech
#ifndef _ANSI_ONLY
  void ParseContentType(CWLDocument* pFrame, const RVCString& sValue, CWLCommandManager* cm);
#endif

  ZipTypeCode GetEZipCode();
	void         SetEZipCode( ZipTypeCode p_eZipCode );

	void         SetExpectedTagType( HtmlQueueItemType p_eExpectedTagType ) { m_eExpectedTagType = p_eExpectedTagType; }
};


#endif _HttpResponse_h_
