/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __SSL_CIPHERS_H__
#define	__SSL_CIPHERS_H__

#ifdef __SSL__

#include <thr.h> 
#include "ssl.h"


// cipher commands
enum {CIPHER_NULL_COMMAND,
      CIPHER_SHOW_ALL, 
      CIPHER_SHOW_ENABLED, 
      CIPHER_ENABLE_ALL, 
      CIPHER_DISABLE_ALL, 
      CIPHER_ENABLE, 
      CIPHER_DISABLE };

#define MAX_CIPHER_SUITES   128
#define NO_CIPHER_SUITE     ((MAX_CIPHER_SUITES)+1)
#define DEFAULT_CIPHER_STRENGTH ((unsigned)-1)

class CWLConnectionsPool;


class CWLSSLCiphers {
private:
	CWLConnectionsPool* m_Cp;
//	RVCriticalSection   m_CipherLock;	// actually, a lock is not needed here since methods can
							// only be called by the command manger (Init) or by the script (some functions).
	bool      m_Init;
	unsigned  m_Strength;	// ciphersuites strength limit
	unsigned  m_RequestedStrength;	// requested ciphersuites strength limit (used for error messages)
	unsigned  m_Count;	// number of enabled cipher suites
	int       m_Suites[MAX_CIPHER_SUITES]; // all enabled cipher suites - this array must be synchronized
																	  // with all cipher suites enabled for all SSL context templates.

public:
	CWLSSLCiphers(CWLConnectionsPool* cp);
	~CWLSSLCiphers() {}

public:
	SSLErr  Init(unsigned strength, bool verifyStrength);
	SSLErr  Update(SSLContext* ctx);
	SSLErr  DoCommand(int command, const char* cipherName);
	SSLErr  SetStrength(unsigned strength);

	void    EnableNamed(const char* cipherName, bool& wasChanged);
	void    DisableNamed(const char* cipherName, bool& wasChanged);
	void    EnableId(int cipherSuite, bool& wasChanged);
	void    DisableId(int cipherSuite, bool& wasChanged);

private:
	SSLErr  EnableAll(bool& wasChanged);
	SSLErr  DisableAll(bool& wasChanged);

private:
	void    setAll();
	void    resetAll();
	SSLErr  setStrength(unsigned strength, bool verifyStrength);
	bool    isEnabled(int cipherSuite);
	int     emptySlot();
	SSLErr  setSuites(SSLContext* ctx);
	void    ShowEnabled();
	void    ShowAll();
};

#endif __SSL__
//extern CWLSSLCiphers gSslCiphers;

#endif __SSL_CIPHERS_H__
