/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvjsapi.h"
#include "platform.h"

#define  JS_DOM_ERROR_FILE "jsdomerrors.frm"
#define WLJSNAME_DOM_REPOSITORY_CLASS           "DomRepository"

#define CWLFrame CWLDocument
#define CWLKvp CKVP

#define SCOPE_EACH_ROUND  0
#define SCOPE_OBJ_CLONED -1
#define SCOPE_PARSE_ONCE -2

class RVOrdered;
class RVCollectableString;
class CWLArray;
class CWLDocument;
class CWLElement;
class CWLForm;
class CWLFrame;
class CWLLocation;
class CWLKvp;
class CWLUrl;
class CWLLink;
class CWLLocation;
class CWLOption;
class CWLScript;
class CWLTable;
class CWLTableRow;
class CWLTableCell;
class CWLCellCollection;
class CWLXmlDocument;
class CWLMeta;
class CWLMetaCollection;
class CWLEmbed;
class CWLApplet;
class RVHashDictionary;


bool DllExport WLAddArrayClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddFramesClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddElementClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddFormClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddFrameClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddWindowClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddKvpClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddLinkClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddScriptClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddAppletClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddCSSClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddLocationClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddBrowserLocationClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddImgClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddEmbedClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddActionClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddOptionClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddTableClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddRowClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddCellClass(JSContext *cx, JSObject *obj);
bool DllExport WLAddMetaClass(JSContext *cx, JSObject *obj);
bool DllExport AddDomRepositoryClass(JSContext *cx, JSObject *obj);
bool DllExport AddFunctionalTestReporterClass(JSContext *cx, JSObject *obj);
bool DllExport AddFunctionalTestObjDetailsClass(JSContext *cx, JSObject *obj);
bool DllExport AddTableCompareReporterClass(JSContext *cx,JSObject *obj);
bool DllExport AddIteratorClass(JSContext *cx, JSObject *obj);


JSObject* WLNewObject(JSContext* cx, JSClass* cls, JSObject* proto, JSObject* parent);
// doDelete indicates that the real object of the private should be delete in any case
//  since it is not part of the DOM
void WLSetNewObject(JSContext* cx, JSObject* obj, JSObject* newObj, CWLDocument* Parent, const char* name, void* value, bool toDelete = false);
bool WLDeleteProperty(JSContext* cx, JSObject* obj, const char* name);
void WLFinalize(JSContext*, JSObject*);

bool DllExport WLIsObjectOfClass(JSContext* cx, jsval v, const char* name);

JSBool WLGetPropertyObject(JSContext* cx, JSObject* obj, const char* prop, JSClass* type, void* ptr, jsval* vp);

JSBool DllExport WLSetIntValue(JSContext* cx, JSObject* obj, const char* name, unsigned value, jsval* rval);
JSBool DllExport WLSetBoolValue(JSContext* cx, JSObject* obj, const char* name, bool value, jsval* rval);
JSBool DllExport WLSetDoubleValue(JSContext* cx, JSObject* obj, const char* name, double value, jsval* rval);
JSBool DllExport WLSetStringValue(JSContext* cx, JSObject* obj, const char* name, const char* value, jsval* rval, size_t length = 0);
JSBool DllExport WLSetActionValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLLocation* value, jsval* rval);
JSBool DllExport WLSetArrayValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, const char* name, void** value, jsval* rval);
JSBool DllExport WLSetCellArrayValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLCellCollection* value, jsval* rval);
JSBool DllExport WLSetElementValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLElement* value, jsval* rval);
JSBool DllExport WLSetFormValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLForm* value, jsval* rval);
JSBool DllExport WLSetFrameValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLFrame* value, jsval* rval);
JSBool DllExport WLSetImgValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLEmbed* value, jsval* rval);
JSBool DllExport WLSetEmbedValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLEmbed* value, jsval* rval);
JSBool DllExport WLSetCSSValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLUrl* value, jsval* rval);
JSBool DllExport WLSetXmlValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLXmlDocument* value, jsval* rval);
JSBool DllExport WLSetKvpValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLKvp* value, jsval* rval);
JSBool DllExport WLSetLinkValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLUrl* value, jsval* rval);
JSBool DllExport WLSetLocationValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLLocation* value, jsval* rval);
JSBool DllExport WLSetOptionValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLOption* value, jsval* rval);
JSBool DllExport WLSetScriptValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLScript* value, jsval* rval);
JSBool DllExport WLSetAppletValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLApplet* value, jsval* rval);
JSBool DllExport WLSetWindowValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLFrame* value, jsval* rval);
JSBool DllExport WLSetTableValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLTable* value, jsval* rval);
JSBool DllExport WLSetRowValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLTableRow* value, jsval* rval);
JSBool DllExport WLSetCellValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLTableCell* value, jsval* rval);
JSBool DllExport WLSetMetaValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLMeta* value, jsval* rval);


unsigned     WLIntValue(jsval val);
double       WLDoubleValue(jsval val);
const char*  WLStringValue(jsval val);
CWLArray*    WLArrayValue(JSContext* cx, jsval val);
CWLLocation* WLLocationValue(JSContext* cx, jsval val);
CWLLink*     WLLinkValue(JSContext* cx, jsval val);
CWLDocument* WLDocumentValue(JSContext* cx, jsval val);

JSBool WLInitProperty(JSContext* cx, JSObject* obj, const char* prop);
JSBool WLInitHiddenProperty(JSContext* cx, JSObject* obj, const char* name);

int ReportJSError(JSContext* cx, const char* frmName, int frmId, ...);

JSObject* IteratorNewObject( JSContext* cx, RVHashDictionary& container );
