/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_HTTP_CACHE__
#define __WL_HTTP_CACHE__

#include <slistcol.h>                              // 1
#include <thr.h>
#include <thr.h>
#include <hashdict.h>
#include <collstr.h>
#include <slistcolque.h>

#include "wlgzip.h"
#include "wlhtmlvirtualqueue.h"
#include "platform.h"
#include "tvpsync.h"

#define CACHE_ENTRY_MISS  -1
#define CACHE_ENTRY_FOUND  1
#define CACHE_ENTRY_NEW    2

class CWLHttpCacheEntry : public RVCollectable {
private:
friend class CWLHttpCache;
	enum eEntryStatus { STATUS_NEW = 1, STATUS_APPEND, STATUS_READY };
	eEntryStatus m_eStatus;         // content status
	unsigned     m_uMethod;         // check freshness only/use heuristics
	unsigned     m_uLastAccess;     // when (what round) was this entry last accessed

	RVCString          m_sResourceUrl;
	RVCString          m_sResourceType;    // from the Content-Type header
	RVCString          m_sReferer;		   // from the Referer header
	HtmlQueueItemType  m_eResourceType;    // from the request
	RVCString          m_sValidator;       // from the ETag header
	RVCString          m_sExpirationTime;  // from the Expires header
	time_t             m_uExpirationTime;  // from the Expires header
	RVCString          m_sArrivalTime;     // from the Date header
	time_t             m_uArrivalTime;     // from the Date header
	RVCString          m_sModifiedTime;    // from the Last-Modified header
	time_t             m_uModifiedTime;    // from the Last-Modified header

	RVCString          m_sContent;         // content of resource (image/html/js/css etc.)
        size_t             m_ByteCount;        // Store is separately to deal with null bytes within buffer
	ZipTypeCode        m_eZipCode;         // zip code of content

	/*
	RVMutexLock m_Mutex;
	RVCondition m_Lock;
	*/

public:
	//RVCString sValues;
	CWLHttpCacheEntry( const char* p_psUrl, const char* p_psReferer, HtmlQueueItemType p_eType );
	virtual ~CWLHttpCacheEntry();
																						
private:
	bool isHeuristicFresh( time_t p_uCurrTime );

	bool generateIfModifiedSinceHeader( RVCString& p_sIfModifiedSinceHeader );
	bool generateIfMatchHeader( RVCString& p_sIfMatchHeader );
	bool generateIfNoneMatchHeader( RVCString& p_sIfNoneMatchHeader );

public:
	bool IsFresh( unsigned p_uMethod );
	bool IsAlreadyExpired();

	void SetContentType( const char* p_psContentType );
        RVCString& GetContentType();
	void SetExpires( const char* p_psExpires );
	void SetLastModified( const char* p_psLastModified );
	void SetETag( const char* p_psETag );
	void SetDate( const char* p_psDate );

	bool GenerateConditionalHeaders( unsigned p_uHttpVersion, RVCString& p_sConditionalHeaders );
	void Reset();

	void     Touch( unsigned p_uRoundNum );
	unsigned GetLastAccess();

	static bool   IsMatchingTypes( HtmlQueueItemType p_eTag, unsigned p_uTypes );

	bool IsOfType( unsigned p_uTypes );
	bool IsReady();

	void         UpdateContent( const char* p_psContentType,
															const char* p_psExpires,
															const char* p_psDate,
															const char* p_psETag,
															const char* p_psLastModified,
															const char* p_psContent, unsigned p_uLength );
	void         ClearContent();
	void         AppendContent( const char* p_psContent, unsigned p_uLength );
	void         ContentBegin();
	void         ContentDone();
	const char*  GetContentData();
	size_t       GetContentLength();
        size_t       GetByteCount() { return m_ByteCount; }
	RVCString&   GetContent();
	RVCString&	 GetReferer() { return m_sReferer;}
	void		 SetReferer(const char* p_sReferer ) { m_sReferer = p_sReferer; }
	void         SetContent( RVCString p_sContent );

	void         SetEZipCode( ZipTypeCode p_eZipCode );
	ZipTypeCode  GetEZipCode();
};

class CWLHttpCache
{
private:
	RVHashDictionary m_CacheDictionary;

	unsigned m_uTimeToLive;
	unsigned m_uCachedTypes;
	
	RVMutexLock m_Mutex;

	//unsigned m_uTotal;

public:
	CWLHttpCache( unsigned p_uTimeToLive = 0 );
	virtual ~CWLHttpCache();

	void                DeleteOldEntries( unsigned p_uCurrRound );
	void                ClearAndDestroy();
	CWLHttpCacheEntry*  CreateNewEntry( unsigned p_uRoundNum, RVCollectableString* p_psKey, RVCollectableString* p_psReferer, HtmlQueueItemType p_eType, unsigned& p_uEntryStatus );
	CWLHttpCacheEntry*  UpdateCacheEntry(CWLHttpCacheEntry* pCacheEntry);

	void   SetCachedTypes( unsigned p_uCachedTypes );
	bool   IsCachedType( HtmlQueueItemType p_eTag, const char* p_psUrl );
	void   RemoveTypes( unsigned p_uTypes );

	/*
	void   ContentAdded( unsigned p_uLength );
	size_t GetTotal();
	size_t ContentSize();
	*/
};

#endif

