/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : Workspace.h
 *
 *	Author : Zion Menahem
 *
 *	Description : 
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/
#ifndef _Workspace_h_
#define _Workspace_h_

/** include files **/
#include <stdio.h>
#include <string.h>
#include <fstream.h>
#include <ordered.h>
#include <collstr.h>
#include <thr.h>
#include <thr.h>

#include <platform.h>
#include <tvpsync.h>
#include "loadenginevariablesdata.h"
#include "wlframecontainer.h"


#include "rvjsapi.h"


#define CWLDataBlock CWLArray

class CWLCommandManager;
class CWLArray;

class CWLHttpPrivateData {
public:
  CWLHttpPrivateData(CLEVariablesData* Variables) ; 
  ~CWLHttpPrivateData() ; 
  CLEVariablesData* m_Variables ;
	CWLDataBlock*     m_FormdataBlock;
	CWLDataBlock*     m_HeaderBlock;
	CWLDataBlock*     m_DataBlock;
	CWLDataBlock*     m_DatafileBlock;
};

class CWLPropIndex {
public:
    int Url ;
    int UserName ;
    int PassWord ;

	//Galtech
#ifndef _ANSI_ONLY
// @@Chaim - remove these (CharEncoding)
//	  int CharSet ;
//	  int DomCharset;
    int EnforceCharEncoding;
#endif

    int NTUserName ;
    int NTPassWord ;
    int UserAgent ;
    int AcceptLanguage ;
    int RedirectionLimit ;
    int RedirectionEnabled ;
    int ConnectionSpeed ; 
    int BrowserCache ;
    int HttpCacheType ;
    int HttpCacheScope ;
		int HttpCacheCachedTypes ;
		int HttpCacheUncachedTypes ;
    int DNSUseCache ;
    int Version ;
    int MultiIPSupport ;
    int MultiIPSupportType ;
    int KeepAlive ;
    int Persistancy ;
    int Proxy ;
    int HttpsProxy ;
    int ProxyUserName ;
    int ProxyPassWord ;
    int ProxyNTUserName ;
    int ProxyNTPassWord ;

    int UseSameProxyForSSL;

    int HttpsProxyUserName ;
    int HttpsProxyPassWord ;
    int HttpsProxyNTUserName ;
    int HttpsProxyNTPassWord ;

    int Parse ;
    int ParseForms ;
    int ParseLinks ;
    int ParseScripts ;
    int ParseTables ;
    int SaveSource ;
    int CovertToUTF8ForParse;
    int SaveHeaders ;
    int Outfile ;
    int SSLVersion ;
    int SSLBitLimit ;
    int SSLUseCache ;
    int SSLClientCertificateFile ;
    int SSLClientCertificatePassword ;
    int SSLCryptoStrength ;
    int LoadGeneratorThreads ;
    int ProbingClientThreads ;
    int Erase ;
    int ParseXML ;
    int XMLStandAlone ;
    int VerifyXML ;
    int UsingTimer ;
    int RTAMThreads ;
    int CookieExpiration ;
    int CookieDomain ;
    int CookiePath ;
    int UNDOC_CookieExpiration ;
    int UNDOC_CookieDomain ;
    int UNDOC_CookiePath ;
    int Target ;
    int AutoNamedTrans ;
    int ExpectedUrl ;
    int AsmDiagnostics ;
    int AsmOn ;
    int SubTransactions;
    int TimerTransactions;
    int TransactionTime;
    int Verification;
    int TimeoutSeverity;
    int VerSetMaxThreads;
    int VerSetMaxRounds;
		int VerSetRounds;
		int ParseMetas;
		int ParseImages;
		int ParseEmbeds;
		int ParseApplets;
		int ParseCss;
		int ParseOthers;
		int GetFrames;
		int GetImages;
		int GetEmbeds;
		int GetApplets;
		int GetScripts;
		int GetCss;
		int GetXml;
		int GetOthers;
		int ClientType;
		int ParseContentCompatability;
    int RequestRetries ;
    int ReceiveTimeout ;
    int EncodeFormdata ;
    int EscapeChar ;
    int LowLevelUnescape ;
    int DecodeBinaryNullAs ;
    int DecodeBinaryStart ;
    int DecodeBinaryEnd ;
    int DeleteEmptyCookies;
    int EncodeData ;
    int HtmlContentTypes ;
    int XmlContentTypes ;
    int Referer ;
    int ContentType;
		int	ResponseContentType;
		int RequestContentMultiPartBoundary;

		int  ConnectTimeout ;
		int  SelectWriteTimeout ;
		int  SelectWriteSecondTimeout ;
		int  SelectWriteSwitchNum ;
		int  SelectTimeout ;
		int  SelectSecondTimeout ;
		int  SelectSwitchNum ;
		int  RecvBufferSize ;
		int  SendBufferSize ;
		int  RequestMultiPartBufferSize;
    int  FoldTransactionsTree;
    int  SaveTransaction ;
    int  CheckFileContent ;
    int  ReportHttpErrors ;
    int  RVProduct;
    int AcceptEncodingGzip;
    int  FilterUrl;

		int	 OverrideHostDefault;
	  int  OverrideCookieDefault;
	  int  OverrideIfModifiedSinceDefault;

    // Chaim - ParseOnce.
    int ParseOnce;
    int ParseFullBuffer;

#ifdef _DEBUG
    int FloatProp ;
#endif
} ; 


class CWLWorkspace {

public:

	enum MultiIPSupportType {
		PerClient=0,
		PerRound=1          
	};

  const char* m_Version;		// pointer to constant
	const char* m_Url;
	int         m_MaxThreads;

	bool        m_MultiIP;
	MultiIPSupportType
				m_MultiIPType;
	bool        m_CacheDns;

	bool        m_UseKeepAlive;
  int         m_Persistancy;

  const char* m_ProxyName;
  const char* m_HttpsProxyName;

  bool        m_UseSameProxyForSSL;
  const char* m_OutFilename;
  const char* m_TimerName;

  //content types
  const char* m_HtmlContentTypes ;
  const char* m_XmlContentTypes ;
  const char* m_ResponseContentType;
  const char* m_RequestContentMultiPartBoundary;

	// Authentication variables
	const char* m_Username;
	const char* m_Password;
	const char* m_ProxyUsername;
	const char* m_ProxyPassword;
	const char* m_ProxyNTUsername;
	const char* m_ProxyNTPassword;

	const char* m_HttpsProxyUsername;
	const char* m_HttpsProxyPassword;
	const char* m_HttpsProxyNTUsername;
	const char* m_HttpsProxyNTPassword;

	const char* m_NTUsername;
	const char* m_NTPassword;

	unsigned    m_SlowConnectionRate;

	//Galtech

#ifndef _ANSI_ONLY
// @@Chaim - remove these (CharEncoding)
//	int       m_CharSet;
//	int       m_nDomCharset;
	bool      m_bEnforceGlobalCharEncoding;
#endif


  // SSL variables.
  int         m_SslVersion;
  int		      m_SslCryptoStrength;
	bool        m_CacheSsl;
  const char* m_CertVerification;	// pointer to constant //stav
  const char* m_CertificateFile;
  const char* m_CertificatePassword;
  unsigned    m_SslBitLimit;
  
  // Default user agent
  const char* m_UserAgent;
  const char* m_sAcceptLanguage;
  const char* m_sReferer ;
  const char* m_sContentType ;

  // Redirection limit
  int         m_RedirectionLimit;
  bool        m_RedirectionEnabled;
  
  // Cache machanizem for each round
  bool				m_BrowserCache;
	unsigned    m_HttpCacheType;
	unsigned    m_HttpCacheScope;
	unsigned    m_HttpCacheCachedTypes;
	unsigned    m_HttpCacheUncachedTypes;
  
  // array structures
  CWLDataBlock* FormdataBlock;
  CWLDataBlock* HeaderBlock;
  CWLDataBlock* DataBlock;
  CWLDataBlock* DatafileBlock;

  // Client Type
  int  m_ClientType;

  // html parse variables.
  unsigned long lastRoundNum;
  int           numOfOnce;
	CWLUserParseFlags::ParseOptions doParse;
  CWLUserParseFlags::ParseOptions isParse;
  RVCString   pageName;
  RVCString   lastPageName;
  bool        m_ParseLinks;
  bool        m_ParseForms;
  bool        m_ParseScripts;
  bool        m_ParseTables;
  bool		m_ParseXml;
  bool		m_ParseMetas;
  bool		m_ParseImages;
  bool		m_ParseEmbeds;
  bool    m_ParseApplets;
  bool		m_ParseCss;
  bool		m_ParseOthers;
  bool    m_GetFrames;
  bool    m_GetImages;
  bool		m_GetEmbeds;
  bool    m_GetApplets;
  bool    m_GetScripts;
  bool		m_GetCss;
  bool		m_GetXmls;
  bool		m_GetOthers;
  bool    m_SaveSource;
  bool    m_CovertToUTF8ForParse;
  bool    m_SaveHeaders;
  bool		m_XmlStandAlone;
  bool		m_VerifyXml;

  // Chaim Apr 2001
  bool    m_ParseOnce;
  bool    m_bParseFullBuffer;
  
  // Transactions handling.
  bool m_bSubTransactions ;
  bool m_bTimerTransactions;
  const char* m_cTransactionTime;
  int  m_nTimeoutSeverity;
  bool m_bFoldTransactionsTree;
  bool m_bSaveTransaction;

  // verification
  bool m_bVerification;
  int  m_nVerSetMaxThreads;
  int  m_nVerSetMaxRounds;
  int  m_nVerSetRounds;
  
  // Undocumented Cookie Path, Domain, expiration check flag
  bool        m_CookiePath;
  bool        m_CookieDomain;
  bool        m_CookieExpiration;

  const char* m_Target;
  bool        m_AutoNamedTrans;
  const char* m_ExpectedUrl;
  bool        m_AsmDiagnostics;
  bool        m_AsmOn;

  int         m_RequestRetries;
  long        m_ReceiveTimeout ; //In milliseconds 
	bool        m_EncodeFormdata;
	const char* m_EscapeChar;
	bool		m_LowLevelUnescape;
	const char* m_DecodeBinaryNullAs;
	int			m_DecodeBinaryStart;
	int			m_DecodeBinaryEnd;
  bool        m_DeleteEmptyCookies;
	bool        m_EncodeData;

	int         m_ConnectTimeout ;
	int         m_SelectWriteTimeout ;
	int         m_SelectWriteSecondTimeout ;
	int         m_SelectWriteSwitchNum ;
  int         m_SelectTimeout;
  int         m_SelectSecondTimeout;
  int         m_SelectSwitchNum;
	int         m_RecvBufferSize;
	int         m_SendBufferSize;
	int					m_RequestMultiPartBufferSize;
  int         m_bCheckFileContent;

	bool				m_OverrideHostDefault;
	bool				m_OverrideCookieDefault;
	bool				m_OverrideIfModifiedSinceDefault;

  bool        m_bReportHttpErrors ; 
  const char* m_sRVProduct ;
  bool        m_bAcceptEncodingGzip;

  const char* m_pcFilterUrl;

  CWLWorkspace();
	~CWLWorkspace();

	void    Init(CWLHttpPrivateData* HttpVars) ;
  bool    IsInit() { return m_Variables ? true : false; } 
	void    ReadData(CWLCommandManager*);
  static  int EraseIndex() {return m_VarIndex.Erase ;}
  static  int UrlIndex() {return m_VarIndex.Url ;}
  static  int ExpectedUrlIndex() {return m_VarIndex.ExpectedUrl ;}
  static  int AutoNamedTransIndex() {return m_VarIndex.AutoNamedTrans ;}
	void    ResetVariables() ;
  static void RegisterHttpProperties () ;

private:
  CLEVariablesData* m_Variables;
	void    freeAll();
	// Calculate the max num of threads for load
	void			ReadMaxThreads(CWLCommandManager* cm);

  static CWLPropIndex m_VarIndex ;
  char* TraceAllDataMembers();
};


#endif
