/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "callbackdata.h"
#include <assert.h>


/********************************************************************
  class CLocation.
********************************************************************/

CLocation::CLocation(const char* pLocation, int nOffset, int nLength)
{
  m_pcLocation = pLocation ;
  m_nOffset    = nOffset ;
  m_nLength    = nLength ;
  m_IsImage	   = false;
}

CLocation::CLocation(const char* pLocation, int nOffset, int nLength, bool bIsImage)
{
  m_pcLocation = pLocation ;
  m_nOffset    = nOffset ;
  m_nLength    = nLength ;
  m_IsImage	   = bIsImage;
}


CLocation::~CLocation()
{}


CLocation::CLocation( const CLocation& src )
{
  m_pcLocation = src.m_pcLocation ;
  m_nOffset    = src.m_nOffset ;
  m_nLength    = src.m_nLength ;
  m_IsImage	   = src.m_IsImage ;
}

  

CLocation& CLocation::operator=( const CLocation& src )
{
  m_pcLocation = src.m_pcLocation ;
  m_nOffset    = src.m_nOffset ;
  m_nLength    = src.m_nLength ;
  m_IsImage	   = src.m_IsImage ;
  return *this;
}




/********************************************************************
  class CHttpHeaders.
********************************************************************/

CHttpHeaders::CHttpHeaders()
{}


CHttpHeaders::~CHttpHeaders()
{}


CHttpHeaders::CHttpHeaders( const CHttpHeaders& src )
{
  m_pcRequestHttpHeader   = src.m_pcRequestHttpHeader ; 
  m_pcResponseHttpHeader  = src.m_pcResponseHttpHeader ;
}


CHttpHeaders& CHttpHeaders::operator=( const CHttpHeaders& src )
{
  m_pcRequestHttpHeader   = src.m_pcRequestHttpHeader ; 
  m_pcResponseHttpHeader  = src.m_pcResponseHttpHeader ;
  return *this ;
}


const char* CHttpHeaders::GetRequestHeader()
{
  return m_pcRequestHttpHeader.isNull() ? NULL : m_pcRequestHttpHeader.data();
}


const char* CHttpHeaders::GetResponseHeader()
{
  return m_pcResponseHttpHeader.isNull() ? NULL : m_pcResponseHttpHeader.data();
}


void  CHttpHeaders::SetRequestHeader(const char* sHeader)
{
  m_pcRequestHttpHeader.append(sHeader);
}


void  CHttpHeaders::SetResponseHeader(const char* sHeader)
{
  m_pcResponseHttpHeader.append(sHeader);
}




/********************************************************************
  class CUrlData.
********************************************************************/


CUrlData::CUrlData()
{
  m_pcUrl     = NULL ;
  m_pcOrigUrl = NULL ;
  m_pcReferer = NULL ;
  m_pcFinalUrl= NULL ;
  m_pcTarget  = NULL ;
  m_pDom      = NULL ;
  m_GetFrames = true;
}


CUrlData::~CUrlData()
{
  Reset();
}


void CUrlData::Reset()
{
  if (m_pcTarget)
    delete []m_pcTarget;
  m_pcUrl     = NULL ;
  m_pcOrigUrl = NULL ;
  m_pcReferer = NULL ;
  m_pcFinalUrl= NULL ;
  m_pcTarget  = NULL ;
  m_pDom      = NULL ;
  m_GetFrames = true;
}


void CUrlData::SetUrl(const char* pcUrl)
{
  m_pcUrl = pcUrl ;
}

void CUrlData::SetOriginalUrl(const char* pcOrigUrl)
{
  m_pcOrigUrl = pcOrigUrl ; 
}

void CUrlData::SetReferer(const char* pcReferer)
{
  m_pcReferer = pcReferer; 
}

void CUrlData::SetFinalUrl(const char* pcFinalUrl)
{
  m_pcFinalUrl = pcFinalUrl ;
}

void CUrlData::SetTarget(const char* pcTarget)
{
  if (pcTarget)
  {
    m_pcTarget = new char[strlen(pcTarget)+1]; 
    strcpy(m_pcTarget, pcTarget);
  }
}

void CUrlData::SetDom(CWLDocument* pDom)
{
  m_pDom = pDom ;
}

void CUrlData::SetGetFrames(bool getFrames)
{
  m_GetFrames = getFrames;
}

CUrlData::CUrlData( const CUrlData& src )
{}


CUrlData& CUrlData::operator=( const CUrlData& src )
{ assert(0); return *this; }



/********************************************************************
  class CUrlDataEx.
********************************************************************/

CUrlDataEx::CUrlDataEx()
{
  m_nHttpStatus = 0 ;
  m_eUrlType    = NONE ;
}


CUrlDataEx::~CUrlDataEx()
{
}

void CUrlDataEx::Reset()
{
  CUrlData::Reset();

  m_nHttpStatus = 0 ;
  m_eUrlType    = NONE ;
}

void CUrlDataEx::SetHttpStatus(int stat)
{
  m_nHttpStatus = stat ;
}

void CUrlDataEx::SetUrlType(HtmlQueueItemType eUrlType)
{
  m_eUrlType = eUrlType;
}

CUrlDataEx::CUrlDataEx( const CUrlDataEx& src )
{}


CUrlDataEx& CUrlDataEx::operator=( const CUrlDataEx& src )
{ assert(0); return *this; }



/********************************************************************
  class CallbackData.
********************************************************************/

CallbackData::CallbackData()
{
  m_pcMimeType     = NULL ;
  m_pContent       = NULL ;
  m_nContentLength = 0 ;
  m_nFileSize      = 0 ;
  m_CheckSum       = 0 ;
}


CallbackData::~CallbackData()
{
  LocationList.clear() ;
  HeadersList.clear() ;
}

void CallbackData::Reset()
{
  m_pcMimeType     = MIME_TYPE_DEFAULT ;  
  m_pContent       = NULL ;
  m_nContentLength = 0 ;
  m_nFileSize      = 0 ;
  m_CheckSum       = 0 ;

  LocationList.clear() ;
  HeadersList.clear() ;

  CUrlDataEx::Reset();

}

// Chaim - avoid sending empty mime types to IDE as we saw it can cause erorrs writing to DB
void CallbackData::SetMimeType(const char* pcMimeType)
{
  if (!pcMimeType || !pcMimeType[0]){
    m_pcMimeType = MIME_TYPE_DEFAULT;
  }
  else {
  m_pcMimeType = pcMimeType ;  
}
}

void CallbackData::SetObjContent(void* pContent)
{
  m_pContent = pContent ;
}

void CallbackData::SetObjContentLength(int nLength)
{
  m_nContentLength = nLength ;
}

//////////////////////////////////////////////////////////////////////////
/*
void CallbackData::AddRequestHeader(const char* Header)
{
  CHttpHeaders *pHeader = &HeadersList.last();

  if ( HeadersList.isEmpty() || pHeader->GetRequestHeader() )
  {
    HeadersList.append(CHttpHeaders());
    HeadersList.last().SetRequestHeader(Header);
  }
  else
  {
    pHeader->SetRequestHeader(Header);
  }
}


void CallbackData::AddResponseHeader(const char* Header)
{
  CHttpHeaders *pHeader = &HeadersList.last();

  if ( HeadersList.isEmpty() || pHeader->GetResponseHeader() )
  {
    HeadersList.append(CHttpHeaders());
    HeadersList.last().SetResponseHeader(Header);
  }
  else
  {
    pHeader->SetResponseHeader(Header);
  }
}
*/
//////////////////////////////////////////////////////////////////////////

void CallbackData::AddRequestHeader(const char* Header)
{
	if (HeadersList.isEmpty())
	{
		HeadersList.append(CHttpHeaders());
		HeadersList.last().SetRequestHeader(Header);
	}
	else
	{
		CHttpHeaders *pHeader = &HeadersList.last();
		if (pHeader->GetRequestHeader())
		{
			HeadersList.append(CHttpHeaders());
			HeadersList.last().SetRequestHeader(Header);
		}
		else
			pHeader->SetRequestHeader(Header);
	}
}

void CallbackData::AddResponseHeader(const char* Header)
{
	if ( HeadersList.isEmpty() )
	{
		HeadersList.append(CHttpHeaders());
		HeadersList.last().SetResponseHeader(Header);
	}
	else
	{
		CHttpHeaders *pHeader = &HeadersList.last();
		if ( pHeader->GetResponseHeader() )
		{
			HeadersList.append(CHttpHeaders());
			HeadersList.last().SetResponseHeader(Header);
		}
		else
			pHeader->SetResponseHeader(Header);
	}
}

//////////////////////////////////////////////////////////////////////////



void CallbackData::AddLocation(const char* pLocation, int nOffset, int nLength, bool bIsImage)
{
  LocationList.append(CLocation(pLocation, nOffset, nLength, bIsImage));
}


CallbackData::CallbackData( const CallbackData& src )
{}


CallbackData& CallbackData::operator=( const CallbackData& src )
{ assert(0); return *this; }

void CallbackData::SetFileSize(int nFileSize)
{
  m_nFileSize = nFileSize ;
}

void CallbackData::SetCheckSum(CHECKSUM_VAR checksum)
{
  m_CheckSum = checksum ;
}


