/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : CommandManager.cpp
 *
 *	Author : Zion Menahem
 *
 *	Description : Based on the HTTPUAR module, this file implements
 *	              the HTTP protocol for JavaScript module.
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

/** include files **/
#include <ace/OS.h>
#ifdef __TV_WINDOWS__
#include "wlxmlfuncs.h"
#endif
#include <math.h>
#include "rvjsapi.h"
#include <RVCString.h>
#include <wlerrorlib.h>
#include "loadengineglobals.h"
#include "wlframecontainer.h"
#include "wlparseflags.h"
#include "wltable.h"
#include "wlqueue.h"
#include "httpdefines.h"
#include "commandmanager.h"
#include "connpool.h"
#include "httprequest.h"
#include "jshttpnames.h"
#include "jshttpobj.h"
#include "wljsreply.h"
#include "workspace.h"
#include "wlfuncs.h"
#include "jscookie.h"
#include "jshtml.h"
#include "httpcookies.h"
#include "tracedll.h"
#include "httpembedderreport.h"   // for callbacks
#include "domitem.h"
#include "associativearray.h"

#include "httpprotocol.h"

#ifdef _WINDOWS
#include "../auth/authorization.h"
#endif _WINDOWS
#ifdef __TV_UNIX__
#include <sys/time.h>
#include <sys/resource.h>
#endif
#include "wlhttpcache.h"

/** local definitions **/
#define MAX_THREADS			8
#define MAX_CONNECTIONS 8

#define IntToString TVPIntToString

#define reportError  m_Connections->ReportError	// not depended by the (HTTP) thread
#define setErrorCode m_Connections->SetErrorCode	// not depended by the (HTTP) thread

/** default settings **/

#define min(a,b) (((a) < (b)) ? (a) : (b))
#define max(a,b) (((a) > (b)) ? (a) : (b))


/** external functions **/
extern void WLParse_AddParseFunction();

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** public functions **/

/** private functions **/

CWLCommandManager::CWLCommandManager(iLEProtocolAgendaInMix& father, bool bCallback/*false*/):
iLEProtocolVCInMix(father),
m_bCallback(bCallback),
m_Page(NULL),
m_UrlQueue(NULL),
m_Workspace(NULL),
m_Cookie(NULL),
m_Request(NULL),
m_RequestThread(NULL),
m_Connections(NULL),
m_LastErrorLock(NULL),
m_SharedDataLock(NULL)
{
  // Chaim - Moved old code to OnThreadActivation
}


CWLCommandManager::~CWLCommandManager()
{

	if (m_UrlQueue)
		m_UrlQueue->releaseAllThreads();

	if (m_Workspace)
		delete m_Workspace;
	if (m_Cookie)
		delete m_Cookie;

	// delete all instances of request threads and http requests
	for (int i = 0; i < m_MaxThreads; i++) 
		RemoveThread (i);
		
  if (m_Request && m_Request[m_MaxThreads])
    delete m_Request[m_MaxThreads];
		
	// Free the Request thread array
	if (m_RequestThread) 
		free(m_RequestThread);
	
	// Free the Request array
	if (m_Request)
		free(m_Request);

	if (m_Connections)
		delete m_Connections;

	if (m_UrlQueue)
		delete m_UrlQueue;

//#ifndef _DEBUG
	if (m_Page)
		delete m_Page;
//#endif

	if (m_LastErrorLock)
		delete m_LastErrorLock;
	if (m_SharedDataLock)
		delete m_SharedDataLock;
}


bool CWLCommandManager::UpdateMaxThreads()
{
	
	int i;
	bool addThreadStatus = true;

	int New_MaxThreads = max(m_Workspace->m_MaxThreads, 1);	// at least one thread should exist

	// min. of 1 threads allowed for client
	if (m_Workspace->m_MaxThreads<1) {
		reportError(HTTP_ERROR_FILE, 76);
	}
	
	// max. of 8 threads allowed for each client
	if (New_MaxThreads > MAX_THREADS) {
    reportError(HTTP_ERROR_FILE, 74, New_MaxThreads, MAX_THREADS);
		New_MaxThreads = MAX_THREADS;		
	}

	// if the new max threads is equel to current then exit
	if (m_MaxThreads == New_MaxThreads)
		return addThreadStatus;

	// create the requests queue
	if (m_UrlQueue)
		m_UrlQueue->setNumMaxThreads(New_MaxThreads);
	else {
		m_UrlQueue = new CWLQueue(New_MaxThreads, this);
		m_Page = new CWLPage(m_UrlQueue);
	}

  WlTraceMsg( "Changing threads num from %d to %d",m_MaxThreads, New_MaxThreads);

	// if we should remove threads then we do so
	for (i = New_MaxThreads; i < m_MaxThreads; i++) 
		RemoveThread (i);
	
	// create additional threads as needed
	for (i = m_MaxThreads; ((i < New_MaxThreads) && (!isAbort()) && (addThreadStatus)); i++)
		addThreadStatus = AddThread (i);

	// Even if the adding of the threads had failed then we set it to the number 
	// of the requested threads. In the destractor the removeing of the threads is checking whether or not
	// the threads are allocated...

	m_MaxThreads = New_MaxThreads;
	
	return addThreadStatus;

}

// Add a new thread to the active threads
bool CWLCommandManager::AddThread(int num) {
		
		bool success = true;

		try {

		// Create the new instance for thread and http request
		m_Request[num] = new CWLHttpRequest;

		// Init the Http request
		m_Request[num]->Init(this, m_Connections, num);

		// [0] is used for direct call (by the master)
		if (num == 0) {
			m_RequestThread[num] = NULL;
			m_Request[num]->SetMaster();
WlTraceMsg( "New request created");
				return success;
		}

		m_RequestThread[num] = new RVThread;

		// create a thread function
		#if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)            
				*m_RequestThread[num] = rvMakeThreadFunction(*m_Request[num], &CWLHttpRequest::SlaveThread);
		#else
				*m_RequestThread[num] = rvMakeThreadFunctionM(CWLHttpRequest, *m_Request[num], int, &CWLHttpRequest::SlaveThread);
		#endif

		// init and start the threads
		m_RequestThread[num]->start();

WlTraceMsg( "New thread and request created");
		}
		catch (RVTHRInvalidPointer) {
				WlTraceMsg("RVTHRInvalidPointer Exception raised at add thread");
				reportError(HTTP_ERROR_FILE, 78, "RVTHRInvalidPointer Exception raised at add thread");
				success = false;
		}
		catch (RVTHRThreadActive) {
				WlTraceMsg("RVTHRThreadActive Exception raised at add thread");
				reportError(HTTP_ERROR_FILE, 78, "RVTHRThreadActive Exception raised at add thread");
				success = false;
		}
		catch (RVTHRResourceLimit) {
		 		WlTraceMsg("RVTHRResourceLimit Exception raised at add thread");
				reportError(HTTP_ERROR_FILE, 78, "RVTHRResourceLimit Exception raised at add thread");
				success = false;
		}
		catch (RVTHRInternalError) {
				WlTraceMsg("RVTHRInternalError Exception raised at add thread");
				reportError(HTTP_ERROR_FILE, 78, "RVTHRInternalError Exception raised at add thread");
				success = false;
		}
		catch (...) {
				WlTraceMsg("Unknown Exception raised at add thread");
				reportError(HTTP_ERROR_FILE, 78, "Unknown Exception raised at add thread");
				success = false;
		}
	return success;
}

// Remove a certain thread from the active threads
void CWLCommandManager::RemoveThread(int num) {

WlTraceMsg( "Removing thread");

	if (m_RequestThread && m_RequestThread[num]) {
		m_RequestThread[num]->join();
		delete m_RequestThread[num];
	
	}
	if (m_Request && m_Request[num])
		delete m_Request[num];

}

void CWLCommandManager::AtEndOfRound()
{
WlTraceMsg( "At end of round !!!!");

	// Clear the cache that was handled during the round
  if (m_UrlQueue)
		if ( m_Workspace->m_HttpCacheScope < HTTP_CACHE_SCOPE_RUN )
		  m_UrlQueue->ClearCache();

	// delete all cookies
	if (m_Cookie)
		m_Cookie->ClearAllCookies();

	// close all connections
	if (m_Connections)
		m_Connections->AtEndOfRound();

 	// reset Frame Container settings
	if(m_Page)
	{
		m_Page->SetFrameMode(JS_Document);
	}
  m_lExpectedDomId = 0 ;

	m_AuthenticateBasicHeaders.remove(0);
	m_AuthenticateNTHeaders.remove(0);
	m_AuthenticateBasicProxyHeaders.remove(0);
	m_AuthenticateNTProxyHeaders.remove(0);

	if ( m_Workspace->m_HttpCacheScope == HTTP_CACHE_SCOPE_ROUND )
		m_HttpCache.ClearAndDestroy();

//  FlushStatsCache();
}


bool CWLCommandManager::IsParsed()
{
	return (m_Workspace->isParse != CWLUserParseFlags::DontParse);
}

RVCString& CWLCommandManager::GetPageName()
{
	return m_Workspace->lastPageName;
}

void CWLCommandManager::ResetProperties()
{
	m_Workspace->ResetVariables();
}

CLEVariablesData* CWLCommandManager::GetVariablesParent()
{
	if (!m_VariablesParent)
		m_VariablesParent = GetCurrentLoadEngineVariablesData();
//		m_VariablesParent = m_Workspace->GetVariablesParent();

	return m_VariablesParent;
}

void CWLCommandManager::SetVariablesParent(CLEVariablesData* vars)
{
	m_VariablesParent = vars;

//	m_Workspace->SetVariablesParent(vars);
}

void CWLCommandManager::ClearDnsCache()
{
	if (m_Connections)
		m_Connections->ClearDnsCache();
}
void CWLCommandManager::ClearSslCache()
{
	if (m_Connections)
		m_Connections->ClearSslCache();
}
void CWLCommandManager::ConnectionClose()
{
	if (m_Connections)
		m_Connections->CloseAllConnections();
}

void CWLCommandManager::SetLastError(WLLogMessage::Severity sev, const char* err)
{
	RVCriticalSection::LockGuard Lock(*m_LastErrorLock);

	m_LastSeverity = sev;
	m_LastError = err;
}


void CWLCommandManager::CloseCBData(){
  // Close any possibly open callback
  // In callback mode we always have only one request.
  m_Request[0]->CloseCBData();
}

int CWLCommandManager::DoCommand(CWLHttpPrivateData* HttpVars, HttpFuncs method, JSContext* cx/*NULL*/)
{
	m_ReadFromFile = false;
  m_bWasHttpCommand = true ;
  RVCString sPageName;
  m_lActiveDomId = 0 ;


  if (CWLConnectionsPool::m_gLocalAddr.GetNum() <= 0)
  {
    reportError(HTTP_ERROR_FILE, 77);
    return RET_ERROR;
  }

  SetJScx(cx) ;

	if (!m_Cookie || !m_Workspace || !m_Connections)
		return RET_ERROR;

	m_Workspace->Init(HttpVars);
	m_Workspace->ReadData(this) ;
	if( m_Connections ){
		m_Connections->Init(this, MAX_CONNECTIONS);
	}
  
	m_HttpCache.SetCachedTypes( m_Workspace->m_HttpCacheCachedTypes );

	// Shahar - 28-11-2002 - Fix 226.004 - Even for a head command
	// we will need later to find headers and status from the DOM.
	// This command had prevented from finding this information 
	// because the DOM couldn't be found later in the frame container

	//  if (method == HTTP_FUNCS_HEAD)
	//   sPageName = "Head";
	//  else

    sPageName = m_Workspace->pageName ;
	// End change - Shahar - 28-11-2002 - Fix 226.004

	// Till will have a strong demand for changing the
	// threads amount more then once we are finishing here 
	if (m_MaxThreads == 0) {
		if (!UpdateMaxThreads()) {
			return RET_ERROR;
		}
	}
	else if (m_Workspace->m_MaxThreads != m_MaxThreads) {
    reportError(HTTP_ERROR_FILE, 75, m_MaxThreads);
	}

  // make sure all threads participate in the game. Uri
  m_UrlQueue->resetOnlyMasterRead();

	if (!m_Request || !m_UrlQueue || !m_Page)
		return RET_ERROR;

	// If the Cache level is not done according to browser logic
	// then we clear the cache before we do the http command
	if (!m_Workspace->m_BrowserCache)
    if ( m_Workspace->m_HttpCacheScope == HTTP_CACHE_SCOPE_COMMAND ){
			m_UrlQueue->ClearCache();
  		m_HttpCache.ClearAndDestroy();
    }

  url = "" ;
  if (m_Workspace->m_Url) {
		url = m_Workspace->m_Url;
    url = url.strip(RVCString::both, ' ') ;
    url = url.strip(RVCString::both, '\t');
  }
  if (url.isNull()) {
      reportError(HTTP_ERROR_FILE, 70);
      return RET_ERROR;
  }

  // call start page download callbacks
  if (IsCallbackEnabled() && (method != HTTP_FUNCS_HEAD)) //not head method(get or post)
  {
    m_PageCBData.Reset();

    m_PageCBData.SetUrl(url) ;
    m_PageCBData.SetOriginalUrl("") ;
    m_PageCBData.SetReferer("") ;
    m_PageCBData.SetTarget(m_Workspace->m_Target) ;

    WlTraceMsg("Start download: \"%s\", \"%s\"", m_PageCBData.m_pcUrl, m_PageCBData.m_pcFinalUrl);
    cHttpEmbedderReportInfoPageStartDownload StartDwnld((CUrlData*)&m_PageCBData, m_lExpectedDomId);
    StartDwnld.fReport(cx);
    m_lActiveDomId = StartDwnld.GetDomId() ;
  }

	int anchorPos = url.first('#');
	if (anchorPos != RV_NPOS)
		url.remove(anchorPos);

  // We check if we need to do a full get request or it is just a frame 
	// Replacement - This is done according to the Target indication
  
  // we now need to create new page in order to save the page/frame headers
  int res;
	CWLUserParseFlags parseFlags;
	//perpare flags
	parseFlags.m_Parse					=	m_Workspace->doParse;
	parseFlags.m_ParseContents	=	false; // this will turn on the use of the new parsing flags
	parseFlags.m_ParseLinks			=	m_Workspace->m_ParseLinks;
	parseFlags.m_ParseForms			= m_Workspace->m_ParseForms;
	parseFlags.m_SaveSource			= m_Workspace->m_SaveSource;
	parseFlags.m_ParseTables		= m_Workspace->m_ParseTables;
	parseFlags.m_ParseScripts		= m_Workspace->m_ParseScripts;
	parseFlags.m_ParseXml 			= m_Workspace->m_ParseXml;
	parseFlags.m_ParseImages    = m_Workspace->m_ParseImages;
	parseFlags.m_ParseEmbeds    = m_Workspace->m_ParseEmbeds;
	parseFlags.m_ParseApplets   = m_Workspace->m_ParseApplets;
	parseFlags.m_ParseCss       = m_Workspace->m_ParseCss;
	parseFlags.m_ParseMetas     = m_Workspace->m_ParseMetas;
	parseFlags.m_ParseOthers    = m_Workspace->m_ParseOthers;
	parseFlags.m_XmlStandAlone  = m_Workspace->m_XmlStandAlone;
	parseFlags.m_VerifyXml			= m_Workspace->m_VerifyXml;
        parseFlags.m_ConvertToUTF8  = m_Workspace->m_CovertToUTF8ForParse;

  // Chaim - Set off ParseOnce. ParseResponse will reset it again if and whn necessary.
  // This will make sure the top level URL is not stored in the backuptransaction queue.
  RVCollectableString rwCstr("");
  SetBackup (false, &rwCstr);

  // If we have a target value then we know that we need to replace the frame
  // else it is a new document
  // m_Workspace->m_Target = "";
  if (m_Workspace->m_Target && m_Workspace->m_Target[0] && (method != HTTP_FUNCS_HEAD)) {
    replaceFrameStatus retFrameStatus;
	  res = m_Page->insertNewFrame(url, parseFlags, m_Workspace->m_Target,retFrameStatus);
    WlTraceMsg( "Insert new Frame with url : %s into target : %s", url, m_Workspace->m_Target);
    if (res == RET_ERROR) {
        // Insert new frame into target had failed
      if (retFrameStatus == replaceFrame_INSERT_FRAME_FAILURE ) {
			  reportError(HTTP_ERROR_FILE, 300, m_Workspace->m_Target);
      }
		  else if (retFrameStatus == replaceFrame_INSERT_QUEUE_FAILURE) {
			  reportError(HTTP_ERROR_FILE, 301, m_Workspace->m_Target);
      }
      else if (retFrameStatus == replaceFrame_DELETE_FAILURE) {
			  reportError(HTTP_ERROR_FILE, 302, m_Workspace->m_Target);
      }
    }
    else if (retFrameStatus == replaceFrame_DELETE_NAME_MATCH_NUMBER_MISMATCH_WARNING) {
      if (m_Workspace ->m_AsmDiagnostics) {
        reportError(HTTP_ERROR_FILE, 303, m_Workspace->m_Target);
      }
    }
    else if (retFrameStatus == replaceFrame_DELETE_NUMBER_MATCH_NAME_MISMATCH_WARNING) {
      if (m_Workspace ->m_AsmDiagnostics) {
			  reportError(HTTP_ERROR_FILE, 304, m_Workspace->m_Target);
      }
    }
  }
  else {
    //res== 0  -> The page already exists and is being used
    //res==-1  -> Page insertion failed.
    //res== 1  -> OK
	  res = m_Page->insertNewPage(sPageName, url, parseFlags);
    WlTraceMsg( "Insert new Page with url : %s", url);
    if (res == -1) {	// insert page failed
		  if (!m_Workspace->doParse)
			  reportError(HTTP_ERROR_FILE, 73, check_null(url.data()));
		  else {
	      reportError(HTTP_ERROR_FILE, 72);
	      return RET_ERROR;
		  }
    }
  }
	if (m_Workspace->doParse)
    //It is not important if doParse==ParseOnce or ParseAlways ebcause the next check of DoParse
    //is a boolean check and both values are <> 0.
    m_Workspace->doParse = (res == 0 ? CWLUserParseFlags::DontParse : CWLUserParseFlags::ParseOnce);

	// use slow connection ?
	if (!m_Connections->UseSlowConnection(m_Workspace->m_SlowConnectionRate, m_MaxThreads))
		reportError(HTTP_ERROR_FILE, 160, check_null(url.data()));

	// set cipher suites
	m_Connections->InitCiphers(m_Workspace->m_SslBitLimit);

  srcIndex = 0;
  if (m_Workspace->m_TimerName && m_Workspace->m_TimerName[0])
		resetTimer(m_Workspace->m_TimerName);

	int retStatus = RET_OK;
	bool first = true;
	HtmlQueueItemType urlType;
  int nRet;

	do {
        CWLTransactionsObject* pTransObj ;
        nRet = CONTINUE;

		    urlType = m_UrlQueue->masterRead((RVCollectable**)&pTransObj);
        
        if (!pTransObj)
          break;

        pTransObj->GetTransaction()->SetBeginTransaction(TVPSysTime());
        if (!pTransObj->IsCached()) //not handling cached objects, (only in probing client-one thread)
        {
		      nRet = m_Request[0]->CondDoCommand(method, first, pTransObj, urlType, m_Path, retStatus);
        }
		// @@Chaim - The transaction can be set as cached after the fetch
//        else if (IsCallbackEnabled())
        if (pTransObj->IsCached() && IsCallbackEnabled())
        {
          CUrlDataEx Data ;
          Data.SetUrl(pTransObj->GetObjUrl()) ;
          Data.SetOriginalUrl(pTransObj->GetOriginalUrl()) ;
          Data.SetReferer(pTransObj->GetReferer()) ;
          Data.SetTarget("") ;
          Data.SetUrlType(urlType) ;
          Data.SetHttpStatus(NONE) ;

          cHttpEmbedderReportInfoUseCachedObject CacheDwnld(&Data);
          CacheDwnld.fReport(GetJScx());
        }

		    // replace the POST with a GET after the first request
		    if (first) {
			    srcIndex++;
			    first = false;
			    if (method == HTTP_FUNCS_POST )
				    method = HTTP_FUNCS_GET;
		    }
    
        WLLogMessage::Severity err = pTransObj->GetTransaction()->GetSeverity();
        pTransObj->GetTransaction()->SetEndTransaction(TVPSysTime(), err, m_Workspace->m_bSaveTransaction);

		    if (nRet == BREAK)
			    break;
	    } while (urlType != NONE);


  if (m_Workspace->m_TimerName && m_Workspace->m_TimerName[0] && !isAbort())
		sendTimer(m_Workspace->m_TimerName);

	if ((retStatus == RET_ERROR || isAbort())/* && m_Workspace->doParse*/)
		m_Page->SetInvalidPage(m_Workspace->pageName);

	m_Connections->AddReport();

	return retStatus;
}

/*******************************************************************
    Name  : UpdateParentTransaction(CWLTransactionData *pParentTrans, CWLTransactionData *pTrans)
    Input : parent and object of type CWLTransactionData
    Action: aggregates transaction time and severity to 'parent transaction'.
*******************************************************************/
void CWLCommandManager::UpdateParentTransaction(CWLTransactionData *pParentTrans, CWLTransactionData *pTrans)
{
  if (pParentTrans->GetTransEnd() < pTrans->GetTransEnd())
    pParentTrans->SetEndTransaction(pTrans->GetTransEnd(), pParentTrans->GetSeverity());
  
  //updating Severity 
  if (pTrans->GetSeverity() >= WLLogMessage::Error)
  {
    if (!pParentTrans->IsErrorUnder())
    {
      pParentTrans->SetErrorUnder();
      if (pTrans->GetSeverity() > pParentTrans->GetSeverity())
        pParentTrans->SetFailureReason("Error occurred in internal transaction");
      pParentTrans->AddEvent("Error occurred in internal transaction","",WLLogMessage::Warning,NULL,NULL,0,false);
    }
    pParentTrans->SetSeverity(pTrans->GetSeverity()); //set error in case of Error
  }
  else if (pTrans->IsFailureReason())
  {
    if (!pParentTrans->IsErrorUnder())
    {
      pParentTrans->SetErrorUnder();
      pParentTrans->SetFailureReason("Error occurred in internal transaction");
      pParentTrans->AddEvent("Error occurred in internal transaction","",WLLogMessage::Warning,NULL,NULL,0,false);
    }
  }
}

/*******************************************************************
    Name  : SendBeginTransaction(CWLTransactionData *pTrans)
    Input : CWLTransactionData*.
    Action: call function Begin Transactions.
*******************************************************************/
void CWLCommandManager::SendBeginTransaction(CWLTransactionData *pTrans, bool bTopFrame/*false*/)
{
  if (bTopFrame)
  {
    CAssociativeArray ExtraData ;
    ExtraData.AddCouple("ActiveDomId", m_lActiveDomId);
    const char* pcExtraData = ExtraData.ToString();
    size_t uLength = strlen(pcExtraData);

    VerificationBeginTransaction(pTrans->GetOrigName(), pTrans->GetTransBegin(), "HTTPPage", (void*)pcExtraData, uLength+1);
  }
  else
  {
    VerificationBeginTransaction(pTrans->GetOrigName(), pTrans->GetTransBegin(), "HTTPInternal", NULL, 0);
  }
  
}


/*******************************************************************
    Name  : EndTransaction
    Input : 
    Action: report all events, and call verification end transaction.
*******************************************************************/
bool CWLCommandManager::EndTransaction(CWLTransactionData *pTrans, bool bSendTimers)
{
  CWLEvent *pEvent;
  //go through all events
  for(int nIndex = 0 ; pEvent = pTrans->GetNextEvent(nIndex) ;)
  {
    const char* pType = pEvent->m_sExtraDataType.isNull() ? "" : pEvent->m_sExtraDataType.data() ;
    void* pExtraData = pEvent->m_ExtraData ? pEvent->m_ExtraData : "" ;

    // The condition for reporting event to Log Message
    if (pEvent->m_ulLength || !pEvent->m_bLog)  
    {
      VerificationRecordEvent(pEvent->m_sReason.data(), 
                              pEvent->m_sDescription.data(), 
                              pEvent->m_dTime, 
                              pEvent->m_Severity,
                              pType, 
                              pExtraData, 
                              pEvent->m_ulLength,
                              pEvent->m_bLog);
    }
    else // report to Log Message
    {
      reportTime( pEvent->m_Severity, pEvent->m_sReason.data(), pEvent->m_dTime );
    }
  }

  if (pTrans->IsFailureReason())
  {
    VerificationSetFailureReason(pTrans->GetFailureReason(), pTrans->GetDescription(),pTrans->GetTransEnd(), false);
  }
  // not reporting transactions to the log window, only events

  return VerificationEndTransaction(pTrans->GetOrigName(),pTrans->GetTransEnd(),pTrans->GetSeverity(),pTrans->GetSave(),bSendTimers, false);
}


/*******************************************************************
    Name  : InsertTransactions(CWLDocument *frame)
    Input : Get the "top frame" - the frame holding the DOM.
            bReport - insert all transactions anyway.
    Action: Recursive method ! go through all frames, and enter 
            the transactions. call "InsertFrameTransactions" which
            enter the frame's objects transactions.
    Remark: Being called after a page has been completely downloaded.
            Updating the endTransaction time of frames according to
            the time of the last sub-frames have been downloaded.
*******************************************************************/
bool CWLCommandManager::InsertTransactions(CWLDocument *frame, bool bReport)
{
  if (!frame)
    return true;
  
  CWLTransactionsObject *pTransObj = frame->GetTransactionObj();
  CWLTransactionData *pSubTrans, *pTrans = pTransObj->GetTransaction();
  unsigned int unNumOfEntries = frame->m_Frames->entries(),unSubTrans=1;
	int  nIndex,nPos;
	double dBegin;
  bool bTopFrame = IsTopFrame(pTransObj) ;
//  bool bSendTransaction = m_Workspace->m_AutoNamedTrans && (bTopFrame || m_Workspace->m_bSubTransactions) ;
  bool bSendTransaction = (bTopFrame || m_Workspace->m_bSubTransactions) ;
  bool bRet = true ;
	bool bHttpSubTrans = false;
	
  if (bTopFrame)
  {
    RVCString pgName(pTrans->GetOrigName());

    if (!m_sTransName.isNull())
      pgName = m_sTransName ;
    else if (m_Workspace->m_ExpectedUrl && *(m_Workspace->m_ExpectedUrl))
	{
      pgName = m_Workspace->m_ExpectedUrl ;
	}
	// Shahar - Patch 226.006 - 20-Feb-2003 - Truncate transaction name so it will not
	// include the query string
    if (m_sTransName.isNull())    // do not add 'Page' when PageName was set by TransName.
	{
		size_t queryPos = pgName.first("?");
		if (queryPos != RV_NPOS) {
			pgName.remove (queryPos);
		}
		if(VerificationGetTransactionDepth()>0)
        pgName += " : Page";
      else
        pgName += " Page";
	}
    pTrans->SetOrigName(pgName.data());
  }

	CWLTransactionData HttpSubTrans("Http sub transactions",0);
 	CWLTransactionData RedirectTrans("Redirections",0); // will be destroyed at end of block

  if (bReport || bSendTransaction)
    SendBeginTransaction(pTrans, bTopFrame);

  //send sub-transactions before all objects in frame/window.
  if (bReport || (bSendTransaction && m_Workspace->m_bTimerTransactions && !pTransObj->IsListEmpty()))
  {
    
		if(!pTransObj->IsListEmpty())
		{
			bHttpSubTrans = true;
			nPos = pTransObj->GetFirstSubTransPos();
			dBegin = pTransObj->GetNextTrans(nPos)->GetTransBegin();
			HttpSubTrans.SetBeginTransaction(dBegin);
  		SendBeginTransaction(&HttpSubTrans);
		}
		//go through all sub-transactions
	
		//taking only the first sub-transactions.
	  for (nIndex = pTransObj->GetFirstSubTransPos(),pSubTrans = pTransObj->GetNextTrans(nIndex) ; pSubTrans ;)
		{
			SendBeginTransaction(pSubTrans);
			if(bHttpSubTrans)
				UpdateParentTransaction(&HttpSubTrans, pSubTrans);
		  //do not send statistics
		  bRet &= EndTransaction(pSubTrans,false);
      pSubTrans = pTransObj->GetNextTrans(nIndex);			
      if (!pSubTrans || pSubTrans->IsFirstSubTrans())
        break;
		}	
			
		if(bHttpSubTrans)
			EndTransaction(&HttpSubTrans,false);

		//handling redirections.
    if (pSubTrans)
		{		
			dBegin = pSubTrans->GetTransBegin();
			RedirectTrans.SetBeginTransaction(dBegin);
		
			SendBeginTransaction(&RedirectTrans);
			while (pSubTrans)
			{
        HttpSubTrans.ResetSeverity();
				dBegin = pSubTrans->GetTransBegin();
				HttpSubTrans.SetBeginTransaction(dBegin);
				SendBeginTransaction(&HttpSubTrans);
    	  while (pSubTrans)
				{	
					SendBeginTransaction(pSubTrans);

					//do not send statistics
					bRet &= EndTransaction(pSubTrans,false);
					UpdateParentTransaction(&HttpSubTrans, pSubTrans);
          pSubTrans = pTransObj->GetNextTrans(nIndex);			
          if (!pSubTrans || pSubTrans->IsFirstSubTrans())
            break;
				}
					
				EndTransaction(&HttpSubTrans,false);
				UpdateParentTransaction(&RedirectTrans,&HttpSubTrans);
			}
			EndTransaction(&RedirectTrans,false);
		}
  }

  // inserting all sub-transactions of that frame.
  bRet &= InsertFrameTransactions(frame, bReport); 

  // go through all frames in the current frame
  for (unsigned int unIndex = 0 ; unIndex < unNumOfEntries ; ++unIndex)
  {
    CWLDocument* pFrame = (CWLDocument*)frame->m_Frames->at(unIndex);
    if (!pFrame->GetOriginalUrl().isNull())
      bRet &= InsertTransactions(pFrame, bReport);
  }

  if (bReport || bSendTransaction)
  {          
    //send statistics only to the top frame
    bRet &= EndTransaction(pTrans,bTopFrame);
  }

  return bRet;
}


/*******************************************************************
    Name  : InsertFrameTransactions(CWLDocument *frame, bool bReport)
    Input : CWLDocument* object.
            bReport - insert all transactions anyway.
    Action: Go through all objects in a frame, and enter their
            transactions.
    Remark: Being called by "InsertTransactions".
            Updating endTransaction time of frame according to the
            time of the last object have been downloaded.
            Do not send statistics at all.
*******************************************************************/
bool CWLCommandManager::InsertFrameTransactions(CWLDocument *frame, bool bReport)
{
  CWLTransactionData *pTrans, *pSubTrans, *pMainTrans;
  CWLTransactionsObject *pTransObj;
  bool bRet = true;
	bool bObjects = false;
	bool bHttpSubTrans = false;
	double dBegin;

	CWLTransactionData HttpSubTrans("Http sub transactions");
	CWLTransactionData ObjTrans("Objects");
	
	pTransObj = frame->GetFirstTransactionObj();
  bool bTopFrame = IsTopFrame(pTransObj) ;

	if(frame->GetNextTransactionObj())
		bObjects = true;

	if(bObjects)
	{
		double dBegin = pTransObj->GetTransaction()->GetSubTransBegin();
		ObjTrans.SetBeginTransaction(dBegin);
		SendBeginTransaction(&ObjTrans);
	}
  // go through all objects in frame
  for (pTransObj = frame->GetFirstTransactionObj(),pMainTrans = pTransObj->GetTransaction(); 
            pTransObj=frame->GetNextTransactionObj();)
  {
    bHttpSubTrans = false;
    pTrans = pTransObj->GetTransaction();

    if (bReport || m_Workspace->m_bSubTransactions)
      SendBeginTransaction(pTrans);

    //inserting http transactions of objects.
    if (bReport || (m_Workspace->m_bSubTransactions && m_Workspace->m_bTimerTransactions && !pTransObj->IsListEmpty()))
    {
      HttpSubTrans.ResetSeverity();
		
			if(!pTransObj->IsListEmpty())
			{
				bHttpSubTrans = true;
				int nPos = pTransObj->GetFirstSubTransPos();
				pSubTrans = pTransObj->GetNextTrans(nPos);
				dBegin = pSubTrans->GetTransBegin();
				HttpSubTrans.SetBeginTransaction(dBegin);
			
				SendBeginTransaction(&HttpSubTrans);
			}
			
			//go through all sub-transactions
      for ( int nIndex = pTransObj->GetFirstSubTransPos() ; pSubTrans = pTransObj->GetNextTrans(nIndex) ;)
      {
        SendBeginTransaction(pSubTrans);      
        bRet &= EndTransaction(pSubTrans, false);
				UpdateParentTransaction(&HttpSubTrans, pSubTrans);
      }
			if(bHttpSubTrans)
				EndTransaction(&HttpSubTrans, false);
    }

    if (bReport || m_Workspace->m_bSubTransactions)
    {
      bRet &= EndTransaction(pTrans,false);
    }
		if(bObjects)
			UpdateParentTransaction(&ObjTrans, pTrans);
  }
	if(bObjects)
  {
		EndTransaction(&ObjTrans, false);
    UpdateParentTransaction(pMainTrans, &ObjTrans);
  }
  return bRet;
}



/*******************************************************************
    Name  : UpdateObjectsBeginTimer(CWLTransactionData *pParentTrans, CWLTransactionData *pTrans)
    Input : parent and object of type CWLTransactionData
    Action: aggregates object transaction begining time to 'Objects folder'.
*******************************************************************/
void CWLCommandManager::UpdateObjectsBeginTimer(CWLTransactionData *pParentTrans, CWLTransactionData *pTrans)
{
  
  if(pParentTrans->GetSubTransBegin() > pTrans->GetTransBegin())
		pParentTrans->SetBeginSubTransaction(pTrans->GetTransBegin());

}


/*******************************************************************
    Name  : UpdateTransactions(CWLDocument *frame)
    Input : Get the "top frame" - the frame holding the DOM.
    Action: Recursive method ! go through all frames, update
            the transactions. call "UpdateFrameTransactions" which
            enter the frame's objects transactions.
*******************************************************************/
bool CWLCommandManager::UpdateTransactions(CWLDocument *frame)
{
  if (!frame)
    return false;

  CWLTransactionsObject *pTransObj = frame->GetTransactionObj();
  CWLTransactionData *pSubTrans, *pTrans = pTransObj->GetTransaction();
  unsigned int unNumOfEntries = frame->m_Frames->entries();
  bool bTopFrame = IsTopFrame(pTransObj) ;
  bool bFail = false;

  /////////should check events here.
  if (pTrans->GetSeverity() >= WLLogMessage::Warning)
     bFail = true ;

  //send sub-transactions before all objects in frame/window.
  if (!pTransObj->IsListEmpty())
  {
    //go through all sub-transactions
    for ( int nIndex = pTransObj->GetFirstSubTransPos() ; pSubTrans = pTransObj->GetNextTrans(nIndex) ;)
    {
      /////////should check events here.
      UpdateParentTransaction(pTrans, pSubTrans);

      if (pSubTrans->GetSeverity() >= WLLogMessage::Warning)
        bFail = true ;
    }
  }

  // calculating time & inserting all sub-transactions of that frame.
  bFail |= UpdateFrameTransactions(frame); 

  for (unsigned int unIndex = 0 ; unIndex < unNumOfEntries ; ++unIndex)
  {
    bFail |= UpdateTransactions((CWLDocument*)frame->m_Frames->at(unIndex));
    
    CWLTransactionData* pFrameTrans = ((CWLDocument*)frame->m_Frames->at(unIndex))->GetTransactionObj()->GetTransaction();
    UpdateParentTransaction(pTrans, pFrameTrans);
    if (pFrameTrans->GetSeverity() >= WLLogMessage::Warning)
      bFail = true ;
  }

  //check transaction timeout here 
  if (bTopFrame)
  {
    double dTransLimit = atof(m_Workspace->m_cTransactionTime);
    WLLogMessage::Severity Severity = (WLLogMessage::Severity)m_Workspace->m_nTimeoutSeverity;

    // verification of transaction time checked & reported.
    if (dTransLimit>0 && dTransLimit < pTrans->GetTransEnd() - pTrans->GetTransBegin())
    {
      pTrans->SetFailureReason("Transaction timeout");
      pTrans->SetSeverity(Severity);
      pTrans->AddEvent("Transaction timeout","", Severity);
    }
  }
         
  return bFail;
}

/*******************************************************************
    Name  : UpdateFrameTransactions(CWLDocument *frame)
    Input : CWLDocument* object.
    Action: Go through all objects in a frame, and enter their
            transactions.
    Remark: Being called by "InsertTransactions".
            Updating endTransaction time of frame according to the
            time of the last object have been downloaded.
            updating cached items and items in cache.
*******************************************************************/
bool CWLCommandManager::UpdateFrameTransactions(CWLDocument *frame)
{
  CWLTransactionData *pTrans, *pSubTrans, *pMainTrans;
  CWLTransactionsObject *pTransObj;
  bool bFail = false;

  for (pTransObj = frame->GetFirstTransactionObj(),pMainTrans = pTransObj->GetTransaction(); 
            pTransObj=frame->GetNextTransactionObj();)
  {
    pTrans = pTransObj->GetTransaction();

    //updating cached items, and items which are in the cache.
    if (!pTransObj->IsCached()) // if object is not cached, we're updating the item in cache table.
    {
      CWLTransactionsObject *pCacheTrans = UrlQueue()->GetCachedTransaction(pTransObj->GetObjUrl());
      if (pCacheTrans)
      {
        pCacheTrans->GetTransaction()->SetBeginTransaction(pTrans->GetTransBegin());
        pCacheTrans->GetTransaction()->SetEndTransaction(pTrans->GetTransEnd(), pTrans->GetSeverity());
        pCacheTrans->GetTransaction()->SetFailureReason(pTrans->GetFailureReason());
        pCacheTrans->ResetFileSize(pTransObj->GetFileSize());
        pCacheTrans->ResetChecksum(pTransObj->GetCheckSum());
      }
    }
    else  //updating item from cache.
    {
      CWLTransactionsObject *pCacheTrans = UrlQueue()->GetCachedTransaction(pTransObj->GetObjUrl());
      if (pCacheTrans)
      {
        pTransObj->GetTransaction()->SetBeginTransaction(pCacheTrans->GetTransaction()->GetTransBegin());
        pTransObj->GetTransaction()->SetEndTransaction(pCacheTrans->GetTransaction()->GetTransEnd(), WLLogMessage::Success);
        pTransObj->ResetFileSize(pCacheTrans->GetFileSize());
        pTransObj->ResetChecksum(pCacheTrans->GetCheckSum());

        pTransObj->AddSubTransaction("Load from Cache", pCacheTrans->GetTransaction()->GetTransBegin());
        pTransObj->GetLastSubTransation()->SetEndTransaction(pCacheTrans->GetTransaction()->GetTransEnd(), pCacheTrans->GetTransaction()->GetSeverity());

        if (pCacheTrans->GetTransaction()->GetSeverity() >= WLLogMessage::Warning)
          pTransObj->GetLastSubTransation()->SetFailureReason("Load file from cache failed.");
      }
    }

    /////////should check events here.
    UpdateParentTransaction(pMainTrans, pTrans);
		UpdateObjectsBeginTimer(pMainTrans, pTrans);

    if (pTrans->GetSeverity() >= WLLogMessage::Warning)
      bFail = true ;

    if (!pTransObj->IsListEmpty())
    {
      //go through all sub-transactions
      for ( int nIndex = pTransObj->GetFirstSubTransPos() ; pSubTrans = pTransObj->GetNextTrans(nIndex) ;)
      {
        /////////should check events here.
        UpdateParentTransaction(pTrans, pSubTrans);

        if (pSubTrans->GetSeverity() >= WLLogMessage::Warning)
          bFail = true ;
      }
    }
  }
  return bFail;
}

/*******************************************************************
    Name  : InsertAllTransactions()
    Input : Get the "top frame" - the frame holding the DOM.
    Action: Recursive method ! go through all frames, and enter 
            the transactions. call "InsertFrameTransactions" which
            enter the frame's objects transactions.
    Remark: Being called after a page has been completely downloaded.
            Updating the endTransaction time of frames according to
            the time of the last sub-frames have been downloaded.
*******************************************************************/
bool CWLCommandManager::InsertAllTransactions()
{
  WlTraceMsg ("function: InsertAllTransactions");

//  bool bReport;
  CWLDocument *frame = m_Page->GetPageTransaction();

  if (!m_Workspace->m_AutoNamedTrans || !frame)
  {
    WLLogMessage::Severity severity = GetErrorSeverity();
    if (!m_sTransName.isNull() && frame)
    {
      CWLTransactionData *pTrans = (frame->GetTransactionObj())->GetTransaction();
      pTrans->SetOrigName(m_sTransName.data());
      if (severity < WLLogMessage::Note)
        pTrans->SetSeverity(severity);

      SendBeginTransaction(pTrans, true);
      EndTransaction(pTrans, true);
    }
    return (severity < WLLogMessage::Error) ;
  }

//  bReport = UpdateTransactions(frame);
  return InsertTransactions(frame, m_bErrorInTransTree);
}

/*******************************************************************
    Name  : UpdateAllTransactions()
    Input : Get the "top frame" - the frame holding the DOM.
    Action: Recursive method ! go through all frames, and update
            all transactions severity, etc.
    Remark: Being called after a page has been completely downloaded.
            Updating the endTransaction time of frames according to
            the time of the last sub-frames have been downloaded.
*******************************************************************/
void CWLCommandManager::UpdateAllTransactions()
{
  WlTraceMsg ("function: UpdateAllTransactions");
  m_bErrorInTransTree = false ;

  CWLDocument *frame = m_Page->GetPageTransaction();

  if (m_Workspace->m_AutoNamedTrans && frame)
  {
    m_bErrorInTransTree = UpdateTransactions(frame);
  }
}

/*******************************************************************
    Name  : UpdateGetTime()
    Action: Change the top's frame transaction's time to current time.
    Remark: Being called after a page has been completely downloaded.
*******************************************************************/
void CWLCommandManager::UpdateGetTime()
{
  CWLDocument *frame = NULL;
  if (m_Page)
    frame = m_Page->GetPageTransaction();
  if (!frame)
    return ;
  CWLTransactionData *pTrans = (frame->GetTransactionObj())->GetTransaction();
  pTrans->SetEndTransaction(TVPSysTime(),pTrans->GetSeverity());
}

/*******************************************************************
    Name  : GetPageTime()
    Action: returns the top frame transaction's time to current time.
    Remark: Being called after a page has been completely downloaded,
            and UpdateGetTime() were already called.
*******************************************************************/
double CWLCommandManager::GetPageTime()
{
  CWLDocument *frame = m_Page->GetPageTransaction();
  if (!frame)
    return 0 ;
  CWLTransactionData *pTrans = (frame->GetTransactionObj())->GetTransaction();
  return (pTrans->GetTransEnd() - pTrans->GetTransBegin());
}

void CWLCommandManager::IncrementRequestOnConnectionCounter(){
  RVCriticalSection::LockGuard Lock(m_RequestOnConnectionCounterGuard);
  m_RequestOnConnectionCounter++;
}

void CWLCommandManager::ResetRequestOnConnectionCounter(){
  RVCriticalSection::LockGuard Lock(m_RequestOnConnectionCounterGuard);
  m_RequestOnConnectionCounter = 0;
}

int CWLCommandManager::GetRequestOnConnectionCounter(){
  RVCriticalSection::LockGuard Lock(m_RequestOnConnectionCounterGuard);
  return m_RequestOnConnectionCounter;
}

/*******************************************************************
    Name  : IsTopFrame()
    Action: Check if the received transaction object is of top frame.
*******************************************************************/
bool CWLCommandManager::IsTopFrame(CWLTransactionsObject* pTransObj)
{
  return (pTransObj->GetObject() == GetPage()->GetPageTransaction()) ;
}

void CWLCommandManager::SetJSDocument(JSObject* obj)
{
	m_JSDocument = obj;
}

JSObject* CWLCommandManager::GetJSDocument()
{
	return m_JSDocument;
}



// js global http functions
// ========================

// read a file and parse it into the frame object
int CWLCommandManager::ParseFile(RVCString& path, CWLDocument* frame)
{
	// convert the url to os path
	RVCString filename = frame->GetLocation().Source();
	if (path.isNull()) {
		size_t lastDir = filename.last('/');
		if (lastDir == RV_NPOS)
			lastDir = filename.last('\\');
		if (lastDir != RV_NPOS)
			path = filename(0, lastDir + 1);
	}
	if (filename(0) != '/' && filename(0) != '\\' && filename.index(':') == RV_NPOS)
		filename.prepend(path);

WlTraceMsg( "Open file: %s", filename.data());
	FILE* fd = fopen(filename.data(), "rb");
	if (!fd) {
		// can't open file
		return reportError(HTTP_ERROR_FILE, 230, filename.data());
	}

	//set the current charset
#ifndef _ANSI_ONLY
// @@Chaim - CharEncoding change - always use loadengine default
//	frame->SetCharset((m_Workspace->m_CharSet == CP_DEFAULT) ? GetMonitorCharSet() : m_Workspace->m_CharSet, m_Workspace->m_bEnforceGlobalCharset);
	frame->SetCharset(GetMonitorCharSet(), m_Workspace->m_bEnforceGlobalCharEncoding);
#endif
	char buff[4096] = "";
	do {
		size_t num = fread(buff, 1, sizeof(buff), fd);
		frame->parse(buff, num);
	} while (!feof(fd));

	fclose(fd);

	return RET_OK;
}


int CWLCommandManager::CreateDom(CWLHttpPrivateData* HttpVars, const char* file, CWLDocument** dom, int mode)
{
	if (!m_Workspace->IsInit()) {
		m_Workspace->Init(HttpVars);
		m_Workspace->ReadData(this);
		if (!UpdateMaxThreads()) {
			return RET_ERROR;
		}
	}

  // make sure only the master participates.
  m_UrlQueue->setOnlyMasterRead();

  m_ReadFromFile = true;

	RVCString name("created dom");
	RVCString url(file);
	m_Workspace->m_SaveSource = true;	// always save the created dom source
	switch (mode) {
		case FullDom:
			// create a dom object (including it's whole frames tree) from files
			// files should be known to the load machine
			{
				//flags object
				CWLUserParseFlags parseFlags;
				//perpare flags
				parseFlags.m_Parse					=	CWLUserParseFlags::ParseAlways;
				parseFlags.m_ParseContents	=	false; // act upon individual flags
				parseFlags.m_ParseLinks			=	true;
				parseFlags.m_ParseForms			= true;
				parseFlags.m_SaveSource			= m_Workspace->m_SaveSource;
				parseFlags.m_ParseTables		= true;
				parseFlags.m_ParseScripts		= true;
				parseFlags.m_ParseXml				= true;
    	  parseFlags.m_ParseImages    = true;
	      parseFlags.m_ParseEmbeds    = true;
	      parseFlags.m_ParseApplets   = true;
	      parseFlags.m_ParseCss       = true;
	      parseFlags.m_ParseMetas     = true;
      	parseFlags.m_ParseOthers    = true;
				parseFlags.m_XmlStandAlone	= true;
        parseFlags.m_ConvertToUTF8  = false;
				m_Page->insertNewPage(name, url, parseFlags);
			}
			break;

		case OnlyTables:
			// create a dom object from a file, containing only tables
			{
				//flags object
				CWLUserParseFlags parseFlags;
				//perpare flags
				parseFlags.m_Parse					=	CWLUserParseFlags::ParseAlways;
				parseFlags.m_ParseContents	=	false;
				parseFlags.m_ParseLinks			=	false;
				parseFlags.m_ParseForms			= false;
				parseFlags.m_SaveSource			= m_Workspace->m_SaveSource;
				parseFlags.m_ParseTables		= true;
				parseFlags.m_ParseScripts		= false;
    	  parseFlags.m_ParseImages    = false;
	      parseFlags.m_ParseEmbeds    = false;
	      parseFlags.m_ParseApplets   = false;
	      parseFlags.m_ParseCss       = false;
	      parseFlags.m_ParseMetas     = false;
      	parseFlags.m_ParseOthers    = false;
				parseFlags.m_ParseXml				= false;
				parseFlags.m_XmlStandAlone	= true;
        parseFlags.m_ConvertToUTF8  = false;
				m_Page->insertNewPage(name, url, parseFlags);
				//m_Page->insertNewPage(name, url, false, false, false, m_Workspace->m_SaveSource, true, false, false, WLNO);
			}
			break;

		default:
			break;
	}
	*dom = m_Page->GetGlobalDoc();
	HtmlQueueItemType urlType;

	m_Path = "";

	int retStatus = RET_OK;
  int ret;
  CWLTransactionsObject* pTransObj ;

	do
  {
		urlType = m_UrlQueue->masterRead((RVCollectable**)&pTransObj);
    if (!pTransObj)
      break;

    if (!pTransObj->IsCached()) //not handling cached objects, (only in probing client-one thread)
  	  ret = m_Request[0]->CondDoCommand(HTTP_FUNCS_NONE, FALSE, pTransObj, urlType, m_Path, retStatus);

		if (ret == BREAK)
  		break;
	} while (urlType != NONE);


  m_UrlQueue->resetOnlyMasterRead();  // return to default settings.

  return retStatus;
}

int CWLCommandManager::CreateExpectedDom(CWLHttpPrivateData* HttpVars, CWLDomItem* pDomItem, CWLDocument** dom)
{
	if (!m_Workspace->IsInit()) {
		m_Workspace->Init(HttpVars);
		m_Workspace->ReadData(this);
		if (!UpdateMaxThreads()) {
			return RET_ERROR;
		}
  }

  // make sure only master participates.
  m_UrlQueue->setOnlyMasterRead();

  RVCString sName("created expected dom");
  const char *pRootUrl = pDomItem->getRootUrl();
  RVCString sUrl = pRootUrl ? pRootUrl : "";

  //flags object
	CWLUserParseFlags parseFlags;
	//perpare flags
	parseFlags.m_Parse					=	CWLUserParseFlags::ParseAlways;
	parseFlags.m_ParseContents	=	false; // use individual flags
	parseFlags.m_ParseLinks			=	true;
	parseFlags.m_ParseForms			= true;
	parseFlags.m_SaveSource			= true;
	parseFlags.m_ParseTables		= true;
	parseFlags.m_ParseScripts		= true;
	parseFlags.m_ParseXml				= true;
        parseFlags.m_ParseImages    = true;
	parseFlags.m_ParseEmbeds    = true;
	parseFlags.m_ParseApplets   = true;
	parseFlags.m_ParseCss       = true;
	parseFlags.m_ParseMetas     = true;
 	parseFlags.m_ParseOthers    = true;
	parseFlags.m_XmlStandAlone	= true;
        parseFlags.m_ConvertToUTF8  = false;
	m_Page->insertNewPage(sName, sUrl, parseFlags);


	*dom = m_Page->GetGlobalDoc();
	HtmlQueueItemType objType;
	int retStatus = RET_OK;
  CWLTransactionsObject* pTransObj ;

	do
  {
    objType = m_UrlQueue->masterRead((RVCollectable**)&pTransObj);

    if (!pTransObj)
      break;

    if (pTransObj->IsCached())      // not handling cached items, like before
      continue;

    if (objType == TERMINATE || objType == RELEASE)
    {
   		if (objType == TERMINATE)
  			retStatus = RET_ERROR;
      break;
    }

    if (objType == FRAME )
    {
      CWLDocument* frame = (CWLDocument*)pTransObj->GetObject();
      CWLItemFrame* pItemFrame = pDomItem->get(frame->GetDocUrl(), (frame->GetFrameTarget()).getStr().data());
      if (!pItemFrame)
      {
  			retStatus = RET_ERROR;
        m_UrlQueue->clearAndDestroy();
        break;
      }
      frame->SetDocUrl(pItemFrame->getUrl());
#ifndef _ANSI_ONLY
			//set the current charset
		  // @@Chaim - CharEncoding change - always use loadengine default
//		  frame->SetCharset((m_Workspace->m_CharSet == CP_DEFAULT) ? GetMonitorCharSet() : m_Workspace->m_CharSet, m_Workspace->m_bEnforceGlobalCharset);
		  frame->SetCharset(GetMonitorCharSet(), m_Workspace->m_bEnforceGlobalCharEncoding);
#endif
  		frame->parse(pItemFrame->getSource(), strlen(pItemFrame->getSource()));
    }

	} while (objType != NONE);

  m_UrlQueue->resetOnlyMasterRead(); // return to default settings.

  return retStatus;
}

CWLDocument* CWLCommandManager::GetParsedHtmlTree()
{
  return m_Page->GetGlobalDoc();
}


// debug functions
// ===============

#ifdef _DEBUG
void CWLCommandManager::ParseHtmlFile(const char* file)
{

	FILE* fd = fopen(file, "r");
	if (!fd)
		return;

	m_UrlQueue = new CWLQueue(4, this);
	m_Page = new CWLPage(m_UrlQueue);
	RVCString name("zion");
	RVCString url("file");
	//flags object
	CWLUserParseFlags parseFlags;
	//perpare flags
	parseFlags.m_Parse					=	CWLUserParseFlags::DontParse;
	parseFlags.m_ParseContents	=	false;
	parseFlags.m_ParseLinks			=	true;
	parseFlags.m_ParseForms			= false;
	parseFlags.m_SaveSource			= false;
	parseFlags.m_ParseTables		= false;
	parseFlags.m_ParseScripts		= false;
	parseFlags.m_ParseXml				= false;
	parseFlags.m_XmlStandAlone	= true;
        parseFlags.m_ConvertToUTF8  = false;
	m_Page->insertNewPage(name, url, parseFlags);
//	m_Page->insertNewPage(name, url, false, true, false, false, false, false, false, false);

	//Galtech: set the current charset
#ifndef _ANSI_ONLY
// @@Chaim - CharEncoding change - always use loadengine default
//	m_Page->getCurrentPage()->SetCharset((m_Workspace->m_CharSet == CP_DEFAULT) ? GetMonitorCharSet() : m_Workspace->m_CharSet, m_Workspace->m_bEnforceGlobalCharset);
	m_Page->getCurrentPage()->SetCharset(GetMonitorCharSet(), m_Workspace->m_bEnforceGlobalCharEncoding);
#endif


	char buff[4096];
	do {
		size_t num = fread(buff, 1, 4095, fd);
		buff[num] = '\0';
		m_Page->getCurrentPage()->parse(buff, num);
	} while (!feof(fd));

	fclose(fd);
}

void CWLCommandManager::DebugFunction(int msg)
{
	switch (msg) {
		case DEBUG_SET_END_OF_ROUND :
			m_RoundNum++;
			AtEndOfRound();
			break;

		default:
			break;
	}
}

char* recv_buff_file = "y:\\keepaliveres.dat";
unsigned recv_buff_size = 16;
bool use_response_file = false;
void CWLCommandManager::ReadResponseFromFile(char* file, int size)
{
	if (file && *file) {
		unsigned len = strlen(file);
		recv_buff_file = (char*)malloc(len + 1);
		memmove(recv_buff_file, file, len);
		recv_buff_file[len] = 0;
	}

	if (size > 0)
		recv_buff_size = size;

	use_response_file = true;
}

void CWLCommandManager::CreateTable(const char* name, int col, int rows, int cols)
{
	if (!m_UrlQueue)
		m_UrlQueue = new CWLQueue(4, this);
	if (!m_Page)
		m_Page = new CWLPage(m_UrlQueue);
	RVCString pagename("zion");
	RVCString url("www.host.com/URI");
	//flags object
	CWLUserParseFlags parseFlags;
	//perpare flags
	parseFlags.m_Parse					=	CWLUserParseFlags::DontParse;
	parseFlags.m_ParseContents	=	false;
	parseFlags.m_ParseLinks			=	true;
	parseFlags.m_ParseForms			= false;
	parseFlags.m_SaveSource			= false;
	parseFlags.m_ParseTables		= false;
	parseFlags.m_ParseScripts		= false;
	parseFlags.m_ParseXml 			= false;
	parseFlags.m_XmlStandAlone	= true;
        parseFlags.m_ConvertToUTF8  = false;
	m_Page->insertNewPage(pagename, url, parseFlags);
//	m_Page->insertNewPage(pagename, url, false, true, false, false, false, false, false, false);
	CWLDocument* pDom = m_Page->GetGlobalDoc();

	CWLTable* table = debugCreateTable(pDom, (char*)name, col, rows, cols);
	pDom->m_Tables->insert(table);
}

#endif



//==================================================





/******************************************************************\
 *	Function name : SetCookie
 *	Description   :
 *
 *	Return type   : int -
 *	Argument      : RVCString& name -
 *	Argument      : RVCString& value -
 *	Argument      : RVCString& path -
 *	Argument      : RVCString& domain -
 *	Argument      : RVCString& expire -
 *
 *	Modifications :
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/
int CWLCommandManager::SetCookie(char* name, char* value, char* domain, char* path, char* expire)
{
	RVCString Name(name), Value(value), Path(path), Domain(domain), Expire(expire);

  if (m_Cookie->InsertCookie(Name, Value, Path, Domain, Expire) == RET_ERROR)  {
    reportError(HTTP_ERROR_FILE, 25, check_null(name), check_null(value), check_null(domain), check_null(path));
    return RET_ERROR;
  }

	return RET_OK;
}


/******************************************************************\
 *	Function name : DeleteCookie
 *	Description   :
 *
 *	Return type   : int -
 *	Argument      : RVCString& name -
 *	Argument      : RVCString& path -
 *	Argument      : RVCString& domain -
 *
 *	Modifications :
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/
int CWLCommandManager::DeleteCookie(const char* name, const char* domain, const char* path)
{
	RVCString Name(name), Path(path), Domain(domain);

  if (m_Cookie->DeleteCookie(Name, Path, Domain) == RET_ERROR)  {
    reportError(HTTP_ERROR_FILE, 27, check_null(name), check_null(domain), check_null(path));
    return RET_ERROR;
  }

	return RET_OK;
}


/******************************************************************\
 *	Function name : ClearAllCookies
 *	Description   :
 *
 *	Return type   : void -
 *
 *	Modifications :
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/
void CWLCommandManager::ClearAllCookies()
{
  m_Cookie->ClearAllCookies();
}



// Chaim
void CWLCommandManager::SetBackup (bool sw, RVCollectableString* key) {
  m_UrlQueue->SetBackup (sw, key);
}

bool CWLCommandManager::UseBackup (RVCollectableString* key) {
  return m_UrlQueue->UseBackup (key);
}

bool CWLCommandManager::BackupExists (RVCollectableString* key) {
  return m_UrlQueue->BackupExists (key);
}


bool CWLCommandManager::JSInitializeForClient(JSContext* cx, JSObject* obj, CLEVariablesData* pVariablesData){

  // Chaim - moved from WLInitHttpGeneratorThread
	JSObject* retobj = NULL;
  bool status = true;


	status &= WLAddHttpFunctions(cx, obj);

	status &= WLAddHttpObject(cx, obj, pVariablesData);
	status &= WLAddHtmlObject(cx, obj);
	status &= WLAddCookieObject(cx, obj);
	status &= WLAddDomObject(cx, obj, &retobj);
	status &= WLAddWindowObject(cx, obj, &retobj);		// browser mode
	status &= WLAddLocationObject(cx, obj, &retobj);	// browser mode
	status &= WLAddFramesObject(cx, obj, &retobj);		// browser mode

  SetVariablesParent(pVariablesData);

  return status;

  return status;
}


bool CWLCommandManager::OnThreadActivation() { 
	WlTraceMsg("HTTP Protocol : Called the protocol callback OnThreadActivation");	
//WlTraceMsg( "Master thread : %x", GetCurrentThreadId());
	m_Initialized = false;

	m_MaxThreads = 0;
	m_RequestThread = NULL;
	m_Request = NULL;
	m_UrlQueue = NULL;
	m_Page = NULL;
	m_VariablesParent = NULL;
	m_JSDocument = NULL;

	srcIndex = 0;

  m_cx = NULL ; //JSContext initialization

	m_Request = (CWLHttpRequest**)malloc(MAX_THREADS * sizeof(CWLHttpRequest*));
	m_RequestThread = (RVThread**)malloc(MAX_THREADS * sizeof(RVThread*));

	for (int i=0;(i<MAX_THREADS);i++) {
		m_Request[i] = NULL;
		m_RequestThread[i] = NULL;
	}

	m_Workspace = new CWLWorkspace;
	m_Cookie = new CWLCookies (m_Workspace);

	m_Connections = new CWLConnectionsPool;

#ifdef _DEBUG
	m_RoundNum = 1;
#endif

	m_LastErrorLock = new RVCriticalSection;
	m_SharedDataLock = new RVCriticalSection;
  m_bWasHttpCommand = false ;
  m_lExpectedDomId  = 0 ;
  m_bErrorInTransTree = false;
  m_RequestOnConnectionCounter = 0;

  return true; 
}

bool CWLCommandManager::BeforeInitClient() { 
	WlTraceMsg("HTTP Protocol : Called the protocol callback BeforeInitClient");	
	return true; 
}

bool CWLCommandManager::AfterInitClient() { 
	WlTraceMsg("HTTP Protocol : Called the protocol callback AfterInitClient");	
  AtEndOfRound(); 
  return true;
}

bool CWLCommandManager::BeforeRound() {
	WlTraceMsg("HTTP Protocol : Called the protocol callback BeforeRound");	
	return true;
}
	
bool CWLCommandManager::AfterRound() {
	WlTraceMsg("HTTP Protocol : Called the protocol callback AfterRound");	
  AtEndOfRound(); // Chaim - PIDE
  return true;
}

bool CWLCommandManager::BeforeTerminateClient() {
	WlTraceMsg("HTTP Protocol : Called the protocol callback BeforeTerminateClient");
  return true;
}

bool CWLCommandManager::AfterTerminateClient()
{
  if (m_UrlQueue)
    m_UrlQueue->terminateAllThreads(); 

  return true;

}

bool CWLCommandManager::OnThreadTermination() {
	WlTraceMsg("HTTP Protocol : Called the protocol callback OnThreadTermination");
	return true;
}

// This method will be used to
// clean up after a thread has been paused and re-started by the engine.
// Currently we have a known issue this is meant to solve that after a restart m_Page is not cleared
// and when we insert new freames we can't find the correct target.
bool CWLCommandManager::OnThreadRestart() {
	WlTraceMsg("HTTP Protocol : Called the protocol callback OnThreadRestart");

  // In mix, the behavior is different and this will be called for some threads that don't need a restart.
  if (!m_Workspace) return true;

  if (m_Page)
		delete m_Page;

	// create the requests queue
	if (m_UrlQueue)
		delete m_UrlQueue;

	int New_MaxThreads = max(m_Workspace->m_MaxThreads, 1);	// at least one thread should exist

  m_UrlQueue = new CWLQueue(New_MaxThreads, this);
	m_Page = new CWLPage(m_UrlQueue);

  return true;
}

bool CWLCommandManager::AfterSleep(long int) 
{ 
  return true; 
}

bool CWLCommandManager::SleepDisabled(long int recordedSleepTime)
{ 
  return true; 
}

bool CWLCommandManager::BeginTransaction(const char* transname, double time, const char* extraDataType, void* extraData, unsigned long len)
{
  return true;
}

CWLCommandManager* WLGetCommandManagerPtr(JSContext *cx)
{
  //construct the c++ object
//  iLEJSEmbedContext* pRVJS = iLEJSEmbedContext::fsGet(cx);
  iLEProtocolJSEmbedContext& a_PEC = piHLProtocolProcess->GetProtocolJSEmbedContext( cx );
  CWLCommandManager* pCM=NULL;
  if( pCM = dynamic_cast<CWLCommandManager*>(&a_PEC) ) {
    // nothing
  }
  else {
    WlAssert(0);
  }
	return pCM;
}
