/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <math.h>
#include <iostream.h>
#include <thr.h>
#include "wlqueue.h"
#include "httpdefines.h"
#include "connpool.h"
//#include "httpssl.h"
#include "commandmanager.h"
#include "workspace.h"
//#include "sslciphers.h"
#include "tvenv.h"
#include "httprequest.h"

#include "WLInsecureConnection.h"
#include "WLSecureSslConnection.h"
#include "WLSslCiphersCollection.h"

#define __WL_TRACE__
#include "tracedll.h"

#define WL_BITS_PER_BYTE 8

#define addCounter     if (!m_Cm->isAbort()) m_Cm->addCounter
#define addAccumulator if (!m_Cm->isAbort()) m_Cm->addAccumulator
#define addMeasurement if (!m_Cm->isAbort()) m_Cm->addMeasurement
#define addTimer       if (!m_Cm->isAbort()) m_Cm->addTimer

#define BITS_PER_SECOND		   "Connection Speed (Bits Per Second)"
#define BITS_PER_SECOND_IN   "Bits Per Second (in)"
#define BITS_PER_SECOND_OUT  "Bits Per Second (out)"
#define BITS_PER_SECOND_LAST "Bits Per Second (last)"
#define WAIT_TIME            "Time To First Byte"
#define DNS_TIME             "DNS Lookup Time"

#define min(a,b) (((a) < (b)) ? (a) : (b))
#define max(a,b) (((a) > (b)) ? (a) : (b))


//#define socket_send(sock, buffer, length, flags)	send(sock, buffer, length, flags)

void UnescapeString (const char* buffer, int &length, bool lowLevelUnescape, const char *userEscapeChar, RVCString &retbuf){
    retbuf = buffer;
	const char *escapeChar = (strlen(userEscapeChar) ? userEscapeChar : "%");
    int originalLength = retbuf.length();
    int CharsRemoved = 0;
	int bufferIndex = 0;
    while ((bufferIndex = retbuf.index(escapeChar, bufferIndex)) !=  RV_NPOS){
	  // If escaping the escapeChar
	    if (retbuf(bufferIndex + 1) == escapeChar[0]){
	        retbuf.remove(bufferIndex + 1, 1);
		CharsRemoved++;
		bufferIndex++;
	  }

	  // Else hexa (2 digits)
	  else {
	      const char *hexaStr = retbuf(bufferIndex + 1, bufferIndex + 3).data();
		int	res;
		char hexaBuf[1];
		sscanf(hexaStr,"%2x", (int)&res);
		hexaBuf[0] = res;
        retbuf.replace(bufferIndex, 3, hexaBuf, 1);
		CharsRemoved += 2;
		bufferIndex++;
	  }
    }

    // Modify content-length
    int Start = retbuf.index("Content-Length: ");

    if (CharsRemoved > 0 && Start != RV_NPOS){
      Start += 16;
      int End = retbuf.index("\r\n", Start);
      const char* lengthStr = retbuf(Start, End-Start+1).data();
      int newLength = atoi(lengthStr) - CharsRemoved;
      char newLengthStr[20];
      itoa(newLength, newLengthStr, 10);
      retbuf.replace(Start, End-Start+1, newLengthStr);
    }
  }


int socket_send(int sock, const char* buffer, int &length, int flags, bool lowLevelUnescape, const char *userEscapeChar){

  // Comverse - replace %0 with null
  RVCString tempbuf;

  if (lowLevelUnescape){
    UnescapeString(buffer, length, lowLevelUnescape, userEscapeChar, tempbuf);
  length = tempbuf.length();
    return send(sock, tempbuf.data(), length, flags);

  }
  else {

    return send(sock, buffer, length, flags);

  }


}


#define socket_recv(sock, buffer, length, flags)	recv(sock, buffer, length, flags)

#ifdef __TV_WINDOWS__
#define GetLastError WSAGetLastError
#else
#define GetLastError() errno
#endif



// struct that contains all the local addresses
//static HTTPLocalAddr localaddr;
// instead of the above class, static class which contain all addresses, and support services.
static GetHostIP g_LocalAddr;


// A basic element for holding a host in a collection of
// alternative host to ip mechanizem for working with
// SSL through proxy - Shahar 17-Dec-2000
class CWLAlternativeHost : public RVCollectable  {
private:
  RVCString  m_host;
public:
  CWLAlternativeHost(RVCString& host) : m_host(host) {}
  virtual ~CWLAlternativeHost() {}

  RVCString& getHostName() { return (m_host); }
  virtual int compareTo(const RVCollectable* element) const;
};

int CWLAlternativeHost::compareTo(const RVCollectable* element) const
{
	CWLAlternativeHost* intElement = (CWLAlternativeHost*)element;

  if (m_host == intElement->getHostName()) {
	  return (0);
	}
	else {
	  return ((m_host < intElement->getHostName()) ? 1 : -1);
 	}
};






// ==================================================================
CWLConnectionsPool::CWLConnectionsPool()
{
	m_Cm = NULL;
	m_Ws = NULL;
	m_MaxConnections = 0;
	m_SlowConnection = NULL;

#ifdef __OPEN_SSL__
	m_SslCipherSuites = new CWLSslCiphersCollection (this);
#else
	m_SslCipherSuites = NULL;
#endif //__OPEN_SSL__

}

CWLConnectionsPool::~CWLConnectionsPool()
{
	m_Pool.clearAndDestroy();
	m_SslContextPool.clearAndDestroy();
	m_SslAlternativeIP.clearAndDestroy();

#ifdef __OPEN_SSL__
	if (m_SslCipherSuites){
    delete m_SslCipherSuites ;
		m_SslCipherSuites = NULL;
	}
#endif //__OPEN_SSL__
}


void CWLConnectionsPool::Init(CWLCommandManager* cm, unsigned max)
{
	m_Cm = cm;
	m_Ws = cm->GetWorkspace();
	m_MaxConnections = max;

#ifdef __OPEN_SSL__
	if (m_SslCipherSuites)
	{
		InitCiphers( m_Ws->m_SslCryptoStrength );
	}
#endif // __OPEN_SSL__

/*
#ifdef __SSL__
	//
  // Install the SSL Plus Ezcrypto modules.
  // (Native, Bsafe and Security Builder
	EZInstallModule(EZ_NATIVE_SHA1_MODULE);
	EZInstallModule(EZ_NATIVE_MD2_MODULE);
	EZInstallModule(EZ_NATIVE_MD5_MODULE);

#ifdef BSAFE
	EZInstallModule(EZ_BSAFE_RSA_MODULE);
	EZInstallModule(EZ_BSAFE_RC2_MODULE);
	EZInstallModule(EZ_BSAFE_RC4_MODULE);
#else
	EZInstallModule(EZ_SECURITY_BUILDER_RSA_MODULE);
	EZInstallModule(EZ_SECURITY_BUILDER_ARC4_MODULE);
	EZInstallModule(EZ_SECURITY_BUILDER_MD5_MODULE);
	EZInstallModule(EZ_SECURITY_BUILDER_SHA1_MODULE);
#endif // BSAFE

	EZInstallModule(EZ_SECURITY_BUILDER_DES_MODULE);
	EZInstallModule(EZ_SECURITY_BUILDER_3DES_MODULE);
	EZInstallModule(EZ_SECURITY_BUILDER_ECC_MODULE);
	EZInstallModule(EZ_SECURITY_BUILDER_ECDH_MODULE);
	EZInstallModule(EZ_SECURITY_BUILDER_PRNG_MODULE);
	EZInstallModule(EZ_SECURITY_BUILDER_DH_MODULE);
	EZInstallModule(EZ_SECURITY_BUILDER_DSA_MODULE);




#endif
*/
}

void CWLConnectionsPool::InitCiphers(unsigned strength)
{
#ifdef __OPEN_SSL__
	if (m_SslCipherSuites)
		m_SslCipherSuites->Init(strength, false);
#endif
}

bool CWLConnectionsPool::UseSlowConnection(unsigned rate, unsigned threads)
{
	if (rate == 0)
		return true;

	if (!m_SlowConnection) {
		m_SlowConnection = new CWLSlowConnection;
		if (!m_SlowConnection)
			return false;
		m_SlowConnection->Init(this, m_Cm, rate, threads > 1 ? true : false);
	}
	else if (rate != m_SlowConnection->Speed()) {
		ReportError(HTTP_ERROR_FILE, 161, m_SlowConnection->Speed());
	}

	return true;
}


void CWLConnectionsPool::IncUsers()
{
	if (m_SlowConnection)
		m_SlowConnection->IncUsers();
}

void CWLConnectionsPool::DecUsers()
{
	if (m_SlowConnection)
		m_SlowConnection->DecUsers();
}

void CWLConnectionsPool::AddReport()
{
	if (m_SlowConnection)
		m_SlowConnection->AddReport();
}



int CWLConnectionsPool::Send(int sock, const char* buffer, int &len, int flags, int& lastRes, bool OverrideDecode)
{
	if (m_SlowConnection) {
		int res = m_SlowConnection->Send(sock, buffer, len, flags, lastRes);
		return res;
	}

  lastRes = socket_send(sock, buffer, len, flags, (OverrideDecode ? false : m_Ws->m_LowLevelUnescape), m_Ws->m_EscapeChar);
	return lastRes;
}

int CWLConnectionsPool::Recv(int sock, char* buffer, int len, int flags, int& lastRes)
{
	if (m_SlowConnection) {
		int res = m_SlowConnection->Recv(sock, buffer, len, flags, lastRes);
		return res;
	}

	lastRes = socket_recv(sock, buffer, len, flags);

	int start = (m_Ws->m_DecodeBinaryStart >= DEFAULT_DECODE_BINARY_START && m_Ws->m_DecodeBinaryStart <= len  ? 0: m_Ws->m_DecodeBinaryStart);
	int end = (m_Ws->m_DecodeBinaryEnd == DEFAULT_DECODE_BINARY_END || m_Ws->m_DecodeBinaryEnd > len ? len: m_Ws->m_DecodeBinaryEnd);

	// check if the m_Ws->m_DecodeBinaryNullAs != NULL
	if( NULL != m_Ws->m_DecodeBinaryNullAs && strlen(m_Ws->m_DecodeBinaryNullAs) == 1 ){
      for (int i = start; i < end; i++){
        if (!buffer[i]){
          buffer[i] = m_Ws->m_DecodeBinaryNullAs[0];
		}
	  }
	}

	return lastRes;
}


void CWLConnectionsPool::AtEndOfRound()
{
	DeleteAllConnections();
	ClearSslCache();
	m_SslAlternativeIP.clearAndDestroy();
	if (m_SlowConnection)
		m_SlowConnection->AtEndOfRound();
}


CWLGenericConnection* CWLConnectionsPool::createConnection( int protocol, sockaddr_in& addr, const char* host, int port, int threadNum, CWLHttpRequest* pHttpReq )
{
//addCounter("Created connections");
	CWLGenericConnection* conn = NULL;

	switch( protocol ) {
	case SSL_PROTOCOL:
		conn = new CWLSecureSslConnection( pHttpReq );
		break;
	default:
		conn = new CWLInsecureConnection( pHttpReq );
	}

	if (conn) {
		if (!conn->Init(m_Cm, this, addr, host, port)) {
			delete conn;
			return NULL;
		}

		m_Pool.insert(conn);
		conn->m_Index = m_Pool.entries();
		WlTraceMsg( "Created connection %d, host[%s], port[%u], ip[%s]", conn->m_Index, host, port, conn->ip());
	}
	return conn;
}


CWLGenericConnection* CWLConnectionsPool::AcquireConnection( int protocol, const char* host, int port, int threadNum, CWLHttpRequest* pHttpReq )
{
	RVCriticalSection::LockGuard lock(m_ConnectionLock);

	CWLGenericConnection* conn = NULL;
	CWLGenericConnection* connTmp = NULL;

	RVCString aHost(host);
	sockaddr_in addr;
	if (!GetSockAddr(addr, aHost, port, true, true))
		return NULL;

	size_t i = 0;
	for (i = 0; i < m_Pool.entries(); i++) {
		connTmp = (CWLGenericConnection*)m_Pool[i];
		if (!connTmp)
			continue;
		if (!connTmp->IsMatching(addr, host, port))
			continue;

		if( protocol != connTmp->m_Protocol )
			continue;

		if (connTmp->IsAvailable()) {
			conn = connTmp;
			if (conn->IsOpen()) {	// if connection is also open, return it
WlTraceMsg( "Using connection %d", connTmp->m_Index);
				conn->Lock();
				return conn;
			}
		}
	}

	if (conn) {
    WlTraceMsg( "Using connection %d, connection is closed", conn->m_Index);
		conn->Lock();
		return conn;
	}

	// if there is still place to add connections
	int e = m_Pool.entries();
	if (m_Pool.entries() < m_MaxConnections) {
		conn = createConnection( protocol, addr, host, port, threadNum, pHttpReq );
		if (conn) {
      WlTraceMsg( "Using connection %d", conn->m_Index);
			conn->Lock();
			return conn;
		}
		return NULL;
	}

	// close and destroy one of the unused connections and create a new one instead
	for (i = 0; i < m_Pool.entries(); i++) {
		conn = (CWLGenericConnection*)m_Pool[i];
		if (conn->IsAvailable()) {
			// might want to check if the connection is closed too

      WlTraceMsg( "Deleting connection %x, %d, sock: %u", conn, conn->m_Index, conn->sock());
			m_Pool.removeAndDestroy(conn);
			conn = createConnection( protocol, addr, host, port, threadNum, pHttpReq );
			if (conn) {
        WlTraceMsg( "Using connection %d", conn->m_Index);
				conn->Lock();
				return conn;
			}
			return NULL;
		}
	}

	return NULL;
}

void CWLConnectionsPool::ReleaseConnection(CWLGenericConnection* conn, int threadNum)
{
WlTraceMsg( "Unlocking connection %d", conn->m_Index);
	if (conn)
		conn->Unlock();
}


void CWLConnectionsPool::CloseAllConnections()
{
	CWLGenericConnection* conn;
	for (size_t i = 0; i < m_Pool.entries(); i++) {
		conn = (CWLGenericConnection*)m_Pool[i];
		if (conn)
			conn->CloseConnection(true);
	}
}


bool CWLConnectionsPool::GetSockAddr(sockaddr_in& sockAddr, RVCString& host, int port, bool resolveOnly, bool reportError)
{
	int err = 0;

  HTTPAddr* addr = NULL;
	int retrials = 3;
  bool actualLookup;

  double StartLookupTime = TVPSysTime();

	do {
		addr = m_DnsAddrTable.getItem(host, port, m_Ws->m_CacheDns, &err, &actualLookup);
		retrials--;
		if (!addr) {
			WlTraceMsg("Host %s not found. Trial %d.", host.data(), 3-retrials);
		}
	} while (!addr && retrials > 0);

  if (actualLookup) {
    addTimer(DNS_TIME, TVPSysTime() - StartLookupTime);
  }

  if (addr == NULL)  {
		SetErrorCode(err);
		WlTraceMsg("Host %s not found after all the retries.", host.data());
		if (reportError) {
		ReportError(HTTP_ERROR_FILE, 22, check_null(host.data()));
		}
		return (NULL);
  }

	bool res = addr->getSockAddrIn(sockAddr, resolveOnly);
	// if not using DNS cache, then this object must be deleted here
	if (!m_Ws->m_CacheDns)
		delete addr;

  return res;
}


//instantiation of GetHostIP static member
long CWLConnectionsPool::m_IPIndex = 0;
GetHostIP CWLConnectionsPool::m_gLocalAddr;
RVMutexLock	CWLConnectionsPool::m_IPIndexLock;

long CWLConnectionsPool::GetMultiIPAddressIndex(){
	RVMutexLock::LockGuard guard(m_IPIndexLock);
	return CWLConnectionsPool::m_IPIndex;
}

long CWLConnectionsPool::GetNextMultiIPAddressIndex(){
	RVMutexLock::LockGuard guard(m_IPIndexLock);
	if (CWLConnectionsPool::m_IPIndex == CWLConnectionsPool::m_gLocalAddr.GetNum() - 1){
		CWLConnectionsPool::m_IPIndex = 0;
	}
	else {
		CWLConnectionsPool::m_IPIndex++;
	}
	return CWLConnectionsPool::m_IPIndex;

}


int CWLConnectionsPool::GetLocalHosts()
{
  int nNumOfIPs = m_gLocalAddr.GetIPAddress() ;
WlTraceMsg("CWLConnectionsPool::GetLocalHosts() num of IPs found = %d", nNumOfIPs);

  // deleting all IPs excluded in TVEnv.
  TVEnvContainerClass tvpc;
  const char *pExcludedIPs = tvpc.TVEGetEnv("EXCLUDED_IPS");
  RVCString strAddr, strExcludedIPs;
  if (pExcludedIPs)
    strExcludedIPs = pExcludedIPs ;
  size_t eqlPtr, prevPtr=0 ;

  while(prevPtr < strExcludedIPs.length())
  {
		// Shahar - Fix 5.0.227 - Fix a problem that prevented from
		// specifing more then 2 IP's
    eqlPtr = strExcludedIPs.index("|", prevPtr);
    if( eqlPtr == RV_NPOS )
    {
      eqlPtr = strExcludedIPs.length();
    }

    strAddr = strExcludedIPs(prevPtr, eqlPtr - prevPtr) ;
    m_gLocalAddr.DeleteAddr( strAddr.data() );
    prevPtr = eqlPtr + 1 ;
  }

//  assert(m_gLocalAddr.GetNum()); //no IP was found or all IPs were deleted

  if ( m_gLocalAddr.GetNum() > 0 )
    return RET_OK ;
  else
  {
    return RET_ERROR ;
  }
}


void CWLConnectionsPool::ClearDnsCache()
{
	DeleteAllConnections();
	m_DnsAddrTable.clearCache();
}


int CWLConnectionsPool::ReportError(const char* frmName, int frmId, ...)
{
  WLLogMessage::Severity sev = WLERRGetErrorSeverity(frmName, frmId);

  char    buf[10240];
  va_list ap;

  va_start(ap, frmId);
  vsprintf(buf, WLERRGetErrorFormat(frmName,frmId), ap);
  va_end(ap);

	// add the Error code string
	if (!m_ErrorCode.isNull()) {
		strcat(buf, m_ErrorCode.data());
		m_ErrorCode.remove(0);
	}

	if (sev > WLLogMessage::Note)
  {
    CWLPage* pPage =  m_Cm->GetPage();
    CWLDocument* pFrame = NULL ;
    if (pPage)
      pFrame = pPage->GetPageTransaction();

    if (!pPage || !pFrame || !m_Ws->m_AutoNamedTrans)
    {
      m_Cm->reportMultiThread(sev, (const char*)buf);
    }
    else
    {
      CWLTransactionData* pTrans = pFrame->GetTransactionObj()->GetTransaction();
      pTrans->SetSeverity(sev);
      pTrans->SetFailureReason((const char*)buf);
      pTrans->AddEvent((const char*)buf, "",sev);
      pFrame->SetSeverity(sev);
    }
  }

  // update last error
	m_Cm->SetLastError(sev, (const char*)buf);

//  return ((sev >= WLLogMessage::Error) ? RET_ERROR : RET_OK );
  return RET_OK;
}


void CWLConnectionsPool::SetErrorCode(int err)
{
	if (!m_ErrorCode.isNull())
		m_ErrorCode.remove(0);

	m_ErrorCode = " Error code: ";
	char code[16];
	sprintf(code, "%d", err);
	m_ErrorCode += code;
}





// ==================================================================
void toFile(char* frm, ...)
{
	char buf[10240];
	va_list ap;
	va_start(ap, frm);
	vsprintf(buf, frm, ap);
	va_end(ap);
	FILE* f = fopen("c:/temp/bps.txt", "a+");
	if (f) {
		fwrite(buf, 1, strlen(buf), f);
		fclose(f);
	}
}

CWLSlowConnection::CWLSlowConnection()
{
	m_Cp = NULL;
	m_Cm = NULL;
	m_MTMode = false;
	m_Rate = 0;
	m_Size = 20;
	m_Users = 0;

	m_Calc.totalBytesMark = 0;
	m_Calc.totalBytes = 0;
	m_Calc.startTime = 0;
	m_Calc.endTime = 0;
	m_Calc.totalTime = 0;

	m_Report.totalBytes = 0;
	m_Report.prevBytes = 0;
	m_Report.startTime = 0;
	m_Report.endTime = 0;
	m_Report.totalTime = 0;
	m_Report.prevTime = 0;

	for (m_VecIndex = 0; m_VecIndex < MAX_VECTOR_SIZE; m_VecIndex++) {
		m_VecReport[m_VecIndex].bytes = 0;
		m_VecReport[m_VecIndex].time = 0;
	}
	m_VecIndex = 0;
	m_TotalBytes = 0;
	m_TotalTime = 0;

	m_TimerSet = false;

#ifdef __TV_WINDOWS__
	DWORD dwVersion = GetVersion();
	if (!(dwVersion & 0x80000000) && LOBYTE(LOWORD(dwVersion)) >= 4) // Windows NT gt 4
		m_MinTime = 20;	// for NT
	else
		m_MinTime = 55;
#else
	m_MinTime = 20;
#endif __TV_WINDOWS__
}

CWLSlowConnection::~CWLSlowConnection()
{
}

void CWLSlowConnection::AtEndOfRound()
{
}

bool CWLSlowConnection::Init(CWLConnectionsPool* cp, CWLCommandManager* cm, unsigned rate, bool mtMode)
{
	m_MTMode = mtMode;
	m_Rate = rate;
	m_SendRate = m_Rate;
	m_RecvRate = m_Rate;
	m_SendBps = (double)m_SendRate / WL_BITS_PER_BYTE;
	m_RecvBps = (double)m_RecvRate / WL_BITS_PER_BYTE;
	// calculate a size that would take the minimum time resolution to send/recv (20ms for NT, 55ms for 95/98)
	double Bpms = m_Rate / WL_BITS_PER_BYTE / 1000;	// bytes per millisecond
	m_Size = max((int)(Bpms * m_MinTime), 20);	// minimum 20 bytes per recv/send (temporary)
	m_Cp = cp;
	m_Cm = cm;

	return true;
}


void CWLSlowConnection::IncUsers()
{
	if (m_MTMode)
		m_UsersLock.acquire();
	if (m_Users == 0) {
		m_TimerSet = false;
	}
	m_Users++;
	if (m_MTMode)
		m_UsersLock.release();
}

void CWLSlowConnection::DecUsers()
{
	if (m_MTMode)
		m_UsersLock.acquire();
	m_Users--;
	if (m_Users == 0) {
		setActiveTimer();
	}
	if (m_MTMode)
		m_UsersLock.release();
}


long calcSleepTime(double calcTime, double opTime, long minTime)
{
	long dt = (long)((calcTime - opTime) * 1000);
	return dt;
}


// the following 2 functions does not include the residu improvement
int CWLSlowConnection::Send(int sock, const char* buffer, int &length, int flags, int& lastRes, bool OverrideDecode)
{
	unsigned done = 0;
	int size;		// bytes to send
	unsigned wantedSize;	// bytes to read

  int privateLength = length;

  // Before we start writeing this should be initialized
  // because there are cases in which we abort before sending
  lastRes = 0;

  if (m_MTMode)
		m_TimeLock.acquire();
	if (!m_TimerSet) {
		m_TimerSet = true;
		resetActiveTimer();
	}
	if (m_MTMode)
		m_TimeLock.release();

	while (privateLength > 0) {
		if (m_Cm->isAbort())
			break;

		wantedSize = calcBytes();
		if (wantedSize == 0)
			continue;

		size = min(wantedSize, (unsigned)privateLength);

		// lock for connection use
		if (m_MTMode)
			m_SocketLock.acquire();

		int sent;
    sent = socket_send(sock, buffer + done, size, flags, (OverrideDecode ? false : m_Cp->m_Ws->m_LowLevelUnescape), m_Cp->m_Ws->m_EscapeChar);
		lastRes = sent;
		if (sent > 0)
			addBytes(sent);

		if (m_MTMode)
			m_SocketLock.release();

		if (sent == 0) {
			break;
		}
		else if (sent < 0) {
			if (done == 0)
				done = -1;
			break;
		}

		privateLength -= sent;
		done += sent;
	}

	return done;
}

int CWLSlowConnection::Recv(int sock, char* buffer, int length, int flags, int& lastRes)
{
	unsigned done = 0;
	unsigned size;		// bytes to read
	unsigned wantedSize;	// bytes to read

  // Before we start writeing this should be initialized
  // because there are cases in which we abort before sending
  lastRes = 0;

  if (m_MTMode)
		m_TimeLock.acquire();
	if (!m_TimerSet) {
		m_TimerSet = true;
		resetActiveTimer();
	}
	if (m_MTMode)
		m_TimeLock.release();

	while (length > 0) {
		if (m_Cm->isAbort())
			break;

		wantedSize = calcBytes();
		if (wantedSize == 0)
			continue;

		size = min(wantedSize, (unsigned)length);

		// lock for connection use
		if (m_MTMode)
			m_SocketLock.acquire();

		int recvd;
		recvd = socket_recv(sock, buffer + done, size, flags);
		lastRes = recvd;
		if (recvd > 0)
			addBytes(recvd);

		if (m_MTMode)
			m_SocketLock.release();

		if (recvd == 0) {
			break;
		}
		else if (recvd < 0) {
			if (done == 0)
				done = -1;
			break;
		}

		length -= recvd;
		done += recvd;
	}

	return done;
}


// calculate number of bytes to receive
unsigned CWLSlowConnection::calcBytes()
{
	if (m_Calc.totalBytesMark > m_Calc.totalBytes)
		return (unsigned)(m_Calc.totalBytesMark - m_Calc.totalBytes);

	TVPWaitTime(m_MinTime);

	double activeTime = TVPSysTime() - m_Calc.startTime + m_Calc.totalTime;	// active socket time
	m_Calc.totalBytesMark = floor(activeTime * m_RecvBps);	// number of bytes we should have sent/recvd

	if (m_Calc.totalBytes >= m_Calc.totalBytes) {
/*
		double   calcTime;	// time should been taken to send/recv
		unsigned delayTime;	// difference between currTime & calcTime in ms

		calcTime = totalBytes / m_RecvBps;
		delayTime = min((unsigned)((activeTime - calcTime) * 1000), m_MinTime);
		TVPWaitTime(delayTime);
*/
		return 0;
	}

	return (unsigned)(m_Calc.totalBytesMark - m_Calc.totalBytes);
}

// value of bytes will always bwe positive
double CWLSlowConnection::addBytes(int bytes)
{
	double currTime = TVPSysTime();

	// add to total bytes
	m_Calc.totalBytes += bytes;
	m_Report.totalBytes += bytes;

	// set end time to be the current time
	m_Calc.endTime = currTime;
	m_Report.endTime = currTime;

	return m_Calc.totalBytes;
}

double CWLSlowConnection::getBytes()
{
	return m_Calc.totalBytes;
}


void CWLSlowConnection::resetActiveTimer()
{
	double currTime = TVPSysTime();
	m_Calc.startTime = currTime;
	m_Report.startTime = currTime;
}

void CWLSlowConnection::setActiveTimer()
{
	m_Report.totalTime += m_Report.endTime - m_Report.startTime;
	m_Calc.totalTime += m_Calc.endTime - m_Calc.startTime;
}

void CWLSlowConnection::addReport(double currTime)
{
	// send a measurement
	double time = m_Report.totalTime - m_Report.prevTime;
	m_Report.prevTime = m_Report.totalTime;
	double bytes = m_Report.totalBytes - m_Report.prevBytes;
	m_Report.prevBytes = m_Report.totalBytes;

	m_TotalBytes -= m_VecReport[m_VecIndex].bytes;
	m_VecReport[m_VecIndex].bytes = bytes;
	m_TotalBytes += m_VecReport[m_VecIndex].bytes;

	m_TotalTime -= m_VecReport[m_VecIndex].time;
	m_VecReport[m_VecIndex].time = time;
	m_TotalTime += m_VecReport[m_VecIndex].time;

	m_VecIndex = (m_VecIndex + 1) % MAX_VECTOR_SIZE;

	time = m_TotalTime;
	bytes = m_TotalBytes;

	if (time > 0) {
		double bps = (double)bytes * WL_BITS_PER_BYTE / time;
		addMeasurement(BITS_PER_SECOND, bps);
	}
}

void CWLSlowConnection::AddReport()
{
	addReport(m_Report.endTime);
}

unsigned CWLSlowConnection::Speed()
{
	return m_Rate;
}

