/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "httpprotocol.h"
#include "httpinit.h"
#include "commandmanager.h"
#include "jshttpobj.h"
#include "jscookie.h"
#include "jscallback.h"
#include "jshtml.h"
#include "wlfuncs.h"
#include "wlxmlparserexport.h"

#ifdef __TV_WINDOWS__
#include "wlxmlfuncs.h"
#endif // __TV_WINDOWS__

#include "WLSecureSslConnection.h"

CHLProtocolProcess* piHLProtocolProcess = NULL;

#ifdef __TV_WINDOWS__
CHLXMLErrorReporter* piErrorReporter = NULL;
#endif // __TV_WINDOWS__

iLEProtocolProcess* NewProtocolProcess() {
	//create 'CHLProtocolProcess' object and sets the global static pointer :
	piHLProtocolProcess =  new  CHLProtocolProcess ();
	WlAssert(piHLProtocolProcess);
	return piHLProtocolProcess;
}


////////////////////////////////////////////////////////////////
//////////////// class CHLProtocolProcess //////////////////////
////////////////////////////////////////////////////////////////

CHLProtocolProcess::CHLProtocolProcess():
  m_CallbackEnabled(false)
{
}

CHLProtocolProcess::~CHLProtocolProcess()
{
	CWLSecureSslConnection::DestroyCryptoLockers();
}

// the implementation of class iLEProtocolProcess
iLEProtocolSession& CHLProtocolProcess::NewProtocolSession()
{
	CHLProtocolSession * aCHLProtocolSession = new CHLProtocolSession(*this);
	WlAssert(aCHLProtocolSession);
	return *aCHLProtocolSession;
}

iLEProtocolAgendaMachineThread& CHLProtocolProcess::NewProtocolAgendaMachineThread(WLProtocolThreadType rProtocolThreadType)
{
	CHLProtocolAgendaMachineThread * aCHLProtocolAgendaMachineThread = new CHLProtocolAgendaMachineThread(*this, rProtocolThreadType);
	WlAssert(aCHLProtocolAgendaMachineThread);
	return *aCHLProtocolAgendaMachineThread;
}

iLEProtocolJSRuntime& CHLProtocolProcess::NewProtocolJSRuntime()
{
	CHLProtocolJSRuntime * aCHLProtocolJSRuntime = new CHLProtocolJSRuntime(*this);
	WlAssert(aCHLProtocolJSRuntime);
	return *aCHLProtocolJSRuntime;
}

bool CHLProtocolProcess::ProcessInitialize()
{
	//@@Hryvach
	CWLSecureSslConnection::LibraryInit();
	//SSL_library_init();                      /* initialize library */
	//SSL_load_error_strings();                /* readable error messages */

  WLHttpInit(false);

#ifdef __TV_WINDOWS__
  piErrorReporter = new CHLXMLErrorReporter();
  SetErrorReporter(piErrorReporter);
#endif // __TV_WINDOWS__

  return true;
}

bool CHLProtocolProcess::ProcessInitializeJava(cRVJVM& rJvm, JNIEnv* rEnv)
{ 
	WlTraceMsg("HTTP Protocol : Called the protocol callback ProcessInitializeJava");

//Chaim this function is probably not needed. should be removed
//  if (!InitJavautilsJNI(rEnv))
//    return false;
//
//  if (!InitLoadEngineJNI(rEnv))
//    return false;
//    
//  if (!RegisterHttpNativeMethods(rEnv))
//    return false;
//
  return true;
}

bool CHLProtocolProcess::ProcessFinalizeJava(cRVJVM& rJvm, JNIEnv* rEnv)
{ 
	WlTraceMsg("HTTP Protocol : Called the protocol callback ProcessFinalizeJava");
  return true;
}

bool CHLProtocolProcess::ProcessTermination()
{ 
	WlTraceMsg("HTTP Protocol : Called the protocol callback ProcessTermination");
#ifdef __TV_WINDOWS__
  delete piErrorReporter;
#endif // __TV_WINDOWS__
  return true;
}


////////////////////////////////////////////////////////////////
//////////////// class CHLProtocolSession/////////////////////
////////////////////////////////////////////////////////////////


CHLProtocolSession::CHLProtocolSession(iLEProtocolProcess & father) 
	: iLEProtocolSession(father)
{
}

CHLProtocolSession::~CHLProtocolSession()
{
}

iLEProtocolAgendaInMix& CHLProtocolSession::NewProtocolAgendaInMix()
{
	CHLProtocolAgendaInMix * aCHLProtocolAgendaInMix = new CHLProtocolAgendaInMix(*this);
	WlAssert(aCHLProtocolAgendaInMix);
	return *aCHLProtocolAgendaInMix;
	
}

bool CHLProtocolSession::SessionInitialize() { 
	WlTraceMsg("HTTP Protocol : Called the protocol callback SessionInitialize");	
  WLHttpInit(false);
	return true;
}

bool CHLProtocolSession::SessionTermination() {
	WlTraceMsg("HTTP Protocol : Called the protocol callback SessionTermination");	
  WLHttpTerminate();
	return true;
}


////////////////////////////////////////////////////////////////
//////////////// class CHLProtocolJSRuntime ////////////////////
////////////////////////////////////////////////////////////////

CHLProtocolJSRuntime::CHLProtocolJSRuntime(iLEProtocolProcess & father) :
	iLEProtocolJSRuntime(father)
{
}
	
CHLProtocolJSRuntime::~CHLProtocolJSRuntime()
{
}

bool CHLProtocolJSRuntime::JSRuntimeInitialize(JSRuntime* rt) // init process, will happen only once
{
  // @@Migration - have to see if I want to move anything from JSInitializeForScript here
  m_JSRuntime = rt;
	WlTraceMsg("HTTP Protocol : Called the protocol callback JSRuntimeInitialize");	
  return true;

}

bool CHLProtocolJSRuntime::JSRuntimeAfterNewMainContext(JSContext* cx)
{
	WlTraceMsg("HTTP Protocol : Called the protocol callback JSRuntimeAfterNewMainContext");	
	return true;
}

bool CHLProtocolJSRuntime::JSRuntimeBeforeDestroyMainContext(JSContext* cx)
{
	WlTraceMsg("HTTP Protocol : Called the protocol callback JSRuntimeBeforeDestroyMainContext");	
	return true;
}


////////////////////////////////////////////////////////////////
//////////////// class CHLProtocolAgendaInMix //////////////////
////////////////////////////////////////////////////////////////

CHLProtocolAgendaInMix::CHLProtocolAgendaInMix(iLEProtocolSession & father) :
	iLEProtocolAgendaInMix(father)
{
}

CHLProtocolAgendaInMix::~CHLProtocolAgendaInMix()
{
}

iLEProtocolVCInMix& CHLProtocolAgendaInMix ::NewProtocolVCInMix()
{
  CWLCommandManager* aCommandManager =
      new CWLCommandManager(*this,
                            dynamic_cast<CHLProtocolProcess*>(&GetProtocolProcess())->GetCallbackEnabled());
	WlAssert(aCommandManager);
	return *aCommandManager;
}

bool CHLProtocolAgendaInMix ::BeforeInitAgenda() {
	WlTraceMsg("HTTP Protocol : Called the protocol callback BeforeInitAgenda");	
	return true;
}


bool CHLProtocolAgendaInMix ::AfterTerminateAgenda() {
	WlTraceMsg("HTTP Protocol : Called the protocol callback AfterTerminateAgenda");	
	return true;
}

bool CHLProtocolAgendaInMix::JSInitializeForScript(JSContext* cx, JSObject* pAgendaGlobal, JSObject* pGeneralGlobal) {
	WlTraceMsg("HTTP Protocol : Called the protocol callback JSInitializeForScript");
  bool status = true;
  // Moved from WLInitHttpGeneratorScript

#ifdef __TV_WINDOWS__
	//xml classes
	status &= WLAddXmlDocumentClass(cx, pGeneralGlobal);
	status &= WLAddXmlNodeClass(cx, pGeneralGlobal);
	status &= WLAddXmlNodeListClass(cx, pGeneralGlobal);
	status &= WLAddXmlNamedNodeMapClass(cx, pGeneralGlobal);
	status &= WLAddXmlImplementationClass(cx, pGeneralGlobal);
	status &= WLAddXmlParseErrorClass(cx, pGeneralGlobal);

#endif __TV_WINDOWS__
  
	// add http-engine classes
	status &= WLAddHttpClass(cx, pAgendaGlobal);
	status &= WLAddJSHeaderClass(cx, pAgendaGlobal);
	status &= WLAddJSFormClass(cx, pAgendaGlobal);
	status &= WLAddJSDataClass(cx, pAgendaGlobal);
	status &= WLAddJSDatafileClass(cx, pAgendaGlobal);
	status &= WLAddCookieClass(cx, pAgendaGlobal);
	status &= WLAddCallbackClass(cx, pAgendaGlobal);
	status &= WLAddHtmlClass(cx, pAgendaGlobal);
 	status &= AddDomRepositoryClass(cx, pAgendaGlobal);
	status &= AddFunctionalTestReporterClass(cx, pAgendaGlobal);
	status &= AddFunctionalTestObjDetailsClass(cx, pAgendaGlobal);
	status &= AddTableCompareReporterClass(cx,pAgendaGlobal);
  status &= AddIteratorClass(cx,pAgendaGlobal);

	// add dom classes
	status &= WLAddActionClass(cx, pAgendaGlobal);
	status &= WLAddArrayClass(cx, pAgendaGlobal);
	status &= WLAddElementClass(cx, pAgendaGlobal);
	status &= WLAddFormClass(cx, pAgendaGlobal);
	status &= WLAddFrameClass(cx, pAgendaGlobal);
	status &= WLAddFramesClass(cx, pAgendaGlobal);
	status &= WLAddImgClass(cx, pAgendaGlobal);
	status &= WLAddEmbedClass(cx, pAgendaGlobal);
	status &= WLAddKvpClass(cx, pAgendaGlobal);
	status &= WLAddLinkClass(cx, pAgendaGlobal);
	status &= WLAddLocationClass(cx, pAgendaGlobal);
	status &= WLAddBrowserLocationClass(cx, pAgendaGlobal);
	status &= WLAddOptionClass(cx, pAgendaGlobal);
	status &= WLAddScriptClass(cx, pAgendaGlobal);
	status &= WLAddAppletClass(cx, pAgendaGlobal);
	status &= WLAddCSSClass(cx, pAgendaGlobal);
	status &= WLAddWindowClass(cx, pAgendaGlobal);
	status &= WLAddTableClass(cx, pAgendaGlobal);
	status &= WLAddRowClass(cx, pAgendaGlobal);
	status &= WLAddCellClass(cx, pAgendaGlobal);
	status &= WLAddMetaClass(cx, pAgendaGlobal);

  // add xmlparser dom classes
  status &= WLAddXMLClasses(cx, pGeneralGlobal);


  return status;
}

bool CHLProtocolAgendaInMix::JSTerminateForScript(JSContext* cx, JSObject* pAgendaGlobal, JSObject* pGeneralGlobal) {
	WlTraceMsg("HTTP Protocol : Called the protocol callback JSTerminateForScript");
  bool status = true;
  return status;
}


////////////////////////////////////////////////////////////////
//////////////// class CHLProtocolAgendaMachineThread //////////
////////////////////////////////////////////////////////////////

CHLProtocolAgendaMachineThread::CHLProtocolAgendaMachineThread(iLEProtocolProcess& father, WLProtocolThreadType ThreadType) 
	: iLEProtocolAgendaMachineThread(father, ThreadType){
}

CHLProtocolAgendaMachineThread::~CHLProtocolAgendaMachineThread() {
}

bool CHLProtocolAgendaMachineThread::MachineThreadInitialize() { 
	WlTraceMsg("HTTP Protocol : Called the protocol callback MachineThreadInitialize");
	return true;
}

bool CHLProtocolAgendaMachineThread::MachineThreadTermination() {
	WlTraceMsg("HTTP Protocol : Called the protocol callback MachineThreadTermination");
	return true;
}



////////////////////////////////////////////////////////////////
//////////////// class CHLXMLErrorReporter//////////////////////
////////////////////////////////////////////////////////////////
#ifdef __TV_WINDOWS__
CHLXMLErrorReporter::CHLXMLErrorReporter(){
}

CHLXMLErrorReporter::~CHLXMLErrorReporter(){
}

bool CHLXMLErrorReporter::vReport(JSContext* cx, WLLogMessage::Severity sev , const char* msg){
  CWLCommandManager* aCM = WLGetCommandManagerPtr(cx);
  return aCM->report(sev, msg);
} 
#endif // __TV_WINDOWS__
