/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wltable.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "TraceDll.h"

#include "propertieshandler.h"

static JSBool
cell_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside cell_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	char* key = NULL;

  CWLFrame* frame = prv->GetParent();

	CWLTableCell* cell = (CWLTableCell*)prv->GetObj();
	if (!cell)
		return JS_TRUE;

	if (JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!prop)
			return JS_TRUE;

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_TAGNAME : {	// return the tagName property
			int type = cell->getCellType();
			if (type == TD)
				return WLSetStringValue(cx, obj, prop, TD_NAME, vp);
			return WLSetStringValue(cx, obj, prop, TH_NAME, vp);
		}

    case PropertiesHandler::EN_CELL_INDEX : {	// return the cellIndex property
			int index = cell->getCellIndex();
			return WLSetIntValue(cx, obj, prop, index, vp);
		}

    case PropertiesHandler::EN_INNERHTML : {	// return the innerHTML property
			const char* inner = cell->getCellInnerHtml();
			return WLSetStringValue(cx, obj, prop, inner, vp);
		}

    case PropertiesHandler::EN_INNERTEXT : {	// return the innerText property
			const char* inner = cell->getCellInnerText();
			return WLSetStringValue(cx, obj, prop, inner, vp);
		}

    case PropertiesHandler::EN_ID : {	// return the id property
			const char* id = cell->getCellId();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    default:;
    }
	}
	return JS_TRUE;
}


static JSBool
cell_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside cell_setProperty");
	return JS_TRUE;
}

JSBool table_CellEnumerate(JSContext *cx, JSObject *obj ){

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_INNER_TEXT, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_INNERHTML, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_CELL_INDEX, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_TAGNAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ID, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}

static void
cell_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside cell_finalize(%x, %x)", cx, obj);
	WLFinalize(cx, obj);
}



JSClass wl_CellClass = {
	WLJSNAME_CLASS_CELL,
	JSCLASS_HAS_PRIVATE,	//ptr to c++ structure that info regarding the object it represents
	JS_PropertyStub,			//callback to C++
	JS_PropertyStub,			//--"--
	cell_getProperty,
	cell_setProperty,
	table_CellEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	cell_finalize
};


/*
static JSBool
cell_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("cell_toString");
	// set rval to tagName
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLTableCell* cell = (CWLTableCell*)prv->GetObj();
	if (!cell)
		return JS_TRUE;

	JSString* str = JS_NewStringCopyZ(cell->getCellType() == TD ? TD_NAME : TH_NAME);
	*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}


static JSBool
cell_valueOf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("cell_valueOf");
	// set rval to cellIndex
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLTableCell* cell = (CWLTableCell*)prv->GetObj();
	if (!cell)
		return JS_TRUE;

	*rval = INT_TO_JSVAL(cell->getCellIndex());

	return JS_TRUE;
}


static JSFunctionSpec cell_methods[] = {
	{wl_toString_str,		cell_toString,	0},
	{wl_valueOf_str,		cell_valueOf,	0},
	{0}
};
*/


static JSBool
InitCellObject(JSContext *cx, JSObject *obj, jsint argc, jsval *argv)
{
WlTraceMsg("initializing Cell object properties");

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_LENGTH))
		return JS_FALSE;

	return JS_TRUE;
}


JSObject* WLNewCellObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewCellObject(%x, %x)", cx, obj);
	// create an instance of type WLCell
	JSObject* proto = NULL;
	JSObject* cell = WLNewObject(cx, &wl_CellClass, proto, obj);
	if (!cell)
		return NULL;

	return cell;
}


bool DllExport WLAddCellClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddCellClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_CellClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
