/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wlframecontainer.h"
#include "wltable.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"


static JSBool
cipher_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside cipher_getProperty");
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	char* key = NULL;
	CWLCipher* cipher = (CWLCipher*)prv->GetObj();
	if (!cipher)
		return JS_TRUE;

	if (JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!prop)
			return JS_TRUE;

		if (!strcmp(prop, WLJSNAME_PROP_SUITE_NAME)) {	// return the name property
			const char* name = cipher->name();
			return WLSetStringValue(cx, obj, prop, name, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SUITE_ID)) {	// return the id property
			int name = cipher->id();
			return WLSetIntValue(cx, obj, prop, id, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SUITE_EXPORTABLE)) {	// return the exportable property
			bool name = cipher->exportable();
			return WLSetBoolValue(cx, obj, prop, id, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SUITE_METHOD)) {	// return the method property
			const char* method = cipher->method();
			return WLSetStringValue(cx, obj, prop, method, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_CIPHER_NAME)) {	// return the cipherName property
			const char* name = cipher->cipherName();
			return WLSetStringValue(cx, obj, prop, name, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_CIPHER_MODE)) {	// return the cipherMode property
			const char* mode = cipher->mode();
			return WLSetStringValue(cx, obj, prop, mode, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SUITE_BITS)) {	// return the keyBits property
			int bits = cipher->keyBits();
			return WLSetIntValue(cx, obj, prop, bits, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HASH_NAME)) {	// return the hashName property
			const char* name = cipher->hashName();
			return WLSetStringValue(cx, obj, prop, name, vp);
		}
	}
	return JS_TRUE;
}


static JSBool
cipher_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside cipher_setProperty");
	return JS_TRUE;
}

static void
cipher_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside cipher_finalize(%x, %x)", cx, obj);
	WLFinalize(cx, obj);
}



JSClass wl_CipherClass = {
	WLJSNAME_CLASS_CIPHER,
	JSCLASS_HAS_PRIVATE,	//ptr to c++ structure that info regarding the object it represents
	JS_PropertyStub,			//callback to C++
	JS_PropertyStub,			//--"--
	cipher_getProperty,
	cipher_setProperty,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	cipher_finalize
};


/*
static JSBool
cipher_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("cipher_toString");
	// set rval to name
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLCipher* cipher = (CWLCipher*)prv->GetObj();
	if (!cipher)
		return JS_TRUE;

	JSString* str = JS_NewStringCopyZ(cipher->name());
	*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}


static JSBool
cipher_valueOf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("cipher_valueOf");
	// set rval to id
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLCipher* cipher = (CWLCipher*)prv->GetObj();
	if (!cipher)
		return JS_TRUE;

	*rval = INT_TO_JSVAL(cipher->id());

	return JS_TRUE;
}


static JSFunctionSpec cipher_methods[] = {
	{wl_toString_str,		cipher_toString,	0},
	{wl_valueOf_str,		cipher_valueOf,		0},
	{0}
};
*/


static JSBool
InitCipherObject(JSContext *cx, JSObject *obj, jsint argc, jsval *argv)
{
WlTraceMsg("initializing Cipher object properties");

//	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_LENGTH))
//		return JS_FALSE;

	return JS_TRUE;
}


JSObject* WLNewCipherObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewCipherObject(%x, %x)", cx, obj);
	// create an instance of type WLCipher
	JSObject* proto = NULL;
	JSObject* cipher = WLNewObject(cx, &wl_CipherClass, proto, obj);
	if (!cipher)
		return NULL;

	return cipher;
}


bool DllExport WLAddCipherClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddCipherClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_CipherClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
