/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : jsCookie.cpp
 *
 *	Author : Zion Menahem
 *
 *	Description : 
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

/** include files **/
#include <stdlib.h>
#include <string.h>

#include "rvjsapi.h"

#include "loadengineglobals.h"
#include "httpdefines.h"
#include "jshttpnames.h"
#include "commandmanager.h"
#include "connpool.h"
#include "replyprops.h"


/** local definitions **/
#define reportError ptrHttp->GetConnPool()->ReportError

/** default settings **/

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** classes definitions **/

/** public functions **/

/** private functions **/



/*********************************************************************\

	Cookie Object

\*********************************************************************/

enum CookieId {
};

static JSPropertySpec cookie_props[] = {
	{0}
};


JSClass cookie_class = {
    WLJSNAME_COOKIE_CLASS,
    0,
    JS_PropertyStub,
		JS_PropertyStub,
		JS_PropertyStub,
		JS_PropertyStub,
    JS_EnumerateStub,
		JS_ResolveStub,
		JS_ConvertStub,
		JS_FinalizeStub
};

static JSBool
cookie_Set(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	int res = -1;

	// get the HTTP object of the current thread
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		if (ptrHttp->isAbort())
			return JS_FALSE;

		JSString* sname   = NULL;
		JSString* svalue  = NULL;
		JSString* sdomain = NULL;
		JSString* spath   = NULL;
		JSString* sexpire = NULL;

		if (argc > 0 && JSVAL_IS_STRING(argv[0]))
			sname   = JSVAL_TO_STRING(argv[0]);
		if (argc > 1 && JSVAL_IS_STRING(argv[1]))
			svalue  = JSVAL_TO_STRING(argv[1]);
		if (argc > 2 && JSVAL_IS_STRING(argv[2]))
			sdomain = JSVAL_TO_STRING(argv[2]);
		if (argc > 3 && JSVAL_IS_STRING(argv[3]))
			spath   = JSVAL_TO_STRING(argv[3]);
		if (argc > 4 && JSVAL_IS_STRING(argv[4]))
			sexpire = JSVAL_TO_STRING(argv[4]);

		char* name   = NULL;
		char* value  = NULL;
		char* domain = NULL;
		char* path   = NULL;
		char* expire = "";

		if (sname)
			name   = JS_GetStringBytes(sname);
		if (svalue)
			value  = JS_GetStringBytes(svalue);
		if (sdomain)
			domain = JS_GetStringBytes(sdomain);
		if (spath)
			path   = JS_GetStringBytes(spath);
		if (sexpire)
			expire = JS_GetStringBytes(sexpire);

		if (name &&	value && path && domain)
			res = ptrHttp->SetCookie(name, value, domain, path, expire);
		else
			reportError(HTTP_ERROR_FILE, 24, check_null(name), check_null(value), check_null(domain), check_null(path), check_null(expire));
	}

	*rval = INT_TO_JSVAL(res);

	return JS_TRUE;
}


static JSBool
cookie_Delete(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	int res = -1;

	// get the HTTP object of the current thread
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp) {
		if (ptrHttp->isAbort())
			return JS_FALSE;

		JSString* sname   = NULL;
		JSString* sdomain = NULL;
		JSString* spath   = NULL;

		if (argc > 0 && JSVAL_IS_STRING(argv[0]))
			sname   = JSVAL_TO_STRING(argv[0]);
		if (argc > 1 && JSVAL_IS_STRING(argv[1]))
			sdomain = JSVAL_TO_STRING(argv[1]);
		if (argc > 2 && JSVAL_IS_STRING(argv[2]))
			spath   = JSVAL_TO_STRING(argv[2]);

		char* name   = NULL;
		char* domain = NULL;
		char* path   = NULL;

		if (sname)
			name   = JS_GetStringBytes(sname);
		if (sdomain)
			domain = JS_GetStringBytes(sdomain);
		if (spath)
			path   = JS_GetStringBytes(spath);

		if (name && domain && path)
			res = ptrHttp->DeleteCookie(name, domain, path);
		else
			reportError(HTTP_ERROR_FILE, 26, check_null(name), check_null(domain), check_null(path));
	}

	*rval = INT_TO_JSVAL(res);

	return JS_TRUE;
}

static JSBool
cookie_ClearAll(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp)
		ptrHttp->ClearAllCookies();

	return JS_TRUE;
}


static JSFunctionSpec cookie_methods[] = {
	{WLJSNAME_COOKIE_SET,			cookie_Set,				5},
	{WLJSNAME_COOKIE_DELETE,	cookie_Delete,		3},
	{WLJSNAME_COOKIE_CLEAR,		cookie_ClearAll,	0},
	{0}
};


bool DllExport WLAddCookieClass(JSContext *cx, JSObject *obj)
{
	// define a class of type Cookie
	if (!JS_InitClass(cx, obj, NULL, &cookie_class, NULL, 1, NULL, cookie_methods, NULL, NULL))
		return false;

	return true;
}

bool DllExport WLAddCookieObject(JSContext *cx, JSObject *obj)
{
	// create an instance of type WLCookie
	JSObject* wlCookie = JS_DefineObject(cx, obj, WLJSNAME_COOKIE_OBJECT, &cookie_class, NULL, JSPROP_PERMANENT);
	if (!wlCookie)
		return false;

	return true;
}
