/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wlfuncs.h"
#include "wlarray.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "jsprivate.h"
#include "TraceDll.h"

#include "propertieshandler.h"

static JSBool
kvp_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside kvp_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLKvp* kvp = (CWLKvp*)prv->GetObj();
	if (!kvp)
		return JS_TRUE;

	if (kvp && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_KEY : {	// return the Key property
			const char* key = kvp->Key();
			return WLSetStringValue(cx, obj, prop, key, vp);
		}

    case PropertiesHandler::EN_VALUE : {	// return the Value property
			const char* value = kvp->Value();
			return WLSetStringValue(cx, obj, prop, value, vp);
		}

    default:;
    }
	}

	return JS_TRUE;
}

static JSBool
kvp_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside kvp_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLKvp* kvp = (CWLKvp*)prv->GetObj();
	if (kvp && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_KEY)) {	// set the Key property
			kvp->setKey(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_VALUE)) {	// return the Value property
			kvp->setValue(WLStringValue(*vp));
			return JS_TRUE;
		}
	}
*/

	return JS_TRUE;
}

JSBool kvp_ClassEnumerate(JSContext *cx, JSObject *obj ){

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_VALUE, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_KEY, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}

static void
kvp_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside kvp_finalize(%x, %x)", cx, obj);
	WLFinalize(cx, obj);
}


JSClass wl_KvpClass = {
	WLJSNAME_CLASS_KVP,
	JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,
	JS_PropertyStub,
	kvp_getProperty,
	kvp_setProperty,
  kvp_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	kvp_finalize
};


/*
static JSBool
kvp_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLKvp* kvp = (CWLKvp*)prv->GetObj();
	if (!kvp)
		return JS_TRUE;

	// set rval to key,value property
	const char* key = kvp->Key();
	const char* value = kvp->Value();
	char* kvString = (char*)JS_malloc(cx, strlen(key)+strlen(value)+4);
	if (kvString) {
		sprintf(kvString, "%s, %s", key, value);
		JSString* str = JS_NewStringCopyZ(cx, kvString);
		if (str)
			*rval = STRING_TO_JSVAL(str);
	}
	JS_free(cx, kvString);

	return JS_TRUE;
}


static JSFunctionSpec kvp_methods[] = {
	{wl_toString_str,	kvp_toString,	0},
	{0}
};
*/


/*
static JSBool
InitKvpObject(JSContext *cx, JSObject *obj, jsint length, jsval *vector)
{
WlTraceMsg("initializing KVP object properties");
	CWLKvp* kvp = new CWLKvp;
	JS_SetPrivate(cx, obj, (void*)kvp);

	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_DOM_SCOPE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_KEY))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_VALUE))
		return JS_FALSE;

	return JS_TRUE;
}


static JSBool
WLKvp(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("WLKvp constructor");
	JSBool ok;

	ok = InitKvpObject(cx, obj, 0, NULL);
	if (ok)
		*rval = OBJECT_TO_JSVAL(obj);

	return ok;
}


bool WLAddKvpObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddKvpObject(%x, %x)", cx, obj);
	// create an instance of type WLKvp
	JSObject* kvp = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_KVP, &wl_KvpClass, NULL, JSPROP_PERMANENT);
	if (!kvp)
		return false;

	// define it's properties & structures
	if (!InitKvpObject(cx, kvp, 0, NULL))
		return false;

	return true;
}
*/


JSObject* WLNewKvpObject(JSContext *cx, JSObject *obj)
{
	// create an instance of type WLKvp
	JSObject* proto = NULL;
	JSObject* kvp = WLNewObject(cx, &wl_KvpClass, proto, obj);
	if (!kvp)
		return NULL;

WlTraceMsg("WLNewKvpObject(%x, %x) ==> %x", cx, obj, kvp);
	return kvp;
}


bool DllExport WLAddKvpClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddKvpClass(%x, %x)", cx, obj);
	// define a class of type WLKvp
	//if (!JS_InitClass(cx, obj, NULL, &wl_KvpClass, WLKvp, 1, NULL, kvp_methods, NULL, NULL))
	if (!JS_InitClass(cx, obj, NULL, &wl_KvpClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
