/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wlframecontainer.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "TraceDll.h"

#include "propertieshandler.h"

static JSBool
meta_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside meta_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
  CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;
	
	CWLFrame* frame = (CWLFrame*)prv->GetParent();
	if (!frame)
		return JS_TRUE;

	CWLMeta* meta = (CWLMeta*)prv->GetObj();
	if (!meta)
		return JS_TRUE;

	if (meta && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_NAME : {	// return the Name property
			const char* name = meta->getName();
			return WLSetStringValue(cx, obj, prop, name, vp);
		}

    case PropertiesHandler::EN_CONTENT : {	// return the id property
			const char* content = meta->getContent();
			return WLSetStringValue(cx, obj, prop, content, vp);
		}

    case PropertiesHandler::EN_HTTPEQUIV : {	// return the target property
			const char* httpEquiv = meta->getHttpEquiv();
			return WLSetStringValue(cx, obj, prop, httpEquiv, vp);
		}

    case PropertiesHandler::EN_URL : {	// return the target property
			CWLUrl* url = meta->getUrl();
			return WLSetLinkValue(cx, obj, frame, url, vp);
		}

    default:;
    }
	}
	
	return JS_TRUE;
}

static JSBool
meta_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside meta_setProperty");

	return JS_TRUE;
}

static void
meta_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside meta_finalize");
	WLFinalize(cx, obj);
}


JSClass wl_MetaClass = {
	WLJSNAME_CLASS_META,
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	meta_getProperty,
	meta_setProperty,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	meta_finalize
};



static JSBool
meta_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLMeta* meta = (CWLMeta*)prv->GetObj();
	if (!meta)
		return JS_TRUE;

	// set rval to content property
	const char* content = meta->getContent();
	JSString* str = JS_NewStringCopyZ(cx, content);
	if (str)
		*rval = STRING_TO_JSVAL(str);
	return JS_TRUE;
}

static JSBool
meta_valueOf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLMeta* meta = (CWLMeta*)prv->GetObj();
	if (!meta)
		return JS_TRUE;

	// set rval to content property
	const char* content = meta->getContent();
	JSString* str = JS_NewStringCopyZ(cx, content);
	if (str)
		*rval = STRING_TO_JSVAL(str);
	return JS_TRUE;
}




static JSFunctionSpec meta_methods[] = {
	{wl_toString_str,	meta_toString,	0},
	{wl_valueOf_str,	meta_valueOf,	0},
	{0}
};



JSObject* WLNewMetaObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewMetaObject(%x, %x)", cx, obj);
	// create an instance of type WLMeta
	JSObject* proto = NULL;
	JSObject* meta = WLNewObject(cx, &wl_MetaClass, proto, obj);
	if (!meta)
		return NULL;

	return meta;
}


bool DllExport WLAddMetaClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddMetaClass(%x, %x)", cx, obj);
	// define a class of type WLMeta
	if (!JS_InitClass(cx, obj, NULL, &wl_MetaClass, NULL, 1, NULL, meta_methods, NULL, NULL))
		return false;

	return true;
}
