/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "wltable.h"
#include "wllocation.h"
#include <ordered.h>

#include "propertieshandler.h"

CWLPrivate::CWLPrivate(const char* objTypeName, void* objCPP_Ptr, CWLDocument* Parent, bool del)
{
  // Init the name of that object, pointer to the data held in the C++ 
  // and the parent for reference counting
	m_TypeName            = objTypeName;
	m_CPP_RealObject      = objCPP_Ptr;
	m_Parent              = Parent;
	m_IsJSCreatedObject   = del;
}

CWLPrivate::~CWLPrivate()
{
	if (m_IsJSCreatedObject) {

    switch(PropertiesHandler::GetClassId(m_TypeName.data()))
    {
    case PropertiesHandler::EN_CLASS_CELL : {
			CWLCellCollection* p = (CWLCellCollection*)m_CPP_RealObject;
			delete p;
      break;
		}

    case PropertiesHandler::EN_CLASS_LOCATION : {
			CWLLocation* p = (CWLLocation*)m_CPP_RealObject;
			delete p;
      break;
		}

    case PropertiesHandler::EN_CLASS_LINK : {
			CWLLink* p = (CWLLink*)m_CPP_RealObject;
			delete p;
      break;
		}

    default:{
			RVOrdered* p = (RVOrdered*)m_CPP_RealObject;
      p->clearAndDestroy();
			delete p;
      break;
		}
    }
	}
}

const char* CWLPrivate::name()
{
	return m_TypeName.data();
}

void* CWLPrivate::GetObj()
{
	return m_CPP_RealObject;
}

CWLDocument* CWLPrivate::GetParent()
{
	return m_Parent;
}
bool CWLPrivate::IsObjectJSCreated()
{
	return m_IsJSCreatedObject;
}
