/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlframecontainer.h"
#include <ordered.h>
#include "wlerrorlib.h"
#include "httpdefines.h"
#include "wlfuncs.h"
#include "parsefuncs.h"
#include "jsreplynames.h"
#include "commandmanager.h"
#include "workspace.h"
#include "jsprivate.h"

#define errFile HTML_ERROR_FILE


static int convertToRegExp(RVCString& input, RVCString& output)
{
  if (input.isNull())
    return RET_ERROR;
  
  size_t inSize = input.length();
  char   temp[512];
  int    j = 0;

  for (size_t i = 0; i < inSize; i++)
  {
    switch(input[i])  {
    case '*':
      temp[j++] = '[';
      temp[j++] = '\\';
      temp[j++] = 'x';
      temp[j++] = '0';
      temp[j++] = '0';
      temp[j++] = '-';
      temp[j++] = '\\';
      temp[j++] = 'x';
      temp[j++] = 'F';
      temp[j++] = 'F';
      temp[j++] = ']';
      temp[j++] = '*';
      break;
    case '+':
    case '?':
    case '.':
    case '[':
    case ']':
    case '^':
    case '$':
      temp[j++] = '\\';
      temp[j++] = input[i];
      break;  
    case '\\':
      temp[j++] = '\\';
      if (i+1 < inSize  &&  input[i+1] == '*')  {
        temp[j++] = '*';
        i++;
      }
      else
        temp[j++] = '\\';
      break;
    default:
      temp[j++] = input[i];
      break;
    }
  }
  temp[j] = '\0';
  output = temp;
  return RET_OK;
}


// 'n' is the index of the argument in the command
CWLFrame* GetFrame(CWLCommandManager* thread, JSContext* cx, uintN n, uintN argc, jsval v)
{
  if (argc > n) {
		if (WLIsObjectOfClass(cx, v, WLJSNAME_CLASS_FRAME)) {
			JSObject* obj = JSVAL_TO_OBJECT(v);
			CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
			if (prv)
				return (CWLFrame*)prv->GetObj();
		}
		else
			return NULL;
	}

  return (CWLFrame*)thread->GetParsedHtmlTree();
}

// 'n' is the index of the argument in the command
CWLDocument* GetWindowOrFrame(CWLCommandManager* thread, JSContext* cx, uintN n, uintN argc, jsval v)
{
  if (argc > n) {
		if (WLIsObjectOfClass(cx, v, WLJSNAME_CLASS_FRAME) || WLIsObjectOfClass(cx, v, WLJSNAME_CLASS_WINDOW)) 
    {
			JSObject* obj = JSVAL_TO_OBJECT(v);
			CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
			if (prv)
				return (CWLDocument*)prv->GetObj();
		}
		else
			return NULL;
	}

  return (CWLDocument*)thread->GetParsedHtmlTree();
}

// 'n' is the index of the argument in the command
const char* GetPageName(CWLCommandManager* thread, JSContext* cx, uintN n, uintN argc, jsval v, CWLFrame* frame)
{
	if (!frame)
		frame = GetFrame(thread, cx, n, argc, v);

	if (frame)
		return frame->GetLocation().Href();

	if (JS_ValueToString(cx, v))
    return (const char*)JS_GetStringBytes(JS_ValueToString(cx, v));

	return NULL;

/*
  if (argc > n)
		if (JS_ValueToString(cx, v) || WLIsObjectOfClass(cx, v, WLJSNAME_CLASS_FRAME))
	    return (const char*)JS_GetStringBytes(JS_ValueToString(cx, v));

  return (thread->GetPageName()).data();
*/
}


const char* GetStringValue(JSContext* cx, jsval val)
{
	const char* res = NULL;

	if (JSVAL_IS_STRING(val)) {
		JSString* str = JSVAL_TO_STRING(val);
		if (str)
			res = JS_GetStringBytes(str);
	}
/*
	JSString* str = JS_ValueToString(cx, val);
	if (str)
		res = (const char*)JS_GetStringBytes(str);
*/

	return res;
}

/*
bool ValueToInt32(JSContext *cx, jsval v, int32 *ip)
{
	jsdouble d;

	if (!JS_ValueToNumber(cx, v, &d))
		return false;

	if (JSDOUBLE_IS_NaN(d) || d <= -2147483649.0 || 2147483648.0 <= d)
		return false;

	*ip = (int32)floor(d + 0.5);     // Round to nearest

	return true;
}
*/


static int ReportError(JSContext* cx, const char* frmName, int frmId, ...)
{
  WLLogMessage::Severity sev = WLERRGetErrorSeverity(frmName, frmId);

  char    buf[10240];
  va_list ap;

  va_start(ap, frmId);
  vsprintf(buf, WLERRGetErrorFormat(frmName,frmId), ap);
  va_end(ap);

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
  bool rtn;
  if (sev > WLLogMessage::Note && ptrHttp)
		rtn = ptrHttp->reportMultiThread(sev, (const char*)buf);

  return (rtn ? RET_OK : RET_ERROR );
}

CWLFrame*  WLParse_GetFrameByIndex(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
	CWLFrame*  retValue;
  RVCString  pn;
//  jsdouble   index = -1;
  int32      index = -1;
  int        retIndex, errId;
  
	frame = GetFrame(thread, cx, 1, argc, argv[1]);
	pn = GetPageName(thread, cx, 1, argc, argv[1], frame);
	frameContainer* cont  = thread->GetPage();

	if (argc < 1)
		errId = 15;

	else if (!thread->IsParsed() || !cont)
		errId = 19;

	else if (!frame)
	  errId = 10;

	//index = JSVAL_TO_INT(argv[0]);
//	else if (!JS_ValueToNumber(cx, argv[0], &index))
	else if (!JS_ValueToECMAInt32(cx, argv[0], &index))
		errId = 16;

  else if (index <= 0)
    errId = 13;

  else if((retIndex = cont->GetFrameByIndex(frame, (int)index, &retValue)) == 0)
    return retValue;

  else if(retIndex == -1)
    errId = 10;

  else if(retIndex == -2)
    errId = 11;

  else if(retIndex == -3)
    errId = 12;

  else if(retIndex == -5)
    errId = 14;

	ReportError(cx, errFile, errId, (int)index, check_null(pn.data()));

  return NULL;
}


CWLFrame*  WLParse_GetFrameByUrl(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
	CWLFrame*  retValue;
  RVCString  pn;
  RVCString  pat;
  RVCString  newPat;
  int        retIndex, errId;
	JSString*  str = NULL;
	const char* val = NULL;

	// read the pattern parameter
	if (argc > 0) {
		val = GetStringValue(cx, argv[0]);
		if (val) {
			pat = val;
			convertToRegExp(pat, newPat);
		}
	}

	frame = GetFrame(thread, cx, 1, argc, argv[1]);
	pn = GetPageName(thread, cx, 1, argc, argv[1], frame);
	frameContainer* cont  = thread->GetPage();

	if (argc < 1)
		errId = 24;

	else if (!val)
		errId = 25;

	else if (!thread->IsParsed() || !cont)
		errId = 29;

	else if (!frame)
	  errId = 20;

  else if ((retIndex = cont->GetFrameByUrl(frame, (char*)newPat.data(), &retValue)) == 0)
    return retValue;
  
  else if (retIndex == -1)
    errId = 20;

  else if (retIndex == -2)
    errId = 21;

  else if (retIndex == -3)
    errId = 22;

  else if (retIndex == -5)
    errId = 23;

	ReportError(cx, errFile, errId, check_null(pat.data()), check_null(pn.data()));

  return NULL;
}

CWLFrame*  WLParse_GetFrameByLocation(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*       retFrame = NULL;
  RVCString       Target;
  int             errId = RET_OK;
	const char*     val = NULL;
  
  findFrameInDocStatus retStatus;

  // First we check if parsing was enabled on the http command
  if (!thread->IsParsed()) {
		errId = 210;
  }
  // If we have those, then we get the location from the args param
  else if (argc > 0) {
		val = GetStringValue(cx, argv[0]);
		if (val) {
			Target = val;
      CWLDocument*  Doc = (CWLDocument*)thread->GetParsedHtmlTree();
      if (Doc) {
        if (Doc -> m_Frames) {
          retStatus = Doc->GetFrameByLocation(Target, retFrame);
          if (retStatus == findFrameInDocStatus_ERROR) {
            errId = 215;
          }
          else if (retStatus == findFrameInDocStatus_NAME_MATCH_NUMBER_MISMATCH_WARNING) {
            if (thread->GetWorkspace()->m_AsmDiagnostics) {
              errId = 216;
            }
          }
          else if (retStatus == findFrameinDocStatus_NUMBER_MATCH_NAME_MISMATCH_WARNING) {
            if (thread->GetWorkspace()->m_AsmDiagnostics) {
              errId = 217;
            }
          }
        }
        else {
          errId = 214;
        }
      }
      else {
        errId = 213;
      }
    }
    else {
		  errId = 212;
    }
	}
  else {
		errId = 211;
  }

  if (errId != RET_OK) {
	  ReportError(cx, errFile, errId, check_null(Target.data()));
  }

  return retFrame;
}

CWLUrl*  WLParse_GetFrameUrl(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
	CWLUrl*    retValue;
  RVCString  pn;
  int        retIndex, errId;

	frame = GetFrame(thread, cx, 0, argc, argv[0]);
	pn = GetPageName(thread, cx, 0, argc, argv[0], frame);
	frameContainer* cont = thread->GetPage();

	if (!cont)
		errId = 189;

	else if (!frame)
		errId = 180;

	else if ((retIndex = cont->GetFrameUrl(frame, &retValue)) == 0)
		return retValue;

	else if (retIndex == -1)
		errId = 180;

	ReportError(cx, errFile, errId, check_null(pn.data()));

  return NULL;
}



CWLUrl*  WLParse_GetLinkByIndex(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
	CWLLink*    retValue;
  RVCString  pn;
  jsdouble   index = -1;
  int        retIndex, errId;

	frame = GetFrame(thread, cx, 1, argc, argv[1]);
	pn = GetPageName(thread, cx, 1, argc, argv[1], frame);
	frameContainer* cont  = thread->GetPage();

	if (argc < 1)
		errId = 35;

	else if (!thread->IsParsed() || !cont)
		errId = 39;

	else if (!frame)
		errId = 30;

	//index = JSVAL_TO_INT(argv[0]);
	else if (!JS_ValueToNumber(cx, argv[0], &index))
		errId = 36;

  else if (index <= 0)
    errId = 33;

  else if ((retIndex = cont->GetHrefByIndex(frame, (int)index-1, &retValue)) == 0)
    return retValue;

  else if (retIndex == -1)
    errId = 30;

  else if (retIndex == -2)
    errId = 31;

  else if (retIndex == -3)
    errId = 32;

  else if (retIndex == -5)
    errId = 34;
  
	ReportError(cx, errFile, errId, (int)index , check_null(pn.data()));
    
  return NULL;
}



CWLUrl*  WLParse_GetLinkByUrl(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
  CWLLink*    retValue;
  RVCString  pn;
  RVCString  pat;
  RVCString  newPat;
  int        errId, retIndex;
	JSString*  str = NULL;
	const char* val = NULL;

	// read the pattern parameter
	if (argc > 0) {
		val = GetStringValue(cx, argv[0]);
		if (val) {
			pat = val;
			convertToRegExp(pat, newPat);
		}
	}

	frame = GetFrame(thread, cx, 1, argc, argv[1]);
  pn = GetPageName(thread, cx, 1, argc, argv[1], frame);
	frameContainer* cont  = thread->GetPage();

	if (argc < 1)
		errId = 44;

	else if (!val)
		errId = 45;

	else if (!thread->IsParsed() || !cont)
		errId = 49;

	else if (!frame)
		errId = 40;

  else if ((retIndex = cont->GetHrefByUrl(frame, (char *)newPat.data(), &retValue)) == 0)
    return retValue;

  else if (retIndex == -1)
    errId = 40;

  else if (retIndex == -2)
    errId = 41;

  else if (retIndex == -3)
    errId = 42;

  else if (retIndex == -4 || retIndex == -5)
    errId = 43;

	ReportError(cx, errFile, errId, pat.data(), check_null(pn.data()));

	return NULL;
}
  

CWLUrl*  WLParse_GetLinkByName(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
	CWLLink*    retValue;
  RVCString  pn;
  RVCString  pat;
  RVCString  newPat;
  int        errId, retIndex;
	JSString*  str = NULL;
	const char* val = NULL;

	// read the pattern parameter
	if (argc > 0) {
		val = GetStringValue(cx, argv[0]);
		if (val) {
			pat = val;
			convertToRegExp(pat, newPat);
		}
	}

	frame = GetFrame(thread, cx, 1, argc, argv[1]);
  pn = GetPageName(thread, cx, 1, argc, argv[1], frame);
	frameContainer* cont  = thread->GetPage();

	if (argc < 1)
		errId = 54;

	else if (!val)
		errId = 55;

	else if (!thread->IsParsed() || !cont)
		errId = 59;

	else if (!frame)
		errId = 50;

  else if ((retIndex = cont->GetHrefByName(frame, (char*)newPat.data(), &retValue)) == 0)
    return retValue;

  else if (retIndex == -1)
    errId = 50;

  else if (retIndex == -2)
    errId = 51;

  else if (retIndex == -3)
    errId = 52;

  else if (retIndex == -4 || retIndex == -5)
    errId = 53;

	ReportError(cx, errFile, errId, check_null(pat.data()), check_null(pn.data()));

  return NULL;
}


const char*  WLParse_GetFieldValue(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*   frame;
  const char*  retValue;
  RVCString   fieldName;
  RVCString   pn;
  int         errId, retIndex;
	JSString*   str = NULL;
	const char* val = NULL;

	// read the pattern parameter
	if (argc > 0) {
		val = GetStringValue(cx, argv[0]);
		if (val) {
			fieldName = val;
		}
	}

	frame = GetFrame(thread, cx, 1, argc, argv[1]);
  pn = GetPageName(thread, cx, 1, argc, argv[1], frame);
	frameContainer* cont  = thread->GetPage();

	if (argc < 1)
		errId = 64;

	else if (!val)
		errId = 65;

	else if (!thread->IsParsed() || !cont)
		errId = 69;

	else if (!frame)
		errId = 60;

  else if ((retIndex = cont->GetFieldValueByPage(frame, (char*)fieldName.data(), &retValue)) == 0)
    return retValue;

  if(retIndex == -1)
    errId = 60;

  else if(retIndex == -2)
    errId = 61;

  else if(retIndex == -4 || retIndex == -3)
    errId = 62;

  else if(retIndex == -5)
    errId = 63;

	ReportError(cx, errFile, errId, check_null(fieldName.data()), check_null(pn.data()));

  return NULL;
}



CWLLocation*  WLParse_GetFormAction(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*     frame;
	CWLLocation*  retValue;
  jsdouble      index = -1;
  RVCString     pn;
  int		        errId, retIndex = 0;

	frame = GetFrame(thread, cx, 1, argc, argv[1]);
	pn = GetPageName(thread, cx, 1, argc, argv[1], frame);
	frameContainer* cont  = thread->GetPage();

	if (argc < 1)
		errId = 105;

	//index = JSVAL_TO_INT(argv[0]) - 1;
  else if (!JS_ValueToNumber(cx, argv[0], &index))
		errId = 106;

	else if (index < 0)
		errId = 101;

	else if (!thread->IsParsed() || !cont)
		errId = 109;

	else if (!frame)
		errId = 100;

  else if ((retIndex = cont->GetFormAction(frame, (int)index-1, &retValue)) == 0)      
		return retValue;

  else if (retIndex == -1)
    errId = 100;//no page name

  else if (retIndex == -2)
    errId = 101;//index suould be > 0

  else if (retIndex == -3)
    errId = 102; //no forms in page

  else if (retIndex == -4)
    errId = 103;//No. of forms in page smaller than index

  else if (retIndex == -5)
    errId = 104;//no parsing of forms

	ReportError(cx, errFile, errId, (int)index , check_null(pn.data()));

  return NULL;
}




const char*  WLParse_GetFieldValueInForm(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*   frame;
  const char*  retValue;
  RVCString   fieldName;
  RVCString   pn;
  jsdouble    index = -1;
  int         errId, retIndex;
	JSString*   str = NULL;
	const char* val = NULL;

	// read the pattern parameter
	if (argc > 1) {
		val = GetStringValue(cx, argv[1]);
		if (val) {
			fieldName = val;
		}
	}

	frame = GetFrame(thread, cx, 2, argc, argv[2]);
  pn = GetPageName(thread, cx, 2, argc, argv[2], frame);
	frameContainer* cont  = thread->GetPage();

	if (argc < 2)
		errId = 76;

	else if (!val)
		errId = 77;

	else if (!thread->IsParsed() || !cont)
		errId = 79;

	else if (!frame)
		errId = 70;

	//index = JSVAL_TO_INT(argv[0]) - 1;
	else if (!JS_ValueToNumber(cx, argv[0], &index))
		errId = 78;

  else if (index < 0)
    errId = 74;

  else if ((retIndex = cont->GetFieldValueByForm(frame, (int)index-1, (char*)fieldName.data(), &retValue)) == 0)
    return retValue;

  else if (retIndex == -1)
    errId = 70;

  else if (retIndex == -2)
    errId = 71;

  else if (retIndex == -3)
    errId = 72;

  else if (retIndex == -4)
    errId = 73;

  else if (retIndex == -6)
    errId = 75;
  
	ReportError(cx, errFile, errId, (int)index, check_null(fieldName.data()), check_null(pn.data()));

  return NULL;
}


// not a real html parse function
char* WLParse_GetQSFieldValue(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	RVCString url(""); // first param - URL+Query String
	RVCString qs(""); //  Query String
	RVCString fn(""); // second param - Field Name
	CWLSearchArray  sa;  // This container will parse and hold the Query String
	const char *val;
	int urlQmark;
	int errId;

	if (argc >= 2) {
		const char* val1 = GetStringValue(cx, argv[0]);
		const char* val2 = GetStringValue(cx, argv[1]);
		if (val1 && val2) {
			url = val1;
			fn = val2;
			if ((urlQmark = url.last('?')) != RV_NPOS) {
				qs = url(urlQmark + 1, url.length() - (urlQmark  + 1));

				sa.LoadURLEncodedString((const char *)qs);
				if ((val = sa.Value((const char *)fn)) != NULL )
					return strdup(val);
			}
			errId = 190;
		}
		else
			errId = 192;
	}
	else
		errId = 191;

	ReportError(cx, errFile, errId, check_null(url.data()), check_null(fn.data()));

	return NULL;
}


const char*  WLParse_GetHeaderValue(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
  RVCString  pn;
	const char*      retVal;
	int        errId, retIndex;
	RVCString  header;
	JSString*  str = NULL;
	const char* val = NULL;

	// read the header parameter
	if (argc > 0) {
		val = GetStringValue(cx, argv[0]);
		if (val) {
			header = val;
			header.toLower();
		}
	}

	frame = GetFrame(thread, cx, 1, argc, argv[1]);
	pn = GetPageName(thread, cx, 1, argc, argv[1], frame);
	frameContainer* cont  = thread->GetPage();

	if (argc < 1)
		errId = 112;

	else if (!val)
		errId = 113;

	else if (!thread->IsParsed() || !cont)
		errId = 119;

	else if (!frame)
		errId = 110;

	else if ((retIndex = cont->GetHeaderValue(frame, header.data(), retVal)) == 0)
		return retVal;

	else if (retIndex == -1)
		errId = 110;

	else if (retIndex == -2)
		errId = 111;

	ReportError(cx, errFile, errId, check_null(header.data()), check_null(pn.data()));

	return NULL;
}


unsigned  WLParse_GetStatusNumber(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
  RVCString  pn;
	int	       retValue;
	int        errId, retIndex = 0;

	frame = GetFrame(thread, cx, 0, argc, argv[0]);
	pn = GetPageName(thread, cx, 0, argc, argv[0], frame);
	frameContainer* cont  = thread->GetPage();

	if (!cont)
		errId = 129;

	else if (!frame)
		errId = 120;

	else {
		retValue = cont->GetStatusNumber(frame);
		if (retValue != 0)
			return retValue;

		else
			errId = 121;
	}

	ReportError(cx, errFile, errId, check_null(pn.data()));

	return RET_ERROR;
}


const char*  WLParse_GetStatusLine(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
  RVCString  pn;
	const char*      retVal;
	int        errId, retIndex;

	frame = GetFrame(thread, cx, 0, argc, argv[0]);
	pn = GetPageName(thread, cx, 0, argc, argv[0], frame);
	frameContainer* cont  = thread->GetPage();

	if (!cont)
		errId = 139;

	else if (!frame)
		errId = 130;

	else if ((retIndex = cont->GetStatusLine(frame, retVal)) == 0)
		return retVal;

	else if (retIndex == -1)
		errId = 130;

	else if (retIndex == -2)
		errId = 131;

	ReportError(cx, errFile, errId, check_null(pn.data()));

	return NULL;
}


const char*  WLParse_GetHost(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
	RVCString  pn;
	const char*      retVal;
	int        errId, retIndex;

	frame = GetFrame(thread, cx, 0, argc, argv[0]);
	pn = GetPageName(thread, cx, 0, argc, argv[0], frame);
	frameContainer* cont  = thread->GetPage();

	if (!cont)
		errId = 149;

	else if (!frame)
		errId = 140;

	else if ((retIndex = cont->GetHost(frame, retVal)) == 0)
		return retVal;

	else if (retIndex == -1)
		errId = 140;

	else if (retIndex == -2)
		errId = 141;

	ReportError(cx, errFile, errId, check_null(pn.data()));

	return NULL;
}


const char*  WLParse_GetHostName(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
  RVCString  pn;
	const char*      retVal;
	int        errId, retIndex;

	frame = GetFrame(thread, cx, 0, argc, argv[0]);
	pn = GetPageName(thread, cx, 0, argc, argv[0], frame);
	frameContainer* cont  = thread->GetPage();

	if (!cont)
		errId = 159;

	else if (!frame)
		errId = 150;

	else if ((retIndex = cont->GetHostName(frame, retVal)) == 0)
		return retVal;

	else if (retIndex == -1)
		errId = 150;

	else if (retIndex == -2)
		errId = 151;

	ReportError(cx, errFile, errId, check_null(pn.data()));

	return NULL;
}


unsigned  WLParse_GetPortNum(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
  RVCString  pn;
	int	       retValue;
	int        errId, retIndex = 0;

	frame = GetFrame(thread, cx, 0, argc, argv[0]);
	pn = GetPageName(thread, cx, 0, argc, argv[0], frame);
	frameContainer* cont  = thread->GetPage();

	if (!cont)
		errId = 169;

	else if (!frame)
		errId = 160;

	else {
		retValue = cont->GetPortNum(frame);
		return retValue;
	}

	ReportError(cx, errFile, errId, check_null(pn.data()));

	return RET_ERROR;
}


const char*  WLParse_GetUri(CWLCommandManager* thread, JSContext* cx, uintN argc, jsval* argv)
{
	CWLFrame*  frame;
  RVCString  pn;
	const char*      retVal;
	int        errId, retIndex;

	frame = GetFrame(thread, cx, 0, argc, argv[0]);
	pn = GetPageName(thread, cx, 0, argc, argv[0], frame);
	frameContainer* cont  = thread->GetPage();

	if (!cont)
		errId = 179;

	else if (!frame)
		errId = 170;

	else if ((retIndex = cont->GetUri(frame, retVal)) == 0)
		return retVal;

	else if (retIndex == -1)
		errId = 170;

	else if (retIndex == -2)
		errId = 171;

	ReportError(cx, errFile, errId, check_null(pn.data()));

	return NULL;
}


