/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlxml.h"
#include "wldocument.h"
#include <string.h>
#include "rvjsapi.h"
#include "platform.h"
#include "wlarray.h"
#include "wllocation.h"
#include "wlform.h"
#include "wlelement.h"
#include "wltable.h"
#include "commandmanager.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "tracedll.h"
#include "wlscript.h"

extern JSObject* WLNewOptionObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewKvpObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewFormObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewArrayObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewElementObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewLinkObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewFrameObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewWindowObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewLocationObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewActionObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewImgObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewEmbedObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewCSSObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewXmlObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewScriptObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewAppletObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewTableObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewRowObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewCellObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewMetaObject(JSContext *cx, JSObject *obj);


typedef JSObject* NewObjectFunction(JSContext*, JSObject*);
JSBool SetAnObjectValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, const char* className, void* value, jsval* rval, NewObjectFunction* newObjFunc, bool toDelete = false);

#include "tracedll.h"

bool WLDeleteProperty(JSContext* cx, JSObject* obj, const char* name)
{
	jsval v;
	JSBool res = JS_LookupProperty(cx, obj, name, &v);
	if (res && JSVAL_IS_OBJECT(v)) {
		JSObject* prop = JSVAL_TO_OBJECT(v);
		//WLFinalize(cx, prop);
		JSBool found;
		JS_SetPropertyAttributes(cx, obj, name, 0, &found);
		JS_DeleteProperty(cx, obj, name);
		return true;
	}

	return false;
}

void WLFinalize(JSContext* cx, JSObject* obj)
{
	// decrement reference count of the dom
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (prv) {
		CWLDocument* Parent = prv->GetParent();
    if (Parent) {
      WlTraceMsg( "Finialize: Decreasing reference to the frame : %d",(int)Parent);
			if (Parent->decRefCount() == 0) {
        WlTraceMsg( "No more reference exist to the frame : %d so we delete it",(int)Parent);
				delete Parent;
			}
    }
		delete prv;
		JS_SetPrivate(cx, obj, NULL);
	}
}

static CWLDocument* CurrentDom(JSContext* cx)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp)
		return (CWLDocument*)ptrHttp->GetParsedHtmlTree();

	return NULL;
}



JSObject* WLNewObject(JSContext* cx, JSClass* cls, JSObject* proto, JSObject* parent)
{
	JSObject* obj = JS_NewObject(cx, cls, NULL, NULL);
//	JS_SetPrototype(cx, obj, proto);
//	JS_SetParent(cx, obj, parent);
	return obj;
}

void WLSetNewObject(JSContext* cx, JSObject* obj, JSObject* newObj, CWLDocument* Parent, const char* name, void* value, bool toDelete)
{

  if (Parent) {
    WlTraceMsg( "Adding reference of an object to the frame : %d",(int)Parent);
		Parent->incRefCount();
  }

	CWLPrivate* prv = new CWLPrivate(name, (void*)value, Parent, toDelete);
	JS_SetPrivate(cx, newObj, (void*)prv);
}


const char* WLClassName(JSContext* cx, jsval v)
{
	if (JSVAL_IS_OBJECT(v))
		return JS_GetClass(cx, JSVAL_TO_OBJECT(v))->name;

	return NULL;
}


bool WLIsClassName(JSContext* cx, jsval v, const char* name)
{
	if (strcmp(name, WLClassName(cx, v)))
		return false;

	return true;
}

bool DllExport WLIsObjectOfClass(JSContext* cx, jsval v, const char* name)
{
	if (!JSVAL_IS_OBJECT(v) || strcmp(name, WLClassName(cx, v)))
		return false;

	return true;
}



// for the following WLSet... functions:
	// using the following call will cause lvalue to be new instance of the property
	// eliminating the call will cause lvalue to point to the property
//	JS_SetProperty(cx, obj, name, rval);


JSBool DllExport WLSetIntValue(JSContext* cx, JSObject* obj, const char* name, unsigned value, jsval* rval)
{
	*rval = INT_TO_JSVAL((int)value);
	return JS_TRUE;
}

JSBool DllExport WLSetBoolValue(JSContext* cx, JSObject* obj, const char* name, bool value, jsval* rval)
{
	if (value)
		*rval = BOOLEAN_TO_JSVAL(JS_TRUE);
	else
		*rval = BOOLEAN_TO_JSVAL(JS_FALSE);
	return JS_TRUE;
}

JSBool DllExport WLSetDoubleValue(JSContext* cx, JSObject* obj, const char* name, double value, jsval* rval)
{
	jsdouble* val = JS_NewDouble(cx, (jsdouble)value);
	*rval = DOUBLE_TO_JSVAL(val);

	return JS_TRUE;
}

JSBool DllExport WLSetStringValue(JSContext* cx, JSObject* obj, const char* name, const char* value, jsval* rval, size_t length)
{
	if (!value)
		return JS_TRUE;

	JSString* str = NULL;
	if (length == 0)
		str =JS_NewStringCopyZ(cx, value);
	else
		str =JS_NewStringCopyN(cx, value, length);
	if (!str)
		return JS_FALSE;

	*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}




JSBool SetAnObjectValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, const char* className, void* value, jsval* rval, NewObjectFunction* newObjFunc, bool toDelete)
{
	if (!value)
		return JS_TRUE;

	JSObject* newObj = newObjFunc(cx, obj);
	if (!newObj)
		return JS_FALSE;

  WLSetNewObject(cx, obj, newObj, Parent, className, value, toDelete);

	*rval = OBJECT_TO_JSVAL(newObj);
	return JS_TRUE;
}

JSBool DllExport WLSetCellArrayValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLCellCollection* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_CELL, value, rval, WLNewArrayObject, true);
}

JSBool DllExport WLSetArrayValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, const char* name, void** value, jsval* rval)
{
	bool toDelete = false;
	if (!*value) {
		toDelete = true;
		*value = (void*)new RVOrdered;
	}

	JSBool ret = SetAnObjectValue(cx, obj, Parent, name, *value, rval, WLNewArrayObject, toDelete);

  JSObject* newObj = JSVAL_TO_OBJECT(*rval);

  return ret;
}

JSBool DllExport WLSetKvpValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLKvp* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_KVP, value, rval, WLNewKvpObject);
}

JSBool DllExport WLSetLocationValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLLocation* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_LOCATION, value, rval, WLNewLocationObject);
}

JSBool DllExport WLSetBrowserLocationValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLLocation* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_BROWSER_LOCATION, value, rval, WLNewLocationObject);
}

JSBool DllExport WLSetActionValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLLocation* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_ACTION, value, rval, WLNewActionObject);
}

JSBool DllExport WLSetImgValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLEmbed* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_IMG, value, rval, WLNewImgObject);
}

JSBool DllExport WLSetEmbedValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLEmbed* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_EMBED, value, rval, WLNewEmbedObject);
}

JSBool DllExport WLSetCSSValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLUrl* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_CSS, value, rval, WLNewCSSObject);
}

//JSBool DllExport WLSetXmlValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLXmlDocument* value, jsval* rval, int setScope)
JSBool DllExport WLSetXmlValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLXmlDocument* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_XML, value, rval, WLNewXmlObject);
}

JSBool DllExport WLSetScriptValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLScript* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_SCRIPT, value, rval, WLNewScriptObject);
}

JSBool DllExport WLSetAppletValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLApplet* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_APPLET, value, rval, WLNewAppletObject);
}

JSBool DllExport WLSetMetaValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLMeta* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_META, value, rval, WLNewMetaObject);
}

JSBool DllExport WLSetLinkValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLUrl* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_LINK, value, rval, WLNewLinkObject);
}

JSBool DllExport WLSetFrameValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLFrame* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_FRAME, value, rval, WLNewFrameObject);
}

JSBool DllExport WLSetWindowValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLFrame* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_WINDOW, value, rval, WLNewWindowObject);
}

JSBool DllExport WLSetElementValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLElement* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_ELEMENT, value, rval, WLNewElementObject);
}

JSBool DllExport WLSetOptionValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLOption* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_OPTION, value, rval, WLNewOptionObject);
}

JSBool DllExport WLSetFormValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLForm* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_FORM, value, rval, WLNewFormObject);
}

JSBool DllExport WLSetTableValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLTable* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_TABLE, value, rval, WLNewTableObject);
}

JSBool DllExport WLSetRowValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLTableRow* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_ROW, value, rval, WLNewRowObject);
}

JSBool DllExport WLSetCellValue(JSContext* cx, JSObject* obj, CWLDocument* Parent, CWLTableCell* value, jsval* rval)
{
	return SetAnObjectValue(cx, obj, Parent, WLJSNAME_CLASS_CELL, value, rval, WLNewCellObject);
}






unsigned WLIntValue(jsval val)
{
	if (JSVAL_IS_INT(val))
		return (unsigned)JSVAL_TO_INT(val);

	return 0;
}

double WLDoubleValue(jsval val)
{
	if (JSVAL_IS_DOUBLE(val))
		return *JSVAL_TO_DOUBLE(val);

	return 0;
}

const char* WLStringValue(jsval val)
{
	if (JSVAL_IS_STRING(val))
		return JS_GetStringBytes(JSVAL_TO_STRING(val));

	return NULL;
}

CWLArray* WLArrayValue(JSContext* cx, jsval val)
{
	// check if object is of class 'WLArray'
	if (WLIsObjectOfClass(cx, val, WLJSNAME_CLASS_ARRAY))
		return (CWLArray*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(val));

	return NULL;
}

CWLLocation* WLLocationValue(JSContext* cx, jsval val)
{
	// check if object is of class 'WLLocation'
	if (WLIsObjectOfClass(cx, val, WLJSNAME_CLASS_LOCATION))
		return (CWLLocation*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(val));

	return NULL;
}

CWLLink* WLLinkValue(JSContext* cx, jsval val)
{
	// check if object is of class 'WLLink'
	if (WLIsObjectOfClass(cx, val, WLJSNAME_CLASS_LINK))
		return (CWLLink*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(val));

	return NULL;
}

CWLDocument* WLDocumentValue(JSContext* cx, jsval val)
{
	// check if object is of class 'WLDocument'
	if (WLIsObjectOfClass(cx, val, WLJSNAME_CLASS_DOCUMENT))
		return (CWLDocument*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(val));

	return NULL;
}

JSBool WLInitProperty(JSContext* cx, JSObject* obj, const char* prop)
{
	return JS_DefineProperty(cx, obj, prop, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
}

JSBool WLInitHiddenProperty(JSContext* cx, JSObject* obj, const char* name)
{
	return JS_DefineProperty(cx, obj, name, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT);
}



// index == -1 indicate that a name is used instead of an index.


CWLKvp* WLGetKvpItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLKvp*)(*array)[index];
	}
	else if (key != NULL) {
		// return the first item that it's name (data) matches the key
		for (unsigned i = 0; i < array->entries(); i++) {
			CWLKvp* pKvp = (CWLKvp*)(*array)[i];
			if (!strcmp(pKvp->Key(), key))
				return pKvp;
		}
	}

	return NULL;
}

CWLElement* WLGetElementItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLElement*)(*array)[index];
	}
	else if (key != NULL) {
		// return the first item that it's name (data) matches the key
		for (unsigned i = 0; i < array->entries(); i++) {
			CWLElement* pElement = (CWLElement*)(*array)[i];
			if (!strcmp(pElement->Name(), key))
				return pElement;
		}
	}

	return NULL;
}

CWLOption* WLGetOptionItemPtr(RVOrdered* array, size_t index)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLOption*)(*array)[index];
	}
	return NULL;
}

CWLFrame* WLGetFrameItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLFrame*)(*array)[index];
	}
	else if (key != NULL) {
		// return the first item that it's name (data) matches the key
		for (size_t i = 0; i < array->entries(); i++) {
			CWLFrame* pFrame = (CWLFrame*)(*array)[i];
			if (!strcmp(pFrame->GetName().data(), key))
				return pFrame;
		}
	}

	return NULL;
}

CWLForm* WLGetFormItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLForm*)(*array)[index];
	}
	else if (key != NULL) {
		// return the first item that it's name (data) matches the key
		for (unsigned i = 0; i < array->entries(); i++) {
			CWLForm* pForm = (CWLForm*)(*array)[i];
			if (!strcmp(pForm->Name(), key))
				return pForm;
		}
	}

	return NULL;
}

CWLEmbed* WLGetEmbedItemPtr(RVOrdered* array, size_t index, const char* key)
{
	size_t entries_ = array->entries();

	if (index != -1) {
		if (index < entries_)
			return (CWLEmbed*)(*array)[index];
	}
	else if (key != NULL) {
		CWLEmbed* pImg;
		// return the first item that it's name (data) matches the key
		for (unsigned i = 0; i < entries_; ++i) {
			pImg = (CWLEmbed*)(*array)[i];
			if (!strcmp(pImg->Href(), key))
				return pImg;
		}
	}

	return NULL;
}

CWLUrl* WLGetCSSItemPtr(RVOrdered* array, size_t index, const char* key)
{
	size_t entries_ = array->entries();

	if (index != -1) {
		if (index < entries_)
			return (CWLUrl*)(*array)[index];
	}
	else if (key != NULL) {
		CWLUrl* pCss;
		// return the first item that it's name (data) matches the key
		for (unsigned i = 0; i < entries_; ++i) {
			pCss = (CWLUrl*)(*array)[i];
			if (!strcmp(pCss->Href(), key))
				return pCss;
		}
	}

	return NULL;
}

CWLXmlDocument* WLGetXmlItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLXmlDocument*)(*array)[index];
	}
	else if (key != NULL) {
		// return the first item that it's Id (data) matches the key
		for (unsigned i = 0; i < array->entries(); i++) {
			CWLXmlDocument* pXml = (CWLXmlDocument*)(*array)[i];
			if (!strcmp(pXml->GetId(), key))
				return pXml;
		}
	}

	return NULL;
}

CWLScript* WLGetScriptItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLScript*)(*array)[index];
	}
	else if (key != NULL) {
		// return the first item that it's Id (data) matches the key
		for (unsigned i = 0; i < array->entries(); i++) {
			CWLScript* pScript = (CWLScript*)(*array)[i];
			if (!strcmp(pScript->GetId(), key))
				return pScript;
		}
	}

	return NULL;
}

CWLApplet* WLGetAppletItemPtr(RVOrdered* array, size_t index, const char* key)
{
	size_t entries_ = array->entries();

	if (index != -1) {
		if (index < entries_)
			return (CWLApplet*)(*array)[index];
	}
	else if (key != NULL) {
		// return the first item that it's Id (data) matches the key
		for (unsigned i = 0; i < entries_; ++i) {
			CWLApplet* pApplet = (CWLApplet*)(*array)[i];
			if (!strcmp(pApplet->GetId(), key))
				return pApplet;
		}
	}
	return NULL;
}

CWLMeta* WLGetMetaItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLMeta*)(*array)[index];
	}
	else if (key != NULL) {
		// return the first item that it's name  matches the key
		for (unsigned i = 0; i < array->entries(); i++) {
			CWLMeta* pMeta = (CWLMeta*)(*array)[i];
			if (pMeta->getName() && !strcmp(pMeta->getName(), key))
				return pMeta;
		}
	}

	return NULL;
}


CWLLink* WLGetLinkItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLLink*)(*array)[index];
	}
	else if (key != NULL) {
		// return the first item that it's name  matches the key
		for (unsigned i = 0; i < array->entries(); i++) {
			CWLLink* pLink = (CWLLink*)(*array)[i];
			if ((pLink->Name() && !strcmp(pLink->Name(), key)) ||
				(pLink->Id() && !strcmp(pLink->Id(), key))   )
				return pLink;
		}
	}

	return NULL;
}


CWLLocation* WLGetLocationItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLLocation*)(*array)[index];
	}
	else if (key != NULL) {
		// return the first item that it's name (data) matches the key
		for (unsigned i = 0; i < array->entries(); i++) {
			CWLLocation* pLink = (CWLLocation*)(*array)[i];
			if (!strcmp(pLink->Href(), key))
				return pLink;
		}
	}

	return NULL;
}

RVCollectableString* WLGetStringItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (RVCollectableString*)(*array)[index];
	}
	else if (key != NULL) {
		// return the first item that it's name (data) matches the key
		for (unsigned i = 0; i < array->entries(); i++) {
			RVCollectableString* pString = (RVCollectableString*)(*array)[i];
			if (!strcmp(pString->data(), key))
				return pString;
		}
	}

	return NULL;
}

CWLTable* WLGetTableItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLTable*)(*array)[index];
	}
	else if (key != NULL) {
		// return the first item that it's name (data) matches the key
		for (unsigned i = 0; i < array->entries(); i++) {
			CWLTable* table = (CWLTable*)(*array)[i];
			if (!strcmp(table->getTableID(), key))
				return table;
		}
	}

	return NULL;
}

CWLTableRow* WLGetRowItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLTableRow*)(*array)[index];
	}

	return NULL;
}

CWLTableCell* WLGetCellItemPtr(RVOrdered* array, size_t index, const char* key)
{
	if (index != -1) {
		if (index < array->entries())
			return (CWLTableCell*)(*array)[index];
	}

	return NULL;
}



int ReportJSError(JSContext* cx, const char* frmName, int frmId, ...)
{
  WLLogMessage::Severity sev = WLERRGetErrorSeverity(frmName, frmId);

  char    buf[1024];
  va_list ap;

  va_start(ap, frmId);
  vsprintf(buf, WLERRGetErrorFormat(frmName,frmId), ap);
  va_end(ap);

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
  bool rtn;
	if (sev > WLLogMessage::Note && ptrHttp)
		rtn = ptrHttp->reportMultiThread(sev, (const char*)buf);

  return( rtn ? -1 : 0 );
}
