/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/*******************************************************
 * File name : wlMultiPart.cpp
 * Author    : Yossi Nakash
 * Created   : 2/1/01
 * ==========:
 * Abstract  : class CWLMultiPart implementation
 *
*******************************************************/
#include "wlmultipart.h"
#include "httpdefines.h"
#include "httprequest.h"
#include "commandmanager.h"
#include <stdio.h>
#include <fstream.h>
#include "gt_strwrapper.h"


CWLMultiPart::CWLMultiPart(CWLHttpRequest* p_Request) :
m_Request(p_Request)
{
//  m_pcFileName[0] = '\0' ;
}

CWLMultiPart::~CWLMultiPart()
{
}


CWLMultiPart::CWLMultiPart( const CWLMultiPart& src )
{
  m_Request = src.m_Request ;
}


CWLMultiPart::operator=( const CWLMultiPart& src )
{
  m_Request = src.m_Request ;
  return 0;
}


const char* CWLMultiPart::GetData()
{
  return m_sMultiPartData.data();
}


int CWLMultiPart::GetDataLength()
{
  return m_sMultiPartData.length();
}


RVCString& CWLMultiPart::GetBoundary()
{
  return m_sBoundary ;
}

/************************************************************************
    method  : CreateMultiPartData
    Input   : CWLDataBlock* , thread num, and round num.
    Purpose : Create multipart data from form fields.
              round and thread num use for creating boundary. could be
              any other number.
    Remark  : Use RVCString::append, for binary data files
    Output  : int, 0 if successful, else error number.
************************************************************************/

int CWLMultiPart::CreateMultiPartData(CWLDataBlock* pFormDataBlock, int nThread, int nRound, RVCString sinBoundery, int charset)
{
  char sBoundary[100]="";
  RVCString sKey, pVal, sContentTypeKey, leFileName;
  int nIndex;
  size_t nEntries = pFormDataBlock->entries();
	if (sinBoundery.isNull()) {
  sprintf(sBoundary, "--Thread:%04d@Round:%04d", nThread, nRound);
  m_sBoundary = sBoundary;
  m_sBoundary.remove(0,2);
	}
	else {
		sprintf(sBoundary, "--%s", sinBoundery.data());
	}

  int nRet = 0 ;

  FILE *fp;
  char *ps=NULL;
  int num;

  m_sMultiPartData.remove(0);

	for (size_t i = 0; i < nEntries ; i++)
  {
    sKey = pFormDataBlock->Key(i);
    pVal = ToANSI(pFormDataBlock->Value(i), charset);

    if ((sKey != VOID_STRING) && (sKey != EMPTY_STRING))
    {
      // in a case of a file object
      if ((nIndex = sKey.index(WL_FILE_NAME)) != RV_NPOS )
      {
        m_sMultiPartData.append(sBoundary);
        m_sMultiPartData.append(CRandLF);
        m_sMultiPartData.append(MULTIPART_HEADER) ;

        sKey.remove(nIndex);
        m_sMultiPartData.append(MULTIPART_NAME);
        m_sMultiPartData.append(sKey);
        m_sMultiPartData.append("\";");

        //new - handling empty file name
	      //Galtech
        if (pVal != VOID_STRING && pVal != EMPTY_STRING)
        {
          m_sMultiPartData.append(MULTIPART_FILE_NAME) ;
          m_sMultiPartData.append(pVal) ;
          m_sMultiPartData.append("\"") ;

//          strcpy(m_pcFileName, pVal);
          m_pcFileName.remove(0);
          m_pcFileName.append(pVal);

          //Concatinate LoadEngine temporary path name to the file name.
          leFileName =  CArrangeMachineTempFiles::GetWlTempFullPath (m_Request->GetCm()->GetJScx() , WLECopiedFile, pVal) ;
          if( (fp = fopen(leFileName,"rb")) == NULL )
              return 230;
          // get file length
          fseek(fp, 0, SEEK_END);
          int length = ftell(fp);
          ps = new char[length+1];
          fseek(fp, 0, SEEK_SET);
          num = fread(ps, sizeof(char), length, fp);
          fclose(fp);
        }
        else  //if there is no file name send it empty
        {
          m_sMultiPartData.append(MULTIPART_FILE_NAME) ;
          m_sMultiPartData.append("\"") ;
        }

        m_sMultiPartData.append(CRandLF) ;

        sContentTypeKey = sKey + WL_CONTENT_TYPE ;

        for (size_t j=i; j < nEntries ; j++)
        {
          if (sContentTypeKey == pFormDataBlock->Key(j))
          {
            m_sMultiPartData.append(CONTENT_TYPE) ;
            m_sMultiPartData.append(pFormDataBlock->Value(j)) ;
            m_sMultiPartData.append(CRandLF) ;
            m_sMultiPartData.append(CRandLF) ;
            break;
          }
        }
        if (ps) {
        	m_sMultiPartData.append(ps, num) ;
        	delete [] ps;
          ps = NULL;
		}
        m_sMultiPartData.append(CRandLF) ;
      }
      else if (!sKey.contains(WL_CONTENT_TYPE))
      {
        m_sMultiPartData.append(sBoundary);
        m_sMultiPartData.append(CRandLF);
        m_sMultiPartData.append(MULTIPART_HEADER) ;

        m_sMultiPartData.append(MULTIPART_NAME);
        m_sMultiPartData.append(sKey);
        m_sMultiPartData.append("\"");
        m_sMultiPartData.append(CRandLF) ;
        m_sMultiPartData.append(CRandLF) ;

		//Galtech
        if (pVal == EMPTY_STRING)
          m_sMultiPartData.append("") ;
        else if (pVal != VOID_STRING)
          m_sMultiPartData.append(pVal) ;

        m_sMultiPartData.append(CRandLF) ;
      }
    }
  }

  m_sMultiPartData.append(sBoundary);
  m_sMultiPartData.append(MULTIPART_END_BOUNDARY) ;
  m_sMultiPartData.append(CRandLF) ;

  return nRet;
}
