/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : wlsslcache.cpp
 *
 *	Author : 
 *
 *	Description : 
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

/** include files **/
#ifdef __SSL__
#include "generalsslfunctions.h"
#include "commandmanager.h"
#include "wlsslcache.h"
#include "connpool.h"
#include "httpssl.h"
#endif

#define __WL_TRACE__
#include "tracedll.h"

/** local definitions **/

/** default settings **/

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** classes definitions **/
WLSSLRecord::WLSSLRecord(SSLBuffer& key)
{
	dataLength = key.length;
	dataBuffer = (uchar*)CD_malloc(dataLength);
	if (dataBuffer)
		memmove(dataBuffer, key.data, dataLength);
}

WLSSLRecord::~WLSSLRecord()
{
	if (dataBuffer)
		CD_free(dataBuffer);
}


int WLSSLRecord::compareTo(const RVCollectable* rwc) const
{
	WLSSLRecord* rec = (WLSSLRecord*)rwc;
	if (dataLength != rec->DataLength())
		return dataLength - rec->DataLength();

	return memcmp(dataBuffer, rec->DataBuffer(), dataLength);
}

bool WLSSLRecord::isEqual(const RVCollectable* rwc) const
{
	return (compareTo(rwc) == 0 ? TRUE : FALSE);
}


unsigned WLSSLRecord::hash() const
{
	unsigned sum = 0;
	int i = dataLength;
	while (i > 0)
		sum += (unsigned)dataBuffer[--i];

	return sum;
}



/** public functions **/
SSLErr InitDatabase(void **dbRef)
{   
	return SSLNoErr;
}

SSLErr DisposeDatabase()
{
	return SSLNoErr;
}


/** private functions **/
SSLErr SSLAddSession (SSLBuffer key, SSLBuffer data, void *sessionRef)
{
  if (!sessionRef) {
	  WlTraceMsg("SSL: ADD Session key [%d: %s] ... NOT Added because there is no session reference\n", key.length, (const char*)key.data);
		return SSLNoErr;
  }

	CWLDBRef* dbRef = (CWLDBRef*)sessionRef;
	CWLConnectionsPool* cp = dbRef->cp;
	RVHashDictionary* db = (RVHashDictionary*)&cp->m_SslCacheDB;
    
//	ASSERT(db != 0);
  if (!db) {
	  WlTraceMsg("SSL: Session key & data [[%d: %s], [%d: %s]]... NOT Added because there is no session database\n", key.length, (const char*)key.data, data.length, (const char*)data.data);
		return SSLNoErr;
  }

	RVCString newKey((const char*)key.data, key.length);
	newKey += dbRef->ctx->m_CertificateFile;
	SSLBuffer newKey2;
	newKey2.length = newKey.length();
	newKey2.data = (unsigned char*)newKey.data();


	WLSSLRecord recKey(key);
	// protect DB while adding a key
	RVReadersWriterLock::WriteLockGuard writer(cp->m_SslLock);
	WLSSLRecord* value = (WLSSLRecord*)db->findValue(&recKey);
  if (value) {
    // record found, remove key-value (replace it with the new key)
  	WlTraceMsg("SSL: Removing old session before adding [[%d: %s], [%d: %s]]...\n", key.length, (const char*)key.data, data.length, (const char*)data.data);
		db->removeAndDestroy(&recKey);
  }

	WlTraceMsg("SSL: adding session key & data [[%d: %s], [%d: %s]]...\n", key.length, (const char*)key.data, data.length, (const char*)data.data);
	db->insertKeyAndValue(new WLSSLRecord(key), new WLSSLRecord(data));	// create new key-value

	return SSLNoErr;
}

SSLErr SSLGetSession (SSLBuffer key, SSLBuffer *sessionData, void *sessionRef)
{
  if (!sessionRef) {
	  WlTraceMsg("SSL: Get Session key [%d: %s]... NOT found because there is no session reference\n", key.length, (const char*)key.data);
		return SSLSessionNotFoundErr;
  }

	CWLDBRef* dbRef = (CWLDBRef*)sessionRef;
	CWLConnectionsPool* cp = dbRef->cp;
	RVHashDictionary* db = (RVHashDictionary*)&cp->m_SslCacheDB;
    
//	ASSERT(db != 0);
  if (!db) {
	  WlTraceMsg("SSL: Get Session key [%d: %s]... NOT found because there is no session database\n", key.length, (const char*)key.data);
		return SSLSessionNotFoundErr;
  }

	RVCString newKey((const char*)key.data, key.length);
	newKey += dbRef->ctx->m_CertificateFile;
	SSLBuffer newKey2;
	newKey2.length = newKey.length();
	newKey2.data = (unsigned char*)newKey.data();


	WlTraceMsg("SSL: Get Session, searching for session key [%d: %s]...\n", key.length, (const char*)key.data);

	WLSSLRecord recKey(key);
	// protect DB while reading a key
	RVReadersWriterLock::ReadLockGuard reader(cp->m_SslLock);
	WLSSLRecord* value = (WLSSLRecord*)(db->findValue(&recKey));
	if (!value) {	// record not found
		WlTraceMsg("SSL: Get session key [%d: %s] not found\n", key.length, (const char*)key.data);
		return SSLSessionNotFoundErr;
	}

	sessionData->length = value->DataLength();
	sessionData->data = (uchar*)CD_malloc(sessionData->length);
  if (!sessionData->data) {
		WlTraceMsg("SSL: Get session key [%d: %s] found but without data\n", key.length, (const char*)key.data);
		return SSLMemoryErr;
  }

	WlTraceMsg("SSL: Get session key [%d: %s] found [%d: %s]\n", key.length, (const char*)key.data, sessionData->length, (const char*)sessionData->data);
	memcpy(sessionData->data, value->DataBuffer(), sessionData->length);

	return SSLNoErr;
}

SSLErr SSLDeleteSession (SSLBuffer key, void *sessionRef)
{
  if (!sessionRef) {
	  WlTraceMsg("SSL: Delete Session key [%d: %s] ... NOT found because there is no session reference\n", key.length, (const char*)key.data);
		return SSLSessionNotFoundErr;
  }

	CWLDBRef* dbRef = (CWLDBRef*)sessionRef;
	CWLConnectionsPool* cp = dbRef->cp;
	RVHashDictionary* db = (RVHashDictionary*)&cp->m_SslCacheDB;
    
//	ASSERT(db != 0);
  if (!db) {
	  WlTraceMsg("SSL: Delete Session key [%d: %s] ... NOT found because there is no session database\n", key.length, (const char*)key.data);
		return SSLSessionNotFoundErr;
  }

	RVCString newKey((const char*)key.data, key.length);
	newKey += dbRef->ctx->m_CertificateFile;
	SSLBuffer newKey2;
	newKey2.length = newKey.length();
	newKey2.data = (unsigned char*)newKey.data();

	WlTraceMsg("SSL: Delete session key [%d: %s]...\n", key.length, (const char*)key.data);

	WLSSLRecord recKey(key);
	// protect DB while deleting a key
	RVReadersWriterLock::WriteLockGuard writer(cp->m_SslLock);
	WLSSLRecord* value = (WLSSLRecord*)db->findValue(&recKey);
	if (!value) {	// record not found
		WlTraceMsg("SSL: Delete session key [%d: %s] not found\n", key.length, (const char*)key.data);
		return SSLSessionNotFoundErr;
	}

	// record found, remove key-value
	db->removeAndDestroy(&recKey);

	return SSLNoErr;
}

/** private functions **/
