/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : Workspace.cpp
 *
 *	Author : Zion Menahem
 *
 *	Description : Based on the HTTPUAR module, this file implements
 *	              the HTTP protocol for JavaScript module.
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

/** include files **/
#include <math.h>
#include "openssl\ssl.h"
#include "rvjsapi.h"
#include <RVCString.h>
#include "tvpnumconversions.h"
#include "httpdefines.h"
#include "commandmanager.h"
#include "workspace.h"
#include "jshttpnames.h"
#include "tracedll.h"

#include "WLSecureSslConnection.h"
#include "WLSslCiphersCollection.h"



/** local definitions **/
#define CONTENT_NAME    "content"

#define WL_NAME_FOR_ONCE		"Once"
#define WL_NAME_FOR_ALWAYS	"Always"
#define WL_NAME_FOR_ALL			"all"
#define WL_NAME_FOR_NONE		"none"
#define WL_NAME_FOR_NO		  "No"

#define IntToString TVPIntToString

/** default settings **/

/** external functions **/
//extern void WLParse_AddParseFunction();

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** public functions **/

/** private functions **/

//class CWLVariables;
//class CWLSslConnection;


#define min(a,b) (((a) < (b)) ? (a) : (b))
#define max(a,b) (((a) > (b)) ? (a) : (b))


//=====================================================
CWLPropIndex CWLWorkspace::m_VarIndex;

CWLHttpPrivateData::CWLHttpPrivateData(CLEVariablesData* Variables)
{
  m_Variables = Variables; 
	m_FormdataBlock = NULL;
	m_HeaderBlock = NULL;
	m_DataBlock = NULL;
	m_DatafileBlock = NULL;
};

CWLHttpPrivateData::~CWLHttpPrivateData()
{
  delete m_Variables; 
	delete m_FormdataBlock;
	delete m_HeaderBlock;
	delete m_DataBlock;
	delete m_DatafileBlock;
};

CWLWorkspace::CWLWorkspace()
{
	lastRoundNum = 0;
	numOfOnce = 1;
	m_Variables = NULL;

	m_Version = NULL;
	m_Url = NULL;
	m_ProxyName = NULL;
  m_HttpsProxyName = NULL;
	m_OutFilename = NULL;
	m_TimerName = NULL;
	m_Username = NULL;
	m_Password = NULL;
	m_ProxyUsername = NULL;
	m_ProxyPassword = NULL;
	m_ProxyNTUsername = NULL;
	m_ProxyNTPassword = NULL;

	m_HttpsProxyUsername = NULL;
	m_HttpsProxyPassword = NULL;
	m_HttpsProxyNTUsername = NULL;
	m_HttpsProxyNTPassword = NULL;

	m_SslVersion = CWLSecureSslConnection::SSL_Version_Undetermined;
	m_SslCryptoStrength = CWLSecureSslConnection::SSL_AllCrypto;

	m_NTUsername = NULL;
	m_NTPassword = NULL;
	m_CertVerification = NULL;
	m_CertificateFile = NULL;
	m_CertificatePassword = NULL;
	m_UserAgent = NULL;
  m_sAcceptLanguage = NULL;
  m_HtmlContentTypes = NULL ;
  m_XmlContentTypes = NULL ;
  m_sReferer = NULL;
  m_sContentType = NULL ;
	m_ResponseContentType = NULL;
	m_RequestContentMultiPartBoundary	= NULL;
  m_sRVProduct = NULL ;

  m_bParseFullBuffer;

  //  Chaim Apr. 2001
  m_ParseOnce = false;
  m_bParseFullBuffer = false;
  m_UseSameProxyForSSL = true;
  m_HttpCacheScope =HTTP_CACHE_SCOPE_ROUND;

  m_pcFilterUrl = "";
}


CWLWorkspace::~CWLWorkspace()
{
//	delete m_Variables;
	m_Variables = NULL;	// the object is deleted by it's creator - the (JS)WLHttp class !
} ;
void CWLWorkspace::Init(CWLHttpPrivateData* HttpVars) 
{
  m_Variables = HttpVars->m_Variables ;
  FormdataBlock = HttpVars->m_FormdataBlock ;
	HeaderBlock = HttpVars->m_HeaderBlock ;
	DataBlock = HttpVars->m_DataBlock ;
	DatafileBlock = HttpVars->m_DatafileBlock ;
};

void CWLWorkspace::ResetVariables() 
{ 
  if (m_Variables) m_Variables->ResetVariablesArray(); 
  if (FormdataBlock) FormdataBlock->clearAndDestroy();
	if (HeaderBlock) HeaderBlock->clearAndDestroy();
	if (DataBlock) DataBlock->clearAndDestroy();
	if (DatafileBlock) DatafileBlock->clearAndDestroy();
}


// This method return the max num of threads for this load
void	CWLWorkspace::ReadMaxThreads(CWLCommandManager* cm)
{
	int TempMaxThreads;

	// Setting max threads for load generator enviroment
	if (cm->IsLoadServer()) {
		const char* val = m_Variables->Get(m_VarIndex.LoadGeneratorThreads).m_Value.StringVal ;

		m_MaxThreads			= MIN_THREADS;
	
		// if the user sets the load engine threads then
		// if it is to multi then we put the default amount of threads in max threads
		// if it is a numeric value then we set max threads to it
		// else we set max threads to minimum value
		if (!strcmpi(val, MULTI_THREADS))
			m_MaxThreads = DEFAULT_NUM_THREADS;
		else {
			TempMaxThreads = atoi(val);
			if (TempMaxThreads > 0) 
				m_MaxThreads = TempMaxThreads;
		};
		return;
	};

	// Setting max threads for probing client enviroment
	if (cm->IsOneClient()) {
		const char* val = m_Variables->Get(m_VarIndex.ProbingClientThreads).m_Value.StringVal ;

		m_MaxThreads = DEFAULT_NUM_THREADS;

		// if the user sets the probing client threads then
		// if it is to single then we put the minimum value in max threads
		// if it is a numeric value then we set max threads to it
		// else we set max threads to default amount of threads
		if (!strcmpi(val, SINGLE_THREAD))
			m_MaxThreads = MIN_THREADS;
		else {
			TempMaxThreads = atoi(val);
			if (TempMaxThreads > 0) 
				m_MaxThreads = TempMaxThreads;
		};
		return;
	};

	// Setting max threads for RTAM enviroment
	if (cm->IsRTAM()) {
		const char* val = m_Variables->Get(m_VarIndex.RTAMThreads).m_Value.StringVal ;
	
		// Insert default values for max threads varieble
		m_MaxThreads		= DEFAULT_NUM_THREADS;

    // if callbacks are enabled, disable multi-threading
    if (cm->IsCallbackEnabled())
      m_MaxThreads = MIN_THREADS;

    // if the user sets the RTAM threads then
		// if it is to single then we put the minimum value in max threads
		// if it is a numeric value then we set max threads to it
		// else we set max threads to default amount of threads
    else if (!strcmpi(val, SINGLE_THREAD))
			m_MaxThreads = MIN_THREADS;
		else {
			TempMaxThreads = atoi(val);
			if (TempMaxThreads > 0) 
				m_MaxThreads = TempMaxThreads;
    } ;
		return;
	};
};
char* CWLWorkspace::TraceAllDataMembers()
{
  //please arrange the variables alphabetically
  WlTraceMsg ("====== Workspace members ===========") ;
  WlTraceMsg ("m_AsmDiagnostics: %d",m_AsmDiagnostics);
  WlTraceMsg ("m_AutoNamedTransaction: %d",m_AutoNamedTrans);
  WlTraceMsg ("m_AsmOn: %d",m_AsmOn);
	WlTraceMsg ("m_bCheckFileContent: %d",m_bCheckFileContent);
	WlTraceMsg ("m_bFoldTransactionsTree: %d",m_bFoldTransactionsTree);
	WlTraceMsg ("m_BrowserCache: %d",m_BrowserCache);
	WlTraceMsg ("m_HttpCacheType: %d",m_HttpCacheType);
	WlTraceMsg ("m_HttpCacheScope: %d",m_HttpCacheScope);
	WlTraceMsg ("m_HttpCacheCachedTypes: %d",m_HttpCacheCachedTypes);
	WlTraceMsg ("m_HttpCacheUncachedTypes: %d",m_HttpCacheUncachedTypes);
	WlTraceMsg ("m_bSaveTransaction: %d",m_bSaveTransaction);
	WlTraceMsg ("m_bSubTransactions: %d",m_bSubTransactions);
	WlTraceMsg ("m_bTimerTransactions: %d",m_bTimerTransactions);
  WlTraceMsg ("m_bVerification: %d",m_bVerification);
	WlTraceMsg ("m_CacheDns: %d",m_CacheDns);
	WlTraceMsg ("m_CacheSsl: %d",m_CacheSsl);
  WlTraceMsg ("m_CertVerification: %s",m_CertVerification);
	WlTraceMsg ("m_CertificateFile: %s",m_CertificateFile);
	WlTraceMsg ("m_CertificatePassword: %s",m_CertificatePassword);
  WlTraceMsg ("m_ClientType: %d",(int)m_ClientType);
  WlTraceMsg ("m_CookiePath: %d",m_CookiePath);
  WlTraceMsg ("m_CookieDomain: %d",m_CookieDomain);
  WlTraceMsg ("m_CookieExpiration: %d",m_CookieExpiration);
  WlTraceMsg ("m_cTransactionTime: %s",m_cTransactionTime);
	WlTraceMsg ("m_EncodeFormdata: %d",m_EncodeFormdata);
	WlTraceMsg ("m_DeleteEmptyCookies: %d",m_DeleteEmptyCookies);
	WlTraceMsg ("m_EscapeChar: %s",m_EscapeChar);
	WlTraceMsg ("m_LowLevelUnescape: %d",m_LowLevelUnescape);
	WlTraceMsg ("m_DecodeBinaryNullAs: %s",m_DecodeBinaryNullAs);
	WlTraceMsg ("m_DecodeBinaryStart: %d",m_DecodeBinaryStart);
	WlTraceMsg ("m_DecodeBinaryEnd: %d",m_DecodeBinaryEnd);
	WlTraceMsg ("m_EncodeData: %d",m_EncodeData);
  WlTraceMsg ("m_ExpectedUrl: %s",m_ExpectedUrl);
  WlTraceMsg ("m_GetFrames: %d",m_GetFrames);
	WlTraceMsg ("m_GetImages: %d",m_GetImages);
  WlTraceMsg ("m_GetEmbeds: %d",m_GetEmbeds);
  WlTraceMsg ("m_GetApplets: %d",m_GetApplets);
  WlTraceMsg ("m_GetCss: %d",m_GetCss);
  WlTraceMsg ("m_GetXmls: %d",m_GetXmls);
  WlTraceMsg ("m_GetScripts: %d",m_GetScripts);
  WlTraceMsg ("m_GetOthers: %d",m_GetOthers);
	WlTraceMsg ("m_HtmlContentTypes: %s",m_HtmlContentTypes);
	WlTraceMsg ("m_MaxThreads: %d", m_MaxThreads ) ;
	WlTraceMsg ("m_MultiIP: %d",m_MultiIP);
	WlTraceMsg ("m_MultiIPType: %s",m_MultiIPType);
  WlTraceMsg ("m_nTimeoutSeverity: %d",m_nTimeoutSeverity);
	WlTraceMsg ("m_NTUsername: %s",m_NTUsername);
	WlTraceMsg ("m_NTPassword: %s",m_NTPassword);
  WlTraceMsg ("m_nVerSetMaxThreads: %d",m_nVerSetMaxThreads);
  WlTraceMsg ("m_nVerSetMaxRounds: %d",m_nVerSetMaxRounds);
  WlTraceMsg ("m_nVerSetRounds: %d",m_nVerSetRounds);
  WlTraceMsg ("m_OutFilename: %s",m_OutFilename);
	WlTraceMsg ("m_Password: %s",m_Password);
  WlTraceMsg ("m_ParseLinks: %d",m_ParseLinks);
  WlTraceMsg ("m_ParseForms: %d",m_ParseForms);
  WlTraceMsg ("m_ParseScripts: %d",m_ParseScripts);
  WlTraceMsg ("m_ParseTables: %d",m_ParseTables);
	WlTraceMsg ("m_ParseXml: %d",m_ParseXml);
	WlTraceMsg ("m_ParseMetas: %d",m_ParseMetas);
	WlTraceMsg ("m_ParseImages: %d",m_ParseImages);
	WlTraceMsg ("m_ParseEmbeds: %d",m_ParseEmbeds);
	WlTraceMsg ("m_ParseApplets: %d",m_ParseApplets);
	WlTraceMsg ("m_ParseCss: %d",m_ParseCss);
	WlTraceMsg ("m_bParseFullBuffer: %d",m_bParseFullBuffer);
	WlTraceMsg ("m_ParseOthers: %d",m_ParseOthers);
  WlTraceMsg ("m_ParseOnce: %d", m_ParseOnce);  
  WlTraceMsg ("m_ProxyName: %s", m_ProxyName);
  WlTraceMsg ("m_UseSameProxyForSSL: %d", m_UseSameProxyForSSL);
  WlTraceMsg ("m_HttpsProxyName: %s", m_HttpsProxyName);
	WlTraceMsg ("m_ProxyUsername: %s",m_ProxyUsername);
	WlTraceMsg ("m_ProxyPassword: %s",m_ProxyPassword);
	WlTraceMsg ("m_ProxyNTUsername: %s",m_ProxyNTUsername);
	WlTraceMsg ("m_ProxyNTPassword: %s",m_ProxyNTPassword);
 	WlTraceMsg ("m_HttpsProxyUsername: %s",m_HttpsProxyUsername);
	WlTraceMsg ("m_HttpsProxyPassword: %s",m_HttpsProxyPassword);
	WlTraceMsg ("m_HttpsProxyNTUsername: %s",m_HttpsProxyNTUsername);
	WlTraceMsg ("m_HttpsProxyNTPassword: %s",m_HttpsProxyNTPassword);
	WlTraceMsg ("m_ReceiveTimeout: %d", m_ReceiveTimeout);
	WlTraceMsg ("m_RecvBufferSize: %d", m_RecvBufferSize);
	WlTraceMsg ("m_SendBufferSize: %d", m_SendBufferSize);
	WlTraceMsg ("m_RequestMultiPartBufferSize: %d",m_RequestMultiPartBufferSize);
	WlTraceMsg ("m_RedirectionLimit: %d",m_RedirectionLimit);
	WlTraceMsg ("m_RedirectionEnabled: %d",m_RedirectionEnabled);
  WlTraceMsg ("m_bReportHttpErrors: %d", m_bReportHttpErrors);
  WlTraceMsg ("m_RequestRetries: %d",m_RequestRetries);
	WlTraceMsg ("m_SaveHeaders: %d",m_SaveHeaders);
	WlTraceMsg ("m_SaveSource: %d",m_SaveSource);
  WlTraceMsg ("m_sContentType: %s", m_sContentType);  
	WlTraceMsg ("m_SlowConnectionRate: %u",m_SlowConnectionRate);
  WlTraceMsg ("m_sReferer: %s",m_sReferer);
  WlTraceMsg ("m_sRVProduct: %s",m_sRVProduct);
  WlTraceMsg ("m_SslVersion: %d",m_SslVersion);
  WlTraceMsg ("m_SslBitLimit: %u",m_SslBitLimit);
  WlTraceMsg ("m_Target: %s",m_Target);
  WlTraceMsg ("m_TimerName: %s",m_TimerName);
	WlTraceMsg ("m_Url: %s", m_Url) ;
	WlTraceMsg ("m_UseKeepAlive: %d",m_UseKeepAlive);
	WlTraceMsg ("m_Persistancy: %d",m_Persistancy);
	WlTraceMsg ("m_Username: %s",m_Username);
	WlTraceMsg ("m_UserAgent: %s",m_UserAgent);
  WlTraceMsg ("m_AcceptLanguage: %s", m_sAcceptLanguage);
  WlTraceMsg ("m_Version: %s", m_Version) ;
	WlTraceMsg ("m_VerifyXml: %d",m_VerifyXml);
	WlTraceMsg ("m_XmlContentTypes: %s",m_XmlContentTypes);
	WlTraceMsg ("m_ResponseContentType: %s",m_ResponseContentType);
	WlTraceMsg ("m_RequestContentMultiPartBoundary: %s",m_RequestContentMultiPartBoundary);
  WlTraceMsg ("m_XmlStandAlone: %d",m_XmlStandAlone);

  //WlTraceMsg ("lastRoundNum: %u",lastRoundNum);
  //WlTraceMsg ("numOfOnce: %d",numOfOnce);
	WlTraceMsg ("doParse: %d",doParse);
  WlTraceMsg ("isParse: %d",isParse) ;
  WlTraceMsg ("pageName: %s", pageName.data());
  WlTraceMsg ("lastPageName: %s",lastPageName.data());
	WlTraceMsg ("m_EncodeFormdata: %d",m_EncodeFormdata);
	WlTraceMsg ("m_DeleteEmptyCookies: %d",m_DeleteEmptyCookies);
	WlTraceMsg ("m_EscapeChar: %s",m_EscapeChar);
	WlTraceMsg ("m_LowLevelUnescape: %d",m_LowLevelUnescape);
	WlTraceMsg ("m_DecodeBinaryNullAs: %s",m_DecodeBinaryNullAs);
	WlTraceMsg ("m_DecodeBinaryStart: %d",m_DecodeBinaryStart);
	WlTraceMsg ("m_DecodeBinaryEnd: %d",m_DecodeBinaryEnd);
	WlTraceMsg ("m_EncodeData: %d",m_EncodeData);
	WlTraceMsg ("m_HtmlContentTypes: %s",m_HtmlContentTypes);
	WlTraceMsg ("m_XmlContentTypes: %s",m_XmlContentTypes);

	WlTraceMsg ("m_ConnectTimeout: %d",m_ConnectTimeout);
	WlTraceMsg ("m_SelectWriteTimeout: %d",m_SelectWriteTimeout);
	WlTraceMsg ("m_SelectWriteSecondTimeout: %d",m_SelectWriteSecondTimeout);
	WlTraceMsg ("m_SelectWriteSwitchNum: %d",m_SelectWriteSwitchNum);
	WlTraceMsg ("m_SelectTimeout: %d",m_SelectTimeout);
	WlTraceMsg ("m_SelectSecondTimeout: %d",m_SelectSecondTimeout);
	WlTraceMsg ("m_SelectSwitchNum: %d",m_SelectSwitchNum);

	WlTraceMsg ("m_OverrideHostDefault: %d",m_OverrideCookieDefault);
	WlTraceMsg ("m_OverrideCookieDefault: %d",m_OverrideCookieDefault);
	WlTraceMsg ("m_OverrideIfModifiedSinceDefault: %d",m_OverrideIfModifiedSinceDefault);

	WlTraceMsg ("m_bFoldTransactionsTree: %d",m_bFoldTransactionsTree);
    WlTraceMsg ("m_bAcceptEncodingGzip: %s", m_bAcceptEncodingGzip ? "yes" : "no" );

  WlTraceMsg ("m_pcFilterUrl: %s",m_pcFilterUrl);

  //Galtech
#ifndef _ANSI_ONLY
    WlTraceMsg ("m_bEnforceGlobalCharEncoding: %s", m_bEnforceGlobalCharEncoding ? "yes" : "no" );
#endif

 
  return "========== End workspace members ============" ;
} ;
void CWLWorkspace::ReadData(CWLCommandManager* cm)
{
  //unsigned long roundNum = cm->CurrentScriptRoundNum();

	// Get HTTP version - default is 1.0
  m_Version = m_Variables->Get(m_VarIndex.Version).m_Value.StringVal ;

	// Get the url value 
	m_Url = m_Variables->Get(m_VarIndex.Url).m_Value.StringVal;

	// Read the max num of threads for this load
	ReadMaxThreads (cm);

  //Galtech
// @@Chaim - remove these (CharEncoding)
#ifndef _ANSI_ONLY
//	m_CharSet = m_Variables->Get(m_VarIndex.CharSet).m_Value.LongVal ;
//	m_nDomCharset = m_Variables->Get(m_VarIndex.DomCharset).m_Value.LongVal ;
	m_bEnforceGlobalCharEncoding = m_Variables->Get(m_VarIndex.EnforceCharEncoding).m_Value.BoolVal;
#endif


	// use a slow connection ? (simulate a modem) - get the baud rate
	m_SlowConnectionRate = m_Variables->Get(m_VarIndex.ConnectionSpeed).m_Value.LongVal ;
  m_SlowConnectionRate = m_SlowConnectionRate > 0 ? m_SlowConnectionRate : 0 ; 

	// Check the DNS caching
	m_CacheDns = m_Variables->Get(m_VarIndex.DNSUseCache).m_Value.BoolVal ;
	m_MultiIP = m_Variables->Get(m_VarIndex.MultiIPSupport).m_Value.BoolVal ;
	if (!strcmp(m_Variables->Get(m_VarIndex.MultiIPSupportType).m_Value.StringVal, WLJSNAME_HTTP_MULTI_IP_SUPPORT_PER_CLIENT)){
		m_MultiIPType = CWLWorkspace::PerClient;
	}
	else {
		m_MultiIPType = CWLWorkspace::PerRound;
	}

    m_bParseFullBuffer = m_Variables->Get(m_VarIndex.ParseFullBuffer).m_Value.BoolVal ;

	// Check the SSL caching
	m_CacheSsl = m_Variables->Get(m_VarIndex.SSLUseCache).m_Value.BoolVal ;

	// Determine if to use proxy.
	m_ProxyName = m_Variables->Get(m_VarIndex.Proxy).m_Value.StringVal ;

	m_UseSameProxyForSSL = m_Variables->Get(m_VarIndex.UseSameProxyForSSL).m_Value.BoolVal ;

	// Determine if to use a timer.
	m_TimerName = m_Variables->Get(m_VarIndex.UsingTimer).m_Value.StringVal ;

	// Determine if to use a file to save the page.
	m_OutFilename = m_Variables->Get(m_VarIndex.Outfile).m_Value.StringVal ;

	// Authentication variables
	m_Username = m_Variables->Get(m_VarIndex.UserName).m_Value.StringVal ;
	m_Password = m_Variables->Get(m_VarIndex.PassWord).m_Value.StringVal ;
	m_ProxyUsername = m_Variables->Get(m_VarIndex.ProxyUserName).m_Value.StringVal ;
	m_ProxyPassword = m_Variables->Get(m_VarIndex.ProxyPassWord).m_Value.StringVal ;
	m_ProxyNTUsername = m_Variables->Get(m_VarIndex.ProxyNTUserName).m_Value.StringVal ;
	m_ProxyNTPassword = m_Variables->Get(m_VarIndex.ProxyNTPassWord).m_Value.StringVal ;

	m_HttpsProxyUsername = m_Variables->Get(m_VarIndex.HttpsProxyUserName).m_Value.StringVal ;
	m_HttpsProxyPassword = m_Variables->Get(m_VarIndex.HttpsProxyPassWord).m_Value.StringVal ;
	m_HttpsProxyNTUsername = m_Variables->Get(m_VarIndex.HttpsProxyNTUserName).m_Value.StringVal ;
	m_HttpsProxyNTPassword = m_Variables->Get(m_VarIndex.HttpsProxyNTPassWord).m_Value.StringVal ;

	m_NTUsername = m_Variables->Get(m_VarIndex.NTUserName).m_Value.StringVal ;
	m_NTPassword = m_Variables->Get(m_VarIndex.NTPassWord).m_Value.StringVal ;

  m_bCheckFileContent = m_Variables->Get(m_VarIndex.CheckFileContent).m_Value.BoolVal ; 

#ifdef __OPEN_SSL__
	// Determine the SSL protocol version.
	const char* sslver = m_Variables->Get(m_VarIndex.SSLVersion).m_Value.StringVal ;
	// @@Chaim - temp
	m_SslVersion = CWLSecureSslConnection::SSL_Version_3_0_With_2_0_Hello;
	m_SslVersion = CWLSecureSslConnection::DetermineVersion(sslver);

	// Determine the SSL crypto strength.
  const char* sslCryptoStrength = m_Variables->Get(m_VarIndex.SSLCryptoStrength).m_Value.StringVal ;
  m_SslCryptoStrength = CWLSecureSslConnection::SSL_AllCrypto;
	m_SslCryptoStrength = CWLSecureSslConnection::DetermineCryptographicStrength(sslCryptoStrength);

	m_CertVerification = SSL_CERT_NO_VERIFICATION; //stav

  m_CertificateFile = m_Variables->Get(m_VarIndex.SSLClientCertificateFile).m_Value.StringVal ;
	m_CertificatePassword = m_Variables->Get(m_VarIndex.SSLClientCertificatePassword).m_Value.StringVal ;

	m_SslBitLimit = DEFAULT_ENCRYPTION_STRENGTH;
	const char* limit = m_Variables->Get(m_VarIndex.SSLBitLimit).m_Value.StringVal ;
	if (limit) {
		if (!stricmp(limit, "none"))
			m_SslBitLimit = DEFAULT_ENCRYPTION_STRENGTH;
		else if (sscanf(limit, "%u", &m_SslBitLimit) != 1)
			m_SslBitLimit = DEFAULT_ENCRYPTION_STRENGTH;
	};
#else
	m_SslVersion = 0;
	m_SslCryptoStrength = 0;
	m_CertVerification = "";
	m_CertificateFile = NULL;
	m_CertificatePassword = NULL;
	m_SslBitLimit = 0;
#endif

	// get the pointers to the blocks

	// get the keep-alive value
	m_UseKeepAlive = m_Variables->Get(m_VarIndex.KeepAlive).m_Value.BoolVal ;

	// get the persistancy value
	m_Persistancy = m_Variables->Get(m_VarIndex.Persistancy).m_Value.LongVal ;

  // get from the HTTP all parsing information.
	m_SaveSource = m_Variables->Get(m_VarIndex.SaveSource).m_Value.BoolVal ;

  m_CovertToUTF8ForParse = m_Variables->Get(m_VarIndex.CovertToUTF8ForParse).m_Value.BoolVal ;

  // Chaim Apr. 2001 - Support for SaveHeaders
  m_SaveHeaders = m_Variables->Get(m_VarIndex.SaveHeaders).m_Value.BoolVal ;


  if (!pageName.isNull())
    pageName.remove(0);

  // ClientType
  const CLEPropertyValue & ClientTypeVar = m_Variables->Get(m_VarIndex.ClientType);
  const char* cClientType = ClientTypeVar.m_Value.StringVal;
  if (!strcmpi(cClientType, WL_CLIENT_TYPE_THIN))
    m_ClientType = ENUM_CLIENT_TYPE_THIN;
  else if (!strcmpi(cClientType, WL_CLIENT_TYPE_THICK))
    m_ClientType = ENUM_CLIENT_TYPE_THICK;
  else if (!strcmpi(cClientType, WL_CLIENT_TYPE_CUSTOM))
    m_ClientType = ENUM_CLIENT_TYPE_CUSTOM;
  else  // the default is NORMAL
    m_ClientType = ENUM_CLIENT_TYPE_NORMAL;


  /*
	A complete REDO of the GetContents flags.
	First, read the value of the individual flags (the new ones)
	After that read the value of the old, existing ones and modify the
	individual flags accordingly (for bakward comp.)
	Uri 5/10/2000
  */
  
  m_GetFrames  = m_Variables->Get(m_VarIndex.GetFrames).m_Value.BoolVal;
  m_GetImages  = m_Variables->Get(m_VarIndex.GetImages).m_Value.BoolVal;
  m_GetEmbeds  = m_Variables->Get(m_VarIndex.GetEmbeds).m_Value.BoolVal;
  m_GetApplets = m_Variables->Get(m_VarIndex.GetApplets).m_Value.BoolVal;
  m_GetScripts = m_Variables->Get(m_VarIndex.GetScripts).m_Value.BoolVal;
  m_GetCss     = m_Variables->Get(m_VarIndex.GetCss).m_Value.BoolVal;
  m_GetXmls    = m_Variables->Get(m_VarIndex.GetXml).m_Value.BoolVal;
  m_GetOthers  = m_Variables->Get(m_VarIndex.GetOthers).m_Value.BoolVal;


  m_VerifyXml = m_Variables->Get(m_VarIndex.VerifyXML).m_Value.BoolVal ;
  m_XmlStandAlone = m_Variables->Get(m_VarIndex.XMLStandAlone).m_Value.BoolVal ;

  //Parse flags
  const CLEPropertyValue & ParseVar = m_Variables->Get(m_VarIndex.Parse);
  const char* Parse = ParseVar.m_Value.StringVal;
  isParse = doParse = CWLUserParseFlags::DontParse;
  if (!strcmpi(Parse, WL_NAME_FOR_NO)) {
		// in order to have headers without parsing we need the page name in any case
		pageName = WL_NAME_FOR_NONE;
		lastPageName = pageName;
  }
  else { //If parse is not "No" than it will be "Always" (no more "Once")
    doParse = CWLUserParseFlags::ParseAlways;
    pageName = WL_NAME_FOR_ALL;
    lastPageName = pageName;
  }

  
  const CLEPropertyValue & ParseLinks = m_Variables->Get(m_VarIndex.ParseLinks);
  const CLEPropertyValue & ParseForms = m_Variables->Get(m_VarIndex.ParseForms);
  const CLEPropertyValue & ParseScripts = m_Variables->Get(m_VarIndex.ParseScripts);
  const CLEPropertyValue & ParseTables = m_Variables->Get(m_VarIndex.ParseTables) ;
  const CLEPropertyValue & ParseXml = m_Variables->Get(m_VarIndex.ParseXML);
  const CLEPropertyValue & ParseMetas = m_Variables->Get(m_VarIndex.ParseMetas);
  const CLEPropertyValue & ParseImages = m_Variables->Get(m_VarIndex.ParseImages);
  const CLEPropertyValue & ParseEmbeds = m_Variables->Get(m_VarIndex.ParseEmbeds);
  const CLEPropertyValue & ParseApplets = m_Variables->Get(m_VarIndex.ParseApplets);
  const CLEPropertyValue & ParseCss = m_Variables->Get(m_VarIndex.ParseCss);
  const CLEPropertyValue & ParseOthers = m_Variables->Get(m_VarIndex.ParseOthers);

  // Chaim - ParseOnce
  const CLEPropertyValue & ParseOnce = m_Variables->Get(m_VarIndex.ParseOnce);

  m_ParseLinks = ParseLinks.m_Value.BoolVal ;
  m_ParseForms = ParseForms.m_Value.BoolVal ;
  m_ParseScripts = ParseScripts.m_Value.BoolVal ;
  m_ParseTables = ParseTables.m_Value.BoolVal ;
  m_ParseXml = ParseXml.m_Value.BoolVal ;
  m_ParseMetas = ParseMetas.m_Value.BoolVal ;
  m_ParseImages = ParseImages.m_Value.BoolVal ;
  m_ParseEmbeds = ParseEmbeds.m_Value.BoolVal ;
  m_ParseApplets = ParseApplets.m_Value.BoolVal ;
  m_ParseCss = ParseCss.m_Value.BoolVal ;
  m_ParseOthers = ParseOthers.m_Value.BoolVal ;

  // Chaim - ParseOnce
  m_ParseOnce = ParseOnce.m_Value.BoolVal ;

  //When Asm is on parse must be set on. Also ParseLinks, ParseForms & ParseMetas
  m_AsmOn = m_Variables->Get(m_VarIndex.AsmOn).m_Value.BoolVal ;
  if (m_AsmOn) {

    //Is the Parse flag was set to "no" by the user then display a warning message
    if ((ParseVar.m_IsSet) && (doParse == CWLUserParseFlags::DontParse))
    {
			char message[1024];
			WLERRGetErrorString( HTTP_ERROR_FILE, 306, message, 1024 );
			cm->reportMultiThread( WLERRGetErrorSeverity( HTTP_ERROR_FILE, 306 ), message );
    }

    doParse = CWLUserParseFlags::ParseAlways ;
    m_ParseOnce  = false;   // Chaim - For ASM, ParseOnce deactivated.
    m_ParseLinks = true;
    m_ParseForms = true;
    m_ParseMetas = true;
    m_GetFrames  = true;

  };

  //If parse was not set explicitly to no then if 
  //one of the parse flags was set explicitly then set Parse to true
  if (doParse == CWLUserParseFlags::DontParse && !ParseVar.m_IsSet) {
      if (  
      ( ParseLinks.m_Value.BoolVal && ParseLinks.m_IsSet)     ||
		  ( ParseForms.m_Value.BoolVal && ParseForms.m_IsSet)     ||
		  ( ParseScripts.m_Value.BoolVal && ParseScripts.m_IsSet) ||
		  ( ParseTables.m_Value.BoolVal && ParseTables.m_IsSet)   ||
		  ( ParseXml.m_Value.BoolVal && ParseXml.m_IsSet)         ||
		  ( ParseMetas.m_Value.BoolVal && ParseMetas.m_IsSet)     ||
		  ( ParseImages.m_Value.BoolVal && ParseImages.m_IsSet)   ||
		  ( ParseEmbeds.m_Value.BoolVal && ParseEmbeds.m_IsSet)   ||
		  ( ParseApplets.m_Value.BoolVal && ParseApplets.m_IsSet) ||
		  ( ParseCss.m_Value.BoolVal && ParseCss.m_IsSet)         ||
		  ( ParseOthers.m_Value.BoolVal && ParseOthers.m_IsSet)    )
		  doParse = CWLUserParseFlags::ParseAlways ; 
  }

  // make sure that the appropriate parsing is turned on for the various Get*
  if(m_GetImages)
    m_ParseImages = true;
  if(m_GetEmbeds)
    m_ParseEmbeds = true;
  if(m_GetApplets)
    m_ParseApplets = true;
  if(m_GetScripts)
    m_ParseScripts = true;
  if(m_GetCss)
    m_ParseCss = true;
  if(m_GetXmls)
    m_ParseXml = true;

  if( m_GetFrames||m_GetImages||m_GetEmbeds||m_GetApplets||m_GetScripts||m_GetCss||m_GetXmls )
    doParse = CWLUserParseFlags::ParseAlways ;

  //Set parse flags only if parse is Once or Always
  if (doParse == CWLUserParseFlags::DontParse ) {
    m_ParseLinks = false ;
    m_ParseForms = false ;
    m_ParseScripts = false ;
    m_ParseTables = false ;
    m_ParseXml = false ;
    m_ParseMetas = false ;
    m_ParseImages = false ;
    m_ParseEmbeds = false ;
    m_ParseApplets = false ;
    m_ParseCss = false ;
    m_ParseOthers = false ;
  }

	isParse = doParse;

	// Set value to user agent if there is so
  m_UserAgent = m_Variables->Get(m_VarIndex.UserAgent).m_Value.StringVal ;

	// if user said specificlly that he doesn't want browser cache
	// then we disable it otherwise we enable it
  m_BrowserCache = m_Variables->Get(m_VarIndex.BrowserCache).m_Value.BoolVal ;

	// defaults should be as follows (preserving the old cache behavior):
	// HttpCacheType: None
	// HttpCacheScope: Single command
	// HttpCacheUncachedTypes: HTML, Dynamic (relevant only when HttpCacheType != "None")
	const char* psTemp = m_Variables->Get(m_VarIndex.HttpCacheType).m_Value.StringVal;
	m_HttpCacheType = HTTP_CACHE_TYPE_NONE;
	if ( psTemp && *psTemp )
	{
		if ( !strcmp( psTemp, "None" ) )
			m_HttpCacheType = HTTP_CACHE_TYPE_NONE;
		else if ( !strcmp( psTemp, "FreshOnly" ) )
			m_HttpCacheType = HTTP_CACHE_TYPE_FRESH;
		else if ( !strcmp( psTemp, "Heuristics" ) )
			m_HttpCacheType = HTTP_CACHE_TYPE_HEURISTICS;
	}

	// using new cache impose using HTTP/1.1
	if ( m_HttpCacheType != HTTP_CACHE_TYPE_NONE )
		m_Version = HTTP_VERSION_11;

	psTemp = m_Variables->Get(m_VarIndex.HttpCacheScope).m_Value.StringVal;
	m_HttpCacheScope = HTTP_CACHE_SCOPE_COMMAND;
	if ( psTemp && *psTemp )
	{
		if ( !strcmp( psTemp, "SingleCommand" ) )
			m_HttpCacheScope = HTTP_CACHE_SCOPE_COMMAND;
		else if ( !strcmp( psTemp, "SingleRound" ) )
			m_HttpCacheScope = HTTP_CACHE_SCOPE_ROUND;
		else if ( !strcmp( psTemp, "WholeRun" ) )
			m_HttpCacheScope = HTTP_CACHE_SCOPE_RUN;
	}

	// assume caching all resource and remove those who should no be cached
	psTemp = m_Variables->Get(m_VarIndex.HttpCacheUncachedTypes).m_Value.StringVal;
  m_HttpCacheCachedTypes = HTTP_CACHE_CACHE_ALL;
	if ( psTemp && *psTemp )
	{
		// if 'None' exist in the value string, it runs over the rest of the value
		if ( strstr ( psTemp, "None" ) )
			m_HttpCacheCachedTypes = HTTP_CACHE_CACHE_ALL;
		else
		{
			if ( strstr( psTemp, "HTML" ) )
				m_HttpCacheCachedTypes ^= HTTP_CACHE_CACHE_HTML;
			if ( strstr( psTemp, "JS" ) )
				m_HttpCacheCachedTypes ^= HTTP_CACHE_CACHE_JS;
			if ( strstr( psTemp, "CSS" ) )
				m_HttpCacheCachedTypes ^= HTTP_CACHE_CACHE_CSS;
			if ( strstr( psTemp, "Image" ) )
				m_HttpCacheCachedTypes ^= HTTP_CACHE_CACHE_IMAGE;
			if ( strstr( psTemp, "Applet" ) )
				m_HttpCacheCachedTypes ^= HTTP_CACHE_CACHE_APPLET;
			if ( strstr( psTemp, "XML" ) )
				m_HttpCacheCachedTypes ^= HTTP_CACHE_CACHE_XML;
			if ( strstr( psTemp, "Dynamic" ) )
				m_HttpCacheCachedTypes ^= HTTP_CACHE_CACHE_DYNAMIC;

			if ( m_HttpCacheCachedTypes == HTTP_CACHE_CACHE_ALL )
			{
				// no valid value is given, use default
				m_HttpCacheCachedTypes ^= HTTP_CACHE_CACHE_HTML; // default value - not caching HTMLs
				m_HttpCacheCachedTypes ^= HTTP_CACHE_CACHE_DYNAMIC; // default value - not caching dynamic (query strings) URLs
			}
		}
	}
	else
	{
		m_HttpCacheCachedTypes ^= HTTP_CACHE_CACHE_HTML; // default value - not caching HTMLs
		m_HttpCacheCachedTypes ^= HTTP_CACHE_CACHE_DYNAMIC; // default value - not caching dynamic (query strings) URLs
	}

  m_RedirectionEnabled = m_Variables->Get(m_VarIndex.RedirectionEnabled).m_Value.BoolVal ;
  m_RedirectionLimit = m_Variables->Get(m_VarIndex.RedirectionLimit).m_Value.LongVal ;
  m_RedirectionLimit = m_RedirectionLimit > MIN_REDIRECTION_LIMIT ? m_RedirectionLimit : MIN_REDIRECTION_LIMIT ; 

	// Get the value for checking cookie path or not - the default is
  // to check the path
  m_CookiePath = m_Variables->Get(m_VarIndex.UNDOC_CookiePath).m_Value.BoolVal & m_Variables->Get(m_VarIndex.CookiePath).m_Value.BoolVal;

  // Get the value for checking cookie path or not - the default is
  // to check the domain
  m_CookieDomain = m_Variables->Get(m_VarIndex.UNDOC_CookieDomain).m_Value.BoolVal & m_Variables->Get(m_VarIndex.CookieDomain).m_Value.BoolVal;

	// Get the value for checking cookie expiration dates or not - the default is
  // not th check this expiration
  m_CookieExpiration = m_Variables->Get(m_VarIndex.UNDOC_CookieExpiration).m_Value.BoolVal | m_Variables->Get(m_VarIndex.CookieExpiration).m_Value.BoolVal;

  m_Target = m_Variables->Get(m_VarIndex.Target).m_Value.StringVal ;

  m_AutoNamedTrans = m_Variables->Get(m_VarIndex.AutoNamedTrans).m_Value.BoolVal ;

  m_ExpectedUrl = m_Variables->Get(m_VarIndex.ExpectedUrl).m_Value.StringVal ;

  m_AsmDiagnostics = m_Variables->Get(m_VarIndex.AsmDiagnostics).m_Value.BoolVal ;

  //transactions
  m_bSubTransactions = m_Variables->Get(m_VarIndex.SubTransactions).m_Value.BoolVal ;
  m_bTimerTransactions = m_Variables->Get(m_VarIndex.TimerTransactions).m_Value.BoolVal ;
  m_cTransactionTime = m_Variables->Get(m_VarIndex.TransactionTime).m_Value.StringVal ;
  m_nTimeoutSeverity = m_Variables->Get(m_VarIndex.TimeoutSeverity).m_Value.LongVal ;
  m_bSaveTransaction = m_Variables->Get(m_VarIndex.SaveTransaction).m_Value.BoolVal ;

  m_bVerification = m_Variables->Get(m_VarIndex.Verification).m_Value.BoolVal ;
	m_nVerSetMaxThreads = m_Variables->Get(m_VarIndex.VerSetMaxThreads).m_Value.LongVal ;
	m_nVerSetMaxRounds = m_Variables->Get(m_VarIndex.VerSetMaxRounds).m_Value.LongVal ;
	m_nVerSetRounds = m_Variables->Get(m_VarIndex.VerSetRounds).m_Value.LongVal ;

  m_RequestRetries = m_Variables->Get(m_VarIndex.RequestRetries).m_Value.LongVal ;

  m_ReceiveTimeout = m_Variables->Get(m_VarIndex.ReceiveTimeout).m_Value.LongVal ;

  m_EncodeFormdata = m_Variables->Get(m_VarIndex.EncodeFormdata).m_Value.BoolVal ;
  m_DeleteEmptyCookies = m_Variables->Get(m_VarIndex.DeleteEmptyCookies).m_Value.BoolVal ;
  m_EscapeChar = m_Variables->Get(m_VarIndex.EscapeChar).m_Value.StringVal ;
  m_LowLevelUnescape = m_Variables->Get(m_VarIndex.LowLevelUnescape).m_Value.BoolVal ;
  m_DecodeBinaryNullAs = m_Variables->Get(m_VarIndex.DecodeBinaryNullAs).m_Value.StringVal ;
  m_DecodeBinaryStart = m_Variables->Get(m_VarIndex.DecodeBinaryStart).m_Value.LongVal ;
  m_DecodeBinaryEnd = m_Variables->Get(m_VarIndex.DecodeBinaryEnd).m_Value.LongVal ;
  m_EncodeData = m_Variables->Get(m_VarIndex.EncodeData).m_Value.BoolVal ;
  m_ConnectTimeout = m_Variables->Get(m_VarIndex.ConnectTimeout).m_Value.LongVal ;
  m_ConnectTimeout = max(m_ConnectTimeout, MIN_CONNECT_TIMEOUT) ;

  m_SelectWriteTimeout = m_Variables->Get(m_VarIndex.SelectWriteTimeout).m_Value.LongVal ;
  m_SelectWriteTimeout = max(m_SelectWriteTimeout, MIN_SELECT_WRITE_TIMEOUT) ;

  m_SelectWriteSecondTimeout = m_Variables->Get(m_VarIndex.SelectWriteSecondTimeout).m_Value.LongVal ;
  m_SelectWriteSecondTimeout = max(m_SelectWriteSecondTimeout, MIN_SELECT_WRITE_SECOND_TIMEOUT) ;

  m_SelectWriteSwitchNum = m_Variables->Get(m_VarIndex.SelectWriteSwitchNum).m_Value.LongVal ;
  m_SelectWriteSwitchNum = max(m_SelectWriteSwitchNum, MIN_SELECT_WRITE_SWITCH_NUM) ;

  m_SelectTimeout = m_Variables->Get(m_VarIndex.SelectTimeout).m_Value.LongVal ;
  m_SelectTimeout = max(m_SelectTimeout, MIN_SELECT_TIMEOUT) ;

  m_SelectSecondTimeout = m_Variables->Get(m_VarIndex.SelectSecondTimeout).m_Value.LongVal ;
  m_SelectSecondTimeout = max(m_SelectSecondTimeout, MIN_SELECT_SECOND_TIMEOUT) ;

  m_SelectSwitchNum = m_Variables->Get(m_VarIndex.SelectSwitchNum).m_Value.LongVal ;
  m_SelectSwitchNum = max(m_SelectSwitchNum, MIN_SELECT_SWITCH_NUM) ;

  m_RecvBufferSize = m_Variables->Get(m_VarIndex.RecvBufferSize).m_Value.LongVal ;
  m_RecvBufferSize = max(m_RecvBufferSize, MIN_BUFFER_SIZE) ;

  m_SendBufferSize = m_Variables->Get( m_VarIndex.SendBufferSize ).m_Value.LongVal ;
  //m_SendBufferSize = max( m_SendBufferSize, MIN_SEND_BUFFER_SIZE ) ;
	
  m_RequestMultiPartBufferSize = m_Variables->Get(m_VarIndex.RequestMultiPartBufferSize).m_Value.LongVal ;
  m_RequestMultiPartBufferSize = max(m_RequestMultiPartBufferSize, MIN_REQUEST_MULTI_PART_BUFFER_SIZE) ;
	
  m_bFoldTransactionsTree = m_Variables->Get(m_VarIndex.FoldTransactionsTree).m_Value.BoolVal ;

  m_HtmlContentTypes = m_Variables->Get(m_VarIndex.HtmlContentTypes).m_Value.StringVal ;
  m_XmlContentTypes  = m_Variables->Get(m_VarIndex.XmlContentTypes).m_Value.StringVal ;
	m_ResponseContentType = m_Variables->Get(m_VarIndex.ResponseContentType).m_Value.StringVal ;
	m_RequestContentMultiPartBoundary = m_Variables->Get(m_VarIndex.RequestContentMultiPartBoundary).m_Value.StringVal ;
  m_sReferer  = m_Variables->Get(m_VarIndex.Referer).m_Value.StringVal ;
  m_sContentType = m_Variables->Get(m_VarIndex.ContentType).m_Value.StringVal ;
  m_bReportHttpErrors = m_Variables->Get(m_VarIndex.ReportHttpErrors).m_Value.BoolVal ;
  m_sRVProduct = m_Variables->Get(m_VarIndex.RVProduct).m_Value.StringVal ;

	m_OverrideHostDefault = m_Variables->Get(m_VarIndex.OverrideHostDefault).m_Value.BoolVal ;
	m_OverrideCookieDefault = m_Variables->Get(m_VarIndex.OverrideCookieDefault).m_Value.BoolVal ;
	m_OverrideIfModifiedSinceDefault = m_Variables->Get(m_VarIndex.OverrideIfModifiedSinceDefault).m_Value.BoolVal ;
  m_bAcceptEncodingGzip = m_Variables->Get(m_VarIndex.AcceptEncodingGzip).m_Value.BoolVal ;
  m_sAcceptLanguage = m_Variables->Get(m_VarIndex.AcceptLanguage).m_Value.StringVal ;

  m_pcFilterUrl = m_Variables->Get(m_VarIndex.FilterUrl).m_Value.StringVal ;

  // If low level unescape is true, don't permit encoding of formdata
  if (m_LowLevelUnescape){
	m_EncodeFormdata = false;
	m_EncodeData = false;
  }

  WlTraceMsg(TraceAllDataMembers());
};

void CWLWorkspace::RegisterHttpProperties ()
{
  CLEPropertyValue DefVal ;

  //Url
  m_VarIndex.Url = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_URL,                  //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //UserName
  m_VarIndex.UserName = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_USER_NAME,            //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //PassWord
  m_VarIndex.PassWord = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PASSWORD,             //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //NTUserName
  m_VarIndex.NTUserName = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_NT_USERNAME,          //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //NTPassWord
  m_VarIndex.NTPassWord = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_NT_PASSWORD,          //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //UserAgent
  m_VarIndex.UserAgent = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_USER_AGENT,               //Name
      CLEPropertyValue::PT_STRING,            //Type
      DefVal.Set (DEFAULT_USER_AGENT),        //Default Value,
      "",                                     //List of values
      CLEPropertyValue::PL_GLOBALS,           //Start level
      CLEPropertyValue::PL_PRIVATE            //End level
     ); 

  //AcceptLanguage
  m_VarIndex.AcceptLanguage = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_ACCEPT_LANGUAGE,          //Name
      CLEPropertyValue::PT_STRING,            //Type
      DefVal.Set (""),                        //Default Value,
      "",                                     //List of values
      CLEPropertyValue::PL_GLOBALS,           //Start level
      CLEPropertyValue::PL_PRIVATE            //End level
     ); 


  //Redirection limit
  m_VarIndex.RedirectionLimit = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_REDIRECTION_LIMIT,                    //Name
      CLEPropertyValue::PT_INT,                           //Type
      DefVal.Set ((long)DEFAULT_REDIRECTION_LIMIT),       //Default Value,
      "",                                                 //List of values
      CLEPropertyValue::PL_GLOBALS,                       //Start level
      CLEPropertyValue::PL_PRIVATE                        //End level
     ); 

  //Redirection enabled
  m_VarIndex.RedirectionEnabled = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_REDIRECTION_ENABLED,                  //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                   //Type
      DefVal.Set (true, CLEPropertyValue::PT_BOOLEAN_STR),//Default Value,
      "",                                                 //List of values
      CLEPropertyValue::PL_GLOBALS,                       //Start level
      CLEPropertyValue::PL_PRIVATE                        //End level
     ); 

  //Galtech
  //CharSet

// @@Chaim - remove these
/*
#ifndef _ANSI_ONLY
  m_VarIndex.CharSet = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_CHARSET,                   //Name
      CLEPropertyValue::PT_INT,                       //Type
      DefVal.Set ((long)CP_DEFAULT), \
      "(long)0,\
			(long)874,\
		(long)932,\
		(long)936,\
		(long)949,\
		(long)950,\
		(long)1200,\
		(long)1250,\
		(long)1251,\
		(long)1252,\
		(long)1253,\
		(long)1254,\
		(long)1255,\
		(long)1256,\
		(long)1257,\
		(long)2022,\
		(long)10000,\
		(long)10008,\
		(long)20866,\
		(long)28593,\
		(long)40850,\
		(long)40852,\
		(long)40862,\
		(long)40866,\
		(long)40869,\
		(long)51932,\
		(long)51949,\
		(long)65000,\
		(long)65001",                                             //List of values
      CLEPropertyValue::PL_GLOBALS,                   //Start level
      CLEPropertyValue::PL_GLOBALS                    //End level
     ); 
  //DomCharset
  m_VarIndex.DomCharset = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_DOM_CHARSET,                   //Name
      CLEPropertyValue::PT_INT,                   //Type
      DefVal.Set ((long)CP_DEFAULT), \
      "(long)0,\
			(long)874,\
		(long)932,\
		(long)936,\
		(long)949,\
		(long)950,\
		(long)1200,\
		(long)1250,\
		(long)1251,\
		(long)1252,\
		(long)1253,\
		(long)1254,\
		(long)1255,\
		(long)1256,\
		(long)1257,\
		(long)2022,\
		(long)10000,\
		(long)10008,\
		(long)20866,\
		(long)28593,\
		(long)40850,\
		(long)40852,\
		(long)40862,\
		(long)40866,\
		(long)40869,\
		(long)51932,\
		(long)51949,\
		(long)65000,\
		(long)65001",                                             //List of values
      CLEPropertyValue::PL_GLOBALS,                   //Start level
      CLEPropertyValue::PL_PRIVATE                    //End level
     ); 
#endif
*/

	//EnforceCharEncoding
	m_VarIndex.EnforceCharEncoding = 
		CLEVariablesData::RegisterProperty
		(	WLJSNAME_HTTP_ENFORCE_CHAR_ENCODING,
			CLEPropertyValue::PT_BOOLEAN_STR,
			DefVal.Set(false, CLEPropertyValue::PT_BOOLEAN_STR),
			"",
			CLEPropertyValue::PL_GLOBALS,                   //Start level
      CLEPropertyValue::PL_GLOBALS                    //End level
     ); 


  //ConnectionSpeed
  m_VarIndex.ConnectionSpeed = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_SLOW_CONN_RATE,                   //Name
      CLEPropertyValue::PT_INT,                       //Type
      DefVal.Set ((long)0),                           //Default Value,
      "",                                             //List of values
      CLEPropertyValue::PL_GLOBALS,                   //Start level
      CLEPropertyValue::PL_GLOBALS                    //End level
     ); 

  //BrowserCache
  m_VarIndex.BrowserCache = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_BROWSER_CACHE,                          //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (true, CLEPropertyValue::PT_BOOLEAN_STR),  //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //HttpCacheType
  m_VarIndex.HttpCacheType = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_HTTP_CACHE_TYPE,                        //Name
      CLEPropertyValue::PT_STRING,                          //Type
      DefVal.Set ( "None" ),                                //Default Value,
      "None,FreshOnly,Heuristics",                          //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //HttpCacheScope
  m_VarIndex.HttpCacheScope = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_HTTP_CACHE_SCOPE,                       //Name
      CLEPropertyValue::PT_STRING,                          //Type
      DefVal.Set ( "SingleCommand" ),                       //Default Value,
      "SingleCommand,SingleRound,WholeRun",                 //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //HttpCacheUncachedTypes
  m_VarIndex.HttpCacheUncachedTypes = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_HTTP_CACHE_UNCACHED_TYPES,              //Name
      CLEPropertyValue::PT_STRING,                          //Type
      DefVal.Set ( "HTML,Dynamic" ),                        //Default Value,
//      "None,HTML,JS,CSS,XML,Applet,Image,Dynamic",          //List of values
      "",          //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //DNSUseCache
  m_VarIndex.DNSUseCache = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_CACHE_DNS,                              //Name                                   
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (true, CLEPropertyValue::PT_BOOLEAN_STR ), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //Version
  m_VarIndex.Version = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_VERSION,             //Name
      CLEPropertyValue::PT_STRING,       //Type
      DefVal.Set (HTTP_VERSION_10),      //Default Value,
      "1.0,1.1",                         //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 


  //MultiIPSupport
  m_VarIndex.MultiIPSupport = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_MULTI_IP_SUPPORT,                       //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                            //End level
     ); 

  //MultiIPSupportType
  m_VarIndex.MultiIPSupportType = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_MULTI_IP_SUPPORT_TYPE,					//Name
      CLEPropertyValue::PT_STRING,							//Type
      DefVal.Set (WLJSNAME_HTTP_MULTI_IP_SUPPORT_PER_CLIENT),  //Default Value,
      "PerClient, PerRound",                                //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                            //End level
     ); 

  //KeepAlive
  m_VarIndex.KeepAlive = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_KEEP_ALIVE,                             //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (true, CLEPropertyValue::PT_BOOLEAN_STR),  //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                            //End level
     ); 

  // Checkpoint request - allow user intervention in Keep-Alive
  m_VarIndex.Persistancy = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PERSISTANCY,                            //Name
      CLEPropertyValue::PT_INT,                             //Type
      DefVal.Set ((long)IGNORE_PERSISTANCY),                //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                            //End level
     ); 

  //Proxy 
  m_VarIndex.Proxy = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PROXY,                 //Name
      CLEPropertyValue::PT_STRING,         //Type
      DefVal.Set (""),                     //Default Value,
      "",                                  //List of values
      CLEPropertyValue::PL_GLOBALS,        //Start level
      CLEPropertyValue::PL_PRIVATE         //End level
     ); 

  //HttpsProxy 
  m_VarIndex.HttpsProxy = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_HTTPSPROXY,                 //Name
      CLEPropertyValue::PT_STRING,         //Type
      DefVal.Set (""),                     //Default Value,
      "",                                  //List of values
      CLEPropertyValue::PL_GLOBALS,        //Start level
      CLEPropertyValue::PL_PRIVATE         //End level
     ); 

  //UseSameProxyForSSL
  m_VarIndex.UseSameProxyForSSL =
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_USE_SAME_PROXY4SSL,  //Name
      CLEPropertyValue::PT_BOOLEAN,      //Type
      DefVal.Set (true),                //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 


  //ProxyUserName
  m_VarIndex.ProxyUserName = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PROXY_USER_NAME,      //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //ProxyPassWord 
  m_VarIndex.ProxyPassWord = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PROXY_PASSWORD,       //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 
  
  //ProxyNTUserName
  m_VarIndex.ProxyNTUserName = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PROXY_NT_USER_NAME,   //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //ProxyNTPassWord 
  m_VarIndex.ProxyNTPassWord = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PROXY_NT_PASSWORD,    //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 
  


  //HttpsProxyUserName
  m_VarIndex.HttpsProxyUserName = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_HTTPSPROXY_USER_NAME,      //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //HttpsProxyPassWord 
  m_VarIndex.HttpsProxyPassWord = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_HTTPSPROXY_PASSWORD,       //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 
  
  //HttpsProxyNTUserName
  m_VarIndex.HttpsProxyNTUserName = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_HTTPSPROXY_NT_USER_NAME,   //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //HttpsProxyNTPassWord 
  m_VarIndex.HttpsProxyNTPassWord = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_HTTPSPROXY_NT_PASSWORD,    //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //ClientType
  m_VarIndex.ClientType = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_CLIENT_TYPE,          //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set ("Thick"),               //Default Value,
      "Normal,Thin,Thick,Custom",         //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_GLOBALS        //End level
     ); 
  
  
  //Parse
  m_VarIndex.Parse = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE,                //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set ("No"),                  //Default Value,
      "No,Yes",                           //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 
  
  //ParseOnce - Chaim
  m_VarIndex.ParseOnce = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE_ONCE,                             //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 
  
  //ParseFullBuffer
  m_VarIndex.ParseFullBuffer = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE_FULL_BUFFER,                      //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 
  
  //ParseForms
  m_VarIndex.ParseForms = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE_FORMS,                            //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR),  //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //ParseLinks
  m_VarIndex.ParseLinks = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE_LINKS,                            //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR),  //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //ParseScripts
  m_VarIndex.ParseScripts = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE_SCRIPTS,                          //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //ParseTables
  m_VarIndex.ParseTables = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE_TABLES,                           //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //ParseMetas
  m_VarIndex.ParseMetas = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE_METAS,                            //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //ParseImages
  m_VarIndex.ParseImages = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE_IMAGES,                           //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //ParseEmbeds
  m_VarIndex.ParseEmbeds = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE_EMBEDS,                           //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //ParseApplets
  m_VarIndex.ParseApplets = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE_APPLETS,                          //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //ParseCss
  m_VarIndex.ParseCss = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE_CSS,                              //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //ParseOthers
  m_VarIndex.ParseOthers = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PARSE_OTHERS,                           //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  // Chaim Apr. 2001 - SaveHeaders
  m_VarIndex.SaveHeaders = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTML_SAVE_HEADERS,                           //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (true, CLEPropertyValue::PT_BOOLEAN_STR),  //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //SaveSource
  m_VarIndex.SaveSource = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTML_SAVE_SOURCE,                            //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //CovertToUTF8ForParse
  m_VarIndex.CovertToUTF8ForParse = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_CONVERT_TO_UTF8,                        //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //Outfile
  m_VarIndex.Outfile = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_OUTFILE,                //Name
      CLEPropertyValue::PT_STRING,          //Type
      DefVal.Set (""),                      //Default Value,
      "",                                   //List of values
      CLEPropertyValue::PL_GLOBALS,         //Start level
      CLEPropertyValue::PL_PRIVATE          //End level
     ); 
  
#ifdef __OPEN_SSL__

  //SSLVersion
  m_VarIndex.SSLVersion = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_SSL_VERSION,                   //Name
      CLEPropertyValue::PT_STRING,            //Type
      DefVal.Set ("SSL_Version_3_0_With_2_0_Hello"), //Default Value,
      "SSL_Version_2_0,SSL_Version_Undetermined,SSL_Version_3_0_With_2_0_Hello,SSL_Version_3_0_Only,SSL_Version_3_0,TLS_Version_1.0_With_2_0_Hello,TLS_Version_1_0_Only,TLS_Version_1_0", //List of values
      CLEPropertyValue::PL_GLOBALS,           //Start level
      CLEPropertyValue::PL_PRIVATE            //End level
     ); 

  //SSLBitLimit

  //calculating the legal list of values.
//  extern void SSLGetStrengthList(char** list);
	RVCString fullList = "None,";
	char* sslStrengthList = NULL;
	// memory is allocated by the called function
//	SSLGetStrengthList(&sslStrengthList);
	sslStrengthList = new char[512];
	strcpy( sslStrengthList, "  40,  56,  64, 128, 168, 196, 256, 512,1024" );
	fullList += sslStrengthList;
//	free(sslStrengthList);
	delete [] sslStrengthList;

  m_VarIndex.SSLBitLimit = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_SSL_BIT_LIMIT,                  //Name
      CLEPropertyValue::PT_STRING,             //Type
      DefVal.Set ("None"),                     //Default Value,
      (char*)fullList.data(),                  //List of values
      CLEPropertyValue::PL_GLOBALS,            //Start level
      CLEPropertyValue::PL_GLOBALS             //End level
     ); 

  //SSLUseCache
  m_VarIndex.SSLUseCache = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_SSL_USE_CACHE,                               //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (true, CLEPropertyValue::PT_BOOLEAN_STR),  //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //SSLClientCertificateFile
  m_VarIndex.SSLClientCertificateFile = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_CERTIFICATE_FILE,            //Name
      CLEPropertyValue::PT_STRING,          //Type
      DefVal.Set (""),                      //Default Value,
      "",                                   //List of values
      CLEPropertyValue::PL_GLOBALS,         //Start level
      CLEPropertyValue::PL_PRIVATE          //End level
     ); 
  
  //SSLClientCertificatePassword
  m_VarIndex.SSLClientCertificatePassword = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_CERTIFICATE_PASSWORD,         //Name
      CLEPropertyValue::PT_STRING,           //Type
      DefVal.Set (""),                       //Default Value,
      "",                                    //List of values
      CLEPropertyValue::PL_GLOBALS,          //Start level
      CLEPropertyValue::PL_PRIVATE           //End level
     ); 
  
  //SSLCryptoStrength
  m_VarIndex.SSLCryptoStrength = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_SSL_CRYPTO_STRENGH,           //Name
      CLEPropertyValue::PT_STRING,           //Type
      DefVal.Set ("SSL_AllCrypto"),          //Default Value,
      "SSL_AllCrypto,SSL_ExportCryptoOnly,SSL_ServerGatedCrypto,SSL_StrongCryptoOnly",   //List of values
      CLEPropertyValue::PL_GLOBALS,          //Start level
      CLEPropertyValue::PL_PRIVATE           //End level
     ); 

#endif __SSL__

  //GetFrames
  m_VarIndex.GetFrames = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_GET_FRAMES,                              //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //GetImages
  m_VarIndex.GetImages = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_GET_IMAGES,                             //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //GetEmbeds
  m_VarIndex.GetEmbeds = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_GET_EMBEDS,                             //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //GetApplets
  m_VarIndex.GetApplets = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_GET_APPLETS,                            //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //GetScripts
  m_VarIndex.GetScripts = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_GET_SCRIPTS,                            //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //GetCss
  m_VarIndex.GetCss = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_GET_CSS,                                //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //GetXml
  m_VarIndex.GetXml = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_GET_XML,                                //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //GetOthers
  m_VarIndex.GetOthers = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_GET_OTHERS,                             //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //LoadGeneratorThreads
  m_VarIndex.LoadGeneratorThreads = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_LOAD_GENERATOR_THREADS,         //Name
      CLEPropertyValue::PT_STRING,                  //Type
      DefVal.Set ("Single"),                        //Default Value,
      "Single,Multiple,1,2,3,4,5,6,7,8",            //List of values
      CLEPropertyValue::PL_GLOBALS,                 //Start level
      CLEPropertyValue::PL_PRIVATE                  //End level
     ); 

  //ProbingClientThreads
  m_VarIndex.ProbingClientThreads = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_PROBING_CLIENT_THREADS,         //Name
      CLEPropertyValue::PT_STRING,                  //Type
      DefVal.Set ("Multiple"),                      //Default Value,
      "Single,Multiple,1,2,3,4,5,6,7,8",            //List of values
      CLEPropertyValue::PL_GLOBALS,                 //Start level
      CLEPropertyValue::PL_PRIVATE                  //End level
     ); 

  //Erase
  m_VarIndex.Erase = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_ERASE,                //Name
      CLEPropertyValue::PT_BOOLEAN,       //Type
      DefVal.Set (true),                  //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_PRIVATE,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //ParseXML
  m_VarIndex.ParseXML = 
    CLEVariablesData::RegisterProperty   
    ( WLJSNAME_HTTP_PARSE_XML,                               //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                      //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR),  //Default Value,
      "",                                                    //List of values
      CLEPropertyValue::PL_GLOBALS,                          //Start level
      CLEPropertyValue::PL_PRIVATE                           //End level
     ); 

  //XMLStandAlone
  m_VarIndex.XMLStandAlone = 
    CLEVariablesData::RegisterProperty   
    ( WLJSNAME_HTTP_XML_STANDALONE,                         //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (true, CLEPropertyValue::PT_BOOLEAN_STR),  //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //VerifyXML
  m_VarIndex.VerifyXML = 
    CLEVariablesData::RegisterProperty   
    ( WLJSNAME_HTTP_VERIFY_XML,                             //Name
      CLEPropertyValue::PT_BOOLEAN_STR,                     //Type
      DefVal.Set (false, CLEPropertyValue::PT_BOOLEAN_STR), //Default Value,
      "",                                                   //List of values
      CLEPropertyValue::PL_GLOBALS,                         //Start level
      CLEPropertyValue::PL_PRIVATE                          //End level
     ); 

  //UsingTimer
  m_VarIndex.UsingTimer = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_TIMER_NAME,          //Name
      CLEPropertyValue::PT_STRING,       //Type
      DefVal.Set (""),                   //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 

  //RTAMThreads
  m_VarIndex.RTAMThreads = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_RTAM_THREADS,         //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set ("Multiple"),            //Default Value,
      "Single,Multiple,1,2,3,4,5,6,7,8",  //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //UNDOC_CookieExpiration
  m_VarIndex.UNDOC_CookieExpiration = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_COOKIE_EXPIRATION,    //Name
      CLEPropertyValue::PT_BOOLEAN,       //Type
      DefVal.Set (false),                 //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //UNDOC_CookieDomain
  m_VarIndex.UNDOC_CookieDomain = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_COOKIE_DOMAIN,       //Name
      CLEPropertyValue::PT_BOOLEAN,      //Type
      DefVal.Set (true),                 //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 

  //UNDOC_CookiePath
  m_VarIndex.UNDOC_CookiePath = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_COOKIE_PATH,         //Name
      CLEPropertyValue::PT_BOOLEAN,      //Type
      DefVal.Set (true),                 //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 

  //CookieExpiration
  m_VarIndex.CookieExpiration = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_DOC_COOKIE_EXPIRATION, //Name
      CLEPropertyValue::PT_BOOLEAN,       //Type
      DefVal.Set (false),                 //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //CookieDomain
  m_VarIndex.CookieDomain = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_DOC_COOKIE_DOMAIN,   //Name
      CLEPropertyValue::PT_BOOLEAN,      //Type
      DefVal.Set (true),                 //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 

  //CookiePath
  m_VarIndex.CookiePath = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_DOC_COOKIE_PATH,     //Name
      CLEPropertyValue::PT_BOOLEAN,      //Type
      DefVal.Set (true),                 //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 


  //Target
  m_VarIndex.Target = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_TARGET,              //Name
      CLEPropertyValue::PT_STRING,       //Type
      DefVal.Set (""),                   //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_PRIVATE,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 

  //AutomaticNamedTransaction
  m_VarIndex.AutoNamedTrans = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_AUTO_NAMED_TRANS,    //Name
      CLEPropertyValue::PT_BOOLEAN,      //Type
      DefVal.Set (true),                 //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_GLOBALS       //End level
     ); 

  //ExpectedUrl
  m_VarIndex.ExpectedUrl = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_EXPECTED_URL,        //Name
      CLEPropertyValue::PT_STRING,       //Type
      DefVal.Set (""),                   //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 

  //AsmDiagnostics
  m_VarIndex.AsmDiagnostics = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_ASM_DIAGNOSTICS,     //Name
      CLEPropertyValue::PT_BOOLEAN,      //Type
      DefVal.Set (false),                //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_GLOBALS       //End level
     ); 

  //AsmOn
  m_VarIndex.AsmOn = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_ASM_ON,              //Name
      CLEPropertyValue::PT_BOOLEAN,      //Type
      DefVal.Set (false),                //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_GLOBALS       //End level
     ); 

  // sub transactions 
  m_VarIndex.SubTransactions =
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_SUB_TRANSACTIONS,    //Name
      CLEPropertyValue::PT_BOOLEAN,      //Type
      DefVal.Set (true),                 //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 

  // sub transactions 
  m_VarIndex.TimerTransactions =
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_TIMER_TRANSACTIONS,  //Name
      CLEPropertyValue::PT_BOOLEAN,      //Type
      DefVal.Set (false),                //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 

  // transaction time
  m_VarIndex.TransactionTime = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_TRANSACTION_TIME,     //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  // save transaction
  m_VarIndex.SaveTransaction =
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_SAVE_TRANSACTION,    //Name
      CLEPropertyValue::PT_BOOLEAN,      //Type
      DefVal.Set (false),                //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 

  // verification 
  m_VarIndex.Verification =
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_VERIFICATION,        //Name
      CLEPropertyValue::PT_BOOLEAN,      //Type
      DefVal.Set (false),                //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 

  // Verification set max threads
  m_VarIndex.VerSetMaxThreads = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_VER_SET_MAX_THREADS,                 //Name
      CLEPropertyValue::PT_INT,                          //Type
      DefVal.Set ((long)(-1)),                           //Default Value,
      "",                                                //List of values
      CLEPropertyValue::PL_GLOBALS,                      //Start level
      CLEPropertyValue::PL_GLOBALS                       //End level
     ); 

  // Verification set max rounds
  m_VarIndex.VerSetMaxRounds = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_VER_SET_MAX_ROUNDS,                  //Name
      CLEPropertyValue::PT_INT,                          //Type
      DefVal.Set ((long)(-1)),                           //Default Value,
      "",                                                //List of values
      CLEPropertyValue::PL_GLOBALS,                      //Start level
      CLEPropertyValue::PL_GLOBALS                       //End level
     ); 

  // Verification set rounds percentage
  m_VarIndex.VerSetRounds = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_VER_SET_ROUNDS,                     //Name
      CLEPropertyValue::PT_INT,                          //Type
      DefVal.Set ((long)(-1)),                           //Default Value,
      "",                                                //List of values
      CLEPropertyValue::PL_GLOBALS,                      //Start level
      CLEPropertyValue::PL_GLOBALS                       //End level
     ); 

  // Timeout Severity 
  m_VarIndex.TimeoutSeverity =
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_TIMEOUT_SEVERITY,    //Name
      CLEPropertyValue::PT_INT,          //Type
      DefVal.Set ((long)WLLogMessage::Error),//Default Value,
      "(long)WLLogMessage::Success,  \
       (long)WLLogMessage::Note,     \
       (long)WLLogMessage::Warning,  \
       (long)WLLogMessage::Error,    \
       (long)WLLogMessage::Fatal",//List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 

  //RequestRetries
  m_VarIndex.RequestRetries = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_REQUEST_RETRIES,     //Name
      CLEPropertyValue::PT_INT,            //Type
      DefVal.Set ((long)9),                //Default Value,
      "",                                  //List of values
      CLEPropertyValue::PL_GLOBALS,        //Start level
      CLEPropertyValue::PL_PRIVATE         //End level
     ); 

  //ReceiveTimeout
  m_VarIndex.ReceiveTimeout = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_RECEIVE_TIMEOUT,       //Name
      CLEPropertyValue::PT_INT,            //Type
      DefVal.Set ((long)900000),           //Default Value in millisecs,
      "",                                  //List of values
      CLEPropertyValue::PL_GLOBALS,        //Start level
      CLEPropertyValue::PL_PRIVATE         //End level
     ); 

  //EscapeChar
  m_VarIndex.EscapeChar = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_ESCAPE_CHAR,		   //Name
      CLEPropertyValue::PT_STRING,         //Type
      DefVal.Set (""),                     //Default Value,
      "",                                  //List of values
      CLEPropertyValue::PL_GLOBALS,        //Start level
      CLEPropertyValue::PL_PRIVATE         //End level
     ); 

  //LowLevelUnescape
  m_VarIndex.LowLevelUnescape = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_LOW_LEVEL_UNESCAPE,	   //Name
      CLEPropertyValue::PT_BOOLEAN  ,      //Type
      DefVal.Set (false),                  //Default Value,
      "",                                  //List of values
      CLEPropertyValue::PL_GLOBALS,        //Start level
      CLEPropertyValue::PL_PRIVATE         //End level
     ); 

  //DecodeBinaryNullAs
  m_VarIndex.DecodeBinaryNullAs = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_DECODE_BINARY_NULL_AS, //Name
      CLEPropertyValue::PT_STRING,         //Type
      DefVal.Set (""),                     //Default Value,
      "",                                  //List of values
      CLEPropertyValue::PL_GLOBALS,        //Start level
      CLEPropertyValue::PL_PRIVATE         //End level
     ); 

  //DecodeBinaryStart
  m_VarIndex.DecodeBinaryStart = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_DECODE_BINARY_START,   //Name
      CLEPropertyValue::PT_INT,			   //Type
      DefVal.Set ((long)DEFAULT_DECODE_BINARY_START), //Default Value,
      "",                                  //List of values
      CLEPropertyValue::PL_GLOBALS,        //Start level
      CLEPropertyValue::PL_PRIVATE         //End level
     ); 

  //DecodeBinaryEnd
  m_VarIndex.DecodeBinaryEnd = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_DECODE_BINARY_END,     //Name
      CLEPropertyValue::PT_INT,			   //Type
      DefVal.Set ((long)DEFAULT_DECODE_BINARY_END), //Default Value,
      "",                                  //List of values
      CLEPropertyValue::PL_GLOBALS,        //Start level
      CLEPropertyValue::PL_PRIVATE         //End level
     ); 

  //EncodeFormdata
  m_VarIndex.EncodeFormdata = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_ENCODE_FORMDATA,       //Name
      CLEPropertyValue::PT_BOOLEAN,        //Type
      DefVal.Set (true),                   //Default Value,
      "",                                  //List of values
      CLEPropertyValue::PL_GLOBALS,        //Start level
      CLEPropertyValue::PL_PRIVATE         //End level
     ); 

  //EDelete Empty Cookies
  m_VarIndex.DeleteEmptyCookies = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_DELETE_EMPTY_COOKIES,  //Name
      CLEPropertyValue::PT_BOOLEAN,        //Type
      DefVal.Set (false),                   //Default Value,
      "",                                  //List of values
      CLEPropertyValue::PL_GLOBALS,        //Start level
      CLEPropertyValue::PL_PRIVATE         //End level
     ); 

  //EncodeData
  m_VarIndex.EncodeData = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_ENCODE_DATA,            //Name
      CLEPropertyValue::PT_BOOLEAN,        //Type
      DefVal.Set (false),                  //Default Value,
      "",                                  //List of values
      CLEPropertyValue::PL_GLOBALS,        //Start level
      CLEPropertyValue::PL_PRIVATE         //End level
     ); 

  // connection timeout
  m_VarIndex.ConnectTimeout = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_CONNECT_TIMEOUT,  										//Name
      CLEPropertyValue::PT_INT,                           //Type
      DefVal.Set ((long)DEFAULT_CONNECT_TIMEOUT),			    //Default Value,
      "",                                                 //List of values
      CLEPropertyValue::PL_GLOBALS,                       //Start level
      CLEPropertyValue::PL_PRIVATE                        //End level
     ); 
  
  // select for write timeout
  m_VarIndex.SelectWriteTimeout = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_SELECT_WRITE_TIMEOUT,						      //Name
      CLEPropertyValue::PT_INT,                           //Type
      DefVal.Set ((long)DEFAULT_SELECT_WRITE_TIMEOUT),		//Default Value,
      "",                                                 //List of values
      CLEPropertyValue::PL_GLOBALS,                       //Start level
      CLEPropertyValue::PL_PRIVATE                        //End level
     ); 

  // select for write second timeout
  m_VarIndex.SelectWriteSecondTimeout = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_SELECT_WRITE_SECOND_TIMEOUT,		            //Name
      CLEPropertyValue::PT_INT,                                 //Type
      DefVal.Set ((long)DEFAULT_SELECT_WRITE_SECOND_TIMEOUT),		//Default Value,
      "",                                                       //List of values
      CLEPropertyValue::PL_GLOBALS,                             //Start level
      CLEPropertyValue::PL_PRIVATE                              //End level
     ); 

  // select for write switch between first to second timeout num
  m_VarIndex.SelectWriteSwitchNum = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_SELECT_WRITE_SWITCH_NUM,						        //Name
      CLEPropertyValue::PT_INT,                           //Type
      DefVal.Set ((long)DEFAULT_SELECT_WRITE_SWITCH_NUM),	//Default Value,
      "",                                                 //List of values
      CLEPropertyValue::PL_GLOBALS,                       //Start level
      CLEPropertyValue::PL_PRIVATE                        //End level
     ); 

  // select wait
  m_VarIndex.SelectTimeout = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_SELECT_TIMEOUT,									      //Name
      CLEPropertyValue::PT_INT,                           //Type
      DefVal.Set ((long)DEFAULT_SELECT_TIMEOUT),					//Default Value,
      "",                                                 //List of values
      CLEPropertyValue::PL_GLOBALS,                       //Start level
      CLEPropertyValue::PL_PRIVATE                        //End level
     ); 

  // select second wait
  m_VarIndex.SelectSecondTimeout = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_SELECT_SECOND_TIMEOUT,					      //Name
      CLEPropertyValue::PT_INT,                           //Type
      DefVal.Set ((long)DEFAULT_SELECT_SECOND_TIMEOUT),		//Default Value,
      "",                                                 //List of values
      CLEPropertyValue::PL_GLOBALS,                       //Start level
      CLEPropertyValue::PL_PRIVATE                        //End level
     ); 

  // select switch between first to second timeout
  m_VarIndex.SelectSwitchNum = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_SELECT_SWITCH_NUM,    					      //Name
      CLEPropertyValue::PT_INT,                           //Type
      DefVal.Set ((long)DEFAULT_SELECT_SWITCH_NUM),		    //Default Value,
      "",                                                 //List of values
      CLEPropertyValue::PL_GLOBALS,                       //Start level
      CLEPropertyValue::PL_PRIVATE                        //End level
     ); 


  // WLJSNAME_HTTP_OVERRIDE_HOST_DEFAULT
  m_VarIndex.OverrideHostDefault = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_OVERRIDE_HOST_DEFAULT,    //Name
      CLEPropertyValue::PT_BOOLEAN,       //Type
      DefVal.Set (false),                 //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 
  // WLJSNAME_HTTP_OVERRIDE_COOKIE_DEFAULT
  m_VarIndex.OverrideCookieDefault = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_OVERRIDE_COOKIE_DEFAULT,    //Name
      CLEPropertyValue::PT_BOOLEAN,       //Type
      DefVal.Set (false),                 //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 
  // WLJSNAME_HTTP_OVERRIDE_IF_MODIFIED_SINCE_DEFAULT
  m_VarIndex.OverrideIfModifiedSinceDefault = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_OVERRIDE_IF_MODIFIED_SINCE_DEFAULT,    //Name
      CLEPropertyValue::PT_BOOLEAN,       //Type
      DefVal.Set (false),                 //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 


  // Receive buffer size
  m_VarIndex.RecvBufferSize = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_BUFFER_SIZE,										      //Name
      CLEPropertyValue::PT_INT,                           //Type
      DefVal.Set ((long)DEFAULT_BUFFER_SIZE),             //Default Value,
      "",                                                 //List of values
      CLEPropertyValue::PL_GLOBALS,                       //Start level
      CLEPropertyValue::PL_PRIVATE                        //End level
     ); 

  // Send buffer size
  m_VarIndex.SendBufferSize = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_SEND_BUFFER_SIZE,										//Name
		CLEPropertyValue::PT_INT,                           //Type
		DefVal.Set ((long)DEFAULT_SEND_BUFFER_SIZE),        //Default Value,
		"",                                                 //List of values
		CLEPropertyValue::PL_GLOBALS,                       //Start level
		CLEPropertyValue::PL_PRIVATE                        //End level
		); 
	
  // Request Multi Part buffer size
  m_VarIndex.RequestMultiPartBufferSize = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_REQUEST_MULTI_PART_BUFFER_SIZE,								//Name
      CLEPropertyValue::PT_INT,																		//Type
      DefVal.Set ((long)DEFAULT_REQUEST_MULTI_PART_BUFFER_SIZE),  //Default Value,
      "",																													//List of values
      CLEPropertyValue::PL_GLOBALS,																//Start level
      CLEPropertyValue::PL_PRIVATE																//End level
     ); 

  // sub transactions 
  m_VarIndex.FoldTransactionsTree =
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_FOLD_TRANSACTIONS_TREE, //Name
      CLEPropertyValue::PT_BOOLEAN,         //Type
      DefVal.Set (true),                    //Default Value,
      "",                                   //List of values
      CLEPropertyValue::PL_PRIVATE,         //Start level
      CLEPropertyValue::PL_PRIVATE          //End level
     ); 

  // content types
  m_VarIndex.HtmlContentTypes = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTML_CONTENT_TYPES,        //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "text/html;",                       //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  // content types
  m_VarIndex.XmlContentTypes = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_XML_CONTENT_TYPES,         //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "text/xml;application/xml;text/vxml;", //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

  //ResponseContentType
  m_VarIndex.ResponseContentType = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_RESPONSE_CONTENT_TYPES,        //Name
      CLEPropertyValue::PT_STRING,            //Type
      DefVal.Set (RESPONSE_CONTENT_TYPE_NOT_DEFINED),//Default Value,
      RESPONSE_CONTENT_TYPE_DEFAULT_VALUES,	  //List of values
      CLEPropertyValue::PL_GLOBALS,           //Start level
      CLEPropertyValue::PL_PRIVATE            //End level
     ); 

  //RequestContentMultiPartBoundary
  m_VarIndex.RequestContentMultiPartBoundary = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_REQUEST_CONTENT_MULTI_PART_BOUNDARY,	//Name
      CLEPropertyValue::PT_STRING,										//Type
      DefVal.Set (""),																//Default Value,
      "",																							//List of values
      CLEPropertyValue::PL_PRIVATE,										//Start level
      CLEPropertyValue::PL_PRIVATE										//End level
     ); 

  //Referer
  m_VarIndex.Referer = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_REFERER,                  //Name
      CLEPropertyValue::PT_STRING,            //Type
      DefVal.Set (""),                        //Default Value,
      "",                                     //List of values
      CLEPropertyValue::PL_PRIVATE,           //Start level
      CLEPropertyValue::PL_PRIVATE            //End level
     ); 

  //ContentType
  m_VarIndex.ContentType = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_CONTENT_TYPE,             //Name
      CLEPropertyValue::PT_STRING,            //Type
      DefVal.Set (""),                        //Default Value,
      "",                                     //List of values
      CLEPropertyValue::PL_GLOBALS,           //Start level
      CLEPropertyValue::PL_PRIVATE            //End level
     ); 

  // CheckFileContent
  m_VarIndex.CheckFileContent =
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_CHECK_FILE_CONTENT,     //Name
      CLEPropertyValue::PT_BOOLEAN,         //Type
      DefVal.Set (false),                   //Default Value,
      "",                                   //List of values
      CLEPropertyValue::PL_GLOBALS,         //Start level
      CLEPropertyValue::PL_PRIVATE          //End level
     ); 

  // ReportHttpErrors
  m_VarIndex.ReportHttpErrors =
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_REPORT_HTTP_ERRORS,     //Name
      CLEPropertyValue::PT_BOOLEAN,         //Type
      DefVal.Set (true),                    //Default Value,
      "",                                   //List of values
      CLEPropertyValue::PL_GLOBALS,         //Start level
      CLEPropertyValue::PL_PRIVATE          //End level
     ); 

  //ContentType
  m_VarIndex.RVProduct = 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_RVPRODUCT,                //Name
      CLEPropertyValue::PT_STRING,            //Type
      DefVal.Set (""),                        //Default Value,
      "WebLOAD,WebFT",                        //List of values
      CLEPropertyValue::PL_GLOBALS,           //Start level
      CLEPropertyValue::PL_GLOBALS            //End level
     ); 

    // CacheImagesInCSJS
  m_VarIndex.AcceptEncodingGzip =
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_ACCEPT_ENCODING_GZIP,   //Name
      CLEPropertyValue::PT_BOOLEAN,         //Type
      DefVal.Set (false),                   //Default Value,
      "",                                   //List of values
      CLEPropertyValue::PL_GLOBALS,         //Start level
      CLEPropertyValue::PL_PRIVATE          //End level
     ); 

//Dummy property for testing since we currently don't a property of float type
#ifdef _DEBUG
  //AWLFloatProperty
  m_VarIndex.FloatProp = 
    CLEVariablesData::RegisterProperty 
    ( "WLFloatProperty",                 //Name
      CLEPropertyValue::PT_FLOAT  ,      //Type
      DefVal.Set ((float)0),             //Default Value,
      "",                                //List of values
      CLEPropertyValue::PL_GLOBALS,      //Start level
      CLEPropertyValue::PL_PRIVATE       //End level
     ); 
#endif

  // FilterUrl
  m_VarIndex.FilterUrl= 
    CLEVariablesData::RegisterProperty 
    ( WLJSNAME_HTTP_FILTER_URL,           //Name
      CLEPropertyValue::PT_STRING,        //Type
      DefVal.Set (""),                    //Default Value,
      "",                                 //List of values
      CLEPropertyValue::PL_GLOBALS,       //Start level
      CLEPropertyValue::PL_PRIVATE        //End level
     ); 

};
