/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#include "BPHttpUI.h"
#include "associativearray.h"
#include "wldocument.h"
#include "wlgeneral.h"

#define printHTTP if (m_nFlagFilter!=FILTER_HTTP && m_nFlagFilter!=FILTER_ALL && m_nFlagFilter!=FILTER_BRUTAL) printf
#define printNoHTTP if (m_nFlagFilter!=FILTER_NO_HTTP && m_nFlagFilter!=FILTER_ALL && m_nFlagFilter!=FILTER_BRUTAL) printf
#define printAll if (m_nFlagFilter!=FILTER_BRUTAL) printf

CHttpProbingClientUITest::CHttpProbingClientUITest(
  cSingleClientProcess& SCP,
  int nRoundsToDo/*0*/,
  int nFlagFilter/*0*/)
:
  ILoadProbingClientUITest( SCP, nRoundsToDo ) 
{
  m_nFlagFilter = nFlagFilter;
}

CHttpProbingClientUITest::~CHttpProbingClientUITest()
{
}

void CHttpProbingClientUITest::Session(
                                        iSingleClientMachineThread& MThread,
	                                      const char* InitializationScript,
	                                      iAgendaFile& File,
	                                      const char* AgendaName,
	                                      int ThreadNum,
	                                      unsigned long RoundNum ) 
{
	fInit( MThread );
	fExecute( InitializationScript, File, AgendaName, ThreadNum, RoundNum );
	fFinal();
}

#ifdef __TV_WINDOWS__
  #define SafeStringVal(x) x
#else
  #define SafeStringVal(x) (x) ? (x) : "(null)"
#endif // __TV_WINDOWS__

tSCActionReporterReturn CHttpProbingClientUITest::pageStartDownload(
                               iReportThreadState& threadState,
                               CUrlData* pUrlData,
                               long lExpectedDomId, 
                               long &lActiveDomId)
{
  RVCString s1("http://www.radview.com/default/base.htm");
  RVCString s2("newUrl/new.htm");
  printAll("*Pg Start Dwnld*\n\tThe Url is : %s\n\tThe Target is : %s\n\tThe Referer is : %s\n",
	   SafeStringVal(pUrlData->m_pcUrl),
	   SafeStringVal(pUrlData->m_pcTarget),
     SafeStringVal(pUrlData->m_pcReferer));
  printNoHTTP("Original Url: %s Referer: %s\n",
	      SafeStringVal(pUrlData->m_pcOrigUrl),
	      SafeStringVal(pUrlData->m_pcReferer));
  lActiveDomId = 666 ;
  printNoHTTP("Expected Dom ID: %d Active Dom : %d\n",
	      lExpectedDomId,
	      lActiveDomId);
  return sSCActionReporterReturnContinue;
}

tSCActionReporterReturn CHttpProbingClientUITest::pageAbortDownload(
                               iReportThreadState& threadState,
                               CUrlData* pUrlData)
{
  printAll("*Pg Abort Dwnld* the url is : %s, The Target is : %s\n",
	   SafeStringVal(pUrlData->m_pcUrl),
	   SafeStringVal(pUrlData->m_pcTarget));
  printNoHTTP("Original Url: %s Referer: %s\n",
	      SafeStringVal(pUrlData->m_pcOrigUrl),
	      SafeStringVal(pUrlData->m_pcReferer));
  return sSCActionReporterReturnContinue;
}

tSCActionReporterReturn CHttpProbingClientUITest::pageEndDownload(
                               iReportThreadState& threadState,
                               CUrlData* pUrlData)
{
  printf("*Pg End Dwnld* the url is : %s, The Target is : %s\n",pUrlData->m_pcUrl, pUrlData->m_pcTarget);
  printf("Original Url: %s Referer: %s\n", pUrlData->m_pcOrigUrl, pUrlData->m_pcReferer);
  printf("Num of frames in page: %d, Document title: %s\n", pUrlData->m_pDom->m_Frames->entries(), pUrlData->m_pDom->GetTitle());
  for (int i = 0; i < pUrlData->m_pDom->m_Frames->entries();++i)
  {
    CWLDocument* frame = (CWLDocument*)pUrlData->m_pDom->m_Frames->at(i);
//		CWLLocation* location = &(frame->GetLocation());
    CWLLocation location = frame->GetLocation() ;
    const char* href = location.Href();
  }
  return sSCActionReporterReturnContinue;
}

tSCActionReporterReturn CHttpProbingClientUITest::objectStartDownload(
                               iReportThreadState& threadState,
                               CUrlData* pUrlData)
{
  printNoHTTP("***Obj Start Dwnld* the url is : %s\n\tThe Target is : %s\n\tThe Referer is%s\n",
	      SafeStringVal(pUrlData->m_pcUrl),
	      SafeStringVal(pUrlData->m_pcTarget),
        SafeStringVal(pUrlData->m_pcReferer));
  printNoHTTP("Original Url: %s Referer: %s\n",
	      SafeStringVal(pUrlData->m_pcOrigUrl),
	      SafeStringVal(pUrlData->m_pcReferer));
  return sSCActionReporterReturnContinue;
}

tSCActionReporterReturn CHttpProbingClientUITest::objectAbortDownload(
                               iReportThreadState& threadState,
                               CUrlDataEx* pUrlDataEx)
{
  printNoHTTP("***Obj Start Dwnld* the url is : %s, The Status is : %d\n",
	      SafeStringVal(pUrlDataEx->m_pcUrl),
	      pUrlDataEx->m_nHttpStatus);
  printNoHTTP("Original Url: %s Referer: %s\n",
	      SafeStringVal(pUrlDataEx->m_pcOrigUrl),
	      SafeStringVal(pUrlDataEx->m_pcReferer));
  return sSCActionReporterReturnContinue;
}

tSCActionReporterReturn CHttpProbingClientUITest::objectEndDownload(
                               iReportThreadState& threadState,
                               CallbackData* pData)
{
  unsigned int i;
  printNoHTTP("***Obj End Dwnld* the url is : %s, The content length is : %d\n",
	      SafeStringVal(pData->m_pcUrl),
	      pData->m_nContentLength);
  printNoHTTP("Original Url: %s Referer: %s, Type: %d\n",
	      SafeStringVal(pData->m_pcOrigUrl),
	      SafeStringVal(pData->m_pcReferer),
	      pData->m_eUrlType);
  printNoHTTP("Mime type = %s\n",pData->m_pcMimeType);
  printNoHTTP("Fileize : %d CheckSum : %d\n",
	      pData->m_nFileSize,
	      pData->m_CheckSum);

  for (i = 0 ; i < pData->HeadersList.entries() ; i++ )
  {
    printHTTP("Request:\n %s\n", pData->HeadersList[i].GetRequestHeader());
    printHTTP("Response:\n %s\n", pData->HeadersList[i].GetResponseHeader());
  }

  printNoHTTP("Locations: \n");
  for (i = 0 ; i < pData->LocationList.entries(); i++ )
  {
    RVCString sLocation = pData->m_pContent ? (char*)pData->m_pContent : "" ;
    if (pData->LocationList[i].m_nOffset + pData->LocationList[i].m_nLength > pData->m_nFileSize)
      sLocation = sLocation(pData->LocationList[i].m_nOffset, pData->LocationList[i].m_nLength);
    printNoHTTP("Location: %s Offset: %d Length : %d Line : %s\n",
      SafeStringVal(pData->LocationList[i].m_pcLocation),
      pData->LocationList[i].m_nOffset,
      pData->LocationList[i].m_nLength,
      sLocation.data()
      );
  }
    
  FILE *outputFile;
#ifdef __TV_WINDOWS__
  outputFile = fopen(OUTPUT_FILENAME, "a+");
#else
  outputFile = fopen(OUTPUT_FILENAME, "a+");
#endif
    fwrite((char*)pData->m_pContent, pData->m_nContentLength,1,outputFile);
	  fclose(outputFile);
  
  return sSCActionReporterReturnContinue;
}

tSCActionReporterReturn CHttpProbingClientUITest::UseCachedObject(
                               iReportThreadState& threadState,
                               CUrlDataEx* pUrlDataEx)
{
  printNoHTTP("***Obj Load From Cache* the url is : %s, The Status is : %d\n",
	      SafeStringVal(pUrlDataEx->m_pcUrl),
	      pUrlDataEx->m_nHttpStatus);
  printNoHTTP("Original Url: %s Referer: %s, Type: %d\n",
	      SafeStringVal(pUrlDataEx->m_pcOrigUrl),
	      SafeStringVal(pUrlDataEx->m_pcReferer),
	      pUrlDataEx->m_eUrlType);
  return sSCActionReporterReturnContinue;
}


void CHttpProbingClientUITest::fRecordEvent(long eventID, long topInstanceID, long instanceID,
												 WLLogMessage::Severity severity, const char* eventName,
												 double time, long lineNumber, const char* fileName,
												 const char* description, const char* extraDataType,
												 const char* extraData)
{
/*
	printf("===================================================\n");
	printf("RecordEvent: %s\n", eventName);
	printf("eventID=%d, topInstanceID=%d, instanceID=%d\n", eventID, topInstanceID, instanceID);
	printf("severity=%d, time=%f, lineNumber=%d\n", severity, time, lineNumber);
	printf("fileName=%s, description=%s\n", fileName, description);
	if(extraData == NULL) {
		printf("extraDataType=%s, extraData=NULL\n", extraDataType);
	}
	else {
		printf("extraDataType=%s, extraData=%s\n", extraDataType, extraData);
	}
	printf("===================================================\n");
*/
}


void CHttpProbingClientUITest::fBeginTransaction(long instanceID, 
															const char* transactionName, 
															const char* transactionType) 
{
/*
	printf("===================================================\n");
	printf("Begin Transaction: %s\n", transactionName);
	printf("instanceID=%d, transactionType=%s\n", instanceID, transactionType);
	printf("===================================================\n");
*/
}


void CHttpProbingClientUITest::fEndTransaction(long topInstanceID, long instanceID, 
														long parentID, unsigned long roundNum,
														const char* reason, const char* transactionName,
														double time, WLLogMessage::Severity severity,
														const char* transactionType, const char* extraData)
{
	printf("===================================================\n");
	printf("End Transaction: %s\n", transactionName);
	printf("topInstanceID=%d, instanceID=%d, parentID=%d\n", topInstanceID, instanceID, parentID);
	printf("reason=%s, time=%f, severity=%d, roundNum=%d\n", reason, time, severity, roundNum);
  
	if(extraData == NULL) {
		printf("transactionType=%s, extraData=NULL\n", transactionType);
	}
	else {
    CAssociativeArray ExtraData;
    ExtraData.FromString(extraData);

		printf("transactionType=%s, extraData=%d\n", transactionType, ExtraData.GetLongValueOfParameter("ActiveDomId"));
	}
	printf("===================================================\n");
}


void CHttpProbingClientUITest::ResetOutputFile()
{
  //create an empty file. if a previous file exists, it is deleted.
  FILE *outputFile;
#ifdef __TV_WINDOWS__
  outputFile = fopen(OUTPUT_FILENAME, "w");
#else
  outputFile = fopen(OUTPUT_FILENAME, "w");
#endif
	fclose(outputFile);
}


