/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef  _HTTP_CALLBACK_TEST_H_
#define _HTTP_CALLBACK_TEST_H_

#include "HttpCallback.h"
#include <RVCString.h>
#include <rw/gslist.h>

class cScriptFile ;

declare(RWGSlist, cScriptFile)


class cScriptFile: public iFile 
{

public:

	cScriptFile(const char* Name);
	virtual ~cScriptFile();

	const char* fGetName() const;
	
	void fSourceExport( iEmbedderContext& ecx, iSourceCommit& );
	
private:
	RVCString m_Name;

};



class cHttpProbingClientUI : public iHttpProbingClient
{
public:

  virtual ~cHttpProbingClientUI();

  tEmbedderReporterReturn pageStartDownload(iFile& file,tLineNumber lnNo,tColumn col,const char* line,const char *url,const char *original_url,const char *referer, const char *target);
  tEmbedderReporterReturn pageAbortDownload(iFile& file,tLineNumber lnNo,tColumn col,const char* line,const char *url,const char *original_url,const char *referer, const char *target);
  tEmbedderReporterReturn pageEndDownload(iFile& file,tLineNumber lnNo,tColumn col,const char* line,const char *url,const char *original_url,const char *referer, const char *target);
  tEmbedderReporterReturn objectStartDownload(iFile& file,tLineNumber lnNo,tColumn col,const char* line,const char *url,const char *original_url,const char *referer);
  tEmbedderReporterReturn objectAbortDownload(iFile& file,tLineNumber lnNo,tColumn col,const char* line,const char *url,const char *original_url,const char *referer, int httpStatus);
  tEmbedderReporterReturn objectEndDownload(iFile& file,tLineNumber lnNo,tColumn col,const char* line,const char *url,const char *original_url,const char *referer, HtmlQueueItemType urlType, void *objContent, int objContentLength);
  tEmbedderReporterReturn UseCachedObject(iFile& file,tLineNumber lnNo,tColumn col,const char* line,const char *url,const char *original_url,const char *referer, HtmlQueueItemType urlType);
  void fCopyFile( iEmbedderContext&, const char*, const char* ) {}
  
	void fBeforeInitAgenda( iInterruptionControlCommit& );
	void fBeforeTerminateAgenda( iInterruptionControlCommit& );
	void fBeforeOnErrorTerminateAgenda( iInterruptionControlCommit& );
	void fBeginClient( iInterruptionControlCommit& );
	void fBeforeInitClient( iInterruptionControlCommit& );
	void fBeforeTerminateClient( iInterruptionControlCommit& );
	void fBeforeOnErrorTerminateClient( iInterruptionControlCommit& );
	bool fBeforeRound( unsigned long, iInterruptionControlCommit& );
	void fBeforeOnScriptAbort( iInterruptionControlCommit& );
	void fEndClient( iInterruptionControlCommit& );

  
  void fReporter(WLLogMessage::Severity,const char *);

  iFile* fGetFile( bool& IsNew, iEmbedderContext&ecx, const char* Name) ;

private:
    RWGSlist(cScriptFile) m_FileList;


////////////////Debugger's pure virtual functions : 

	void fScriptHook( iScriptookable&, bool Creating ) {}

	tEmbedderReporterReturn fStubEmbedderReporter(iFile&,tLineNumber,tColumn,const char*,iEmbedderReportInfo&){return( sEmbedderReporterReturnContinue );}
		
  tReporterReturn fReporter(iFile& file,tLineNumber lineN,tColumn col,const char* line,iReportInfo& info);
  void fReportDMHook(tInterruptionControl& ctrl,iFile& file,tLineNumber line){}
  tNormalDMHookReturn vInterruptDMHook_iLoadProbingClient(tInterruptionControl&,iFile&,tLineNumber){return sNormalDMHookReturnContinue;}
	tNormalDMHookReturn fBreakPointDMHook(tInterruptionControl&,iFile&,tLineNumber){return sNormalDMHookReturnContinue;}
	tNormalDMHookReturn fKeywordDMHook(tInterruptionControl&,iFile&,tLineNumber){return sNormalDMHookReturnContinue;}
	tThrowDMHookReturn fThrowDMHook(tInterruptionControl&,iFile&,tLineNumber){return sThrowDMHookReturnApply;}

protected:

	tNormalDMHookReturn fDMHookX2(tInterruptionControl&,const char*,iFile&,tLineNumber){return sNormalDMHookReturnContinue;}

  bool fDMHookX1(tInterruptionControl&,const char*,iFile&,tLineNumber){return true;}
	
  bool fDMHookX0(const char*,iFile&,tLineNumber,tInterruptionControl&){return true;}

};


#endif // _HTTP_CALLBACK_TEST_H_

