/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <ace/OS.h>
#include "wlxmlcom.h"
#include "wlxmlfuncs.h"
#include <string.h>
#include "rvjsapi.h"
#include "xmlreplyprops.h"
#include "jsxmlreplynames.h"
#include "jsxmlprivate.h"
#include "jsxmlwrappers.h"

CWLXmlParseErrorWrapper::CWLXmlParseErrorWrapper(IXMLDOMParseErrorPtr* pParseErrorPtr) {
	m_pXMLParseError = pParseErrorPtr;
}
	

IXMLDOMParseErrorPtr* CWLXmlParseErrorWrapper::GetInterfacePtr() {
	return m_pXMLParseError;
}

CWLXmlParseErrorWrapper::~CWLXmlParseErrorWrapper() {
		if(m_pXMLParseError)
			delete m_pXMLParseError;
	}


static JSBool
xmlparseerror_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	TRY_COM {
		WlTraceMsg("Inside xmlparseerror_getProperty");
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		IXMLDOMParseErrorPtr pError = *(((CWLXmlParseErrorWrapper*)(prv->GetObj()))->GetInterfacePtr());
		if ((pError != NULL) && JSVAL_IS_STRING(id)) {
			JSString* str = JSVAL_TO_STRING(id);
			char* prop = JS_GetStringBytes(str);
			if (!strcmp(prop, WLJSNAME_PROP_ERRORCODE)) {				
				long errorCode = pError->GeterrorCode();
				return WLXmlSetIntValue(cx, obj, prop, (unsigned)errorCode, vp);
			}		
			else if (!strcmp(prop, WLJSNAME_PROP_FILEPOS)) {	
				long filePos = pError->Getfilepos();
				return WLXmlSetIntValue(cx, obj, prop, (unsigned)filePos, vp);
			}
			else if (!strcmp(prop, WLJSNAME_PROP_LINE)) {	
				long line = pError->Getline();
				return WLXmlSetIntValue(cx, obj, prop, (unsigned)line, vp);
			}
			else if (!strcmp(prop, WLJSNAME_PROP_LINEPOS)) {	
				long linePos = pError->Getlinepos();
				return WLXmlSetIntValue(cx, obj, prop, (unsigned)linePos, vp);
			}
			else if (!strcmp(prop, WLJSNAME_PROP_REASON)) {	
				_bstr_t reason = pError->Getreason();
				if(!reason)
					reason = "";
				return WLXmlSetStringValue(cx, obj, NULL, (char*)reason, vp);
			}
			else if (!strcmp(prop, WLJSNAME_PROP_SRCTEXT)) {	
				_bstr_t src = pError->GetsrcText();
				if(!src)
					src = "";
				return WLXmlSetStringValue(cx, obj, NULL, (char*)src, vp);				
			}
			else if (!strcmp(prop, WLJSNAME_PROP_URL)) {	
				_bstr_t url = pError->Geturl();
				if(!url)
					url = "";
				return WLXmlSetStringValue(cx, obj, NULL, (char*)url, vp);
			}
		}
		return JS_TRUE;
	}
	CATCH_COM(cx)
}


static JSBool
xmlparseerror_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp) {
	WlTraceMsg("Inside xmlparseerror_setProperty");
	return JS_TRUE;
}


static void
xmlparseerror_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside xmlparseerror_finalize(%x, %x)", cx, obj);
	WLXmlFinalize(cx, obj);
}																																																												

JSClass wl_XmlParseErrorClass = {
	WLJSNAME_CLASS_XMLPARSE_ERROR,
	JSCLASS_HAS_PRIVATE,	//ptr to msxml interface structure that info regarding the object it represents
	JS_PropertyStub,			//callback to C++
	JS_PropertyStub,			//--"--
	xmlparseerror_getProperty,
	xmlparseerror_setProperty,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	xmlparseerror_finalize
};

//constructor
static JSBool
WLXmlParseError(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
//no constructor for doc type
	return JS_TRUE;
}

JSObject* WLNewXmlParseErrorObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewXmlParseErrorObject(%x, %x)", cx, obj);
	// create an instance 
	JSObject* proto = NULL;
	JSObject* xmlparseerror = WLNewXmlObject(cx, &wl_XmlParseErrorClass, proto, obj);
	if (!xmlparseerror)
		return NULL;

	// define it's methods
//	if (!JS_DefineFunctions(cx, script, script_methods))
//		return NULL;

	return xmlparseerror;
}


bool DllExport WLAddXmlParseErrorClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddXmlParseErrorClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_XmlParseErrorClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
