/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <ace/OS.h>
#include "wlxmlfuncs.h"
#include <collstr.h>
#include <string.h>
#include "rvjsapi.h"

#include "platform.h"
#include "LoadEngineGlobals.h"
#include "xmlreplyprops.h"
#include "jsxmlreplynames.h"
#include "jsxmlprivate.h"
#include "wlframecontainer.h"

#include "xmlerrorreporter.h"
#include "tracedll.h"

#define DELETED "DELETED"


extern JSObject* WLNewXmlNodeObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewXmlDocumentObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewXmlDocumentObject(JSContext *cx, JSObject *obj, CWLXmlDocument* xml, CWLDocument* doc);
extern JSObject* WLNewXmlNodeListObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewXmlParseErrorObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewXmlNamedNodeMapObject(JSContext *cx, JSObject *obj);
extern JSObject* WLNewXmlImplementationObject(JSContext *cx, JSObject *obj);

#include "tracedll.h"



wlXmlError::wlXmlError(RVCString& err)
{
	errorRef = err.data();
}

const char* wlXmlError::ErrorMessage()
{
	return errorRef.data();
}


CWLXmlInterfacePtr::CWLXmlInterfacePtr(IXMLDOMDocumentPtr* newDocPtr)
{
	m_pDocPtr = newDocPtr;
}

CWLXmlInterfacePtr::~CWLXmlInterfacePtr()
{
	delete m_pDocPtr;
}

IXMLDOMDocumentPtr* CWLXmlInterfacePtr::GetInterfacePtr()
{
	return m_pDocPtr;
}



bool WLXmlDeleteProperty(JSContext* cx, JSObject* obj, const char* name)
{
	jsval v;
	JSBool res = JS_LookupProperty(cx, obj, name, &v);
	if (res && JSVAL_IS_OBJECT(v)) {
		JSObject* prop = JSVAL_TO_OBJECT(v);
		WLXmlFinalize(cx, prop);
		JSBool found;
		JS_SetPropertyAttributes(cx, obj, name, 0, &found);
		JS_DeleteProperty(cx, obj, name);
		return true;
	}

	return false;
}

void WLXmlFinalize(JSContext* cx, JSObject* obj)
{
	CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
	if (prv) {
		CWLDocument* parent = prv->GetParent();
    if (parent) {
      WlTraceMsg( "Decreasing reference to the frame : %d",(int)parent);
      if (parent->decRefCount() == 0) {
        WlTraceMsg( "No more reference exist to the frame : %d so we delete it",(int)parent);
				delete parent;				
      }
    }
		delete prv;
		JS_SetPrivate(cx, obj, NULL);
	}
}

JSObject* WLNewXmlObject(JSContext* cx, JSClass* cls, JSObject* proto, JSObject* parent)
{
	JSObject* obj = JS_NewObject(cx, cls, NULL, parent);
	return obj;
}

void WLSetNewXmlObject(JSContext* cx, JSObject* obj, JSObject* newObj, const char* name, void* value, bool toDelete)
{
	CWLXmlPrivate* prv = new CWLXmlPrivate((void*)value);
	JS_SetPrivate(cx, newObj, (void*)prv);
}

// for the following WLSet... functions:
	// using the following call will cause lvalue to be new instance of the property
	// eliminating the call will cause lvalue to point to the property
//	JS_SetProperty(cx, obj, name, rval);


JSBool DllExport WLXmlSetIntValue(JSContext* cx, JSObject* obj, const char* name, unsigned value, jsval* rval)
{
	*rval = INT_TO_JSVAL((int)value);
	return JS_TRUE;
}

JSBool DllExport WLXmlSetBoolValue(JSContext* cx, JSObject* obj, const char* name, bool value, jsval* rval)
{
	if (value)
		*rval = BOOLEAN_TO_JSVAL(JS_TRUE);
	else
		*rval = BOOLEAN_TO_JSVAL(JS_FALSE);
	return JS_TRUE;
}

JSBool DllExport WLXmlSetDoubleValue(JSContext* cx, JSObject* obj, const char* name, double value, jsval* rval)
{
	jsdouble* val = JS_NewDouble(cx, (jsdouble)value);
	*rval = DOUBLE_TO_JSVAL(val);

	return JS_TRUE;
}

JSBool DllExport WLXmlSetStringValue(JSContext* cx, JSObject* obj, const char* name, const char* value, jsval* rval, size_t length)
{
	if (!value)
		return JS_TRUE;

	JSString* str = NULL;
	if (length == 0)
		str =JS_NewStringCopyZ(cx, value);
	else
		str =JS_NewStringCopyN(cx, value, length);
	if (!str)
		return JS_FALSE;

	*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}

JSBool DllExport WLXmlSetNodeInterfaceValue(JSContext* cx, JSObject* obj, CWLXmlNodeWrapper* value,jsval*  rval)
{
	if(*(value->GetInterfacePtr()) == NULL){
		*rval = OBJECT_TO_JSVAL(0);
		return JS_TRUE;
	}
	JSObject* XMLNode = WLNewXmlNodeObject(cx, obj);
		if(!XMLNode)
			return JS_FALSE;

	WLSetNewXmlObject(cx, obj, XMLNode, WLJSNAME_CLASS_XML_NODE, (void*)value);
	*rval = OBJECT_TO_JSVAL(XMLNode);
	return JS_TRUE;
}

//this is used by objects in the dom to return the document interface
JSBool DllExport WLXmlSetDocumentInterfaceValue(JSContext* cx, JSObject* obj, CWLXmlDocumentWrapper* value, jsval*  rval)
{
	if(value == NULL)
		return JS_TRUE;
	JSObject* XMLDoc = WLNewXmlDocumentObject(cx, obj);
		if(!XMLDoc)
			return JS_FALSE;

	WLSetNewXmlObject(cx, obj, XMLDoc, WLJSNAME_CLASS_XMLDOCUMENT, (void*)value);
	*rval = OBJECT_TO_JSVAL(XMLDoc);
	return JS_TRUE;
}

//this is used by jsXml to create xml documents from html object
JSBool DllExport WLXmlSetDocumentInterfaceValue(JSContext* cx, JSObject* obj, CWLDocument* parent, CWLXmlDocument* xml, bool htmlProps, jsval* rval)
{
	//create the document object
	JSObject* XMLDoc = WLNewXmlDocumentObject(cx, obj, xml, parent);
		if(!XMLDoc)
			return JS_FALSE;

	*rval = OBJECT_TO_JSVAL(XMLDoc);
	return JS_TRUE;
}


JSBool DllExport WLXmlSetNodeListInterfaceValue(JSContext* cx, JSObject* obj,CWLXmlNodeListWrapper* value,jsval*  rval)
{
	if(value == NULL)
		return JS_TRUE;
	JSObject* XMLNodeList = WLNewXmlNodeListObject(cx, obj);
		if(!XMLNodeList)
			return JS_FALSE;

	WLSetNewXmlObject(cx, obj, XMLNodeList, WLJSNAME_CLASS_XML_NODELIST, (void*)value);
	*rval = OBJECT_TO_JSVAL(XMLNodeList);
	return JS_TRUE;
}

JSBool DllExport WLXmlSetNamedNodeMapInterfaceValue(JSContext* cx, JSObject* obj,CWLXmlNamedNodeMapWrapper* value, jsval*  rval)
{
	if(value == NULL)
		return JS_TRUE;//?
	JSObject* XMLNamedNodeMap = WLNewXmlNamedNodeMapObject(cx, obj);
	if(!XMLNamedNodeMap)
		return JS_FALSE;

	WLSetNewXmlObject(cx, obj, XMLNamedNodeMap, WLJSNAME_CLASS_XMLNAMED_NODEMAP, (void*)value);
  *rval = OBJECT_TO_JSVAL(XMLNamedNodeMap);
	return JS_TRUE;	
}

JSBool DllExport WLXmlSetParseErrorInterfaceValue(JSContext* cx, JSObject* obj,CWLXmlParseErrorWrapper* value, jsval* rval){
	if(value == NULL)
		return JS_TRUE;//?
	JSObject* XMLParseError = WLNewXmlParseErrorObject(cx, obj);
	if(!XMLParseError)
		return JS_FALSE;

	WLSetNewXmlObject(cx, obj, XMLParseError, WLJSNAME_CLASS_XMLPARSE_ERROR, (void*)value);
  *rval = OBJECT_TO_JSVAL(XMLParseError);
	return JS_TRUE;	
}

JSBool WLXmlSetImplementationInterfaceValue(JSContext* cx, JSObject* obj,CWLXmlImplementationWrapper* value, jsval* rval)
{
	if(value == NULL)
		return JS_TRUE;//?
	JSObject* XMLImplementation = WLNewXmlImplementationObject(cx, obj);
	if(!XMLImplementation)
		return JS_FALSE;

	WLSetNewXmlObject(cx, obj, XMLImplementation, WLJSNAME_CLASS_XMLIMPLEMENTATION, (void*)value);
  *rval = OBJECT_TO_JSVAL(XMLImplementation);
	return JS_TRUE;	
}


bool WLXmlJsvalToVariant(jsval& vp, _variant_t& var)
{
	if(JSVAL_IS_INT(vp)){
		var = (long)JSVAL_TO_INT(vp);
		return true;
	}
	else if(JSVAL_IS_DOUBLE(vp)){
		var = *(double*)JSVAL_TO_DOUBLE(vp);
		return true;
	}
	else if(JSVAL_IS_STRING(vp)){
		var = (char*)JS_GetStringBytes(JSVAL_TO_STRING(vp));
		return true;
	}
	else if(JSVAL_IS_BOOLEAN(vp)){
		VARIANT_BOOL temp = JSVAL_TO_BOOLEAN(vp) == JS_TRUE ? -1 : 0;
		var = temp;
		return true;
	}
	return false;//exhausted all types in js - cannot be an object.
}


JSBool ReportXMLError(JSContext* cx, const char* frmName, int frmId, ...)
{
  WLLogMessage::Severity sev = WLERRGetErrorSeverity(frmName, frmId);

  char    buf[1024];
  va_list ap;

  va_start(ap, frmId);
	  vsprintf(buf, WLERRGetErrorFormat(frmName,frmId), ap);
  va_end(ap);

  // @@Chaim - Report through loadengine?
	//CWLCommandManager* ptrHttp = (CWLCommandManager*)getCurrentThreadData();
//	CWLCommandManager* ptrHttpThread = WLGetCommandManagerPtr(cx);
//	if (sev > WLLogMessage::Note && ptrHttpThread) {
//		return ptrHttpThread->report(sev, (const char*)buf);
//		}

	return JS_TRUE;
}


JSBool	succeeded(JSContext* cx, HRESULT hr)
{
  WlAssert(piHLErrorReporter);
	if(!SUCCEEDED(hr))
		return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 11);
	return JS_TRUE;
}

JSBool WLXmlSetVariantValue(JSContext *cx, JSObject *obj, _variant_t& var, jsval* vp)
{
  WlAssert(piHLErrorReporter);
	if(var.vt == VT_BSTR)
	{
		_bstr_t value(var.bstrVal, false);
		return WLXmlSetStringValue( cx,  obj, NULL, (char*)value, vp);
	}
	else if(var.vt == VT_BOOL)
	{
		bool boolval = bool(var);
		return  WLXmlSetBoolValue( cx,  obj, NULL, boolval, vp);
	}
	else if(var.vt == VT_I4)
	{
		long  longval = long(var);
		return WLXmlSetIntValue( cx,  obj, NULL, longval, vp);				
	}
	else if(var.vt == VT_I2)
	{
		short  shortval = short(var);
		return WLXmlSetIntValue( cx,  obj, NULL, shortval,  vp);		
	}
	else if(var.vt == VT_R4 || var.vt == VT_R8)
	{
		double doubleVal = double(var);
		return WLXmlSetDoubleValue(cx, obj, NULL, doubleVal, vp);
	}
	else if(var.vt == VT_NULL)
		return JS_TRUE;
	return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 14);//other VARTYPE we do not handle or ??
}

// index == -1 indicate that a name is used instead of an index.

bool checkNodeTypeByString(char* type,...)
{
	va_list ap;
	char* var;
	va_start(ap, type);
	while(1){
		if(!(var = va_arg(ap, char*)))
			return false;
		else if(!strcmp(type, var))
			return true;
	}
  va_end(ap);
	return false;
}

int checkNodeTypeByInt(int type,...)
{
	int var = 0;
	va_list ap;
	va_start(ap, type);		
	while(var != -1){
		var = va_arg(ap, int);
		if(type == var)
			return type;
	}
  va_end(ap);
	return 0;
}

char* ConvertNodeTypeToString(int type)
{
	switch(type){
		case(1):
			return "element";
		case(2):
			return "attribute";
		case(3):
			return "text";
		case(4):
			return "cdatasection";
		case(5):
			return "entityreference";
		case(6):
			return "entity";
		case(7):
			return "processingInstruction";
		case(8):
			return "comment";
		case(9):
			return "document";
		case(10):
			return "documenttype"; 
		case(11):
			return "documentfragment";
		case(12):
			return "notation";
	default:
			return NULL;
	}
}


void COMErrorDescription(
	_com_error err,
	RVCString& description )
	{
	LPOLESTR temp_str_w;

	if( err.ErrorInfo() ) {
		temp_str_w = (LPOLESTR)err.Description();
		USES_CONVERSION;
		description = OLE2A(temp_str_w);
		}
	else {
		description = err.ErrorMessage();
		}

	}


RVCString COMErrorDescription(
	_com_error err
	) {
	RVCString ret;
	COMErrorDescription( err, ret );
	return ret;
	}

JSBool COMReportError(
	JSContext* cx,
	_com_error& err )
	{
  WlAssert(piHLErrorReporter);
	JSBool ret;

	switch( err.Error() ) {
		
		case -2147221164L: //Class is not registered.
			ret = piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 99 );
			break;
			
		default: {
			ret = piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 10, COMErrorDescription(err).data() );
			}

		}

	return ret;
	}
