/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef _GeneralProcess_H
#define _GeneralProcess_H

#define DEFAULT_LE_SLEEP_RANDOM_MIN			-1
#define DEFAULT_LE_SLEEP_RANDOM_MAX			-1
#define DEFAULT_LE_SLEEP_DEVIATION			-1
#define DEFAULT_LE_TIMEOUT_BEFORE_OPERATION -1


class iLEProtocolProcess;
class cDll;
class CJSLoadRuntime;

#include "platform.h"
#include "loadengineexternaldecs.h"

#include <collint.h>
#include "jsjavacore.h"
#include <hashdict.h>
#include <thr.h>
#include "rvjsmachinethread.h"
#include "wlleprotocolcontainer.h"
#include "wldlls.h"

//####### The declaration of the class CPropIndex   #########

class DllExport CPropIndex {

public:
	explicit CPropIndex();
	~CPropIndex();
	void RegisterLEProperties();

public:
	int m_IndexDisableSleep;
	int m_IndexSleepRandomMin;
	int m_IndexSleepRandomMax;
	int m_IndexSleepDeviation;
	int m_IndexExceptionEnabled;
	int	m_ReportCSJSInternalErrors;
	int m_IndexTimeoutBeforeOperation;
	int m_IndexSendClientStatistics;
	int m_IndexSendClientStatisticsFilter;
	int	m_IndexJVMType;
	int	m_IndexCharEncoding;
};



//####### The declaration of the class CGeneralProcess  #########

class DllExport CGeneralProcess {
private:
	static CGeneralProcess* msSingleton;
public:
	static CGeneralProcess& fsGetSingleton();
public:

	explicit CGeneralProcess();
	virtual ~CGeneralProcess();
	
	bool Init(RVCString& oDllName);
	void Terminate();

	void fThreadStart( cLEJSMachineThreadKey&, iLEJSMachineThread& );
	void fThreadEnd( cLEJSMachineThreadKey& );

	iLEJSMachineThread* fGetifThread( cLEJSMachineThreadKey& );
	iLEJSMachineThread& fGetThread( cLEJSMachineThreadKey& );
	iLEJSMachineThread* fGetifCurrentThread();
	iLEJSMachineThread& fGetCurrentThread();
	double fGetCurrentTime() {return vGetCurrentTime();}


																							//THE DIRECTORY WITH ALL IT"S FILE
public:
	//functions for supporting C++ protocols
	WLProtocolIndex GetNumProtocols();
  cDLLContainer& GetDllProtocols();
	iLEProtocolContainer& GetLEProtocolProcessContainer();
  void ProcessInitializeJava(cRVJVM&, JNIEnv*);  
  void ProcessFinalizeJava(cRVJVM&, JNIEnv*);  

  void ProtocolProcessAfterStartLoad(int ReportResolutionSize);

	//other functions
	RVCString& GetTempDirectory() { return m_TempDirectory; }
	int GetMasterPid();//Note: This data member 'm_MasterPid' is not being set when an instance of that class is
										//being initialized. It gets is value only in the main loop of the slave. Thus when you call
										// the function 'GetMasterPid' it may cause an assert if the data member is still not initialized.
	void SetMasterPid( int masterPid);

	cRVJVMManager& fGetJVMManager();

	bool fIsJSGCInExecutionContext();

  CJSLoadRuntime& GetLoadRuntime() { assert(m_LoadJSRuntime); return *m_LoadJSRuntime; };

public:
	//contains the indexes of the LE global properties.
	CPropIndex m_PropIndex;	

private:

  bool  m_InitPerformed;
	//data members for supporting C++ protocols
  cDLLContainer m_DllProtocols;

  // Chaim - this must be destroyed before the DLL's are unloaded
  // in m_DllProtocols dtor.
  iLEProtocolContainer m_ProtocolProcessContainer;

  typedef iLEProtocolProcess* (*PFN_NewProtocolProcces) ();
	
	//other data members
	int m_MasterPid;//Note: This data member is not being set when an instance of that class is
									//being initialized. It gets is value only in the main loop of the slave. Thus when you call
									// the function 'GetMasterPid' it may cause an assert if the data member is still not initialized.

	RVCString m_TempDirectory;

	cRVJVMManager mJVMManager;

	RVHashDictionary mThreadsDict;
	mutable RVReadersWriterLock mThreadsDictLock;

  CJSLoadRuntime* m_LoadJSRuntime;

	virtual double vGetCurrentTime();
	bool mIsJSGCInExecutionContext; // evaluate once 

  bool InitProtocols(WLProtocolIndex, RVCString& /*[out]*/);//Init the c++ protocols (wlhttp128.dll, jscomobject.dll, etc)
  bool ProtocolProcessInit(RVCString& oDllName);
  void ProtocolProcessTermination();

  void ExceptionHandler(std::exception*);

};



#endif

