/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef dJSJavaCore_h_
#define dJSJavaCore_h_

class CGeneralProcess;

#include "tracedll.h"
#include "platform.h"
#include "rvjsapi.h"
#include "rvjsjava.h"
#include <thr.h>
#include <thr.h>
#include "rvextras_liveconnect.h"
#include "tvpsync.h"
//#include "exception"
#include <exception>

class DllExport cRVJVM;
class DllExport cRVJVMManager;

//static bool fsAttachCurrentThread( JavaVM*, JNIEnv*& oJEnv ); // called by f[same] or fsJSJ[same]
//static JavaVM* fsCreateJVM( JavaVM*& oJVM, JNIEnv*& oJEnv );

typedef jint (WINAPI *tfsJNICreateJavaVM)( JavaVM** o, void** oJEnv, void* rArgs );

class DllExport cRVJVM
{
// instance will hold the needed stuff
// handles both JavaVM and JSJavaVM, lazily creates only the needed.
public:
	explicit cRVJVM( cRVJVMManager& );
	virtual ~cRVJVM();

	bool fInit();
  bool fJSJFinal();
	bool fFinal();

private:
	bool fCreateJVMInternal( JavaVM*& oJVM, JNIEnv*& oJEnv );

	void fJVMMainThread();
	bool fCreateJVM( JNIEnv*& );

public:
// for LoadEngine
	bool fAttachCurrentThread( JNIEnv*& o );
	bool fJSJAttachCurrentThread( JSJavaThreadState*& o, JNIEnv* );

	bool fDetachCurrentThread();
	bool fJSJDetachCurrentThread( JSJavaThreadState* );

	void fAddClassPath( const char* );
	void fSetClassPath( const char* );
	void fSetJIT( const char* );
	void fSetJVMPath( const char* );

        bool fCreateJavaVM(); 

public:
// for RVJSJ
	JavaVM* fGetJVM();
	JNIEnv* fGetEnv(); 
	cRVJVMManager& fGetManager();

private:
	cRVJVMManager& mManager;
	JavaVM* volatile mJVM;
	JNIEnv* volatile mJNIEnv;
	JSJavaVM* volatile mJSJVM;
	mutable RVCriticalSection mJVMLock;		

#ifdef __TV_UNIX__
	void* m_hJVMDLL;
#else
	HINSTANCE m_hJVMDLL;
#endif //__TV_UNIX__

	RVCString mClassPath;
	RVCString mJIT;
	RVCString mJVMPath;

	RVThread mJVMThread;
	TVPManualEvent mJVMStartThreadEvent;
	TVPManualEvent mJVMTerminateLiveConnectEvent;
	TVPManualEvent mJVMTerminateLiveConnectDoneEvent;
	TVPManualEvent mJVMDestroyJVMEvent;
	TVPManualEvent mJVMDestroyJVMDoneEvent;

	static const char* const msxClassPathSep;

	bool fsCallInitFunc( cRVJVM& rJVM, JNIEnv* rJEnv );
	
	bool fsCallFinalFunc( cRVJVM& rJVM, JNIEnv* rJEnv );

  void ExceptionHandler(std::exception*);

private:
	explicit cRVJVM(cRVJVM& r): mManager(r.mManager) {WlAssert(0);}	
};

class DllExport cRVJVMManager 
{
// assumes singleton in process
private:
	static cRVJVMManager* msSingleton;
	static cRVJVMManager& fsGetSingleton();
public:
	explicit cRVJVMManager( CGeneralProcess& );
	virtual ~cRVJVMManager();

	bool fInit();
  bool fJSJFinal();
	bool fFinal();
	
	cRVJVM& fGetJVM();

	static bool fsJSInit( JSContext*, JSObject* );

// RVJSJ API
	CGeneralProcess& fGetProcess();	

private:
	CGeneralProcess& mProcess;
	cRVJVM mJVM; // currently supporting one JVM


// JSJCallbacks	
  static JSContext* fsJSJMapJSJThreadToJSContext(
		JSJavaThreadState* rJSJEnv,
#ifdef OJI
		void *rJavaAppletObj,
#endif
		JNIEnv *rJEnv,
		char** rError);

	static JSJavaThreadState* fsJSJMapJSContextToJSJThread( JSContext*, char** oErr );

	static JSObject* fsJSJMapJavaObjectToJSObject( JNIEnv*, void* rJavaObject, char** rErr );

	static void fsJSJErrorReporter( const char* rMsg );

	static JSBool fsJSJCreateVM( SystemJavaVM** oJVM, JNIEnv** oJEnv, void* rArgs );
	static JSBool fsJSJDestroyVM( SystemJavaVM*, JNIEnv* );

	static JNIEnv* fsJSJAttachCurrentThread( SystemJavaVM* );
  static JSBool fsJSJDetachCurrentThread( SystemJavaVM*, JNIEnv* );

	static SystemJavaVM* fsJSJGetJVM( JNIEnv* );

	static JSJCallbacks msxJSJCallbacks;

	static iRVJSJSection* fsEnterJava( JSContext*, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType );
	static void fsExitJava( JSContext*, JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType, iRVJSJSection* );

};


#endif
