/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// LoadEngineVerification.h: interface for the CWLTransactionManager class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LOADENGINEVERIFICATION_H__C59C4073_136B_11D3_B5DC_00A0C9458D0D__INCLUDED_)
#define AFX_LOADENGINEVERIFICATION_H__C59C4073_136B_11D3_B5DC_00A0C9458D0D__INCLUDED_

#include <collstr.h>
#include <hashdict.h>
#include <ordered.h>

#include "wlerrorlib.h"
#include "loadgeneratorslave.h"

class DllExport IGeneralThreadTransactionManager;
class CLoadThreadInstanceScript;
class CLoadTransaction;
class CLoadTransactionEvent;
class CLoadThreadsManager;


class IGeneralThreadTransactionManager {
  friend class CLoadTransaction;
private:
  unsigned long             m_TransactionIdBase;
	unsigned long             m_TransactionTopInstanceIdBase;
  RVCString                 m_FailureReason;

  
  bool                      m_ErrorOccured;
  bool                      m_RecordLogMessages;
  bool                      m_Save;

protected:
  RVOrdered                 m_TransactionsList;
  RVOrdered                 m_TransactionsStack;
  RVOrdered                 m_EventsList;

private:

  CLoadTransaction*         findTransaction(const char* transactionName);
  void                      Reset();
	void                      Send();
  void                      SendEvent(const char* reason, const char* description, double time, WLLogMessage::Severity severity, const char* extraDataType, void* extraData, unsigned long len, bool IsLog, bool IsReport );
  void                      SendAllEvents();

	WLLogMessage::Severity GetErrorSeverity();
	bool Report( WLErrorException& );
	bool ReportCond( bool Cond, WLErrorException& );
	int GetAgendaNumRunning();

  virtual void AddLast(const char*, double, CWLStatPointTypeEnum::PointType) = 0;

	virtual CLoadGeneralThreadInstance* GetThreadInstance() = 0;
	virtual void SendWhole() = 0;
  virtual void SendEvent( CLoadTransactionEvent& ) = 0;

  virtual bool IsSendTransactions() = 0;
  virtual double vGetTimeFromBegin() = 0;
	virtual double vGetBeginTime() = 0;
	virtual double vGetCurrentTime() = 0;

	// Eyal 28.03.00
	virtual void RecordEventCallBack(CLoadTransactionEvent&, long topInstanceID, long instanceID ) = 0;
  virtual void BeginTransactionCallBack(CLoadTransaction& ) = 0;
  virtual void EndTransactionCallBack(CLoadTransaction& ) = 0;

public:
	IGeneralThreadTransactionManager();
	virtual ~IGeneralThreadTransactionManager();

  void SetFailureReason(const char* reason, const char* description, double time, bool isEvent );
  void RecordEvent(const char* reason, const char* description, double time, WLLogMessage::Severity severity, const char* extraDataType, void* extraData, unsigned long len, bool IsLog, bool IsReport);
  bool BeginTransaction(const char* transname, double time, const char* extraDataType, void* extraData, unsigned long len);
  bool EndTransaction(const char* transname, double time, WLLogMessage::Severity severity, bool save, bool SendTimers, bool IsLog );
  bool EndTransactionFailure(const char* transname, double time, WLLogMessage::Severity severity, const char* reason, bool SendTimers, bool IsLog, bool isFailureEvent );
  void SendFailureForOpenTransactions(bool Failed, bool SendTimers);

  const char* GetCurrentTransaction();
  int         GetTransactionDepth();

private:

  void InsertEvent(CLoadTransactionEvent *event);

};




class CLoadThreadTransactionManager: public IGeneralThreadTransactionManager {

private:
  CLoadThreadInstanceScript *m_MyThreadInstanceScript;

	CLoadGeneratorSlave* GetLoadGenerator();

public:
//	CLoadThreadTransactionManager(CLoadThreadInstanceScript*);
		CLoadThreadTransactionManager() {}

		void InitParent(CLoadThreadInstanceScript*);
// IGeneralThreadTransactionManager 
private:
	CLoadGeneralThreadInstance* GetThreadInstance();

  void AddLast(const char*, double, CWLStatPointTypeEnum::PointType);

  bool IsSendTransactions();
  double vGetTimeFromBegin();
	double vGetBeginTime();
	double vGetCurrentTime();

	void SendWhole();
  void SendEvent(CLoadTransactionEvent &event);

	virtual void RecordEventCallBack(CLoadTransactionEvent&, long topInstanceID, long instanceID) {}
  virtual void BeginTransactionCallBack(CLoadTransaction& )  {}
  virtual void EndTransactionCallBack(CLoadTransaction& )    {}

	};


#endif // !defined(AFX_LOADENGINEVERIFICATION_H__C59C4073_136B_11D3_B5DC_00A0C9458D0D__INCLUDED_)

