/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef __TV_READ_FILE_H
#define __TV_READ_FILE_H
#include <ordered.h>
#include <RVCString.h>
#include <slistcolque.h>
#include <collstr.h>
#include <hashdict.h>
//#include <thr.h>
//#include <thr.h>
#include "platform.h"
#include "tvpsync.h"
#include "loadgeneratorslaveproxy.h"
#include "loadagendacontext.h"
#ifdef __TV_WINDOWS__
//#include "loadagendamanager.h"
#else
//#include "loadagendamanager.h"
#endif
#include "associativearray.h"

class CLoadAgendaManager;
class CLoadGeneralThreadInstanceScript;

#ifndef LINE_SIZE
#define LINE_SIZE       10240
#endif

class CWLInputLine : public RVCollectableString {
private:
  long m_LineNumber;
  long m_FileRound;
public:
  CWLInputLine(const char* line, long lineNumber, long fileRound);
  virtual ~CWLInputLine();

  const char* GetLine();
  long GetLineNumber();
  long GetFileRound();
};


class CWLInputFileHandler : public RVHashDictionary {
	RVCriticalSection creationMutex;
  CLoadAgendaManager* m_LoadAgendaManager;
  int m_ChunksInQueue;
public:
  CWLInputFileHandler::CWLInputFileHandler() : m_ChunksInQueue(1) {};
  virtual CWLInputFileHandler::~CWLInputFileHandler() {};
  CAssociativeArray* InputFileActions(CAssociativeArray&);
  void Terminate();
  void SetLoadAgendaManager(CLoadAgendaManager* lam) {m_LoadAgendaManager = lam;};
  void SetChunksInQueue(int num) {m_ChunksInQueue = num;};
  int ChunksInQueue() {return m_ChunksInQueue;};
	CLoadAgendaManager* GetLoadAgendaManager() {return m_LoadAgendaManager;};
};

class CWLFileDictionary1;

class CWLStoreQueue : public RVSlistCollectablesQueue {

private:
  int               valid;
  RVCString         file_name;
  FILE              *input;
  size_t            maxItems;
  int               linenum;
  int               fileround;
  TVPSysThread      filler_thr;
  RVCriticalSection mutex;
  TVPManualEvent    event;
  TVPManualEvent    m_TerminateEvent;
  bool              m_Terminate;
	int								m_LinesPerChunk;
  // Random vars
  long       m_lFileSize;
  bool       m_Random;
  CWLFileDictionary1* m_FileDict;

  int               CreateThreadForReading();
  int               FillTheQueue();
  int               QueueStart();

public:
  CWLStoreQueue(const char* filename,
                      bool  random,
                CWLFileDictionary1* filedict,
								long MaxThreadsPerSlave);
  virtual ~CWLStoreQueue();

  int           isValid() {return valid;}
  void          Terminate();
  CAssociativeArray* GetNextItem();
  void          QueueMainLoop();
  void          ClearQueue();
  void SetChunksInQueue(int num);
};


class CWLFileDictionary1 : public RVHashDictionary {
private:
  RVCriticalSection CreationMutex;
  CWLInputFileHandler* m_FileHandler;
  const char* m_ScriptName;
public:
  CWLFileDictionary1(CWLInputFileHandler*, const char*);
  virtual ~CWLFileDictionary1();

  int             isValid();

  int             AddFile(const char* filename, bool random);
  CAssociativeArray* GetNextItem(const char* filename);
  void            Terminate();
  int             ChunksInQueue();
  const char*     ScriptName();
};

class IGeneralManagerScriptData;
class CAssociativeArray;


class CWLFileDictionary : public RVHashDictionary {
private:
	bool m_Terminate;
	long m_MinItems;
  RVCriticalSection CreationMutex;
	IGeneralManagerScriptData* m_ManagerScriptData;
//        RVTRecursiveLock<RVMutexLock> GetMoreLines;
public:
	explicit CWLFileDictionary( IGeneralManagerScriptData* );
	virtual ~CWLFileDictionary();
	bool AddFile(const char* filename, bool random);
	CWLInputLine* GetNextLine(const char* filename, iLEAgendaJSEmbedContext* embedContext);
	void FillTheQueue(CAssociativeArray* AssoArray);
//	void Release();
	void Terminate();
};


class CFileDictKey : public RVCollectableString {
public:
  CFileDictKey(const char*);
  CFileDictKey(const RVCString&);
  virtual ~CFileDictKey() {}
  virtual bool isEqual(const RVCollectable* a) const;
};
#endif // of __TV_READ_FILE_H

