/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef __TV_READ_FILE_INTERFACE_H
#define __TV_READ_FILE_INTERFACE_H

#include <RVCString.h>

#include "loadinputfile.h"

#include "rvjsapi.h"

class IGeneralManagerScriptData;
class CLoadGeneralThreadInstanceScript;

class DllExport IFileDictionary;
class DllExport CSCFileDictionary;
class IInputFile;
class ISCInputFile;
class CSCSeqInputFile;
class CSCRandomInputFile;


class IInputLineOutpar {

public:
	virtual void LetLine( const char* ) = 0;
	virtual void LetLine( const RVCString& ) = 0;
	virtual void LetLineNumber( long ) = 0;
	virtual void LetFileRound( long ) = 0;
	};


class CInputLineBuf:
	public virtual IInputLineOutpar {

public:
	explicit CInputLineBuf();
	virtual ~CInputLineBuf();

	const char* GetLine();
	bool IfLineNumber();
	long GetLineNumber();
	bool IfFileRound();
	long GetFileRound();


private:
	RVCString mLine;
	bool mIfLineNumber;
	long mLineNumber;
	bool mIfFileRound;
	long mFileRound;

	void LetLine( const char* );
	void LetLine( const RVCString& );
	void LetLineNumber( long );
	void LetFileRound( long );
	};




class IFileDictionary {

public:
	virtual ~IFileDictionary();

	virtual bool AddFile( const char*, bool, JSContext* ) = 0;
	virtual bool ReadLine( const char*, IInputLineOutpar&, iLEAgendaJSEmbedContext*, JSContext* ) = 0;

	};



class CLoadFileDictionary: public IFileDictionary {

public:
	explicit CLoadFileDictionary( IGeneralManagerScriptData* );

	bool AddFile( const char*, bool, JSContext* );
  bool ReadLine( const char*, IInputLineOutpar&, iLEAgendaJSEmbedContext*, JSContext* );

	CWLFileDictionary& GetWLFileDict(); 

private:
	CWLFileDictionary m_Files;
	};


class CSCFileDictionary: public IFileDictionary {

public:
	explicit CSCFileDictionary();
	virtual ~CSCFileDictionary();

	bool AddFile( const char*, bool, JSContext* );
	bool ReadLine( const char*, IInputLineOutpar&, iLEAgendaJSEmbedContext*, JSContext* );

private:
	RVHashDictionary mFiles;

	ISCInputFile* NewFile( bool );
	bool Add( ISCInputFile*, const char*, JSContext* );
	};



class IInputFile: public RVCollectable {

public:
	virtual ~IInputFile();

	virtual bool Open( const char* ) = 0;
	virtual bool ReadLine( IInputLineOutpar& ) = 0;
	};


class ISCInputFile:	public IInputFile {

public:
	explicit ISCInputFile();
	virtual ~ISCInputFile();

protected:
	FILE* m_File;
	long m_Round;
	long m_LineNum;

	bool Open_( const char* );
	void ReadLine_( IInputLineOutpar& );
	};


class CSCSeqInputFile: public ISCInputFile {

public:
	explicit CSCSeqInputFile();

	bool Open( const char* );
	bool ReadLine( IInputLineOutpar& );
	};

class CSCRandomInputFile: public ISCInputFile {

public:
	explicit CSCRandomInputFile();

	bool Open( const char* );
	bool ReadLine( IInputLineOutpar& );

private:
	unsigned long m_Size;
	};



#endif // of __TV_READ_FILE_INTERFACE_H

