/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef _WL_LoadjsShell_h_
#define _WL_LoadjsShell_h_
 
#include <collstr.h>
#include <tvordvec.h>
#include <bag.h>

#include "platform.h"

class CGeneralProcess;
class CWLJSScript;
class CLoadGeneralThreadInstance;
class CLoadGeneralThreadInstanceScript;
class iReport;
class IGeneralManagerScriptData;

#include "loadenginevariablesdata.h"

class CJSLoadScope;
class CJSLoadScript;
class CJSLoadRuntime;
 
#include <RVCString.h>
class RVCriticalSection;

class iLEProtocolAgendaInMix;
class iLEProtocolSession;
class CLoadThreadInstanceScript;

class iLEProtocolContainer;

class CIncludeFileInfo: public RVCollectable {
public:
  RVCString  m_FileName;

  RVCString   m_ScriptCode;
  JSScript*   m_Script;
  JSObject*  m_SO;
  bool        m_Execute;

  CIncludeFileInfo(const char* filename, int CodeLength, const char* CodeBytes, bool execute);

};

class DllExport CWLJSBaseObject : public RVCollectableString {
private:
  RVCString       m_RetValue;
	bool m_OwnScope;
protected:
	CJSLoadScope*   m_Scope;
	RVCString      m_Errors;
	int             m_NumErrors;
	int m_ActiveCtr;
	int m_ActiveAntiCtr;
	int m_UserSectionActiveCtr;
	bool m_IsRunningJava;

public:
  CWLJSBaseObject();
  ~CWLJSBaseObject();

	bool Init(CJSLoadRuntime& runtime, JSContext* guardcx,JSObject* proto);
	bool Init(CJSLoadScope&);
  virtual void Terminate();

	bool IsActive();
	bool IsUserSectionActive();

  const char* CWLJSBaseObject::Evaluate(const char* command);
	bool CWLJSBaseObject::Evaluate(RVCString& command);

	bool CallFunctionName( const char* name, uintN argc, jsval* argv, jsval& rval );
	void        AddError(const char*);
	void reportApply(	iReport& );
	void        ClearError();
	const char* GetError();
	int         GetNumErrors();

	CJSLoadScope*    GetScope() { return m_Scope; }
	JSContext*       GetContext();
	JSObject*        GetObject();

	void EnterJava();// will set the boolean m_IsRunningJava 
	void ExitJava();// will reset the boolean m_IsRunningJava 

protected:
	void Activate();
	void Deactivate();
	void UserSectionActivate();
	void UserSectionDeactivate();
	void ReportException();

};

class DllExport CWLJSScript : public CWLJSBaseObject {
private:
  JSScript*       m_Script;
  JSObject*      m_SO;

	JSObject* m_Proto;

  RVCString       m_ScriptCode;
  RVCString       m_FileName; 
  //  RVCriticalSection*  m_GCNewContextMutex;

//  CLoadEngineVariablesData *m_VariablesData;
  JSObject*       m_wlGlobal;
  CLEVariablesData *m_VariablesData;

	int m_InInitAgendaCtr;
	
// RWCStrings that keep a script that will be executed in a certain 
// times. by calling the function EvaluateScript.
	RVCString     m_EvaluationStringBeforeInitClient;
	RVCString     m_EvaluationStringAfterInitAgenda;
//	RVCString			m_EvaluationStringOnThreadTermination;
	RVCString			m_EvaluationStringBeforeThreadActivation;
	RVCString			m_EvaluationStringOnThreadActivation;
	RVCString			m_EvaluationStringAfterTerminateClient;
	RVCString			m_EvaluationStringBeforeRound;
	RVCString			m_EvaluationStringAfterRound;
	RVCString			m_EvaluationStringAfterTerminateAgenda;

	// in order to disable appending the script that creates 
	// the javaThreadWrapper object twice.
	bool m_IsJavaThreadWrapperCreationScriptExists;

	int m_numScriptInMix;

public:
	CLEPropWorkSpace m_PropWorkSpace;

public:
  RVBag m_IncludeFiles;

public:
// this enum will be use to declare when to run a certain script.
// The 'WLAfterTerminateAgenda' is still not in use.
	enum WhenToRunScript {WLAfterInitAgenda,WLBeforeThreadActivation,WLOnThreadActivation,
												WLBeforeInitClient,WLBeforeRound,WLAfterRound,
												WLAfterTerminateClient,WLAfterTerminateAgenda};

public:
	CWLJSScript(int numScriptInMix);
	~CWLJSScript();

	bool IsInInitAgenda();

  typedef enum {IncludeFileOK, 
                IncludeFileErrorCompilingScript, 
                IncludeFileErrorExecutingScript } IncludeFileStatus;
  bool            Init(CJSLoadRuntime& pLoadRuntime, IGeneralManagerScriptData& pManagerScriptData, iLEProtocolContainer& pProtocolSessionContainer ,const char* agendaFileName);
  void            Terminate();

  void            JSGC(bool maybe);
  bool            InitAgenda(const char* InitializationScript);

  bool            TerminateAgenda(IGeneralManagerScriptData&);
  bool            OnErrorTerminateAgenda(IGeneralManagerScriptData&);

	JSObject* GetProto() {return m_Proto;}

  JSObject*       GetwlGlobal() {return m_wlGlobal;}
  JSScript*       GetScript()   { return m_Script; }
  bool	          LoadScript(const char* filename, int ScriptLength, const char* ScriptCode);
	bool IncludeFile( const char* FileName, int CodeLength, const char* CodeBytes, bool Execute );
	IncludeFileStatus IncludeFileCompile(CIncludeFileInfo*);

  CLEVariablesData* GetLoadEngineVariablesData() { return m_VariablesData;}

  const char* GetFileName() {return m_FileName;}
  const char* GetScriptCode() {return m_ScriptCode;}
//  RVCriticalSection* GetGCNewContextMutex() { return m_GCNewContextMutex; }
	
	// The function apppend a string to the suitible RVCString. according to it's second parameter.
	// The appended script will later be executed in the time that is represented by the second parmeter.
	void AppendStringForEvaluation(const char*,CWLJSScript::WhenToRunScript whenToRunScript);
	RVCString&  GetEvaluationStringBeforeInitClient();
	RVCString&  GetEvaluationStringAfterInitAgenda();
//	RVCString&	GetEvaluationStringOnThreadTermination();
	RVCString&  GetEvaluationStringBeforeThreadActivation();
	RVCString&  GetEvaluationStringOnThreadActivation();
	RVCString&	GetEvaluationStringAfterTerminateClient();
	RVCString&	GetEvaluationStringBeforeRound();
	RVCString&	GetEvaluationStringAfterRound();
	RVCString&	GetEvaluationStringAfterTerminateAgenda();

	void SetJavaThreadWrapperCreationScript();
	bool IsJavaThreadWrapperCreationScriptExists();

	CGeneralProcess& fGetProcess();
	CLEVariablesData* GetVariablesData();

	double GetVersionNumber();
	void InitVersionNumber();
private:
  bool InitAgendaInternal(const char* InitializationScript);
	double							m_VersionNumber;
	bool								m_VersionNumberSet;
};


//

class iLEProtocolVCInMix;
class iLEProtocolAgendaInMix;
 
class DllExport CWLJSThread : public CWLJSBaseObject {
private:
  JSScript*       m_Script;
  JSObject*      m_SO;
  CWLJSScript			*m_JSScript;

	int m_InRoundCtr;

  CLEVariablesData* m_GlobalVariablesData;
  CLEVariablesData* m_LocalVariablesData;
  //CLoadEngineVariablesData *m_VariablesData1;

	//WebLoad default behavior is to show in the statistics every call to java method.
	//This boolean determine if we want to disable the show of the default statistics.
	bool	m_IsJavaDefaultStatistics;

	bool m_OwnScope;


public:
	CWLJSThread(CWLJSScript& pJSScript, bool OwnScope );
	~CWLJSThread();

	bool IsInRound();

  void  SetJSContext();
  bool	Init(CJSLoadRuntime& runtime, CLoadGeneralThreadInstanceScript* TIS, IGeneralManagerScriptData& GMSD, int nThread);

	bool IncludeFiles( CWLJSScript& );
  bool  IncludeFile(CIncludeFileInfo*);

  bool InitClient(CLoadGeneralThreadInstanceScript* TIS);
  bool TerminateClient();
	void Terminate();
  bool OnScriptAbort();
  bool OnErrorTerminateClient();

	bool	ExecScript(unsigned long RoundNum);

  CWLJSScript* GetParent()   { return m_JSScript; }

  CLEVariablesData* GetLoadEngineVariablesData() { return m_LocalVariablesData;}
	void ToggleJavaDefaultStatistics(bool);
	bool IsDisabledJavaDefaultStatistics();

	void SetJavaThreadWrapperCreationScript();
	bool IsJavaThreadWrapperCreationScriptExists();

  CWLJSScript& GetJSScript();
};

#endif //_WL_LoadjsShell_h_

