/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// @@Merged
#ifndef __WL_LOADREPORTINFO_H
#define __WL_LOADREPORTINFO_H

#pragma warning( disable : 4250 )

#include "wlerrorlib.h"
#include "rvjs.h"
#include "loadjsshell.h"
#include "loadengineglobals.h"

class CLoadThreadInstanceScript;

class IGeneralManagerScriptData;

class DllExport iReport;
class DllExport cJSInternalReport;
class DllExport iLoadReportInfo;
class DllExport cLoadReportInfoAgenda;
class DllExport cLoadReportInfoClient;
class DllExport iLoadReport;
class DllExport cLoadReport;
class DllExport cLoadReportHard;

class DllExport iLEEmbedderCallContext;
class DllExport cLEHardEmbedderCallContext;
class DllExport cLEJSEmbedderCallContext;
class DllExport cReportInfoPosition;

class cReportInfoPosition;

// Some comments by Itaj:
// In this whole thing, i assume that GC never cause internal JS errors.
// So we never need to check if JSContext is for GC.
// The formatting of messages to the LoadLog is included in iReport.
// They should be separated o another group of classes that will be used only on the Load side, not SC.

class iReport {
protected:
	explicit iReport( iReportThreadState*, iReportInfo& );
	virtual ~iReport();

private:
	iReportThreadState* mThreadState;
	iReportInfo& mInfo;
	
public:
	iReportThreadState* fGetThreadState() const;
	iReportInfo& fGetInfo() const;

	virtual WLLogMessage::Severity vGetSeverity() const = 0;
		WLLogMessage::Severity fGetSeverity() const { return vGetSeverity(); }

	virtual const char* vGetMessage() const = 0;
		const char* fGetMessage() const { return vGetMessage(); }

	virtual bool vIsEvent() const;
		bool fIsEvent() const {return vIsEvent();}

	virtual bool vHasTime() const;
		bool fHasTime() const {return vHasTime();}

	virtual double vGetTime() const;
		double fGetTime() const {return vGetTime();}

	virtual iReportThreadState* vGetifReportThreadState() const;
		iReportThreadState* fGetifReportThreadState() const {return vGetifReportThreadState();}

};



class cJSInternalReport: public iReport
{
public:
	explicit cJSInternalReport( iLEJSEmbedContext&, iReportThreadState&, cInternalReportInfo& );

private:
	WLLogMessage::Severity vGetSeverity() const;
	const char* vGetMessage() const;

private:
	WLLogMessage::Severity mSeverity;
	RVCString mMessage;
};



class iLoadReportInfo:
	public iPlainReportInfo 
{
protected:
	explicit iLoadReportInfo( WLLogMessage::Severity, const char*, bool rIsApply, bool rIsEvent, bool rHasTime, double rTime, iReportThreadState* );

public:
	WLLogMessage::Severity fGetSeverity() const;
	const char* fGetMessage() const;
	bool fIsApply() const;
	bool fIsEvent() const;
	bool fHasTime() const;
	double fGetTime() const;
	iReportThreadState* fGetifReportThreadState() const;

private:
	WLLogMessage::Severity mSeverity;
	const char* mMessage;
	bool mIsApply;
	bool mIsEvent;
	bool mHasTime;
	double mTime;
	iReportThreadState* mReportThreadState;


public:
	bool vIsStop() const;
};



class cLoadReportInfoAgenda: public iLoadReportInfo
{
public:
	explicit cLoadReportInfoAgenda( WLLogMessage::Severity, const char* );
	explicit cLoadReportInfoAgenda( WLLogMessage::Severity, const char*, double rTime );
	explicit cLoadReportInfoAgenda( WLLogMessage::Severity, const char*, bool rHasTime, double rTime, iReportThreadState* );
};

class cLoadReportInfoClient: public iLoadReportInfo
{
public:
	explicit cLoadReportInfoClient( WLLogMessage::Severity, const char*, bool rIsApply, bool rIsLog, bool rIsEvent, bool rHasTime, double Time );
	explicit cLoadReportInfoClient( WLLogMessage::Severity, const char*, bool rIsApply, bool rIsLog, bool rIsEvent, bool rHasTime, double Time, iReportThreadState* );

	bool fIsLog() const;

private:
	bool mIsLog;
};


class iLoadReport: public iReport
{
protected:
	explicit iLoadReport( iReportThreadState*, iLoadReportInfo& );

private:
	WLLogMessage::Severity vGetSeverity() const;
	const char* vGetMessage() const;
	bool vIsEvent() const;
	bool vHasTime() const;
	double vGetTime() const;
	iReportThreadState* vGetifReportThreadState() const;

};


class cLoadReport: public iLoadReport
{
public:
	explicit cLoadReport( iReportThreadState&, iLoadReportInfo& );

private:
	RVCString mMessage;

	const char* vGetMessage() const;
};


class cLoadReportHard: public iLoadReport
{
public:
	explicit cLoadReportHard( iLoadReportInfo& );
};


/*
class cLoadReportEvent: public iReport
{
public:
	explicit cLoadReportEvent( WLLogMessage::Severity, const char* );

	WLLogMessage::Severity vGetSeverity() const;
	const char* vGetMessage() const;

private:
	WLLogMessage::Severity mSeverity;
	RVCString mMessage;
};
*/


/*
class cLoadEmbedderCallContext: public virtual iEmbedderCallContext {

public:

  explicit cThreadHardEmbedderCallContext( CloadThreadInstanceScript& );

	void fLoadReport( cLoadReportInfo& _ ) {vLoadReport_iLoad(_);}
protected:
	virtual void vLoadReport_iLoad( cLoadReportInfo& ) = 0;

private: 
	void vReport_iEmbedderCallContext( iReportInfo& );
	};
*/


class iLEEmbedderCallContext: public virtual iEmbedderCallContext {
public:
	explicit iLEEmbedderCallContext( iLEJSEmbedContext& );

protected:
	iLEJSEmbedContext& mEvalInter;
};


class cLEHardEmbedderCallContext: public virtual iLEEmbedderCallContext {

public:
	explicit cLEHardEmbedderCallContext( iLEJSEmbedContext& );

	bool fGetReturn();

private: 
	bool mReturn;

private: 
	void vReport_iEmbedderCallContext( iReportInfo& );
};


class cLEJSEmbedderCallContext: public virtual cJSEmbedderCallContext, public virtual iLEEmbedderCallContext {

public:
	explicit cLEJSEmbedderCallContext( JSContext*, iLEJSEmbedContext& );

};



class cReportInfoPosition: public iDebugExecReportInfo
{
public:
	explicit cReportInfoPosition( coReportThreadState& );
	~cReportInfoPosition();
	
//overrides
	tDebugReporterReturn vApplyDebug( cJSDReportCommit& );
	void vApplyJSExec( iJSExecReportCommit& );
	bool vIsStop() const;


public:
	bool mDone;
	coReportThreadState& mReportThreadState;
};

bool LoadReport2( iLEJSEmbedContext&, JSContext*, iReportInfo& );

bool DllExport LoadReportNoExn( JSContext*, iReportInfo& );

bool DllExport LoadReport( iLEJSEmbedContext&, JSContext*, iReportInfo&, WLLogMessage::Severity ErrorInTransTree = WLLogMessage::Success );

bool DllExport LoadReport( iLEJSEmbedContext&, JSContext*, WLErrorException&, WLLogMessage::Severity ErrorInTransTree = WLLogMessage::Success  );

bool DllExport LoadReport( iLEJSEmbedContext&, JSContext*, WLLogMessage::Severity, const char* Message, WLLogMessage::Severity ErrorInTransTree = WLLogMessage::Success  );

bool DllExport LoadReport( JSContext*, iReportInfo&, WLLogMessage::Severity ErrorInTransTree = WLLogMessage::Success );

bool DllExport LoadReport( JSContext*, WLErrorException&, WLLogMessage::Severity ErrorInTransTree = WLLogMessage::Success  );

bool DllExport LoadReport( JSContext*, WLLogMessage::Severity, const char* Message, WLLogMessage::Severity ErrorInTransTree = WLLogMessage::Success  );
bool DllExport LoadReportTime( JSContext*, WLLogMessage::Severity, const char* Message, double Time, WLLogMessage::Severity ErrorInTransTree = WLLogMessage::Success );








#endif // __WL_LOADREPORTINFO_H
