/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_THREAD_INSTANCE_H
#define	__WL_THREAD_INSTANCE_H


#include <ordered.h>
#include <collint.h>
#include <collstr.h>

#include "platform.h"
#include "wlerrorlib.h"
#include "loadjsshell.h"
#include "tvpsync.h"
#include "loadenginestatistics.h"
#include "loadgeneratorscript.h"

#include "threadglobalshandler.h"
#include "rvjsjava.h"
#include "rvjsreport.h"
#include "rvinheritarc.h"

//#include "WLLEProtocol.h"

#define WLE_THREAD_RUN			1
#define WLE_THREAD_TERMINATE	2
#define WLE_THREAD_PAUSE		3


class IGeneralGlobalsRequestHandler;
class CThreadGlobalsRequestHandler;
class iLoadSingleClient;

class iLEProtocolAgendaMachineThread;

class CLoadGeneralThreadInstance {
public:

  unsigned long                       m_RoundNum;

	explicit CLoadGeneralThreadInstance();
	virtual ~CLoadGeneralThreadInstance();
  
	virtual void ToggleJavaDefaultStatistics(bool) = 0;
	//WebLoad default behavior is to show in the statistics every call to java method.
	//This boolean determine if we want to disable the show of the default statistics.
	virtual bool IsDisabledJavaDefaultStatistics() = 0;

	virtual iLEJSMachineThread& vGetMachineThread() = 0;

  //
  // virtual functions from ProbingClient Thread Data
  virtual bool Init() = 0;
	virtual bool Terminate();
  virtual bool internalIsAbort() = 0;
  bool isAbort(bool CheckInScript = true);
  virtual void sleep(int milisec);
	virtual bool IsJSActive() = 0;
	virtual JSContext* vCurrentJSContext() = 0;

	virtual void endMultiThread() = 0;

	virtual IGeneralThreadTransactionManager& GetTransactionManager() = 0;

	virtual void reportApply( iReport& ) = 0;
	virtual void reportLog( iReport& ) = 0;
	void reportApplyEvent( WLLogMessage::Severity, const char* );

	virtual bool report( WLLogMessage::Severity, const char* );
	virtual bool reportTime( WLLogMessage::Severity, const char*, double Time, WLLogMessage::Severity ErrorInTransTree = WLLogMessage::Success );

	virtual bool report( WLErrorException& );
	virtual bool reportTime( WLErrorException&, double Time );

	virtual bool reportMultiThread( iReportInfo& ) = 0;
	virtual bool reportMultiThread( WLLogMessage::Severity, const char* );
	virtual bool reportMultiThread( WLErrorException& );

  virtual WLLogMessage::Severity getErrorSeverity() = 0;
  virtual CLEVariablesData* GetCurrentLoadEngineVariablesData() = 0;


  virtual bool InScript() = 0;
  virtual bool IsOneClient() = 0;
  virtual bool IsLoadServer() = 0;
  virtual bool IsRTAM() = 0;
	virtual WLEThreadType GetThreadType() = 0;
  virtual unsigned long TotalRoundNum() = 0;
  virtual unsigned long CurrentScriptRoundNum() = 0;
	virtual unsigned long GetRoundNumScriptInSlave() = 0;
	virtual unsigned long GetRoundNumMixInSlave() = 0;

	virtual void OnSevereError() = 0; 
  virtual int GetSlaveThreadNum() = 0;
  virtual int GetLGThreadNum() = 0;
  virtual IFileDictionary* GetFileDict() = 0;
  virtual int GetAgendaNumRunning() = 0;
  virtual const char* GetLoadGeneratorNo() = 0;
  virtual int GetErrorCounter() = 0;
  virtual int GetWarningCounter() = 0;
  virtual void IncrementErrorCounter() = 0;
  virtual void IncrementWarningCounter() = 0;
  virtual const char* GetAgendaName() = 0;
  virtual iLEVCJSThread& vGetVCJSThread() = 0;
  iLEVCJSThread& fGetVCJSThread() {return vGetVCJSThread(); }
  virtual CLoadGeneralThreadInstanceScript* GetCurrentScript() = 0;
	virtual CWLJSScript& GetJSScript() = 0;
	virtual IGeneralGlobalsRequestHandler *GetGlobalsRequestHandler() = 0;
	virtual CGeneralProcess& fGetProcess() = 0;
	virtual iSession& vGetSession() = 0;
	virtual JSObject* GetGlobalObject() = 0;

	// Transaction

  virtual void VerificationSetFailureReason(const char* reason, const char* description, double time, bool isEvent );
  virtual void VerificationRecordEvent(const char* reason, const char* description, double time, WLLogMessage::Severity severity = WLLogMessage::Note, const char* extraDataType = NULL, void* extraData = NULL, unsigned long len = 0, bool IsLog = true );
  virtual bool VerificationBeginTransaction(const char* transname, double time, const char* extraDataType, void* extraData, unsigned long len);
  virtual bool VerificationEndTransaction(const char* transname, double time, WLLogMessage::Severity severity, bool save, bool SendTimers, bool IsLog );
  virtual bool VerificationEndTransactionFailure(const char* transname, double time, WLLogMessage::Severity severity,const char* reason, bool SendTimers, bool IsLog, bool isFailureEvent );
  virtual int  VerificationGetTransactionDepth();
  virtual const char* VerificationGetCurrentTransaction();

	virtual void UserLock() = 0;
	virtual void UserUnlock() = 0;

  virtual SynchronizationPointStatus SynchronizationPoint(int timeoutmillis) = 0;
  virtual JSJavaThreadState* getJSJ_thread() = 0;
  virtual void setJSJ_thread(JSJavaThreadState* jsj_thread) = 0;

	virtual int GetNumScriptInMix() = 0;

	virtual void SignNewJSContext( JSContext* ) = 0;

	virtual void CreateApplet(JSContext*                cx,
                            CLoadGeneralThreadInstanceScript*
                                                      TIS,
                            const char*               appletClassName,
										        const char*               appletCodeBase,
                            const char*               appletDocumentBase,
  											    RVHashDictionary*         appletParams,
                            CWLJSJavaApplet*&         javaApplet) = 0;

  virtual bool ClassAppletExists(JSContext*  cx,
                                 const char* appletClassName,
 							                   bool& classAppletExists) = 0;

	virtual bool AddJavaStream(JSContext*  cx,
                             const char* streamName,
                             const char* codeBase,
                             RVCString& javaStream) = 0;

	virtual bool JavaStreamExists(const char* streamName) = 0;
	void fEnterControlMachineThread();
	void fExitControlMachineThread();

	virtual bool vIsUserSection() = 0;

	bool inCriticalSection();
	void fEnterCriticalSection();
	void fExitCriticalSection();

private:
	bool	m_InCriticalSection;
};

//class CLoadThreadInstance;
class CLoadThreadsManager;
class iLEProtocolVCInMix;

class CLoadThreadInstance : public eiRVInheritArc< CLoadThreadInstance, RVCollectableInt >,
                            public CLoadGeneralThreadInstance,
                            public iLEJSAgendaMachineThread {
	
	friend class CLoadThreadInstanceScript;

#ifndef __TV_linux__
public:	using CLoadGeneralThreadInstance::report;
#endif

protected:
  CLoadThreadsManager                 *m_ThreadsManager;
  CLoadThreadInstanceScriptContainer  m_ScriptContainer;
  CLoadThreadInstanceScript           *m_CurrentScript;
  CLoadThreadInstanceScript           *m_PreviousScript;
	CThreadGlobalsRequestHandler        *m_GlobalsRequestHandler;

  bool                                m_SingleTest;
  bool                                m_InitDone;
	unsigned long m_RoundNumScriptInSlave;
	unsigned long m_RoundNumMixInSlave;

  RVCriticalSection   *m_LogMutex;

  JSJavaThreadState*  m_jsj_thread;
  RVThreadSelf m_ThreadSelf;

protected:
  void mainLoopFunc();
  int  loadLoopFunc();

public:
  //
  // vars for error handling.
  WLLogMessage::Severity              m_ErrorSeverity;
  //RVCString                           m_ErrorMsg;
  //
  RVThread                            m_WorkThread;

  bool m_FirstTimeSelectingScript;

  bool                m_SevereErrorOccurred;

public:
  CLoadThreadInstance(CLoadThreadsManager  *agen, int id, CGeneralProcess& );
  virtual ~CLoadThreadInstance();

	
  //virtual void ThreadExit() {sys_thr.end(0);}

	bool IsJSActive();
	JSContext* vCurrentJSContext();
  bool Init();
	
  int run(RVCString& errMessage); 
  void stop();
  void mainLoop();
  int loadLoop();
  void ExceptionHandler(std::exception*);
  virtual int DoCommand() { return 0; }
  bool DoNextCommand();
  bool internalIsAbort();
  bool InScript();
  bool IsOneClient();
  bool IsLoadServer();
  bool IsRTAM() {return false;};
	WLEThreadType GetThreadType();

 	IGeneralThreadTransactionManager& GetTransactionManager();

	void endMultiThread();

	void reportApply( iReport& );
	void reportLog(	iReport& );

	bool reportMultiThread( iReportInfo& );

  void OnSevereError();
  int  GetLGThreadNum();
  unsigned long TotalRoundNum() { return m_RoundNum; }
  unsigned long CurrentScriptRoundNum() { return m_CurrentScript->m_RoundNum; }
	unsigned long GetRoundNumScriptInSlave();
	unsigned long GetRoundNumMixInSlave();
  void IncrementRoundNum() {m_RoundNum++;}
  IFileDictionary* GetFileDict() {return m_CurrentScript->GetFileDict(); }
  int GetAgendaNumRunning();
  const char* GetAgendaName();
  const char* GetLoadGeneratorNo();
  int GetErrorCounter();
  int GetWarningCounter();
  void IncrementErrorCounter();
  void IncrementWarningCounter();
  iLEVCJSThread& vGetVCJSThread() {return *m_CurrentScript;}
  virtual CLoadGeneralThreadInstanceScript* GetCurrentScript()    {return (CLoadGeneralThreadInstanceScript*)m_CurrentScript;}
	virtual CWLJSScript& GetJSScript();
  virtual IGeneralGlobalsRequestHandler *GetGlobalsRequestHandler();
	CLoadGeneratorSlave* GetLoadGeneratorSlave();
	CGeneralProcess& fGetProcess();
	iSession& vGetSession();
	CLoadThreadsManager *GetThreadsManager()                 {return m_ThreadsManager;}
	JSObject* GetGlobalObject();

  CLEVariablesData* GetCurrentLoadEngineVariablesData() { return m_CurrentScript->GetLoadEngineVariablesData(); }
  void SetCurrentScript(CLoadThreadInstanceScript* cs) {m_CurrentScript = cs; }
  void SetSingleTestFlag() {m_SingleTest = true;};

  void addLast(const char *name, double l, CWLStatPointTypeEnum::PointType type);
	void ToggleJavaDefaultStatistics(bool);
	bool IsDisabledJavaDefaultStatistics();
	iLEJSMachineThread& vGetMachineThread();
  void resetTimer(const char *name);
  void sendTimer(const char *name);
  SynchronizationPointStatus SynchronizationPoint(int timeoutmillis);
  int GetSlaveThreadNum() {return value();};
  WLLogMessage::Severity getErrorSeverity() {return m_ErrorSeverity;};
  JSJavaThreadState* getJSJ_thread() {return m_jsj_thread;};
  void setJSJ_thread(JSJavaThreadState* jsj_thread) {m_jsj_thread = jsj_thread;};
	virtual void SignNewJSContext( JSContext* );

	virtual void UserLock() { m_CurrentScript->UserLock(); };
	virtual void UserUnlock() { m_CurrentScript->UserUnlock(); };
	virtual int GetNumScriptInMix();

	virtual void CreateApplet(JSContext*          cx,
                            CLoadGeneralThreadInstanceScript*
                                                TIS,
                            const char*         appletClassName,
										        const char*         appletCodeBase,
                            const char*         appletDocumentBase,
										        RVHashDictionary*   appletParams,
                            CWLJSJavaApplet*&   javaApplet);

  virtual bool ClassAppletExists(JSContext*  cx,
                                 const char* appletClassName,
 							                   bool& classAppletExists);

	virtual bool AddJavaStream(JSContext*  cx,
                             const char* streamName, 
                             const char* codeBase,
                             RVCString& javaStream);

	virtual bool JavaStreamExists(const char* streamName);

	bool vIsUserSection();

};

class CLoadThreadsVector : public RVOrdered {
public:
  CLoadThreadsVector() {}
  virtual ~CLoadThreadsVector() { clearAndDestroy(); }

  CLoadThreadInstance* GetThread(size_t i) { return static_cast< eiRVInheritArc<CLoadThreadInstance,RVCollectableInt>* >(at(i))->down_ptr(); }
  CLoadThreadInstance* operator[](size_t i) { return static_cast< eiRVInheritArc<CLoadThreadInstance,RVCollectableInt>* >(at(i))->down_ptr(); }
};


class iLEProtocolVCInMix;

class CRTAMThreadInstance : public CLoadGeneralThreadInstance {
	friend class iLoadSingleClient;
	friend class iLoadProbingClient;
private:
	iLoadSingleClient& m_SingleClient;
  CRTAMThreadInstanceScript           *m_CurrentScript;
  CWLJSScript& m_jsScript;
  CWLJavaStreamTable  m_JavaStreamTable;

  WLLogMessage::Severity            m_ErrorSeverity;
  RVCriticalSection   *m_LogMutex;
  int m_myId;
	int m_RoundNum;
  JSJavaThreadState*  m_jsj_thread;


public:
  CRTAMThreadInstance( iLoadSingleClient&, CWLJSScript&, int ID );
  virtual ~CRTAMThreadInstance();

  //
  // virtual functions from ProbingClient Thread Data
  bool Init();

  // CRTAMThreadInstance doesn't have a script container for holding the instance's scripts
  // (RTAM only has one script) so it's the responsibilty of the caller to set it;
  void setCurrentScript(CRTAMThreadInstanceScript* pTIS) { m_CurrentScript = pTIS;};
	
  bool internalIsAbort();
	bool IsJSActive();
	JSContext* vCurrentJSContext();

  void addLast(const char *name, double l, CWLStatPointTypeEnum::PointType type);
	void resetTimer(const char *name);
	void sendTimer(const char *name);

  void ToggleJavaDefaultStatistics(bool);
	//WebLoad default behavior is to show in the statistics every call to java method.
	//This boolean determine if we want to disable the show of the default statistics.
	bool IsDisabledJavaDefaultStatistics();

	iLEJSMachineThread& vGetMachineThread();

	IGeneralThreadTransactionManager& GetTransactionManager();

	void endMultiThread();

	void reportApply( iReport& );
	void reportLog(	iReport& );

	bool reportMultiThread( iReportInfo& );

  CLEVariablesData* GetCurrentLoadEngineVariablesData();
  
	void OnSevereError(); 

  int GetSlaveThreadNum() {return m_myId;}
  int  GetLGThreadNum() {return GetSlaveThreadNum();}
  bool InScript();
  bool IsOneClient() {return false;}; // no single client test
  bool IsLoadServer() {return false;};
  bool IsRTAM() {return true;};
	WLEThreadType GetThreadType();

	void SetRoundNum( unsigned long RoundNum );
  unsigned long TotalRoundNum() {return m_RoundNum;};
  unsigned long CurrentScriptRoundNum() { return m_RoundNum;};
	unsigned long GetRoundNumScriptInSlave();
	unsigned long GetRoundNumMixInSlave();

  WLLogMessage::Severity getErrorSeverity() {return m_ErrorSeverity;};

  SynchronizationPointStatus SynchronizationPoint(int timeoutmillis) {return SynchronizationPointOK;};

  IFileDictionary* GetFileDict();
  int GetAgendaNumRunning() {return 1;}
  const char* GetLoadGeneratorNo() {return "1";}
  int GetErrorCounter();
  int GetWarningCounter();
  void IncrementErrorCounter();
  void IncrementWarningCounter();
  const char* GetAgendaName();
  iLEVCJSThread& vGetVCJSThread();
  JSJavaThreadState* getJSJ_thread() {return m_jsj_thread;};
  void setJSJ_thread(JSJavaThreadState* jsj_thread) {m_jsj_thread = jsj_thread;};

	virtual void UserLock() {};
	virtual void UserUnlock() {};

	IGeneralGlobalsRequestHandler* GetGlobalsRequestHandler();

//	iLEProtocolVCInMix* GetiLEProtocolVCInMix(WLProtocolIndex);
  CLoadGeneralThreadInstanceScript* GetCurrentScript()    {WlAssert(m_CurrentScript); return (CLoadGeneralThreadInstanceScript*)m_CurrentScript;}

	CWLJSScript& GetJSScript();
	CGeneralProcess& fGetProcess();
	iSession& vGetSession();

	virtual int GetNumScriptInMix();

	void SignNewJSContext( JSContext* );

	JSObject* GetGlobalObject();

  virtual void CreateApplet(JSContext*          cx,
                            CLoadGeneralThreadInstanceScript*
                                                TIS,
                            const char*         appletClassName,
                            const char*         appletCodeBase,
                            const char*         appletDocumentBase,
                            RVHashDictionary*   appletParams,
                            CWLJSJavaApplet*&   javaApplet);

  virtual bool ClassAppletExists(JSContext*  cx,
                                 const char* appletClassName,
 							                   bool& classAppletExists);

	virtual bool AddJavaStream(JSContext*  cx,
                             const char* streamName,
                             const char* codeBase,
                             RVCString& javaStream);

	virtual bool JavaStreamExists(const char* streamName);
//	void fActionReportCommit( iActionReportInfo& );

	bool vIsUserSection();

};


#endif //__WL_THREAD_INSTANCE_H


