/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_LOADGENERATORSCRIPTDATA_H
#define	__WL_LOADGENERATORSCRIPTDATA_H
class CGeneralProcess;

class CWLJSScript;
#include <RVCString.h>
#include <set.h>
class IFileDictionary;
class IGeneralGlobalsRequestHandler;
class CLoadThreadsManagerScript;
class CAssociativeArray;
class iLEProtocolEmbedContext;
class CLEVariablesData;

#include "loadagendacontext.h"
#include "associativearray.h"
#include "loadjsshell.h"
#include "generalprocess.h"
#include "wlleprotocolcontainer.h"

//#include <bag.h>

class DllExport IGeneralManagerScriptData;

class IGeneralManagerScriptData: public iLEAgendaJSEmbedContext  {

public:	
	explicit IGeneralManagerScriptData();
	virtual ~IGeneralManagerScriptData();

  bool IncludeFile(const char*, bool );
  virtual bool vIncludeFile_IGeneralManagerScriptData(const char*, bool ) = 0;

  virtual bool CopyFile(const char*, const char* ) = 0;

	virtual bool IsSpawn() = 0;


  virtual bool InInitAgenda() = 0;

  virtual IFileDictionary* GetFileDict() = 0;
  virtual void AppendStringForEvaluation(const char*,CWLJSScript::WhenToRunScript whenToRunScript) = 0;

  virtual RVCString& ScriptName() = 0;
	virtual RVCString& ScriptFileName() = 0;

	virtual void SendMessage( CAssociativeArray&, int ) = 0;

	virtual bool IsJavaThreadWrapperCreationScriptExists() = 0;
	virtual void SetJavaThreadWrapperCreationScript() = 0;

  // void StatPackFunc();
  // void StatGarbageFunc();
  // void JSGCFunc();

	void reportApply( iReport& );

  iLEProtocolJSEmbedContext& GetProtocolJSEmbedContext(WLProtocolIndex);

  void ProtocolBeforeInitAgenda();
  void ProtocolAfterTerminateAgenda();
  void ProtocolJSInitializeForScript(JSContext* cx, JSObject* AgendaGlobal, JSObject* GeneralGlobal, iLEProtocolContainer& pProtocolSessionContainer);
  void ProtocolJSTerminateForScript(JSContext* cx, JSObject* AgendaGlobal, JSObject* GeneralGlobal);
  void InsertProtocolAgendaInMix(iLEProtocolContainer&);
  iLEProtocolContainer& GetLEProtocolAgendaInMixContainer() { return m_ProtocolAgendaInMixContainer;};

	virtual bool vIsUserSection();

  virtual bool isAbort(bool CheckInScript = true) { return false; };
	virtual void UserLock() {};
	virtual void UserUnlock() {};

protected:
	iLEProtocolContainer m_ProtocolAgendaInMixContainer;
  

private:
	RVSet m_IncludeFiles;
};


class CLoadManagerScriptData:
	public IGeneralManagerScriptData {

public:	
	explicit CLoadManagerScriptData() {}; 
	~CLoadManagerScriptData();

	void InitParent(CLoadThreadsManagerScript* );

  bool vIncludeFile_IGeneralManagerScriptData(const char*, bool );
  bool CopyFile(const char*, const char* );

	CWLJSScript& GetJSScript();
	CGeneralProcess& fGetProcess();
	iSession& vGetSession();

  bool InInitAgenda();

	bool IsSpawn();

	IFileDictionary* GetFileDict();
	IGeneralGlobalsRequestHandler* GetGlobalsRequestHandler();
  void AppendStringForEvaluation(const char*,CWLJSScript::WhenToRunScript whenToRunScript);

  RVCString& ScriptName();
	RVCString& ScriptFileName();

	void SendMessage( CAssociativeArray&, int );

	void reportLog( iReport& );
	void OnSevereError(); 

	iLEJSMachineThread& vGetMachineThread();

	void vEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType );
	void vExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType );

	bool vIsJSGC();

	void SetJavaThreadWrapperCreationScript();
	bool IsJavaThreadWrapperCreationScriptExists();

	CLoadThreadsManagerScript* GetLoadThreadsManagerScript()const { return m_ManagerScript; }

	virtual int GetNumScriptInMix();

  virtual int GetErrorCounter();
  virtual int GetWarningCounter();
  virtual void IncrementErrorCounter();
  virtual void IncrementWarningCounter();

private:
	CLoadThreadsManagerScript* m_ManagerScript;

  // For iLEAgendaJSEmbedContext
  CWLJSBaseObject& GetJSBaseObject();

	};







#endif //__WL_LOADGENERATORSCRIPTDATA_H

