/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// ThreadGlobalsHandler.h: interface for the CThreadGlobalsRequestHandler class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(_THREADGLOBALSHANDLER_H__)
#define _THREADGLOBALSHANDLER_H__

#include "tvpsync.h"
#include "associativearray.h"
#include "loadgeneratorslave.h"
#include "loadthreadinstance.h"
#include "globalvariableproperties.h"


#define LOAD_GENERATOR_GLOBALS_NAME  "LGGLOBALS"
#define MONITOR_GLOBALS_NAME         "METAGLOBALS"

class DllExport IGeneralGlobalsRequestHandler;

class IGeneralGlobalsRequestHandler
{
public:
	enum Level{LGLevelPerAgendaInMix, LGLevelPerAgenda, MetaLevelPerAgenda, MetaLevel};

public:
	virtual void Globals_Set(const char* variableName,
													 CGlobalVariableProperties *newProperties,
													 Level level) = 0;

	virtual void Globals_Add(const char* variableName, 
													 CGlobalVariableProperties *newProperties,
													 Level level) = 0;

	virtual CGlobalVariableProperties* Globals_SyncAdd(const char* variableName, 
																								     CGlobalVariableProperties *newProperties,
																								     Level level) = 0;

	virtual CGlobalVariableProperties *Globals_Get(const char* variableName,
																								 Level level) = 0;

	virtual void ReleaseObject(void) {}
	virtual void HandleReplay(CAssociativeArray &replay) {}

	virtual inline bool WaitMessage(void) { return false; }
	virtual inline bool IsAbort(void) { return false; }
	virtual void SendRequest(CAssociativeArray &request, Level level) {}
	virtual void AddGlobalVriablesObjectName(JSContext* cx, CAssociativeArray &request, Level level){}
};


class CThreadsManagerGlobalsRequestHandler: public IGeneralGlobalsRequestHandler
{
public:
	CThreadsManagerGlobalsRequestHandler(CLoadThreadsManager *threadsManager);
	virtual ~CThreadsManagerGlobalsRequestHandler(void);

	virtual void Globals_Set(const char* variableName, CGlobalVariableProperties *newProperties, Level level);
	virtual void Globals_Add(const char* variableName, CGlobalVariableProperties *newProperties, Level level);
	virtual CGlobalVariableProperties* Globals_SyncAdd(const char* variableName, CGlobalVariableProperties *newProperties, Level level);
	virtual CGlobalVariableProperties *Globals_Get(const char* variableName, Level level);

	virtual void ReleaseObject(void);
	virtual void HandleReplay(CAssociativeArray &replay);

	virtual inline bool WaitMessage(void);
	virtual inline bool IsAbort(void);
	virtual void SendRequest(CAssociativeArray &request, Level level);
	virtual void AddGlobalVriablesObjectName(CAssociativeArray &request, Level level);

protected:
	bool											 m_Active;
	CGlobalVariableProperties	*m_Message;
	TVPManualEvent						 m_GetEvent;
	CLoadGeneratorSlave				*m_pMyLoadGeneratorSlave;
	CLoadThreadsManager       *m_ThreadsManager;
};



class CThreadGlobalsRequestHandler: public CThreadsManagerGlobalsRequestHandler
{
public:
	CThreadGlobalsRequestHandler(CLoadThreadInstance *threadInstance);

	virtual inline bool WaitMessage(void);
	virtual inline bool IsAbort(void);
	virtual void AddGlobalVriablesObjectName(CAssociativeArray &request, Level level);

private:
	CLoadThreadInstance *m_ThreadInstance;
};

#endif // !defined(_THREADGLOBALSHANDLER_H__)

