/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLJSJavaApplet.h: interface for the CWLJSJavaApplet class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WLJSJAVAAPPLET_H__2AEEF92A_63E0_438A_9D1A_A42A9CD571E3__INCLUDED_)
#define AFX_WLJSJAVAAPPLET_H__2AEEF92A_63E0_438A_9D1A_A42A9CD571E3__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include <jni.h>

#include <hashdict.h>
#include <thr.h>
#include <thr.h>
#include <collstr.h>

#include "platform.h"
#include "rvjsapi.h"
class CLoadGeneralThreadInstanceScript;

class CWLJSJavaApplet;
//
// Global functions.

bool DllExport InitLoadEngineJNI(JNIEnv* env);
bool DllExport TerminateLoadEngineJNI(JNIEnv* env);

/*****************************************************************
* Class CWLJavaJniManager
*****************************************************************/

//////////////////////////////////////////////////////////////////////
// Class definition
//////////////////////////////////////////////////////////////////////
class DllExport CWLJavaJniManager
{
private:
  bool m_Initialized;
	static CWLJavaJniManager* msSingleton;

public:
	static CWLJavaJniManager& fsGetSingleton();

public:
  explicit CWLJavaJniManager();
  virtual ~CWLJavaJniManager() {}

  bool Init(JNIEnv* env);
  void Terminate(JNIEnv* env);

  bool fsAddStream( JSContext*   cx, 
                    const char*  streamName, 
                    const char*  codeBase,
									  const char*  javaStream, 
                    size_t       streamLen);

  bool fsClassAppletExists(JSContext*    cx,
                           const char*   appletClassName,
                           bool&         classAppletExists);

  CWLJSJavaApplet* fsCreateJavaApplet(JSContext*                  cx,
                                      CLoadGeneralThreadInstanceScript*
                                                                  TIS,
                                      CWLJSJavaApplet*&           applet,
													            const char*                 appletClassName,
													            const char*                 appletCodeBase,
                                      const char*                 appletDocumentBase,
                                      RVHashDictionary*           appletParams);

  //bool fsInitJavaApplet(JSContext* cx, jobject applet, jint threadNum);
  //bool fsStartJavaApplet(JSContext* cx, jobject applet, jint threadNum);
  bool fsStopJavaApplet(JSContext* cx, jobject applet, jint threadNum);

private:
	//
	// WLAppletHandler variables & functions.
  jclass      m_appletHandlerClass;
  jmethodID   m_midCreateApplet;
	jmethodID   m_midCreateAndRunApplet;
  jmethodID   m_midInitApplet;
  jmethodID   m_midStartApplet;
  jmethodID   m_midStopApplet;
	jmethodID   m_midSetAuthUsrAndPwd;

	//
	// JarResources variables and functions
  jclass    m_jarResourcesClass;
  jobject   m_jarResourcesObject;
  jmethodID m_midSetStream;
  jmethodID m_midIsResourceFullnameExists;
  jmethodID m_midGetSystemJarResources;


//
// RVThrowable variables & functions.
private:
  jclass      m_RVThrowableClass;
  jmethodID   m_midStackTraceToString;

public:
  void fsGetStackTrace(JNIEnv* env, jthrowable e, RVCString& s);
	bool fsHandleJVMException(JNIEnv* env, JSContext* cx, int frmErrorNum = 0);
	bool fsHandleJVMException(JNIEnv* env, JSContext* cx, const char* className, int frmErrorNum = 0);
};

/*****************************************************************
* Class CWLJSJavaApplet  
*****************************************************************/
// BEGIN:


class DllExport CWLJSJavaApplet  
{
protected:
	jsval         m_Applet;
  jobject       m_JavaApplet;
  int           m_ThreadNum;
  JSContext*    m_cx;

public:
  char         readyState;

public:
	CWLJSJavaApplet();
	CWLJSJavaApplet(JSContext* cx, jsval& applet, jobject jApplet, jint threadNum);
	~CWLJSJavaApplet();

	jsval&  GetJSApplet()   { return m_Applet; }
	jobject GetJavaApplet() { return m_JavaApplet; }

	bool Init(JSContext* cx, jsval& applet, jobject jApplet, jint threadNum);
  bool StopApplet(JSContext* cx);
};
// END.


/*****************************************************************
* Class CWLAppletFactory  
*****************************************************************/
// BEGIN:
/*
#define APPLET_CREATION_JSFUNCTION    "CreateWLJavaApplet"
#define CREATION_JSFUNCTION_ARGC 3 
#define CLASS_APPLET_EXIST_JSFUNCTION "ClassAppletExists"
#define ADD_STREAM_JSFUNCTION         "AddStreamToLoader"
#define STOP_APPLET_JSFUNCTION        "StopApplet"
*/
class CWLAppletFactory
{
public:
	static CWLJSJavaApplet* CreateJavaApplet(JSContext*                 cx, 
                                           CLoadGeneralThreadInstanceScript*
                                                                      TIS,
                                           CWLJSJavaApplet*&          applet,
											                     const char*                appletClassName,
											                     const char*                appletCodeBase,
                                           const char*                appletDocumentBase,
                                           RVHashDictionary*          appletParams);

  static bool ClassAppletExists( JSContext*   cx,
                                 const char*  appletClassName,
                                 bool&        classAppletExists);
                                         
};
// END.


/*****************************************************************
* Class CWLAppletsTable  
*****************************************************************/
// BEGIN:

class CWLJavaStreamTable: public RVHashTable
{
private:
  RVReadersWriterLock m_InsertionLock;

protected:
	inline RVCollectableString* FindJavaStream(const char* streamName);

public:
	bool AddJavaStream( /*[IN]*/JSContext*   cx, 
                      /*[IN]*/const char*  streamName, 
                      /*[IN]*/const char*  codeBase,
				              /*[IN]*/const char*  javaStream, 
                      /*[IN]*/size_t       streamLen);

	void RemoveJavaStream(/*[IN]*/const char* streamName);

	bool JavaStreamExists(/*[IN]*/const char* streamName);
  
private:
  bool HandleCabinet(char* streamName,const char* codeBase,JSContext* cx, const char* javaStream,size_t streamLen);
  
};

#ifdef __TV_WINDOWS__
LRESULT
   WINAPI
   CabinetCallback ( IN PVOID pMyInstallData,
                     IN UINT Notification,
                     IN UINT Param1,
                     IN UINT Param2 );


class CWLCabCalbackParam
{
public:
  CWLCabCalbackParam(JSContext* cx, const char* codeBase);
  ~CWLCabCalbackParam(){}
public:
  const char* m_codeBase;
  JSContext*  m_cx;
  
};
#endif // __TV_WINDOWS__

// END.

#endif // !defined(AFX_WLJSJAVAAPPLET_H__2AEEF92A_63E0_438A_9D1A_A42A9CD571E3__INCLUDED_)

