/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_JSObjects_H
#define __WL_JSObjects_H
 
#include "platform.h"
#include "wlleprotocolcontainer.h"
#include "rvjsmachinethread.h"
#include <thr.h>

class CJSLoadScope;
class CJSLoadScript;
class CJSLoadRuntime;
class cLEJSMainEmbedContext;
class CGeneralProcess;
class cLEJSGCThread;

struct JSContext;
struct JSObject;
struct JSScript;
struct JSRuntime;


cLEJSGCThread* fsGetJSGCThread( JSContext* cx );
iLEJSMachineThread* fsGetMachineThread( JSContext* cx );

class DllExport CJSLoadScope {
private:
	JSContext*      m_Context;
  JSObject*       m_Object;
  JSContext*      m_GuardContext;

public:
	CJSLoadScope(CJSLoadRuntime& runtime, JSContext* guardcx, UINT cxSize, JSObject* proto = NULL);
	~CJSLoadScope();

  JSObject*       GetObject()     { return m_Object; }
	JSContext*      GetContext()    { return m_Context; }
  bool            IsMain();
};

typedef enum { UseOldStyle, UseNewStyle, UseDefault} JSGCGuardType;

class DllExport CJSGCRequestBeginEnd
{
private:

  JSContext* m_Context;
  bool       m_EndDone;
	bool			 m_NewStyle;
	bool			 m_GCLockDone; // Sometimes it can't be locked because we have
													 // no machine thread set in ctor but we do have one in dtor
													 // so we need to check

public:
  CJSGCRequestBeginEnd(JSContext* cx, JSGCGuardType guardType = UseDefault);
  ~CJSGCRequestBeginEnd();
  void RequestEnd();
};

class DllExport CJSGCRequestSuspendResume {
private:
  JSContext* m_Context;
  bool       m_ResumeDone;
	int m_DepthSave;
	bool			 m_NewStyle;
	long			 m_GCLockDepthSave; 
	bool			 m_GCUnlockDone; //Sometimes it can't be locked because we have
													 // no machine thread set in ctor but we do have one in dtor
													 // so we need to check
public:
  CJSGCRequestSuspendResume(JSContext* cx, JSGCGuardType guardType = UseDefault);
  ~CJSGCRequestSuspendResume();
  void RequestResume();
};

void DllExport YieldRequest(JSContext* cx);
#endif //__WL_JSObjects_H

