/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// JVMAutoDiscovery.cpp: implementation of the CJVMAutoDiscovery class.
//
//////////////////////////////////////////////////////////////////////

#include "JVMAutoDiscovery.h"
#include <collstr.h>
#include "tracedll.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CJVMAutoDiscovery::CJVMAutoDiscovery()
{

}


CJVMAutoDiscovery::~CJVMAutoDiscovery()
{

}


bool CJVMAutoDiscovery::handleFile( const char* filePath )
{
	CWLErrorInfo errorInfo;
#ifdef USE_MSXML
	CRVXMLDocument XMLDoc(MSXMLParser);
#else
	CRVXMLDocument XMLDoc;
#endif //USE_MSXML

	// load XML file
	XMLDoc.load( filePath, errorInfo );
	if( errorInfo.GetErrorId() ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return false;
	}

	// parse XML file
	CRVXMLElement rootElement = XMLDoc.get_documentElement( errorInfo );
	if( errorInfo.GetErrorId() ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return false;
	}

	if( rootElement.is_empty( errorInfo ) ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return true;
	}

	RVCString rootName = rootElement.get_name( errorInfo );
	if( errorInfo.GetErrorId() ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return false;
	}

	CRVXMLElementList children = rootElement.get_childElements( errorInfo );
	if( errorInfo.GetErrorId() ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return false;
	}

	RVCString* pJVMType = NULL;
	RVCString* pJVMPath = NULL;

	// iterate over all nodes and collect JVM info
	for( int i = 0; i < children.entries(); i++ ){
		RVCString NodeName = children[i].get_name( errorInfo );
		if( errorInfo.GetErrorId() || 0 != strcmp( "JVM", NodeName.data() ) ){
			if( errorInfo.GetErrorId() ){
				m_LastErrorDescription = errorInfo.GetErrorMsg();
			}
			continue;
		}

		pJVMType = new RVCString;
		pJVMPath = new RVCString;
		if( NULL == pJVMType || NULL == pJVMPath ){
			if( NULL != pJVMType ) delete pJVMType;
			if( NULL != pJVMPath ) delete pJVMPath;
			m_LastErrorDescription = "Memory allocation failed.";
			return false;
		}

		// get Dll info
		if( !loadJVMPath( &(children[i]), *pJVMType, *pJVMPath ) )
			continue;
		RVCollectableString* key = new RVCollectableString(*pJVMType);
		RVCollectableString* val = new RVCollectableString(*pJVMPath);
		RVCollectable* aStatus = m_JVMDict.insertKeyAndValue( (RVCollectable*) key, (RVCollectable*) val );
		WlAssert(aStatus); // already exist. either inserted twice, or terminated and restarted (thread pooling)
	}

	return true;
}


const char* CJVMAutoDiscovery::getLastError()
{
	return m_LastErrorDescription.data();
}


bool CJVMAutoDiscovery::discoverPath( const char* path, const char* spec, bool lookInSubDirs )
{
	m_LastErrorDescription = "";

	bool res = FindFile( path, spec, lookInSubDirs, this );


	return res;
}

RVCString* CJVMAutoDiscovery::findJVMPath(RVCString* JVMType){
	RVCollectableString* key = new RVCollectableString(*JVMType);
	RVCollectable* val = m_JVMDict.findValue((RVCollectable*) key);
	if (val){
		return (RVCString*)(RVCollectableString*)val;
	}
	else {
		return NULL;
	}
}

bool CJVMAutoDiscovery::loadJVMPath( CRVXMLElement* pJVMNode, RVCString& JVMType, RVCString& JVMPath)
{
	CWLErrorInfo errorInfo;

	CRVXMLElementList children = pJVMNode->get_childElements( errorInfo );
	if( errorInfo.GetErrorId() ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return false;
	}

	for( int i = 0; i < children.entries(); i++ ){
		RVCString JVMShowName = children[i].get_name( errorInfo );
		if( errorInfo.GetErrorId() ){
			m_LastErrorDescription = errorInfo.GetErrorMsg();
			return false;
		}

		if( 0 == strcmp( "Type", JVMShowName.data() ) ){
			JVMType = children[i].get_value( errorInfo );
			if( errorInfo.GetErrorId() ){
				m_LastErrorDescription = errorInfo.GetErrorMsg();
				return false;
			}
			continue;
		}

		if( 0 == strcmp( "Path", JVMShowName.data() ) ){
			JVMPath = children[i].get_value( errorInfo );
			if( errorInfo.GetErrorId() ){
				m_LastErrorDescription = errorInfo.GetErrorMsg();
				return false;
			}
			continue;
		}
	}

	return true;
}
