/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// ProtocolAutoDiscovery.cpp: implementation of the CProtocolAutoDiscovery class.
//
//////////////////////////////////////////////////////////////////////

#include "ProtocolAutoDiscovery.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CProtocolAutoDiscovery::CProtocolAutoDiscovery()
{

}


CProtocolAutoDiscovery::~CProtocolAutoDiscovery()
{

}


bool CProtocolAutoDiscovery::handleFile( const char* filePath )
{
	CWLErrorInfo errorInfo;
#ifdef USE_MSXML
	CRVXMLDocument XMLDoc(MSXMLParser);
#else
	CRVXMLDocument XMLDoc;
#endif //USE_MSXML

	// load XML file
	XMLDoc.load( filePath, errorInfo );
	if( errorInfo.GetErrorId() ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return false;
	}

	// parse XML file
	CRVXMLElement rootElement = XMLDoc.get_documentElement( errorInfo );
	if( errorInfo.GetErrorId() ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return false;
	}

	if( rootElement.is_empty( errorInfo ) ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return true;
	}

	RVCString rootName = rootElement.get_name( errorInfo );
	if( errorInfo.GetErrorId() ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return false;
	}

	CRVXMLElementList children = rootElement.get_childElements( errorInfo );
	if( errorInfo.GetErrorId() ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return false;
	}

	RVCString* pDllName = NULL;
	RVCString* pDllDisplayName = NULL;
	RVCString* pDllVersion = NULL;
	RVCString* pDllVendor = NULL;

	// iterate over all nodes and collect dll info
	for( size_t i = 0; i < children.entries(); i++ ){
		RVCString NodeName = children[i].get_name( errorInfo );
		if( errorInfo.GetErrorId() || 0 != strcmp( "Dll", NodeName.data() ) ){
			if( errorInfo.GetErrorId() ){
				m_LastErrorDescription = errorInfo.GetErrorMsg();
			}
			continue;
		}

		pDllName = new RVCString;
		pDllDisplayName = new RVCString;
		pDllVersion = new RVCString;
		pDllVendor = new RVCString;
		if( NULL == pDllName || NULL == pDllDisplayName || NULL == pDllVersion || NULL == pDllVendor ){
			if( NULL != pDllName ) delete pDllName;
			if( NULL != pDllDisplayName ) delete pDllDisplayName;
			if( NULL != pDllVersion ) delete pDllVersion;
			if( NULL != pDllVendor ) delete pDllVendor;
			m_LastErrorDescription = "Memory allocation failed.";
			return false;
		}

		// get Dll info
		if( !getDllInfo( &(children[i]), *pDllName, *pDllDisplayName, *pDllVersion, *pDllVendor ) )
			continue;

		// handle Dll parameters
		if( (*m_pIndex) < m_ArraySize ){
			m_pProtocols[*m_pIndex] = pDllName;
			m_pProtocolNames[*m_pIndex] = pDllDisplayName;
			// currently we don't use dll version and dll vendor info
			delete pDllVersion;
			delete pDllVendor;
			(*m_pIndex)++;
		}
		else{
			m_LastErrorDescription = "Not all protocols were added because of small array size.";
			return false;
		}
	}

	return true;
}


const char* CProtocolAutoDiscovery::getLastError()
{
	return m_LastErrorDescription.data();
}


bool CProtocolAutoDiscovery::discoverPath( const char* path, const char* spec, bool lookInSubDirs,
										  RVCString** pProtocols, RVCString** pProtocolNames, int* pIndex, int arraySize )
{
	m_LastErrorDescription = "";

	if( NULL == pProtocols || NULL == pProtocolNames || NULL == pIndex || arraySize <= *pIndex )
		return true;

	m_pProtocols = pProtocols;
	m_pProtocolNames = pProtocolNames;
	m_pIndex = pIndex;
	m_ArraySize = arraySize;

	bool res = FindFile( path, spec, lookInSubDirs, this );

	m_pProtocols = NULL;
	m_pProtocolNames = NULL;
	m_pIndex = NULL;
	m_ArraySize = 0;

	return res;
}


bool CProtocolAutoDiscovery::getDllInfo( CRVXMLElement* pDllNode, RVCString& DllName, RVCString& DllDisplayName, RVCString& DllVersion, RVCString& DllVendor )
{
	CWLErrorInfo errorInfo;

	DllName = pDllNode->get_value( errorInfo );
	if( errorInfo.GetErrorId() ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return false;
	}

	CRVXMLElementList children = pDllNode->get_childElements( errorInfo );
	if( errorInfo.GetErrorId() ){
		m_LastErrorDescription = errorInfo.GetErrorMsg();
		return false;
	}

	for( size_t i = 0; i < children.entries(); i++ ){
		RVCString DllShowName = children[i].get_name( errorInfo );
		if( errorInfo.GetErrorId() ){
			m_LastErrorDescription = errorInfo.GetErrorMsg();
			return false;
		}

		if( 0 == strcmp( "FileName", DllShowName.data() ) ){
			DllName = children[i].get_value( errorInfo );
			if( errorInfo.GetErrorId() ){
				m_LastErrorDescription = errorInfo.GetErrorMsg();
				return false;
			}
			continue;
		}

		if( 0 == strcmp( "DisplayName", DllShowName.data() ) ){
			DllDisplayName = children[i].get_value( errorInfo );
			if( errorInfo.GetErrorId() ){
				m_LastErrorDescription = errorInfo.GetErrorMsg();
				return false;
			}
			continue;
		}

		if( 0 == strcmp( "Version", DllShowName.data() ) ){
			DllVersion = children[i].get_value( errorInfo );
			if( errorInfo.GetErrorId() ){
				m_LastErrorDescription = errorInfo.GetErrorMsg();
				return false;
			}
			continue;
		}

		if( 0 == strcmp( "Vendor", DllShowName.data() ) ){
			DllVendor = children[i].get_value( errorInfo );
			if( errorInfo.GetErrorId() ){
				m_LastErrorDescription = errorInfo.GetErrorMsg();
				return false;
			}
		}
	}

	return true;
}
