/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <iostream>

#include "tracedll.h"
#include "wlerrorlib.h"

#include "associativearray.h"
#include "tvenv.h"
#include "tracedll.h"
#include "loadgenerator.h"
#include "loadagendamanager.h"
#include "loadengineglobals.h"
#include "loadengineinifileproperties.h"

//
// ***** Agenda handler class *****
//

// Constructors & destructors.
CLoadAgendaManager::CLoadAgendaManager(const char *name, long MaxThreadsPerSlave) :
  RVCollectableString(name)
{
	m_AgendaPaused = false;
  m_ScheduleNumRunning = 0;
  m_AgendaThrottled = false;
  m_SevereErrorOccurred = false;
  m_ThrottleNumRunning = -1;

  m_OneclientFlag = false;
  m_NumRunning=0;
  m_NumThreads=0;
  m_NumSlaves=0;

  m_MaxThreadsPerSlave = MaxThreadsPerSlave;

	m_LGSlaveTimeout = CLEIniFileProperties::LoadGeneratorSlaveTimeOut;

  /*
  tvpc.reset();
	const char* LGCMTimeout = (char *)tvpc.LEGetEnv("LGCommandManagerTimeout");
  if (LGCMTimeout!=NULL)
		m_CommandsManager.SetTimeout(atoi(LGCMTimeout));*/

  m_CopyFileObject.SetLoadAgendaManager(this);
  m_OutputFileHandler.SetLoadAgendaManager(this);
  m_InputFileHandler.SetLoadAgendaManager(this);
	m_LGGlobalsHandler.SetLoadAgendaManager(this);

  m_SlaveCountManager.Insert(0, 0);
  m_AgendaStatisticsManager.SetMyCLoadAgendaManager(this);
}

CLoadAgendaManager::~CLoadAgendaManager()
{
}

bool CLoadAgendaManager::setNumActive(int num)
{
  WlTraceMsg("setNumActive : %d",num);
  if (m_SevereErrorOccurred) return true;
  int i;
  //
  // The main of this class.
  // Setting the right number of threads.
  int expectedSlaves = num / m_MaxThreadsPerSlave;
  int lastSlaveNumRunning = num % m_MaxThreadsPerSlave;
  if (lastSlaveNumRunning!=0)
    expectedSlaves++;
  else
    lastSlaveNumRunning = m_MaxThreadsPerSlave;

  m_InputFileHandler.SetChunksInQueue(expectedSlaves>m_NumSlaves?expectedSlaves:m_NumSlaves);

  // 
  // initialize the command-manager with the maximum between expectedSlaves and m_NumSlaves.
  // Because we need to wait for an answer from all slaves that will exist before the m_CommandsManager.Wait()
  long requestId = m_CommandsManager.Init((expectedSlaves>m_NumSlaves?expectedSlaves:m_NumSlaves));
  //
  // Tell slaves how many to run.
  for (i = 0; i<expectedSlaves && i<m_NumSlaves; i++) {
    if (i<expectedSlaves-1)
      ((CLoadGeneratorSlaveProxy *)m_SlaveProxyArray[i])->SetSlaveNumRunning(requestId, m_MaxThreadsPerSlave);
    else
      ((CLoadGeneratorSlaveProxy *)m_SlaveProxyArray[i])->SetSlaveNumRunning(requestId, lastSlaveNumRunning);
  }
  //
  // Tell the rest of the slaves to run 0 threads.
  for (i = expectedSlaves; i<m_NumSlaves; i++) {
    ((CLoadGeneratorSlaveProxy *)m_SlaveProxyArray[i])->SetSlaveNumRunning(requestId, 0);
  }
  //
  // Create new slaves if necessary.
  while (m_NumSlaves < expectedSlaves) {
    m_NumSlaves++;
    CLoadGeneratorSlaveProxy *proxy = new CLoadGeneratorSlaveProxy(this, m_NumSlaves, false);
    m_SlaveProxyArray.insert(proxy);
    int baseThreadId = (m_NumSlaves-1)*m_MaxThreadsPerSlave;
    if (m_NumSlaves < expectedSlaves)
      proxy->CreateSlave(requestId, baseThreadId, m_MaxThreadsPerSlave);
    else
      proxy->CreateSlave(requestId, baseThreadId, lastSlaveNumRunning);
  }

  if (!m_CommandsManager.Wait()) {
    CLoadAgendaCommandsManager::ErrorReason reason = m_CommandsManager.Reason();
    if ((reason == CLoadAgendaCommandsManager::Timeout) || (reason == CLoadAgendaCommandsManager::ErrorActivatingApp)) {
      m_SevereErrorOccurred = true;
		  CAssociativeArray ErrorMessage;
		  ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
      if (reason == CLoadAgendaCommandsManager::Timeout) {
		    ErrorMessage.AddCouple("$AGENDALOGMESSAGE", "Slave is not responding to command");
      }
      else if (reason == CLoadAgendaCommandsManager::ErrorActivatingApp) {
		    ErrorMessage.AddCouple("$AGENDALOGMESSAGE", "Error while invoking slave");
      }
		  ErrorMessage.AddCouple("$AGENDANAME", m_Name.data());
		  ErrorMessage.AddCouple("$TIME", m_pMyLoadGenerator->GetTimeFromBeginning());
		  reportLogMessageString(ErrorMessage);
      m_pMyLoadGenerator->StopMe();
			WlTraceMsg("call stopMe %s",ErrorMessage.ToString());
      return false;
    }
    // in other cases the one who create an error is responsible for the error messages and
    // stoping the LG
  }

  m_NumRunning = num;
  if (m_NumThreads<num)
    m_NumThreads=num;

  return true;
}

  //
  // Added by Yahali Nov, 14 2001 for the ThreadPool changes
bool CLoadAgendaManager::LoadNumThreads(int num)
{
  WlTraceMsg("CLoadAgendaManager::LoadNumThreads(%d)", num);
  WlAssert(m_NumSlaves==1);
  WlTraceMsg("setNumActive : %d",num);
  if (num <= m_MaxThreadsPerSlave)
    return true;

  if (m_SevereErrorOccurred) return true;
  //
  // The main of this class.
  // Setting the right number of threads.
  int expectedSlaves = num / m_MaxThreadsPerSlave;
  if (num % m_MaxThreadsPerSlave!=0)
    expectedSlaves++;

  m_InputFileHandler.SetChunksInQueue(expectedSlaves);

  // 
  // initialize the command-manager with the maximum between expectedSlaves and m_NumSlaves.
  // Because we need to wait for an answer from all slaves that will exist before the m_CommandsManager.Wait()
  long requestId = m_CommandsManager.Init(expectedSlaves-1);
  //
  // Create new slaves if necessary.
  while (m_NumSlaves < expectedSlaves) {
    m_NumSlaves++;
    CLoadGeneratorSlaveProxy *proxy = new CLoadGeneratorSlaveProxy(this, m_NumSlaves, false);
    m_SlaveProxyArray.insert(proxy);
    int baseThreadId = (m_NumSlaves-1)*m_MaxThreadsPerSlave;
    proxy->CreateSlave(requestId, baseThreadId, 0);
  }

  if (!m_CommandsManager.Wait()) {
    CLoadAgendaCommandsManager::ErrorReason reason = m_CommandsManager.Reason();
    if ((reason == CLoadAgendaCommandsManager::Timeout) || (reason == CLoadAgendaCommandsManager::ErrorActivatingApp)) {
      m_SevereErrorOccurred = true;
		  CAssociativeArray ErrorMessage;
		  ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
      if (reason == CLoadAgendaCommandsManager::Timeout) {
		    ErrorMessage.AddCouple("$AGENDALOGMESSAGE", "Slave is not responding to command");
      }
      else if (reason == CLoadAgendaCommandsManager::ErrorActivatingApp) {
		    ErrorMessage.AddCouple("$AGENDALOGMESSAGE", "Error while invoking slave");
      }
		  ErrorMessage.AddCouple("$AGENDANAME", m_Name.data());
		  ErrorMessage.AddCouple("$TIME", m_pMyLoadGenerator->GetTimeFromBeginning());
		  reportLogMessageString(ErrorMessage);
      m_pMyLoadGenerator->StopMe();
			WlTraceMsg("call stopMe %s",ErrorMessage.ToString());
      return false;
    }
    // in other cases the one who create an error is responsible for the error messages and
    // stoping the LG
  }

  //if (m_NumThreads<num)
  //  m_NumThreads=num;

  return true;
}

// agenda loaduar functions.
bool CLoadAgendaManager::LoadAgenda(int size)
{
  WlTraceMsg("CLoadAgendaManager::LoadAgenda(%d)", size);
  //
  // Create the first Proxy.
  m_NumSlaves++;
  m_firstLoadGeneratorSlaveProxy = new CLoadGeneratorSlaveProxy(this, m_NumSlaves, true);
  m_SlaveProxyArray.insert(m_firstLoadGeneratorSlaveProxy);
  // & Wait for the Proxy to Finish.
  long requestId = m_CommandsManager.Init(1);
  if (!m_firstLoadGeneratorSlaveProxy->CreateSlave(requestId, 0, 0))
    return false;

  if (!m_CommandsManager.Wait())
    return false;

  //
  // Added by Yahali Nov, 14 2001 for the ThreadPool changes
  if (m_pMyLoadGenerator->m_ThreadPoolOn && !m_OneclientFlag) {
      LoadNumThreads(size);
  }
  
  //
  // Done - set status.
  if (m_OneclientFlag)
    m_Status = "AGENDAREADY";
  else
    m_Status = "READY";

  return true;
}

bool CLoadAgendaManager::LoadOneClientAgenda()
{
  m_OneclientFlag = true;
  return LoadAgenda(1);
}

bool CLoadAgendaManager::StartSession(CAssociativeArray& StartSessionCommand)
{
  //m_StartTime = startTime;
  //m_SliceSize = sliceSize;
  m_StartSessionCommand = StartSessionCommand.ToString();
  //
  // Start Load on the first slave.
  long requestId = m_CommandsManager.Init(m_NumSlaves);
  RVOrderedIterator spIterator(m_SlaveProxyArray);
  while (spIterator()) {
  CAssociativeArray *Message = new CAssociativeArray;
  Message->FromString(m_StartSessionCommand);
    CLoadGeneratorSlaveProxy* proxy = (CLoadGeneratorSlaveProxy*)spIterator.key();
    proxy->StartSession(requestId, Message);
  }

  if (!m_CommandsManager.Wait())
    return false;

  return true;
}

bool CLoadAgendaManager::StartAgenda(int n)
{
  bool ok;
  m_ScheduleNumRunning = n;
  if (!m_AgendaPaused && !m_AgendaThrottled) {
    ok = setNumActive(n);
    if (n==0)
      m_Status = "PAUSE";
    else
      m_Status = "RUN";
  }
  return ok;
}

bool CLoadAgendaManager::ThrottleAgenda(int n)
{
  bool ok = true;
  if (m_OneclientFlag)
    return false;

  m_ThrottleNumRunning = n;
  m_AgendaThrottled = true;

  if (!m_AgendaPaused)
    ok = setNumActive(n);

  return ok;
}

bool CLoadAgendaManager::UnThrottleAgenda()
{
  if (!m_AgendaThrottled)
    return true;

  m_ThrottleNumRunning = -1;
  m_AgendaThrottled = false;

  if (!m_AgendaPaused)
    return StartAgenda(m_ScheduleNumRunning);
  else
    return true;
}

bool CLoadAgendaManager::UserPauseAgenda()
{
  if (m_AgendaPaused)
    return true;

  m_AgendaPaused = true;
  bool ok = setNumActive(0);
  m_Status = "PAUSE";
  return ok;
}

bool CLoadAgendaManager::GoAgenda()
{
  if (!m_AgendaPaused)
    return true;

  m_AgendaPaused = false;
  if (m_AgendaThrottled)
    return ThrottleAgenda(m_ThrottleNumRunning);
  else
    return StartAgenda(m_ScheduleNumRunning);
}

bool CLoadAgendaManager::StopAgenda()
{
  if (m_Status == "STOP")
    return true;

  m_Status = "STOP";

  WlTraceMsg("CLoadAgendaManager::StopAgenda - Started. NumSlaves: %d",m_NumSlaves);
  //
  // stop all threadsmanagers.
  // Loop:
  long requestId = m_CommandsManager.Init(m_NumSlaves);
  RVOrderedIterator spIterator(m_SlaveProxyArray);
  while (spIterator()) {
    CLoadGeneratorSlaveProxy* proxy = (CLoadGeneratorSlaveProxy*)spIterator.key();
    WlTraceMsg("CLoadAgendaManager::StopAgenda - stopSlave");
    proxy->StopSlave(requestId);
  }
  //
  // Wait for Confirmation.
  bool status = m_CommandsManager.Wait();
  WlTraceMsg("CLoadAgendaManager::StopAgenda - After Wait. Status: %d",status);

	m_LGGlobalsHandler.Terminate();
  m_OutputFileHandler.Terminate();
  m_InputFileHandler.Terminate();

  WlTraceMsg("CLoadAgendaManager::StopAgenda - Done");
  return status;
}

void CLoadAgendaManager::ReportAgenda()
{
  CAssociativeArray ReportMessage;

  if (m_OneclientFlag) {
    if (m_Status == "STOP") {
      ReportMessage.AddCouple("$STATUS","STOP");
      ReportMessage.AddCouple("$AGENDARUNNING",0L);
    } else if (m_AgendaPaused) {
      ReportMessage.AddCouple("$STATUS","PAUSE");
      ReportMessage.AddCouple("$AGENDARUNNING",0L);
    } else {
      ReportMessage.AddCouple("$STATUS",m_Status);
      ReportMessage.AddCouple("$AGENDARUNNING",(long)m_NumRunning);
    }

    ReportMessage.AddCouple("$AGENDATIME", m_pMyLoadGenerator->GetTimeFromBeginning());

    if (strcmp(ReportMessage["$STATUS"],"STOP"))
			m_pMyLoadGenerator->SendMessage(ReportMessage, 2);
		else
			m_pMyLoadGenerator->SendMessage(ReportMessage, 5);

  } else {
    ReportMessage.AddCouple("$AGENDANAME", m_Name);
    ReportMessage.AddCouple("$AGENDATIME", m_pMyLoadGenerator->GetTimeFromBeginning());

    if (m_Status == "STOP") {
      ReportMessage.AddCouple("$AGENDASTATUS","STOP");
      ReportMessage.AddCouple("$AGENDARUNNING",0L);
    } else if (m_AgendaPaused) {
      ReportMessage.AddCouple("$AGENDASTATUS","PAUSE");
      ReportMessage.AddCouple("$AGENDARUNNING",0L);
    } else if (m_AgendaThrottled) {
      ReportMessage.AddCouple("$AGENDASTATUS","THROTTLED");
      ReportMessage.AddCouple("$AGENDARUNNING", (long)m_ThrottleNumRunning);
    } else {
      ReportMessage.AddCouple("$AGENDASTATUS",m_Status);
      ReportMessage.AddCouple("$AGENDARUNNING",(long)m_NumRunning);
    }
    ReportMessage.AddCouple("$AGENDAREACHABLES", (long)m_NumRunning);
    ReportMessage.AddCouple("$AGENDASIZE",(long)m_NumThreads);

		if (strcmp(ReportMessage["$AGENDASTATUS"],"STOP"))
			m_pMyLoadGenerator->SendMessage(ReportMessage, 2);
		else
			m_pMyLoadGenerator->SendMessage(ReportMessage, 5);
  }
}

bool CLoadAgendaManager::reportLogMessageString(CAssociativeArray& ErrorMessage)
{
  WLLogMessage::Severity severity = (WLLogMessage::Severity)ErrorMessage.GetLongValueOfParameter("$SEVERITY");
  const char* errorMessage = ErrorMessage.GetValueOfParameter("$AGENDALOGMESSAGE");
  if (m_pMyLoadGenerator) {
    if (m_pMyLoadGenerator->m_GeneratorType == CLoadGenerator::StandAlone) {
      if (m_pMyLoadGenerator->IsSendLogMessage() || severity > WLLogMessage::Note)
        cout << WLLogMessage::Name(severity) << " : " << errorMessage << endl;
      m_pMyLoadGenerator->AddLogMessage();
      return (severity != WLLogMessage::Fatal);
    }

    if (severity >= WLLogMessage::Fatal) {
      m_pMyLoadGenerator->SendMessage(ErrorMessage, 2);
			WlTraceMsg("call stopMe %s",ErrorMessage.ToString());
      m_pMyLoadGenerator->StopMe();
    }
    else {
      if (severity >= m_pMyLoadGenerator->GetMustLogBaseSeverity() || m_pMyLoadGenerator->IsSendLogMessage())
        m_pMyLoadGenerator->SendMessage(ErrorMessage, 4);
    }
    m_pMyLoadGenerator->AddLogMessage();
  }
  else {
    cout << errorMessage << endl;
	}

  return (severity != WLLogMessage::Fatal);
}

bool CLoadAgendaManager::reportLoadGeneratorMessage(CAssociativeArray& ErrorMessage)
{
  WLLogMessage::Severity severity = (WLLogMessage::Severity)ErrorMessage.GetLongValueOfParameter("$SEVERITY");
  const char* errorMessage = ErrorMessage.GetValueOfParameter("$GENERATORLOGMESSAGE");
  WlTraceMsg("CLoadAgendaManager::reportLoadGeneratorMessage - %s", errorMessage);
  if (m_pMyLoadGenerator) {
    if (m_pMyLoadGenerator->m_GeneratorType == CLoadGenerator::StandAlone) {
      if (m_pMyLoadGenerator->IsSendLogMessage() || severity > WLLogMessage::Note)
        cout << WLLogMessage::Name(severity) << " : " << errorMessage << endl;
      m_pMyLoadGenerator->AddLogMessage();
      return (severity != WLLogMessage::Fatal);
    }
    if (severity >= WLLogMessage::Fatal) {
      m_pMyLoadGenerator->SendMessage(ErrorMessage, 2);
			WlTraceMsg("call stopMe %s",ErrorMessage.ToString());
      m_pMyLoadGenerator->StopMe();
    }
    else {
      if (m_pMyLoadGenerator->GetMustLogBaseSeverity() || m_pMyLoadGenerator->IsSendLogMessage())
        m_pMyLoadGenerator->SendMessage(ErrorMessage, 4);
    }
    m_pMyLoadGenerator->AddLogMessage();
  }
  else { // m_pMyLoadGenerator==NULL which indicates single-test.
    cout << errorMessage << endl;
	}

  return (severity != WLLogMessage::Fatal);
}

void CLoadAgendaManager::SlaveFatalError()
{
  RVMutexLock::LockGuard lock(m_FatalErrorMutex);
	if (m_SevereErrorOccurred)
		return;

  if (m_Status != "STOP") {
    m_CommandsManager.Cancel();
    m_SevereErrorOccurred = true;
		WlTraceMsg("call stopMe ");
    m_pMyLoadGenerator->StopMe();
  }
}

bool CLoadAgendaManager::CopyMonitorFile(const char* sourcefilename,
																				 const char* targetfilename,
																				 const char* agendaPath,
																				 RVCString& errormessage)
{
  return m_CopyFileObject.CopyMonitorFile(sourcefilename, targetfilename, agendaPath, errormessage);
}

bool CLoadAgendaManager::IncludeMonitorFile(const char* sourcefilename,
																						const char** data,
																						const char* agendaPath,
																						RVCString& errormessage)
{
  return m_CopyFileObject.IncludeMonitorFile(sourcefilename, data, agendaPath, errormessage);
}

bool CLoadAgendaManager::StopSendingLogMessages() {
  RVOrderedIterator spIterator(m_SlaveProxyArray);
  while (spIterator()) {
    CLoadGeneratorSlaveProxy* proxy = (CLoadGeneratorSlaveProxy*)spIterator.key();
    proxy->StopSendingLogMessages();
  }
	return true;
}
bool CLoadAgendaManager::StopSendingTransactions() {
  RVOrderedIterator spIterator(m_SlaveProxyArray);
  while (spIterator()) {
    CLoadGeneratorSlaveProxy* proxy = (CLoadGeneratorSlaveProxy*)spIterator.key();
    proxy->StopSendingTransactions();
  }
	return true;
}
//////////////////// CLoadAgendaCommandsManager Implementation /////////////////
CLoadAgendaCommandsManager::CLoadAgendaCommandsManager()
{
  m_Id = 0;
  Terminate();
}

CLoadAgendaCommandsManager::~CLoadAgendaCommandsManager()
{
}

// returns the Command ID.
long CLoadAgendaCommandsManager::Init(int numSlaves)
{
  RVMutexLock::LockGuard lock(m_Mutex);
  WlTraceMsg("CLoadAgendaCommandsManager::Init");
  m_Count = numSlaves;
  m_Status = true;
  m_Ready.Reset();
  m_Id++;
  return m_Id;
}

void CLoadAgendaCommandsManager::Terminate()
{
  RVMutexLock::LockGuard lock(m_Mutex);
  WlTraceMsg("CLoadAgendaCommandsManager::Terminate");
  m_Count = 0;
  m_Ready.Set();
}

bool CLoadAgendaCommandsManager::Cancel(ErrorReason reason)
{
  RVMutexLock::LockGuard lock(m_Mutex);

  WlTraceMsg("CLoadAgendaCommandsManager::Cancel");
  m_Status = false;
  m_Reason = reason;
  m_Ready.Set();

  return true;
}

bool CLoadAgendaCommandsManager::Wait()
{
//  RVWaitStatus reason = m_Ready.Wait(m_Timeout);
  RVWaitStatus reason = m_Ready.Wait();
  if (reason == RV_THR_TIMEOUT) {
    m_Status = false;
    m_Reason = Timeout;
  }
  WlTraceMsg("CLoadAgendaCommandsManager::Wait - done");
  return m_Status;
}

bool CLoadAgendaCommandsManager::SlaveDone(int id, bool status)
{
  RVMutexLock::LockGuard lock(m_Mutex);

  if (m_Id != id)
    return false;

  if (m_Status) {

    WlTraceMsg("CLoadAgendaCommandsManager::SlaveDone");
    if (!status) {
      m_Status = status;
      m_Ready.Set();
    } else {
      m_Count--;
      if (m_Count==0)
        m_Ready.Set();
    }
  }

  return true;
}

//////////////////// CLoadAgendaManagerContainer Implementation /////////////////
/*
CLoadAgendaManager *CLoadAgendaManagerContainer::AddThreadsManager(const char *name)
{
  if (GetThreadsManager(name))
    return NULL;

  CLoadAgendaManager *ah = new CLoadAgendaManager(name);
  insertKeyAndValue((RVCollectable *) new RVCollectableString(name), (RVCollectable *) ah);

  if (ah == NULL)
    return NULL;
  else {
    //set the parent workspace for the agenda.
    ////ah->myWorkSpace.SetParentWorkSpace(&myWorkSpace);
    return ah;
  }
}


CLoadAgendaManager *CLoadAgendaManagerContainer::GetThreadsManager(const char *name) const
{
    RVCollectableString k(name);
    return (CLoadAgendaManager*)findValue((RVCollectable *) &k);
}


void CLoadAgendaManagerContainer::DeleteThreadsManager(const char *name)
{
  CLoadAgendaManager* ah = GetThreadsManager(name);
  if (ah){
    removeAndDestroy(ah);
  }
}

*/

