/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifdef _USE_SMARTHEAP_
#include <smrtheap.hpp>
#endif // _USE_SMARTHEAP_

#include "platform.h"
#include "tvtables.h"
#ifdef __TV_UNIX__
#include "tvenv.h"
#endif
#include "tracedll.h"
#include "wlerrorlib.h"
#include "loadengineglobals.h"
#include "loadgeneratorscript.h"
#include "loadengineinifileproperties.h"
#include "loadenginedecs.h"


#ifdef __TV_UNIX__
#include <pthread.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
void HttpLoadTerminateThread(int sig)
{
	pthread_exit((void *)-1);
}
#endif


int main(int argc, char *argv[])
{
#ifdef __TV_UNIX__
	TVEnvContainerClass tvpc;
	char thisFileName[MAX_PATH];
	memset(thisFileName,0,sizeof(thisFileName));
	(void)realpath(argv[0],thisFileName);
	const char* pathDir = tvpc.TVPGetPathDirectory(thisFileName);
	if( !TVEInit(pathDir))
	{
		TVPFatalErrorLog("httpload: Init Terminated with error: (%s)",TVEError());
		exit(-1);
	}
	// create signal handler
	tvpc.reset();
	const char *SignalMask = tvpc.TVEGetEnv(ENV_SIGNAL_MASK);
	TVPSignalHandler *hnd = new TVPSignalHandler(SignalMask?SignalMask:"");
	// Terminate Thread on various signals
	if (!tvpc.TVEEnvDefined("DO_NOT_OVERRIDE_SIGNALS","yes")) {
		hnd->TVPSigSet(SIGBUS,HttpLoadTerminateThread);
		hnd->TVPSigSet(SIGSEGV,HttpLoadTerminateThread);
	}
	
#ifdef _USE_SMARTHEAP_
	MemRegisterTask();
#endif // _USE_SMARTHEAP_
	
#endif
	CTraceMsg::Init("LoadEngine");//to enable to use the Trace.exe
	
	WlTraceMsg("Starting LoadGeneratorMain");
	
	int argc2 = argc;
	char** argv2 = argv;
	
	// Support of new parameters for command line activation, copied from httpload.cpp
	const char *tmp;
	bool  StandAlone = false;
	
	char argvData[10][100];
	char* argvp[10];
	
	// Check if first parameter is a .js file name
	if (argc > 1 && (tmp = (const char*)strrchr(argv[1], '.')) && !stricmp(tmp, ".js")){
		// Stand alone mode has no fixed number of parameters and we can only we're in it
		// if the the 1st parameter is a file name for a js script. So remember it now for later
		StandAlone = true;
	}
	else if(argc==2) {
		StandAlone = false;
		argv2 = argvp;
		
		argc2 = 0;
		argvp[0] = argv[0];
		
		FILE* InputFile = fopen( argv[1], "r" );
		WlAssert(InputFile);
		while( !feof(InputFile) ) {
			int status = fscanf( InputFile, "%s", argvData[argc2] );
			if( (status!=0) && (status!=EOF) ) {
				argvp[argc2+1] = argvData[argc2];
				++argc2;
			}
		}
		++argc2;
	}
	else {
		argc2 = argc;
		argv2 = argv;
	}
	
	
	if (StandAlone){
		LoadGeneratorTest(argc2, argv2, WLE_ADDPROTOCOL_HTTP);
	}
	// If we're not in Stand Alone mode, go back to the old way of deciding what the mode is
	else if (argc2 >= 5){
		LoadGeneratorMain(argc2, argv2, WLE_ADDPROTOCOL_HTTP);
	}
	else if (argc2 >= 3){
		LoadGeneratorSlave(argc2, argv2, WLE_ADDPROTOCOL_HTTP);
	}
	else{
#ifndef __TV_UNIX__
		MessageBox(0, "Illegal number of Parameters", "Load Generator", MB_OK);
#else
		printf("Illegal number of Parameters\n");
#endif
	}
	
	
#ifdef _USE_SMARTHEAP_
	/* unregister with SmartHeap */
	MemUnregisterTask();
#endif // _USE_SMARTHEAP_
	
	
	
	TVSTerminate();
	WLERRTerminate();
	//exit(0);
	return 0;
}



