/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvjsapi.h"

#include "platform.h"
#include "tvpsync.h"
#include "tracedll.h"
#include "loadjsdefs.h"
#include "loadengineglobals.h"

/*
class WLJSGlobalObjMutex : public TVPMutex {
private:
  HANDLE lockerTD;
public:
  WLJSGlobalObjMutex() : lockerTD(NULL) {}
  ~WLJSGlobalObjMutex() {}
  bool ObjectProtectedFirstTime();

};

bool WLJSGlobalObjMutex::ObjectProtectedFirstTime()
{
  HANDLE currentTD = GetCurrentThread();
  if (lockerTD == currentTD) {
    //WlTraceMsg("* Unlock: WLJSGlobalObjMutex::ObjectProtectedFirstTime");
    lockerTD == NULL;
    Unlock();
    return true;
  }
  //WlTraceMsg("* Before lock: WLJSGlobalObjMutex::ObjectProtectedFirstTime");
  Lock();
  //WlTraceMsg("* After lock: WLJSGlobalObjMutex::ObjectProtectedFirstTime");
  lockerTD = currentTD;
  return false;
}

static JSBool
WLJSGlobalObjAddProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
  JSBool retval;
	WLJSGlobalObjMutex *pMutex = (WLJSGlobalObjMutex*)JS_GetPrivate(cx, obj);
	const char* name = (const char*)JS_GetStringBytes(JSVAL_TO_STRING(id));
  if (pMutex->ObjectProtectedFirstTime())
    return JS_TRUE;
  else 
  	retval = JS_DefineProperty(cx, obj, name, *vp, NULL, NULL, JSPROP_ENUMERATE);
	return retval;
}

static JSBool
WLJSGlobalObjDelProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
  JSBool retval;
	WLJSGlobalObjMutex *pMutex = (WLJSGlobalObjMutex*)JS_GetPrivate(cx, obj);
	const char* name = (const char*)JS_GetStringBytes(JSVAL_TO_STRING(id));
  if (pMutex->ObjectProtectedFirstTime())
    return JS_TRUE;
  else 
	  retval = JS_DeleteProperty(cx, obj, name);

  return retval;
}

static JSBool
WLJSGlobalObjGetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
  JSBool retval;
	WLJSGlobalObjMutex *pMutex = (WLJSGlobalObjMutex*)JS_GetPrivate(cx, obj);
	const char* name = (const char*)JS_GetStringBytes(JSVAL_TO_STRING(id));
  if (pMutex->ObjectProtectedFirstTime())
    return JS_TRUE;
  else 
  	retval = JS_GetProperty(cx, obj, name, vp);
	return retval;
}

static JSBool
WLJSGlobalObjSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
  JSBool retval;
	WLJSGlobalObjMutex *pMutex = (WLJSGlobalObjMutex*)JS_GetPrivate(cx, obj);
	const char* name = (const char*)JS_GetStringBytes(JSVAL_TO_STRING(id));
  if (pMutex->ObjectProtectedFirstTime())
    return JS_TRUE;
  else 
  	retval = JS_SetProperty(cx, obj, name, vp);
	return retval;
}

static void
WLJSGlobalObjFinalize(JSContext *cx, JSObject *obj)
{
	WLJSGlobalObjMutex *pMutex = (WLJSGlobalObjMutex*)JS_GetPrivate(cx, obj);
  delete pMutex;
}
*/
// global object (WLGlobal) to hold global variables
static JSClass wlGlobal_class = {
	WLJSNAME_GLOBAL_CLASS,
	JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,	// add Property
  JS_PropertyStub,	// del Property
  CLEVariablesData::JsGet,	// get Property
  CLEVariablesData::JsSet,	// set Propery
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
  JS_FinalizeStub
}; 

JSObject* WLJSAddGlobalObject(JSContext* cx, JSObject* obj, CLEVariablesData* pVariablesData)
{
  // the same object name used for Both wlGlobalPerThread_class & wlGlobal_class
  JSObject* wlGlobal = JS_DefineObject(cx, obj, WLJSNAME_GLOBAL_OBJECT, &wlGlobal_class, NULL, JSPROP_PERMANENT);
	if (!wlGlobal)
		return wlGlobal;

  //
  // Registering the VariablesData as the PrivateData of the context.
  JS_SetPrivate(cx, wlGlobal, (void*)pVariablesData);

  //  WLJSGlobalObjMutex *pMutex = new WLJSGlobalObjMutex;
//	JS_SetPrivate(cx, wlGlobal, (void*)pMutex);

	return wlGlobal;
}

// global object (WLGlobal) per thread to hold global variables
static JSClass wlGlobalPerThread_class = {
	WLJSNAME_GLOBAL_PER_THREAD_CLASS,
	JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,	// add Property
  JS_PropertyStub,	// del Property
  CLEVariablesData::JsGet,	// get Property
  CLEVariablesData::JsSet,	// set Propery
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
  JS_FinalizeStub
}; 

bool WLJSIsGlobalPerThreadObject( JSContext* cx, JSObject* obj )
{
	return ( JS_InstanceOf( cx, obj, &wlGlobalPerThread_class, 0 ) ? true : false );
}

JSObject* WLJSAddGlobalPerThreadObject(JSContext* cx, JSObject* obj, CLEVariablesData* pVariablesData)
{
  // the same object name used for Both wlGlobalPerThread_class & wlGlobal_class
  JSObject* wlGlobal = JS_DefineObject(cx, obj, WLJSNAME_GLOBAL_OBJECT, &wlGlobalPerThread_class, NULL, JSPROP_PERMANENT);
	if (!wlGlobal)
		return wlGlobal;

  //
  // Registering the VariablesData as the PrivateData of the context.
  JS_SetPrivate(cx, wlGlobal, (void*)pVariablesData);

  //  WLJSGlobalObjMutex *pMutex = new WLJSGlobalObjMutex;
//	JS_SetPrivate(cx, wlGlobal, (void*)pMutex);

	return wlGlobal;
}



/*
class wlGlobalsValue : public RVCollectableInt {
private:
  RVCString strValue;
  double    numValue;

public:
  enum      ValueType { IntValue, DoubleValue, StringValue } valueType;

  wlGlobalsValue(const char*);
  wlGlobalsValue(long);
  wlGlobalsValue(double);

  SetValue(const char*);
  SetValue(long);
  SetValue(double);
  
  operator const char* () const { return (const char*) strValue; }
  operator long () const { return (long)numValue; }
  operator double () const { return numValue; }

  ValueType Type() {return valueType;}
  
  virtual ~ADLValue()  {}

}

wlGlobalsValue::wlGlobalsValue(const char* value) {
  strValue = value;
  valueType = StringValue;
}
wlGlobalsValue::wlGlobalsValue(long value) {
  numValue = (double)value;
  valueType = IntValue;
}
wlGlobalsValue::wlGlobalsValue(double value) {
  numValue = value;
  valueType = DoubleValue;
}
wlGlobalsValue::SetValue(const char* value) {
  strValue = value;
  valueType = StringValue;
}
wlGlobalsValue::SetValue(long value) {
  numValue = (double)value;
  valueType = IntValue;
}
wlGlobalsValue::SetValue(double value) {
  numValue = value;
  valueType = DoubleValue;
}

class wlGlobalsHash : public RVHashDictionary {
public:
  void SetValue(const char*,long);
  void SetValue(const char*,double);
  void SetValue(const char*,const char*);

  ADLValue* GetValue(const char* key);
  

}
*/


static JSClass wlLocal_class = {
	"WLLocals",
	JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,	// add Property
  JS_PropertyStub,	// del Property
  CLEVariablesData::JsGet,	// get Property
  CLEVariablesData::JsSet,	// set Propery
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
  JS_FinalizeStub
};

bool WLJSAddLocalObject(JSContext* cx, JSObject* obj, CLEVariablesData* pVariablesData)
{
  JSObject* wlLocal = JS_DefineObject(cx, obj, "wlLocals", &wlLocal_class, NULL, JSPROP_PERMANENT);
	if (!wlLocal)
		return false;

  //
  // Registering the VariablesData as the PrivateData of the context.
  JS_SetPrivate(cx, wlLocal, (void*)pVariablesData);

	return true;
}


bool AddGlobalsClass( JSContext* cx, JSObject* glob )
{
	bool Rtn( true );

	if( Rtn ) {
		Rtn = JS_InitClass(cx, glob, NULL, &wlGlobal_class, NULL, 0, NULL, NULL, NULL, NULL);
	}

	if( Rtn ) {
		Rtn = JS_InitClass(cx, glob, NULL, &wlGlobalPerThread_class, NULL, 0, NULL, NULL, NULL, NULL);
	}

	if( Rtn ) {
		Rtn = JS_InitClass(cx, glob, NULL, &wlLocal_class, NULL, 0, NULL, NULL, NULL, NULL);
	}

	return Rtn;
}
