/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "wlerrorlib.h"

#include "associativearray.h"
#include "loadexceptions.h"
#include "loadenginestatistics.h"
#include "loadengineglobals.h"
#include "loadgeneratorslave.h"
#include "loadthreadsmanager.h"
#include "loadreportinfo.h"






void IThreadsManager::reportLogCurrent(WLLogMessage::Severity rSeverity, const char* rMessage ) { 
	cLoadReportInfoAgenda loadRepInf( rSeverity, rMessage );
	cLoadReportHard errRep( loadRepInf );
	reportLog( errRep );
}


//
// ***** Agenda handler class *****
//

// Constructors & destructors.
CLoadThreadsManager::CLoadThreadsManager(const char *name) :
  m_ScriptManager(),
  m_ActiveThreads(0)
{
  m_OneclientFlag = false;
  m_Terminate = false;
  m_NumRunning=0;
  m_MixNumRunning=0;

  m_ThreadsStartEvent.Set();
  m_ThreadsRestartEvent.Reset();

	m_SynchronizationPointMutex = new RVMutexLock;
  m_EventIsTerminating = false;
  m_SendStatisticTimers = true;
	m_ThreadsManagerGlobalsHandler = NULL;
	m_ScriptManager.InitParent(this);
}

CLoadThreadsManager::~CLoadThreadsManager()
{
	delete m_ThreadsManagerGlobalsHandler; 
  delete m_SynchronizationPointMutex;
}

void CLoadThreadsManager::Init(const char *name, 
															 const char *realName,
															 CLoadGeneratorSlave *slave,
															 bool oneClientFlag)
{
	m_Name = name;
	m_RealName = realName;
	m_pMyLoadGeneratorSlave = slave;
	m_OneclientFlag = oneClientFlag;

  m_ThreadsManagerGlobalsHandler = new CThreadsManagerGlobalsRequestHandler(this);
}

bool CLoadThreadsManager::setNumActive(int num)
{
	RVMutexLock::LockGuard lock(*m_SynchronizationPointMutex);
	m_ScriptManager.ReleaseSynchronizationPoints();
	//  WlTraceMsg("CLoadThreadsManager::setNumActive: Alive Threads: %d", m_AliveThreadsCounter.Get());

  if (num == m_NumRunning)
    return true;

  if (num < m_NumRunning){
    m_TreatedThreadsCounter = m_NumRunning - num;
//    m_TreatedThreadsOn = true;
    WlTraceMsg("CLoadThreadsManager::setNumActive: Decreasing num threads from %d to %d", m_NumRunning ,num);
    m_NumRunning=num;
    //
    // waiting for the threads to start.
    RVWaitStatus waitStatus;
	  RVMutexLock::UnlockGuard lock(*m_SynchronizationPointMutex);
    while ((waitStatus = m_TreatedThreadsEvent.Wait(1000)) == RV_THR_TIMEOUT) {
      WlTraceMsg("CLoadThreadsManager::setNumActive: m_TreatedThreadsCounter = %d", m_TreatedThreadsCounter);
      if (m_Terminate)
        return true;
    }
    m_TreatedThreadsEvent.Reset();
    ::rvSleep(500);
  }
  else {
    if (!SetNumThreads(num)) {
        //
        // here we need to throw an error back to the main-loop.
        // need also to simulate the error and check if it works here
        // Yahali 12-May-1998
        //m_ErrorMsg = "Error in Init of new client"
      reportLogCurrent(WLLogMessage::Fatal, "Error creating threads for virtual clients (Probably due to lack of system resources)");
      m_pMyLoadGeneratorSlave->StopMe();
      return false;
    }

    bool a_AnyTreatedThreads = (m_ActiveThreads < num);

    // Chaim - if we already have threads up and running (thread pool)
    if (a_AnyTreatedThreads)
      m_TreatedThreadsCounter = num - m_NumRunning;
    else
      m_TreatedThreadsCounter = 0;

//    m_TreatedThreadsOn = true;
    WlTraceMsg("CLoadThreadsManager::setNumActive: Increasing num threads from %d to %d", m_NumRunning ,num);
    m_NumRunning=num;

    m_ThreadsStartEvent.Reset();
    m_ThreadsRestartEvent.Set();

    //
    //
    bool CreateError = false;
		RVCString errMessage;
    for (int i = m_ActiveThreads; i < m_NumRunning; i++){
      if (m_ThreadsVec[i]->run(errMessage) != 0){
        //
        // here we need to throw an error back to the main-loop.
        // need also to simulate the error and check if it works here
        // Yahali 12-May-1998
        //m_ErrorMsg = "cannot run new clients";
        reportLogCurrent(WLLogMessage::Fatal, (const char*)errMessage);
        m_pMyLoadGeneratorSlave->StopMe();
        CreateError=true;
        for (int j = i; j<m_NumRunning; j++) IncrementTreatedThreads();
        m_NumRunning=i;
        m_ActiveThreads=i;
        break;
      }
    }
    //
    // waiting for the threads to start.
    if (a_AnyTreatedThreads){
      RVWaitStatus waitStatus;
      while ((waitStatus = m_TreatedThreadsEvent.Wait(1000)) == RV_THR_TIMEOUT) {
        WlTraceMsg("CLoadThreadsManager::setNumActive: m_TreatedThreadsCounter = %d", m_TreatedThreadsCounter);
        if (m_Terminate)
          return true;
      }
    }
    m_TreatedThreadsEvent.Reset();

    // Let threads starts ASAP
    if (a_AnyTreatedThreads) m_ThreadsRestartEvent.Reset();
    m_ThreadsStartEvent.Set();

    if (!CreateError) {
      if (m_ActiveThreads < num)
        m_ActiveThreads = num;
      //return true;
    }
    else
      return false;
  }

  return true;
}

//
// Added by Yahali Nov, 14 2001 for the ThreadPool changes
bool CLoadThreadsManager::LoadNumThreads(int num)
{
  WlTraceMsg("CLoadThreadsManager::LoadNumThreads(num=%d)",num);
  if (num <= m_ActiveThreads)
    return true;

  if (!SetNumThreads(num)) {
    //
    // here we need to throw an error back to the main-loop.
    // need also to simulate the error and check if it works here
    // Yahali 12-May-1998
    //m_ErrorMsg = "Error in Init of new client"
    reportLogCurrent(WLLogMessage::Fatal, "Error in building client");
    m_pMyLoadGeneratorSlave->StopMe();
    return false;
  }

  m_TreatedThreadsCounter = num;
  //m_TreatedThreadsOn = true;

  m_ThreadsStartEvent.Reset();
  m_ThreadsRestartEvent.Set();

  //
  //
  bool CreateError = false;
	RVCString errMessage;
  for (int i = m_ActiveThreads; i < num; i++){
    if (m_ThreadsVec[i]->run(errMessage) != 0){
      //
      // here we need to throw an error back to the main-loop.
      // need also to simulate the error and check if it works here
      // Yahali 12-May-1998
      //m_ErrorMsg = "cannot run new clients";
      reportLogCurrent(WLLogMessage::Fatal, (const char*)errMessage);
      m_pMyLoadGeneratorSlave->StopMe();
      CreateError=true;
      for (int j = i; j<num; j++) IncrementTreatedThreads();
      m_ActiveThreads=i;
      break;
    }
  }

  //
  // waiting for the threads to start.
  WlTraceMsg("CLoadThreadsManager::LoadNumThreads, Waiting for threads to start");
  RVWaitStatus waitStatus;
  while ((waitStatus = m_TreatedThreadsEvent.Wait(1000)) == RV_THR_TIMEOUT) {
    WlTraceMsg("CLoadThreadsManager::setNumActive: m_TreatedThreadsCounter = %d", m_TreatedThreadsCounter);
    if (m_Terminate)
      return true;
  }
  WlTraceMsg("CLoadThreadsManager::LoadNumThreads, m_TreatedThreadsEvent.Reset()");
  m_TreatedThreadsEvent.Reset();
  ::rvSleep(500);

  WlTraceMsg("CLoadThreadsManager::LoadNumThreads, m_ThreadsRestartEvent.Reset()");
  m_ThreadsRestartEvent.Reset();

  WlTraceMsg("CLoadThreadsManager::LoadNumThreads, m_ThreadsStartEvent.Set()");
  m_ThreadsStartEvent.Set();

  if (!CreateError) {
    if (m_ActiveThreads < num)
      m_ActiveThreads = num;
    //return true;
  }
  else
    return false;

  return true;
}

void CLoadThreadsManager::IncrementTreatedThreads()
{
  RVCriticalSection::LockGuard lock(m_TreatedThreadsMutex);
  WlTraceMsg("CLoadThreadsManager::IncrementTreatedThreads, m_TreatedThreadsCounter=%d",m_TreatedThreadsCounter);
  //if (!m_TreatedThreadsOn)
  //  return;
  m_TreatedThreadsCounter--;
  if (m_TreatedThreadsCounter == 0)
    m_TreatedThreadsEvent.Set();
  WlTraceMsg("CLoadThreadsManager::IncrementTreatedThreads, Done, m_TreatedThreadsCounter=%d",m_TreatedThreadsCounter);
}

bool CLoadThreadsManager::SetNumThreads(int size)
{
  CLoadThreadInstance *thread;

  //WlTraceMsg("SetNumThreads(%d), increasing from %d",size, m_ThreadsVec.entries());

  for (int i = m_ThreadsVec.entries(); i < size; i++){
    thread = new CLoadThreadInstance(this, m_ThreadsVec.entries(), m_pMyLoadGeneratorSlave->fGetProcess() );
    if (!thread){
      return false;
    }
    //
    // Yahali 21-Nov-2001 
    // Moved to thread first-time initialization
    if (!m_pMyLoadGeneratorSlave->m_ThreadPoolOn) {
	    if (!thread->Init()) {
				return false;
			}
    }
    m_ThreadsVec.append( static_cast< eiRVInheritArc< CLoadThreadInstance, RVCollectableInt >* >(thread) );
  }
  return true;
}

void CLoadThreadsManager::OnSevereError()
{
  if (m_ErrorFlag.Set() == 0){ //save err message in agenda's work space
    //m_Terminate = true;
    m_ScriptManager.OnSevereError();
    if (m_pMyLoadGeneratorSlave)
      m_pMyLoadGeneratorSlave->StopMe();
  }
}


bool CLoadThreadsManager::IsOneClient() {
  return m_OneclientFlag;
}

bool CLoadThreadsManager::IsLoadServer() {
  return (!m_OneclientFlag);
}

bool CLoadThreadsManager::LoadScript(const char* scriptName, const char* scriptFileName, const char* ScriptCode, double weight, const char* InitializationScript, int numScriptInMix)
{
  setCurrentThreadsManager(this);
  return m_ScriptManager.LoadScript(scriptName, scriptFileName, ScriptCode, weight, InitializationScript , numScriptInMix);
}

void CLoadThreadsManager::StartLoad(double startTime, int sliceSize)
{
  //m_StartTime = startTime;
  m_ScriptManager.StartLoad(startTime, sliceSize);
}

bool CLoadThreadsManager::StopAgenda()
{
//  if (m_Status == "STOP")
//    return true;

//  m_Status = "STOP";

  WlTraceMsg("CLoadThreadsManager::StopAgenda - Started");

  WlTraceMsg("CLoadThreadsManager::StopAgenda - Wait for threads to terminate");
  //
  // stop all threads.
  m_TreatedThreadsCounter = m_AliveThreadsCounter.Get();
	{
  	RVMutexLock::LockGuard lock(*m_SynchronizationPointMutex);
		m_ScriptManager.ReleaseSynchronizationPoints();
		m_NumRunning=0;
    WlTraceMsg("CLoadThreadsManager::StopAgenda - After numRunning. NumRunning:%d",m_NumRunning);
		m_Terminate = true;
    WlTraceMsg("CLoadThreadsManager::StopAgenda - After Terminate. NumRunning:%d",m_NumRunning);
	}

  //  release threads from waiting for the input file
  m_ScriptManager.InputFileTerminate();

  m_ThreadsRestartEvent.Set();
  //
  // waiting for the threads to terminate.
  WlTraceMsg("CLoadThreadsManager::StopAgenda - before while. NumRunning:%d",m_NumRunning);
  
  while (m_TreatedThreadsEvent.Wait(1000) == RV_THR_TIMEOUT) {
    WlTraceMsg("CLoadThreadsManager::StopAgenda - inside while. NumRunning:%d",m_NumRunning);
    if (m_AliveThreadsCounter.Get() == 0)
      break;
  }
  //
  // terminate the agenda.
  WlTraceMsg("CLoadThreadsManager::StopAgenda - Wait for script to terminate");
  setCurrentThreadsManager(this);
  m_ScriptManager.Terminate();

  // @@Chaim - try to kill threads
  for (int i = 0; i < m_ActiveThreads; i++){
    m_ThreadsVec[i]->stop();
  }

  try {
//    For Performance, leave the clearAndDestroy in comments although it is "memory leak".
//    The Stop process stuck when the Dtor was activated
//    m_ThreadsVec.clearAndDestroy(); // clear vector before thread destruction
  } catch (...) {
    WlTraceMsg("Fatal Error in CLoadThreadsManager::StopAgenda");
    #ifdef _DEBUG
    CAssociativeArray	ErrorMessage;
    ErrorMessage.AddCouple("$GENERATORLOGMESSAGE","Fatal Error in CLoadThreadsManager::StopAgenda");
    ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
    ErrorMessage.AddCouple("$TIME", m_pMyLoadGeneratorSlave->fGetTimeFromBegin());
    m_pMyLoadGeneratorSlave->SendMessage(ErrorMessage, 2);
    throw;
    #endif
  }

  WlTraceMsg("CLoadThreadsManager::StopAgenda - Done");
  return 1;
}



void CLoadThreadsManager::reportLog(
	iReport& rReport,
	int threadNum)
{ 
	WLLogMessage::Severity aSeverity( rReport.fGetSeverity() );
	const char* aMessage = ( rReport.fGetMessage() );
	bool aHasTime( rReport.fHasTime() );
	double aTime( rReport.fGetTime() );


  if (m_pMyLoadGeneratorSlave) {
    if (m_pMyLoadGeneratorSlave->m_GeneratorType == CLoadGeneratorSlave::StandAlone) {
      if (m_pMyLoadGeneratorSlave->IsSendLogMessage() || aSeverity > WLLogMessage::Note)
        cout << WLLogMessage::Name(aSeverity) << " : " << aMessage << endl;
      m_pMyLoadGeneratorSlave->AddLogMessage();
      return;
    }
    CAssociativeArray ErrorMessage;
    ErrorMessage.AddCouple("$AGENDALOGMESSAGE", aMessage);
    ErrorMessage.AddCouple("$AGENDANAME", m_Name.data());
    ErrorMessage.AddCouple("$TIME", (aHasTime ? aTime : m_pMyLoadGeneratorSlave->fGetTimeFromBegin() ) );
    if (threadNum != -1){
	    ErrorMessage.AddCouple("$THREADNUM", (long)threadNum);
	}
    ErrorMessage.AddCouple("$SEVERITY", (long)aSeverity);
    if (aSeverity >= m_pMyLoadGeneratorSlave->GetMustLogBaseSeverity())
      m_pMyLoadGeneratorSlave->SendMessage(ErrorMessage, 2);
    else {
      if (m_pMyLoadGeneratorSlave->IsSendLogMessage())
        m_pMyLoadGeneratorSlave->SendMessage(ErrorMessage, 4);
    }
    m_pMyLoadGeneratorSlave->AddLogMessage();
  }
  else { // m_pMyLoadGenerator==NULL which indicates single-test.
    cout << aMessage << endl;
	}
}

int CLoadThreadsManager::GetAgendaNumRunning()
{
  return m_NumRunning;
}

const char* CLoadThreadsManager::GetLoadGeneratorNo() 
{
  return m_pMyLoadGeneratorSlave->GetLoadGeneratorNo();
}

int CLoadThreadsManager::GetErrorCounter() 
{
  return m_pMyLoadGeneratorSlave->GetErrorCounter();
}

int CLoadThreadsManager::GetWarningCounter() 
{
  return m_pMyLoadGeneratorSlave->GetWarningCounter();
}

void CLoadThreadsManager::IncrementErrorCounter() 
{
  m_pMyLoadGeneratorSlave->IncrementErrorCounter();
}

void CLoadThreadsManager::IncrementWarningCounter() 
{
  m_pMyLoadGeneratorSlave->IncrementWarningCounter();
}

void CLoadThreadsManager::InputFileData(CAssociativeArray* AssoArrayCommand) {
  m_ScriptManager.InputFileData(AssoArrayCommand);
}
