/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <RVCString.h>

#include "wlerrorlib.h"
#include "loadengineselistener.h"
#include "loadjsshell.h"
#include "loadexceptions.h"
#include "loadengineglobals.h"
#include "loadthreadinstance.h"
#include "loadthreadsmanager.h"
#include "wljsruntime.h"

#include "loadgeneratorscript.h"
#include "loadengineverificationclasses.h"

#include "rtamscript.h"
#include "tracedll.h"

#include "loadengineinifileproperties.h"
#include "loadenginedecs.h"
#include "loadjserrorfuncs.h"
#include "tvenv.h"
#include "gt_unicode.h"

#ifdef __TV_WINDOWS__
#include <objbase.h>
#endif

#include "jsjavacore.h"

//####### The implementation of the class cSingleClientProcess   #########

cSingleClientProcess::cSingleClientProcess()
:
	CGeneralProcess()
{
	CLEIniFileProperties::Init();
	CGeneralProcess::SetMasterPid( TVPgetpid() );
}

cSingleClientProcess::~cSingleClientProcess() {
}

void cSingleClientProcess::fTerminate() {

  CGeneralProcess::Terminate();
}

bool cSingleClientProcess::fInit()
{
	RVCString errNameDll;
  if (!CGeneralProcess::Init(errNameDll)){
    return false;
  }

	return true;
}




iSingleClientMachineThread::iSingleClientMachineThread()
:
	iLEJSAgendaMachineThread( cSingleClientProcess::fsGetSingleton() )
{}

iSingleClientMachineThread::~iSingleClientMachineThread()
{}

cSingleClientMachineThread::cSingleClientMachineThread()
{}




iSCActionReportInfo::iSCActionReportInfo()
:
	mDoneApply(false)
{}


iSCActionReportInfo::~iSCActionReportInfo()
{
	WlAssert( mDoneApply );
}

void iSCActionReportInfo::fReport(
	JSContext* rContext )
{
	bool aStatus = LoadReport( rContext, *this );
	WlAssert(aStatus);
}


tDebugReporterReturn iSCActionReportInfo::vApplyDebug( cJSDReportCommit& rCommit )
{
	WlAssert(!mDoneApply);

	tDebugReporterReturn rtn;

	tSCActionReporterReturn aReturnValue( sSCActionReporterReturnContinue );
	WlAssert( dynamic_cast<iLoadSingleClient*>(&(rCommit.fGetUi())));
	fApply( aReturnValue, static_cast<iLoadSingleClient&>(rCommit.fGetUi()), &rCommit.fGetThreadState() );

	switch(aReturnValue) {
	case sSCActionReporterReturnContinue:
		rtn = sDebugReporterReturnApply;
		break;
	case sSCActionReporterReturnDM:
		rtn = sDebugReporterReturnDM;
		break;
	default:
		WlAssert(0);
	}
	mDoneApply = true;

	return rtn;
}

void iSCActionReportInfo::vApplyJSExec( iJSExecReportCommit& rCommit )
{
	if( !mDoneApply ) {
		tSCActionReporterReturn aReturnValue( sSCActionReporterReturnContinue );

		WlAssert( dynamic_cast<iLEJSEmbedContext*>( &rCommit.fGetEmbedContext() ) );
		iLEJSEmbedContext& aEmbedContext = static_cast<iLEJSEmbedContext&>( rCommit.fGetEmbedContext() );


		WlAssert( dynamic_cast<iLoadSingleClient*>(&aEmbedContext.fGetSession()) );
		fApply( aReturnValue, static_cast<iLoadSingleClient&>( aEmbedContext.fGetSession() ), &rCommit.fGetThreadState() );

		WlAssert( aReturnValue == sSCActionReporterReturnContinue );
		mDoneApply = true;
	}
}

bool iSCActionReportInfo::vIsStop() const
{
	return false;
}


void iSCActionReportInfo::vApplyHard(
	iLEJSEmbedContext& rEmbedContext )
{
	WlAssert( !mDoneApply );

	tSCActionReporterReturn aReturnValue( sSCActionReporterReturnContinue );

	WlAssert( dynamic_cast<iLoadSingleClient*>( &rEmbedContext.fGetSession()) );
	fApply( aReturnValue, static_cast<iLoadSingleClient&>( rEmbedContext.fGetSession() ), NULL );

	WlAssert( aReturnValue == sSCActionReporterReturnContinue );
	mDoneApply = true;
}


cSCMainFileCommit::cSCMainFileCommit(
	iAgendaFile& File,
	CWLJSScript& Script ):
	iFileSourceCommit(File),
	mScript(Script),
	mStatus(false) {}

void cSCMainFileCommit::fTakeFile(
	const char* FileEncription,
	int Length,
	const char* Bytes ) {

/*GALTECH*/
#ifndef _ANSI_ONLY
	int countA = 0;
	//Calling with buffer NULL, in order to know real bytes
	countA = ANSIToUTF8(GetMonitorCharSet(), Bytes, Length, NULL, 0);
	char *UtfBuf = new char[countA + 1];
	//Real conversion
	ANSIToUTF8(GetMonitorCharSet(), Bytes, Length, UtfBuf, countA);
	UtfBuf[countA] = 0;
	mStatus = mScript.LoadScript(FileEncription, countA, UtfBuf);
	delete [] UtfBuf;
#else
    mStatus = mScript.LoadScript(FileEncription, Length, Bytes);
#endif
	}

bool cSCMainFileCommit::fGetStatus() {
	return mStatus;
	}




cSCFileCommit::cSCFileCommit(
	iAgendaFile& File,
	iLoadSingleClient& SC,
	bool Execute ):
	iFileSourceCommit(File),
	mSC(SC),
	mExecute(Execute),
	mStatus(true) {
	}

void cSCFileCommit::fTakeFile(
	const char* PointerEncryption,
	int Length,
	const char* Bytes ) {

	if( !mSC.m_jsScript->IncludeFile( PointerEncryption, Length, Bytes, mExecute ) ) {
		// @@Chaim (inherited from Itaj): should use iEmbedderCallContext instead
		mStatus = false;
	}
}

bool cSCFileCommit::fGetStatus() const {
	return mStatus;
}


const char cSCGlobalsRequestHandler::msObjectName[] = "ProbingClientGlobalsObjectName";

void cSCGlobalsRequestHandler::Globals_Set(
	const char* variableName,
	CGlobalVariableProperties *newProperties,
	Level level) {

	CGlobalVariableProperties* properties = new CGlobalVariableProperties(newProperties->GetType(), newProperties->GetUnformatedCharValue());
	mGlobalsTable.Set( msObjectName, variableName, properties );
	}

void cSCGlobalsRequestHandler::Globals_Add(
	const char* variableName,
	CGlobalVariableProperties *newProperties,
	Level level) {

	CGlobalVariableProperties* properties = new CGlobalVariableProperties(newProperties->GetType(), newProperties->GetUnformatedCharValue());
	mGlobalsTable.Add( msObjectName, variableName, properties );
	}

CGlobalVariableProperties* cSCGlobalsRequestHandler::Globals_SyncAdd(
	const char* variableName,
	CGlobalVariableProperties *newProperties,
	Level level) {

	CGlobalVariableProperties* properties = new CGlobalVariableProperties(newProperties->GetType(), newProperties->GetUnformatedCharValue());
	return mGlobalsTable.SyncAdd( msObjectName, variableName, properties );
	}

CGlobalVariableProperties* cSCGlobalsRequestHandler::Globals_Get(
	const char* variableName,
	Level level) {

	CGlobalVariableProperties* properties = mGlobalsTable.Get( msObjectName, variableName );
	CGlobalVariableProperties* Rtn = new CGlobalVariableProperties(properties->GetType(), properties->GetUnformatedCharValue());
	return Rtn;
	}



cSCThreadTransactionManager::cSCThreadTransactionManager():mTransactionEventID(0)
{}

void cSCThreadTransactionManager::InitParent(
	iLoadSingleClient& Parent)
{
	mSC = &Parent;
}


CLoadGeneralThreadInstance* cSCThreadTransactionManager::GetThreadInstance() {
	return mSC->m_ThreadInstance;
	}


void cSCThreadTransactionManager::AddLast(
	const char* name,
	double l,
	CWLStatPointTypeEnum::PointType type)
{
	mSC->fThreadAddLast( name, l, type );
}

void cSCThreadTransactionManager::SendWhole() {
	}

void cSCThreadTransactionManager::SendEvent( CLoadTransactionEvent& ) {
	}

void cSCThreadTransactionManager::RecordEventCallBack(CLoadTransactionEvent& transactionEvent,
																											long topInstanceID,
																											long instanceID)
{
	mSC->fRecordEvent(mTransactionEventID++,
									 topInstanceID,
									 instanceID,
									 transactionEvent.m_Severity,
									 transactionEvent.m_Name.data(),
									 transactionEvent.m_Time,
									 transactionEvent.m_LineNumber,
									 transactionEvent.m_FileName.data(),
									 transactionEvent.m_Description.data(),
									 transactionEvent.m_ExtraDataType.data(),
									 transactionEvent.m_ExtraData.data());
}

void cSCThreadTransactionManager::BeginTransactionCallBack(CLoadTransaction& transaction)
{
	mSC->fBeginTransaction(transaction.m_InstanceID,
												transaction.m_TransactionName.data(),
												transaction.m_ExtraDataType.data());
}

void cSCThreadTransactionManager::EndTransactionCallBack(CLoadTransaction& transaction)
{
  const char* extraData = NULL;

  if(transaction.m_ExtraData.length() > 0) {
    extraData = transaction.m_ExtraData.data();
  }

	mSC->fEndTransaction(transaction.m_TopInstanceID,
											transaction.m_InstanceID,
											transaction.m_ParentID,
											mSC->fRoundNum(),
											transaction.m_Reason.data(),
											transaction.m_TransactionName.data(),
											transaction.m_Time,
											transaction.m_Severity,
											transaction.m_ExtraDataType.data(),
											extraData);
}

bool cSCThreadTransactionManager::IsSendTransactions() {
	return true;
	}

double cSCThreadTransactionManager::vGetTimeFromBegin() {
	return mSC->fGetTimeFromBegin();
	}

double cSCThreadTransactionManager::vGetBeginTime() {
	return mSC->fGetBeginTime();
	}

double cSCThreadTransactionManager::vGetCurrentTime() {
	return mSC->fGetProcess().fGetCurrentTime();
}


cSCThreadsManager::cSCThreadsManager()
{}

void cSCThreadsManager::InitParent(
	iLoadSingleClient& Parent)
{
	mSC = &Parent;
}



void cSCThreadsManager::reportLog(
	iReport& rReport,
  int threadNum)
{
	// this one is called only by hard proto reporter
	mSC->fExecReporter( NULL, rReport.fGetInfo() );
}

const char* cSCThreadsManager::GetAgendaName() {
	return mSC->m_AgendaName.data();
	}

const char* cSCThreadsManager::GetLoadGeneratorNo() {
	return 0;
	}

int cSCThreadsManager::GetErrorCounter() 
{
  return mSC->GetErrorCounter();
}

int cSCThreadsManager::GetWarningCounter() 
{
  return mSC->GetWarningCounter();
}

void cSCThreadsManager::IncrementErrorCounter() 
{
  mSC->IncrementErrorCounter();
}

void cSCThreadsManager::IncrementWarningCounter() 
{
  mSC->IncrementWarningCounter();
}
/////////////////////////////////////////////////////////////////////
//									cSCStatClient
////////////////////////////////////////////////////////////////////



cSCStatClient::cSCStatClient()
:
  mDictionary(0),
	mTimingDictionary()
{}

cSCStatClient::~cSCStatClient()
{}
/*
void cSCStatClient::GetNewDictionary()
{
  if (pCurrentDictionary)
    pCurrentDictionary->ReleaseClient();
  pCurrentDictionary = pGenerator->GetNewDictionary(clientId, pCurrentDictionary);
  pCurrentDictionary->SetClient(this);
}

void cSCStatClient::ReleaseDictionary()
{
  pCurrentDictionary = NULL;
}

double cSCStatClient::GetCurrentEndTime()
{
  return pCurrentDictionary->GetSlice()->GetEndTimeSlice();
}
*/
void cSCStatClient::fAddLast(const char *measurement, double v , CWLStatPointTypeEnum::PointType type)
{
	bool status = mDictionary.AddLast(measurement,v,type);
	WlAssert(status);
}


void cSCStatClient::fResetTimer(const char *name)
{
	mTimingDictionary.ResetTimer(name);
}


void cSCStatClient::fSendTimer(const char *name)
{
	double time=mTimingDictionary.SendTimer(name);
  fAddLast(name,time,CWLStatPointTypeEnum::Timer);
}



cSCManagerScriptData::cSCManagerScriptData()
:
	IGeneralManagerScriptData()
{}

void cSCManagerScriptData::InitParent(
	iLoadSingleClient& Parent)
{
	mSC = &Parent;
}

CWLJSBaseObject& cSCManagerScriptData::GetJSBaseObject() {
  WlAssert (mSC);
  return *mSC->m_jsScript;
}



cSCManagerScriptData::~cSCManagerScriptData() {}

CSCFileDictionary& cSCManagerScriptData::GetInputFiles()
{
	return mInputFiles;
}



CWLJSScript& cSCManagerScriptData::GetJSScript() {
	return *mSC->m_jsScript;
	}

CGeneralProcess& cSCManagerScriptData::fGetProcess(){
	return mSC->fGetProcess();
}

iSession& cSCManagerScriptData::vGetSession(){
	return *mSC;
}

void cSCManagerScriptData::reportLog(
	iReport& rReport )
{
	WLLogMessage::Severity aSeverity( rReport.fGetSeverity() );
	const char* aMessage = ( rReport.fGetMessage() );

	mSC->fExecReporter( rReport.fGetThreadState(), rReport.fGetInfo() );
/* @@Chaim (inherited from Itaj): is this change ok?
	if( IsJSActive() ) {
		mSC->fJSReporter( aSeverity, aMessage );
	}	else {
		mSC->fReporter( aSeverity, aMessage );
	}
*/
}



bool cSCManagerScriptData::vIncludeFile_IGeneralManagerScriptData(
	const char* FileName,
	bool Execute ) {

	cLEJSEmbedderCallContext EC( mSC->m_jsScript->GetContext(), *this );
	//return IncludeFile( FileName, Execute, ErrorMessage );

	bool IsNew;
	iAgendaFile* File = mSC->fGetFile( IsNew, EC, FileName );
	bool Rtn = (File ? true : false);

	if( IsNew ) {

		if( Rtn ) {
			cSCFileCommit Commit( *File, *mSC, Execute );
			File->fSourceExport( EC, Commit );
			Rtn = EC.fGetJSReturn();
			if(Rtn) {
				Rtn = Commit.fGetStatus();
			}
		}
	}

	return Rtn;
}


void cSCManagerScriptData::OnSevereError() {
	mSC->OnSevereError();
}

bool cSCManagerScriptData::CopyFile(
	const char* Source,
	const char* Target ) {

	cLEJSEmbedderCallContext EC( mSC->m_jsScript->GetContext(), *this );
	mSC->fCopyFile( EC, Source, Target );

	return EC.fGetJSReturn();
	}


void cSCManagerScriptData::AppendStringForEvaluation(
	const char* Str ,CWLJSScript::WhenToRunScript whenToRunScript)
{
	mSC->m_jsScript->AppendStringForEvaluation( Str, whenToRunScript );
}




bool cSCManagerScriptData::InInitAgenda() {
	return( mSC->m_jsScript->IsInInitAgenda() );
	}

bool cSCManagerScriptData::IsSpawn() {
	return false;
	}


IFileDictionary* cSCManagerScriptData::GetFileDict() {
	return &GetInputFiles();
	}

IGeneralGlobalsRequestHandler* cSCManagerScriptData::GetGlobalsRequestHandler() {
	return &mGlobalsRequestHandler;
}


RVCString& cSCManagerScriptData::ScriptName() {
	//WlAssert(0); // @@Chaim (inherited from Itaj): not impl
	static RVCString X("the script name"); return X;
	}


RVCString& cSCManagerScriptData::ScriptFileName() {
	WlAssert(0); // @@Chaim (inherited from Itaj): not impl
	static RVCString X; return X;
	}


void cSCManagerScriptData::SendMessage(
	CAssociativeArray& Message,
	int Priority ) {
	WlAssert(0); // @@Chaim (inherited from Itaj): not impl, requires polymorph caller
}


iLEJSMachineThread& cSCManagerScriptData::vGetMachineThread()
{
	WlAssert(mSC->mMThread);
	return *mSC->mMThread;
}

int cSCManagerScriptData::GetErrorCounter(){
	return mSC->GetErrorCounter();
}

int cSCManagerScriptData::GetWarningCounter(){
	return mSC->GetWarningCounter();
}

void cSCManagerScriptData::IncrementErrorCounter(){
	mSC->IncrementErrorCounter();
}

void cSCManagerScriptData::IncrementWarningCounter(){
	mSC->IncrementWarningCounter();
}

void cSCManagerScriptData::vEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType rCallType )
{
	WlAssert(mSC->m_jsScript);
	mSC->m_jsScript->EnterJava();
}

void cSCManagerScriptData::vExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType rCallType )
{
	WlAssert(mSC->m_jsScript);
	mSC->m_jsScript->ExitJava();
}

bool cSCManagerScriptData::vIsJSGC()
{
	return true;
}

void cSCManagerScriptData::SetJavaThreadWrapperCreationScript()
{
	WlAssert(mSC->m_jsScript);
	mSC->m_jsScript->SetJavaThreadWrapperCreationScript();
}

bool cSCManagerScriptData::IsJavaThreadWrapperCreationScriptExists()
{
	WlAssert(mSC->m_jsScript);
	return mSC->m_jsScript->IsJavaThreadWrapperCreationScriptExists();
}


int cSCManagerScriptData::GetNumScriptInMix() {
	return 1;
}
/*
void cSCManagerScriptData::fActionReportCommit( iActionReportInfo& rReportInfo )
{
	rReportInfo.fCommit( *mSC );
}
*/


////////////////////////////////////////////////////////////////
//
//      The implementation of class iLoadSingleClient
//
////////////////////////////////////////////////////////////////


iLoadSingleClient::iLoadSingleClient(
	cSingleClientProcess& rProcess,
	bool IsDebuggerActive)
:
	iSession(rProcess),
	iSpaceUi( IsDebuggerActive ),
	mMThread(NULL),
	mTransactionManager(),
	mThreadsManager(),
	mManagerScriptData(),
	mInitAgendaRun(false),
	mIsAbort(false),
	m_ThreadInstance(0),
	m_ThreadInstanceScript(0),
	m_jsScript(0),
	mBeginTime(0)
{
	mTransactionManager.InitParent(*this);
	mThreadsManager.InitParent(*this);
	mManagerScriptData.InitParent(*this);
}

iLoadSingleClient::~iLoadSingleClient() {
}



void iLoadSingleClient::fInit(
	iSingleClientMachineThread& rMThread,
  const char* pDisabledProtocol[])
{
	mMThread = &rMThread;

  CLEIniFileProperties::UseNewJSGCGuardStyle = false;


	char path[WL_MAX_PATH];
	strcpy(path,CArrangeMachineTempFiles::GetWlTempPath(fGetProcess().GetMasterPid()).data());

	CArrangeMachineTempFiles::LECreateDirectory(path);

  iSession::fInit(pDisabledProtocol);
	
}


cVirtualLoadEngine *picVirtualLoadEngine = NULL;


bool iLoadSingleClient::fExecuteProtoInit(
	const char* InitializationScript,
	iAgendaFile& File,
	const char* AgendaName,
	bool& exceptionOccured) {

	exceptionOccured = false;

	mBeginTime = fGetProcess().fGetCurrentTime();

	m_AgendaName = AgendaName;
	bool IsOK = false;

  try { LE_SET_SE_TRANSLATOR();
		setCurrentThreadsManager(&mThreadsManager);

		m_jsScript = new CWLJSScript(1);

		TVEnvContainerClass tvpc;
		const char* agendaFileName = tvpc.TVPGetPathName( File.fGetName() );

		m_jsScript->Init(fGetProcess().GetLoadRuntime(), mManagerScriptData, GetLEProtocolSessionContainer(), agendaFileName);

    picVirtualLoadEngine = new cVirtualLoadEngine();
    SetVirtualLoadEngine(picVirtualLoadEngine);
		iSpace::fSetRuntime(fGetProcess().GetLoadRuntime().fGet());
    
		iSpace::fInit();
		iSpace::fSignJSContext( m_jsScript->GetContext() );



		{
			cLEHardEmbedderCallContext EC( mManagerScriptData ); // scoped
			cSCMainFileCommit SourceCommit( File, *m_jsScript );
			File.fSourceExport( EC, SourceCommit );
			IsOK = SourceCommit.fGetStatus();
		}

		if( IsOK ) {

			mInitAgendaRun = true;
			cInterruptionControlCommit intrCtrl(*this);
			fBeforeInitAgenda( intrCtrl );
			m_jsScript->InitVersionNumber();
			mManagerScriptData.ProtocolBeforeInitAgenda();
			IsOK = m_jsScript->InitAgenda( InitializationScript );
		}
	}

	catch (std::exception* e){
		RVCString FullExcName ("");
		WLERRGetErrorMsg(e, FullExcName);
		RVCString StackTrace("");
		if (WLERRGetStackTrace(e, StackTrace) && StackTrace.length() > 0){
			WlErrorMsg("Exception %s caught in iLoadSingleClient::fExecuteProtoInit\nTrace:\n%s", FullExcName, StackTrace);
			printf("Exception %s caught in iLoadSingleClient::fExecuteProtoInit\nTrace:\n%s", FullExcName, StackTrace);
		}
		else {
			WlErrorMsg("Exception %s caught in iLoadSingleClient::fExecuteProtoInit", FullExcName.data());
			printf("Exception %s caught in iLoadSingleClient::fExecuteProtoInit", FullExcName.data());
		}
		exceptionOccured = true;
		delete e;	// Only for catch of pointer
	}
	
	catch (std::exception& e){
		RVCString FullExcName ("");
		WLERRGetErrorMsg(&e, FullExcName);
		WlTraceMsg("Exception %s caught in iLoadSingleClient::fExecuteProtoInit", FullExcName.data());
		exceptionOccured = true;
	}
	
	catch (...) {
		exceptionOccured = true;
	}


	return IsOK;
}


bool iLoadSingleClient::fExecuteThread(
	int ThreadNum,
	unsigned long RoundNum )
{

	bool IsOK = true;			// Used in thread execution loop
	bool retIsOK = true;	// Used as return value. Not that this is true UNLESS
												// an exception occured (Windows, C++ etc but not an
												// error or JavaScript exception during the agenda execution)
  mRoundNum = RoundNum;

  try { LE_SET_SE_TRANSLATOR();

		m_ThreadInstance = new CRTAMThreadInstance(*this,*m_jsScript, ThreadNum);
    m_ThreadInstanceScript = new CRTAMThreadInstanceScript(m_ThreadInstance, this);

    m_ThreadInstance->setCurrentScript(m_ThreadInstanceScript);
		m_ThreadInstance->Init();

    m_ThreadInstanceScript->Init();

    {
		  cLEJSEmbedContextControlMachineThread aGuard( *m_ThreadInstanceScript );
		  CJSGCRequestBeginEnd GCGuard( m_ThreadInstanceScript->m_MyJSThread.GetContext() );
	
		  iSpace::fSignJSContext( m_ThreadInstanceScript->m_MyJSThread.GetContext() );

      // @@Chaim (inherited from Itaj): to fix SpiderMonkey bug with cleaning cloned proto exception objects
	    JS_InitStandardClasses(m_ThreadInstanceScript->m_MyJSThread.GetContext(), m_ThreadInstanceScript->m_MyJSThread.GetObject() );

		  if( ! m_ThreadInstanceScript->m_MyJSThread.Evaluate(m_jsScript->GetEvaluationStringBeforeThreadActivation()) ) {
			  return false;
		  }
		  if( ! m_ThreadInstanceScript->m_MyJSThread.Evaluate(m_jsScript->GetEvaluationStringOnThreadActivation()) ) {
			  return false;
		  }

		  {
			  CJSGCRequestBeginEnd GCGuard(m_ThreadInstanceScript->m_MyJSThread.GetContext());

        m_ThreadInstanceScript->ProtocolOnThreadActivation();

		  }


		  cInterruptionControlCommit intrCtrl(*this);
		  fBeginClient( intrCtrl );
		  fBeforeInitClient(intrCtrl );
		  IsOK = m_ThreadInstanceScript->m_MyJSThread.InitClient(m_ThreadInstanceScript);
		  IsOK = IsOK && ( m_ThreadInstanceScript->m_MyThreadInstance->getErrorSeverity() != WLLogMessage::Error ) &&
			  ( m_ThreadInstanceScript->m_MyThreadInstance->getErrorSeverity() != WLLogMessage::Fatal );

		  // rounds loop
		  while( IsOK && (!mIsAbort) && fBeforeRound( RoundNum, intrCtrl ) ) {
			  m_ThreadInstance->m_ErrorSeverity = WLLogMessage::Success;
			  m_ThreadInstance->SetRoundNum( RoundNum );

			  {
				  jsval result;
				  m_ThreadInstanceScript->m_MyJSThread.CallFunctionName( "UNDOC_BeforeRound" ,0,0,result );
			  }
			  if( ! m_ThreadInstanceScript->m_MyJSThread.Evaluate(m_jsScript->GetEvaluationStringBeforeRound()) ) {
				  return false;
			  }

        if (!m_ThreadInstanceScript->ProtocolBeforeRound()){
          return false;
        }

			  fThreadResetTimer("Round Time");
			  bool RoundStatus = m_ThreadInstanceScript->m_MyJSThread.ExecScript(RoundNum);
			  if( RoundStatus ) {
				  fThreadSendTimer("Round Time");
				  RoundStatus = ( m_ThreadInstanceScript->m_MyThreadInstance->getErrorSeverity() < WLLogMessage::Error );
			  }
	      mTransactionManager.SendFailureForOpenTransactions( RoundStatus, true );
			  if( !RoundStatus ) {
				  cInterruptionControlCommit intrCtrl(*this);
				  fBeforeOnScriptAbort( intrCtrl );
				  m_ThreadInstanceScript->m_MyJSThread.OnScriptAbort();
					
					// Chaim - added mIsAbort to handle StopGenerator() correctly
					// (it doesn't change the severity)
				  if( mIsAbort || m_ThreadInstanceScript->m_MyThreadInstance->getErrorSeverity() >= WLLogMessage::Fatal ) {
            // Chaim - fix of bugs 16047 and 16050
					    IsOK = false;
					  }
				  }

        RoundStatus = RoundStatus && m_ThreadInstanceScript->m_MyJSThread.Evaluate(m_jsScript->GetEvaluationStringAfterRound());

        m_ThreadInstanceScript->ProtocolAfterRound();
				m_ThreadInstanceScript->fExitCriticalSection();

			  {
				  jsval result;
				  m_ThreadInstanceScript->m_MyJSThread.CallFunctionName( "UNDOC_AfterRound" ,0,0,result );
			  }

				// GC should be forced now that we have a shared runtime
			  m_jsScript->JSGC(false);

			  ++RoundNum;
        mRoundNum = RoundNum;
			  if( IsOK ) {
				  fThreadAddLast("Successful Rounds", 1, CWLStatPointTypeEnum::Counter);
			  } else {
				  fThreadAddLast("Failed Rounds", 1, CWLStatPointTypeEnum::Counter);
			  }
			  fThreadAddLast("Rounds", 1, CWLStatPointTypeEnum::Counter);

				// The CompletedRounds counter is updated in RTAM even though the script/function
				// cache is not used for RTAM (see comment in GetStatment in rvjsclient.cpp)
				IncrementCompletedRounds();

		  }
		  if(IsOK) {
			  cInterruptionControlCommit intrCtrl(*this);
			  fBeforeTerminateClient( intrCtrl );
        WlTraceMsg("Before m_MyJSThread.TerminateClient");
			  m_ThreadInstanceScript->m_MyJSThread.TerminateClient();
        WlTraceMsg("After m_MyJSThread.TerminateClient");
		  }	else {
      // bug 15822
			  cInterruptionControlCommit intrCtrl(*this);
			  fBeforeOnErrorTerminateClient( intrCtrl );
        WlTraceMsg("Before m_MyJSThread.OnErrorTerminateClient");
			  m_ThreadInstanceScript->m_MyJSThread.OnErrorTerminateClient();
        WlTraceMsg("After m_MyJSThread.OnErrorTerminateClient");
        WlTraceMsg("Before m_MyJSThread.TerminateClient");
			  m_ThreadInstanceScript->m_MyJSThread.TerminateClient();
        WlTraceMsg("After m_MyJSThread.TerminateClient");
		  }
      WlTraceMsg("Before running user define AfterTerminateClient");
		  m_ThreadInstanceScript->m_MyJSThread.Evaluate(m_jsScript->GetEvaluationStringAfterTerminateClient());
      WlTraceMsg("After running user define AfterTerminateClient");

      WlTraceMsg("Before running user define ProtocolAfterTerminateClient");
      m_ThreadInstanceScript->ProtocolAfterTerminateClient();
      WlTraceMsg("After running user define ProtocolAfterTerminateClient");

      WlTraceMsg("Before running user define fEndClient");
		  fEndClient( intrCtrl );
      WlTraceMsg("After running user define fEndClient");

		}
	}

	catch (std::exception* e){
		m_ThreadInstanceScript->ProtocolExceptionHandler(*e);
		RVCString FullExcName ("");
		WLERRGetErrorMsg(e, FullExcName);
		RVCString StackTrace("");
		if (WLERRGetStackTrace(e, StackTrace) && StackTrace.length() > 0){
			WlErrorMsg("Exception %s caught in iLoadSingleClient::fExecuteThread\nTrace:\n%s", FullExcName, StackTrace);
			printf("Exception %s caught in iLoadSingleClient::fExecuteThread\nTrace:\n%s", FullExcName, StackTrace);
		}
		else {
			WlErrorMsg("Exception %s caught in iLoadSingleClient::fExecuteThread", FullExcName.data());
			printf("Exception %s caught in iLoadSingleClient::fExecuteThread", FullExcName.data());
		}
		retIsOK = false;
		delete e;	// Only for catch of pointer
	}
	
	catch (std::exception& e){
		m_ThreadInstanceScript->ProtocolExceptionHandler(e);
		RVCString FullExcName ("");
		WLERRGetErrorMsg(&e, FullExcName);
		WlErrorMsg("Exception %s caught in iLoadSingleClient::fExecuteThread", FullExcName.data());
		printf("Exception %s caught in iLoadSingleClient::fExecuteThread", FullExcName.data());
		retIsOK = false;
	}
	
	catch (...) {
		WlErrorMsg("Unknown exception caught in iLoadSingleClient::fExecuteThread");
		printf("Unknown exception caught in iLoadSingleClient::fExecuteThread");
    WLErrorException e("...", WLLogMessage::Fatal);
		m_ThreadInstanceScript->ProtocolExceptionHandler(e);
		retIsOK = false;
	}


	try
	{
  WlTraceMsg("Before JSGC");

  // @@Chaim - crash I saw at Gal's
//	m_jsScript->JSGC(false);
  WlTraceMsg("After JSGC");

  JSContext* SaveContext = m_ThreadInstanceScript->m_MyJSThread.GetContext();
  WlTraceMsg("Before m_MyJSThread.Terminate");
	m_ThreadInstanceScript->m_MyJSThread.Terminate();
  WlTraceMsg("After m_MyJSThread.Terminate");
	delete m_ThreadInstanceScript;
	m_ThreadInstanceScript=0;
	delete m_ThreadInstance;
	m_ThreadInstance=0;
	}
	catch (...){
		WlTraceMsg("Error in dtors");
	}

	return retIsOK;
	}




void iLoadSingleClient::fExecuteProtoTerminate(
	bool IsOK ) {

	// m_jsScript->JSGC(false);

	setCurrentManagerScriptData( &mManagerScriptData, m_jsScript->GetContext() );

	if( mInitAgendaRun ) {

		if(IsOK) {
			cInterruptionControlCommit intrCtrl(*this);
			fBeforeTerminateAgenda( intrCtrl );
      WlTraceMsg("Before m_jsScript->TerminateAgenda");
			m_jsScript->TerminateAgenda(mManagerScriptData);
      WlTraceMsg("After m_jsScript->TerminateAgenda");
		}
		else {
      // bug 15822
			cInterruptionControlCommit intrCtrl(*this);
			fBeforeOnErrorTerminateAgenda( intrCtrl );
      WlTraceMsg("Before m_jsScript->OnErrorTerminateAgenda");
			m_jsScript->OnErrorTerminateAgenda(mManagerScriptData);
      WlTraceMsg("After m_jsScript->OnErrorTerminateAgenda");
      WlTraceMsg("Before m_jsScript->TerminateAgenda");
			m_jsScript->TerminateAgenda(mManagerScriptData);
      WlTraceMsg("After m_jsScript->TerminateAgenda");
		}

    // Chaim - fix of bugs 16047 and 16050
    WlTraceMsg("Before JSGC");
  	m_jsScript->JSGC(false);
    WlTraceMsg("After JSGC");
	}

  else {
		cInterruptionControlCommit intrCtrl(*this);
		fBeforeOnErrorTerminateAgenda( intrCtrl );
    WlTraceMsg("Before m_jsScript->OnErrorTerminateAgenda");
		m_jsScript->OnErrorTerminateAgenda(mManagerScriptData);
    WlTraceMsg("After m_jsScript->OnErrorTerminateAgenda");
    // Force GC to remove script objects spidermnokey may have created
    WlTraceMsg("Before JSGC");
  	m_jsScript->JSGC(false);
    WlTraceMsg("After JSGC");
  }


		//
		// get the log-messages & statistics.
		// m_ThreadInstance->DumpStatisticsToString(stat);
		// log = m_ThreadInstance->GetLogMessagesBuffer();
		// return m_ThreadInstanceScript->getErrorSeverity();

	setCurrentThreadsManager(0);

  if (picVirtualLoadEngine){
    WlTraceMsg("Before delete picVirtualLoadEngine");
    delete picVirtualLoadEngine;
    WlTraceMsg("After delete picVirtualLoadEngine");
  }

  WlTraceMsg("Before iSession::fFinal");
  iSession::fFinal();
  WlTraceMsg("After iSession::fFinal");
 
}


void iLoadSingleClient::fFinal() {

	char pathTemp[WL_MAX_PATH];
	strcpy(pathTemp,CArrangeMachineTempFiles::GetWlTempPath(fGetProcess().GetMasterPid()).data());
	RVCString path(pathTemp);

	TVPRemoveAllDirectories(path);

	// We need to force GC here just to be sure all the smart pointers to agenda files are
	// destroyed before the agenda files
  m_jsScript->JSGC(false);


  m_jsScript->Terminate();
  delete m_jsScript; m_jsScript=0;

	iSpace::fFinal(); 

}

bool iLoadSingleClient::fIsAbort() const {
	return mIsAbort;
	}

unsigned long iLoadSingleClient::fRoundNum() const {
	return mRoundNum;
	}

void iLoadSingleClient::vAbort_iDebugger() {
	iSpace::fAbort_iDebugger();
	mIsAbort = true;
	}




tDebugReporterReturn iLoadSingleClient::vDebugReporter_iUi( 
	iReportThreadState& rThreadState, 
	iReportInfo& rInfo )
{
	if( iLoadReportInfo* aInfo = dynamic_cast<iLoadReportInfo*>(&rInfo) ) {
		if( aInfo->fGetSeverity() >= WLLogMessage::Fatal ) {
			OnSevereError();
			cLoadReport aReport( rThreadState, *aInfo );
			fReportApply(aReport);
		}
	} else {
	}
	tDebugReporterReturn rtn = fDebugReporter( rThreadState, rInfo );
	return rtn;
}




void iLoadSingleClient::OnSevereError() {
	mIsAbort = true;
}

RVCString& iLoadSingleClient::GetEvaluationStringBeforeInitClient()
{
	return	m_jsScript->GetEvaluationStringBeforeInitClient();
}

RVCString& iLoadSingleClient::GetEvaluationStringAfterInitAgenda()
{
	return	m_jsScript->GetEvaluationStringAfterInitAgenda();
}
/*
RVCString&	iLoadSingleClient::GetEvaluationStringOnThreadTermination()
{
return	m_jsScript->GetEvaluationStringOnThreadTermination();
}
*/
RVCString& iLoadSingleClient::GetEvaluationStringBeforeThreadActivation()
{
	return	m_jsScript->GetEvaluationStringBeforeThreadActivation();
}

RVCString& iLoadSingleClient::GetEvaluationStringOnThreadActivation()
{
	return	m_jsScript->GetEvaluationStringOnThreadActivation();
}

RVCString&	iLoadSingleClient::GetEvaluationStringAfterTerminateClient()
{
	return	m_jsScript->GetEvaluationStringAfterTerminateClient();
}

RVCString&	iLoadSingleClient::GetEvaluationStringBeforeRound()
{
	return	m_jsScript->GetEvaluationStringBeforeRound();
}

RVCString&	iLoadSingleClient::GetEvaluationStringAfterRound()
{
	return	m_jsScript->GetEvaluationStringAfterRound();
}

RVCString&	iLoadSingleClient::GetEvaluationStringAfterTerminateAgenda()
{
	return	m_jsScript->GetEvaluationStringAfterTerminateAgenda();
}


IGeneralGlobalsRequestHandler* iLoadSingleClient::GetGlobalsRequestHandler() {
	return mManagerScriptData.GetGlobalsRequestHandler();
}


bool iLoadSingleClient::IsUserSection() {
	if( m_ThreadInstanceScript->m_MyJSThread ) {
		return m_ThreadInstanceScript->m_MyJSThread.IsUserSectionActive();
	} else {
		return m_jsScript->IsUserSectionActive();
	}
}

bool iLoadSingleClient::fBeforeSleep(
	long int Time )
{
	return true;
}

void iLoadSingleClient::fAfterSleep() {
}


cSingleClientProcess& iLoadSingleClient::fGetProcess()
{
	WlAssert( dynamic_cast<cSingleClientProcess*>(&iSession::fGetProcess()) );
	return static_cast<cSingleClientProcess&>(iSession::fGetProcess());
}

double iLoadSingleClient::vGetBeginTime()
{
	return mBeginTime;
}

void iLoadSingleClient::fReportApply(iReport& aReport){
	if (m_ThreadInstance){
		m_ThreadInstance->reportApply(aReport);
	}
	else {
		mManagerScriptData.reportApply(aReport);
	}
}


iLoadProbingClient::iLoadProbingClient(
	cSingleClientProcess& rProcess,
	bool IsDebuggerActive )
:
	iLoadSingleClient( rProcess, IsDebuggerActive )
{}

iLoadProbingClient::~iLoadProbingClient()
{}



bool iLoadProbingClient::fExecute(
	const char* InitializationScript,
	iAgendaFile& File,
	const char* AgendaName,
	int ThreadNum,
	unsigned long RoundNum) {

	bool exceptionOccured = false;

	bool InitIsOK = fExecuteProtoInit( InitializationScript, File, AgendaName , exceptionOccured);
	bool IsOK = true;
	if(InitIsOK && !exceptionOccured) {
		IsOK = fExecuteThread( ThreadNum, RoundNum );
	}
	fExecuteProtoTerminate( IsOK );
	return IsOK && !exceptionOccured;
}

tNormalDMHookReturn iLoadProbingClient::vInterruptDMHook_iUi(
	tInterruptionControl& InterruptionControl,
	bool& oIsHookPosition, 
	tStepping rStepping, 
	iDMThreadState& ThreadState)
{
	return vInterruptDMHook_iLoadProbingClient( InterruptionControl, oIsHookPosition, rStepping, ThreadState );
}

void iLoadProbingClient::fProtoAddLast(const char *name, double l, CWLStatPointTypeEnum::PointType type)
{
}

void iLoadProbingClient::fProtoResetTimer(const char *name)
{
}

void iLoadProbingClient::fProtoSendTimer(const char *name)
{
}

void iLoadProbingClient::fThreadAddLast(const char *name, double l, CWLStatPointTypeEnum::PointType type)
{
	mStatClient.fAddLast( name, l, type );
}

void iLoadProbingClient::fThreadResetTimer(const char *name)
{
	mStatClient.fResetTimer( name );
}

void iLoadProbingClient::fThreadSendTimer(const char *name)
{
	mStatClient.fSendTimer( name );
}


iLoadSAFHeaders::iLoadSAFHeaders(
	cSingleClientProcess& rProcess)
:
	iLoadSingleClient( rProcess, true )
{}

iLoadSAFHeaders::~iLoadSAFHeaders() {}


bool iLoadSAFHeaders::fExecute(
	const char* InitializationScript,
	iAgendaFile& File,
	const char* AgendaName)
{
	bool exceptionOccured;
	bool IsOK = fExecuteProtoInit( InitializationScript, File, AgendaName, exceptionOccured);
	fExecuteProtoTerminate( IsOK );
	return IsOK;
}


void iLoadSAFHeaders::fExecReporter( iReportThreadState*, iReportInfo& rInfo ) {
	fReporter( rInfo );
}

tDebugReporterReturn iLoadSAFHeaders::fDebugReporter(
	iReportThreadState& ThreadState,
	iReportInfo& ReportInfo )
{
	fReporter( ReportInfo );
	return sDebugReporterReturnApply;
}


void iLoadSAFHeaders::fScriptHook(iScriptHookable&, bool Creating ) {}



void iLoadSAFHeaders::fBeforeInitAgenda( iInterruptionControlCommit& ) {}
void iLoadSAFHeaders::fBeforeTerminateAgenda( iInterruptionControlCommit& ) {}
void iLoadSAFHeaders::fBeforeOnErrorTerminateAgenda( iInterruptionControlCommit& ) {}

void iLoadSAFHeaders::fBeginClient( iInterruptionControlCommit& ) {
	WlAssert(0);
}

void iLoadSAFHeaders::fBeforeInitClient( iInterruptionControlCommit& ) {
	WlAssert(0);
}

void iLoadSAFHeaders::fBeforeTerminateClient( iInterruptionControlCommit& ) {
	WlAssert(0);
}

void iLoadSAFHeaders::fBeforeOnErrorTerminateClient( iInterruptionControlCommit& ) {
	WlAssert(0);
}

bool iLoadSAFHeaders::fBeforeRound( unsigned long, iInterruptionControlCommit& ) {
	WlAssert(0);
	return false;
}


void iLoadSAFHeaders::fBeforeOnScriptAbort( iInterruptionControlCommit& ) {
	WlAssert(0);
}

void iLoadSAFHeaders::fEndClient( iInterruptionControlCommit& ) {
	WlAssert(0);
}


void iLoadSAFHeaders::fReportDMHook(tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& ) {
	WlAssert(0);
	}

tNormalDMHookReturn iLoadSAFHeaders::vInterruptDMHook_iUi(tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& ) {
	WlAssert(0);
	return sNormalDMHookReturnContinue;
	}

tNormalDMHookReturn iLoadSAFHeaders::fBreakPointDMHook(tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& ) {
	WlAssert(0);
	return sNormalDMHookReturnContinue;
	}

tNormalDMHookReturn iLoadSAFHeaders::fKeywordDMHook(tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& ) {
	return sNormalDMHookReturnContinue;
	}

tThrowDMHookReturn iLoadSAFHeaders::fThrowDMHook(tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& ) {
	return sThrowDMHookReturnApply;
	}


bool iLoadSAFHeaders::fInvokeDMHook( bool, tInvocationDMHookType, iDMThreadState* )
{
	return true;
}

bool cVirtualLoadEngine::vShouldReportInternalCSJSErrors(JSContext* Cx){
  return gShouldReportInternalCSJSErrors(Cx);
}







