/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "send2tml.h"
#include "wlerrorlib.h"
#include "tvpnumconversions.h"

#include "associativearray.h"
#include "loadjsshell.h"
#include "loadenginestatistics.h"
#include "loadengineglobals.h"
#include "loadgeneratorslave.h"
#include "loadthreadinstance.h"
#include "loadthreadsmanager.h"
#include "loadexceptions.h"

#include "loadreportinfo.h"
#include "rtamscript.h"
#include "loadengineexternaldecs.h"

#include "wlleprotocol.h"

#include <time.h>

#include "loadgeneratorscript.h"


////////////////////////////////////////////////////////////////
//        the implementation of class CRTAMThreadInstance
/////////////////////////////////////////////////////////////////


CRTAMThreadInstance::CRTAMThreadInstance(iLoadSingleClient& SingleClient,
                                         CWLJSScript& jsScript, 
                                         int ID ):
m_SingleClient(SingleClient),
m_jsScript(jsScript),
//  m_StatisticsDictionary(NULL),
m_ErrorSeverity(WLLogMessage::Success),
m_myId(ID),
m_jsj_thread(NULL)

{
  m_LogMutex = new RVCriticalSection();
  
}
CRTAMThreadInstance::~CRTAMThreadInstance() {
  delete m_LogMutex;
}


bool CRTAMThreadInstance::Init() {
	srand((unsigned)time( NULL ));
	rand();//in order to skip the first random number (because it happened 
	       // that all first numbers of each thread was close )

  return true;

}


bool CRTAMThreadInstance::internalIsAbort() {
	if (inCriticalSection())
		return false;

  return m_SingleClient.fIsAbort();
}


bool CRTAMThreadInstance::IsJSActive() {
  WlAssert(m_CurrentScript);
  return m_CurrentScript->m_MyJSThread.IsActive();
}

JSContext* CRTAMThreadInstance::vCurrentJSContext() {
  WlAssert(m_CurrentScript);
  return m_CurrentScript->m_MyJSThread.GetContext();
}

WLEThreadType CRTAMThreadInstance::GetThreadType() {
  return WLEThreadTypeBP;
}


void CRTAMThreadInstance::addLast(const char *name, double l, CWLStatPointTypeEnum::PointType type)
{
  m_SingleClient.fThreadAddLast(name, l, type);
}

void CRTAMThreadInstance::ToggleJavaDefaultStatistics( bool rValue )
{
  WlAssert(m_CurrentScript);
	m_CurrentScript->m_MyJSThread.ToggleJavaDefaultStatistics( rValue );
}

bool CRTAMThreadInstance::IsDisabledJavaDefaultStatistics()
{
  WlAssert(m_CurrentScript);
	return m_CurrentScript->m_MyJSThread.IsDisabledJavaDefaultStatistics();
}

iLEJSMachineThread& CRTAMThreadInstance::vGetMachineThread()
{
	return *m_SingleClient.mMThread; //@@Chaim (inherited from Itaj): rewrite m_SingleClient.fGetMachineThread() {assert...}
}

void CRTAMThreadInstance::resetTimer(const char *name)
{
  m_SingleClient.fThreadResetTimer(name);
}

void CRTAMThreadInstance::sendTimer(const char *name)
{
  m_SingleClient.fThreadSendTimer(name);
}


CLEVariablesData* CRTAMThreadInstance::GetCurrentLoadEngineVariablesData()
{
  return m_jsScript.GetLoadEngineVariablesData();
}

bool CRTAMThreadInstance::InScript() {
  WlAssert(m_CurrentScript);
  return m_CurrentScript->m_MyJSThread.IsInRound();
}


void CRTAMThreadInstance::OnSevereError() {
	m_SingleClient.OnSevereError();
}


void CRTAMThreadInstance::SetRoundNum( unsigned long RoundNum ) {
  m_RoundNum = RoundNum;
}


IGeneralThreadTransactionManager& CRTAMThreadInstance::GetTransactionManager() {
	return m_SingleClient.mTransactionManager;
}


void CRTAMThreadInstance::endMultiThread() {
  WlAssert(m_CurrentScript);
  RVCriticalSection::LockGuard lock(*m_LogMutex);
  m_CurrentScript->m_MyJSThread.SetJSContext();
}


void CRTAMThreadInstance::reportApply(
	iReport& rReport ) 
{
	
	WLLogMessage::Severity aSeverity = rReport.fGetSeverity();
  
  if (m_ErrorSeverity < aSeverity) {
    m_ErrorSeverity = aSeverity;
		}

  // @@@@Chaim (inherited from Itaj): m_CurrentScript->m_TransactionManager.RecordEvent(WLLogMessage::LongName(Severity), Message, TVPSysTime(), Severity, "Message", "" , 0, true, true);
   if (aSeverity == WLLogMessage::Fatal) {
	  OnSevereError();
	}
}

void CRTAMThreadInstance::reportLog(
	iReport& rReport ) 
{
	WLLogMessage::Severity aSeverity( rReport.fGetSeverity() );
	const char* aMessage = ( rReport.fGetMessage() );
//	bool aHasTime( rReport.fHasTime() );
//	double aTime( rReport.fGetTime() );

	m_SingleClient.fExecReporter( rReport.fGetThreadState(), rReport.fGetInfo() );
/* @@Chaim (inherited from Itaj): is this change ok?
	if( IsJSActive() ) { 
		m_SingleClient.fJSReporter( aSeverity, aMessage );
		}
	else {
		m_SingleClient.fReporter( aSeverity, aMessage );
		}
*/
	reportApplyEvent( aSeverity, aMessage );

	}



bool CRTAMThreadInstance::reportMultiThread(
	iReportInfo& r )
{
  WlAssert(m_CurrentScript);
  RVCriticalSection::LockGuard lock(*m_LogMutex);
  m_CurrentScript->m_MyJSThread.SetJSContext();
  return LoadReport( *m_CurrentScript, m_CurrentScript->fCurrentJSContext(), r );
};


IFileDictionary* CRTAMThreadInstance::GetFileDict() {
  return &m_SingleClient.mManagerScriptData.GetInputFiles();
		//mInputFiles;
}


const char* CRTAMThreadInstance::GetAgendaName() {
  return m_SingleClient.m_AgendaName;
}

int CRTAMThreadInstance::GetErrorCounter() 
{
  return m_SingleClient.GetErrorCounter();
}

int CRTAMThreadInstance::GetWarningCounter() 
{
  return m_SingleClient.GetWarningCounter();
}

void CRTAMThreadInstance::IncrementErrorCounter() 
{
  m_SingleClient.IncrementErrorCounter();
}

void CRTAMThreadInstance::IncrementWarningCounter() 
{
  m_SingleClient.IncrementWarningCounter();
}

//
iLEVCJSThread& CRTAMThreadInstance::vGetVCJSThread() {
  return *GetCurrentScript();
}

unsigned long CRTAMThreadInstance::GetRoundNumScriptInSlave() {
  return m_RoundNum;
}

unsigned long CRTAMThreadInstance::GetRoundNumMixInSlave() {
  return m_RoundNum;
}




IGeneralGlobalsRequestHandler* CRTAMThreadInstance::GetGlobalsRequestHandler() {
  return m_SingleClient.GetGlobalsRequestHandler();
	} 


/*
iLEProtocolVCInMix* CRTAMThreadInstance::GetiLEProtocolVCInMix(WLProtocolIndex p_ProtocolIndex) {
  WlAssert(m_CurrentScript);
  return (iLEProtocolVCInMix*)m_CurrentScript->GetProtocolVCInMix().GetProtocolAtIndex(p_ProtocolIndex);
}
*/

CWLJSScript& CRTAMThreadInstance::GetJSScript(){
	return m_jsScript;
}

CGeneralProcess& CRTAMThreadInstance::fGetProcess() {
	return m_SingleClient.fGetProcess();
}

iSession& CRTAMThreadInstance::vGetSession() {
	return m_SingleClient;
}


int CRTAMThreadInstance::GetNumScriptInMix() {
	return 1;
}

void CRTAMThreadInstance::SignNewJSContext( JSContext* rCx )
{
	m_SingleClient.fSignJSContext( rCx ); // debugger support
}

JSObject* CRTAMThreadInstance::GetGlobalObject()
{
  WlAssert(m_CurrentScript);
	return m_CurrentScript->m_MyJSThread.GetObject();
}

void CRTAMThreadInstance::CreateApplet(JSContext*           cx, 
                                       CLoadGeneralThreadInstanceScript*
                                                            TIS,
                                       const char*          appletClassName,
                                       const char*          appletCodeBase,
                                       const char*          appletDocumentBase,
                                       RVHashDictionary*    appletParams,
                                       CWLJSJavaApplet*&    javaApplet)
{
  WlAssert(m_CurrentScript);

	javaApplet = CWLAppletFactory::CreateJavaApplet(cx,
                                                  TIS,
                                                  javaApplet,
												                          appletClassName,
                                                  appletCodeBase,
												                          appletDocumentBase,
												                          appletParams);
                                              
}

bool CRTAMThreadInstance::ClassAppletExists(JSContext*  cx,
                                            const char* appletClassName,
                                            bool& classAppletExists)
{

	return CWLAppletFactory::ClassAppletExists(cx,
												                     appletClassName,
                                             classAppletExists);
}


bool CRTAMThreadInstance::AddJavaStream(JSContext*  cx,
                                        const char* streamName,
                                        const char* codeBase,
                                        RVCString&  javaStream)
{
  bool ret = JavaStreamExists(streamName);
  if ( !ret )
	  ret = m_JavaStreamTable.AddJavaStream(cx, 
                                          streamName,
                                          codeBase,
                                          javaStream.data(),
                                          javaStream.length());
  return ret;
}
bool CRTAMThreadInstance::JavaStreamExists(const char* streamName)
{
	return m_JavaStreamTable.JavaStreamExists(streamName);
}
/*
void CRTAMThreadInstance::fActionReportCommit( iActionReportInfo& rReportInfo )
{
	rReportInfo.fCommit( m_SingleClient );
}
*/

bool CRTAMThreadInstance::vIsUserSection()
{
	return m_CurrentScript->m_MyJSThread.IsUserSectionActive();
}
