/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvcsjscompilerintervention.h"
#include "rvjsclient.h"
#include "tracedll.h"
#include "loadengineinifileproperties.h"

#include "gt_unidef.h"


JSBool fsReferenceErrorCallback (JSContext *cx, const char *name){
  if (CLEIniFileProperties::StandardReferenceErrorBehavior){
    return JS_FALSE;
  }
  else {
    WlTraceMsg("Assuming %s is a function/variable that will be declared later", name);
    return JS_TRUE;
  }
}


JSBool fsCompileErrorCallback (JSContext *cx, uintN errorNumber, const char *linebuf,
                               const char *filename, uintN currentLine){

  iLEJSClientEmbedContext* aCSJSEmbedContext = dynamic_cast<iLEJSClientEmbedContext*>(iLEJSEmbedContext::fsGet(cx));
  if (aCSJSEmbedContext){

    if (errorNumber == JSMSG_PAREN_BEFORE_FORMAL ||
		errorNumber == JSMSG_UNTERM_CLASS){
      aCSJSEmbedContext->mCompileErrorInfo.errorNumber = errorNumber;
      aCSJSEmbedContext->mCompileErrorInfo.linebuf = linebuf;
      aCSJSEmbedContext->mCompileErrorInfo.filename = filename;
      aCSJSEmbedContext->mCompileErrorInfo.currentLine = currentLine;
    }
    else {
      aCSJSEmbedContext->mCompileErrorInfo.errorNumber = 0;
      aCSJSEmbedContext->mCompileErrorInfo.linebuf = "";
      aCSJSEmbedContext->mCompileErrorInfo.filename = "";
      aCSJSEmbedContext->mCompileErrorInfo.currentLine = 0;
    }
  }

  return JS_TRUE;
  
}

CompileErrorType fsGetCurrentCompileErrorType(JSContext *cx){

  iLEJSClientEmbedContext* aCSJSEmbedContext = dynamic_cast<iLEJSClientEmbedContext*>(iLEJSEmbedContext::fsGet(cx));
  if (aCSJSEmbedContext){

    if (aCSJSEmbedContext->mCompileErrorInfo.errorNumber == 0){
      return CompileErrorUnfixable;
    }

    // @@Chaim - there's probably a lot of cases we'll miss, so use reg exp eventually
	// also change -1 to RV_NPOS

    if (aCSJSEmbedContext->mCompileErrorInfo.errorNumber == JSMSG_PAREN_BEFORE_FORMAL){
      int x1 = aCSJSEmbedContext->mCompileErrorInfo.linebuf.index(".");
      int x2 = aCSJSEmbedContext->mCompileErrorInfo.linebuf.index("(");
      if (x1 != RV_NPOS && x2 != RV_NPOS && x1 < x2)
        return CompileErrorVBStyleFunctionDefine;
      else
        return CompileErrorUnfixable;
    }

	if (aCSJSEmbedContext->mCompileErrorInfo.errorNumber == JSMSG_UNTERM_CLASS){
    // @@Chaim - I'm assuming there is no other case where SpiderMonkey returns this code
    // and that even if there is, we'll catch it in the fsFixCompileError callback itself
		return CompileErrorInvalidRegExpreAllowedByIE;
    /*
		// We're looking for the following regular expression syntax that only IE supports:
		//       /[/]/
		int x1 = aCSJSEmbedContext->mCompileErrorInfo.linebuf.index("/");
		int x2 = aCSJSEmbedContext->mCompileErrorInfo.linebuf.index("[");
		int x3 = aCSJSEmbedContext->mCompileErrorInfo.linebuf.index("\\");
		int x4 = aCSJSEmbedContext->mCompileErrorInfo.linebuf.index("/", x2);
		int x5 = aCSJSEmbedContext->mCompileErrorInfo.linebuf.index("]");
		if (x1 == RV_NPOS || x2 == RV_NPOS || x1 > x2 || x4 == x3 + 1 || x5 == RV_NPOS || x4 > x5){
			return CompileErrorUnfixable;
		}
		else {
			return CompileErrorInvalidRegExpreAllowedByIE;
		}
    */
		
	}

    return CompileErrorUnfixable;
  }

  return CompileErrorUnfixable;
}

// @@Chaim - so far, we can fix the "function window.onload()" problem and only if the
// entire defition is written on the same line, and not, for example:
//      function
//        window.onload()
// we can also fix the regular expression syntax that only IE supports:
//      /[/]/
// and I don't think it can actually be coded in more than one line, so the caveat for
// the function definition issue doesn't apply
bool fsFixCompileError(JSContext *cx, CompileErrorType pCompileErrorType, RVCString& pSource){
  iLEJSClientEmbedContext* aCSJSEmbedContext = dynamic_cast<iLEJSClientEmbedContext*>(iLEJSEmbedContext::fsGet(cx));
  WlAssert(aCSJSEmbedContext);

  // linebuf contains CR at the end (not all the time, but even when it doesn't, stripping away
  // the last char still allows us to find the line.
  // @@Chaim - if in the future the intervention involves very short lines, then we might
  // have to review the above assumption.
  RVCString searchString = aCSJSEmbedContext->mCompileErrorInfo.linebuf;
  searchString.remove(searchString.length()-1);

  size_t aIndex = 0, x1 = 0, x2 = 0;
  
  switch (pCompileErrorType){
  case CompileErrorVBStyleFunctionDefine:

    // First, find the error, which is looks like: function window.onload()
    aIndex = pSource.index(searchString);
    WlAssert(aIndex != RV_NPOS);
    
    // Remove the first occurence of the string "function" at the location we just found
    pSource.remove(pSource.index("function", aIndex), 8);

    // Now, insert the string " = new Function" before the left parantheses
    pSource.insert(pSource.index("(", aIndex), " = function");
    
    break;
	
  case CompileErrorInvalidRegExpreAllowedByIE:
    // First, find the error, which is looks like: /[/]/
    aIndex = pSource.index(searchString);
    WlAssert(aIndex != RV_NPOS);

	// Now insert the escape character \ before the terminator character /
	// To make things simpler, we just look for any / character within the terminators
	// and add a backslash, if there isn't one already, since we already know there's
	// a problem

	x1 = pSource.index("/", aIndex);
	do {
		x2 = pSource.index("/", x1 + 1);
		if (pSource[x2-1] != '\\'){
			pSource.insert(x2, '\\');
			break;
		}
		else {
			x1 = x2 + 1;
		}
	} while (x2 != RV_NPOS);

	break;

  case CompileErrorUnfixable:
    WlAssert(0);
  default:
    WlAssert(0);
  }

  return true;

}

// @@Chaim - codes found in Travelers
#define RVJS_ISSPACEEXTRA(c) ((c) == 0xA0 || (c) == 0x09 || (c) == 0xC3 || (c) == 0xC2 || (c) == 0x82)

// Returns true if this is a white space.
// The extra white space check is relevant only for CSJS and for
// CP_DEFAULT or CP_UTF8 charsets.
JSBool fsExtraWhiteSpaceCallback (JSContext* cx, int32 c){
 
  iLEJSClientEmbedContext* aCSJSEmbedContext = dynamic_cast<iLEJSClientEmbedContext*>(iLEJSEmbedContext::fsGet(cx));

  if (aCSJSEmbedContext &&
//      (aCSJSEmbedContext->GetCharset() == CP_DEFAULT || aCSJSEmbedContext->GetCharset() == CP_UTF8) &&
      RVJS_ISSPACEEXTRA(c)){

    return JS_TRUE;
  }
  else {
    return JS_FALSE;
  }
}


JSBool fsReservedKeywordsCallback (JSContext* cx){
  if (CLEIniFileProperties::EnforceReservedKeywords){
    return JS_FALSE;
  }
  else {
    return JS_TRUE;
  }
}
