/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvjsapi.h"

#include "associativearray.h"
#include "loadengineglobals.h"
#include "loadgeneratorscript.h"
#include "loadexceptions.h"
#include "wlerrorlib.h"
#include "tablecomparereporter.h"
#include "loadreportinfo.h"
#include "generalprocess.h"
#include "loadthreadinstance.h"

#define WLJSNAME_TABLECOMPARE_REPORTER_CLASS          "TableCompareReporter"
#define WLJSNAME_TABLECOMPARE_REPORTER_NAME           "Name"
#define WLJSNAME_TABLECOMPARE_REPORTER_DESCRIPTION    "Description"
#define WLJSNAME_TABLECOMPARE_REPORTER_TABLE1         "Table1"
#define WLJSNAME_TABLECOMPARE_REPORTER_TABLE2         "Table2"
#define WLJSNAME_TABLECOMPARE_REPORTER_NUMLINES       "NumLines"
#define WLJSNAME_TABLECOMPARE_REPORTER_COORDINATE     "Coordinate"
#define WLJSNAME_TABLECOMPARE_REPORTER_RANGE1         "Range1"
#define WLJSNAME_TABLECOMPARE_REPORTER_RANGE2         "Range2"
#define WLJSNAME_TABLECOMPARE_REPORTER_DESCRIPTION    "Description"
#define WLJSNAME_TABLECOMPARE_REPORTER_ ""
#define WLJSNAME_TABLECOMPARE_REPORTER_ADD_MESSAGE    "AddMessage"
#define WLJSNAME_TABLECOMPARE_REPORTER_ADD_ERROR      "AddError"
#define WLJSNAME_TABLECOMPARE_REPORTER_ISDIRTY        "IsDirty"
#define WLJSNAME_TABLECOMPARE_REPORTER_CLEAR          "Clear"
#define WLJSNAME_TABLECOMPARE_REPORTER_REPORT         "Report"

static JSBool
TableCompareReporterCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  if ((argc != 1) && (argc != 2)) {
    WLErrorException e(LoadExceptionFrmFile, 221, WLJSNAME_TABLECOMPARE_REPORTER_CLASS, 1, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_TABLECOMPARE_REPORTER_CLASS, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if ((argc == 2))
    if (!JSVAL_IS_STRING(argv[1])) {
    WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_TABLECOMPARE_REPORTER_CLASS, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

  CAssociativeArray *array = new CAssociativeArray;
  array->AddCouple(WLJSNAME_TABLECOMPARE_REPORTER_NUMLINES, 0L);
  array->AddCouple(WLJSNAME_TABLECOMPARE_REPORTER_COORDINATE, 1L);

  JS_SetPrivate(cx, obj, array);

  JS_DefineProperty(cx,obj,WLJSNAME_TABLECOMPARE_REPORTER_NAME,argv[0],NULL,NULL,JSPROP_ENUMERATE);
  if (argc == 2)
    JS_DefineProperty(cx,obj,WLJSNAME_TABLECOMPARE_REPORTER_DESCRIPTION,argv[1],NULL,NULL,JSPROP_ENUMERATE);
  else
    JS_DefineProperty(cx,obj,WLJSNAME_TABLECOMPARE_REPORTER_DESCRIPTION,JSVAL_VOID,NULL,NULL,JSPROP_ENUMERATE);
  
  JS_DefineProperty(cx,obj,WLJSNAME_TABLECOMPARE_REPORTER_TABLE1,JSVAL_VOID,NULL,NULL,JSPROP_ENUMERATE);
  JS_DefineProperty(cx,obj,WLJSNAME_TABLECOMPARE_REPORTER_TABLE2,JSVAL_VOID,NULL,NULL,JSPROP_ENUMERATE);
  
  *rval = OBJECT_TO_JSVAL(obj);
	return JS_TRUE;
}

static JSBool
TableCompareReporterAddMessage(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_TABLECOMPARE_REPORTER_CLASS"."WLJSNAME_TABLECOMPARE_REPORTER_ADD_MESSAGE, 3);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}
	if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_TABLECOMPARE_REPORTER_CLASS"."WLJSNAME_TABLECOMPARE_REPORTER_ADD_MESSAGE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);
  long numLines = array->GetLongValueOfParameter(WLJSNAME_TABLECOMPARE_REPORTER_NUMLINES);
  numLines++;
  array->AddCouple(WLJSNAME_TABLECOMPARE_REPORTER_NUMLINES, numLines);
  array->AddArrayCouple(WLJSNAME_TABLECOMPARE_REPORTER_RANGE1, "", numLines);
  array->AddArrayCouple(WLJSNAME_TABLECOMPARE_REPORTER_RANGE2, "", numLines);
  array->AddArrayCouple(WLJSNAME_TABLECOMPARE_REPORTER_DESCRIPTION, JS_GetStringBytes(JSVAL_TO_STRING(argv[0])), numLines);
  
	return JS_TRUE;
}

static JSBool
TableCompareReporterAddError(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 3) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_TABLECOMPARE_REPORTER_CLASS"."WLJSNAME_TABLECOMPARE_REPORTER_ADD_ERROR, 3);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}
  for (int i=0;i<3;i++) {
	  if (!JSVAL_IS_STRING(argv[0])) {
      WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_TABLECOMPARE_REPORTER_CLASS"."WLJSNAME_TABLECOMPARE_REPORTER_ADD_ERROR, i+1);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	  }
  }

	CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);
  long numLines = array->GetLongValueOfParameter(WLJSNAME_TABLECOMPARE_REPORTER_NUMLINES);
  numLines++;
  array->AddCouple(WLJSNAME_TABLECOMPARE_REPORTER_NUMLINES, numLines);
  array->AddArrayCouple(WLJSNAME_TABLECOMPARE_REPORTER_RANGE1, JS_GetStringBytes(JSVAL_TO_STRING(argv[0])), numLines);
  array->AddArrayCouple(WLJSNAME_TABLECOMPARE_REPORTER_RANGE2, JS_GetStringBytes(JSVAL_TO_STRING(argv[1])), numLines);
  array->AddArrayCouple(WLJSNAME_TABLECOMPARE_REPORTER_DESCRIPTION, JS_GetStringBytes(JSVAL_TO_STRING(argv[2])), numLines);

  return JS_TRUE;
}
 
static JSBool
TableCompareReporterClear(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_TABLECOMPARE_REPORTER_CLASS"."WLJSNAME_TABLECOMPARE_REPORTER_CLEAR, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);
  array->clearAndDestroy();
  array->AddCouple(WLJSNAME_TABLECOMPARE_REPORTER_NUMLINES, 0L);
  array->AddCouple(WLJSNAME_TABLECOMPARE_REPORTER_COORDINATE, 1L);
  jsval v=JSVAL_VOID;
  JS_SetProperty(cx,obj,WLJSNAME_TABLECOMPARE_REPORTER_TABLE1,&v);
  v=JSVAL_VOID;
  JS_SetProperty(cx,obj,WLJSNAME_TABLECOMPARE_REPORTER_TABLE2,&v);

  return JS_TRUE;
}

static JSBool
TableCompareReporterReport(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_TABLECOMPARE_REPORTER_CLASS"."WLJSNAME_TABLECOMPARE_REPORTER_REPORT, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);
  jsval nameProperty, descriptionProperty, table1Property, table2Property;
  const char *name="Undefined", *description="";

  if (JS_GetProperty(cx, obj, WLJSNAME_TABLECOMPARE_REPORTER_NAME, &nameProperty)==JS_TRUE)
    name = JS_GetStringBytes(JSVAL_TO_STRING(nameProperty));
  
  if (JS_GetProperty(cx, obj, WLJSNAME_TABLECOMPARE_REPORTER_DESCRIPTION, &descriptionProperty)==JS_TRUE)
    if (JSVAL_IS_STRING(descriptionProperty))
      description = JS_GetStringBytes(JSVAL_TO_STRING(descriptionProperty));
  
  if (JS_GetProperty(cx, obj, WLJSNAME_TABLECOMPARE_REPORTER_TABLE1, &table1Property)==JS_TRUE)
    if (JSVAL_IS_STRING(table1Property))
      array->AddCouple(WLJSNAME_TABLECOMPARE_REPORTER_TABLE1, JS_GetStringBytes(JSVAL_TO_STRING(table1Property)));
  if (JS_GetProperty(cx, obj, WLJSNAME_TABLECOMPARE_REPORTER_TABLE2, &table2Property)==JS_TRUE)
    if (JSVAL_IS_STRING(table2Property))
      array->AddCouple(WLJSNAME_TABLECOMPARE_REPORTER_TABLE2, JS_GetStringBytes(JSVAL_TO_STRING(table2Property)));
  
  const char* data = array->ToString();

	CLoadGeneralThreadInstanceScript* tis = getCurrentThreadInstanceScript(cx);
  if (tis) {
    CLoadGeneralThreadInstance* ti = tis->m_MyThreadInstance;
    ti->VerificationRecordEvent(name, 
                               description, 
															 ti->fGetProcess().fGetCurrentTime(),
                               WLLogMessage::Note, 
                               "TableCompare", 
                               (void*)data, 
                               strlen(data),
															 true ); // @@Chaim (inherited from Itaj): or take parameter?
  }

  TableCompareReporterClear(cx, obj, 0, NULL, NULL);

  return JS_TRUE;
}

static void
TableCompareReporterFinalize(JSContext *cx, JSObject *obj)
{
	CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);
  delete array;
}

JSClass TableCompareReporterClass = {
  WLJSNAME_TABLECOMPARE_REPORTER_CLASS,
  JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,	// add Property
  JS_PropertyStub,	// del Property
  JS_PropertyStub,	// get Property
  JS_PropertyStub,	// set Property
  JS_EnumerateStub,	
	JS_ResolveStub,
	JS_ConvertStub,
	TableCompareReporterFinalize
};

static JSFunctionSpec TableCompareReporter_methods[] = {
	{WLJSNAME_TABLECOMPARE_REPORTER_ADD_MESSAGE,	TableCompareReporterAddMessage,	1},
	{WLJSNAME_TABLECOMPARE_REPORTER_ADD_ERROR,	  TableCompareReporterAddError,	  3},
	{WLJSNAME_TABLECOMPARE_REPORTER_CLEAR,		    TableCompareReporterClear,	    0},
	{WLJSNAME_TABLECOMPARE_REPORTER_REPORT,		    TableCompareReporterReport,	    0},
	{0}
};


bool AddTableCompareReporterClass(JSContext *cx, JSObject *obj)
{
  if (!JS_InitClass(cx, obj, NULL, &TableCompareReporterClass, TableCompareReporterCtor, 1, NULL, TableCompareReporter_methods, NULL, NULL))
		return false;
  
  return true;
}

