/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//@@Merged
#include "wljsexception.h"

#include "tracedll.h"
#include "loadjsdefs.h"
#include "loadexceptions.h"

#include "loadreportinfo.h"


/////////////////////////////////////////////////////////////////////////
//%%%%%%%%%%%%%%%%%%%%%%% CExceptionError Class %%%%%%%%%%%%%%%%%%%%%%%%%
/////////////////////////////////////////////////////////////////////////
CExceptionError::CExceptionError(iReportInfo &Info)
{
	WlAssert( dynamic_cast<iLoadReportInfo*>(&Info) );
  iLoadReportInfo& LoadInfo = static_cast<iLoadReportInfo&>(Info);
  mSeverity = LoadInfo.fGetSeverity();
  mMessage = LoadInfo.fGetMessage();
  mTime = LoadInfo.fGetTime();
  mHasTime = LoadInfo.fHasTime();
}

CExceptionError::CExceptionError(WLLogMessage::Severity	severity, const char* msg, bool HasTime, double time)
:
  mSeverity( severity ),
  mMessage( msg ),
  mTime( time ),
  mHasTime(HasTime)
{}
/////////////////////////////////////////////////////////////////////////
//%%%%%%%%%%%%%%%%%%%%%%% CJSExceptionPrivate Class %%%%%%%%%%%%%%%%%%%%%%%%%%%%
/////////////////////////////////////////////////////////////////////////

CJSExceptionPrivate::CJSExceptionPrivate(
	WLLogMessage::Severity severity, 
	const char* msg, 
	bool HasTime, 
	double time )
{
#ifdef _DEBUG
  switch ( severity ) {
  case WLLogMessage::Note:
  case WLLogMessage::Warning:
  case WLLogMessage::Error:
  case WLLogMessage::Fatal:
    break;
  case WLLogMessage::Success: // why the hell?
  default:
    WlAssert(0);
    break;
  }
#endif // _DEBUG
  if (severity > WLLogMessage::Warning)
  {
    m_Severity = severity;
    m_Message = msg;
    m_HasTime = HasTime;
    m_Time = time;
    m_SevereIndex = m_ExceptionList.entries();
  }
  CExceptionError *e;
  if (HasTime) {
	  cLoadReportInfoAgenda loadRepInf(severity, msg, time);
	  e = new CExceptionError( loadRepInf );
  }
  else {
    cLoadReportInfoAgenda loadRepInf(severity, msg);
    e = new CExceptionError( loadRepInf );
  }

  m_ExceptionList.insert(e);

}

CJSExceptionPrivate::~CJSExceptionPrivate()
{
  m_ExceptionList.clearAndDestroy();
}

void CJSExceptionPrivate::SetException(iReportInfo &Info)
{
  WlAssert( dynamic_cast<iLoadReportInfo*>(&Info) );
  iLoadReportInfo& LoadInfo = static_cast<iLoadReportInfo&>(Info);
#ifdef _DEBUG
  switch ( LoadInfo.fGetSeverity()) {
  case WLLogMessage::Note:
  case WLLogMessage::Warning:
  case WLLogMessage::Error:
  case WLLogMessage::Fatal:
    break;
  case WLLogMessage::Success: // why the hell?
  default:
    WlAssert(0);
    break;
  }
#endif // _DEBUG
  
	if (LoadInfo.fGetSeverity() > WLLogMessage::Warning && LoadInfo.fGetSeverity() > m_Severity)
  {
    m_Severity = LoadInfo.fGetSeverity();
    m_Message = LoadInfo.fGetMessage();
    m_HasTime = LoadInfo.fHasTime();
    m_Time = LoadInfo.fGetTime();
    m_SevereIndex = m_ExceptionList.entries();
  }
    
  CExceptionError *e = new CExceptionError(LoadInfo);
  m_ExceptionList.insert(e);
  
}

//////////////////////////////////////////////////////////////////////////////////////
//%%%%%%%%%%%%%%%%%%%%%%% JSExceptionObjectClass object %%%%%%%%%%%%%%%%%%%%%%%%%%%%
///////////////////////////////////////////////////////////////////////////////////////

static void JSExceptionObjectFinalize(JSContext *cx, JSObject *obj)
{
	CJSExceptionPrivate* pJSException = ( CJSExceptionPrivate* )JS_GetPrivate( cx, obj );

	if ( pJSException != NULL )
		delete pJSException;
}

JSBool JSExceptionObject_GetSeverity(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval* rval)
{
	CJSExceptionPrivate* pJSException = ( CJSExceptionPrivate* )JS_GetPrivate( cx, obj );

	*rval = INT_TO_JSVAL(pJSException->GetSeverity());
		
    return JS_TRUE;
}

JSBool JSExceptionObject_GetMessageByIndex(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval* rval)
{
  if(argc != 1) {
		WLErrorException e(LoadExceptionFrmFile, 220, "Exception Object", 2);
		cLoadReportInfoAgenda loadRepInf(e.GetErrorSeverity(), e.GetErrorMsg());
		return LoadReportNoExn( cx, loadRepInf );
	}

  if (!JSVAL_IS_INT(argv[0])) {
		WLErrorException e(LoadExceptionFrmFile, 223, "Exception Object", 1);
		cLoadReportInfoAgenda loadRepInf(e.GetErrorSeverity(), e.GetErrorMsg());
		return LoadReportNoExn( cx, loadRepInf );
	}
	
	unsigned int indx = JSVAL_TO_INT(argv[0]);
	CJSExceptionPrivate* pJSException = ( CJSExceptionPrivate* )JS_GetPrivate( cx, obj );
  if (indx >= pJSException->GetExceptionList().entries()){
		cLoadReportInfoAgenda loadRepInf(WLLogMessage::Warning, "Exception Object - the index you entered is not in scope");
		return LoadReportNoExn( cx, loadRepInf );
	}

  CExceptionError *e = (CExceptionError*)pJSException->GetExceptionList().at(indx);
 	RVCString strMsg = e->GetMessage();
	JSString* str = JS_NewStringCopyN(cx, strMsg.data(), strMsg.length());
	WlAssert(str);
	*rval = STRING_TO_JSVAL(str);

  return JS_TRUE;
}

JSBool JSExceptionObject_ReportLog(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval* rval)
{
	CJSExceptionPrivate* pJSException = ( CJSExceptionPrivate* )JS_GetPrivate( cx, obj );

	WLLogMessage::Severity aSeverity;

	if( argc >= 1 ) {
		
	  int32 aX0;
		if( !JS_ValueToInt32( cx, argv[0], &aX0 ) ) {
			WLErrorException e( LoadExceptionFrmFile, 223, WLJSNAME_EXCEPTION_CLASS".""ReportLog", 1 );
			return LoadReport( cx, e );
		}

		aSeverity = (WLLogMessage::Severity)aX0;
	} else {
		aSeverity = pJSException->GetSeverity();
	}

	cLoadReportInfoClient loadRepInf( aSeverity, pJSException->GetMessage(), false, true, true, pJSException->HasTime(), pJSException->GetTime(), &pJSException->GetReportThreadState() );
	return LoadReportNoExn( cx, loadRepInf );
}


JSBool JSExceptionObject_GetSeverityByIndex(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval* rval)
{
  if(argc != 1) {
		WLErrorException e(LoadExceptionFrmFile, 220, "Exception Object", 2);
		cLoadReportInfoAgenda loadRepInf(e.GetErrorSeverity(), e.GetErrorMsg());
		return LoadReportNoExn( cx, loadRepInf );
	}

  if (!JSVAL_IS_INT(argv[0])) {
		WLErrorException e(LoadExceptionFrmFile, 223, "Exception Object", 1);
		cLoadReportInfoAgenda loadRepInf(e.GetErrorSeverity(), e.GetErrorMsg());
		return LoadReportNoExn( cx, loadRepInf );
	}

  unsigned int indx = JSVAL_TO_INT(argv[0]);
	CJSExceptionPrivate* pJSException = ( CJSExceptionPrivate* )JS_GetPrivate( cx, obj );
  if (indx >= pJSException->GetExceptionList().entries()){
		cLoadReportInfoAgenda loadRepInf(WLLogMessage::Warning, "Exception Object - the index you entered is not in scope");
		return LoadReportNoExn( cx, loadRepInf );
	}

  CExceptionError *e = (CExceptionError*)pJSException->GetExceptionList().at(indx);
  *rval = INT_TO_JSVAL(e->GetSeverity());
  	
  return JS_TRUE;
}

/* @@
JSBool JSExceptionObject_GetFile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval* rval)
{
	CJSExceptionPrivate* pJSException = ( CJSExceptionPrivate* )JS_GetPrivate( cx, obj );
	
	RVCString strFileName = pJSException->GetFileName();
	JSString* str = JS_NewStringCopyN(cx, strFileName.data(), strFileName.length());
	WlAssert(str);
	*rval = STRING_TO_JSVAL(str);

    return JS_TRUE;
}*/

/*JSBool JSExceptionObject_GetLine(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval* rval)
{
	CJSExceptionPrivate* pJSException = ( CJSExceptionPrivate* )JS_GetPrivate( cx, obj );

	*rval = INT_TO_JSVAL(pJSException->GetLineNum());

	return JS_TRUE;
}
*/

JSBool JSExceptionObject_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval* rval)
{
	CJSExceptionPrivate* pJSException = ( CJSExceptionPrivate* )JS_GetPrivate( cx, obj );
	
	RVCString strMessage = pJSException->GetMessage();
	JSString* str = JS_NewStringCopyN(cx, strMessage.data(), strMessage.length());
	WlAssert(str);
	*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}

JSBool JSExceptionObject_GetMessage(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval* rval)
{
	CJSExceptionPrivate* pJSException = ( CJSExceptionPrivate* )JS_GetPrivate( cx, obj );
	
	RVCString strMessage = pJSException->GetMessage();
	JSString* str = JS_NewStringCopyN(cx, strMessage.data(), strMessage.length());
	WlAssert(str);
	*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}

JSClass JSExceptionObjectClass = 
{
    WLJSNAME_EXCEPTION_CLASS, JSCLASS_HAS_PRIVATE,
    JS_PropertyStub,  JS_PropertyStub,JS_PropertyStub, JS_PropertyStub,
    JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub,  JSExceptionObjectFinalize
};

static JSFunctionSpec JSExceptionObjectClass_methods[] = 
{
	{ WLJSNAME_EXCEPTION_GET_SEVERITY,	JSExceptionObject_GetSeverity,	0 },
	//{ "GetFile",						JSExceptionObject_GetFile,		0 },
	//{ "GetLine",						JSExceptionObject_GetLine,		0 },
  { "GetSeverityByIndex",						JSExceptionObject_GetSeverityByIndex,		1 },
  { "GetMessageByIndex",						JSExceptionObject_GetMessageByIndex,		1 },
  { "ReportLog",						JSExceptionObject_ReportLog,		1 },
	{ WLJSNAME_EXCEPTION_TO_STRING,		JSExceptionObject_toString,		0 },
	{ WLJSNAME_EXCEPTION_GET_MESSAGE,	JSExceptionObject_toString,		0 },
	{ 0 }
};


static JSBool JSExceptionObjectCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if(argc != 2) {
		WLErrorException e(LoadExceptionFrmFile, 220, "Exception Object", 2);
		cLoadReportInfoAgenda loadRepInf(e.GetErrorSeverity(), e.GetErrorMsg());
		return LoadReportNoExn( cx, loadRepInf );
	}

	if (!JSVAL_IS_INT(argv[0])) {
		WLErrorException e(LoadExceptionFrmFile, 223, "Exception Object", 1);
		cLoadReportInfoAgenda loadRepInf(e.GetErrorSeverity(), e.GetErrorMsg());
		return LoadReportNoExn( cx, loadRepInf );
	}

	if (!JSVAL_IS_STRING(argv[1])) {
		WLErrorException e(LoadExceptionFrmFile, 222, "Exception Object", 2);
		cLoadReportInfoAgenda loadRepInf(e.GetErrorSeverity(), e.GetErrorMsg());
		return LoadReportNoExn( cx, loadRepInf );
    }


	WLLogMessage::Severity severity = (WLLogMessage::Severity)JSVAL_TO_INT(argv[0]);
		
	if ( severity != WLLogMessage::Error
			&& severity != WLLogMessage::Fatal) {

		cLoadReportInfoAgenda loadRepInf(WLLogMessage::Fatal, "Illegal severity parameter");
		return LoadReportNoExn( cx, loadRepInf );
	}
	
		
	// Initialize the pJSException . 
	CJSExceptionPrivate* pJSException = new CJSExceptionPrivate(severity, JS_GetStringBytes(JSVAL_TO_STRING(argv[1])), false, -1 ); //@@Chaim (inherited from Itaj): filename and line-number
			
	if(!JS_SetPrivate( cx, obj, pJSException ))
		return JS_FALSE;

	return JS_TRUE;
}


// -------------------- Functions exposed to the outside -------------------- 


// Creates new js exeption object with the parameters.....
JSObject* NewJSExceptionObject(JSContext *cx, WLLogMessage::Severity severity, const char* message, bool HasTime, double time )
{
	
	// create an instance of type JSExceptionObject
	JSObject* element = JS_NewObject(cx, &JSExceptionObjectClass, NULL, NULL);

	if (!element)
		return NULL;

	// Initialize the pJSException with inputs. 
	CJSExceptionPrivate* pJSException = new CJSExceptionPrivate(severity, message, HasTime, time);
	cReportInfoPosition aPosition( static_cast<coReportThreadState&>( pJSException->GetReportThreadState() ) );
	LoadReportNoExn( cx, aPosition );
			
	// do i need to define the metods or ..
	if(!JS_SetPrivate( cx, element, pJSException ))
		return NULL;

	return element;
}

// Performs registrations for the js engine.
bool AddJSExceptionClass(JSContext *cx, JSObject *obj)
{
  if (!JS_InitClass(cx, obj, NULL, &JSExceptionObjectClass, JSExceptionObjectCtor, 1, NULL, JSExceptionObjectClass_methods, NULL, NULL))
		return false;
  
  return true;
}

JSClass* GetJsExceptionClass()
{
	return &JSExceptionObjectClass;
}

