/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlleprotocolcontainer.h"
#include "wlleprotocol.h"


iLEProtocolContainer::iLEProtocolContainer():
  m_InitPerformed(false)
{
}

iLEProtocolContainer::~iLEProtocolContainer(){

  WlAssert(m_InitPerformed);

  // Chaim - we don't use clearAndDestroy so we can control the order of the delete's.
  for (WLProtocolIndex i = m_NumProtocols - 1; i >= 0; i--){
    delete m_Container[i]; 
  }
}

void iLEProtocolContainer::InitProtocolContainer(WLProtocolIndex NumProtocols){

  for (WLProtocolIndex i = 0; i < NumProtocols; i++)
    m_Container.append(NULL);
  
  m_NumProtocols = NumProtocols;
  m_InitPerformed = true;
}

iLEProtocolBase& iLEProtocolContainer::GetProtocolAtIndex(WLProtocolIndex p_ProtocolIndex){
  WlAssert(m_InitPerformed);
  WlAssert(p_ProtocolIndex < m_NumProtocols);
  WlAssert(m_Container[p_ProtocolIndex]);;
  WlAssert(dynamic_cast<iLEProtocolBase*>(m_Container[p_ProtocolIndex]));
  return *static_cast<iLEProtocolBase*>(m_Container[p_ProtocolIndex]);
}

void iLEProtocolContainer::InsertProtocolAtIndex(WLProtocolIndex p_ProtocolIndex, iLEProtocolBase& pProtocol){
  WlAssert(m_InitPerformed);
  WlAssert(p_ProtocolIndex < m_NumProtocols);
  WlAssert(!m_Container[p_ProtocolIndex]);;
  m_Container[p_ProtocolIndex] = &pProtocol;
}

WLProtocolIndex iLEProtocolContainer::NumProtocols(){
  WlAssert(m_InitPerformed);
  return m_NumProtocols;
}

bool iLEProtocolContainer::ProtocolExists(WLProtocolIndex p_ProtocolIndex){
  WlAssert(m_InitPerformed);
  WlAssert(p_ProtocolIndex < m_NumProtocols);
  return (m_Container[p_ProtocolIndex] != NULL);
}
