/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <time.h>
#include <thr.h>
#include <thr.h>

#include "rvjsapi.h"

#define NUM_THREADS 10

JSRuntime *rt;
JSContext *super_context;
JSObject  *super_object;
JSScript  *script;
JSObject  **so;
RVCriticalSection coutMutex;

int roundsCounter = 0;
time_t startTime;
RVCriticalSection roundsCounterMutex;

RVThreadFunction GCThread;

void IncNumRounds() 
{
  RVCriticalSection::LockGuard lock(roundsCounterMutex);
  if (roundsCounter==0)
    startTime=time(NULL);
  roundsCounter++;
  if (roundsCounter%10000 == 0)
  {
    RVCriticalSection::LockGuard lock(coutMutex);
    cout << "Total rounds: " << roundsCounter << " after " << time(NULL)-startTime << " Seconds."<< endl;
  }
}

int threadsRunning = 0;
bool terminate  = false;

static JSClass stub_class = {
	"stub_class", 0,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	JS_FinalizeStub
};

JSBool proto_resolve_stub(JSContext *cx, JSObject *obj, jsval id);

static JSClass proto_stub_class = {
	"proto_stub_class", JSCLASS_NEW_RESOLVE,
	JS_PropertyStub,	// add
	JS_PropertyStub,	// del
	JS_PropertyStub,	// get
	JS_PropertyStub,	// set
	JS_EnumerateStub,
  proto_resolve_stub,
	JS_ConvertStub,
	JS_FinalizeStub
};

static JSBool
my_f(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {

  //*rval = JSVAL_ONE;

	jsval testVal;
	//JSBool ret = JS_GetProperty(cx, obj, "TestVal", &testVal);
  JSBool ret = JS_LookupProperty(cx, obj, "TestVal", &testVal);
  
  return JS_TRUE;	
}

void
JSGCFunc() {
  {
    RVCriticalSection::LockGuard lock(coutMutex);
    cout << "*** Starting GC Thread" << endl;
  }
	JSContext* cx = JS_NewContext(rt,8192);
  int round=0;
  while (!terminate) {
		JS_GC(cx);
    round++;
    if (round%1000 == 0)
    {
      RVCriticalSection::LockGuard lock(coutMutex);
      cout << "*** GC Thread, round " << round << endl;
    }
		//::rvSleep(100);
  }
  JS_DestroyContext(cx);
}

#define WLJSNAME_ROUND_NUM "RoundNum"
#define WLJSNAME_THREAD_NUM "ThreadNum"
#define WLJSNAME_CLIENT_NUM "ClientNum"

void RunScript(int threadNum, JSContext *cx) {
	JSObject  *obj;

	jsval result;

  {
    RVCriticalSection::LockGuard lock(coutMutex);
    threadsRunning++;
    if (threadsRunning == NUM_THREADS)
      cout << "All threads started" << endl;
  }

	if (!cx) return;

  JS_SetContextThread(cx);

  JS_BeginRequest(cx);

  //obj = JS_NewObject(cx, &proto_stub_class, super_object, NULL);
  obj = JS_NewObject(cx, &stub_class, NULL, NULL);
	if (!obj) return;

  JS_SetGlobalObject(cx,obj);

  //if (!JS_InitStandardClasses(cx,obj)) return;

  JS_DefineFunction(cx ,obj,"my_f",my_f,0,0);
  //
  
  // compile the script
  JSScript *script = JS_CompileFile(cx,obj,"jstry.js");
	if (!script) return;

  // create script object and AddRoot for the GC
  so = (JSObject**)malloc(sizeof(JSObject *));
	*so = JS_NewScriptObject(cx,script);
	if (!*so) return;
  if (!JS_AddRoot(cx,so)) return;
  //*/
  JS_EndRequest(cx);

  while (threadsRunning<NUM_THREADS)
    ::rvSleep(1000);

  int round=0;
  while(!terminate) {
    round++;
		JS_BeginRequest(cx);
		JS_ExecuteScript(cx,obj,script,&result);
    JS_EndRequest(cx);
    IncNumRounds();
		//JS_GC(cx);
    //if (round%1000 == 0)
    //{
    //  RVCriticalSection::LockGuard lock(coutMutex);
    //  cout << "Thread " << threadNum << " round " << round << endl;
    //}
	}
  GCThread.join();
  JS_DestroyContext(cx);
  {
    RVCriticalSection::LockGuard lock(coutMutex);
    threadsRunning--;
    if (threadsRunning == 0)
      cout << "Done Thread " << threadNum << endl;
  }
}

int main(int argc, char *argv[])
{
  // create runtime
  rt = JS_NewRuntime(8L*1024L*1024L);
	if (!rt) return 1;

  // create super cx
  super_context = JS_NewContext(rt,8192);
	if (!super_context) return 1;

  // create super obj
  super_object = JS_NewObject(super_context, &stub_class, NULL, NULL);
	if (!super_object) return 1;

  // add js functions to the super js
  if (!JS_InitStandardClasses(super_context,super_object)) return 1;

  //JS_DefineFunction(super_context ,super_object,"my_f",my_f,1,0);

  // compile the script
  /*script = JS_CompileFile(super_context,super_object,"jstry.js");
	if (!script) return 1;

  // create script object and AddRoot for the GC
  so = (JSObject**)malloc(sizeof(JSObject *));
	*so = JS_NewScriptObject(super_context,script);
	if (!*so) return 1;
  if (!JS_AddRoot(super_context,so)) return 1;
  */

  GCThread = rvMakeThreadFunction(JSGCFunc);
  GCThread.start();
  ::rvSleep(500);
  
  for (int i=1; i<=NUM_THREADS; i++) {
  	JS_BeginRequest(super_context);
  	JSContext *cx = JS_NewContext(rt,8192);
		JS_EndRequest(super_context);
    RVThreadFunction T = rvMakeThreadFunction(RunScript,i,cx);
		T.start();
	}

  char buf[80];
  cin.getline(buf,sizeof(buf));
  terminate = true;
  //GCThread.join();
  while (threadsRunning>0)
    ::rvSleep(100);
  cout << "Done ...";
  
  return 0;
}

