/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#include "nmt.h"



	namespace
nMT {


iSynch::iSynch() :
	mHandle(0) {}

iSynch::~iSynch() {
	CloseHandle( mHandle );
	}

HANDLE iSynch::fGetHandle() const {
	return mHandle;
	}

void iSynch::fTake() {
	DWORD X0 = WaitForSingleObject( mHandle, INFINITE );
	switch(X0) {
	case WAIT_FAILED:
		throw "error";
		break;
	default:
		break;
		}
	}

bool iSynch::fTryTake(
	int Timeout ) {
	bool Rtn;
	switch( WaitForSingleObject( mHandle, Timeout ) ) {
	case WAIT_TIMEOUT:
		Rtn = false;
		break;
	case WAIT_OBJECT_0:
		Rtn = true;
		break;
	default:
		throw "error";
		break;
		}
	return(Rtn);
	}



iThread::iThread( 
	int Priority ) {
	fCtor( Priority );
	}


iThread::~iThread() {
	fTerminate();
	}

DWORD WINAPI iThread::fsStartRoutineWrap( LPVOID Par ) {
	return( (reinterpret_cast< iThread* >(Par))->fMain() );
	}

void iThread::fCtor(
	int Priority ) {

	fTerminate();
	//must be suspended so assignment takes place before executing.
	mHandle = CreateThread( 0, 0,
		fsStartRoutineWrap, (LPVOID)(this), CREATE_SUSPENDED, 0 );
	if( !mHandle ) {
		throw( "can't create thread" );
		}
	if( !SetThreadPriority( mHandle, Priority ) ) {
		throw( "can't set priority" );
		}
	}


void iThread::fBegin() {

	fTerminate();
	if( -1 == ResumeThread( mHandle ) ) {
		throw( "cant begin" );
		}
	}

void iThread::fTerminate() {

	DWORD Status;
	if( mHandle )
	if( !fGetExitCode( Status ) )
	if( Status == STILL_ACTIVE ) {
		TerminateThread( mHandle, 0 );
		}

	}


BOOL iThread::fGetExitCode(
	DWORD& Code ) 
	const {

	return GetExitCodeThread( mHandle, &Code );
	}



int iThread::fPriorityGet() 
	const {

	return GetThreadPriority( mHandle );
	}


void iThread::fRelease() {
	}





cMultipleSynch::cMultipleSynch(
	int SynchTn,
	iSynch* SynchAry[] ) :
	mSynchTn(0),
	mSynchAry(0) {
	fCtor( SynchTn, SynchAry );
	}

void cMultipleSynch::fCtor(
	int SynchTn,
	iSynch* SynchAry[] ) {

	mSynchTn = SynchTn;
	mSynchAry = new iSynch*[ SynchTn ];
	for( int SynchI = 0; SynchI < SynchTn; ++SynchI ) {
		mSynchAry[ SynchI ] = SynchAry[ SynchI ];
		}

	}

cMultipleSynch::~cMultipleSynch() {
	delete[] mSynchAry;
	}


void cMultipleSynch::fTake() {
	HANDLE* HandleAry = new HANDLE[ mSynchTn ];
	for( int SynchI = 0; SynchI < mSynchTn; ++SynchI ) {
		HandleAry[ SynchI ] = mSynchAry[ SynchI ]->fGetHandle();
		}
	switch( WaitForMultipleObjects( mSynchTn, HandleAry, TRUE, INFINITE ) ) {
	case WAIT_FAILED:
		throw "error";
		break;
	default:
		break;
		}
	delete[] HandleAry;
	}


void cMultipleSynch::fRelease() {
	for( int SynchI = 0; SynchI < mSynchTn; ++SynchI ) {
		mSynchAry[ SynchI ]->fRelease();
		}
	}


cEvent::cEvent(
	bool LockNow ) {
	fCtor( LockNow );
	}

void cEvent::fCtor(
	bool LockNow ) {
	mHandle = CreateEvent( 0, TRUE, LockNow ? FALSE : TRUE, 0 );
	}


void cEvent::fRelease() {
	SetEvent( mHandle );
	}

void cEvent::fLock() {
	ResetEvent( mHandle );
	}

void cEvent::fPulse() {
	PulseEvent( mHandle );
	}



cMutex::cMutex(
	bool LockNow ) {
	fCtor( LockNow );
	}

void cMutex::fCtor(
	bool LockNow ) {
	mHandle = CreateMutex( 0, LockNow ? TRUE : FALSE, 0 );
	}


void cMutex::fRelease() {
	ReleaseMutex( mHandle );
	}



cSemaphore::cSemaphore(
	int Size,
	int LockedTn ) {
	fCtor( Size, Size-LockedTn );
	}

cSemaphore::cSemaphore(
	bool LockNow,
	int Size ) {
	fCtor( Size, Size-(LockNow?1:0) );
	}

void cSemaphore::fCtor(
	int Size,
	int FreeTn ) {
	mHandle = CreateSemaphore( NULL, FreeTn, Size, NULL );
	}


void cSemaphore::fRelease() {
	ReleaseSemaphore( mHandle, 1, 0 );
	}




cList::cList() :
	mLocker(),
	mEmptyHang(true) {
	mEmptyHang.fLock();
	}

cList::~cList() {
	}

void cList::fPushTake() {
	mLocker.fTake();
	}

void cList::fPullTake() {
	iSynch* SynchAry[2];// = { &mLocker, &mEmptyHang };
	SynchAry[0] = &mLocker;
	SynchAry[1] = &mEmptyHang;

	cMultipleSynch MultSynch( 2, SynchAry );
	MultSynch.fTake();
	}

void cList::fRelease() {
	mLocker.fRelease();
	}

void cList::fReleaseEmpty() {
	fSetEmpty();
	fRelease();
	}

void cList::fReleaseNotEmpty() {
	fSetNotEmpty();
	fRelease();
	}

void cList::fSetEmpty() {
	mEmptyHang.fLock();
	}

void cList::fSetNotEmpty() {
	mEmptyHang.fRelease();
	}






cSStm::cSStm():
	mData(),
	mSynch() {
	}

cSStm::~cSStm() {}

	void 
cSStm::fPush( 
	cElement& Element ) 
{
	mSynch.fPushTake();
	mData.push_front( Element );
	mSynch.fReleaseNotEmpty();
	}

	void 
cSStm::fPull( 
	cElement& Element ) 
{
	mSynch.fPullTake();
	Element = mData.back();
	mData.pop_back();
	if( mData.empty() ) {
		mSynch.fReleaseEmpty();
	}	else {
		mSynch.fRelease();
		}
	}



cSStmManage::cSStmManage() :
	mIdMap() {
	}

cSStmManage::~cSStmManage() {}

	void 
cSStmManage::fAdd( 
	cSStm& SStm,
	const cId& Id ) {
	mIdMap[ Id ] = &SStm;
	}

	cSStm* 
cSStmManage::fGet(
	const cId& Id ) {
	cSStm* Rtn = 0;
	cIdMap::iterator I;
	I = mIdMap.find( Id );
	if( I != mIdMap.end() ) {
		Rtn = I->second;
		}
	return Rtn;
	}



iThreadInput::iThreadInput(
	int Priority ) 
: 
	iThread(Priority),
	mId() 
{}

void iThreadInput::fInput( 
	cSStm::cElement& Element ) 
{
	mInput.fPush( Element );
	}

void iThreadInput::fInit(
	const cId& Id )
{
	mId = Id;
	}

iThreadInput::cId& iThreadInput::fGetId() 
{
	return mId;
	}


cSStm& iThreadInput::fGetSStm() 
{
	return mInput;
	}
	



iThreadTerminateRequest::iThreadTerminateRequest():
	mEventTerminate( true ) {}

void iThreadTerminateRequest::fEnd() {
	mEventTerminate.fRelease();
	}

bool iThreadTerminateRequest::fContinue() {
	return !mEventTerminate.fTryTake();
	}


} // nMT




