/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef dNMT
#define dNMT

#pragma warning( disable: 4786 )

#include <string>
#include <list>
#include <map>

//#include <windows.h>

#include "wlerrorlib.h"

	namespace
nMT {

	class DllExport iSynch;
	class DllExport iThread;
	class DllExport cMultipleSynch;
	class DllExport cEvent;
	class DllExport cMutex;
	class DllExport cSemaphore;

	class DllExport cList;
	class DllExport cSStm;
	class DllExport cSStmManage;


	class DllExport iThreadInput;
	class DllExport iThreadTerminateRequest;



	class
iSynch {
	
	friend class cMultipleSynch;
		
public:
	
	explicit iSynch();
	virtual ~iSynch();

	virtual void fTake();
	virtual bool fTryTake( int = 0 );
	virtual void fRelease() = 0;

protected:
	HANDLE mHandle;

	virtual HANDLE fGetHandle() const;
	};



	class
iThread :
	virtual public iSynch {

public:

	explicit iThread( int = THREAD_PRIORITY_NORMAL);
	virtual ~iThread();

	virtual DWORD fMain() = 0;
	virtual BOOL fGetExitCode( DWORD& ) const;
	virtual int fPriorityGet() const;

	virtual void fBegin();
	virtual void fTerminate();

	virtual void fRelease();

private:
	static DWORD WINAPI fsStartRoutineWrap( LPVOID );
	void fCtor( int );

	};


	class
cMultipleSynch {

public:

	explicit cMultipleSynch( int, iSynch*[] );
	virtual ~cMultipleSynch();

	virtual void fTake();
	virtual void fRelease();

protected:
	iSynch** mSynchAry;
	int mSynchTn;

private:
	void fCtor( int, iSynch*[] );

	};



	class
cEvent:
	virtual public iSynch {

public:

	explicit cEvent( bool = true );

	virtual void fRelease();
	virtual void fLock();
	virtual void fPulse();

private:
	void fCtor( bool );
	};



	class
cMutex:
	virtual public iSynch {

public:

	explicit cMutex( bool = false );

	virtual void fRelease();

private:
	void fCtor( bool );
	};


	class
cSemaphore:
	virtual public iSynch {

public:

	explicit cSemaphore( int, int = 0 );
	explicit cSemaphore( bool = false, int = 1 );

	virtual void fRelease();

private:
	void fCtor( int, int );
	};



	class
cList{

public:

	explicit cList();
	virtual ~cList();

	virtual void fPushTake();
	virtual void fPullTake();
	virtual void fRelease();
	virtual void fReleaseNotEmpty();
	virtual void fReleaseEmpty();

protected:
	cMutex mLocker;
	cEvent mEmptyHang;
	//bool mIsEmpty;

private:
	void fCtor();
	void fSetEmpty();
	void fSetNotEmpty();
	};


class cSStmManage;
class cSStm;
class iThreadInput;


	class
cSStm {

public:
		
	typedef std::string cElement;
	typedef std::list<cElement> cSStmData;

	explicit cSStm();
	virtual ~cSStm();

	virtual void fPush( cElement& );
	virtual void fPull( cElement& );

protected:
	cSStmData mData;
	nMT::cList mSynch;
	};



	class
cSStmManage {

public:

	typedef std::string cId;
	typedef std::map< cId, cSStm* > cIdMap;

	explicit cSStmManage();
	virtual ~cSStmManage();
	virtual void fAdd( cSStm&, const cId& );
	virtual cSStm* fGet( const cId& );

private:
	cIdMap mIdMap;
	};



	class
iThreadInput: 
	virtual public iThread {

public:

	typedef cSStmManage::cId cId;
	
	explicit iThreadInput( int = THREAD_PRIORITY_NORMAL);
	void fInput( cSStm::cElement& );
	virtual void fInit( const cId& );
	cId& fGetId();

	virtual cSStm& fGetSStm();

protected:
	cSStm mInput;
	cId mId;
	};


	class
iThreadTerminateRequest:
	virtual public iThread {

public:

	explicit iThreadTerminateRequest();
	virtual void fEnd();

protected:
	virtual bool fContinue();

private:
	cEvent mEventTerminate;
	};

} // nMT

#endif //dNMT

