/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef dPCUi
#define dPCUi

#pragma warning( disable: 4786 )

#define doEventLog 0
#define doInput 1
#define doScriptHook 0
#define doBreakpoints 1
#define doFlowControl 1
#define dInvokeHook 0
#define dIsDebugger 1
#define dDebuggerInfoPath "z:\\fX0\\fX0"


#include <list>

#include "rvjsapi.h"
// #include "rvjs.h"
#include "nmt.h"

//#include "platform.h"
#include "tvtables.h"
#include "tracedll.h"
//#include "wlerrorlib.h"
//#include "loadengineglobals.h"
#include "rtamscript.h"
//#include "LoadReportInfo.h"

#include <stdlib.h>
#include <fstream>
#include <process.h>

namespace nPCUi {

using namespace nJs;


class iPCUi;

class cScriptFile: public iAgendaFile {

public:

	explicit cScriptFile(const char*, iPCUi&);
	virtual ~cScriptFile();

	const char* fGetName() const;
	
	void fSourceExport( iEmbedderCallContext&, iSourceCommit& );

	void vPick()
	{}

	bool vDrop()
	{
/*
		delete this;
		return false;
*/
		return true;
  }
	
private:
	std::string mName;
	iPCUi& mUi;
public:
#if doBreakpoints
	std::list< tLineNumber > mBreakpointList;
#endif
};





class iGeneralUi {

protected:
	static std::string fsReportFormat(	WLLogMessage::Severity, const char* );
	};


class iPCUi: public iLoadProbingClient, RV_INHERIT_PROTECTED iGeneralUi 
{

public:

	explicit iPCUi(cSingleClientProcess&);

	virtual ~iPCUi();

	void fSetLastRound( long int );

	iAgendaFile* fGetFile( bool&, iEmbedderCallContext&, const char*);

	void fCopyFile( iEmbedderCallContext&, const char*, const char* );


	void fScriptHook( iScriptHookable&, bool Creating );

	
	void fJSReporter( iReportThreadState&, iReportInfo&, const char* rReporterName );
	
	void fExecReporter( iReportThreadState*, iReportInfo& );
	tDebugReporterReturn fDebugReporter( iReportThreadState& ,iReportInfo& );

	tSCActionReporterReturn fStubEmbedderReporter(iFile&,tLineNumber,tColumn,const char*,iSCActionReportInfo&);
		
	void fReportDMHook( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& );
	tNormalDMHookReturn vInterruptDMHook_iLoadProbingClient( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& );
	tNormalDMHookReturn fBreakPointDMHook( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& );
	tNormalDMHookReturn fKeywordDMHook( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& );
	tThrowDMHookReturn fThrowDMHook( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& );

	bool fDMHookX3( const char* rMsg, tInvocationDMHookType, iDMThreadState* );
	bool fInvokeDMHook( bool rIsFunction, tInvocationDMHookType, iDMThreadState* );

	void fSetDebugger(iDebugger&,nMT::cSStm&);

	virtual void fRecordEvent(long eventID,
			  										long topInstanceID,
			  										long instanceID,
														WLLogMessage::Severity severity,
														const char* eventName,
														double time,
														long lineNumber,
														const char* fileName,
														const char* description, 
														const char* extraDataType,
														const char* extraData);

	virtual void fBeginTransaction(long instanceID, 
																 const char* transactionName, 
																 const char* transactionType);

	virtual void fEndTransaction(long topInstanceID, 
															 long instanceID, 
															 long parentID, 
															 unsigned long roundNum,
															 const char* reason,
															 const char* transactionName,
															 double time, 
															 WLLogMessage::Severity severity,
															 const char* transactionType, 
															 const char* extraData);
protected:

	tNormalDMHookReturn fDMHookX2(tInterruptionControl&, bool& oIsHookPosition, tStepping, bool rIsStep, const char*, iDMThreadState* );

	bool fDMHookX1(tInterruptionControl&, bool& oIsHookPosition, tStepping, bool rIsStep, const char*, iDMThreadState* );
	
	bool fDMHookX0(const char*, iDMThreadState* ,tInterruptionControl&, bool& oIsHookPosition, tStepping, bool rIsStep );

	void fBeforeInitAgenda( iInterruptionControlCommit& );
	void fBeforeTerminateAgenda( iInterruptionControlCommit& );
	void fBeforeOnErrorTerminateAgenda( iInterruptionControlCommit& );
	void fBeginClient( iInterruptionControlCommit& );
	void fBeforeInitClient( iInterruptionControlCommit& );
	void fBeforeTerminateClient( iInterruptionControlCommit& );
	void fBeforeOnErrorTerminateClient( iInterruptionControlCommit& );
	bool fBeforeRound( unsigned long, iInterruptionControlCommit& );
	void fBeforeOnScriptAbort( iInterruptionControlCommit& );
	void fEndClient( iInterruptionControlCommit& );

  void fProtoAddLast(const char *name, double l, CWLStatPointTypeEnum::PointType type) {}
	void fProtoResetTimer(const char *name) {}
	void fProtoSendTimer(const char *name) {}
  void fThreadAddLast(const char *name, double l, CWLStatPointTypeEnum::PointType type) {}
	void fThreadResetTimer(const char *name) {}
	void fThreadSendTimer(const char *name) {}

public:
	typedef nMT::cSStm::cElement cInputElement;
protected:
	virtual void vInputPull( cInputElement& ) = 0;
public:
	void fInputPull( cInputElement& _X0 ) { vInputPull( _X0 ); }	

private:
	tStepping mStepping;
	std::list< std::auto_ptr<cScriptFile> > mFiles;
	long int mLastRound;

};


class cPCUiMT: public iPCUi
{
public:
	explicit cPCUiMT( cSingleClientProcess&, nMT::cSStm& );

private:
	void vInputPull( cInputElement& );

	nMT::cSStm& mSStm;
};



class cPCUi: public iPCUi
{
public:
	explicit cPCUi( cSingleClientProcess& );

private:
	void vInputPull( cInputElement& );
};



class TestThreadInstanceScript:
	public CLoadGeneralThreadInstanceScript  {
public:

	TestThreadInstanceScript(CLoadGeneralThreadInstance*);

	~TestThreadInstanceScript();

	static CLoadGeneralThreadInstanceScript* fsNew(CLoadGeneralThreadInstance*);
		
	
private:
	int mIn;
	static int msCount;

	};

bool WLInitTestGeneratorThread(JSContext*, JSObject*, CLEVariablesData*);

bool WLInitTestGeneratorScript(JSContext* cx, JSObject* global, JSObject* GeneralGlobal );



class cThreadPC: public nMT::iThreadInput {

public:

	explicit cThreadPC(iPCUi&, const char*);

	DWORD fMain();

protected:
	iPCUi& mPC;

private:
	const char* mFileName;


	};



class cPCUiThread:
	public nMT::iThreadInput 
{

public:

	explicit cPCUiThread(nMT::cSStmManage&, const char*, cSingleClientProcess& );

	DWORD fMain();

private:
	nMT::cSStmManage* mSStmManage;
	const char* mFileName;
	cSingleClientProcess& mProcess;
};



	} // nPCUi

#endif // dPCUi

