/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "charset.h"
#include "tracedll.h"


#define INSERT_CHAR(sym,c) insertKeyAndValue(new RVCollectableString((sym)), \
	                                         new RVCollectableString((char)(c),1))

CWLCharSet::CWLCharSet()
{
        WlAssert( !msSingleton );
        msSingleton = this;

	spRE = new RVCRegexp("&#?[a-zA-Z0-9]+;");
	hexnum[0] = '0';  // hex prefix
	hexnum[1] = 'x'; 

	INSERT_CHAR("nbsp",32);
	INSERT_CHAR("iexcl",161);
	INSERT_CHAR("cent",162);
	INSERT_CHAR("pound",163);
	INSERT_CHAR("curren",164);
	INSERT_CHAR("yen",165);
	INSERT_CHAR("brvbar",166);
	INSERT_CHAR("sect",167);
	INSERT_CHAR("uml",168);
	INSERT_CHAR("copy",169);
	INSERT_CHAR("ordf",170);
	INSERT_CHAR("laquo",171);
	INSERT_CHAR("not",172);
	INSERT_CHAR("shy",173);
	INSERT_CHAR("reg",174);
	INSERT_CHAR("macr",175);
	INSERT_CHAR("deg",176);
	INSERT_CHAR("plusmn",177);
	INSERT_CHAR("sup2",178);
	INSERT_CHAR("sup3",179);
	INSERT_CHAR("acute",180);
	INSERT_CHAR("micro",181);
	INSERT_CHAR("para",182);
	INSERT_CHAR("middot",183);
	INSERT_CHAR("cedil",184);
	INSERT_CHAR("sup1",185);
	INSERT_CHAR("ordm",186);
	INSERT_CHAR("raquo",187);
	INSERT_CHAR("frac14",188);
	INSERT_CHAR("frac12",189);
	INSERT_CHAR("frac34",190);
	INSERT_CHAR("iquest",191);
	INSERT_CHAR("Agrave",192);
	INSERT_CHAR("Aacute",193);
	INSERT_CHAR("Acirc",194);
	INSERT_CHAR("Atilde",195);
	INSERT_CHAR("Auml",196);
	INSERT_CHAR("Aring",197);
	INSERT_CHAR("AElig",198);
	INSERT_CHAR("Ccedil",199);
	INSERT_CHAR("Egrave",200);
	INSERT_CHAR("Eacute",201);
	INSERT_CHAR("Ecirc",202);
	INSERT_CHAR("Euml",203);
	INSERT_CHAR("Igrave",204);
	INSERT_CHAR("Iacute",205);
	INSERT_CHAR("Icirc",206);
	INSERT_CHAR("Iuml",207);
	INSERT_CHAR("ETH",208);
	INSERT_CHAR("Ntilde",209);
	INSERT_CHAR("Ograve",210);
	INSERT_CHAR("Oacute",211);
	INSERT_CHAR("Ocirc",212);
	INSERT_CHAR("Otilde",213);
	INSERT_CHAR("Ouml",214);
	INSERT_CHAR("times",215);
	INSERT_CHAR("Oslash",216);
	INSERT_CHAR("Ugrave",217);
	INSERT_CHAR("Uacute",218);
	INSERT_CHAR("Ucirc",219);
	INSERT_CHAR("Uuml",220);
	INSERT_CHAR("Yacute",221);
	INSERT_CHAR("THORN",222);
	INSERT_CHAR("szlig",223);
	INSERT_CHAR("agrave",224);
	INSERT_CHAR("aacute",225);
	INSERT_CHAR("acirc",226);
	INSERT_CHAR("atilde",227);
	INSERT_CHAR("auml",228);
	INSERT_CHAR("aring",229);
	INSERT_CHAR("aelig",230);
	INSERT_CHAR("ccedil",231);
	INSERT_CHAR("egrave",232);
	INSERT_CHAR("eacute",233);
	INSERT_CHAR("ecirc",234);
	INSERT_CHAR("euml",235);
	INSERT_CHAR("igrave",236);
	INSERT_CHAR("iacute",237);
	INSERT_CHAR("icirc",238);
	INSERT_CHAR("iuml",239);
	INSERT_CHAR("eth",240);
	INSERT_CHAR("ntilde",241);
	INSERT_CHAR("ograve",242);
	INSERT_CHAR("oacute",243);
	INSERT_CHAR("ocirc",244);
	INSERT_CHAR("otilde",245);
	INSERT_CHAR("ouml",246);
	INSERT_CHAR("divide",247);
	INSERT_CHAR("oslash",248);
	INSERT_CHAR("ugrave",249);
	INSERT_CHAR("uacute",250);
	INSERT_CHAR("ucirc",251);
	INSERT_CHAR("uuml",252);
	INSERT_CHAR("yacute",253);
	INSERT_CHAR("thorn",254);
	INSERT_CHAR("yuml",255);
	INSERT_CHAR("quot",34);
	INSERT_CHAR("amp",38);
	INSERT_CHAR("lt",60);
	INSERT_CHAR("gt",62);
}

void CWLCharSet::convert(RVCString &str)
{
	size_t location = 0, len;
	RVCollectableString *c;
	char  ch;
	int   num;

	if(str.isNull())
		return;
	
	while( (location = str.index(*spRE,&len,location)) != RV_NPOS && len > 2 )
	{
		if( str[location+1] == '#' )  //! anumeric reference looks like:
			                          //! &#Hnn;
			if( str[location+2] == 'x' || str[location+2] == 'X' ) //! In Hexa format
			{
				if( (len - 4) <= 2 ) //! length of hexa num must be in range 00-FF
				{
					strncpy(hexnum+2,(str.data()+location+3),len - 4); //! copy the hex digits
					hexnum[2+len-4] = '\0'; //! at the end of hex
					sscanf(hexnum,"%i",&num);
					ch = (char)num;
					str.replace(location,len,&ch,1);
				}
			}
			else  //! in decimal
			{
				sscanf(str.data()+location+2,"%i",&num); //! read the number
				if( num < 256 ) //! with in ASCII
				{
					ch = (char)num;
					str.replace(location,len,&ch,1);
				}
			}
		else //! symbolic reference
		{
			RVCollectableString s(str(location+1,len-2)); //! take the symbolic code
			if( (c = (RVCollectableString *)findValue(&s)) != NULL )
				str.replace(location,len,c->data()); //! and replace it with the appropriate char
		}

		++location; //! skip over the char
	}
}



CWLCharSet* CWLCharSet::msSingleton = NULL;
static CWLCharSet singletonCWLCharSet;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWLCharSet& CWLCharSet::fsGetSingleton()
{
        WlAssert( msSingleton );
        return *msSingleton;
}

