/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : rvHTMLParseUtils.cpp
 *
 *
 *	Description : General functions extracting strings
 *	              htmlGetStringValue extracts a string, if it finds it's end
 *								& returns a ptr to it's end.
 *
 *	History :
 *
 *	Modifications :
 *		Who				When				What
 *		---				----				----
\******************************************************************/

#include <ctype.h>
#include <RVCString.h>
#include <regexp.h>
#include "rvhtmlparseutils.h"
#include "rvhtmlattributefinder.h"
#include "tracedll.h"

bool htmlIsWhiteSpace(char c)
{
  switch(c) {
	case ' ':
	case '\t':
	case '\n':
	case '\r':
		return 1;
  }
  return 0;
}

bool htmlSkipWhiteSpaces(RVCString& sBuff, size_t buffLength, size_t& beginIndex)
{
	while(++beginIndex < buffLength)
	{
		/*char	c = buff(beginIndex);
		  if(c != ' ' && c != '\n' && c != '\r' && c != '\t')
			break;*/
		switch(sBuff(beginIndex)) {
		  case ' ':
		  case '\n':
		  case '\r':
		  case '\t': continue;
		  default: return true;
		}
	}
	return false;
}

bool htmlSkipNonWhiteSpaces(RVCString& sBuff, size_t buffLength, size_t& beginIndex)
{
	while(++beginIndex < buffLength)
	{
		switch(sBuff(beginIndex)) {
		  case ' ':
		  case '\n':
		  case '\r':
		  case '\t': 
      case '>': 
      case '<': 
        return true;
		  default: continue;
		}
	}
	return false;
}

strVal htmlGetStringValue(RVCString& sBuff, RVCString& str, size_t beginIndex, size_t& startAttributsPtr, size_t& endAttributsPtr, size_t buffLength)  //get a desired string value(URL, NAME= , VALUE=, ....)
{
  //size_t beginIndex = buff.index("=", strPtr);
  //size_t buffLength = buff.length();

	if(!htmlSkipWhiteSpaces(sBuff, buffLength, beginIndex))
		return HTML_STRING_NOT_COMPLETE;
	str.remove(0);
	char quote = sBuff(beginIndex);
	char currentChar;
	startAttributsPtr = beginIndex;

  if(quote =='"' || quote == '\'')
  {
    ++startAttributsPtr;
    while(++beginIndex < buffLength)
    {
      currentChar = sBuff(beginIndex);
      if(currentChar == quote)
      {
        if(beginIndex >	endAttributsPtr)
          endAttributsPtr = beginIndex;
        return HTML_STRING_COMPLETE;
      }
      
      //if( !htmlIsWhiteSpace(currentChar) )
      str.append(currentChar, 1);
    }
  }
  else
  {
    do{
      currentChar = sBuff(beginIndex);
      if(htmlIsWhiteSpace(currentChar) || currentChar == '>')
      {
        if(beginIndex >	endAttributsPtr)
          endAttributsPtr = beginIndex;
        return HTML_STRING_COMPLETE;
      }
      str.append(currentChar, 1);
    }
    while(++beginIndex < buffLength);
  }
  return 	HTML_STRING_NOT_COMPLETE;
}



/*! Compare elementStr with buff starting from BuffPos.
*/
bool isElementOfType(const char* elementStr, size_t elementLength, RVCString& pBuff, size_t buffPos, char nextChar)
{
  if(pBuff.length() - buffPos < elementLength)
    return false;
  
  const char *c_buff = pBuff.data() + buffPos;
  
  if(strnicmp(c_buff,elementStr,elementLength) == 0 &&
      (nextChar == (pBuff)(buffPos + elementLength) || nextChar == '~' || (pBuff)(buffPos + elementLength) == '>' ))
    {
      return true;
    }
    else
      return false;
}



/**! remove leading and trailing \n \r \t " " 
	in url remove \n\r\t and " " replace to "%20" and resolve all other %xx
*/
void stripUrl(RVCString& url, bool bUnEscapeURL/* = true*/)
{
  int i;
	int len;

	len = url.length();
	i = 0;
  while (i < len) { //! remove space chars at the beginning
    if (url(i) == ' ' || url(i) == '\t' || url(i) == '\n' || url(i) == '\r')
			++i;
	else
	  break;
  }
	if( i > 0 )
		url.remove(0, i);

	len = url.length();
	i = len - 1;
  while (i > 0) {
    if (url(i) == ' ' || url(i) == '\t' || url(i) == '\n' || url(i) == '\r')
			--i;
	else
	  break;
  }
	if(i < len - 1)
	  url.remove(i+1);

  bool bIsJavaScriptURL = false;
  if (url.length() > 10 && RVCString(url(0,11)).compareTo("javascript:", RVCString::ignoreCase) == 0)
    bIsJavaScriptURL = true;

	char symbol;
  size_t strPtr = 0;
  int tmpInt;
	bool afterQuestion = false;
	char digits[3] = {'0','0','\0'};
  for (strPtr = 0; strPtr < url.length(); )
  {
		switch (url(strPtr))
		{
		case '\n':
		case '\r':
		case '\t':
			url.remove(strPtr, 1);
			break;

		case ' ':
			if(!bIsJavaScriptURL)
			{
				url.replace(strPtr, 1, "%20");
				strPtr += 3;
			}
			else
				++strPtr;

			break;

		case '?':
			afterQuestion = true;
			++strPtr;
			break;

		case '%':
			if (bUnEscapeURL)
			{
				if(afterQuestion)
				{
					++strPtr;
					break;
				}
				
				tmpInt = 0;
				strncpy(digits,url.data()+strPtr+1,2);
				sscanf(digits, "%x", &tmpInt);
				if(tmpInt == 0x20)
				{
					strPtr += 3;
				}
				else
				{
					symbol = (char)tmpInt;
					url.replace(strPtr++,3, (const char*)&symbol,1);
				}
			} else
			{
				++strPtr;
			}
			break;

		default:
			++strPtr;
			break;
		}
  }

}


// the url(0) should be first letter of the protocol
protclType getProtocol(const RVCString &url, int& p_oProtocolLength)
{
  p_oProtocolLength = -1; // User should check for -1 instead of isNull()

  if (url.index("fxcn")) 
  {
    int debugStop = 0;
    //webload/demo/random/fxcn
  }
#ifdef _DEBUG
 	RVCRegexp REprotocol("^[a-zA-Z]+:");
	RVCString protocol_d = url(REprotocol);
  p_oProtocolLength = protocol_d.length();

  //	if (protocol_d.isNull())
  //    return NO_protcl; // no protocol was found, it`s a relative url
#endif

  size_t nIndex = url.index(":");
  if (nIndex == RV_NPOS )
  {
    #ifdef _DEBUG
    WlAssert(protocol_d.isNull());
    #endif
    return NO_protcl;
  }
  
  

  // String must be alphabetic
  for (size_t i=0 ; i<nIndex ; i++)
  {
    if ( !isalpha (url(i)))
      return NO_protcl;
  }
  
  p_oProtocolLength = nIndex+1;
    
  RVCString protocol = url(0, nIndex+1);
  protocol.toLower();
  

  switch(protocol(0)) {
  case 'c': case 'C':
    if (protocol.compareTo("clsid:") == 0)
      return CLSID_prop;
    break;
    
  case 'f': case 'F':
    if (protocol.compareTo("ftp:") == 0)
      return FTP_protcl;
    else if (protocol.compareTo("file:") == 0)
      return FILE_protcl;
    break;
    
  case 'h': case 'H':
    if ((protocol.compareTo("http:") == 0))
      return HTTP_protcl;
    else if (protocol.compareTo("https:") == 0)
      return HTTPS_protcl;
    break;
    
  case 'j': case 'J':
    if (protocol.compareTo("javascript:") == 0)
      return JAVASCRIPT_protcl;
    break;
    
  case 'm': case 'M':
    if (protocol.compareTo("mailto:") == 0)
      return MAILTO_protcl;
    break;
  }
  return OTHER_protcl;
}





/*
// the url(0) should be first letter of the protocol
protclType getProtocol(const RVCString &url, RVCString& protocol)
{
	RVCRegexp REprotocol("^[a-zA-Z]+:");
	protocol = url(REprotocol);

	if (protocol.isNull())
    return NO_protcl; // no protocol was found, it's a relative url
  
  switch(protocol(0)) {
  case 'c': case 'C':
    if (protocol.compareTo("clsid", RVCString::ignoreCase) == 0)
      return CLSID_prop;
    break;
    
  case 'f': case 'F':
    if (protocol.compareTo("ftp:", RVCString::ignoreCase) == 0)
      return FTP_protcl;
    else if (protocol.compareTo("file:", RVCString::ignoreCase) == 0)
      return FILE_protcl;
    break;
    
  case 'h': case 'H':
    if (protocol.compareTo("http:", RVCString::ignoreCase) == 0)
      return HTTP_protcl;
    else if (protocol.compareTo("https:", RVCString::ignoreCase) == 0)
      return HTTPS_protcl;
    break;
    
  case 'j': case 'J':
    if (protocol.compareTo("javascript:", RVCString::ignoreCase) == 0)
      return JAVASCRIPT_protcl;
    break;
    
  case 'm': case 'M':
    if (protocol.compareTo("mailto:", RVCString::ignoreCase) == 0)
      return MAILTO_protcl;
    break;
  }
  return OTHER_protcl;
}
*/


DllExport void wlOverwriteTagWithBlanks(RVCString& p_sString,RVCString openingTag,RVCString closingTag)
{
	long lTagLength = openingTag.length() + closingTag.length();
	RVCString insertingText;// = "<!-- -->"; // 7 + 1
	RVCString emptyText;
	RVCString closeChar = ">";
	
	size_t buffPtr = 0, buffLength = p_sString.length();
	size_t openingTagIndex, closingTagIndex;
	
	while (buffPtr < buffLength) 
	{	
		openingTagIndex = p_sString.index(openingTag, buffPtr, RVCString::ignoreCase);
		closingTagIndex = p_sString.index(closingTag, buffPtr, RVCString::ignoreCase);
		
		if (openingTagIndex != RV_NPOS 
			&& closingTagIndex != RV_NPOS
			&& openingTagIndex < closingTagIndex) 
		{
			size_t removeLength = closingTagIndex - openingTagIndex + closingTag.length();
			if (removeLength < 7)
			{
				buffPtr = openingTagIndex + removeLength;
				continue;
			}
			p_sString.remove(openingTagIndex, removeLength);
			// insert instead: "<p>" + spaces + "</p>";
			emptyText.resize(removeLength - 7);
			insertingText = "<p>" + emptyText + "</p>";
			
			p_sString.insert(openingTagIndex, insertingText);
			int tempLen = insertingText.length();
			buffPtr = openingTagIndex + insertingText.length();  //closingTagIndex + /* 3 */ closingTag.length() - removeLength + insertingText.length();
		} 
		else
			break;
	}
}



void wlOverwriteScriptWithBlanks(RVCString& p_sString)
{
	RVCString openingTag = "<SCRIPT";
	RVCString closingTag = "</SCRIPT>";
	size_t buffPtr = 0, buffLength = p_sString.length();
	size_t openingTagIndex, closingTagIndex;
	
	while (buffPtr < buffLength) 
	{	
		openingTagIndex = p_sString.index(openingTag, buffPtr, RVCString::ignoreCase);
		closingTagIndex = p_sString.index(closingTag, buffPtr, RVCString::ignoreCase);
		
		if (openingTagIndex != RV_NPOS 
			&& closingTagIndex != RV_NPOS
			&& openingTagIndex < closingTagIndex) 
		{
			//Find end of Opening Script
			size_t endOfOpeningTagIndex = p_sString.index(">", openingTagIndex + 1);
			if (endOfOpeningTagIndex != RV_NPOS &&
				endOfOpeningTagIndex < closingTagIndex)
			{
				size_t removeLength = closingTagIndex - endOfOpeningTagIndex - 1;
				if (removeLength > 0)
				{
					RVCString emptyText;
					emptyText.resize(removeLength);
					p_sString.remove(endOfOpeningTagIndex + 1, removeLength);
					p_sString.insert(endOfOpeningTagIndex + 1, emptyText);
				}
				buffPtr = closingTagIndex + closingTag.length();
			}
		} 
		else
			break;
		
	}
}

DllExport  void wlSkipTagBody(const RVCString& p_sTagname, RVCString& p_sString)
{
	RVCString openingTag = "<" + p_sTagname;
	RVCString closingTag = "</" + p_sTagname + ">";
	RVCString insertingText = "<" + p_sTagname + ">" + "</" + p_sTagname + ">";
	size_t lt=0;
	bool quit = false;
	while (!quit) {
	if ((lt = p_sString.index("<", lt)) == RV_NPOS)
		break; // while
	if(p_sString.length() - lt < p_sTagname.length()+1)// <script or <noscript
		break; // while

	switch(p_sString(lt+1)) {
		case '!' :
			//! skip comments if any....
			if (p_sString(lt+2)=='-' && p_sString(lt+3)=='-') { 	
				lt = p_sString.index("-->",lt+4);
				if( lt == RV_NPOS ) //! end comment not found
					quit = true;
			}
		break;

		default://case 's': case 'S':
		{ 
			RVCString tag = p_sString(lt, p_sTagname.length()+1);
			//! we are looking for opening tag
			if (tag.compareTo(openingTag, RVCString::ignoreCase) == 0) {
				size_t endTagIndex = p_sString.index(closingTag, lt+p_sTagname.length()+1, RVCString::ignoreCase);
				if(endTagIndex==RV_NPOS) //for broken html
				{
					quit = true;
					break; // switch
				}
				size_t startCommentIndex = p_sString.index("<!--", lt+p_sTagname.length()+1);
				//! trying to determine closing tag
				//! taking the first closing tag that is not in comment
				if (startCommentIndex != RV_NPOS && startCommentIndex < endTagIndex) {
					size_t endCommentIndex = p_sString.index("-->", startCommentIndex);
					if (endCommentIndex != RV_NPOS && endCommentIndex > endTagIndex) {
					//! skip comment, look for closing scirpt tag:
						endTagIndex = p_sString.index(closingTag, endCommentIndex, RVCString::ignoreCase);
						if (endTagIndex == RV_NPOS) {
							quit = true;
							break; // switch
						}
					}
				}

				if (endTagIndex == RV_NPOS) {
					quit = true;
					break; // switch
				}
				
				size_t removeLength = endTagIndex - lt + p_sTagname.length() + 3;
				p_sString.remove(lt, removeLength);
				p_sString.insert(lt, insertingText);
				lt = endTagIndex + p_sTagname.length() + 3 - removeLength + insertingText.length() - 1;
			}
		}
		break;
	} //! switch
		++lt;
	} //! while(1)		 
}


/*! In the buff starting from the buffPtr index trying to determine attribute if it's
 *  name=value pair or just name.
 * Returning status adn filling attributeName, attributeValue, the place where value
 * starts if any. BuffPtr points at blank after the attribute(pair/one) or after
 * the delimiter '>', depends who comes first.
 */
void htmlFindNextAttribute(RVCString& p_sBuff, RVCString& attributeName, RVCString& attributeValue, size_t& buffPtr, size_t& attValueStart)
{
	size_t length = p_sBuff.length(), index;
	size_t attributeIndex = RV_NPOS, delimiterIndex = RV_NPOS,
		endAttributeIndex = RV_NPOS, equalSignIndex = RV_NPOS;
	char c;
	static	RVCRegexp sp("[>\\t\\r\\n ]"); // oooo
	
	for(index = buffPtr; index < length; ++index){
		c = (p_sBuff)(index);
		switch(c) {
				case '=':
					if (attributeIndex != RV_NPOS) {
						if (endAttributeIndex == RV_NPOS)
						endAttributeIndex = index;
					equalSignIndex = index;
					} else { //! Illegal char
						size_t i = p_sBuff.index(sp, index);
						if (i == RV_NPOS)
							return;
						if ((p_sBuff)(i) == '>') {
							delimiterIndex = i;
							break;
						}
						index = i;
					}

					break;

				case '>':
					if (attributeIndex != RV_NPOS && endAttributeIndex == RV_NPOS)
						endAttributeIndex = index;
					delimiterIndex = index;
					break;

				case ' ':
				case '\t':
				case '\n':
				case '\r':
        case '"':
					if (attributeIndex != RV_NPOS && endAttributeIndex == RV_NPOS)
						endAttributeIndex = index;
					continue;

				default:
					if (isalnum(c) || c == '-') {
						if (attributeIndex == RV_NPOS)
							attributeIndex = index;
						else if (endAttributeIndex != RV_NPOS)
							break;
					} else { //! Illegal char
						size_t i = p_sBuff.index(sp, index);
						if (i == RV_NPOS)
							return;
						if ((p_sBuff)(i) == '>') {
							delimiterIndex = i;
							break;
						}
						index = i;
					}
					continue;
		}
    break;
  }
  
  if(attributeIndex != RV_NPOS && endAttributeIndex != RV_NPOS)
  {
    attributeName  = (p_sBuff)(attributeIndex, endAttributeIndex - attributeIndex);
    
    if( equalSignIndex == RV_NPOS )  // no value !!
    {
      attributeValue = "";
      buffPtr = endAttributeIndex;
      return;
    } else {
      size_t endAttributsPtr = 0, startAttributsPtr = 0;
      strVal attVal = htmlGetStringValue(p_sBuff, attributeValue, equalSignIndex, startAttributsPtr, endAttributsPtr, length);
      if(attVal == HTML_STRING_NOT_COMPLETE)
        return;
      else// attVal == HTML_STRING_COMPLETE
      {
        attValueStart = startAttributsPtr;
        buffPtr = endAttributsPtr;
        return;
      }
    }
  } else if(delimiterIndex != RV_NPOS) {
    buffPtr = delimiterIndex+1;
    return ;
  } else
    return;
}


DllExport  void wlSkipOnloadAttribute(RVCString& p_sString)
{
	RVCRegexp REonload("[oO][nN][lL][oO][aA][dD][\n\r\t ]*=");
	RVCRegexp REonmouseover("[oO][nN][mM][oO][uU][sS][eE][oO][vV][eE][rR][\\n\\r\\t ]*=");
	RVCRegexp REonmouseout("[oO][nN][mM][oO][uU][sS][eE][oO][uU][tT][\\n\\r\\t ]*=");
	RVCString insertingOnloadText = "onload=\"\"";
	RVCString insertingOnmouseoverText = "onmouseover=\"\"";
	RVCString insertingOnmouseoutText = "onmouseout=\"\"";
	size_t REindex, RElength, buffPtr, attValueStart;
	RVCString attName, attValue;

	for (buffPtr = 0, attValueStart = RV_NPOS; buffPtr < p_sString.length(); buffPtr = REindex + insertingOnloadText.length()) {
		REindex = p_sString.index(REonload, &RElength, buffPtr);
		if (REindex == RV_NPOS)
			break;
		buffPtr = REindex;
		htmlFindNextAttribute(p_sString, attName, attValue, buffPtr, attValueStart);
		RElength = attValueStart + attValue.length() - REindex;
		if (p_sString(REindex + RElength ) == '"')
			++RElength;
		p_sString.remove(REindex, RElength);
		p_sString.insert(REindex, insertingOnloadText);
	}

	//! onmouseover
  
  
	for (buffPtr = 0, attValueStart = RV_NPOS; buffPtr < p_sString.length(); buffPtr = REindex + insertingOnmouseoverText.length()) {
		REindex = p_sString.index(REonmouseover, &RElength, buffPtr);
		if (REindex == RV_NPOS)
			break;
		buffPtr = REindex;
		htmlFindNextAttribute(p_sString, attName, attValue, buffPtr, attValueStart);
		RElength = attValueStart + attValue.length() - REindex;
		if (p_sString(REindex + RElength ) == '"')
			++RElength;
		p_sString.remove(REindex, RElength);
		p_sString.insert(REindex, insertingOnmouseoverText);
	}

	//! onmouseout
	for (buffPtr = 0, attValueStart = RV_NPOS; buffPtr < p_sString.length(); buffPtr = REindex + insertingOnmouseoutText.length()) {
		REindex = p_sString.index(REonmouseout, &RElength, buffPtr);
		if (REindex == RV_NPOS)
			break;
		buffPtr = REindex;
		htmlFindNextAttribute(p_sString, attName, attValue, buffPtr, attValueStart);
		RElength = attValueStart + attValue.length() - REindex;
		if (p_sString(REindex + RElength ) == '"')
			++RElength;
		p_sString.remove(REindex, RElength);
		p_sString.insert(REindex, insertingOnmouseoutText);
	}
}

