/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// RVXMLElement.h: interface for the RVXMLElement class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _RVXMLELEMENT_H_
#define _RVXMLELEMENT_H_
#include <RVCString.h>

#include <tvsldict.h>
#include <tvslist.h>
#include "wlerrorlib.h"
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMElement.hpp>

class RVXMLElement;

typedef RVTValSlist<RVXMLElement> RVXMLElementList;
typedef RVTValSlistDictionary<RVCString, RVCString> AttrDict;

XERCES_CPP_NAMESPACE_USE


class DllExport RVXMLElement
{
friend class RVXMLDocument;
public:
	virtual ~RVXMLElement();
	AttrDict get_attributes(CWLErrorInfo& p_ErrorInfo);
	RVCString get_attribute(RVCString p_sAttributeName, CWLErrorInfo& p_ErrorInfo);
	bool set_attribute(RVCString p_sAttributeName, RVCString p_sAttributeValue, CWLErrorInfo& p_ErrorInfo);
	RVCString get_name(CWLErrorInfo& p_ErrorInfo);
	RVCString get_value(CWLErrorInfo& p_ErrorInfo);
	bool set_value(RVCString p_sValue, CWLErrorInfo& p_ErrorInfo);
	RVXMLElement get_parentElement(CWLErrorInfo& p_ErrorInfo);
	RVXMLElementList get_childElements(CWLErrorInfo& p_ErrorInfo);
	RVXMLElement add_childElement(RVXMLElement& p_RVXMLElement, CWLErrorInfo& p_ErrorInfo);
	RVXMLElement add_childElement(RVCString p_sName, RVCString p_sValue, CWLErrorInfo& p_ErrorInfo);
	RVXMLElement add_childElement(RVCString p_sName, RVCString p_sValue, AttrDict p_Attributes, CWLErrorInfo& p_ErrorInfo);
	bool is_empty(CWLErrorInfo& p_ErrorInfo);
	RVCString get_xml(CWLErrorInfo& p_ErrorInfo);
	RVXMLElementList getElementsByTagName(RVCString p_sTagName, CWLErrorInfo& p_ErrorInfo);

private:
  DOMElement* m_pXMLDOMElement;
  XERCES_CPP_NAMESPACE::DOMDocument* m_pXMLDocument;
  RVXMLElement(XERCES_CPP_NAMESPACE::DOMDocument* p_pXMLDomDocument, DOMElement* p_pXMLDomElement);
	RVXMLElement(XERCES_CPP_NAMESPACE::DOMDocument* p_pXMLDomDocument, RVCString p_sName, RVCString p_sValue = "");
};

#endif // _RVXMLELEMENT_H_
