/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  




//////////////////////////////////////////////////////////////////////


#include <assert.h>
#include "rvxmldocument.h"
#include <xercesc/dom/DOM.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include "transcode.h"

XERCES_CPP_NAMESPACE_USE

/*!
* RVXMLDocument::RVXMLDocument(RVCString p_sRootElementName) :
* \brief 
* Contructor
* \param p_sRootElementName (in)
* \return void
*/

bool RVXMLDocument::m_Initialized = false;

RVXMLDocument::RVXMLDocument(RVCString p_sRootElementName) :
	m_RootElement(NULL,NULL)
{
    if (!m_Initialized) {
      try {
         XMLPlatformUtils::Initialize();
      }
      catch (const XMLException& toCatch)
     {
          char* message = XMLString::transcode(toCatch.getMessage());
          cerr << "Error during initialization! :\n"
						<< RVCString(message) << endl;
          XMLString::release(&message);
      }
      m_Initialized = true;
    }

    static const XMLCh gLS[] = { chLatin_L, chLatin_S, chNull };
    DOMImplementation *impl = DOMImplementationRegistry::getDOMImplementation(gLS);
    DOMBuilder* parser = ((DOMImplementationLS*)impl)->createDOMBuilder(DOMImplementationLS::MODE_SYNCHRONOUS, 0);

    m_DOMparser = parser;
    m_pDocument =NULL;

    m_pDocument = impl->createDocument();
		
		m_RootElement.m_pXMLDocument = m_pDocument;
		XMLCh* rootName = Galtech_transcode(p_sRootElementName.data());
		m_RootElement.m_pXMLDOMElement = m_pDocument->createElement(rootName);
		delete[] rootName;
}

RVXMLDocument::~RVXMLDocument()
{
  if (m_DOMparser) {
    delete m_DOMparser;
    m_DOMparser = NULL;
  }
}

/*!
* bool RVXMLDocument::load(RVCString p_sURL, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Loads XML from URL
* \param p_sURL (in)
* \param p_ErrorInfo (out)   - Error Info
* \return bool
*/
bool RVXMLDocument::load(RVCString p_sURL, CWLErrorInfo& p_ErrorInfo)
{
	try
	{
    
    XMLCh* pcURI = XMLString::transcode(p_sURL.data());
    XERCES_CPP_NAMESPACE::DOMDocument* doc = m_DOMparser->parseURI(pcURI);
    XMLString::release(&pcURI);

     if ( doc != NULL) {
       m_pDocument = doc;
  	    DOMElement* newRoot;
			  newRoot = m_pDocument->getDocumentElement();
        m_RootElement = RVXMLElement(m_pDocument, newRoot);
  
        return true;
     }
		// do something with p_ErrorInfo
		return false;
	}
  catch (const XMLException& e)
  {
//		m_parseErrors.insertKeyAndValue("errorType",RVCString(e.getErrorType()));
//		m_parseErrors.insertKeyAndValue("errorCode", RVCString("" + e.getCode()));
//		m_parseErrors.insertKeyAndValue("filepos", RVCString("" + e.getSrcFile() ));
//		m_parseErrors.insertKeyAndValue("line", RVCString(""+ e.getSrcLine()));
		char* trMessage = Galtech_transcode(e.getMessage());
     cerr << "\nError during parsing: '" << p_sURL << "'\n"
           << "Exception message is:  \n"

           << RVCString(trMessage) << "\n" << endl;
		 delete[] trMessage;
      return false;
  }

	catch (...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return false;
	}

}
/*!
* RVCString RVXMLDocument::get_xml(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Gets XML String
* \param p_ErrorInfo (out)   - Error Info
* \return RVCString
*/
#ifdef notdef
RVCString RVXMLDocument::get_xml(CWLErrorInfo& p_ErrorInfo)
{
	if (m_pDocument == NULL || m_RootElement.m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return "";
	}
	try 
	{
		return RVCString(m_pDocument->xml);
	} 
	catch (...)
	{
		// do something with p_ErrorInfo
		assert(0);
		return "";
	}
}
#endif //notdef
/*!
* RVXMLElement RVXMLDocument::get_documentElement(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Gets XML root element
* \param p_ErrorInfo (out)   - Error Info
* \return RVXMLElement
*/
RVXMLElement RVXMLDocument::get_documentElement(CWLErrorInfo& p_ErrorInfo)
{
	if (m_pDocument == NULL || m_RootElement.m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return RVXMLElement(NULL, NULL);
	}
	try
	{
		return m_RootElement;
	}
	catch (...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return RVXMLElement(NULL,NULL);
	}
}

/*!
* RVTValSlistDictionary<RVCString, RVCString> RVXMLDocument::get_parseError(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns a dictionary with XML parsing errors
* \param p_ErrorInfo (out)   - Error Info
* \return RVTValSlistDictionary<RVCString, RVCString>
*/
RVTValSlistDictionary<RVCString, RVCString> RVXMLDocument::get_parseError(CWLErrorInfo& p_ErrorInfo)
{
	RVTValSlistDictionary<RVCString, RVCString> errorDict;
	try
	{
#ifdef notdef
    // we have to set some error handler hand when parsing add all 
    // all error to the dictonary
  	ErrorHandler* pErrObj = m_DOMparser->getErrorHandler();
    if (pErrObj) {
		  errorDict.insertKeyAndValue("errorCode", RVCString("" + pErrObj->errorCode));
		  errorDict.insertKeyAndValue("filepos", RVCString("" + pErrObj->filepos));
		  errorDict.insertKeyAndValue("line", RVCString(pErrObj->line +""));
		  errorDict.insertKeyAndValue("linepos",RVCString(pErrObj->linepos + ""));
		  errorDict.insertKeyAndValue("reason",RVCString(pErrObj->reason));
		  errorDict.insertKeyAndValue("srcText",RVCString(pErrObj->srcText));
		  errorDict.insertKeyAndValue("url",RVCString(pErrObj->url));
    }
#endif //notdef
	}
	catch (...)
	{
			assert(0);
			// do something with p_ErrorInfo
			errorDict.clear();
	}
	return errorDict;
}
