/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// RVXMLElement.cpp: implementation of the RVXMLElement class.
//
//////////////////////////////////////////////////////////////////////

#include <assert.h>
#include "rvxmlelement.h"

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp> 
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/util/XMLString.hpp>
#include "transcode.h"
//Galtech
#include "gt_strwrapper.h"

XERCES_CPP_NAMESPACE_USE
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

RVXMLElement::~RVXMLElement()
{
}
/*!
* AttrDict RVXMLElement::get_attributes(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns a dictionary of XML Element attributes.
* \param p_ErrorInfo (out)   - Error Info
* \return RVTValSlistDictionary<RVCString,RVCString>

*
*/
AttrDict RVXMLElement::get_attributes(CWLErrorInfo& p_ErrorInfo)
{
	AttrDict dict;
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return dict;
	}
	try
	{	
    DOMNamedNodeMap*  attributeMap;
		attributeMap = m_pXMLDOMElement->getAttributes();
		DOMNode *item;
		for (unsigned int i = 0 ; i < attributeMap->getLength() ; i++)
		{
			item = attributeMap->item(i);
      if ( item != NULL ) {
        const XMLCh* name = item->getNodeName();
        const XMLCh* value = item->getNodeValue();
        char* trName = XMLString::transcode(name);
		    char* trValue = XMLString::transcode(value);
			  dict.insertKeyAndValue(RVCString(trName), RVCString(trValue));
        XMLString::release(&trName);
        XMLString::release(&trValue);
      }
		}
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		dict.clear();
	}
	return dict;
}
/*!
* RVCString RVXMLElement::get_attribute(RVCString p_sAttributeName, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns attribute value of the submited XML Element.
* \param p_ErrorInfo (out)   - Error Info
* \param p_sAttributeName (in)   - Attribute Name
* \return RVCString
*/
RVCString RVXMLElement::get_attribute(RVCString p_sAttributeName, CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return "";
	}

	try
	{
		XMLCh* trAttr = XMLString::transcode(p_sAttributeName.data());
		const XMLCh* attrValue = m_pXMLDOMElement->getAttribute(trAttr);
		XMLString::release(&trAttr);
        
    char* trVal = XMLString::transcode(attrValue);
		RVCString rwTrVal(trVal);
    XMLString::release(&trVal);
		return rwTrVal;
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo - attrib ute not found
		return "";
	}
}
/*!
* bool RVXMLElement::set_attribute(RVCString p_sAttributeName, RVCString p_sAttributeValue, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Sets attribute value of the submited XML Element.
* \param p_ErrorInfo (out)   - Error Info
* \param p_sAttributeName (in)   - Attribute Name
* \param p_sAttributeValue (in)   - Attribute Name
* \return bool
*/
bool RVXMLElement::set_attribute(RVCString p_sAttributeName, RVCString p_sAttributeValue, CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return false;
	}

	try
	{
    
    XMLCh* attrName = Galtech_transcode(p_sAttributeName.data());
    XMLCh* attrValue = Galtech_transcode(p_sAttributeValue.data());
		m_pXMLDOMElement->setAttribute(attrName, attrValue);
		delete[] attrName;
		delete[] attrValue;
		return true;
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return false;
	}
}

/*!
* RVCString RVXMLElement::get_name(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns XML Element name
* \param p_ErrorInfo (out)   - Error Info
* \return RVCString
*/
RVCString RVXMLElement::get_name(CWLErrorInfo& p_ErrorInfo)
{	
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return "";
	}

	try
	{
    const XMLCh* nodeName = m_pXMLDOMElement->getNodeName();
    char* trNodeName = XMLString::transcode(nodeName);
		RVCString rwNodeName(trNodeName);
    XMLString::release(&trNodeName);
    return rwNodeName;
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return "";
	}
}

/*!
* RVCString RVXMLElement::get_value(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns XML Element value
* \param p_ErrorInfo (out)   - Error Info
* \return RVCString
*/
RVCString RVXMLElement::get_value(CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return "";
	}

	try
	{
    if (!m_pXMLDOMElement->hasChildNodes())
      return "";
    DOMNodeList* childList = m_pXMLDOMElement->getChildNodes();
    DOMNode* node;
    const XMLCh* value;

		for (unsigned int i = 0 ; i < childList->getLength() ; i++)
		{
			node = childList->item(i);
      if ( node != NULL ) {
        if ( DOMNode::TEXT_NODE == node->getNodeType() ) {
          value = node->getNodeValue();
					char* trValue = Galtech_transcode(value);
					RVCString rwValue(trValue);
					delete[] trValue;
          return rwValue;
        }
      }
		}
		return "";
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return "";
	}
}
/*!
* RVCString RVXMLElement::set_value(RVCString p_sValue, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Sets XML Element value
* \param p_ErrorInfo (out)   - Error Info
* \param p_sValue (in)   - Value
* \return bool
*/
bool RVXMLElement::set_value(RVCString p_sValue, CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return false;
	}

	try
	{
    if (m_pXMLDOMElement->hasChildNodes()) {
      DOMNodeList* childList = m_pXMLDOMElement->getChildNodes();
      DOMNode* node;
      XMLCh* attrValue = Galtech_transcode(p_sValue.data());

		  for (unsigned int i = 0 ; i < childList->getLength() ; i++)
		  {
			  node = childList->item(i);
        if ( node != NULL ) {
          if ( DOMNode::TEXT_NODE == node->getNodeType() ) {
      		  node->setNodeValue(attrValue);
						delete[] attrValue;
      		  return true;
          }
        }
		  }
    }
    return false;
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return false;
	}
}
/*!
* RVXMLElement RVXMLElement::get_parentElement(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns parent RVXMLElement object, returns empty object in case of error or if there is no parent
* \param p_ErrorInfo (out)   - Error Info
* \return RVXMLElement
*/
RVXMLElement RVXMLElement::get_parentElement(CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return RVXMLElement(NULL,NULL);
	}

	try
	{
  	DOMNode*  parent = m_pXMLDOMElement->getParentNode();
		return RVXMLElement(m_pXMLDocument,(DOMElement *)parent);
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return RVXMLElement(NULL,NULL);
	}
}
/*!
* RVXMLElementList RVXMLElement::get_childElements(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns RVTValSlist<RVXMLElement> of child elements
* \param p_ErrorInfo (out)   - Error Info
* \return RVXMLElementList (RVTValSlist<RVXMLElement>)
*/
RVXMLElementList RVXMLElement::get_childElements(CWLErrorInfo& p_ErrorInfo)
{
	RVXMLElementList list;
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return list;
	}


	try
	{
    if (m_pXMLDOMElement->hasChildNodes()) {
      DOMNodeList* childList = m_pXMLDOMElement->getChildNodes();
      DOMNode* node;

		  for (unsigned int i = 0 ; i < childList->getLength() ; i++)
		  {
			  node = childList->item(i);
        if ( node != NULL ) {
          if ( DOMNode::ELEMENT_NODE == node->getNodeType() ) {
  				  list.insert(RVXMLElement(m_pXMLDocument,(DOMElement *)node));
          }
        }
		  }
    }
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		list.clear();
	}
	return list;
}

/*!
* RVXMLElement RVXMLElement::add_childElement(RVCString p_sName, RVCString p_sValue, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Adds child element, returns new child
* \param p_ErrorInfo (out)   - Error Info
* \param p_sName (in)   - Name of new element
* \param p_sValue (in)   - Value of new elements
* \return RVXMLElement
*
*/
RVXMLElement RVXMLElement::add_childElement(RVCString p_sName, RVCString p_sValue, CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		RVXMLElement(NULL,NULL);
	}

	try
	{
		//Galtech
		p_sValue = ToANSI(p_sValue);

		RVXMLElement newElement(m_pXMLDocument, p_sName, p_sValue);
		DOMElement* newChild = (DOMElement *)m_pXMLDOMElement->appendChild(newElement.m_pXMLDOMElement);
		newElement.m_pXMLDOMElement = newChild;
		return newElement;
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return RVXMLElement(NULL,NULL);
	}

}
/*!
* RVXMLElement RVXMLElement::add_childElement(RVXMLElement& p_RVXMLElement, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Adds child element, returns new child
* \param p_ErrorInfo (out)   - Error Info
* \param p_RVXMLElement (in)   - New element
* \return RVXMLElement
*
*/
RVXMLElement RVXMLElement::add_childElement(RVXMLElement& p_RVXMLElement, CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		RVXMLElement(NULL,NULL);
	}

	try
	{
		DOMNode* clonedNode = p_RVXMLElement.m_pXMLDOMElement->cloneNode(true);	
		DOMElement* newChild = (DOMElement *)m_pXMLDOMElement->appendChild(clonedNode);
		return RVXMLElement(m_pXMLDocument, newChild);
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return RVXMLElement(NULL,NULL);
	}
}
/* RVXMLElement RVXMLElement::add_childElement(RVCString p_sName, RVCString p_sValue, AttrDict p_Attributes, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Adds child element, returns new child
* \param p_ErrorInfo (out)   - Error Info
* \param p_sName (in)   - New element's name
* \param p_sValue (in)   - New element's value
* \param p_Attributes (in)   - New element's attribute dictionary
* \return RVXMLElement
*
*/
RVXMLElement RVXMLElement::add_childElement(RVCString p_sName, RVCString p_sValue, AttrDict p_Attributes, CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		RVXMLElement(NULL,NULL);
	}

	try
	{
		return *this;
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return *this;
	}
}

/*!
* RVXMLElement::RVXMLElement(DOMDocument* p_pXMLDomDocument, DOMElement* p_pXMLDomElement) :
* \brief 
* Contructor
* \param p_pXMLDomDocument (in)
* \param p_pXMLDomElement (in)
* \return void
*/
RVXMLElement::RVXMLElement( XERCES_CPP_NAMESPACE::DOMDocument* p_pXMLDomDocument, DOMElement* p_pXMLDomElement) :
	m_pXMLDocument(p_pXMLDomDocument),
	m_pXMLDOMElement(p_pXMLDomElement)
{
}

/*!
* RVXMLElement::RVXMLElement(DOMDocument* p_pXMLDomDocument, RVCString p_sName, RVCString p_sValue)
* \brief 
* Contructor
* \param p_pXMLDomDocument (in)
* \param p_sName (in)
* \param p_sValue (in)
* \return void
*/

RVXMLElement::RVXMLElement(XERCES_CPP_NAMESPACE::DOMDocument* p_pXMLDomDocument, RVCString p_sName, RVCString p_sValue)
	:	m_pXMLDocument(p_pXMLDomDocument)
{
  XMLCh* sName = Galtech_transcode(p_sName.data());
  XMLCh* sValue = Galtech_transcode(p_sValue.data());
  m_pXMLDOMElement = p_pXMLDomDocument->createElement(sName);

  DOMText* prodDataVal = p_pXMLDomDocument->createTextNode(sValue);
	delete[] sName;
	delete[] sValue;
  m_pXMLDOMElement->appendChild((DOMNode *)prodDataVal);
}

/*!
* bool RVXMLElement::is_empty(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns true is the element is empty
* \param p_ErrorInfo (out)   - Error Info
* \return bool
*/
bool RVXMLElement::is_empty(CWLErrorInfo& p_ErrorInfo)
{
	return (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL);
}
/*!
* RVCString RVXMLElement::get_xml(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Gets XML String
* \param p_ErrorInfo (out)   - Error Info
* \return RVCString
*/
RVCString RVXMLElement::get_xml(CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return "";
	}
	try 
	{
    assert(0);
		return "";
//ofer		return RVCString(m_pXMLDOMElement->xml);
	} 
	catch (...)
	{
		// do something with p_ErrorInfo
		return "";
	}
}

/*!
* RVXMLElementList RVXMLElement::getElementsByTagName(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns a list of all descendant elements that match the supplied name.
* \param p_ErrorInfo (out)   - Error Info
* \return RVXMLElementList
*/

RVXMLElementList RVXMLElement::getElementsByTagName(RVCString p_sTagName, CWLErrorInfo& p_ErrorInfo)
{
	RVXMLElementList list;

	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return list;
	}

	try
	{
    XMLCh* sTagName = Galtech_transcode(p_sTagName.data());
		DOMNodeList* p_resultList = m_pXMLDOMElement->getElementsByTagName(sTagName);
		delete[] sTagName;
		//Check hr
		for (unsigned int i = 0 ; i<p_resultList->getLength(); i++)
		{
			list.insert(RVXMLElement(m_pXMLDocument,(DOMElement *)p_resultList->item(i)));
		}
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		list.clear();
	}
	return list;
}
