/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLStatPointTypeEnum.cpp: implementation of the CWLStatPointTypeEnum class.
//
//////////////////////////////////////////////////////////////////////
#include <RVCString.h>
#include <assert.h>
#include <math.h>
#include "wlstatmanagerglobals.h"

#include "wlstatpointtypeenum.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


const char* MeasurementTypeNames[] = {
  "1",    // "Measurement",
  "2",    // "Timer",
  "4",    // "Counter",
  "8",    // "Accumulator",
  "16",   // "LoadSize",
  "32",   // "TransactionCounter",
  "64",   // "TransactionTimer",
  "128",  // "TransactionReasonCounter"
  "256",  // MeasurementPerSec  ===========> never pass from here
  "512",  // CounterPerSec      ===========> never pass from here
  "1024",  // NTPerformance 
  "2048"   // AccumulatorPerSec
};

const char* MeasurementTypeStringNames[] = {
  "Measurement",
  "Timer",
  "Counter",
  "Accumulator",
  "LoadSize",
  "TransactionCounter",
  "TransactionTimer",
  "TransactionReasonCounter",
  "MeasurementPerSec",
  "CounterPerSec",
  "NTPerformance",
  "AccumulatorPerSec"
};

CWLStatPointTypeEnum::CWLStatPointTypeEnum()
{

}

CWLStatPointTypeEnum::~CWLStatPointTypeEnum()
{

}

void CWLStatPointTypeEnum::GetEnumTypeFromString(const RVCString &StringType,  PointType  &retType)
{
  if (StringType.compareTo(TIMER_STR) == 0) 
    retType = Timer;
  else if ( (StringType.compareTo(COUNTER_STR)==0) || (StringType.compareTo("Stat")==0) ) 
    retType = Counter;
  else if (StringType.compareTo(LOAD_SIZE_STR) == 0) 
    retType = LoadSize;
  else if (StringType.compareTo(MEASUREMENT_STR) == 0) 
    retType = Measurement;
  else if (StringType.compareTo(PERF_MON_STR) == 0) 
    retType = NTPerformance;
  else if (StringType.compareTo(COUNTER_PER_SEC_STR) == 0) 
    retType = CounterPerSec;
  else if (StringType.compareTo(MEASUREMENT_PER_SEC_STR) == 0) 
    retType = MeasurementPerSec;
  else if (StringType.compareTo(TRANSACTION_COUNTER_STR) == 0)
    retType =TransactionCounter; 
  else if (StringType.compareTo(TRANSACTION_TIMER_STR) == 0)
    retType = TransactionTimer;
  else if (StringType.compareTo(TRANSACTION_REASON_COUNTER_STR) == 0)
    retType = TransactionReasonCounter;
  else if (StringType.compareTo(ACCUMULATOR_STR) == 0)
    retType = Accumulator;
  else if (StringType.compareTo(ACCUMULATOR_PER_SEC_STR) == 0)
    retType = AccumulatorPerSec;

  else
  {
    assert(0);
    return ;
  }

}


const char* CWLStatPointTypeEnum::GetTypeName(PointType type)
{
  double dtype = (double)type;
  if ( dtype == 0 )     // have the same resault as log(1)/log(2)
    return "0";

  double dindex = log(dtype)/log(2);
  int index = (int)floor(dindex+0.5);
  return MeasurementTypeNames[index];
}

const char* CWLStatPointTypeEnum::GetStringFromEnumType(PointType type)
{
  double dtype = (double)type;
  if ( dtype == 0 )     // have the same resault as log(1)/log(2)
    return "NotSet";

  double dindex = log(dtype)/log(2);
  int index = (int)floor(dindex+0.5);
  return MeasurementTypeStringNames[index];

}

