/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// FileTransferHandler.h: interface for the FileTransferHandler class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FILETRANSFERHANDLER_H__B874D983_20CB_11D4_87DA_0004AC15A422__INCLUDED_)
#define AFX_FILETRANSFERHANDLER_H__B874D983_20CB_11D4_87DA_0004AC15A422__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <thr.h>
#include <thr.h>
#include <stdio.h>
#include "testtalknetconnection.h"
#include "associativearray.h"
#include "netevent.h"
#include "testtalkdefs.h"

// Definitions:
#define FILE_CHUNK_SIZE           51200 // 50K 
#define NUMOF_CHUNKS_TO_WAIT_FOR  5

#define FILECOPY_RECEIVER_READY		'A'		
#define FILECOPY_CHUNK				    'B'
#define FILECOPY_LAST_CHUNK			  'C'
#define FILECOPY_CHUNK_ACK				'D'
#define STOP_RECEIVER				      'E'
#define STOP_SENDER								'F'

#define STR_FILECOPY_RECEIVER_READY		"A"
#define STR_FILECOPY_CHUNK				    "B"
#define STR_FILECOPY_LAST_CHUNK			  "C"
#define STR_FILECOPY_CHUNK_ACK				"D"
#define STR_STOP_RECEIVER				      "E"
#define STR_STOP_SENDER								"F"

typedef TVPEventQueue::WaitReturnType QueueWaitReturnType;

class CTestTalkHostConnection;


// Class Header:
class CFileTransferHandler: RVMonitor<RVMutexLock>
{
public:
	enum CompletionState {CopySucceed, OpenFailed, CloseFailed, ReadFaild, WriteFailed, CopyReleased};

	struct CompletionInfo
	{
		int          m_CompletionState;
		const char*	 m_FileName;
	};
	
public:
	CFileTransferHandler(CTestTalkHostConnection *hostConnection);
	virtual ~CFileTransferHandler(void);

	CFileTransferHandler::CompletionInfo* SendFile(const char* srcFilePath, 
																			  				 const char* dstFilePath, 
																								 TestTalkHostEvent* addressInfo);

	CFileTransferHandler::CompletionState ReceiveFile(const char* dstFilePath, 
																						  			TestTalkHostEvent* addressInfo);
	
	void Terminate(void);

	virtual void HandleFileEvent(void* eventMsg, 
															 int len, 
															 short type, 
															 short subtype);

private:

	void ResetSender(void);
	void ResetReceiver(void);
	void StopSender(CompletionState status);
	void StopReceiver(void);
	void WaitForReceiver(void); 
	void WaitLastChunkEventAck(void);
	void SignalSendingEvent(void);
	inline void SetNumOfExpectedEventAcks(int value);
	QueueWaitReturnType WaitFileChunkEvent(TestTalkHostEvent*& event);
	void PutFileChunkEvent(TestTalkHostEvent *fileEvent);
	void SetFileChunkEventAck(void);
	
	
	CFileTransferHandler::CompletionState HandlerSendFile(const char* srcFilePath, 
  																											const char* dstFilePath, 
																												TestTalkHostEvent* addressInfo);

	CFileTransferHandler::CompletionState HandlerReceiveFile(const char* dstFilePath, 
																						  						 TestTalkHostEvent* addressInfo);


private:
	int m_Counter;
	int m_MaxCounterValue;
	bool m_Send, m_Receive, m_ObjectTerminated;
	RVCondition m_Transfer;
	TVPEventQueue m_FileChunksQueue;
	CTestTalkHostConnection *m_Connection;
	CompletionState m_SenderStatus, m_ReceiverStatus; 
};

#endif // !defined(AFX_FILETRANSFERHANDLER_H__B874D983_20CB_11D4_87DA_0004AC15A422__INCLUDED_)

