/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// Local Variables:
// mode: c++
// End:
//

#ifndef __TestTalk_NETEVENT_H
#define __TestTalk_NETEVENT_H

#include "clntsrv.h"
#include "netmsg.h"

typedef enum {
  TestTalkClientEventType,
  TestTalkHostEventType
} TestTalkEventType;

class TestTalkClientEvent: public TestTalkMessage, public BaseEvent {

public:
	TestTalkClientEvent( long msgId,
                       const char* th,
                       const char* tc,
                       const char* fh,
                       const char* fc,
                       QType qt,
                       short mt,
                       short st,
                       void* d,
                       long  l) :
    TestTalkMessage(msgId, th, tc, fh, fc, qt, mt, st, d, l) {}

  TestTalkClientEvent(void* buf, long l, short t, short s_t) :
    TestTalkMessage(buf, l, t, s_t) {}

  TestTalkClientEvent(const char* cn,ClientObj& clnt, int op) :
    TestTalkMessage(0, "", cn, "", "", DataQ, 0, 0, (void*)"", 1),
    BaseEvent(clnt,op)
	{
	}

  virtual ~TestTalkClientEvent() {}
  
  virtual RVClassID isA() const
  {
    return (RVClassID)TestTalkClientEventType;
  }

  virtual bool isEqual(const RVCollectable* e) const
  {
    return (((TestTalkClientEvent*)e)->isA()==isA() && strcmp(((TestTalkClientEvent*)e)->getToClient(),(const char*)toClient)==0);
  }

	virtual unsigned hash() const
  {
		return RVCString(toClient).hash();
  }
  
  virtual char *nameOf() const
  {
    return ("TestTalkClientEvent");
  }
};


class TestTalkHostEvent: public TestTalkMessage, public BaseEvent {

public:
  
  TestTalkHostEvent( long msgId,
                     const char* th,
                     const char* tc,
		                 const char* fh,
                     const char* fc,
                     QType qt,                       
                     short mt,
                     short st,
                     void* d,
                     long  l) :
    TestTalkMessage(msgId, th, tc, fh, fc, qt, mt, st, d, l) {}
  
  TestTalkHostEvent(void* buf, long l, short t, short s_t) :
    TestTalkMessage(buf, l, t, s_t) {}

  TestTalkHostEvent(const char* thn, ClientObj& clnt, int op) : 
    TestTalkMessage(0, thn, "", "", "", DataQ, 0, 0, (void*)"", 1),
		BaseEvent(clnt,op)
	{
	}

  virtual ~TestTalkHostEvent() {}

  virtual RVClassID isA() const
  {
    return (RVClassID)TestTalkHostEventType;
  }

  virtual bool isEqual(const RVCollectable* e) const
  {
    return (((TestTalkHostEvent*)e)->isA()==isA() && strcmp(((TestTalkHostEvent*)e)->getToHost(),(const char*)toHost)==0);
  }

  virtual unsigned hash() const
  {
    return RVCString(toHost).hash();
  }

  virtual char *nameOf() const
  {
    return ("TestTalkHostEvent");
  }
};

#endif

