/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_NETKERNEL_H
#define __WL_NETKERNEL_H

#include "platform.h"
#include "tvpsync.h"
#include "associativearray.h"
#include "testtalkclientobj.h"
#include "netkernelmessagehandler.h"
#include "rfsservice.h"

#include "processgroupmanager.h"
#include <slistcolque.h>


class CTestTalkMain;
class EASTestTalkClient;


class CNetKernel : public CNetMessageHandler
{
private:
  long                seqNo;
  unsigned short      m_TestTalkPort;
	bool                m_IsTestTalkService; 

	RVThread            m_Thread;
	EASTestTalkClient  *m_EASPingClient;
	RFSService*         m_RFSService;

	ProcessesGroupManager    m_processesHashDictinary;	
	RVSlistCollectablesQueue m_queueKillRequest;

	int m_TimeBeforeForceKill;//in seconds
        int  m_TimeBeforeForceKillProductFactor;
	long m_KillerThreadWakeUpTimeOut; //in milliseconds
	
private:
  void GetDefaultValues();
	bool CreateLoaderProcess(CAssociativeArray& assoArrayCommand);
	void HandleProcessCreationFailure(CAssociativeArray& origCreationMessage, const char* filePath, int errorCode = 0);
#ifdef __TV_WINDOWS__
	void CreateReomteProcess(CAssociativeArray& assoArrayCommand);
#endif // __TV_WINDOWS__
	bool KillProcesses(RVCString& generatorUID);
	
public:
	CNetKernel(bool isTestTalkService = false);
	~CNetKernel();

  void  Start(unsigned short port);
  void Run();
  void Stop();

	virtual void SendMessageErrorHandler( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, bool status);
	virtual void MessageHandler( CAssociativeArray& AssoArrayCommand );
	virtual CNetHandlerMessage* SenderWaitMessage();

  void iAmClient(CAssociativeArray& Message);
  void CreateLoadGenerator(CAssociativeArray& Message, bool IsProtocol = false);
  void CreateWebLOAD50LoadGenerator(CAssociativeArray& Message);
  bool CreateLoadGeneratorSlave(CAssociativeArray& Message);
  bool CreateWebLOAD50LoadGeneratorSlave(CAssociativeArray& Message);

#ifdef __TV_WINDOWS__
  void CreateWebLOAD(CAssociativeArray& Message);
  void CreateWebFT(CAssociativeArray& Message);
#endif // __TV_WINDOWS__

	bool IsTestTalkService();

	bool TryToKillProcesses();
	int GetTimeBeforeForceKill() { return m_TimeBeforeForceKill; }
	long GetKillerThreadWakeUpTimeOut() { return m_KillerThreadWakeUpTimeOut; }
};

// *************** Exceptions ******************/

class DllExport CNetKernelException : public WLErrorException
{
public:
  // the format of copy ctor should be X(const X&) but since WebRM 4.6 uses the same dlls we cannot modify the interface
  // in 6.0 we should use X(const X&) for ALL platforms
#ifdef __TV_UNIX__
	CNetKernelException(const WLErrorException& error) : WLErrorException(error) {}
#else
	CNetKernelException(WLErrorException& error) : WLErrorException(error) {}
#endif //__TV_UNIX__
  virtual ~CNetKernelException() RV_EXCEPTION_THROW {}
};

class DllExport CNetInitFailedException : public CNetKernelException
{
public:
	CNetInitFailedException( const char* name ) : 
    CNetKernelException( WLErrorException("loadengine.frm", 32, name) )	{}
  virtual ~CNetInitFailedException() RV_EXCEPTION_THROW {}
};


#endif //__WL_NETKERNEL_H

