/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// TestTalkNetConnection.h : header file
//
#ifndef __TestTalkNetInstance_H
#define __TestTalkNetInstance_H

#include <RVCString.h>
#include <slistcolque.h>

#include "testtalkclass.h"
#include "streamsocket.h"
#include "tvpeventqueue.h"
#include "filetransferhandler.h"

#include "clntsrv.h"
#include "netevent.h"

class TestTalkClientInstance;
class CFileTransferHandler;

/****************** CTestTalkHostConnection Definition ******************/

class CTestTalkHostConnection : public ClientObj
{
	friend class CTestTalkEventServer;
private:
	RVCString             m_Name;
	RVThread              m_ReaderThread;
	RVThread              m_WriterThread;
	RVThread							m_FileReaderThread;
	RVThread							m_FileWriterThread;
	RVCriticalSection			m_FileWriterThreadSync;
	RVCriticalSection     m_Touch;
	TVPManualEvent        m_ConnectionDone;
	
	TVPEventQueue        *m_MsgsQueue;
	TVPEventQueue        *m_SendFilesQueue;
	TVPEventQueue        *m_ReceiveFilesQueue;
	StreamSocket          m_StreamSocket;
	bool                  m_IsStarted;
	bool                  m_Terminate;
	bool									m_FileWriterThreadRunning;
	bool									m_FileReaderThreadRunning;
	
	CTestTalkClass       *m_TestTalkMain;
	CFileTransferHandler *m_FileCopyHandler;
	
	double                m_LastMessageTime;
	
protected:  // protected member functions
	
	virtual int doneReadingMessage(void* msg, int len, short type, short subtype);
	
public:     // public memeber functions
	CTestTalkHostConnection(const char* host, CTestTalkClass *ttMain);
	CTestTalkHostConnection(SOCKET s, CTestTalkClass *ttMain);
	virtual ~CTestTalkHostConnection();
	
	void Init();
	virtual void Terminate(bool waitForThreads = true);
	
	void StartReader();
	void StartWriter();
	void StartFileReader();
	void StartFileWriter();
		
	void emptyMessagesQueue();
	void emptyFileMessagesQueue();

	int  doReading();
	int  doWriting();
	int  doFileReading();
	int  doFileWriting();

	int  doConnect();
	
	int  newMessage(BaseEvent& be);
	int  sendMessageToClient(TestTalkClientEvent* clientEvent);
	void putFileMessage(TestTalkHostEvent* hostEvent);

	virtual char *nameOf() const
	{
		return ("CTestTalkHostConnection");
	}
};

/****************** CTestTalkClientConnection Definition ******************/

class CTestTalkClientConnection : public ClientObj
{
private:
	RVCString           m_Name;
	RVThread            m_ReaderThread;
	RVThread            m_WriterThread;
	RVCriticalSection   m_Touch;
	
	TVPEventQueue      *m_MsgsQueue;
	StreamSocket        m_StreamSocket;
	bool                m_IsStarted;
	bool                m_Terminate;
	
	CTestTalkClass         *m_TestTalkMain;
	TestTalkClientInstance *clientInst;
	
protected:  // protected member functions
	
public:     // public memeber functions
	CTestTalkClientConnection(SOCKET s, CTestTalkClass *ttMain);
	virtual ~CTestTalkClientConnection();
	
	void Init();
	virtual void Terminate(bool waitForThreads = true);
	
	void StartReader();
	void StartWriter();
	
	int doReading();
	int doWriting();
	
	int newMessage(BaseEvent& be);
	
	virtual char *nameOf() const
	{
		return ("CTestTalkClientConnection");
	}
};


#endif

