/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// TestTalkRequester.h: interface for the CTestTalkNetRequester class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TESTTALKREQUESTER_H__548E62E3_7CBF_11D4_8838_0004AC15A422__INCLUDED_)
#define AFX_TESTTALKREQUESTER_H__548E62E3_7CBF_11D4_8838_0004AC15A422__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <iostream.h>
#include <time.h>
#include <RVCString.h>
#include <iomanip.h>
#include <fstream.h>

#include "tracedll.h"
#include "netkernelmessagehandler.h"
#include "testtalkclientobj.h"


class DllExport CTestTalkNetRequester: protected TestTalkClientObject
{
public:
	// Type of information to retrieve with ping
	enum InformationType{ WLVersion, SystemInfo, All };

	// Data structure to hold retrieved information
	struct CInformationObject
	{
		RVCString hostName;
		double    pingDuration;
		RVCString wlVersion;
		RVCString systemInfo;
		RVCString ipConfig;
		RVCString WebLoad_ini;
		RVCString Tvcti_dll;
	};

public:
	CTestTalkNetRequester(void);
	virtual ~CTestTalkNetRequester(void);

	// Initialize net configurations
	bool Init(/*[IN]*/const char* netName);

	// Terminate any net connections
	void Terminate(void);

	// Send simple ping to the specified host.
	// The method will return an information object if the remote
	// testtalk is alive and null else (you should delete the returned object).
	// Valid fields in the returned info object are: hostName and pingDuration.
	CTestTalkNetRequester::CInformationObject* TestTalkPing(/*[IN]*/const char* hostName);

	// Sends a ping with request for geting information and
	// waits until the information arrives.
	// Returns an information object if the remote testtalk is alive
	// and the request has succeeded and null else (you should delete the returned object).
	// Valid fields in the returned info object are: hostName and pingDuration in all cases.
	// More Valid fields: If type = Version - wlVersion
	//										If type = SystemInfo - SystemInfo, IPConfig, WebLoad_ini, Tvcti_dll
	//										If type = All - all fields are valid.
	CTestTalkNetRequester::CInformationObject* 
		TestTalkInformationPing(/*[IN]*/const char* hostName, 
														/*[IN]*/CTestTalkNetRequester::InformationType type);

	// Gets an information object and save it into reportFile.
	bool GenerateReportFile(/*[OUT]*/ofstream& reportFile, 
			  									/*[IN]*/CTestTalkNetRequester::CInformationObject* infoObject);
	
protected:
	// Sends any ping message to remote testtalk,
	// return true if sending was successfull false else
	virtual bool SendPing(/*[IN]*/const char* data, 
												/*[IN]*/size_t dataSize, 
												/*[IN]*/const char* hostName);
	
	inline char* WaitForPingReply(void);

	bool CreatePingMessage(/*[IN]*/const char* hostName,
												 /*[IN]*/CTestTalkNetRequester::InformationType type,
												 /*[OUT]*/CAssociativeArray& pingMessage);

	CTestTalkNetRequester::CInformationObject*
	CreatePingReplyInfoObject(/*[IN]*/CAssociativeArray& pingReply);

protected:
	unsigned long clientID;
};

#endif // !defined(AFX_TESTTALKREQUESTER_H__548E62E3_7CBF_11D4_8838_0004AC15A422__INCLUDED_)

