/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// UDPReceiver.h: interface for the UDPReceiver class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_UDPSERVER_H__7079ABC9_70F2_11D4_8831_0004AC15A422__INCLUDED_)
#define AFX_UDPSERVER_H__7079ABC9_70F2_11D4_8831_0004AC15A422__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "socketlib.h"
#include <thr.h>

#define NO_FLAGS_SET 0
#define MAXBUFLEN 256


class UDPReceiver  
{
public:
	UDPReceiver();
	virtual ~UDPReceiver();

	// When overriding the next methods 
	// call the current methods from the new ones
	// to ensure proper work (UDPReceiver::<Init | Start | Stop>
	virtual int  Init(u_short port);
	virtual int  Start(void);
	virtual void Stop(void);

	// Return true if Receiver is running (after invoking Start()).
	bool ReceiverStatusRunning(void);

	// Receives data from any client connecting to the receiver's
	// port and put it in buffer.
	int  ReceiveFrom(char* buffer, size_t bufferLen);

	// Return true if there is ready data to read from the socket,
	// and can wait for data if needed.
	int  SocketReadyForRead(long seconds = 0, long microseconds = 0);

	// Close net connection.
	void Terminate(void);

	// Implement MessageHandler to handle received messages.
	// The method is called every time a new message arrives.
	virtual void MessageHandler(const char* buffer);
	
protected:
	// Activated by calling Start().
	void ThreadLoop(void);

	// Called in a loop from ThreadLoop.
	virtual int Run(/*OUT*/char* buffer, /*IN*/int bufferLen);

protected:
	bool                m_Running;
	SOCKET						  m_RecvSocket;
	struct sockaddr_in  m_RecvSockAddr;
  RVThreadFunction    m_ReceiverThread;
	bool                m_Initialized;
};

#endif // !defined(AFX_UDPSERVER_H__7079ABC9_70F2_11D4_8831_0004AC15A422__INCLUDED_)

