/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// CEASTestTalkServer.cpp: implementation of the CEASTestTalkServer class.
//
//////////////////////////////////////////////////////////////////////

#include "ceastesttalkserver.h"
#include "eastesttalkserver.h"
#include "associativearray.h"
#include "tracedll.h"
#include "tvenv.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEASTestTalkServer::CEASTestTalkServer()
{
	m_Owner = NULL;
}

CEASTestTalkServer::CEASTestTalkServer(EASTestTalkServer* owner)
{
	m_Owner = owner;
}

CEASTestTalkServer::~CEASTestTalkServer()
{
	m_Owner = NULL;
}

int CEASTestTalkServer::Init(u_short serverPort)
{
	int initStatus;
	char localHostName[MAXBUFLEN]; 

	CSocketLib::GetLocalHostName(localHostName, MAXBUFLEN);
	m_EASTTServerName = localHostName;

	TVEnvContainerClass tvpc;
	const char* clientPort = tvpc.TVEGetEnv("UDP_CLIENT_PORT");
	if(!clientPort)
		return 0;

	initStatus = m_Sender.Init(atoi(clientPort));
	if(initStatus) {
		if(serverPort == 0) {
			tvpc.reset();
			const char* serverPort = tvpc.TVEGetEnv("UDP_SERVER_PORT");

			if(!serverPort)
				return 0;

			m_ServerPort = atoi(serverPort);
		}
		else {
			m_ServerPort = serverPort;
		}
		initStatus = UDPReceiver::Init(m_ServerPort);
	}
	return initStatus;
}

int CEASTestTalkServer::Start(void)
{
	// start the receiver thread 
	return UDPReceiver::Start();  
}

void CEASTestTalkServer::Stop(void)
{
	UDPReceiver::Stop();  
	m_Sender.Terminate();
}

int CEASTestTalkServer::InitPing(const char* replyHost,
																 long        replyPort,
																 const char* toHost,
																 long        duration)
{
	CAssociativeArray buffer;

	buffer.AddCouple("$RHOST", replyHost);
	buffer.AddCouple("$RPORT", replyPort);
	buffer.AddCouple("$TIME",  duration);
	buffer.AddCouple("$NAME",  toHost);

	return m_Sender.SendTo(toHost, buffer.ToString());
}

const char* CEASTestTalkServer::GetServerName(void)
{
	return m_EASTTServerName.data();
}

void CEASTestTalkServer::MessageHandler(const char* buffer)
{
	if(m_Owner) {
		m_Owner->OnPong(buffer);
	}
	else {
		OnPong(buffer);
	}
}

void CEASTestTalkServer::OnPong(const char* hostName) 
{
	cout<<hostName<<endl;
	WlTraceMsg("CEASTestTalkServer::OnPong - received: %s", hostName);
}
