/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// CFileTransferHandler.cpp: implementation of the CFileTransferHandler class.
//
//////////////////////////////////////////////////////////////////////

#ifdef __TV_WINDOWS__
#include "stdafx.h"
#endif
#include "tracedll.h"
#include "filetransferhandler.h"
#include "testtalktoolkit.h"
//Galtech
#include "gt_unicode.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFileTransferHandler::CFileTransferHandler(CTestTalkHostConnection *hostConnection)
:m_Transfer(mutex())
{
	WlTraceMsg("CFileTransferHandler::CFileTransferHandler - initializing.");

	m_Connection = hostConnection;
	m_Counter = NUMOF_CHUNKS_TO_WAIT_FOR;
	m_MaxCounterValue = NUMOF_CHUNKS_TO_WAIT_FOR;
	m_Send = m_Receive = m_ObjectTerminated = false;
}

CFileTransferHandler::~CFileTransferHandler(void)
{
	m_ObjectTerminated = true;
	m_FileChunksQueue.Terminate();
	m_Connection = NULL;
}


//////////////////////////////////////////////////////////////////////
// Private methodes implementation
//////////////////////////////////////////////////////////////////////

QueueWaitReturnType CFileTransferHandler::WaitFileChunkEvent(TestTalkHostEvent*& event)
{
	RVCollectable *hevent;
	QueueWaitReturnType type = m_FileChunksQueue.waitEvent(hevent, (long)60000);
	event = (TestTalkHostEvent *)hevent;
	return type;
}

void CFileTransferHandler::WaitLastChunkEventAck(void)
{
	LockGuard lock(monitor()); // acquire monitor mutex

	// wait
	while(m_Receive && m_Counter < m_MaxCounterValue) {
		m_Transfer.wait();
	}
}

void CFileTransferHandler::SignalSendingEvent(void)
{
	LockGuard lock(monitor()); // acquire monitor mutex

	m_Counter--;
	//WlTraceMsg("CFileTransferHandler::SignalSendingEvent(Counter = %d)", m_Counter);
	while(m_Send && !m_Counter) {
		m_Transfer.wait();
		//WlTraceMsg("CFileTransferHandler::SignalSendingEvent(Counter = %d, TransferStatus = %d), realesed.\n", m_Counter, m_TransferStatusOK);
	}
}

inline void CFileTransferHandler::SetNumOfExpectedEventAcks(int value)
{
	m_Counter = value;
	m_MaxCounterValue = value;
}

// @Method: SendFile
// @Param fileEvent: a request to m_Send a file. It should contain sourceFileName<src>
//					 and remoteFileName<dst>.
CFileTransferHandler::CompletionState
CFileTransferHandler::HandlerSendFile(const char* srcFilePath,
  																		const char* dstFilePath,
 																			TestTalkHostEvent *addressInfo)
{
	FILE *infile;
	m_Send = true;
	bool __TransferComplete = false;
	char buffer[FILE_CHUNK_SIZE + 1];
	char type = FILECOPY_CHUNK;

	ResetSender();
	//Galtech
	char ansiSrcFile[_MAX_PATH];
	UTF8ToANSI(GetACP(), srcFilePath,  -1, ansiSrcFile, _MAX_PATH);

	if( !(infile = fopen(ansiSrcFile, "rb")) )
		return OpenFailed;

	SetNumOfExpectedEventAcks(1);
	m_Connection->newMessage(*(TestTalkToolKit::CreateSendHostEvent(addressInfo, (void*)dstFilePath,
																							strlen(dstFilePath)+1, NV_FILE_TRANSFER)));
	SignalSendingEvent();
	SetNumOfExpectedEventAcks(NUMOF_CHUNKS_TO_WAIT_FOR);

	while(m_Send && !__TransferComplete) {

		size_t sizeRead = fread(buffer, sizeof(char), FILE_CHUNK_SIZE, infile);

		if(ferror(infile)) {
			break;
		}
		if(feof(infile)) {
			type = FILECOPY_LAST_CHUNK;
			__TransferComplete = true;
		}
		buffer[sizeRead] = type;
		m_Connection->newMessage(*(TestTalkToolKit::CreateSendHostEvent(addressInfo, buffer, sizeRead+1, NV_FILECOPY_EVENT)));

		SignalSendingEvent();
		if(__TransferComplete)
			WaitLastChunkEventAck();
	}/*while*/

	int closeStatus = fclose(infile);
	if(!__TransferComplete) { // An error occured before sending all file chunks.
		if(m_Send || m_ObjectTerminated) {
			m_Connection->newMessage(*(TestTalkToolKit::CreateSendHostEvent(addressInfo, (void *)STR_STOP_RECEIVER, 1, NV_FILECOPY_EVENT)));
			if(m_Send) {
				WaitForReceiver();
				return ReadFaild;
			}
			if(m_ObjectTerminated) {
				return CopyReleased;
			}
		}
	}

	WaitForReceiver();

	if(m_ReceiverStatus != CopyReleased && m_ReceiverStatus != CopySucceed)
		return CopyReleased;

	return ((closeStatus == 0)? CopySucceed:CloseFailed);
}

CFileTransferHandler::CompletionState
CFileTransferHandler::HandlerReceiveFile(const char* dstFilePath,
  																			 TestTalkHostEvent *addressInfo)
{
	FILE *outfile;
	m_Receive = true;
	TestTalkHostEvent* event = NULL;
	bool __TransferComplete = false;

	ResetReceiver();
	//Galtech
	char ansiDstFile[_MAX_PATH];
	UTF8ToANSI(GetACP(), dstFilePath,  -1, ansiDstFile, _MAX_PATH);

	if( !(outfile = fopen(ansiDstFile, "wb")) )
		return OpenFailed;

	m_Connection->newMessage(*(TestTalkToolKit::CreateReturnHostEvent(addressInfo,
  													 (void *)STR_FILECOPY_RECEIVER_READY, 1, NV_FILECOPY_EVENT)));

	while(m_Receive && !__TransferComplete) {

		QueueWaitReturnType queueReturnType = WaitFileChunkEvent(event);

		if(event && queueReturnType == TVPEventQueue::Success) {

			fwrite(event->getData(), sizeof(char), event->getDatalen()-1, outfile);

			if(ferror(outfile)) {
				delete event;
				break;
			}

			if(((char*)event->getData())[event->getDatalen()-1] == FILECOPY_LAST_CHUNK)
				__TransferComplete = true;

			m_Connection->newMessage(*(TestTalkToolKit::CreateReturnHostEvent(event,
															   (void *)STR_FILECOPY_CHUNK_ACK, 1, NV_FILECOPY_EVENT)));
			delete event;
			event = NULL;
		}
		else
			break;
	} /* while */

	int closeStatus = fclose(outfile);

	// An error occured before writing all file chunks.
	if(!__TransferComplete || !m_Receive) {
		m_FileChunksQueue.Terminate();
		TestTalkToolKit::DeleteFile(ansiDstFile);
		if(!m_Receive)
			return CopyReleased;
		else
			return WriteFailed;
	}
	return ((closeStatus == 0)? CopySucceed:CloseFailed);
}


//////////////////////////////////////////////////////////////////////
// Public methodes implementation
//////////////////////////////////////////////////////////////////////

CFileTransferHandler::CompletionInfo*
	CFileTransferHandler::SendFile(const char* srcFilePath,
												  			 const char* dstFilePath,
																 TestTalkHostEvent* addressInfo)
{
	WlTraceMsg("CFileTransferHandler::SendFile(dstFilePath=%s) - starting", dstFilePath); //@@eyal: trace

	CompletionState status = HandlerSendFile(srcFilePath, dstFilePath, addressInfo);

	CompletionInfo* copyStatus = new CompletionInfo();

	if(status != CopyReleased) {
		copyStatus->m_CompletionState = status;
		copyStatus->m_FileName = srcFilePath;
	}
	else {
		copyStatus->m_CompletionState = m_ReceiverStatus;
		copyStatus->m_FileName = dstFilePath;
	}

	WlTraceMsg("CFileTransferHandler::SendFile - done"); //@@eyal: trace
	return copyStatus;
}

CFileTransferHandler::CompletionState
	CFileTransferHandler::ReceiveFile(const char* dstFilePath,
												  					TestTalkHostEvent* addressInfo)
{
	WlTraceMsg("CFileTransferHandler::ReceiveFile(dstFilePath=%s) - starting", dstFilePath); //@@eyal: trace

	CompletionState status = HandlerReceiveFile(dstFilePath, addressInfo);

	char   buffer[20];
	size_t bufferSize;

	itoa(status, buffer, 10);
	bufferSize = strlen(buffer);

	buffer[bufferSize] = STOP_SENDER;

	m_Connection->newMessage(*(TestTalkToolKit::CreateReturnHostEvent(addressInfo, buffer, bufferSize+1, NV_FILECOPY_EVENT)));

	WlTraceMsg("CFileTransferHandler::ReceiveFile - done"); //@@eyal: trace

	return status;
}

void CFileTransferHandler::PutFileChunkEvent(TestTalkHostEvent *fileEvent)
{
	WlTraceMsg("CFileTransferHandler::PutFileChunkEvent"); //@@eyal: trace

	m_FileChunksQueue.putEvent(fileEvent);
}

void CFileTransferHandler::SetFileChunkEventAck(void)
{
	LockGuard lock(monitor()); // acquire monitor mutex

	m_Counter++;
	m_Transfer.signal();

	WlTraceMsg("CFileTransferHandler::SetFileChunkEventAck(Counter=%d)", m_Counter); //@@eyal: trace
}

void CFileTransferHandler::HandleFileEvent(void* eventMsg,
																					 int len,
																					 short ttType,
																					 short ttSubtype)
{
	WlTraceMsg("CFileTransferHandler::HandleFileEvent(type=%d)", ttType); //@@eyal: trace

	if(!m_Send && !m_Receive) {
		return;
	}
	TestTalkHostEvent message(eventMsg, len, ttType, ttSubtype);

	char type = ((char*)message.getData())[message.getDatalen() - 1];

	switch (type)
	{
		case FILECOPY_CHUNK:
		case FILECOPY_LAST_CHUNK:
		{
			PutFileChunkEvent(new TestTalkHostEvent(eventMsg, len, ttType, ttSubtype));
		}
		break;

		case FILECOPY_CHUNK_ACK:
		case FILECOPY_RECEIVER_READY:
		{
			SetFileChunkEventAck();
		}
		break;

		case STOP_RECEIVER:
		case STOP_SENDER:
		{
			if(type == STOP_RECEIVER) {
				StopReceiver();
			}
			else if(type == STOP_SENDER) {
				char* event = (char*)message.getData();
				event[message.getDatalen() - 1] = '\0';
				StopSender(CompletionState(atoi(event)));
			}
		}
		break;
	}
}

void CFileTransferHandler::StopSender(CompletionState status)
{
	LockGuard lock(monitor()); // acquire monitor mutex
	m_Send = false;
	m_Counter = NUMOF_CHUNKS_TO_WAIT_FOR;
	m_ReceiverStatus = status;
	m_Transfer.signalAll();
}

void CFileTransferHandler::StopReceiver(void)
{
	m_Receive = false;
	m_FileChunksQueue.Terminate();
	m_FileChunksQueue.release();
}

void CFileTransferHandler::ResetSender(void)
{
	m_Counter = NUMOF_CHUNKS_TO_WAIT_FOR;
	m_MaxCounterValue = NUMOF_CHUNKS_TO_WAIT_FOR;
}

void CFileTransferHandler::ResetReceiver(void)
{
	m_FileChunksQueue.Terminate();
	m_FileChunksQueue.Init();
}

void CFileTransferHandler::WaitForReceiver(void)
{
	while(m_Send) {
		LockGuard lock(monitor());
		m_Transfer.wait(1000);
	}
}

void CFileTransferHandler::Terminate(void)
{
	static bool inTerminate = false;

	m_ObjectTerminated = true;
	{
		LockGuard lock(monitor()); // acquire monitor mutex

		if(inTerminate)
			return;
		inTerminate = true;
	}

	StopSender(CopyReleased);
	StopReceiver();
	m_FileChunksQueue.Terminate();
	inTerminate = false;
}

