/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <RVCString.h>
#include <collstr.h>

#include "platform.h"
#include "tracedll.h"

#include "tvpsync.h"
#include "netmsg.h"
#include "tvenv.h"
#include "gt_unidef.h"
#include "gt_portable.h"

#include "nvspackmsg.h"
#include "testtalkdefs.h"
#include "associativearray.h"
#include "messagehandlerimp.h"


///////////////////  CNetHandlerMessage decleration ////////////////////

static long MessageHandlerId = 0;

///////////////////  CNetMessageTTObject implementation ////////////////////

void CNetMessageTTObject::SetMessageHandler(CNetMessageHandlerImp* nmh)
{
  m_MyNetMessageHandler = nmh;
}

bool CNetMessageTTObject::NewMessageEvent(TypedMessageObject* tmo, short qtype)
{
  switch (qtype)
  {
    case DataQ:
    case ControlQ:
      return true;

    case AckQ:
      {
        TestTalkMessage* ttm = NVSTmo2Ttm(tmo);
        m_MyNetMessageHandler->HandleAck(ttm);
        WlTraceMsg("CNetMessageTTObject::NewMessageEvent - Recieved Ack for message ID %d", ttm->getMessageId());
        //m_MyNetMessageHandler->HandleEvent((const char*)ttm->getData());
        delete ttm;
      }
      return false;

    default:
      return false;
  }

  return false;
}


///////////////////  CNetMessageHandlerImp implementation ////////////////////

CNetMessageHandlerImp::CNetMessageHandlerImp(CNetMessageHandler* owner) :
  m_Owner(owner),
  m_ErrorMsg(""),
  m_Initialized(false),
	m_UseCallbacksModel(true)
{
	char pid[20];
#ifdef __TV_WINDOWS__
	sprintf(pid,"%d",_getpid());
#else
	sprintf(pid,"%d", getpid());
#endif
	m_Owner->m_SenderName += pid;

  id = ++MessageHandlerId;
  char cid[20];
  sprintf(cid,"%d",id);
  m_Owner->m_SenderName += "Sender";
  m_Owner->m_SenderName += cid;

  m_LastRecievedMessageTime = 0;

  m_SenderThread = NULL;
  m_RecieverThread = NULL;

  m_TestTalkClient.SetMessageHandler(this);
  m_MessageCounter = 0;

	m_MessageListeneres = NULL;
}

CNetMessageHandlerImp::~CNetMessageHandlerImp()
{
  Terminate();
  if (m_SenderThread)
    delete m_SenderThread;
  if (m_RecieverThread)
    delete m_RecieverThread;
	if (m_MessageListeneres)
		delete [] m_MessageListeneres;
}

void CNetMessageHandlerImp::UseCallbacksModel(bool callbacksModel)
{
	m_UseCallbacksModel = callbacksModel;
}

void CNetMessageHandlerImp::AddMessagesListener(INetMessagesListener* listener)
{
	//
	// currently supports only one listener
	if( m_MessageListeneres == NULL )
	{
		m_MessageListeneres = new INetMessagesListener*[1];
	}

	m_MessageListeneres[0] = listener;
}

void CNetMessageHandlerImp::RemoveMessagesListener(INetMessagesListener *listener)
{
	if( m_MessageListeneres )
	{
		m_MessageListeneres[0] = NULL;
	}
}

bool CNetMessageHandlerImp::Init(unsigned short testtalkport)
{
  //WlTraceMsg("CNetMessageHandlerImp::Init(%s) : Starting.",m_TestTalkClient.NETWhoAmI());
  //
  m_TestTalkPort = testtalkport;
  // if alread Initialized, terminate first.
  if (m_Initialized)
    Terminate();

  //
  // create the sender thread functor:
  if (m_SenderThread)
    delete m_SenderThread;
  m_SenderThread = new RVThread;
  if (m_RecieverThread)
    delete m_RecieverThread;
  m_RecieverThread = new RVThread;
  #if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)
    *m_SenderThread = rvMakeThreadFunction(
                        *this,                                // instance
                        &CNetMessageHandlerImp::StartSender      // member function
                      );
    *m_RecieverThread = rvMakeThreadFunction(
                        *this,                                // instance
                        &CNetMessageHandlerImp::StartReciever      // member function
                      );
  #else
    *m_SenderThread = rvMakeThreadFunctionM(
                        CNetMessageHandlerImp                    // class
                        *this,                                // instance
                        void,                                 // return value
                        &CNetMessageHandlerImp::StartSender      // member function
                      );
    *m_RecieverThread = rvMakeThreadFunctionM(
                        CNetMessageHandlerImp                    // class
                        *this,                                // instance
                        void,                                 // return value
                        &CNetMessageHandlerImp::StartReciever      // member function
                      );
  #endif

  m_Initialized = true;
  m_MessagesQueue.Init();
  //
  // Start the sender thread.
  m_SenderStartEvent.Reset();
  m_SenderDoneEvent.Reset();
  m_SenderThread->start();
  m_SenderStartEvent.Wait();

	if( m_UseCallbacksModel )
	{
  //
  // Start the receiver thread.
  m_RecieverStartEvent.Reset();
  m_RecieverDoneEvent.Reset();
  m_RecieverThread->start();
  m_RecieverStartEvent.Wait();
	}

  m_LastRecievedMessageTime = TVPSysTime();

	bool recieverDoneEvent = false;

	if( m_UseCallbacksModel )
	{
		recieverDoneEvent = m_RecieverDoneEvent.Get();
	}

  if (!m_SenderDoneEvent.Get() && !recieverDoneEvent )
	{
    //WlTraceMsg("CNetMessageHandlerImp::Init(%s) : Done successfully.",m_TestTalkClient.NETWhoAmI());
    return true;
  } 
	else
	{
    Terminate();
    WlTraceMsg("CNetMessageHandlerImp::Init(%s) : Done failure.",m_TestTalkClient.NETWhoAmI());
    return false;
  }
}

void CNetMessageHandlerImp::Terminate()
{
  //WlTraceMsg("CNetMessageHandlerImp::Terminate(%s) : Starting.",m_TestTalkClient.NETWhoAmI());
  if (!m_Initialized)
    return;

  m_Initialized = false;

  //
  // Make sure there are no more messages,
  // than terminate the messages queue, and wait for the sender to terminate
  TVPTimerObject timeout;
  while (!IsSendQueueEmpty() && timeout.get() < 10)
    ::rvSleep(10);

  m_MessagesQueue.Terminate();

	//
	// Tedrminate the sender
  int st = m_SenderThread->getExecutionState();
  
  if ( m_SenderThread->getExecutionState() != RV_THR_INITIAL && 
	   m_SenderThread->join(8000) == RV_THR_TIMEOUT ) 
	{
    try 
		{
      m_SenderThread->terminate();
    }
		catch (...) {}
  }

  //
  // Terminate the Networking
  m_TestTalkClient.NETTerminate();

	//
	// Tedrminate the receiver
  if (m_UseCallbacksModel && m_RecieverThread->join(5000) == RV_THR_TIMEOUT) 
	{
    try 
		{
      m_RecieverThread->terminate();
    } 
		catch (...) {}
  }

  //WlTraceMsg("CNetMessageHandlerImp::Terminate(%s) : Done.",m_TestTalkClient.NETWhoAmI());
}

void CNetMessageHandlerImp::SendMessage( CNetHandlerMessage::SendType sendtype, CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, int priority)
{
	char*  message;
	size_t messageLength;

  if(!Message.KeyExists("$FROM"))
    Message.AddCouple("$FROM", m_TestTalkClient.NETWhoAmI());

	Message.ToString(message, messageLength);

	//Galtech
	RVCString convertedMessage;
	if (!_strnicmp(m_TestTalkClient.NETWhoAmI(), "GeneratorProxy", strlen("GeneratorProxy")) ||
		!_strnicmp(m_TestTalkClient.NETWhoAmI(), "theTmlProxy", strlen("theTmlProxy")))
	{
		CAssociativeArray::Convert(CP_ACP, CP_UTF8, message, convertedMessage);
		message = (char*)convertedMessage.data();
		messageLength = convertedMessage.length();
	}

  CNetHandlerMessage *mh = new CNetHandlerMessage(message, messageLength, SendTo, ErrorMsg);
  mh->m_Priority = priority;
  mh->m_Id = ++m_MessageCounter;
  mh->m_SendType = sendtype;
  mh->value(m_MessageCounter);
	m_MessagesQueue.putEvent(mh, priority);
}


void CNetMessageHandlerImp::StartSender()
{
	CTraceMsg::RegisterThread ();
  WlTraceMsg("CNetMessageHandlerImp::StartSender(%s) - Started",m_TestTalkClient.NETWhoAmI());
  RVThreadSelf self = ::rvThread();
  self.setPriority(self.getMaxPriority());
  //TVPSysThread::setPriority(TVPSysThread::Critical);

  TVPManualEvent::SetGuard event(m_SenderDoneEvent);
  try {
    SenderMainLoop();
  } catch (...) {
    WlTraceMsg("Fatal Error in CNetMessageHandlerImp::StartSender(%s)",m_TestTalkClient.NETWhoAmI());
    #ifdef _DEBUG
    throw;
    #endif
  }
  //WlTraceMsg("CNetMessageHandlerImp::StartSender(%s) - Done", m_TestTalkClient.NETWhoAmI());
  return;
}

void CNetMessageHandlerImp::StartReciever()
{
	CTraceMsg::RegisterThread ();
  WlTraceMsg("CNetMessageHandlerImp::StartReciever(%s) - Started",m_TestTalkClient.NETWhoAmI());
  RVThreadSelf self = ::rvThread();
  self.setPriority(self.getMaxPriority());

  TVPManualEvent::SetGuard event(m_RecieverDoneEvent);
  m_RecieverStartEvent.Set();
  try {
    const char* message;
    while ((message = m_TestTalkClient.NETWaitMessage()) != NULL) {
      m_LastRecievedMessageTime = TVPSysTime();
      HandleEvent(message);
    }
  } catch (...) {
    WlTraceMsg("Fatal Error in CNetMessageHandlerImp::StartReciever(%s)",m_TestTalkClient.NETWhoAmI());
    #ifdef _DEBUG
    throw;
    #endif
  }
  WlTraceMsg("CNetMessageHandlerImp::StartReciever(%s) - Done",m_TestTalkClient.NETWhoAmI());
  return;
}

void CNetMessageHandlerImp::SenderMainLoop()
{
  //WlTraceMsg("CNetMessageHandlerImp::SenderMainLoop(%s) : started", m_TestTalkClient.NETWhoAmI());
  //WlTraceMsg("CNetMessageHandlerImp::SenderMainLoop : Initializing %s", m_Owner->m_SenderName.data());
  if (!m_TestTalkClient.NETInit(m_Owner->m_SenderName, m_TestTalkPort)) {
    WlTraceMsg("CNetMessageHandlerImp::SenderMainLoop Error initializing sender %s.", m_Owner->m_SenderName.data());
    m_SenderStartEvent.Set();
    return;
  }
  m_SenderStartEvent.Set();

  bool  status = false;
	int   retryCounter  = 0;
	int   maxRetryTimes = 1;
	long  timeIntervals = 0;
	CNetHandlerMessage* pMessage;

	TVEnvContainerClass tvpc;
	const char* retry = tvpc.TVEGetEnv("TESTTALK_INTERNAL_SEND_RETRY_TIMES");
	if(retry != NULL) {
		maxRetryTimes = atoi(retry);
	}
  tvpc.reset();
	const char* intervals = tvpc.TVEGetEnv("TESTTALK_INTERNAL_SEND_TIME_INTERVALS");
	if(intervals != NULL) {
		timeIntervals = atoi(intervals);
	}

  while ((pMessage = m_Owner->SenderWaitMessage()) != NULL)
  {
    //WlTraceMsg("CNetMessageHandlerImp::SenderMainLoop(%s) - Sending message (to:%s, %s)", m_TestTalkClient.NETWhoAmI(), pMessage->GetRecipient(), pMessage->GetMessage());
		status = false;
		retryCounter  = 0;

		while( !status && retryCounter < maxRetryTimes ) {
			switch (pMessage->GetSendType()) {
				case CNetHandlerMessage::Wait:
					status = m_TestTalkClient.NETSendMessage(pMessage->GetMessage(), pMessage->GetMessageLength(), pMessage->GetRecipient());
					break;
				case CNetHandlerMessage::NoResponse:
					status = m_TestTalkClient.NETSendMessageNoresponse(pMessage->GetMessage(), pMessage->GetMessageLength(), pMessage->GetRecipient());
					break;
				case CNetHandlerMessage::NoWait:
					status = m_TestTalkClient.NETSendMessageNowait(pMessage->GetId(), pMessage->GetMessage(), pMessage->GetMessageLength(), pMessage->GetRecipient());
					break;
				case CNetHandlerMessage::FileCopy:
					status = m_TestTalkClient.NETCopyFile(pMessage->GetId(), pMessage->GetMessage(), pMessage->GetMessageLength(), pMessage->GetRecipient());
					break;
			}
			if(!status && retryCounter > 0) {
				WlTraceMsg("CNetMessageHandlerImp::SenderMainLoop(%s) - Sending message (to:%s) failed, retry %d", m_TestTalkClient.NETWhoAmI(), pMessage->GetRecipient(), retryCounter);
				// sergei - 3/28/01
				#ifdef __tv_UNIX__
					rvSleep(timeIntervals);
				#else
					Sleep(timeIntervals);
				#endif
			}
			retryCounter++;
		}
    //WlTraceMsg("CNetMessageHandlerImp::SenderMainLoop(%s) - Sending message (ret:%d)", m_TestTalkClient.NETWhoAmI(), (int)status);
    if (!status) 
		{
 	    HandleSendMessageError(pMessage);
      delete pMessage;
    } 
		else if (pMessage->GetSendType() != CNetHandlerMessage::NoWait && pMessage->GetSendType() != CNetHandlerMessage::FileCopy)
		{
      m_Owner->SendMessageSucceded(pMessage->GetMessage(), pMessage->GetRecipient());
      delete pMessage;
    }
		else 
		{
      m_SentMessages.putEvent(pMessage);
    }
  }
}

void CNetMessageHandlerImp::HandleSendMessageError(CNetHandlerMessage* message, bool status)
{
	CAssociativeArray	origMessage;
  origMessage.FromString(message->GetMessage());

	if(message->GetSendType() == CNetHandlerMessage::FileCopy) {
		m_Owner->CopyFileDone(CNetMessageHandler::CopyFileFailed,
													origMessage.GetValueOfParameter("$LOCALFILE"),
													origMessage.GetValueOfParameter("$LOCALFILE"),
													origMessage.GetValueOfParameter("$REMOTEFILE"),
													message->GetRecipient());
	}

	WlTraceMsg("CNetMessageHandlerImp::HandleSendMessageError, message to %s failed", message->GetRecipient());

	m_LastSendError  = message->GetErrorMsg();
	m_LastSendError += ", Original Message = ";
  m_LastSendError += origMessage.ToString();

  m_Owner->SendMessageErrorHandler(origMessage, message->GetRecipient(), message->GetErrorMsg(), status);
}

void CNetMessageHandlerImp::HandleAck(TestTalkMessage* ttm)
{
  CNetHandlerMessage* pMessage = NULL;
  while (!m_SentMessages.isEmpty()) {
    pMessage = (CNetHandlerMessage*)m_SentMessages.get();
    if (pMessage->GetId() == ttm->getMessageId())
      break;
    delete pMessage;
    pMessage = NULL;
  }
  if (pMessage) {
		if (ttm->getType() == NV_SENDMESSAGE_STATUS_FAIL) {
      WlTraceMsg("CNetMessageHandlerImp::HandleAck - Message id %d Failed",pMessage->GetId());
 	    HandleSendMessageError(pMessage);
    } else {
      WlTraceMsg("CNetMessageHandlerImp::HandleAck - Message id %d Succeeded",pMessage->GetId());

			switch(pMessage->GetSendType())
			{
				case CNetHandlerMessage::FileCopy:
				{
					CAssociativeArray	fileCopyAck, origMessage;
					fileCopyAck.FromString((char*)ttm->getData());
					origMessage.FromString(pMessage->GetMessage());

					long copyStatus = fileCopyAck.GetLongValueOfParameter("$FILECOPYCOMPLETIONSTATE");
					const char* copyStatusFileName = fileCopyAck.GetValueOfParameter("$FILENAME");
					m_Owner->CopyFileDone(CNetMessageHandler::CopyFileCompletionState(copyStatus),
																copyStatusFileName,
																origMessage.GetValueOfParameter("$LOCALFILE"),
																origMessage.GetValueOfParameter("$REMOTEFILE"),
																pMessage->GetRecipient());

					if( m_MessageListeneres )
					{
						m_MessageListeneres[0]->CopyFileDoneDispatched(copyStatus,
																														 copyStatusFileName,
																														 origMessage.GetValueOfParameter("$LOCALFILE"),
																														 origMessage.GetValueOfParameter("$REMOTEFILE"),
																														 pMessage->GetRecipient(),
																														 fileCopyAck);
					}
				}
				break;

				default:
				{
					m_Owner->SendMessageSucceded(pMessage->GetMessage(), pMessage->GetRecipient());
					break;
				}
			}
    }
    delete pMessage;
  }
}

void CNetMessageHandlerImp::HandleEvent(const char* AssoArrayString)
{
	try
	{
	  CAssociativeArray	AssoArrayCommand;
		if( AssoArrayString == NULL )
			return;

    //WlTraceMsg("CNetMessageHandlerImp::HandleEvent: Received command (%s)",(const char*)AssoArrayString);

		//Galtech
		RVCString convertedMessage;
		if (!_strnicmp(m_TestTalkClient.NETWhoAmI(), "GeneratorProxy", strlen("GeneratorProxy")) ||
			!_strnicmp(m_TestTalkClient.NETWhoAmI(), "theTmlProxy", strlen("theTmlProxy")))
		{
			CAssociativeArray::Convert(CP_UTF8, CP_ACP, AssoArrayString, convertedMessage);
			AssoArrayString = (char*)convertedMessage.data();
		}

		AssoArrayCommand.FromString( AssoArrayString );

    m_Owner->MessageHandler(AssoArrayCommand);

		if( m_MessageListeneres )
		{
			WlTraceMsg("CNetMessageHandlerImp::HandleEvent: command dispatched to listener(%d)", m_MessageListeneres[0]);
			m_MessageListeneres[0]->MessageDispatched(AssoArrayCommand);
		}


  } catch (WLErrorException e) {
    WlTraceMsg("CNetMessageHandlerImp::HandleEvent: Exception: %s",e.GetErrorMsg());
    //
    //return;
  } catch (...) {
    WlTraceMsg("CNetMessageHandlerImp::HandleEvent: Unhandled exception.");
    #ifdef _DEBUG
    throw;
    #endif
  }
}

bool CNetMessageHandlerImp::ReadNextMessage(/*[OUT]*/CAssociativeArray& message)
{
	if( m_UseCallbacksModel )
	{
		return false;
	}

  try 
	{
    const char* newMessage = NULL;

    newMessage = m_TestTalkClient.NETWaitMessage();
    m_LastRecievedMessageTime = TVPSysTime();

		if( newMessage )
		{
			//Galtech - Eyal
			RVCString convertedMessage;
			if (!_strnicmp(m_TestTalkClient.NETWhoAmI(), "GeneratorProxy", strlen("GeneratorProxy")) ||
				!_strnicmp(m_TestTalkClient.NETWhoAmI(), "theTmlProxy", strlen("theTmlProxy")))
			{
				CAssociativeArray::Convert(CP_UTF8, CP_ACP, newMessage, convertedMessage);
				newMessage = (char*)convertedMessage.data();
			}

			message.FromString( newMessage );
			return true;
		}
  }
  catch (WLErrorException e) 
	{
    WlTraceMsg("CNetMessageHandlerImp::ReadNextMessage: Exception: %s",e.GetErrorMsg());
	}
	catch (...) 
	{
    WlTraceMsg("Fatal Error in CNetMessageHandlerImp::ReadNextMessage(%s)",m_TestTalkClient.NETWhoAmI());
    #ifdef _DEBUG
    throw;
    #endif
  }

	return false;
}

const char* CNetMessageHandlerImp::GetLastErrorMessage()
{
	return m_LastSendError.data(); 
}

