/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// rfsservice.cpp: implementation of the RFSService class.
//
//////////////////////////////////////////////////////////////////////

#include "rfsservice.h"
#include "tvenv.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

RFSService::RFSService(CNetMessageHandler* connection)
:RFS(connection)
{

}

RFSService::~RFSService()
{

}


//////////////////////////////////////////////////////////////////////
// Methods impl
//////////////////////////////////////////////////////////////////////

void RFSService::ProcessRFSRequest(CAssociativeArray& message, RVCString& symbol)
{
	bool bRet = false;
	RVCString exceptionMsg;
	CAssociativeArray retValue;
	const char* initiatorRemoteHost = message.GetValueOfParameter("$FROM");

	try
	{
		if( symbol.compareTo("$GETDIRPATH") == 0 )
		{
			const char* dirPath = NULL;

			if( message.KeyExists("$GETTVTOP") )
			{	
				TVEnvContainerClass tvpc;
				dirPath = tvpc.TVEGetEnv("TV_TOP");
			}
			else if( message.KeyExists("$GETTMPDIR") )
			{	
				dirPath = TVPGetTempDirectory();
			}
			else if( message.KeyExists("$GETUSRDIR") )
			{		
				TVEnvContainerClass tvpc;
				dirPath = tvpc.TVPGetUserDir();
			}

			if( dirPath )
			{
				bRet = true;
				retValue.AddCouple("$RETDIRPATH", dirPath);
			}
			else
			{
				exceptionMsg = "could not find the specified directory";
				retValue.AddCouple("$RETDIRPATH", "null");
			}
		}
		else if( symbol.compareTo("$CREATEDIR") == 0 )
		{
			TVEnvContainerClass tvpc;
			const char* dirPath = message.GetValueOfParameter("$DIRPATH");

			if( message.GetLongValueOfParameter("$DELOLDDIR") == 1 )
			{
				int retry = 10;

				while( tvpc.TVPFileExist(dirPath) && retry-- > 0 )
				{
					TVPRemoveAllDirectories(dirPath);
				}

				tvpc.reset();
			}

			bRet = TVPCreateDirectory(dirPath);

			if( !bRet && tvpc.TVPFileExist(dirPath) )
			{
				bRet = true;
			}
				
			retValue.AddCouple("$CREATEDIRSTATUS", long(bRet));
		}
		else if( symbol.compareTo("$DELETEFILE") == 0 )
		{
			bRet = true;

			if( message.GetLongValueOfParameter("$ISDIR") == 1 )
			{
				TVPRemoveAllDirectories(message.GetValueOfParameter("$FILEPATH"));
			}
			else
			{
				if( remove(message.GetValueOfParameter("$FILEPATH")) != 0 ) 
				{
					bRet =  false;
				}
			}
				
			retValue.AddCouple("$DELETEFILESTATUS", long(bRet));
		}
		else if( symbol.compareTo("$GETARCHIVE") == 0 )
		{
			bRet = true;

			try
			{
				RVCString address;
				RVCString resultDir;
				FilesAssociationArchive archive;

				archive.FromString( message.GetValueOfParameter("$ARCHIVE") );	
				mConnection->ParseHostName(address, message.GetValueOfParameter("$FROM"), false);

				if( !SendArchive(address, &archive) )
				{
					bRet = false;
				}
			}
			catch(RFSException& exception)
			{
				bRet = false;
				exceptionMsg = exception.GetMessage();
			}

			retValue.AddCouple("$GETARCHIVESTATUS", long(bRet));
			retValue.AddCouple("$RESULTPATH", "");
		}
	}
	catch(...)
	{
		bRet = false;
		exceptionMsg = "unknown exception in RFSService.ProcessRFSRequest";
	}

	retValue.AddCouple("$BRETVAL", long(bRet));
	retValue.AddCouple("$EXCEPTIONMSG", exceptionMsg);

	SendRFSResponse(initiatorRemoteHost, "", &retValue);
}

void RFSService::SendRFSResponse(const char* remoteHost, const char* responseSymbol, CAssociativeArray* sendMessage)
{
	RVCString error(RVSize_T(40));

	error += "RFS response error, symbol: ";
	error += responseSymbol;

	sendMessage->AddCouple("$RFSResponse", responseSymbol);

	mConnection->SendMessage(*sendMessage, remoteHost, error);
}
