/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "testtalkeventserver.h"
#include "testtalklistensockets.h"
#include "testtalkclass.h"
#include "testtalktoolkit.h"


CTestTalkClass::CTestTalkClass()
{
  m_EventServer = new CTestTalkEventServer(this);    
  m_NetListenSock = new CTestTalkNetListener(this);    
  m_ClientsListenSock = new CTestTalkClientListener(this);
}

CTestTalkClass::~CTestTalkClass()
{
  delete m_EventServer;
  delete m_NetListenSock;
  delete m_ClientsListenSock;
}

bool CTestTalkClass::Init()
{
  //WlTraceMsg("Starting CTestTalkClass::Init");

	char szName[128] = "host";
  struct in_addr sin_addr;
  CSocketLib::GetLocalHostName(szName, 128);
	m_LocalHostName = szName;
  CSocketLib::GetHostByName(szName, sin_addr);
	m_LocalHostAddress = inet_ntoa(sin_addr);

  WlTraceMsg("CTestTalkClass::Init - host: '%s', Address '%s'",m_LocalHostName.data(), m_LocalHostAddress.data());
  m_NetListenSock->SetPort(m_tcpNetPort);

  if (!m_NetListenSock->Activate())
	{
		ReportPortBusyError(m_tcpNetPort);
    return false;
	}
  //WlTraceMsg("CTestTalkClass::Init Opened net port %d", (int)m_tcpNetPort);

  m_ClientsListenSock->SetPort(m_tcpClientsPort);
  if (!m_ClientsListenSock->Activate())
	{
		ReportPortBusyError(m_tcpClientsPort);
    return false;
	}

	MarkTestTalkAlive();
  //WlTraceMsg("CTestTalkClass::Init Opened clients port %d", (int)m_tcpClientsPort);

  return true;
}

void CTestTalkClass::Terminate()
{
  //WlTraceMsg("Starting CTestTalkClass::Terminate");
  //
  // Terminate listen sockets.:
  //WlTraceMsg("CTestTalkClass::Terminate - Closing net listen socket");
  m_NetListenSock->Terminate();
  //WlTraceMsg("CTestTalkClass::Terminate - Closing Clients listen socket");
  m_ClientsListenSock->Terminate();

	//
	// clean up the event server
	m_EventServer->cleanUp();

  //
  // Wait for listen threads to terminate.
  m_NetListenSock->m_ListenThread.join(5000);
  m_ClientsListenSock->m_ListenThread.join(1000);

  //WlTraceMsg("CTestTalkClass::Terminate - Done");
}

void CTestTalkClass::MarkTestTalkAlive()
{
#ifndef __TV_UNIX__
	HANDLE hMutex;
	if( TestTalkToolKit::GetLocalHostOS() == "Win2000" )
    hMutex = CreateMutex( NULL, false, "Global\\WLTestTalk" );
	else
		hMutex = CreateMutex( NULL, false, "WLTestTalk" );
#endif
}

bool CTestTalkClass::IsOtherTestTalkAlive()
{
#ifndef __TV_UNIX__
	HANDLE hMutex;
	if( TestTalkToolKit::GetLocalHostOS() == "Win2000" )
	{
    hMutex = CreateMutex( NULL, false, "Global\\WLTestTalk" );
	}
	else
	{
		hMutex = CreateMutex( NULL, false, "WLTestTalk" );
	}

	//
	// IsTestTalkProcessRunning() ???
	if( hMutex == NULL || GetLastError() == ERROR_ALREADY_EXISTS )
	{
		return true;
	}
#endif

	return false;
}

void CTestTalkClass::ReportPortBusyError(int portNumber)
{
#ifndef __TV_UNIX__
  if( !IsOtherTestTalkAlive() )
	{
		char buffer[1024];

	  sprintf(buffer, "Port %d is taken by other application. Could not start TestTalk.", portNumber);
		MessageBox(0, buffer, "TestTalk Initialization Error", MB_OK | MB_ICONERROR);
	}
#else
	printf("Port %d is taken by other instance of TestTalk or by other application. Could not start TestTalk.", portNumber);
#endif
}

void CTestTalkClass::ErrorMessageHandler(WLLogMessage::Severity severity, const char* errorMsg)
{
}

