/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// tmlnet.cpp
//               TML's Interface With The Net Server
//
//   who          when          what
//  Iris      05-11-93      Created
// Yahali     21-Nov-93     Write the body of the empty stubs.
// Yahali      28-Dec-93      Add multi-message functionality.
//

#ifndef __TV_UNIX__
#include <crtdbg.h>
#else
#define  _CrtSetDbgFlag(p)  NULL
#endif

#include "tvenv.h"
#include "testtalkdefs.h"
#include "testtalkclientobjimp.h"
#include "nvspackmsg.h"
#include "nvsnames.h"

#include "testtalkclientobj.h"
#include "tnerror.h"
#include "send2tml.h"
#include "tracedll.h"
#include "testtalkclientcommunicator.h"
#include "testtalkclientobjimp.h"


CTestTalkClientObject::CTestTalkClientObject(const char* name, TestTalkClientObject *client) :
  m_Client(client),
	m_pRetTtm(NULL),
	m_IsNETTmlUp(false),
	m_NetLastStatus(NETSTATUS_SUCCESS)
{


	EnterProcedure("CTestTalkClientObject::CTestTalkClientObject");
	m_MsgQs[DataQ] = new CTestTalkClientQueue;
	m_MsgQs[ControlQ] = new CTestTalkClientQueue;
	m_MsgQs[AckQ] = new CTestTalkClientQueue;
  m_CTestTalkClientCommunicator = new CTestTalkClientCommunicator(*this);

	TVEnvContainerClass tvpc;
	if( tvpc.TVEGetEnv("TESTTALK_INTERNAL_ACK_ON_SEND_WAIT_TIMEOUT") != NULL ) {
		tvpc.reset();
		m_WaitForAckOnSendTimout = atoi(tvpc.TVEGetEnv("TESTTALK_INTERNAL_ACK_ON_SEND_WAIT_TIMEOUT"));
	}
	else {
		m_WaitForAckOnSendTimout = 60000;
	}
}

CTestTalkClientObject::~CTestTalkClientObject()
{

	//WlTraceMsg("CTestTalkClientObject::~CTestTalkClientObject - Started");
  try {
    NETTerminate();
    delete m_MsgQs[DataQ];
    delete m_MsgQs[ControlQ];
    delete m_MsgQs[AckQ];
    delete m_CTestTalkClientCommunicator;
    if (m_pRetTtm)
	    NVSDeleteMsg(m_pRetTtm);
  } catch (...) {
  	//WlTraceMsg("CTestTalkClientObject::~CTestTalkClientObject - Fatal Error.");
  }
	//WlTraceMsg("CTestTalkClientObject::~CTestTalkClientObject - Done");
}

int CTestTalkClientObject::sendMessageToTestTalk(TestTalkMessage& ce)
{
	EnterProcedure("CTestTalkClientObject::sendMessageToTestTalk");
	bool st = m_CTestTalkClientCommunicator->sendMessage(
                    ce.getPackedBuf(),
                    ce.getPackedBufLen(),
                    ce.getType(),
                    ce.getSubtype()
                    );

	if(!st) {
		WlTraceMsg("CTestTalkClientObject::sendMessageToTestTalk failed");
	}

	return ((int)st);
}

bool CTestTalkClientObject::NETInit(const char* tmlInstance, unsigned short port)
{
	EnterProcedure("CTestTalkClientObject::NETInit");
	//sergei 3/24/01 - debugPrint(3,("NetInit(%s,%d)\n",tmlInstance,hWnd));

  if (port == 0) {
    m_TestTalkClientsPort = 0;
    TVEnvContainerClass tvpc;
    const char* sClientsPort = tvpc.TVEGetEnv("TESTTALK_CLIENT_PORT");
    if (sClientsPort!=NULL)
      m_TestTalkClientsPort = atoi(sClientsPort);
    if (m_TestTalkClientsPort == 0)
      m_TestTalkClientsPort = 9010;
  } else {
    m_TestTalkClientsPort = port;
  }

	m_MsgQs[DataQ]->Init();
	m_MsgQs[ControlQ]->Init();
	m_MsgQs[AckQ]->Init();

	m_NetLastStatus = NETSTATUS_SUCCESS;
	//
	// check the tmlInstance name to decide which message to
	// send to NetView.
  strcpy(m_NetClientData.clientName,tmlInstance);

	//
	// create the window-thread:
  m_CTestTalkClientCommunicator->Start();
  if (!m_CTestTalkClientCommunicator->Valid())
    return false;

  TypedMessageObject *ret = waitTypedMessage(ControlQ);
  if (!ret)
    return false;

  // copy the name.
	memcpy((void*)&m_NetClientData,ret->MsgBuf,sizeof(NetClientData));
  m_IsNETTmlUp = (ret->type != 0);

	//sergei 3/24/01 - debugPrint(3,("NETInit: The Name I got from NetView is: %s Queue=%d\n", NETWhoAmI(),m_NetClientData.tvnetQ));

  return (m_IsNETTmlUp);
}

bool CTestTalkClientObject::NETTurnOn()
{
	//
	// terminate the queues
	m_MsgQs[DataQ]->Init();
	m_MsgQs[ControlQ]->Init();
	m_MsgQs[AckQ]->Init();

  return true;
}

bool CTestTalkClientObject::NETTurnOff()
{
	//
	// terminate the queues
	m_MsgQs[DataQ]->Terminate();
	m_MsgQs[ControlQ]->Terminate();
	m_MsgQs[AckQ]->Terminate();

  return true;
}

/***********************************************************************
/* Name : NETTerminate
/* Purpose : terminate the TMLNET session.
/*
/* Description : closes the Q windows and notifies NetView of Termination.
/*
/* Input : None
/*
/* Output : None
/*
/* Return values: None
/*
/* Creation: Yahali, 22-Nov-93
/* Updates :  Who          When         What
/*
/***********************************************************************/
void CTestTalkClientObject::NETTerminate()
{
	EnterProcedure("CTestTalkClientObject::NETTerminate");

	// check wether a net termination was already done.
	if (!m_IsNETTmlUp)
		return;

	m_IsNETTmlUp = false;

	// send TestView a termination message.
	BOOL ret = m_CTestTalkClientCommunicator->sendMessage((void *)"", 1, NV_CLIENT_TERMINATED, 0);
	//WlTraceMsg("CTestTalkClientObject::NETTerminate - Sending termination message to '%s' returned %d",m_NetClientData.clientName, ret);
	//
	// terminate the window thread.
  try {
	  m_MsgQs[DataQ]->release();
	  m_MsgQs[ControlQ]->release();
	  m_MsgQs[AckQ]->release();
	  //WlTraceMsg("CTestTalkClientObject::NETTerminate - Terminate listen thread");
    if (m_CTestTalkClientCommunicator)
      m_CTestTalkClientCommunicator->Terminate();
	  //WlTraceMsg("CTestTalkClientObject::NETTerminate - done.");
  } catch (...) {
  }
	return;
}

bool CTestTalkClientObject::copyFile( long msgId,
																		  const char *data, size_t dataSize,
																			const char *toName, const char *toHost,
																			QType toQ )

{
	//WlTraceMsg("CTestTalkClientObject::copyFile(%s) - started", NETWhoAmI());
	//
	// set the type to be local/send message according to host.
	short type;
	//
	// @@Eyal: NV_LOCAL_FILECOPY cause dead locks...
	//if (strcmp(toHost,m_NetClientData.localHostName) == 0)
        //	type = NV_LOCAL_FILECOPY;
	//else
		type = NV_FILECOPY;
	//
	// pack the message and send to NetView.
	TestTalkMessage ce( msgId,
		toHost,
		toName,
		m_NetClientData.localHostName,
		m_NetClientData.clientName,
		toQ,
		type,
		0,
		(void*)data,
		dataSize);

	//
	// send the message.
	int pmStat = sendMessageToTestTalk(ce);
	if (!pmStat) {
		m_NetLastStatus = NETERROR_REQUESTTOTVNETFAILED;
		return false;
	}
	return true;
}

/***********************************************************************
/* Name : sendMessage
/* Purpose : send a message to a givven address (remote tml).
/*
/* Input : data - the message to be sent.
/*         addr - the address of the remote TML.
/*
/* Output : None
/*
/* Return values: Success/Fail.
/*
/* Creation: Yahali, 28-Dec-93
/* Updates :  Who          When         What
/*
/***********************************************************************/
bool CTestTalkClientObject::sendMessage( const char *data,
																				 size_t      dataSize,
																         const char *toName,
																         const char *toHost,
																         QType toQ ,
                                         int timeout)
{
  //WlTraceMsg("CTestTalkClientObject::sendMessage(%s) - started", NETWhoAmI());
	//
	// set the type to be local/send message according to host.
	short type;
	if (strcmp(toHost,m_NetClientData.localHostName) == 0)
		type = NV_LOCALMESSAGE;
	else
		type = NV_SENDMESSAGE;
	//
	// pack the message and send to NetView.
	TestTalkMessage ce( 0,
                      toHost,
											toName,
                      m_NetClientData.localHostName,
											m_NetClientData.clientName,
                      toQ,
                      type,
											0,
                      (void*)data,
											dataSize);
	//
	// clean the controt queue
	TestTalkMessage* tempTTM;
	while (tempTTM = peekTTMessage(ControlQ))
		NVSDeleteMsg(tempTTM);
	//
	// send the message.
	int pmStat = sendMessageToTestTalk(ce);
  if (!pmStat)
  {
    //tmlnetError((TNESendMsgToNetView));
    m_NetLastStatus = NETERROR_REQUESTTOTVNETFAILED;
		WlTraceMsg("CTestTalkClientObject::sendMessage(%s) - send error %d", NETWhoAmI(), m_NetLastStatus);

		return(false);
  }

  //
  // wait for a return-status from NetView:
	//WlTraceMsg("CTestTalkClientObject::sendMessage(%s) - Waiting for ControlQ", NETWhoAmI());
	TestTalkMessage* rce = waitTTMessage(ControlQ, timeout);

	if(!rce) {
		WlTraceMsg("CTestTalkClientObject::sendMessage(%s) - wait error %d", NETWhoAmI());
		return false;
	}

	int retstat = rce->getType();

	//WlTraceMsg("CTestTalkClientObject::sendMessage(%s) - Retstat = %d", NETWhoAmI(), retstat);

  //
	// check for error: then tmlName will be ""
  if (retstat != NV_SENDMESSAGE_STATUS_SUCCESS)
  {
    //tmlnetError((TNESendMsgFailed,(const char*)rce->getData()));
    m_NetLastStatus = NETERROR_SENDMESSAGEFAILURE;
		NVSDeleteMsg(rce);
		WlTraceMsg("CTestTalkClientObject::sendMessage(%s) - ack on send error %d", NETWhoAmI(), m_NetLastStatus);
		return false;
  }
	NVSDeleteMsg(rce);
	//WlTraceMsg("CTestTalkClientObject::sendMessage(%s) - Done", NETWhoAmI());
  return true;
}

/***********************************************************************
/* Name : sendMessageNoresponse
/* Purpose : send a message to a givven address (remote tml).
/*
/* Input : data - the message to be sent.
/*         addr - the address of the remote TML.
/*
/* Output : None
/*
/* Return values: Success/Fail.
/*
/* Creation: Yahali, 28-Dec-93
/* Updates :  Who          When         What
/*
/***********************************************************************/
bool CTestTalkClientObject::sendMessageNoresponse( const char *data,
																									 size_t      dataSize,
          																         const char *toName,
					          											         const char *toHost,
										          						         QType toQ)
{
  //WlTraceMsg("CTestTalkClientObject::sendMessage(%s) - started", NETWhoAmI());
	//
	// set the type to be local/send message according to host.
	short type;
	if (strcmp(toHost,m_NetClientData.localHostName) == 0)
		type = NV_LOCALMESSAGE_NORESPONSE;
	else
		type = NV_SENDMESSAGE_NORESPONSE;
	//
	// pack the message and send to NetView.
	TestTalkMessage ce( 0,
                      toHost,
											toName,
                      m_NetClientData.localHostName,
											m_NetClientData.clientName,
                      toQ,
                      type,
											0,
                      (void*)data,
											dataSize);
	//
	// send the message.
	int pmStat = sendMessageToTestTalk(ce);
  if (!pmStat)
  {
    //tmlnetError((TNESendMsgToNetView));
    m_NetLastStatus = NETERROR_REQUESTTOTVNETFAILED;
		return(false);
  }

  return true;
}

/***********************************************************************
/* Name : sendMessageNowait
/* Purpose : send a message to a givven address (remote tml).
/*
/* Input : data - the message to be sent.
/*         addr - the address of the remote TML.
/*
/* Output : None
/*
/* Return values: Success/Fail.
/*
/* Creation: Yahali, 28-Dec-93
/* Updates :  Who          When         What
/*
/***********************************************************************/
bool CTestTalkClientObject::sendMessageNowait( long msgId,
                                               const char *data,
																							 size_t      dataSize,
          																     const char *toName,
					          											     const char *toHost,
										          						     QType toQ)
{
  //WlTraceMsg("CTestTalkClientObject::sendMessage(%s) - started", NETWhoAmI());
	//
	// set the type to be local/send message according to host.
	short type;
	if (strcmp(toHost,m_NetClientData.localHostName) == 0)
		type = NV_LOCALMESSAGE_NOWAIT;
	else
		type = NV_SENDMESSAGE_NOWAIT;
	//
	// pack the message and send to NetView.
	TestTalkMessage ce( msgId,
                      toHost,
											toName,
                      m_NetClientData.localHostName,
											m_NetClientData.clientName,
                      toQ,
                      type,
											0,
                      (void*)data,
											dataSize);
	//
	// send the message.
	int pmStat = sendMessageToTestTalk(ce);
  if (!pmStat)
  {
    //tmlnetError((TNESendMsgToNetView));
    m_NetLastStatus = NETERROR_REQUESTTOTVNETFAILED;
		WlTraceMsg("CTestTalkClientObject::sendMessageNowait(%s) - send failed, error=%d", NETWhoAmI(), m_NetLastStatus);
		return(false);
  }

  return true;
}

bool CTestTalkClientObject::NETCopyFile(long msgId, const char *data, size_t dataSize, const char *addr)
{
	WlTraceMsg("CTestTalkClientObject::NETCopyFile(data=%s)", data); //@@eyal: trace
	//
	// check wether the TMLNET was initialized correctly:
	checkNetActive(false);
	//
	// check the data & address they must not be null or 0-length.
	assert(data!=NULL && strlen(data)!=0);
	assert(addr!=NULL && strlen(addr)!=0);

	m_NetLastStatus = NETSTATUS_SUCCESS;

	//
	// send the Message to TestTalk:
	CTestTalkAddress address("NoClient", addr);
	if(strlen(address.HostName()) == 0) {
				return (false);
	}
	//
	// Send the Message.
	return copyFile(msgId, data, dataSize, address.ClientName(), address.HostName(), DataQ);
}

/***********************************************************************
/* Name : NETSendMultiMessage
/* Purpose : send a message to a multi-Address list.
/*
/* Input : data - the message to be sent.
/*         addr - the address of the remote TML.
/*
/* Output : None
/*
/* Return values: Success/Fail.
/*
/* Creation: Yahali, 28-Dec-93
/* Updates :  Who          When         What
/*
/***********************************************************************/
bool CTestTalkClientObject::NETSendMultiMessage( const char *data, size_t dataSize, const char* addr )
{
/*
  //
	// check wether the TMLNET was initialized correctly:
	checkNetActive(false);

	assert(data!=NULL && strlen(data)>0);
	assert(clientsList!=NULL);

	m_NetLastStatus = NETSTATUS_SUCCESS;

	if (strlen(clientsList)==0) return false;

  TestTalkMessage ce(clientsList,
                      "",
											m_NetClientData.localHostName,
                      m_NetClientData.clientName,
                      DataQ,
                      NV_MULTIMESSAGE,
                      0,
											(void*)data,
                      dataSize);

  int pmStat = sendMessageToTestTalk(ce);
  if (!pmStat)
  {
		//tmlnetError((TNESendMultiMsgToNetView));
    m_NetLastStatus = NETERROR_REQUESTTOTVNETFAILED;
		return(false);
	}
*/
  return(true);
}

bool CTestTalkClientObject::NETSendMessage(const char *data, size_t dataSize, const char *addr)
{
	WlTraceMsg("CTestTalkClientObject::NETSendMessage(data=%s)", data); //@@eyal: trace
	//
	// check wether the TMLNET was initialized correctly:
	checkNetActive(false);
	//
	// check the data & address they must not be null or 0-length.
	assert(data!=NULL && strlen(data)!=0);
	assert(addr!=NULL && strlen(addr)!=0);

	m_NetLastStatus = NETSTATUS_SUCCESS;

  //
  // send the Message to TestTalk:
  CTestTalkAddress address(addr);
	if (strlen(address.ClientName())==0 || strlen(address.HostName())==0)
  {
		//tmlnetError((TNEIlgAddrFormat,addr));
		return (false);
	}
  //
  // Send the Message and wait for ack.
  // Default timeout is 60000, and timeout can be also set
	// in webload.ini (m_WaitForAckOnSendTimout holds the timeout value).
  bool retStatus = sendMessage(data, dataSize, address.ClientName(), address.HostName(), DataQ, m_WaitForAckOnSendTimout);

  return retStatus;
}

bool CTestTalkClientObject::NETSendMessageNoresponse(const char *data, size_t dataSize, const char *addr)
{
	//
	// check wether the TMLNET was initialized correctly:
	checkNetActive(false);
	//
	// check the data & address they must not be null or 0-length.
	assert(data!=NULL && strlen(data)!=0);
	assert(addr!=NULL && strlen(addr)!=0);

	m_NetLastStatus = NETSTATUS_SUCCESS;

  //
  // send the Message to TestTalk:
  CTestTalkAddress address(addr);
	if (strlen(address.ClientName())==0 || strlen(address.HostName())==0)
  {
		//tmlnetError((TNEIlgAddrFormat,addr));
		return (false);
	}

  return sendMessageNoresponse(data, dataSize, address.ClientName(), address.HostName(), DataQ);
}

bool CTestTalkClientObject::NETSendMessageNowait(long msgId, const char *data, size_t dataSize, const char *addr)
{
	//
	// check wether the TMLNET was initialized correctly:
	checkNetActive(false);
	//
	// check the data & address they must not be null or 0-length.
	assert(data!=NULL && strlen(data)!=0);
	assert(addr!=NULL && strlen(addr)!=0);

	m_NetLastStatus = NETSTATUS_SUCCESS;

  //
  // send the Message to TestTalk:
  CTestTalkAddress address(addr);
	if (strlen(address.ClientName())==0 || strlen(address.HostName())==0)
  {
		//tmlnetError((TNEIlgAddrFormat,addr));
		return (false);
	}

  return sendMessageNowait(msgId, data, dataSize, address.ClientName(), address.HostName(), DataQ);
}


const char* CTestTalkClientObject::waitMessage(QType qtype)
{
	WlTraceMsg("CTestTalkClientObject::waitMessage");
	//
	// check wether the TMLNET was initialized correctly:
	checkNetActive(NULL);

	if (m_pRetTtm)
		NVSDeleteMsg(m_pRetTtm);

	m_NetLastStatus = NETSTATUS_SUCCESS;

	m_pRetTtm = waitTTMessage(qtype);
	if (!m_pRetTtm)
	{
		m_IsNETTmlUp = false;
		checkNetActive(NULL);
	}

	const char* retmsg = (const char*)m_pRetTtm->getData();
	//sergei 3/24/01 - debugPrint(3,("received message: '%s'\n", m_pRetTtm->printOnString()));

	return (retmsg);
}


/********************* NET FETCH MESSAGE ************************/
/* The function returns a const char* Message from Tml's Net    */
/* window if there is a message availble, otherwise it returns  */
/* NULL.                                                        */
/****************************************************************/
const char* CTestTalkClientObject::NETFetchMessage()
{
	WlTraceMsg("CTestTalkClientObject::NETFetchMessage");
	//
	// check wether the TMLNET was initialized correctly:
	checkNetActive(NULL);

	if (m_pRetTtm)
		NVSDeleteMsg(m_pRetTtm);

	m_NetLastStatus = NETSTATUS_SUCCESS;

	m_pRetTtm = peekTTMessage(DataQ);
	if (!m_pRetTtm)
		return NULL;

	const char* retmsg = (const char*)m_pRetTtm->getData();
	//sergei 3/24/01 - debugPrint(3,("received message: '%s'\n", m_pRetTtm->printOnString()));

	return (retmsg);
}

/********************* NET WAIT MESSAGE *************************/
/* The function returns a const char* Message from Tml's Net    */
/* window. If there isn't a message availble, it waits until a  */
/* message is placed in the Tml's net window message queue and  */
/* then it return it.                                           */
/****************************************************************/
const char* CTestTalkClientObject::NETWaitMessage()
{
	WlTraceMsg("CTestTalkClientObject::NETWaitMessage");
	//
	// check wether the TMLNET was initialized correctly:
	checkNetActive(NULL);
	return waitMessage(DataQ);
}

const char* CTestTalkClientObject::NETWhoAmI()
{
	//
	// check wether the TMLNET was initialized correctly:
	checkNetActive("");

	m_NetLastStatus = NETSTATUS_SUCCESS;

	//
	// get the name of the current client & return it.
  if (m_WhoAmI.isNull()) {
	  CTestTalkAddress address(m_NetClientData.clientName,m_NetClientData.localHostName);
    m_WhoAmI = address.Address();
    char cPort[20];
    sprintf(cPort, "%d", (int)m_NetClientData.TestTalkPort);
    m_WhoAmI.append(":");
    m_WhoAmI.append(cPort);
  }
  return m_WhoAmI.data();
}

const char* CTestTalkClientObject::NETLocalHostName()
{
	//
	// check wether the TMLNET was initialized correctly:
	checkNetActive("");

	return ( m_NetClientData.localHostName );
}

int CTestTalkClientObject::NETGetLastStatus()
{
	return m_NetLastStatus;
}

const char* CTestTalkClientObject::NETGetLastErrorMessage()
{
	return m_ErrorMsg;
}

void CTestTalkClientObject::NewMessageEvent(TypedMessageObject* tmo, short qtype)
{
  bool status = true;

  switch (qtype)
  {
    case DataQ:
    case AckQ:
      {
        try {
          status = m_Client->NewMessageEvent(tmo, qtype);
        } catch (...) {
          WlTraceMsg("Fatal error occured in: CTestTalkClientObject::NewMessageEvent");
        }
      }
      break;
    case ControlQ:
      {
      }
      break;

    default:
      break;
  }

  if (status)
    m_MsgQs[qtype]->putMessage(tmo);
  //else
  //  delete tmo;
}


/////////////////////  CTestTalkClientQueue ///////////////////////

CTestTalkClientQueue::CTestTalkClientQueue()
{
}

CTestTalkClientQueue::~CTestTalkClientQueue()
{
}

void CTestTalkClientQueue::putMessage(TypedMessageObject* tmo)
{

	putEvent((RVCollectable *)tmo);
}

TestTalkMessage* CTestTalkClientQueue::peekTTMessage()
{
	TypedMessageObject* tmo = peekMessage();
	if (!tmo)
		return NULL;
	TestTalkMessage* ttm = NVSTmo2Ttm(tmo);
	return ttm;
}

TypedMessageObject* CTestTalkClientQueue::peekMessage()
{
	if (!isEmpty())
		return waitMessage();
	else
		return NULL;
}

TestTalkMessage* CTestTalkClientQueue::waitTTMessage(DWORD dwTimeout)
{
	TypedMessageObject* tmo = waitMessage(dwTimeout);
	if (!tmo)
		return NULL;
	TestTalkMessage* ttm = NVSTmo2Ttm(tmo);
	return ttm;
}

TypedMessageObject* CTestTalkClientQueue::waitMessage(DWORD dwTimeout)
{
  RVCollectable* pMessage;
  TVPEventQueue::WaitReturnType ret;

  ret = waitEvent(pMessage, dwTimeout);
  switch (ret)
	{
    case TVPEventQueue::Success :
      return (TypedMessageObject*)pMessage;

    case TVPEventQueue::TimedOut :
      WlTraceMsg("CTestTalkClientQueue::waitMessage - WaitMessage Timeout");
      break;

    default:
      break;
  }

	return NULL;
}

