/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "testtalklistensockets.h"
#include "testtalknetconnection.h"
#include "tracedll.h"

//---------------- CTestTalkListenerSocket Impementation --------------------//

CTestTalkListenerSocket::CTestTalkListenerSocket(CTestTalkClass *ttMain) :
  m_Terminate(false), m_TestTalkMain(ttMain)
{
  m_Terminate = false;
  //m_ListenThread = new RVThread;
  #if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)            
    m_ListenThread = rvMakeThreadFunction(
                        *this,                                // instance
                        &CTestTalkListenerSocket::ListenProc     // member function
                      );
  #else
    m_ListenThread = rvMakeThreadFunctionM( 
                        CTestTalkListenerSocket                  // class
                        *this,                                // instance
                        void,                                 // return value
                        &CTestTalkListenerSocket::ListenProc     // member function
                      );
  #endif
}

CTestTalkListenerSocket::~CTestTalkListenerSocket()
{
  Terminate();
  //delete m_ListenThread;
}

bool CTestTalkListenerSocket::Activate()
{
  m_StreamSocket.SetSocketStyle();
  m_StreamSocket.CreateSocket();
#ifdef __TV_UNIX__
  m_StreamSocket.SetReuseAddressOption(TRUE);
#endif
  if (m_StreamSocket.BindSocket(m_Port) == WINSOCK_ERROR)
    return false;

  if (m_StreamSocket.Listen() == WINSOCK_ERROR)
    return false;

  m_ListenThread.start();

  return true;
}

void CTestTalkListenerSocket::Terminate()
{
  RVCriticalSection::LockGuard Lock(m_Touch);
  if (!m_Terminate) {
    WlTraceMsg("CTestTalkListenerSocket::Terminate(%d)", m_StreamSocket.m_Socket);
    m_StreamSocket.CloseSocket();
    m_Terminate = true;
  }
}

void CTestTalkListenerSocket::ListenProc()
{
  RVThreadSelf self = ::rvThread();
  self.setPriority(self.getMaxPriority());
  //TVPSysThread::setPriority(TVPSysThread::Critical);

	WlTraceMsg("CTestTalkListenerSocket::ListenProc(%d)", m_StreamSocket.m_Socket);

  SOCKET socket;
  sockaddr_in sAddress;
  int acceptStatus = WINSOCK_NOERROR;
  while (!m_Terminate ) {
    acceptStatus = m_StreamSocket.Accept(socket, sAddress);

    if (acceptStatus != WINSOCK_NOERROR) 
		{
			WlTraceMsg("CTestTalkListenerSocket::ListenProc(%d), Accept failed", m_StreamSocket.m_Socket);
      break;
		}

    // now create a new host connection object.]
    NewSocketAccepted(socket);
  }

  Terminate();
}

//---------------- CTestTalkNetListener Impementation --------------------//

CTestTalkNetListener::CTestTalkNetListener(CTestTalkClass *ttMain) :
  CTestTalkListenerSocket(ttMain)
{
}

CTestTalkNetListener::~CTestTalkNetListener()
{
}

void CTestTalkNetListener::NewSocketAccepted(SOCKET s)
{
  CTestTalkHostConnection *hc = new CTestTalkHostConnection(s, m_TestTalkMain);
}

//---------------- CTestTalkClientListener Impementation --------------------//

CTestTalkClientListener::CTestTalkClientListener(CTestTalkClass *ttMain) :
  CTestTalkListenerSocket(ttMain)
{
}

CTestTalkClientListener::~CTestTalkClientListener()
{
}

void CTestTalkClientListener::NewSocketAccepted(SOCKET s)
{
  CTestTalkClientConnection *cc = new CTestTalkClientConnection(s, m_TestTalkMain);
}

