/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "tvenv.h"
#include "testtalkmain.h"


CTestTalkMain::CTestTalkMain(bool isTestTalkService) :
m_IsNetKernelUp(false), m_NetKernel(isTestTalkService)
{
  m_tcpNetPort = 0;
  TVEnvContainerClass tvpc;
  const char* sNetPort = tvpc.TVEGetEnv("TESTTALK_NETWORK_PORT");
  if (sNetPort!=NULL)
    m_tcpNetPort = atoi(sNetPort);
  if (m_tcpNetPort == 0)
    m_tcpNetPort = 9000;

  m_tcpClientsPort = 0;
  tvpc.reset();
  const char* sClientsPort = tvpc.TVEGetEnv("TESTTALK_CLIENT_PORT");
  if (sClientsPort!=NULL)
    m_tcpClientsPort = atoi(sClientsPort);
  if (m_tcpClientsPort == 0)
    m_tcpClientsPort = 9010;
}

CTestTalkMain::~CTestTalkMain()
{
}

bool CTestTalkMain::Init()
{
  //WlTraceMsg("Starting CTestTalkMain::Init");

  if (!CTestTalkClass::Init())
    return false;
  RunNetKernel();

  return true;
}

void CTestTalkMain::Terminate()
{
  //WlTraceMsg("Starting CTestTalkMain::Terminate");

  m_NetKernel.Stop();
  CTestTalkClass::Terminate();
  
  //WlTraceMsg("CTestTalkMain::Terminate - Done");
}

void CTestTalkMain::RunNetKernel()
{
	if (m_IsNetKernelUp)
		return;

	m_IsNetKernelUp=true;
	//
	// create the NetKernel instance.
  m_NetKernel.Start(m_tcpClientsPort);
}

bool CTestTalkMain::IsTestTalkService()
{
	return m_NetKernel.IsTestTalkService();
}

void CTestTalkMain::ErrorMessageHandler(WLLogMessage::Severity severity, const char* errorMsg)
{
  WlTraceMsg(errorMsg);
}

