/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// TestTalkPing.cpp : Defines the entry point for the console application.
//
#include <iostream.h>
#include <fstream.h>
#include <RVCString.h>
#include "testtalknetrequester.h"
#include "tvenv.h"
#include "runtesttalk.h"
#include "tracedll.h"


bool TestViewCheck(char* argv[]);


int main(int argc, char* argv[])
{
	int								 start = 1;
	short              netPort = 9000;
	RVCString          address;
	CAssociativeArray  pingMessage;
  CTestTalkNetRequester pingClient;
	CTestTalkNetRequester::InformationType infoType;
	CTestTalkNetRequester::CInformationObject* info;
  	TVEnvContainerClass tvpc;

//	CTraceMsg::Init("TestTalkPing");

#ifdef __TV_WINDOWS__

  if( IsTestTalkProcessRunning() == 0 ) {
    RunTestTalk();
  }

#endif

#ifdef __TV_UNIX__
  //initialize Environment
  char thisFileName[MAX_PATH];
  if( !TVEInit(tvpc.TVPGetPathDirectory( (const char *)realpath(argv[0],thisFileName))))
  {
    TVPFatalErrorLog("%s: Init Terminated with error: (%s)",argv[0],TVEError());
    return FALSE;
  }

#endif // __TV_UNIX__

	CTraceMsg::Init("TestTalkPing");

	if(argc < 2 || strcmp(argv[1], "/?") == 0) {
		cout<<"\nUsage: TestTalkPing [-i, -v] {Hosts/IP List} \n"<<endl;
		cout<<"-i: Get system and WebLoad information from remote TestTalk\n"<<
		      "    The information is saved into TestTalkReportFile.txt"<<endl
			  <<"-v: Print remote WebLoad version information."<<endl;
		return 0;
	}

	if( TestViewCheck(argv) )
	{
		return 1; 
	}

	// handle information options {-i, -v}:
	if(strcmp(argv[start], "-i") == 0) {
		start++;
		infoType = CTestTalkNetRequester::SystemInfo;
		if(argc > 2 && strcmp(argv[start], "-v") == 0) {
			start++;
			infoType = CTestTalkNetRequester::All;
		}
	}
	else if(strcmp(argv[start], "-v") == 0) {
		start++;
		infoType = CTestTalkNetRequester::WLVersion;
		if(argc > 2 && strcmp(argv[start], "-i") == 0) {
			start++;
			infoType = CTestTalkNetRequester::All;
		}
	}

	// get testtalk network port
	const char* strNetPort = tvpc.TVEGetEnv("TESTTALK_NETWORK_PORT");
  if(strNetPort)
    netPort = atoi(strNetPort);
  
	// try to init the client
	if(!pingClient.Init("TTPing")) {
		cout<<"Failed to start TestTalkPing"<<endl;
		exit(0);
	}

	bool simplePing = (start == 1)? true : false;

	for(int i = start; i < argc; i++) {
		if(simplePing) {
			info = pingClient.TestTalkPing(argv[i]);
		}
		else {
			info = pingClient.TestTalkInformationPing(argv[i], infoType);
			if(info && infoType != CTestTalkNetRequester::WLVersion) {
				tvpc.reset();
				RVCString reportFileName = tvpc.TVERuntimeResolve("#(TV_BIN)/TestTalkReportFile.txt");
				ofstream report(reportFileName, ios::app);
				pingClient.GenerateReportFile(report, info);
				report.close();
			}
		}

		if(info) {
			RVCString version = "";
			double time = info->pingDuration;
			if(time < 10)
				time = 10;
			
			if(!simplePing && infoType != CTestTalkNetRequester::SystemInfo) {
				version =  "  Version=";
				version += info->wlVersion;
			}
			cout<<"Reply From "<<argv[i]<<": Time="<<time<<"ms  "<<
			      "Net Port="<<netPort<<version.data()<<endl;
			delete info;
		}
		else
			cout<<"Can't Establish Connection To Host: "<<argv[i]<<endl;
	}

	pingClient.Terminate();
	return 0;
}




bool TestViewCheck(char* argv[])
{
	// 
	// BEGIN: Temp Code - Olga
	if( strcmp(argv[1], "-c") == 0 )
	{
		CAssociativeArray pingMsg, replyMsg, message;
		CNetMessageHandler handler("nocallbacktest");

		handler.UseCallbacksModel(true);
		handler.Init();

		pingMsg.AddCouple("$TTPING", "localhost");

		handler.SendMessage(pingMsg, "theTml@localhost:9000", "Failed to send ping message");

		pingMsg.Reset();
		pingMsg.AddCouple("$ISTESTTALKSERVICE", "localhost");

		handler.SendMessage(pingMsg, "theTml@localhost:9000", "Failed to send ping message");

		const char* errorMessage = handler.GetLastErrorMessage();

		//if( handler.ReadNextMessage(replyMsg) )
		//{
		//	printf("%s\n", replyMsg.ToString());
		//}
		rvSleep(2000);

		//message.AddCouple("$CREATEWEBLOAD", "");
		//message.AddCouple("$ARG0", argv[2]);

		//handler.SendMessage(message, "theTml@localhost:9000", "Failed to send message");

		message.Reset();
		message.AddCouple("$CREATEWEBFT", "Create WebFT Process Remotly");
    message.AddCouple("$ARPARAM", "/a");
    message.AddCouple("$FILENAME", "");                   
    message.AddCouple("$FILENAMETOSAVE", "None");
    message.AddCouple("$ROUNDNUM", "2");                 
    message.AddCouple("$MODE", "/tv");
    message.AddCouple("$TESTVIEWNETNAME", handler.WhoAmI());
    message.AddCouple("$IPPARAM", "/ip");
    message.AddCouple("$INPUTPARAMS", "None"); 
		message.AddCouple("$BROWSER_TYPE_FLAG", "0"); 
		message.AddCouple("$BROWSER_TYPE", "IE"); 
		
		message.AddCouple("$PROCESSUID", "123456"); 

		handler.SendMessage(message, "theTml@localhost:9000", "Failed to send message");

		message.Reset();
		message.AddCouple("$KILLPROCESS", "123456"); 
		message.AddCouple("$PROCESSUID", "123456"); 

		rvSleep(3000);

		handler.SendMessage(message, "theTml@localhost:9000", "Failed to send message");

		//if( handler.ReadNextMessage(replyMsg) )
		//{
		//	printf("%s\n", replyMsg.ToString());
		//}

		rvSleep(10000);
		handler.Terminate();

		return true;
	}
	// 
	// END: Temp Code - Olga

	return false;
}


