/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// TestTalkToolKit.cpp: implementation of the TestTalkToolKit class.
//
//////////////////////////////////////////////////////////////////////

#include "testtalktoolkit.h"
#include "netevent.h"
#include "gt_unicode.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TestTalkToolKit::TestTalkToolKit()
{
}

TestTalkToolKit::~TestTalkToolKit()
{
}


//////////////////////////////////////////////////////////////////////
// Public Methods Implementation
//////////////////////////////////////////////////////////////////////

bool TestTalkToolKit::ReadFileToBuffer(FILE* file, char*& buffer, size_t& bufferLen)
{
#ifndef __TV_UNIX__
	        bufferLen = _filelength(fileno(file));
#else
	        int temp;
	        temp = fseek(file, 0, SEEK_END);
	        bufferLen = ftell(file);
	        temp = fseek(file, 0, SEEK_SET);
#endif
	buffer = new char[bufferLen + 1];

	size_t sizeRead = fread(buffer, sizeof(char), bufferLen, file);
	if(ferror(file)) {
		delete buffer;
		return false;
	}
	return true;
}

bool TestTalkToolKit::SaveBufferToFile(FILE* file, const char* buffer, const size_t bufferLen)
{
	fwrite((void*)buffer, sizeof(char), bufferLen, file);

	if(ferror(file)) {
		return false;
	}

	return true;
}

bool TestTalkToolKit::DeleteFile(const char* fileName)
{
	if(remove(fileName) != 0) {
		return false;
	}
	return true;
}


bool TestTalkToolKit::CopyFile(/*[IN]*/char* srcFilePath,
															 /*[IN]*/char* dstFilePath,
															 /*[OUT]*/TestTalkToolKit::CopyFileCompletionState& state,
											         /*[OUT]*/char*& stateFileName)
{
	const size_t FILE_CHUNK_SIZE = 204800; // 200K

	FILE* infile;
	FILE* outfile;
	char  buffer[FILE_CHUNK_SIZE];
	stateFileName = srcFilePath;

	//Galtech
	char ansiSrcFile[_MAX_PATH], ansiDstFile[_MAX_PATH];
	UTF8ToANSI(GetACP(), srcFilePath,  -1, ansiSrcFile, _MAX_PATH);
	UTF8ToANSI(GetACP(), dstFilePath,  -1, ansiDstFile, _MAX_PATH);

	if( (infile = fopen(ansiSrcFile, "rb")) ) {
		if( !(outfile = fopen(ansiDstFile, "wb")) ) {
			fclose(infile);
			state = OpenFileFailed;
			stateFileName = dstFilePath;
			return false;
		}
	}
	else {
		state = OpenFileFailed;
		return false;
	}

	while(!feof(infile)) {
		size_t sizeRead = fread(buffer, sizeof(char), FILE_CHUNK_SIZE, infile);

		if(ferror(infile)) {
			fclose(infile);
			fclose(outfile);
			remove(ansiDstFile);
			state = ReadFileFailed;
			return false;
		}

		fwrite(buffer, sizeof(char), sizeRead, outfile);

		if(ferror(outfile)) {
			fclose(infile);
			fclose(outfile);
			remove(ansiDstFile);
			state = WriteFileFailed;
			stateFileName = dstFilePath;
			return false;
		}
	}
	if(fclose(infile)) {
	 state = CloseFileFailed;
	 return false;
	}

	if(fclose(outfile)) {
		state = CloseFileFailed;
		stateFileName = dstFilePath;
		return false;
	}

	state = CopyFileSucceed;
	return true;
}


TestTalkHostEvent* TestTalkToolKit::CreateSendHostEvent(TestTalkMessage* event,
																												void* buffer,
																												size_t bufferSize,
																												int newMessageType,
																												QType queueType)
{
	return new TestTalkHostEvent(event->getMessageId(),
															 event->getToHost(),
															 event->getToClient(),
															 event->getFromHost(),
															 event->getFromClient(),
															 ControlQ,
															 newMessageType, 0,
															 buffer, bufferSize);
}


TestTalkHostEvent* TestTalkToolKit::CreateReturnHostEvent(TestTalkMessage* event,
																													void* buffer,
																													size_t bufferSize,
																													int newMessageType,
																													QType queueType)
{
	return new TestTalkHostEvent(event->getMessageId(),
															 event->getFromHost(),
															 event->getFromClient(),
															 event->getToHost(),
															 event->getToClient(),
															 queueType,
															 newMessageType, 0,
															 buffer, bufferSize);
}


TestTalkClientEvent* TestTalkToolKit::CreateReturnClientEvent(TestTalkMessage* event,
																															void* buffer,
																															size_t bufferSize,
																															int newMessageType,
																															QType queueType)
{
	return new TestTalkClientEvent(event->getMessageId(),
																 event->getFromHost(),
																 event->getFromClient(),
																 event->getToHost(),
																 event->getToClient(),
																 queueType,
																 newMessageType, 0,
																 buffer, bufferSize);
}


RVCString TestTalkToolKit::GetLocalHostOS()
{
	RVCString os("");

#ifndef __TV_UNIX__
	OSVERSIONINFO VersionInformation;
	VersionInformation.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
	if ( !GetVersionEx( &VersionInformation ) )
		return RVCString("");
	
	if ( VersionInformation.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS )
	{
		if ( VersionInformation.dwMinorVersion == 0 )
		{
			os = "Win95";
		}
		else 
		{
			os = "Win98";
		}
	}
	
	if ( VersionInformation.dwPlatformId == VER_PLATFORM_WIN32_NT )
	{
		if ( VersionInformation.dwMajorVersion == 5 )
		{
			os = "Win2000";
		}
		else
		{
			os = "WinNT";
		}
	}
#else
	os = "Unix";
#endif

	return RVCString(os);
}


RVCString TestTalkToolKit::GetErrorString(int getLastErrorCode)
{
	RVCString message("");

#ifndef __TV_UNIX__

	LPVOID lpMsgBuf;

	if(FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | 
										FORMAT_MESSAGE_FROM_SYSTEM | 
										FORMAT_MESSAGE_IGNORE_INSERTS,
										NULL,
										getLastErrorCode,
										MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
										(LPTSTR) &lpMsgBuf,
										0,
										NULL ))
	{
		message = (LPCTSTR)lpMsgBuf;
		LocalFree( lpMsgBuf );
	}

#endif

	return RVCString(message);
}

int TestTalkToolKit::GetLastError()
{
	int errorCode = -1;

#ifndef __TV_UNIX__
	errorCode = GetLastError();
#endif

	return errorCode;
}
