/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// ErrorLibLoader.cpp: implementation of the CErrorLibLoader class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ErrorLibLoader.h"
#include <RVCString.h>

CErrorLibLoader dllLoader;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CErrorLibLoader::CErrorLibLoader()
{
}

CErrorLibLoader::~CErrorLibLoader()
{
    Unload();
}

void CErrorLibLoader::Load()
{
    //ASSERT(0);
    m_hDLL = LoadLibrary( "WLFRMFiles.dll" );
    if ( m_hDLL )
    {
     //   DWORD err;
        RVCString funcStr( "?WLERRGetErrorSeverity@@YA?AW4Severity@WLLogMessage@@PBDH@Z" );
        m_GETERRORSEVERITYFunc = (GETERRORSEVERITY)GetProcAddress( m_hDLL,funcStr );
        if ( m_GETERRORSEVERITYFunc == NULL )
        {
           LPVOID lpMsgBuf;
            FormatMessage( 
                FORMAT_MESSAGE_ALLOCATE_BUFFER | 
                FORMAT_MESSAGE_FROM_SYSTEM | 
                FORMAT_MESSAGE_IGNORE_INSERTS,
                NULL,
                GetLastError(),
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                (LPTSTR) &lpMsgBuf,
                0,
                NULL 
            );
            // Process any inserts in lpMsgBuf.
            // ...
            // Display the string.
            //MessageBox( NULL, (LPCTSTR)lpMsgBuf, "Error", MB_OK | MB_ICONINFORMATION );
            // Free the buffer.
            LocalFree( lpMsgBuf );
        }
        funcStr = "?WLERRLoadErrorString@@YAHPBD0@Z";
        m_LOADERRORSTRINGFunc = (LOADERRORSTRING)GetProcAddress( m_hDLL, funcStr );
        funcStr = "?WLERRGetErrorFormat@@YAPBDPBDH@Z";
        m_GETERRORFORMATFunc = (GETERRORFORMAT)GetProcAddress( m_hDLL, funcStr );
        funcStr = "?WLERRTerminate@@YAXXZ";
        m_TERMINATEFunc = (TERMINATE)GetProcAddress( m_hDLL, funcStr );

        funcStr = "?WLERRGetErrorMsg@@YAXPAVexception@@AAVRWCString@@@Z";
        m_WLERRGERRORMSGFunc = ( WLERRGERRORMSG )GetProcAddress( m_hDLL, funcStr );
    }
}

void CErrorLibLoader::Unload()
{
   if ( m_hDLL )
       FreeLibrary( m_hDLL );
}

int CErrorLibLoader::WLERRGetErrorSeverity(const char* errFileName, int fmtId)
{
    if ( m_hDLL && m_GETERRORSEVERITYFunc )
        return m_GETERRORSEVERITYFunc( errFileName, fmtId );

    return 0;
}

int CErrorLibLoader::WLERRLoadErrorString(const char* errFileName, const char* data)
{
    if ( m_hDLL && m_LOADERRORSTRINGFunc )
        return m_LOADERRORSTRINGFunc( errFileName, data );

    return 0;
}

const char* CErrorLibLoader::WLERRGetErrorFormat(const char* errFileName, int fmtId)
{
    if ( m_hDLL && m_GETERRORFORMATFunc )
        return m_GETERRORFORMATFunc( errFileName, fmtId );

    return 0;
}

void CErrorLibLoader::WLERRTerminate()
{
    if ( m_hDLL && m_TERMINATEFunc )
        m_TERMINATEFunc();
}

void CErrorLibLoader::WLERRGetErrorMsg(std::exception* e, RVCString& s)
{
    if ( m_hDLL && m_WLERRGERRORMSGFunc )
        m_WLERRGERRORMSGFunc( e, s );
}
