/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// Local Variables:
// mode:c++
// End:
//
// Definition of Client & Server abstruct classes.
//

#ifndef _tv__clntSrv_h
#define _tv__clntSrv_h

#include <assert.h>
#include <stddef.h>
#include <collint.h>
#include <hashtab.h>

#include "platform.h"
#include "tvdebug.h"

#ifndef OK
#define	OK      1
#define	FAILURE -1
#endif

#ifndef	TRUE
#define	TRUE  1
#define	FALSE 0
#endif

#define MAX_SERVICES  256

/* tentative declaration */
class ClientObj;
class ServerObj;   // tentative

#ifdef _TestTalkUtilDll
#define ClientServerDllExport DllExport
#else
#define ClientServerDllExport DllImport
#endif

class ClientServerDllExport BaseEvent: public RVCollectableInt {
	friend class ServerObj;

protected:
	ClientObj* client;
	int        opcode;

	virtual int dispatchMe();

public:
	BaseEvent(ClientObj& clnt, int op): client(&clnt), opcode(op)
	{
    assert(client != NULL); 
	}

	BaseEvent() : client(NULL), opcode(0) {}

	virtual ~BaseEvent() {}

	int dispatchEvent(BaseEvent& e);

	int getOpcode()	{		return (opcode);	}

	ClientObj& getClient()
	{
		return (*client);
	}

	virtual bool isEqual(const RVCollectable* e) const
	{
		return 0;
	}

	int isClient(ClientObj& clnt) const
	{
		return (&clnt == client);
	}

	//
	// this function sould be defined in the derived classes,
	// according to the events they'll define.
	virtual unsigned hash() const
	{
		return 0;
	}

	virtual RVClassID isA() const
	{
		return RVClassID(0);
	}

	virtual char *nameOf() const
	{
		return ("BaseEvent");
	}
};

class ClientServerDllExport ValidateObject {

protected:
	int success;

public:
	ValidateObject() : success(TRUE) {}
	virtual ~ValidateObject() {}

	virtual int valid() const
	{
		return(success);
	}

	virtual char *nameOf() const
	{
		return ("ValidateObject");
	}
};

class ClientServerDllExport CollectableValidateObject : public RVCollectableInt, public ValidateObject {

public:
	CollectableValidateObject();
	virtual ~CollectableValidateObject() {}

	virtual bool isEqual(const RVCollectable* e) const
	{
		return 0;
	}
	//
	// this function sould be defined in the derived classes,
	// according to the events they'll define.
	virtual unsigned hash() const
	{
		return 0;
	}

	virtual RVClassID isA() const
	{
		return RVClassID(0);
	}

	virtual char *nameOf() const
	{
		return ("CollectableValidateObject");
	}
};



class ClientServerDllExport ClientObj : public CollectableValidateObject {

protected:
	typedef int (ClientObj::*FUNCP)(BaseEvent&);

	FUNCP     Services[MAX_SERVICES];   // Services table
	int       TopService;               // The upper-most service
	int       NumServices;              // Actual number of services

protected:
	// put a service function in the services table
	virtual int  signOn(FUNCP funcp);

	// assign a callback. register a service (by its op-code) in another
	// object to be activated by event.
	virtual int  pleaseRegister(ServerObj& destination, BaseEvent& event);
	virtual void pleaseUnRegister(ServerObj& destination, BaseEvent& event);
	virtual void pleaseUnRegisterAll(ServerObj& destination);

public:
	ClientObj();

	virtual ~ClientObj() {}

	// Dispatch is called by another object in order to activate a service
	virtual int dispatch(BaseEvent& event);

	virtual char *nameOf() const
	{
		return ("ClientObj");
	}
};

class ClientServerDllExport ServerObj : public ClientObj {

private:
	RVHashTable* eventsTable;

protected:
	virtual void dispatchEvent(BaseEvent& event);

	BaseEvent& findEvent(BaseEvent& event) const;

public:
	ServerObj(int tableSize = 37);

	virtual ~ServerObj();

	virtual int  registerEvent(BaseEvent& event);

	virtual void unRegisterEvent(BaseEvent& event);

	virtual void unRegisterClient(ClientObj& client);

	virtual int  applyRegistration()  // apply any Register/UnRegister activities
	{
		return TRUE;
	}

	virtual char *nameOf() const
	{
		return ("ServerObj");
	}

};


#endif // _tv__clntSrv_H

