/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//////////////////////////////////////////////////////////////////////
// AssociativeArray.cpp: implementation of the CAssociativeArray class.
//
// Modified: 13/4/2000
//////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
//#include <rw/strstrea.h>
#include <stdio.h>
#include <malloc.h>

#include <hashdict.h>
#include <RVCString.h>
#include <collstr.h>
#include <collectable.h>
#include <RVCTokenizer.h>
#include "tvtables.h"
#include "tvenv.h"

#include "associativearray.h"
//Galtech
#include "gt_unicode.h"

///////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
///////////////////////////////////////////////////////////////////////////////

CAssociativeArray::CAssociativeArray( int Capacity ) : RVHashDictionary( Capacity )
{}


CAssociativeArray::~CAssociativeArray()
{
	clearAndDestroy();
}


///////////////////////////////////////////////////////////////////////////////
// Private methods implementation
///////////////////////////////////////////////////////////////////////////////

inline size_t CAssociativeArray::ParseInt(const char* &input)
{
	int len;
	const char* blankPos;
	char tempNum[MAX_NUM_SIZE];

	blankPos = strchr(input, ' ');

	if (blankPos == NULL) {
		throw CIllegalAssoSyntaxException("Error"/*input*/);
	}

	len = blankPos - input;

	if ((len >= (MAX_NUM_SIZE - 1))  || (len <= 0)) {
		throw CIllegalAssoSyntaxException("Error"/*input*/);
	}

	strncpy(tempNum, input, len);
	tempNum[len] = 0;
	input = blankPos + 1;

	size_t ret = atoi(tempNum);
	//Galtech
	if (!ret && strcmp(tempNum, "0"))
		throw CIllegalAssoSyntaxException("Error"/*input*/);

	return ret;
}

// @@@@
const char* CAssociativeArray::LoadString(const char* input, char*& tokenBuffer,
																					size_t& bufferSize, RVCString& s, bool UseActualSize)
{
	if (*input != '\'') {
		throw CIllegalAssoSyntaxException(input);
	}

	input++;
	size_t size = ParseInt(input);

	// @@Chaim - for PROXYPOSTDATA - make sure to do this after ParseInt.
	// Important note - this only works since PROXYPOSTDATA is the last entry in the associative array
	// We should probably also check if we're encoding the data
	//@@Chaim - decode binary data ?
//	if (UseActualSize && ShouldDecodeBinaryData()){
//		size = strlen(input);
//	}

	if(size > bufferSize) {
		tokenBuffer = (char *)realloc(tokenBuffer, size + 1);
		bufferSize = size;
	}

	s.remove(0);
	memcpy(tokenBuffer, input, size);
	tokenBuffer[size] = 0;
	s.append(tokenBuffer, size);
	input = input + size + 1;

	return input;
}


///////////////////////////////////////////////////////////////////////////////
// Public methods implementation
///////////////////////////////////////////////////////////////////////////////

BOOL CAssociativeArray::Convert(UINT fromEncoding, UINT toEncoding, 
								  const char* fromString, RVCString& toString)
{
	char buf[64];
	toString.remove( 0 );

	char*        tokenBuffer = NULL;
	const char*  input;
	size_t       bufferSize;
	RVCString	   kToken;
	RVCString	   vToken;

	if (!fromString)
		return FALSE;

	try
	{
		//
		// the first token in the string is number of couples( key + value )
		//
		input = fromString;
		tokenBuffer = (char *)malloc(BUFFER_SIZE + 1);
		bufferSize = BUFFER_SIZE;

		int NumOfArguments = ParseInt(input);

		sprintf( buf ,"%d", NumOfArguments );
		toString=buf;

		RVCString vTokenConverted;
		for ( int i = 0; i < NumOfArguments; i++ )
		{
			input = LoadString(input, tokenBuffer, bufferSize, kToken);
			sprintf( buf, " '%d ", kToken.length());
			toString.append(buf);
			toString.append(kToken.data());

			input = LoadString(input, tokenBuffer, bufferSize, vToken);
			Convert(fromEncoding, toEncoding, vToken, vTokenConverted);
			sprintf( buf, " '%d ", vTokenConverted.length());
			toString.append(buf);
			toString.append(vTokenConverted.data());
		}

		free(tokenBuffer);
		toString.append(" ");
	}
	catch ( ... )
	{
		// Ofir Allocation Conflict
		if (tokenBuffer)
			free(tokenBuffer);
//			delete [] tokenBuffer;

		int nSize = ANSIToANSI(fromEncoding, toEncoding, 
			fromString, -1, NULL, 0);
		char *tmp = new char[nSize];
		ANSIToANSI(fromEncoding, toEncoding, 
			fromString, -1, tmp, nSize);
		toString = tmp;
 		delete [] tmp;
	}

	return  TRUE;
}


// @@@@
const char* CAssociativeArray::ToString()
{
	char buf[64];
	toString.remove( 0 );
	//
	// if the hash table is empty - do nothing;
	//
	if ( !isEmpty() )
	{
		RVCollectableString *Key;
		RVCollectableString *Value;
		//
		// adding the number of hash table's entries to output string;
		//
		sprintf( buf ,"%d", entries() );
		toString=buf;

		RVHashDictionaryIterator Iterator( *this );
		//
		// do while their are entries in the hash table;
		//
		while( Iterator() )
		{
			Key	  = ( RVCollectableString* )Iterator.key();
			Value = ( RVCollectableString* )Iterator.value();
			//
			// the format of the string is: 'length key 'length value.....
			//
			sprintf( buf, " '%d ", Key->length());
			toString.append(buf);
			toString.append(Key->data());

			sprintf( buf, " '%d ", Value->length());
			toString.append(buf);
			toString.append(Value->data(), Value->length());
		}
	}
	toString.append(" ");
	return (const char*)toString;
}

void CAssociativeArray::ToString(/*OUT*/char*& outString,
																 /*OUT*/size_t& outStringLength)
{
	ToString();
	outString = (char*)toString.data();
	outStringLength = toString.length();
}

void CAssociativeArray::FromString( const char* fromString )
{
	char*        tokenBuffer;
	const char*  input;
	size_t       bufferSize;
	RVCString	   kToken;
	RVCString	   vToken;

	try
	{
		//
		// the first token in the string is number of couples( key + value )
		//
		input = fromString;
		tokenBuffer = (char *)malloc(BUFFER_SIZE + 1);
		bufferSize = BUFFER_SIZE;

    try {
		int NumOfArguments = ParseInt(input);

		for ( int i = 0; i < NumOfArguments; i++ )
		{
			input = LoadString(input, tokenBuffer, bufferSize, kToken);

			input = LoadString(input, tokenBuffer, bufferSize, vToken);

			AddCouple(kToken.data(), vToken.data(), vToken.length());
		}
    }
    catch(...) {
      free(tokenBuffer);
      tokenBuffer = NULL;
      throw;
    }

		free(tokenBuffer);
	}
	catch ( ... )
	{
		throw CIllegalAssoSyntaxException(fromString);
	}
}

void CAssociativeArray::GetValueOfParameter( const char* Key,
		     																		 void*&  OutValue,
																						 size_t& OutValueLength )
{
	RVCollectableString CollKey( Key );

	RVCollectableString* pValue;

	pValue = ( RVCollectableString* )findValue( &CollKey );

	if ( pValue == NULL )
	{
		throw CAssoKeyDoesNotExistException( Key, ToString() );
	}

	OutValue = (void*)pValue->data();
	OutValueLength = pValue->length();
}

RVCString* CAssociativeArray::GetValueOfParameterAndDecodeIf( const char* Key, bool decodeBinaryData )
{
	RVCollectableString CollKey( Key );

	RVCollectableString* pValue;

	pValue = ( RVCollectableString* )findValue( &CollKey );

	RVCollectableString tmpValue = *pValue;

	// @@Chaim - add encoding for Comverse here

	if ( pValue == NULL )
	{
		throw CAssoKeyDoesNotExistException( Key, ToString() );
	}
	long len = pValue->length();
	if (decodeBinaryData){
		long retIndex = 0;
		for (long i = 0; i < len; i++){
			unsigned char val = tmpValue[i];
			if (val < 32 || val > 126 || val == '\\' || val == '\"'){
				char Hex[5];
				sprintf(Hex, "%%%02x", (unsigned char)(tmpValue[i]));
				pValue->replace(retIndex, 1, Hex);
				retIndex += 3;
			}
			else if (val == '%'){
				pValue->insert(retIndex, "%");
				retIndex += 2;
			}
			else {
				retIndex++;
			}
		}
		return pValue;
	}
	else {
		return pValue;
	}
}

const char* CAssociativeArray::GetValueOfParameter( const char* Key )
{
	RVCollectableString CollKey( Key );

	RVCollectableString* pValue;

	pValue = ( RVCollectableString* )findValue( &CollKey );

	if ( pValue == NULL )
	{
		throw CAssoKeyDoesNotExistException( Key, ToString() );
	}

	return pValue->data();
}

RVCString* CAssociativeArray::GetStringValueOfParameter( const char* Key )
{
	RVCollectableString CollKey( Key );

	RVCollectableString* pValue;

	pValue = ( RVCollectableString* )findValue( &CollKey );

	if ( pValue == NULL )
	{
		throw CAssoKeyDoesNotExistException( Key, ToString() );
	}

	return (RVCString*)pValue;
}


long CAssociativeArray::GetLongValueOfParameter( const char* Key )
{
	return atoi(GetValueOfParameter(Key));
}

double CAssociativeArray::GetDoubleValueOfParameter( const char* Key )
{
	return atof(GetValueOfParameter(Key));
}

const char* CAssociativeArray::GetValueOfParameter( const char* Key , const char* DefaultValue)
{
	RVCollectableString CollKey( Key );

	RVCollectableString* pValue;

	pValue = ( RVCollectableString* )findValue( &CollKey );

	if ( pValue == NULL )
		return DefaultValue;
	else
		return pValue->data();
}

long CAssociativeArray::GetLongValueOfParameter( const char* Key , long DefaultValue )
{
	const char* ret = GetValueOfParameter(Key, NULL);
	if (ret!=NULL)
		return atoi(ret);
	else
		return DefaultValue;
}

double CAssociativeArray::GetDoubleValueOfParameter( const char* Key , double DefaultValue )
{
	const char* ret = GetValueOfParameter(Key, NULL);
	if (ret!=NULL)
		return atof(ret);
	else
		return DefaultValue;
}

const char* CAssociativeArray::GetValueOfParameterArray( const char* Key, int index )
{
	RVCString CKey( Key );
	char cIndex[20];
	sprintf(cIndex,"%d", index);
	CKey.append(cIndex);

	return GetValueOfParameter(CKey.data());
}

long CAssociativeArray::GetLongValueOfParameterArray( const char* Key, int index )
{
	return atoi(GetValueOfParameterArray(Key, index));
}

double CAssociativeArray::GetDoubleValueOfParameterArray( const char* Key, int index )
{
	return atof(GetValueOfParameterArray(Key, index));
}

const char* CAssociativeArray::GetValueOfParameterArray( const char* Key, int index, const char* DefaultValue)
{
	RVCString CKey( Key );
	char cIndex[20];
	sprintf(cIndex,"%d", index);
	CKey.append(cIndex);

	return GetValueOfParameter(CKey.data(), DefaultValue);
}

long CAssociativeArray::GetLongValueOfParameterArray( const char* Key, int index, long DefaultValue )
{
	const char* ret = GetValueOfParameterArray(Key, index, NULL);
	if (ret!=NULL)
		return atoi(ret);
	else
		return DefaultValue;
}

double CAssociativeArray::GetDoubleValueOfParameterArray( const char* Key, int index, double DefaultValue )
{
	const char* ret = GetValueOfParameterArray(Key, index, NULL);
	if (ret!=NULL)
		return atof(ret);
	else
		return DefaultValue;
}

void CAssociativeArray::AddCouple( const char* Key, const void* Value, size_t valueLength )
{
	RVCollectableString *collKey = new RVCollectableString(Key);
	RVCollectableString *collValue = new RVCollectableString((const char*)Value, valueLength);
	if (!insertKeyAndValue( collKey, collValue )) {
		RVCollectableString *oldValue = (RVCollectableString *)findValue(collKey, collValue);
		delete oldValue;
		delete collKey;
	}
}

void CAssociativeArray::AddCouple( const char* Key, const char* Value )
{
	RVCollectableString *collKey = new RVCollectableString(Key);
	RVCollectableString *collValue = new RVCollectableString(Value);
	if (!insertKeyAndValue( collKey, collValue )) {
		RVCollectableString *oldValue = (RVCollectableString *)findValue(collKey, collValue);
		delete oldValue;
		delete collKey;
	}
}

void CAssociativeArray::AddCouple( const char* Key, long Value )
{
	char cnum[100];
	sprintf(cnum, "%d", Value);
	AddCouple( Key, cnum );
}

void CAssociativeArray::AddCouple( const char* Key, double Value )
{
	char cnum[100];
	sprintf(cnum, "%f", Value);
	AddCouple( Key, cnum );
}

void CAssociativeArray::AddArrayCouple( const char* Key, const char* Value, int index )
{
	RVCString CKey = Key;
	char cIndex[20];
	sprintf(cIndex,"%d", index);
	CKey.append(cIndex);
	AddCouple( CKey.data(), Value );
}

void CAssociativeArray::AddArrayCouple( const char* Key, long Value, int index )
{
	RVCString CKey = Key;
	char cIndex[20];
	sprintf(cIndex,"%d", index);
	CKey.append(cIndex);
	AddCouple( CKey.data(), Value );
}

void CAssociativeArray::AddArrayCouple( const char* Key, double Value, int index )
{
	RVCString CKey = Key;
	char cIndex[20];
	sprintf(cIndex,"%d", index);
	CKey.append(cIndex);
	AddCouple( CKey.data(), Value );
}

bool CAssociativeArray::KeyExists( const char* Key )
{
	RVCollectableString CollKey( Key );

	RVCollectableString* pValue;

	pValue = ( RVCollectableString* )findValue( &CollKey );

	if ( pValue == NULL )
		return false;
	else
		return true;
}

void CAssociativeArray::Reset()
{
	clear();
// Ofir Memory Leak - SHOULD UNREMARK AND CHECK
//	clearAndDestroy();
}


