/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//
// Implementation of Client & Server Objects
//
#ifdef __TV_WINDOWS__
//#include "stdafx.h"
#endif
#include <assert.h>
#include <malloc.h>
#include <memory.h>

#include "clntsrv.h"

static int serialValidateObject = 0;

CollectableValidateObject::CollectableValidateObject() :
	RVCollectableInt(++serialValidateObject) {}

/*
int BaseEvent::isEqual(const CLObject& e) const
{
  return (&(((BaseEvent&)e).client) == &client &&
			((BaseEvent&)e).opcode == opcode);
}
*/

int BaseEvent::dispatchMe()
{
  assert(client != NULL);
  return (client->dispatch((BaseEvent&)*this));
}

int BaseEvent::dispatchEvent(BaseEvent& e)
{
  assert(client != NULL);
  e.opcode = opcode;
  return (client->dispatch(e));
}

//
// CLIENT
//

ClientObj::ClientObj(): TopService(0), NumServices(0)
{
	for(int index = 0; index < MAX_SERVICES; index++)
		Services[index] = NULL;
}

//
// Register a service function in the services table. Returns the service number.
//
int ClientObj::signOn(FUNCP funcp)
{
        int index;

	assert(funcp!=NULL);

	if( TopService == MAX_SERVICES )
		return(FAILURE);   // No more place

	if( NumServices == TopService )  // No "holes"
		{
			Services[++TopService] = funcp;
			NumServices = TopService;
			return(TopService);
		}
	else   // there are 'holes', so look for the first one
    {
      for(index = 0 ; index < TopService || Services[index] == NULL ; index++);
      assert(index == TopService);
      Services[index] = funcp;
      NumServices++;
      assert( NumServices <= TopService );

			return(index);
    }
}

//
// Ask a server object to register the service for the specified event
//
int ClientObj::pleaseRegister(ServerObj& destination, BaseEvent& event)
{
  return( destination.registerEvent(event) );
}
//
// UnRegister a service
//
void ClientObj::pleaseUnRegister(ServerObj& destination, BaseEvent& event)
{
	destination.unRegisterEvent(event);
}

void ClientObj::pleaseUnRegisterAll(ServerObj& destination)
{
  destination.unRegisterClient((ClientObj&)*this);
}

//
// Dispatch the events to the registered functions.
//
int ClientObj::dispatch(BaseEvent& event)
{
  assert(event.getOpcode() < MAX_SERVICES);
  assert(Services[event.getOpcode()] != NULL );

	return ((this->*(Services[event.getOpcode()]))(event));
}

//
// Client with EXTERNAL services
//

//
// SERVER
//
ServerObj::ServerObj(int tableSize)
{
  // Set flag to zero
	eventsTable = new RVHashTable(tableSize);
	assert(eventsTable != NULL);
}


ServerObj::~ServerObj()
{
  try {
	  assert(eventsTable != NULL);
	  eventsTable->clearAndDestroy();
	  delete eventsTable;
	  eventsTable = NULL;
  }  catch (...) {}
}

void ServerObj::dispatchEvent(BaseEvent& event)
{
	if (!event.dispatchMe())
		{
			ClientObj& client = event.getClient();
			unRegisterClient(client);
			delete &client;
		}
}

BaseEvent& ServerObj::findEvent(BaseEvent& event) const
{
	BaseEvent* be = (BaseEvent*)eventsTable->find(&event);
	return (*be);
}

int ServerObj::registerEvent(BaseEvent& event)
{
	//
	// remove equal event if any.
	ServerObj::unRegisterEvent(event);

	eventsTable->insert(&event);

	return TRUE;
}

void ServerObj::unRegisterEvent(BaseEvent& event)
{
	BaseEvent* be = (BaseEvent*)eventsTable->find(&event);
	if (be != NULL)
		eventsTable->remove(be);
}

void ServerObj::unRegisterClient(ClientObj& client)
{  
	if(eventsTable == NULL)
		return;
	RVHashTableIterator hti(*eventsTable);
	while (hti())
		{
			BaseEvent *event = (BaseEvent *)hti.key();
			if (event->isClient(client))
			{
				hti.remove();
				unRegisterEvent(*event);
				delete event;
			}
	}
}

