/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//processgroupmanager.cpp @@avidan42
#include "processgroupmanager.h"

/////////////////////////////////////////////////////////////////////////////////
//
//        The implementation of class ProcessGroup
//
/////////////////////////////////////////////////////////////////////////////////

ProcessGroup::ProcessGroup()
{
  processGroupVersion = PG_VER_LG_GENERAL;
}

ProcessGroup::~ProcessGroup()
{
}

	
ProcessID ProcessGroup::AddProcess(PROCESS_HANDLE pHandler) 
{
	ProcessInfo* pInfo = new ProcessInfo(pHandler);
	push(pInfo);
	return index(pInfo);
}

	
PROCESS_HANDLE ProcessGroup::RemoveProcess(ProcessID pID) 
{
	PROCESS_HANDLE pRetHandler;
	ProcessInfo* pInfo;
	pInfo = (ProcessInfo*)removeAt(pID);
	pRetHandler = pInfo->pHandle;
	delete pInfo;
	pInfo = NULL;
	return pRetHandler;
}

	
bool ProcessGroup::KillProcess(ProcessID pID)
 {
	bool Rtn;
	ProcessInfo* pInfo;
	pInfo = (ProcessInfo*)removeAt(pID);

	if(pInfo) 
        {
		if( TVPKillProcess(pInfo->pHandle) ) //'TVPKillProcess' close the handle.
                {
			Rtn = true;
		} 
                else 
                {
			Rtn = false;
		}

		delete pInfo;
		pInfo = NULL;
	}

	return Rtn;
}
	
bool ProcessGroup::TerminateGroup(void) 
{
	bool Rtn(true);
	ProcessInfo* pInfo;

	while( pInfo = (ProcessInfo*)pop() ) //iterate and kill all processes of the group.
	{
		if( TVPKillProcess(pInfo->pHandle) ) 
		{
		} 
		else 
		{
			Rtn = false;
		}

		delete pInfo;
		pInfo = NULL;
	}

	return Rtn;
}



/////////////////////////////////////////////////////////////////////////////////
//
//        The implementation of class ProcessGroupManager
//
/////////////////////////////////////////////////////////////////////////////////

ProcessGroup* ProcessesGroupManager::AddProcessToGroup(RVCString& groupID, PROCESS_HANDLE& pHandler) 
{
	RVCollectableString* keyCS = new RVCollectableString(groupID);
	ProcessGroup* listHandles = NULL;
	RVCollectable* retKey = NULL;

	if(listHandles = GetProcessGroup(groupID) ) {
			listHandles->AddProcess(pHandler);
	} 
        else {
		listHandles = new ProcessGroup();
		listHandles->AddProcess(pHandler); 
		if( ( retKey = insertKeyAndValue(keyCS, listHandles) ) == NULL ) {//the key is already in the collection
			return NULL;
		}
	}
	return listHandles;
}

ProcessGroup* ProcessesGroupManager::AddProcessToGroup(RVCString& groupID,
																											 PROCESS_HANDLE& pHandler,
																											 int processGroupVersion)
{
	ProcessGroup* processGroup = AddProcessToGroup(groupID, pHandler);

	if(processGroup != NULL)
	{
		processGroup->processGroupVersion = processGroupVersion;
	}

	return processGroup;
}

ProcessGroup* ProcessesGroupManager::GetProcessGroup(RVCString& key) 
{
	RVCollectableString keyCS(key);
	ProcessGroup* listHandles = NULL;
	if(listHandles = (ProcessGroup*)findValue(&keyCS) ) {
		return listHandles;
	}
	return listHandles;
}

bool ProcessesGroupManager::TerminateGroup(RVCString& groupKey) 
{
	bool Rtn(true);
	RVCollectableString* keyCS = NULL;
	ProcessGroup* listHandles = NULL;
	listHandles = GetProcessGroup(groupKey);
	if(listHandles) {
		Rtn = listHandles->TerminateGroup();
		RVCollectableString keyCS(groupKey);
		removeAndDestroy(&keyCS);
	} else {
		Rtn = false;
	}
	return Rtn;
}

