/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// SupportInformationGenerator.cpp: implementation of the CSupportInformationGenerator class.
//
//////////////////////////////////////////////////////////////////////

#include "supportinformationgenerator.h"
#include "socketlib.h"
#include "tvenv.h"
#include "platform.h"

#ifdef __TV_WINDOWS__
#define GETCWD _getcwd
#else
#define _MAX_PATH 260
#define GETCWD getcwd
#endif

 //////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSupportInformationGenerator::CSupportInformationGenerator()
{
}

CSupportInformationGenerator::~CSupportInformationGenerator()
{
}


//////////////////////////////////////////////////////////////////////
// Private methods implementation
//////////////////////////////////////////////////////////////////////

/************************************************************
#BEGIN: Methods for windows platform only
*************************************************************/
#if defined(__TV_WINDOWS__)
void CSupportInformationGenerator::GetDiskInfo(ostream& out)
{
	const int len = 200;
	char buffer[len];
	char *drive, driveTypeStr[20];
	const char *line = "\n-------------------------------------------------------------------\n";

	GetLogicalDriveStrings(len, buffer);

	int l = -1;
  drive = buffer;
	u_long lpSectorsPerCluster;
	u_long lpBytesPerSector;
	u_long lpNumberOfFreeClusters;
	u_long lpTotalNumberOfClusters;
	//ULARGE_INTEGER lpFreeBytesAvailable;
	//ULARGE_INTEGER lpTotalNumberOfBytes;
	//ULARGE_INTEGER lpTotalNumberOfFreeBytes;
	FARPROC pGetDiskFreeSpaceEx = GetProcAddress( GetModuleHandle("kernel32.dll"), "GetDiskFreeSpaceExA");

	out<<line<<"Logical Drives Information:"<<line;

  while(true) {
		drive += (l + 1);
		if(*drive == 0) break;

		UINT driveType = GetDriveType(drive);

		if(GetDriveType(drive) == DRIVE_FIXED)
			strcpy(driveTypeStr, " (Local Drive)");
		else {
			if(GetDriveType(drive) == DRIVE_REMOTE)
				strcpy(driveTypeStr, " (Remote Drive)");
			else {
				l = strlen(drive);
				continue;
			}
		}
		/*if(pGetDiskFreeSpaceEx) {
			GetDiskFreeSpaceEx(drive, (PULARGE_INTEGER)&lpFreeBytesAvailable,
				(PULARGE_INTEGER)&lpTotalNumberOfBytes,
				(PULARGE_INTEGER)&lpTotalNumberOfFreeBytes);
			out<<drive<<driveTypeStr;
			out<<"\n\tTotal Number Of Bytes: "<<lpTotalNumberOfBytes.LowPart;
			if(lpTotalNumberOfBytes.HighPart != 0) out<<lpTotalNumberOfBytes.HighPart;
			out<<"\n\tTotal Number Of Free Bytes: "<<lpTotalNumberOfFreeBytes.LowPart;
			if(lpTotalNumberOfFreeBytes.HighPart != 0) out<<lpTotalNumberOfFreeBytes.HighPart;
			out<<"\n";
		}
		else*/ {
			GetDiskFreeSpace(drive, &lpSectorsPerCluster, &lpBytesPerSector,
				&lpNumberOfFreeClusters, &lpTotalNumberOfClusters);
			out<<drive<<"\n";
			out<<"Sectors Per Cluster: "<<lpSectorsPerCluster<<"\n";
			out<<"Bytes Per Sector: "<<lpBytesPerSector<<"\n";
			out<<"Number Of Free Clusters: "<<lpNumberOfFreeClusters<<"\n";
			out<<"Total Number Of Clusters: "<<lpTotalNumberOfClusters<<"\n";
		}
		l = strlen(drive);
  }
}

void CSupportInformationGenerator::GetWindowsSystemInfo(ostream& out)
{
	LPTSTR lpszSystemInfo;     // pointer to system information string
	DWORD cchBuff = 256;       // size of computer or user name
	TCHAR tchBuffer2[BUFFER];  // buffer for concatenated string

	DWORD dwResult;            // function return value

	int aTabs[1] = {260};      // tab stop for TabbedTextOut

	TCHAR tchBuffer[BUFFER];   // buffer for expanded string
	int nSize;                 // size of string

	lpszSystemInfo = tchBuffer2;
	const char *line = "\n-------------------------------------------------------------------\n";

	// Get and display the name of the computer.
 	GetComputerName(lpszSystemInfo, &cchBuff);
 	sprintf(tchBuffer, "Computer name: %s", lpszSystemInfo);
	out<<tchBuffer<<"\n";


	// Get and display the system directory.
	GetSystemDirectory(lpszSystemInfo, MAX_PATH);
 	nSize = sprintf(tchBuffer, "System directory: %s", lpszSystemInfo);
	out<<tchBuffer<<"\n";

	// Get and display the Windows directory.
	GetWindowsDirectory(lpszSystemInfo, MAX_PATH);
	sprintf(tchBuffer, "Windows directory: %s", lpszSystemInfo);
	out<<tchBuffer<<"\n";

	// Display the "environment variables" header.
 	out<<line<<"Environment variables (partial list):"<<line;

	// Expand the OS and Processor environment variables.
 	dwResult = ExpandEnvironmentStrings(
							"OS=%OS% \
						 \nNUMBER_OF_PROCESSORS=%NUMBER_OF_PROCESSORS% \
						 \nPROCESSOR_ARCHITECTURE=%PROCESSOR_ARCHITECTURE% \
						 \nPROCESSOR_IDENTIFIER=%PROCESSOR_IDENTIFIER% \
						 \nPROCESSOR_LEVEL=%PROCESSOR_LEVEL% \
						 \nPROCESSOR_REVISION=%PROCESSOR_REVISION%",
						 lpszSystemInfo, BUFFER);

	out<<lpszSystemInfo<<"\n";

	// Expand the TMP and PATH environment variables.

	dwResult = ExpandEnvironmentStrings(
		"TMP=%TMP% \
		\nPATH=%PATH% \
		\nCLASSPATH=%CLASSPATH% \n",
		lpszSystemInfo,
		BUFFER);

	out<<lpszSystemInfo;
}

void CSupportInformationGenerator::GetHWProfile(ostream& out)
{
	SYSTEM_INFO siSysInfo;   // struct. for hardware information
	int aTabs[1] = {260};    // tab stop for TabbedTextOut

	TCHAR tchBuffer[BUFFER]; // buffer for expanded string
	int nSize;               // size of string

	const char *line = "\n-------------------------------------------------------------------\n";

	// Display the "hardware information" header.
	out<<line<<"Hardware information:"<<line;

	// Copy the hardware information to the SYSTEM_INFO structure.
	GetSystemInfo(&siSysInfo);

	// Display the contents of the SYSTEM_INFO structure.

	out<<"Number of Processors: "<<siSysInfo.dwNumberOfProcessors<<"\n";

	sprintf(tchBuffer,
		"Page size: %u\nProcessor Type: %u",
		siSysInfo.dwPageSize,
		siSysInfo.dwProcessorType);
	out<<tchBuffer<<"\n";

	nSize = sprintf(tchBuffer,
		"Minimum app address: %lx\tMaximum app address: %lx",
		siSysInfo.lpMinimumApplicationAddress,
		siSysInfo.lpMaximumApplicationAddress);
	out<<tchBuffer<<"\n";

	out<<"Processor Architecture: "<<siSysInfo.wProcessorArchitecture<<"\n";
	out<<"Processor Level: "<<siSysInfo.wProcessorLevel<<"\n";
	out<<"Processor Revision: "<<siSysInfo.wProcessorRevision<<"\n";
	out<<"Active Processor Mask: "<<siSysInfo.dwActiveProcessorMask<<"\n";
}

BOOL CSupportInformationGenerator::DisplaySystemVersion(ostream& out)
{
	OSVERSIONINFOEX osvi;
	BOOL bOsVersionInfoEx;

	// Try calling GetVersionEx using the OSVERSIONINFOEX structure,
	// which is supported on Windows 2000.
	//
	// If that fails, try using the OSVERSIONINFO structure.

	ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

	if( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &osvi)) )
	{
		// If OSVERSIONINFOEX doesn't work, try OSVERSIONINFO.

		osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
		if (! GetVersionEx ( (OSVERSIONINFO *) &osvi) )
			return FALSE;
	}

	switch (osvi.dwPlatformId)
	{
	case VER_PLATFORM_WIN32_NT:

		// Test for the product.

		if ( osvi.dwMajorVersion <= 4 )
			out<<"Microsoft Windows NT ";

		if ( osvi.dwMajorVersion == 5 )
			out<<"Microsoft Windows 2000 ";

		// Test for workstation versus server.

		else
		{
			HKEY hKey;
			char szProductType[80];
			DWORD dwBufLen;

			RegOpenKeyEx( HKEY_LOCAL_MACHINE,
				"SYSTEM\\CurrentControlSet\\Control\\ProductOptions",
				0, KEY_QUERY_VALUE, &hKey );
			RegQueryValueEx( hKey, "ProductType", NULL, NULL,
				(LPBYTE) szProductType, &dwBufLen);
			RegCloseKey( hKey );
			if ( lstrcmpi( "WINNT", szProductType) == 0 )
				out<<"Workstation ";
			if ( lstrcmpi( "SERVERNT", szProductType) == 0 )
				out<<"Server ";
		}

		// Display version, service pack (if any), and build number.

		out<<"version "<<osvi.dwMajorVersion<<"."<<osvi.dwMinorVersion<<" "<<
			osvi.szCSDVersion<<"(Build "<<(osvi.dwBuildNumber & 0xFFFF)<<")"<<"\n\n";

		break;

	case VER_PLATFORM_WIN32_WINDOWS:

		if ((osvi.dwMajorVersion > 4) ||
			((osvi.dwMajorVersion == 4) && (osvi.dwMinorVersion > 0)))
		{
			out<<"Microsoft Windows 98 \n";
		}
		else out<<"Microsoft Windows 95 \n";

		break;

	case VER_PLATFORM_WIN32s:

		out<<"Microsoft Win32s \n";
		break;
	}
	return TRUE;
}

void CSupportInformationGenerator::GetMemoryStatus(ostream& out)
{
	MEMORYSTATUS stat;
	char *divisor = "K"; // "M"
	const char *line = "\n-------------------------------------------------------------------\n";

	GlobalMemoryStatus (&stat);

	out<<line<<"Computer's Memory Status:"<<line;

	out<<"There are "<<(stat.dwTotalPhys/DIV)<<" total "<<divisor<<"bytes of physical memory.\n";
	out<<"There are "<<(stat.dwAvailPhys/DIV)<<" free "<<divisor<<"bytes of physical memory.\n";
	out<<"There are "<<(stat.dwTotalVirtual/DIV)<<" total "<<divisor<<"bytes of virtual memory.\n";
	out<<"There are "<<(stat.dwAvailVirtual/DIV)<<" free "<<divisor<<"bytes of virtual memory.\n";
}

void CSupportInformationGenerator::GetServicesList(ostream& out)
{
	const char *line = "\n-------------------------------------------------------------------\n";

	out<<line<<"Services Report:"<<line;

	SC_HANDLE schSCManager = OpenSCManager(
									         NULL,                   // machine (NULL == local)
													 NULL,                   // database (NULL == default)
													 SC_MANAGER_ALL_ACCESS); // access required
	if(schSCManager == NULL)
		return;

	#define                                       SZ_ENUM_BUF 4096
  ENUM_SERVICE_STATUS          essServiceStatus[SZ_ENUM_BUF];
  DWORD   dwBufSize					 = sizeof(essServiceStatus);
  DWORD   dwBytesNeeded      = 0;
  DWORD   dwServicesReturned = 0;
  DWORD   dwResumeHandle     = 0;
  DWORD   dwI                = 0;
  BOOLEAN bFound = FALSE;

  int status = EnumServicesStatus(schSCManager,
												          SERVICE_WIN32,
																  SERVICE_STATE_ALL,
																  (LPENUM_SERVICE_STATUS)&essServiceStatus,
																  dwBufSize,
																  &dwBytesNeeded,
																  &dwServicesReturned,
																  &dwResumeHandle);
	if(!status)
		return;

	for(u_long i = 0; i < dwServicesReturned ; i++) {
		out<<"Service Name: "<<essServiceStatus[i].lpDisplayName<<endl;

		switch(essServiceStatus[i].ServiceStatus.dwCurrentState)
		{
		case SERVICE_RUNNING:
			out<<"      Status: Running  ";
			break;
		case SERVICE_PAUSED:
			out<<"      Status: Paused   ";
			break;
		case SERVICE_STOPPED:
			out<<"      Status: Stoped   ";
			break;

		default:
			out<<"      Status: Stoped   ";
		}

		LPQUERY_SERVICE_CONFIG lpqscBuf;
    DWORD dwBytesNeeded;

    // Open a handle to the service.

    SC_HANDLE schService = OpenService(
							schSCManager,											 // SCManager database
							essServiceStatus[i].lpServiceName, // name of service
							SERVICE_QUERY_CONFIG);             // need QUERY access

    if(schService != NULL) {

			// Allocate a buffer for the configuration information.

			lpqscBuf = (LPQUERY_SERVICE_CONFIG) LocalAlloc(LPTR, 4096);

			// Get the configuration information.
			if(QueryServiceConfig(schService, lpqscBuf, 4096, &dwBytesNeeded)) {
				switch(lpqscBuf->dwStartType)
				{
				case SERVICE_AUTO_START:
					out<<"(Automatic)"<<endl;
					break;
				case SERVICE_DEMAND_START:
					out<<"(Manual)"<<endl;
					break;
				case SERVICE_DISABLED:
					out<<"(Disabled)"<<endl;
					break;
				case SERVICE_SYSTEM_START:
					out<<"(System)"<<endl;
					break;
				case SERVICE_BOOT_START:
					out<<"(Boot)"<<endl;
					break;
				}
			}
		}
	}
	out<<endl;
}

bool CSupportInformationGenerator::GenerateWindowsInfo(/*[IN]*/const char *_SysInfoFileName)
{
	ofstream informationStream(_SysInfoFileName);

	if(informationStream.fail()) {
		return false;
	}

	DisplaySystemVersion(informationStream);
	GetWindowsSystemInfo(informationStream);
	GetHWProfile(informationStream);
	GetMemoryStatus(informationStream);
	GetDiskInfo(informationStream);
	GetServicesList(informationStream);
	informationStream.close();

	return true;
}
#endif
/************************************************************
#END: Methods for windows platform only
*************************************************************/

bool CSupportInformationGenerator::GenerateIPConfigInfo(/*[IN]*/const char *_IpConfigFileName)
{
#if defined(__TV_WINDOWS__)
	OSVERSIONINFO osvi;
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx (&osvi);

	// If OS is Windows NT or Windows 2000
	if(osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) {
		RVCString commandLine = "ipconfig.exe ";
		commandLine += "/all >> ";
		commandLine += _IpConfigFileName;

		if(system(commandLine.data()) >= 0)
			return true;
	}
	else {
		// If OS is Windows 95 or Windows 98
		if(osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) {
			RVCString commandArgs = " /all /batch ";
			commandArgs += "\"";
      commandArgs += _IpConfigFileName;
      commandArgs += "\"";

			if(TVPExecApplication("winipcfg", commandArgs.data()))
				return true;
		}
		else // OS is Win32s on Windows 3.1
			return false; // generate error
	}
#endif
	return false;
}

bool CSupportInformationGenerator::CheckGenerateError(int status)
{
	if(status < 0 || (!status && errno == ENOENT)) {
		return false;
	}
	return true;
}


//////////////////////////////////////////////////////////////////////
// Public methods implementation
//////////////////////////////////////////////////////////////////////

bool CSupportInformationGenerator::GenerateSystemInfo(/*[IN]*/char* _IpConfigFileName,
																											/*[IN]*/char* _SysInfoFileName)
{
	bool status = false;

#if defined(__TV_WINDOWS__)

	if(GenerateIPConfigInfo(_IpConfigFileName) &&
		 GenerateWindowsInfo(_SysInfoFileName)      ) {

		status = true;
	}

#else

	/*char command[COMMAND_LINE_SIZE];
	strcpy(command, "unix.bat ");    // replace batch file name
	strcat(command, args);

	int status = system(command);*/

#endif

	return status;
}

bool CSupportInformationGenerator::GetWLDirectoryPath(char *string, int strLength)
{
	if( GETCWD( string, strLength ) == NULL ) {
		return false;
	}
	strcat(string, "\\");
	return true;
}

bool CSupportInformationGenerator::CreateInformationDB(CAssociativeArray &infoDB)
{
	char _IpConfigFileName[_MAX_PATH + 1];
	char _SysInfoFileName[_MAX_PATH + 1];

	char hostName[50];
	CSocketLib::GetLocalHostName(hostName, 50);
	infoDB.AddCouple("$HOSTNAME", hostName);

   /* Get the current working directory: */
	GetWLDirectoryPath(_SysInfoFileName, _MAX_PATH);
	GetWLDirectoryPath(_IpConfigFileName, _MAX_PATH);

	strcat(_IpConfigFileName, "IpConfig.txt"); 
	strcat(hostName, ".txt");
	strcat(_SysInfoFileName, hostName);

  //  [7/11/2005] Alon : add commas for full path
  RVCString sIpConfigFileName("\"" );
  sIpConfigFileName += _IpConfigFileName ;
  sIpConfigFileName += "\"";

  RVCString sSysInfoFileName("\"" );
  sSysInfoFileName += _SysInfoFileName ;
  sSysInfoFileName += "\"";

	//if( GenerateSystemInfo(_IpConfigFileName, _SysInfoFileName) ) {
  if( GenerateSystemInfo((char *)sIpConfigFileName.data(), (char *)sSysInfoFileName.data()) ) {
		AddFileToDB("$SYSTEMREPORT", _SysInfoFileName, infoDB);
		AddFileToDB("$IPCONFIG", _IpConfigFileName, infoDB);

		remove(_SysInfoFileName);  // DeleteFile
		remove(_IpConfigFileName); // DeleteFile
	}
	else {
		infoDB.AddCouple("$IPCONFIG", "Not Available - failed on creation");
		infoDB.AddCouple("$SYSTEMREPORT", "Not Available - failed on creation");
	}

	TVEnvContainerClass tvpc;
	if(!AddFileToDB("$TVCTIDLL", tvpc.TVERuntimeResolve("#(TV_BIN)\\tvcti.dll"), infoDB))
		infoDB.AddCouple("$TVCTIDLL", "Not Found");

	tvpc.reset();
	if(!AddFileToDB("$WEBLOADINI", tvpc.TVERuntimeResolve("#(TV_BIN)\\webload.ini"), infoDB))
		infoDB.AddCouple("$WEBLOADINI", "Not Found");

	return true;
}

bool CSupportInformationGenerator::AddFileToDB(const char *key, const char *_FilePath, CAssociativeArray &infoDB)
{
	RVCString string;

	// Handle possible errors on next lines.
	ifstream _DB_FILE(_FilePath);
	if(_DB_FILE.fail()) {
		return false;
	}

	//string.readFile(_DB_FILE);
	readFile(string,_DB_FILE);
	infoDB.AddCouple(key, string.data());
	_DB_FILE.close();

	return true;
}

bool CSupportInformationGenerator::GenerateReportFile(ofstream &reportFile, CAssociativeArray &infoDB)
{
	const char *string;
	const char *line = "\n===============================================================================\n";

	if(!infoDB.KeyExists("$HOSTNAME")) {
		return false;
	}

	try {
		string = infoDB.GetValueOfParameter("$HOSTNAME");
		reportFile<<line<<"Complete Report For Host: "<<string<<endl<<line;

		string = infoDB.GetValueOfParameter("$IPCONFIG");
		reportFile<<line<<"IP Configuration: "<<line<<endl<<string;

		string = infoDB.GetValueOfParameter("$SYSTEMREPORT");
		reportFile<<line<<"System And Hardware Configuration: "<<line<<endl<<string;

		string = infoDB.GetValueOfParameter("$TVCTIDLL");
		reportFile<<line<<"Webload Product Information(tvcti.dll): "<<line<<endl<<string;

		string = infoDB.GetValueOfParameter("$WEBLOADINI");
		reportFile<<line<<"Webload.ini File: "<<line<<endl<<string;
	}
	catch(...) {
		return false;
	}
	return true;
}
