/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//
// Implementation class for TestTalkServiceConfig
#include "testtalkserviceconfig.h"
#include "tvenv.h"
#include <iostream.h>
#include <RVCString.h>
#include <stdio.h>


TestTalkServiceConfig::TestTalkServiceConfig(const char* szServiceName, 
																						 const char* szDisplay)
{
	strcpy(m_service, szServiceName);
	strcpy(m_display, szDisplay);
}

bool TestTalkServiceConfig::ResolveTestTalkServiceName(void)
{
	TVEnvContainerClass tvpc;
	const char* version	= tvpc.TVEGetEnv("TESTTALK_SERVICE_VERSION_NAME");

	if(version) {
		RVCString versionName    = version;
		RVCString serviceName    = TESTALK_SERVICE_NAME;    // defined in TestTalkServiceConfig.h
		RVCString serviceDisplay = TESTALK_SERVICE_DISPLAY; // defined in TestTalkServiceConfig.h

		serviceName += versionName;
		serviceDisplay += " ";
		serviceDisplay += versionName;

		strcpy(m_service, serviceName.data());
		strcpy(m_display, serviceDisplay.data());

		return true;
	}
	return false;
}

void TestTalkServiceConfig::Install(DWORD dwType, DWORD dwStart,
																		const char* lpDepends, 
																		const char* lpName,
																		const char* lpPassword)
{
	SC_HANDLE hSCM = NULL;
	SC_HANDLE hService = NULL;
	
	if(IsInstalled() == TRUE)
		return;
	
	char szFilePath[_MAX_PATH];
	::GetModuleFileName(NULL, szFilePath, _MAX_PATH);

	hSCM = OpenSCManager(NULL, NULL, SC_MANAGER_CREATE_SERVICE);
	if(!hSCM)
	{
		ErrorPrinter("OpenSCManager");
		goto cleanup;
	}
	
	hService = CreateService(hSCM,
		m_service,
		m_display,
		SERVICE_ALL_ACCESS,
		dwType,
		dwStart,
		SERVICE_ERROR_NORMAL,
		szFilePath,
		NULL,
		NULL,
		lpDepends,
		lpName,
		lpPassword);
	if(!hService)
	{
		ErrorPrinter("CreateService");
		goto cleanup;
	}
	else
	{
		printf("%s Created\n", m_service);
	}
	
cleanup:
	if(hService) CloseServiceHandle(hService);
	if(hSCM) CloseServiceHandle(hSCM);
	
	return;
}

void TestTalkServiceConfig::Remove(BOOL bForce)
{
	SC_HANDLE hSCM = NULL;
	SC_HANDLE hService = NULL;
	BOOL bSuccess = FALSE;
	hSCM = ::OpenSCManager(NULL, NULL, SC_MANAGER_CONNECT);
	if(!hSCM)
	{
		ErrorPrinter("OpenSCManager");
		return;
	}
	
	hService = ::OpenService(hSCM, m_service, DELETE);
	if(!hService)
	{
		ErrorPrinter("OpenService");
		goto cleanup;
	}
	
	// Force the service to stop 
	if(bForce == TRUE)
	{
		SERVICE_STATUS status;
		::ControlService(hService, SERVICE_CONTROL_STOP, &status);
		printf("%s stopped\n", m_service);
	}
	
	bSuccess = ::DeleteService(hService);
	if (bSuccess)
		printf("%s removed\n", m_service); 
	else
		ErrorPrinter("DeleteService");
	
cleanup:
	if(hService) CloseServiceHandle(hService);
	if(hSCM) CloseServiceHandle(hSCM);
	
	return;
}

BOOL TestTalkServiceConfig::IsInstalled()
{
	BOOL bRes = FALSE;
	
	SC_HANDLE hSCM = ::OpenSCManager(NULL, NULL, SC_MANAGER_CONNECT);
	if(hSCM != NULL)
	{
		SC_HANDLE hService = ::OpenService(hSCM, m_service, SERVICE_QUERY_CONFIG);
		if(hService != NULL)
		{
			bRes = TRUE;
			::CloseServiceHandle(hService);
		}
		::CloseServiceHandle(hSCM);
	}
	return bRes;
}

void TestTalkServiceConfig::ChangeStartType(BOOL bAutostart)
{
	SC_HANDLE hSCM = 0;
	SC_HANDLE hService = 0;
	SC_LOCK hLock = 0; 
	DWORD dwStartType = 0;
	BOOL bRet = FALSE;
	
	hSCM = OpenSCManager(NULL, NULL, GENERIC_READ | GENERIC_EXECUTE);
	if(!hSCM)
	{
		ErrorPrinter("OpenSCManager");
		goto cleanup;
	}
	
	hLock = LockDatabase(hSCM); // Wrapper tells us who locked if locked
	if(!hLock)
	{
		printf("Service Configuration could not be changed\n");
		goto cleanup;
	}
	
	hService = OpenService(hSCM, m_service, SERVICE_CHANGE_CONFIG);
	if(!hService)
	{
		ErrorPrinter("OpenService");
		goto cleanup;
	}
	
	dwStartType = (bAutostart) ? SERVICE_AUTO_START : SERVICE_DEMAND_START;
	bRet = ChangeServiceConfig(hService,
		SERVICE_NO_CHANGE,
		dwStartType,
		SERVICE_NO_CHANGE,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL);
	
	if(bRet)
		printf("Changed Service Configuration Successfully\n");
	else
		ErrorPrinter("ChangeServiceConfig");
	
cleanup:
	if(hService) CloseServiceHandle(hService);
	if(hLock) UnlockServiceDatabase(hLock);
	if(hSCM) CloseServiceHandle(hSCM);
	return;
}

void TestTalkServiceConfig::GetConfig()
{
	SC_HANDLE hSCM = 0;
	SC_HANDLE hService = 0;
	BOOL bRet = FALSE;
	
	LPQUERY_SERVICE_CONFIG pqscBuf = {0};
	DWORD dwBytesNeeded = 0;
	
	hSCM = OpenSCManager(NULL, NULL, GENERIC_READ);
	if(!hSCM)
	{
		ErrorPrinter("OpenSCManager");
		goto cleanup;
	}
	
	hService = OpenService(hSCM, m_service, SERVICE_QUERY_CONFIG);
	if(!hService)
	{
		ErrorPrinter("OpenService");
		goto cleanup;
	}
	
	// Obtain the number of bytes needed
	bRet = QueryServiceConfig(hService, 0, 0, &dwBytesNeeded);
	
	// Allocate the proper size
	pqscBuf = (LPQUERY_SERVICE_CONFIG)LocalAlloc(LPTR, dwBytesNeeded);
	if(!pqscBuf)
	{
		ErrorPrinter("LocalAlloc", TRUE);
		goto cleanup;
	}
	
	// Call with the right amount
	bRet = QueryServiceConfig(hService,	pqscBuf, dwBytesNeeded, &dwBytesNeeded);
	if(bRet)
	{
		printf("\nService Configuration for %s: \n", m_service);
		printf("Display Name: %s \n", pqscBuf->lpDisplayName);
		printf("Type: 0x%d \n", pqscBuf->dwServiceType);
		printf("Start Type: 0x%d \n", pqscBuf->dwStartType);
		printf("Error Level: 0x%d \n", pqscBuf->dwErrorControl);
		printf("Binary path: %s \n", pqscBuf->lpBinaryPathName);
		printf("Load Order Group: %s \n", pqscBuf->lpLoadOrderGroup);
		printf("Tag ID: %d \n", pqscBuf->dwTagId);
		printf("Dependencies:  \n"); 
		
		if(pqscBuf->lpDependencies)
		{
			char* pszDepend = 0;
			int i = 0;
			pszDepend = &pqscBuf->lpDependencies[i];
			while(*pszDepend != 0) 
			{
				printf("%s \n", pszDepend);
				i += strlen(pszDepend) + 1;
				pszDepend = &pqscBuf->lpDependencies[i];
			}
		}
		printf("Login Under: %s \n", pqscBuf->lpServiceStartName);
	}
	else
		ErrorPrinter("QueryServiceConfig");
	
cleanup:
	if(pqscBuf) LocalFree(pqscBuf);
	if(hService) CloseServiceHandle(hService);	
	if(hSCM) CloseServiceHandle(hSCM);
	
	return;
}

void TestTalkServiceConfig::Status()
{
	SC_HANDLE hSCM = 0;
	SC_HANDLE hService = 0 ;
	BOOL bRet = FALSE;
	SERVICE_STATUS ss;
	
	hSCM = OpenSCManager(NULL, NULL, GENERIC_READ);
	if(!hSCM)
	{
		ErrorPrinter("OpenSCManager");
		goto cleanup;
	}
	
	hService = OpenService(hSCM, m_service, SERVICE_QUERY_STATUS);
	if(!hService)
	{
		ErrorPrinter("OpenService");
		goto cleanup;
	}
	
	bRet = QueryServiceStatus(hService, &ss);
	if(bRet)
	{
		printf("Service Status for %s \n", m_service);
		printf("Type: 0x%d \n", ss.dwServiceType);
		printf("Current State: 0x%d \n", ss.dwCurrentState);
		printf("Controls Accepted: %d \n", ss.dwControlsAccepted);
		printf("Win32 Exit: %d \n", ss.dwWin32ExitCode);
		printf("Service Exit: %d \n", ss.dwServiceSpecificExitCode);
		printf("Checkpoint: 0x%d \n", ss.dwCheckPoint);
		printf("WaitHint: 0x%d \n", ss.dwWaitHint);
	}
	else
		ErrorPrinter("QueryServiceStatus");
	
cleanup:
	if(hService) CloseServiceHandle(hService);
	if(hSCM) CloseServiceHandle(hSCM);
	return;
}

void TestTalkServiceConfig::Dependencies()
{
	SC_HANDLE hSCM = 0;
	SC_HANDLE hService = 0;
	BOOL bRet = 0;
	
	DWORD dwBytesNeeded = 0;
	DWORD dwServicesReturned = 0;
	LPENUM_SERVICE_STATUS pessBuf = {0};
	
	hSCM = OpenSCManager(NULL, NULL, GENERIC_READ);
	if(!hSCM)
	{
		ErrorPrinter("OpenSCManager");
		goto cleanup;
	}
	
	hService = OpenService(hSCM, m_service, SERVICE_ENUMERATE_DEPENDENTS);
	if(!hService)
	{
		ErrorPrinter("OpenService");
		goto cleanup;
	}
	
	// Find out how much to allocate
	bRet = EnumDependentServices(hService,
		SERVICE_STATE_ALL,
		0,
		0,
		&dwBytesNeeded,
		&dwServicesReturned);
	
	if(dwBytesNeeded == 0)
	{
		printf("%s has no dependencies\n", m_service);
		goto cleanup;
	}
	
	// Allocate the proper size
	pessBuf = (LPENUM_SERVICE_STATUS)LocalAlloc(LPTR, dwBytesNeeded);
	if(!pessBuf)
	{
		ErrorPrinter("LocalAlloc", TRUE);
		goto cleanup;
	}
	
	bRet = EnumDependentServices(hService,
		SERVICE_STATE_ALL,
		pessBuf,
		dwBytesNeeded,
		&dwBytesNeeded,
		&dwServicesReturned);
	if(bRet)
	{
		LPENUM_SERVICE_STATUS pess = 0;
		printf("Services Dependent On %s \n", m_service);
		for(DWORD i = 0 ; i < dwServicesReturned ; i++)
		{
			pess = &pessBuf[i];
			printf("%s \n", pess->lpDisplayName);
			printf("Current State: 0x%d \n", pess->ServiceStatus.dwCurrentState);
		}
	}
	else
		ErrorPrinter("EnumDependentServices");
	
cleanup:
	if(pessBuf) LocalFree(pessBuf);
	if(hService) CloseServiceHandle(hService);
	if(hSCM) CloseServiceHandle(hSCM);
	
	return;
}

void TestTalkServiceConfig::Start()
{
	SC_HANDLE hSCM = 0;
	SC_HANDLE hService = 0;
	BOOL bRet = 0;
	DWORD dwOldCheck = 0;
	SERVICE_STATUS ss;
	
	hSCM = OpenSCManager(NULL, NULL, GENERIC_READ);
	if(!hSCM)
	{
		ErrorPrinter("OpenSCManager");
		goto cleanup;
	}
	
	hService = OpenService(hSCM, m_service, SERVICE_START | SERVICE_QUERY_STATUS);
	if(!hService) 
	{
		ErrorPrinter("OpenService");
		goto cleanup;
	}
	
	if(!StartService(hService, 0, NULL))
	{
		ErrorPrinter("StartService");
		goto cleanup;
	}
	
	// Optionally, make sure service is running before continuing...
	bRet = QueryServiceStatus(hService, &ss);
	if(!bRet)
	{
		ErrorPrinter("QueryServiceStatus", TRUE);
		goto cleanup;
	}
	
	while(SERVICE_RUNNING != ss.dwCurrentState)
	{
		dwOldCheck = ss.dwCheckPoint;
		Sleep(ss.dwWaitHint);
		
		bRet = QueryServiceStatus(hService, &ss);
		if(!bRet)
		{
			ErrorPrinter("QueryServiceStatus", TRUE);
			goto cleanup;
		}
		
		// Service changed state or did not increment in time, so break
		if(dwOldCheck >= ss.dwCheckPoint)
			break;
	}
	
	if(SERVICE_RUNNING == ss.dwCurrentState)
		printf("%s started successfully.\n", m_service);
	else
	{
		printf("%s start unsuccessful: \n", m_service);
		printf("Current State: 0x%d \n", ss.dwCurrentState);
		printf("Win32 Exit: %d \n", ss.dwWin32ExitCode);
		printf("Service Exit: %d \n", ss.dwServiceSpecificExitCode);
		printf("Checkpoint: 0x%d \n", ss.dwCheckPoint);
		printf("WaitHint: 0x%d \n", ss.dwWaitHint);
	}
	
cleanup:
	if(hService) CloseServiceHandle(hService);
	if(hSCM) CloseServiceHandle(hSCM);
	return;
}

void TestTalkServiceConfig::Control(DWORD dwControl)
{
	SC_HANDLE hSCM = 0;
	SC_HANDLE hService = 0;
	BOOL bRet = 0;
	
	SERVICE_STATUS ss;
	
	hSCM = OpenSCManager(NULL, NULL, GENERIC_READ);
	if(!hSCM)
	{
		ErrorPrinter("OpenSCManager");
		goto cleanup;
	}
	
	hService = OpenService(hSCM, m_service, GENERIC_EXECUTE);
	if(!hService) 
	{
		ErrorPrinter("OpenService");
		goto cleanup;
	}
	
	bRet = ControlService(hService, dwControl, &ss);
	if(bRet)
	{
		printf("Service Status for %s \n", m_service);
		printf("Type: 0x%d \n", ss.dwServiceType);
		printf("Current State: 0x%d \n" ,ss.dwCurrentState);
		printf("Controls Accepted: %d\n" ,ss.dwControlsAccepted);
		printf("Win32 Exit: %d\n" ,ss.dwWin32ExitCode);
		printf("Service Exit: %d\n" ,ss.dwServiceSpecificExitCode);
		printf("Checkpoint: 0x %d\n" ,ss.dwCheckPoint);
		printf("WaitHint: 0x %d\n" ,ss.dwWaitHint);		
	}
	else
		ErrorPrinter("ControlService");
	
cleanup:
	if (hService) CloseServiceHandle(hService);
	if (hSCM) CloseServiceHandle(hSCM);
	return;
}

// Utility Function locks the SCM database and feeds back status on error
SC_LOCK TestTalkServiceConfig::LockDatabase(SC_HANDLE hSCM)
{
	SC_LOCK hLock = 0;
	LPQUERY_SERVICE_LOCK_STATUS pqlsBuf;
	hLock = LockServiceDatabase(hSCM);
	
	// Lock not available
	if(!hLock)
	{
		DWORD dwBytesNeeded = 0;
		
		// Allocate buffer for lock information
		// includes enough for service
		// name string--no need to re-call
		pqlsBuf = (LPQUERY_SERVICE_LOCK_STATUS)LocalAlloc(LPTR, sizeof(QUERY_SERVICE_LOCK_STATUS) + MAX_SERVICE_NAME + 1);
		if(!pqlsBuf)
			ErrorPrinter("LocalAlloc", TRUE);
		
		BOOL bRet = QueryServiceLockStatus(hSCM, pqlsBuf,
			sizeof(QUERY_SERVICE_LOCK_STATUS) + MAX_SERVICE_NAME + 1, &dwBytesNeeded);
		if(!bRet)
		{
			ErrorPrinter("QueryServiceLockStatus");
			LocalFree(pqlsBuf);
			return NULL;
		}
		
		// Output who locked it if locked
		if(pqlsBuf->fIsLocked)
			printf("Locked by: %s for %d seconds\n", pqlsBuf->lpLockOwner, pqlsBuf->dwLockDuration);
		else
			printf("Database Not Locked\n");	
		
		LocalFree(pqlsBuf);
	}
	return hLock;
}

DWORD TestTalkServiceConfig::ErrorPrinter(const char* psz, DWORD dwErr)
{
	LPVOID lpvMsgBuf;
	if(!FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER,
		0, dwErr, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPSTR   )&lpvMsgBuf, 0, 0))
	{
		printf("%s failed: Unknown error %x\n", psz, dwErr);
	}
	else
		printf("%s failed: %s\n", psz, (LPSTR   )lpvMsgBuf);
	
	LocalFree(lpvMsgBuf);
	return dwErr;
}
