/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __TRACE_DLL_H
#define __TRACE_DLL_H

#define TRACE_MESSAGE_LEN 512
#define MESSAGE_LEN 102400
#define TRACE_OUTPUT_MSG  100
#define TRACE_GETLINE_MSG 101

#ifndef __WL_TRACE__
  #ifndef __WL_NOTRACE__
    #ifdef _DEBUG
    #define __WL_TRACE__
    #endif
  #endif
#endif

#include "platform.h"
#include "stdio.h"
#include <RVCString.h>
#include <thr.h>
#include <hashtab.h>
#include <hashdict.h>

#ifdef __TV_UNIX__
#include <thr.h>
#endif



#ifndef __TV_UNIX__
#ifdef TRACEDLL_EXPORTS
#define TRACEDLL_API __declspec(dllexport)
#else
#define TRACEDLL_API __declspec(dllimport)
#endif
#else
#define TRACEDLL_API
#define WINAPI
#endif

TRACEDLL_API int WINAPI TraceSendTextFunc(const char* msg, ...);

extern TRACEDLL_API bool WlTrace;
extern TRACEDLL_API bool WlSessionTrace;

// ***********************************************
// Trace Class. An instance of this class is created
// for each trace message.
//
// Aviad Israeli (24/1/2000)
// ***********************************************
class TRACEDLL_API CTraceMsg {
public:
  CTraceMsg(int Stam) {} ;
	int PreDefVals(const char *pModuleName,
                 const char *pFileName,
                 const int  LineNo) ;

  int SendMsg (const char *pMsg, ...) ;
  int SendErr (const char *pMsg, ...) ;
  int TraceMsgNoFilter(const char *pMsg, ...); 
  int SendErrWithCallStack (const char *pMsg, ...) ;
  int SendErrAssert (int assertInd,const char *pMsg, ...) ;
  int SendErrAssert (const char *pMsg, ...) ;
//#if defined(__TV_solaris__)
//CTraceMsg &operator=(const int Stam) {return (*this);}
//#endif

  static void Init (const char *pProgramName) ;
	static void SetTrace (const bool Mode, const bool ToFile = true, const char *pProgramName = NULL) ;
	static RVCString GetTraceFileName();
	static RVCString GetErrorFileName();
	static RVCString GetTracePath();
	static void RegisterThread (int virtualClientNum = -1);
	static void RegisterThreadInfo (const char* threadInfo = "Virtual Client?");
	static void RegisterAgenda (const char* agendaName);
	static void UnregisterAgenda (const char* agendaName);
  static void SetSessionTrace(const char* sTraceFileName, const char* sModuleList = NULL);
  static void CloseSessionTrace();

private:
#ifndef __TV_UNIX__
  static char m_ProgramName[_MAX_PATH];  //Program name (i.e. Exe component
#else
  static char m_ProgramName[MAX_PATH];  //Program name (i.e. Exe component
#endif
  static RVCString m_TraceModules ;      //List of module names to trace
  static RVCString m_NotTraceModules;    //List of module names not to trace
  static bool m_ToTraceFile ;            //Indication whether to trace to a file
  static FILE *m_pTraceFile ;            //A handle to the trace file
  static RVCString m_TraceFileName ;     //The name of the trace file
  static RVCriticalSection m_Guard ;     //Lock guard for critical section
  static long m_OutLineNo ;              //Seq no. for the output lines
  static long m_fileSeq;                 //Seq no. for the file name
  static long m_recordSizeCount;
  static RVCString m_TraceFilePath;
  static long m_FileSizeLimit;
  static long m_FileLimit;
  static long m_ThreadLimit;
	static long m_ThreadStart;
  static long m_ErrFileSeq;                 //Seq no. for the file name
  static long m_ErrRecordSizeCount;
  static long m_ErrFileSizeLimit;
  static long m_ErrFileLimit;
  static bool m_ErrFileCyclic;
  static RVHashTable m_ThreadHashTab;
  static RVHashDictionary m_ThreadInfoHashDict;
  static RVHashTable m_AgendaNameHashTab;
  static bool m_FirstWriteToErr;
  static bool m_FirstWriteToTrace;
  static RVCString m_ErrorFileName;
#ifndef __TV_UNIX__
  static RVCriticalSection m_Guard_err ;
#else
  static RVTRecursiveLock<RVMutexLock> _m_Guard_err;
#endif
  static FILE *m_pErrorFile ;
  static long m_OutLineNoErr;
	static long m_LineSizeLimit;
  static FILE *m_pSessionTraceFile ;
  static RVCString m_SessionTraceModules ;      //List of module names to trace

  static void OpenTraceFile () ;
  static void OpenErrorFile () ;

  char m_ModuleName[50] ;               //The current link component (i.e. Dll name, Lib Name, Exe name)
#ifndef __TV_UNIX__
  char m_FileName[_MAX_PATH] ;          //Source file name for the trace message
#else
  char m_FileName[MAX_PATH] ;          //Source file name for the trace message
#endif
  int  m_LineNo ;                       //Line no in source file where the trace was activated.
};

#ifndef __MODULE__
#error For the TraceAPP module, Add to the preprocessor options of this project (Debug & Release): __MODULE__="ProgectName"
#endif

//#if !defined(__TV_solaris__) || defined(__TV_solaris7__)
#define WlTraceMsg if (WlTrace || WlSessionTrace) CTraceMsg TrcObj=0 || TrcObj.PreDefVals (__MODULE__,__FILE__,__LINE__) || TrcObj.SendMsg
#define WlTraceMsgPosition( drModule, drFile, drLine ) if (1) CTraceMsg TrcObj=0 || TrcObj.PreDefVals ( (drModule), (drFile), (drLine) ) || TrcObj.SendMsg
//#else
// The 'xar' command cannot handle the following construct correctly, and fatal compilation errors are
// generated during compilations during 'xar' processing.  This error occurs for CC, Version 4.2, 30 Oct, 1996.
// #define WlTraceMsg if (WlTrace) CTraceMsg TrcObj=0 || TrcObj.PreDefVals (__MODULE__,__FILE__,__LINE__) || TrcObj.SendMsg
// Use the following work-around to keep the port going. (tlf, 03-April-2000)
//#define __WL_TRACE__
//#define WlTraceMsg if(WlTrace) TraceSendError
//#endif

//#ifdef __WL_TRACE__
//#define TraceSendText TraceSendTextFunc
//#define TraceSendError TraceSendTextFunc
//#else
#define TraceSendText
#define TraceSendError TraceSendTextFunc
//#endif

#define WlErrorMsg if (1) CTraceMsg ErrObj=0 || ErrObj.PreDefVals (__MODULE__,__FILE__,__LINE__) || ErrObj.SendErr
#define WlErrorMsgPosition( drModule, drFile, drLine ) if (1) CTraceMsg ErrObj=0 || ErrObj.PreDefVals ( (drModule), (drFile), (drLine) ) || ErrObj.SendErrWithCallStack
#define WlErrorMsgAssert if (1) CTraceMsg ErrObj=0 || ErrObj.PreDefVals (__MODULE__,__FILE__,__LINE__) || ErrObj.SendErrAssert
#define WlAssertMsg( drAssert ) if(!(drAssert)) WlErrorMsgAssert
#define WlAssert( drAssert ) WlAssertMsg( (drAssert) )("")

extern "C" DllExport void RvErrorMsgPosition( const char* module, const char* file, long line, const char* s );
extern "C" DllExport void RvTraceMsgPosition( const char* module, const char* file, long line, const char* s );


#ifdef _DEBUG
#ifdef __TV_WINDOWS__
#define _USE_PURIFY_
#endif // __TV_WINDOWS__
#endif // _DEBUG

#ifdef _USE_PURIFY_
DllExport void ReportNewPurifyLeaks(bool bRound,const char *msg=NULL);
DllExport void ReportAllPurifyLeaks(bool bRound,const char *msg=NULL);
DllExport void ClearPurifyLeaks(bool bRound,const char *msg=NULL);
DllExport void ReportPurifyMemory(bool bRound,const char *msg=NULL);
#endif // _USE_PURIFY_

#ifdef _USE_PURIFY_
#define PURIFY_NEW_LEAKS(round,msg) ReportNewPurifyLeaks(round,msg)
#define PURIFY_ALL_LEAKS(round,msg) ReportAllPurifyLeaks(round,msg)
#define PURIFY_CLEAR_LEAKS(round,msg) ClearPurifyLeaks(round,msg)
#define PURIFY_MEMORY(round,msg) ReportPurifyMemory(round,msg)
#else
#define PURIFY_NEW_LEAKS(round,msg)
#define PURIFY_ALL_LEAKS(round,msg)
#define PURIFY_CLEAR_LEAKS(round,msg)
#define PURIFY_MEMORY(round,msg)
#endif // _USE_PURIFY_


#endif // __TRACE_DLL_H

