/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifdef __TV_WINDOWS__
#include "stdafx.h"
#endif

#include "tracedll.h"
#include "nvsmsg.h"
#include "wlerrorlib.h"

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <assert.h>

#include <stdlib.h>
#include <time.h>
#include <sys/timeb.h>
#include "tvenv.h"
#include <RVCString.h>
#include <regexp.h>
#include <thr.h>
#include <thr.h>
#include <hashtab.h>
#include <collint.h>
#include <collstr.h>
#ifdef _USE_PURIFY_
#ifdef __TV_WINDOWS__
#include "pure.h"
#endif // __TV_WINDOWS__
#ifdef __TV_solaris__
#include "purify.h"
#endif // __TV_solaris__
#endif // _USE_PURIFY_
#include <sys/stat.h>

#define CHAR_SEPARATOR "@"
#define MESSAGE_LINE_SIZE 250
// declaration for PurifyLog
static bool s_bPurifyRound = false;
static bool s_bPurifyInstrumentation = false;
static bool s_bPurifyInstrumentInuseMemory = false;

//Declarations of static data members and global variables of CTraceMsg class.
TRACEDLL_API bool  WlTrace = false ;
TRACEDLL_API bool  WlSessionTrace = false ;
TRACEDLL_API bool  WlTraceSelective = false;

char CTraceMsg::m_ProgramName[] ="NoName" ;
RVCString CTraceMsg::m_TraceModules ;
RVCString CTraceMsg::m_SessionTraceModules ;
RVCString CTraceMsg::m_NotTraceModules ;
RVCString CTraceMsg::m_TraceFileName ("") ;
bool CTraceMsg::m_ToTraceFile  ;
FILE *CTraceMsg::m_pTraceFile  ;
RVCriticalSection CTraceMsg::m_Guard;
long CTraceMsg::m_OutLineNo = 0 ;
long CTraceMsg::m_fileSeq = 0;
long CTraceMsg::m_recordSizeCount = 0;
RVCString CTraceMsg::m_TraceFilePath ("");
long CTraceMsg::m_FileSizeLimit = 0;
long CTraceMsg::m_FileLimit = 0;
long CTraceMsg::m_ThreadLimit = 0;
long CTraceMsg::m_ThreadStart = 0;
long CTraceMsg::m_ErrFileSeq = 0;
long CTraceMsg::m_ErrRecordSizeCount = 0;
long CTraceMsg::m_ErrFileSizeLimit = 0;
long CTraceMsg::m_ErrFileLimit = 0;
bool CTraceMsg::m_ErrFileCyclic = false;
RVHashTable  CTraceMsg::m_ThreadHashTab;
RVHashDictionary CTraceMsg::m_ThreadInfoHashDict;
RVHashTable CTraceMsg::m_AgendaNameHashTab;
bool CTraceMsg::m_FirstWriteToErr = true;
bool CTraceMsg::m_FirstWriteToTrace = true;
RVCString CTraceMsg::m_ErrorFileName ("");
long CTraceMsg::m_LineSizeLimit = 0;

#ifndef __TV_UNIX__
RVCriticalSection CTraceMsg::m_Guard_err;
#else
RVTRecursiveLock<RVMutexLock> CTraceMsg::_m_Guard_err;
#endif

FILE *CTraceMsg::m_pErrorFile  ;
long CTraceMsg::m_OutLineNoErr=0;
FILE *CTraceMsg::m_pSessionTraceFile;

int WINAPI TraceSendTextFunc(const char* msg, ...)
{
  if (msg == NULL)
		return 0;

  char errBuf[102400];
  BOOL ret;
  memset(errBuf, 0, sizeof(errBuf));
  va_list ap;
  va_start(ap, msg);
  try {
    vsprintf(errBuf, msg, ap);
    /*  } catch (...) {}*/
  } catch (...) {
    strcpy (errBuf,"*#*#*#*# Error in creating trace message  for format string: \" ");
    strcat (errBuf, msg) ;
    strcat (errBuf, "\" *#*#*#*#");
  }

  va_end(ap);
  //
  // send the data
#ifdef  __TV_WINDOWS__
  //Write to trace window (only for WINDOWS platform)
  static HWND traceWindow = (HWND)GetProfileInt( "WebLoad-TraceApp", "m_hWnd", 0 );//TVNSGetGlobalProcessPort(TVNS_TRACEAPP);
  if( !traceWindow ) {
    return 0;
  }
  //
  // pack the Data into global-memory.
  HCLIENTDATA h = NVSPackData((void*)errBuf, strlen(errBuf), TRACE_OUTPUT_MSG, 0);
  ret = NVSSendMsg(h, traceWindow, 0);
  NVSFreeData(h);
#else
  //for UNIX write to STD error
  fprintf(stderr,"%s\n", errBuf) ;
  fflush(stderr);
  ret = 1;  // OK, I hope.
#endif
  return ret;
}

void CTraceMsg::OpenTraceFile ()
{

  m_recordSizeCount = 0;

  if (m_fileSeq >= m_FileLimit)
	  m_fileSeq = 1;
  else
	  m_fileSeq++;

  m_TraceFileName = m_TraceFilePath;
  m_TraceFileName += m_ProgramName ;
  char buffer[20];
  ltoa(m_fileSeq,buffer,10);
  m_TraceFileName += "_";
  m_TraceFileName += buffer;
  m_TraceFileName += ".TRC" ;

  if (!m_pTraceFile) m_pTraceFile = fopen( m_TraceFileName, "w" ) ;
}

RVCString CTraceMsg::GetTraceFileName()
{
	return m_TraceFileName;
}

RVCString CTraceMsg::GetErrorFileName()
{
	return m_ErrorFileName;
}

RVCString CTraceMsg::GetTracePath()
{
	RVCString sTracePath = m_TraceFilePath; 
	sTracePath.remove( sTracePath.last('\\') );
	return sTracePath;
}

void CTraceMsg::OpenErrorFile ()
{
  m_ErrorFileName =  m_TraceFilePath;
  m_ErrorFileName += m_ProgramName ;

  if (!CTraceMsg::m_ErrFileCyclic){
	// Chaim - format time
  struct timeb timebuf;
  struct tm *currTime;
  char TimeStr[30] ;
  int  MillSec ;
	
#ifndef __TV_UNIX__
    ftime(&timebuf);
    currTime = localtime( &timebuf.time ); /* Convert to local time. */
    MillSec = (int)timebuf.millitm ;
#else
    time_t secs_now;

    tzset();
    time(&secs_now);
#ifndef HAVE_LOCALTIME_R
    currTime = localtime(&secs_now);
#else
    struct tm tn;
    localtime_r(&secs_now,&tn); /* get dst information */
    currTime = &tn;
#endif

    MillSec = 0;
#endif
    sprintf( TimeStr,"%d-%d-%02d-%02d-%02d-%03d", currTime->tm_mon+1, currTime->tm_mday,
                                          currTime->tm_hour, currTime->tm_min, currTime->tm_sec, MillSec );
    m_ErrorFileName += "_";
    m_ErrorFileName += TimeStr;
  }

  m_ErrorFileName += ".ERR" ;

  m_ErrRecordSizeCount = 0;

  if (CTraceMsg::m_ErrFileCyclic){
  bool bOpenForAppend = true;

  if (m_ErrFileSeq >= m_ErrFileLimit)
  {
    bOpenForAppend = false;
	  m_ErrFileSeq = 1;
  }
  else
	  m_ErrFileSeq++;

  if(bOpenForAppend)
  {
    struct stat statbuf;
    m_pErrorFile = fopen( m_ErrorFileName, "a" ) ;
    if(fstat(fileno(m_pErrorFile), &statbuf) != -1)
    {
      if(statbuf.st_size >= m_ErrFileSizeLimit)
      {
        fclose(m_pErrorFile);
        OpenErrorFile();
      }
    }    
  }
    else {
      m_pErrorFile = fopen( m_ErrorFileName, "w" ) ;
    }
  }
  else{
  m_pErrorFile = fopen( m_ErrorFileName, "w" ) ;
}
}

//***********************************************
// Static class member, initialization routine.
// Gets webload.ini trace parameters and set the global
// variable of the trace mechanizm.
//
// Aviad Israeli (24/1/2000)
//***********************************************
void CTraceMsg::Init (const char *pProgramName) //The name of the current program (i.e. Exe component)
{
  const char *pEnvOption;
  RVCString EnvOption ;
	RVCString tmpProgramName (pProgramName);


  //Get Trace option from webload.ini file and set the global variable.
  WlTrace = false ;
  WlSessionTrace = false;
  TVEnvContainerClass tvpc;
  pEnvOption = tvpc.TVEGetEnv("TRACE");
  if (pEnvOption!=NULL) {
    EnvOption = pEnvOption ;
    if (EnvOption.index("YES",0, RVCString::ignoreCase ) != RV_NPOS) {
        WlTrace = true ; //Activate Trace
  }
  }

  //Get the list of modules to trace from webload.ini file.
  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("TRACED_MODULES");
  if (pEnvOption!=NULL)
    m_TraceModules = pEnvOption ;
  else
    m_TraceModules = "";

  //Get the list of modules not to trace from webload.ini file.
  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("NOT_TRACED_MODULES");
  if (pEnvOption!=NULL)
    m_NotTraceModules = pEnvOption ;
  else
    m_NotTraceModules = "";

  //Check in webload.ini whether to trace to a file or a window
  m_ToTraceFile = true ;
  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("TRACE_TO_WINDOW");
  if (pEnvOption!=NULL) {
    EnvOption = pEnvOption ;
    if (EnvOption.index("YES",0, RVCString::ignoreCase ) != RV_NPOS)
        m_ToTraceFile = false ; //Show trace message in TraceApp window (NT) or std_output (Unix)
  }

  //Get trace file prefix from webload.ini and create the full trace file name.
  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("TRACE_FILE_PREFIX");
  if (pEnvOption != NULL) {
      EnvOption = pEnvOption ;
      m_TraceFilePath = EnvOption + "_" ;
  }

  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("TRACE_FILESIZE_LIMIT");
  if (pEnvOption != NULL) {
	  m_FileSizeLimit = atol(pEnvOption) * 1000;
  }else{
	  m_FileSizeLimit = -1;
  }

  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("TRACE_FILE_LIMIT");
  if (pEnvOption != NULL) {
      m_FileLimit = atol(pEnvOption);	
  }
  
  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("TRACE_THREAD_LIMIT");
  if (pEnvOption != NULL) {
      m_ThreadLimit = atol(pEnvOption);
			if (tmpProgramName.index("HttpLoad", 0, RVCString::ignoreCase) != RV_NPOS) {
				WlTraceSelective = true ;
			}
  }
	else {
	  m_ThreadLimit = -1;
  }

  tvpc.reset();
	pEnvOption = tvpc.TVEGetEnv("TRACE_THREAD_START");
	if (pEnvOption != NULL) {
		m_ThreadStart = atol(pEnvOption);
			if (tmpProgramName.index("HttpLoad", 0, RVCString::ignoreCase) != RV_NPOS) {
				WlTraceSelective = true ;
			}
	}
	else{
		m_ThreadStart = -1;
	}

  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("TRACE_ERR_FILESIZE_LIMIT");
  if (pEnvOption != NULL) {
	  m_ErrFileSizeLimit = atol(pEnvOption) * 1000;
  }else{
	  m_ErrFileSizeLimit = -1;
  }

  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("TRACE_ERR_FILE_LIMIT");
  if (pEnvOption != NULL) {
      m_ErrFileLimit = atol(pEnvOption);	
  }

  CTraceMsg::m_ErrFileCyclic = false ;
  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("TRACE_ERR_FILE_CYCLIC");
  if (pEnvOption!=NULL) {
    EnvOption = pEnvOption ;
    if (EnvOption.index("YES",0, RVCString::ignoreCase ) != RV_NPOS)
        CTraceMsg::m_ErrFileCyclic = true ; //Activate Trace
  }

  //Get purify option from webload.ini file and set the global variable.
  s_bPurifyRound = false ;
  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("PURIFY_ROUND");
  if (pEnvOption!=NULL) {
    EnvOption = pEnvOption ;
    if (EnvOption.index("YES",0, RVCString::ignoreCase ) != RV_NPOS)
        s_bPurifyRound = true ; //Activate Trace
  }

	 //Get purify option from webload.ini file and set the global variable.
  s_bPurifyInstrumentation = false ;
  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("PURIFY_INSTRUMENTATION");
  if (pEnvOption!=NULL) {
    EnvOption = pEnvOption ;
    if (EnvOption.index("YES",0, RVCString::ignoreCase ) != RV_NPOS) {
        s_bPurifyInstrumentation = true ; //Activate Trace
        s_bPurifyInstrumentInuseMemory = true ;
    }
  }

  if (s_bPurifyInstrumentInuseMemory) {
    tvpc.reset();
    pEnvOption = tvpc.TVEGetEnv("PURIFY_INSTRUMENT_INUSE_MEMORY");
    if (pEnvOption!=NULL) {
      EnvOption = pEnvOption ;
      if (EnvOption.index("YES",0, RVCString::ignoreCase ) != RV_NPOS) {
	        s_bPurifyInstrumentInuseMemory = true ;
      }
      else {
	        s_bPurifyInstrumentInuseMemory = false ;
      }
    }
  }

  tvpc.reset();
  pEnvOption = tvpc.TVEGetEnv("TRACE_LINE_SIZE");
  if (pEnvOption != NULL) {
	  m_LineSizeLimit = atol(pEnvOption);
  }else{
	  m_LineSizeLimit = MESSAGE_LINE_SIZE;
  }

//  m_ProgramName = pProgramName;

//  m_TraceFileName += pProgramName ;
//  char buffer[20];
//  ltoa(CTraceMsg::m_fileSeq,buffer,10);
//  m_TraceFileName += "_";
//  m_TraceFileName += buffer;
//  m_TraceFileName += ".TRC" ;

  strcpy (m_ProgramName, pProgramName );

  if (WlTrace && m_ToTraceFile) OpenTraceFile () ;
  
}


//***********************************************
// Set initial precompiler values for trace message
//
// Aviad Israeli (24/1/2000)
//***********************************************

int CTraceMsg::PreDefVals (const char *pModuleName, //Link component name (i.e Dll name, Lib name etc)
                           const char *pFileName,   //Source file name
                           const int  LineNo)       //The line number in the source file where the trace routine was activated
{

    strcpy( m_ModuleName, pModuleName );
    strcpy( m_FileName, pFileName );
    m_LineNo = LineNo ;
    return 0 ;
}

//*************************************************************
// Enable to set the trace option in after the initialization.
//
// Aviad Israeli (15/3/2000)
//*************************************************************
void CTraceMsg::SetTrace (const bool Mode, const bool ToFile, const char *pProgramName)
{
  //This procedure is MTSafe.
  RVCriticalSection::LockGuard Lock(m_Guard);

  WlTrace = Mode ;
  m_ToTraceFile = ToFile;
  
  if(pProgramName) // create a new trace file with a new name
  {
		strcpy (m_ProgramName, pProgramName );
		if (WlTrace && m_ToTraceFile) 
		{
		  if(m_pTraceFile)
		  {
			  fclose(m_pTraceFile);
			  m_pTraceFile=NULL;
		  }

		  OpenTraceFile();
		}
  }
  else
  {
	 if (WlTrace && m_ToTraceFile) 
		   OpenTraceFile () ;
  }
}

void CTraceMsg::SetSessionTrace(const char* sTraceFileName, const char* sModuleList)
{
  if (!m_pSessionTraceFile)
  {
    WlSessionTrace = true;
    if(sModuleList)
    {
      m_SessionTraceModules = sModuleList;
    }
    m_pSessionTraceFile = fopen( sTraceFileName, "w" );
  }
}

void CTraceMsg::CloseSessionTrace()
{
  if(m_pSessionTraceFile)
  {
    WlSessionTrace = false;
    fclose(m_pSessionTraceFile);
    m_pSessionTraceFile = NULL;
  }
}



//***********************************************
// Create and write the trace message
//
// Aviad Israeli (24/1/2000)
//***********************************************
int CTraceMsg::SendMsg (const char *pMsg, // Format string for user message
                        ...)              // Arguments for user message
{
  char TempTraceMsg[MESSAGE_LEN];
  struct timeb timebuf;
  struct tm *currTime;
  char TimeStr[30] ;
  char ThreadIdStr[30] ;
  int  MillSec ;
  RVCString TraceMsg = "" ;
  TraceMsg.capacity (512) ;
  RVCString TraceMsgHeader = "";
  TraceMsgHeader.capacity (100) ;
  RVCString SearchModuleName = CHAR_SEPARATOR;
  RVCString SearchFileName = CHAR_SEPARATOR;
  RVCString SearchProgramName = CHAR_SEPARATOR;
  char LineNoStr[10] ;
  char ShortFileName[MAX_PATH];
  char HeadSeparator[5] = " | " ;
  char Separator[5] = "#" ;
  char OutLineNoStr[20] ;
  bool bOnlySessionTrace=false;

  try {
    char *SlashP;
    #ifdef  __TV_WINDOWS__
      SlashP = __STRRCHR ( m_FileName, '\\' ) ;
    #else
      SlashP = __STRRCHR ( m_FileName, '/' ) ;
    #endif
    if (SlashP == NULL ) SlashP = m_FileName-1 ;
    strcpy( ShortFileName, SlashP+1 );

    SearchModuleName += m_ModuleName ;
    SearchModuleName += CHAR_SEPARATOR ;
    SearchFileName += ShortFileName ;
    SearchFileName += CHAR_SEPARATOR ;
    SearchProgramName += m_ProgramName ;
    SearchProgramName += CHAR_SEPARATOR ;

    //Check if current program/module should be traced
    //A trace message will be displayed if one of the following conditions is true:
    // 1. Traced_Modules variable (in webload.ini) is empty or missing.
    // 2. Traced_Modules variable (in webload.ini) contains the program name.
    // 3. Traced_Modules variable (in webload.ini) contains the module name.
    // 4. Traced_Modules variable (in webload.ini) contains the file name (aaa.cpp).
    // 5. Not_Traced_Modules variable (in webload.ini) does not contains the module name.
    // 6. Not_Traced_Modules variable (in webload.ini) does not contains the file name.
    if ((m_NotTraceModules.index(SearchModuleName,0, RVCString::ignoreCase ) != RV_NPOS) ||
        (m_NotTraceModules.index(SearchFileName,0, RVCString::ignoreCase ) != RV_NPOS) ||
        ((m_TraceModules != "") &&
         (m_TraceModules.index(SearchProgramName,0, RVCString::ignoreCase ) == RV_NPOS) &&
         (m_TraceModules.index(SearchModuleName,0, RVCString::ignoreCase ) == RV_NPOS) &&
         (m_TraceModules.index(SearchFileName,0, RVCString::ignoreCase ) == RV_NPOS))) {

      if(m_SessionTraceModules.isNull() ||
         ((m_SessionTraceModules.index(SearchProgramName,0, RVCString::ignoreCase ) == RV_NPOS) &&
         (m_SessionTraceModules.index(SearchModuleName,0, RVCString::ignoreCase ) == RV_NPOS) &&
         (m_SessionTraceModules.index(SearchFileName,0, RVCString::ignoreCase ) == RV_NPOS)))
      {
       return 0;
      }else{
        bOnlySessionTrace = true;
      }
          
    }


	// if this is a new thread do the following:
	//		if    the limit of threads for tracing was reached skip the trace
	//		else  add the thrad id to the hash table and write the message
/*
	if(m_ThreadLimit > 0){
		RVCollectableInt* x = new RVCollectableInt(::rvThreadHash(::rvThreadId()));
		if(!m_ThreadHashTab.find(x)){
			if(m_ThreadHashTab.entries() >= m_ThreadLimit)
				return 0;
			else
				m_ThreadHashTab.insert(x);
		}
	}
*/
	if (WlTraceSelective) {
		RVCollectableInt* x = new RVCollectableInt(::rvThreadHash(::rvThreadId()));
		if(!m_ThreadHashTab.find(x)){
			return 0;
		}
	}

    //***************************
    //Format the Trace message
    //***************************
  if (m_FirstWriteToTrace){
    m_FirstWriteToTrace = false;
    // Go over all thread and agenda data
    RVHashTableIterator agendaIter(m_AgendaNameHashTab);
    RVCollectable* nextAgenda;
    while ((nextAgenda = agendaIter())){
      WlAssert(dynamic_cast<RVCollectableString*>(nextAgenda));
      RVCollectableString* agendaName = static_cast<RVCollectableString*>(nextAgenda);
      SendMsg("Agenda %s", agendaName->data());
    }

    RVHashDictionaryIterator threadIter(m_ThreadInfoHashDict);
    RVCollectable* nextThread;
    while ((nextThread = threadIter())){
      WlAssert(dynamic_cast<RVCollectableInt*>(nextThread));
      RVCollectableInt* threadId = static_cast<RVCollectableInt*>(nextThread);
      RVCollectable* temp = threadIter.value();
      WlAssert(dynamic_cast<RVCollectableString*>(temp));
      RVCollectableString* threadInfo = static_cast<RVCollectableString*>(temp);
      SendMsg("Thread %d (%x), %s", threadId->value(), threadId->value(), threadInfo->data());
    }
  }


    
	//Get current thread id and format it into a string.
    sprintf( ThreadIdStr,"%d", ::rvThreadHash(::rvThreadId()));

    //Get current time and format it into a string.
#ifndef __TV_UNIX__
    ftime(&timebuf);
    currTime = localtime( &timebuf.time ); /* Convert to local time. */
    MillSec = (int)timebuf.millitm ;
#else
    time_t secs_now;

    tzset();
    time(&secs_now);
#ifndef HAVE_LOCALTIME_R
    currTime = localtime(&secs_now);
#else
    struct tm tn;
    localtime_r(&secs_now,&tn); /* get dst information */
    currTime = &tn;
#endif

    MillSec = 0;
#endif
    sprintf( TimeStr,"%d/%d-%02d:%02d:%02d.%03d", currTime->tm_mday, currTime->tm_mon+1,
                                          currTime->tm_hour, currTime->tm_min, currTime->tm_sec, MillSec );
    sprintf( LineNoStr,"%d", m_LineNo );


    if (!m_ToTraceFile) {
      TraceMsgHeader +=  m_ProgramName ;
      TraceMsgHeader += HeadSeparator ;
    } ;
    TraceMsgHeader += m_ModuleName ;
    TraceMsgHeader += "/" ;
    TraceMsgHeader += ShortFileName ;
    TraceMsgHeader += "(" ;
    TraceMsgHeader += LineNoStr ;
    TraceMsgHeader += ")" ;
    TraceMsgHeader += HeadSeparator ;
    TraceMsgHeader += TimeStr ;
    TraceMsgHeader += HeadSeparator ;
    TraceMsgHeader += ThreadIdStr ;
    TraceMsgHeader += HeadSeparator ;
    TraceMsgHeader += Separator ;
    TraceMsgHeader += "> " ;

    //Adding user message and arguments
    va_list ap;
    va_start(ap, pMsg);
    try {
      vsprintf(TempTraceMsg, pMsg, ap);
    } catch (...) {
         strcpy (TempTraceMsg,"*#*#*#*# Error in creating trace message  for format string: \" ");
         strcat (TempTraceMsg, pMsg) ;
         strcat (TempTraceMsg, "\" *#*#*#*#");
        }
    va_end(ap);

    //From this point the procedure is MTSafe.
    //The guard is needed for the increasing of the LineNo and for
    //writing to the trace file
    RVCriticalSection::LockGuard Lock(m_Guard);

    sprintf ( OutLineNoStr, "%d", ++m_OutLineNo) ;
    TraceMsg += OutLineNoStr ;
    TraceMsg += HeadSeparator ;
    TraceMsg += TraceMsgHeader ;
    TraceMsg += TempTraceMsg ;

    //Wherever there is a CR or \n or CRLF add the message header (i.e. file name, module name..)
		char	curr, next; 
		int		headLen			= strlen(TraceMsgHeader);
		int		msgLen			= TraceMsg.length(); 
    int		i						= headLen; 
		bool	newLine			= false;
		int		loopCounter = 0;

		while (i < msgLen ) {

			loopCounter++;

			curr = TraceMsg.data()[i]; 

			if (i == msgLen-1) {
				next = curr ; 
			}
			else
			{
				next = TraceMsg.data()[i+1]; 
			}
	  
      if (curr == '\x0d') {
				newLine = true;
				if (next == '\x0a') {
					//CR + LF, remove the CR
					TraceMsg.remove(i,1);
			  }
				else {
		  	  //Only CR, replace with LF
					TraceMsg.replace(i, 1,'\n'); 
				}
			}
			else if (curr == '\x0a') {
				//Only LF
				newLine = true ; 
			}

			else if (loopCounter == m_LineSizeLimit ) {
				TraceMsg.insert (i, '\n');
				newLine = true;
			}

			if (newLine) {
				//Add new line headers
				loopCounter = 0;
      i ++ ;
      sprintf ( OutLineNoStr, "%d", ++m_OutLineNo) ;
      strcat ( OutLineNoStr , HeadSeparator ) ;
      TraceMsg.insert ( i ,  OutLineNoStr + TraceMsgHeader) ;
				i += headLen + strlen(OutLineNoStr)  ;
				msgLen = TraceMsg.length() ;
				newLine = false ; 
		  }
		  else
			{
				i++;
			}
    }


    //**********************************************************
    //Sending the trace message to the trace window/trace file
    //**********************************************************
    if(m_pSessionTraceFile)
    {
       fprintf (m_pSessionTraceFile, "%s\n", TraceMsg.data()) ;
       fflush ( m_pSessionTraceFile ) ;
    }
    if(!bOnlySessionTrace)
    {
    if (m_ToTraceFile && m_pTraceFile) {
       //Write to file
       fprintf (m_pTraceFile, "%s\n", TraceMsg.data()) ;
       fflush ( m_pTraceFile ) ;
	   m_recordSizeCount = m_recordSizeCount + ((strlen(TraceMsg.data()) + 2)*sizeof(char));
	   if (m_recordSizeCount >= m_FileSizeLimit && m_FileSizeLimit > 0){
		   fclose(m_pTraceFile);
		   m_pTraceFile=NULL;
		   OpenTraceFile () ;
	   }
    }
    else
    {
      #ifdef  __TV_WINDOWS__
        //Write to trace window (only for WINDOWS platform)
        static HWND traceWindow = (HWND)GetProfileInt( "WebLoad-TraceApp", "m_hWnd", 0 );
	      if( !traceWindow ) {
		      return 0;
        }
        // pack the Data into global-memory.
        HCLIENTDATA h = NVSPackData((void*)TraceMsg.data(), TraceMsg.length(), TRACE_OUTPUT_MSG, 0);
        BOOL ret = NVSSendMsg(h, traceWindow, 0);
        NVSFreeData(h);
      #else
        //for UNIX write to STD error
        fprintf(stderr,"%s\n", TraceMsg.data()) ;
        fflush(stderr);
      #endif
      }
    }

  } catch (...) {}

  return 0 ;
}

//********************************************************************
// Create and write the trace message
//
// Aviad Israeli (24/1/2000)
//
// Greg, Note following function derived from CTraceMsg::SendMsg
//       The difference is that Trace_Msg doesn't evaluate "TRACE" section 
//       of the webload.ini  
//
//  Params: 
//          IN const char *pMsg   Specifies format string
//          IN ...   arguments according to the format string     
//********************************************************************
int CTraceMsg::TraceMsgNoFilter (const char *pMsg, // Format string for user message
                                 ...)              // Arguments for user message
{
  char TempTraceMsg[102400];
  struct timeb timebuf;
  struct tm *currTime;
  char TimeStr[30] ;
  char ThreadIdStr[30] ;
  int  MillSec ;
  RVCString TraceMsg = "" ;
  TraceMsg.capacity (512) ;
  RVCString TraceMsgHeader = "";
  TraceMsgHeader.capacity (100) ;
  RVCString SearchModuleName = CHAR_SEPARATOR;
  RVCString SearchFileName = CHAR_SEPARATOR;
  RVCString SearchProgramName = CHAR_SEPARATOR;
  char LineNoStr[10] ;
  char ShortFileName[MAX_PATH];
  char HeadSeparator[5] = " | " ;
  char Separator[5] = "#" ;
  char OutLineNoStr[20] ;


  try {
    char *SlashP;
    #ifdef  __TV_WINDOWS__
      SlashP = __STRRCHR ( m_FileName, '\\' ) ;
    #else
      SlashP = __STRRCHR ( m_FileName, '/' ) ;
    #endif
    if (SlashP == NULL ) SlashP = m_FileName-1 ;
    strcpy( ShortFileName, SlashP+1 );

    SearchModuleName += m_ModuleName ;
    SearchModuleName += CHAR_SEPARATOR ;
    SearchFileName += ShortFileName ;
    SearchFileName += CHAR_SEPARATOR ;
    SearchProgramName += m_ProgramName ;
    SearchProgramName += CHAR_SEPARATOR ;

    //***************************
    //Format the Trace message
    //***************************
  if (m_FirstWriteToTrace){
    m_FirstWriteToTrace = false;
    // Go over all thread and agenda data
    RVHashTableIterator agendaIter(m_AgendaNameHashTab);
    RVCollectable* nextAgenda;
    while ((nextAgenda = agendaIter())){
      WlAssert(dynamic_cast<RVCollectableString*>(nextAgenda));
      RVCollectableString* agendaName = static_cast<RVCollectableString*>(nextAgenda);
      SendMsg("Agenda %s", agendaName->data());
    }

    RVHashDictionaryIterator threadIter(m_ThreadInfoHashDict);
    RVCollectable* nextThread;
    while ((nextThread = threadIter())){
      WlAssert(dynamic_cast<RVCollectableInt*>(nextThread));
      RVCollectableInt* threadId = static_cast<RVCollectableInt*>(nextThread);
      RVCollectable* temp = threadIter.value();
      WlAssert(dynamic_cast<RVCollectableString*>(temp));
      RVCollectableString* threadInfo = static_cast<RVCollectableString*>(temp);
      SendMsg("Thread %d (%x), %s", threadId->value(), threadId->value(), threadInfo->data());
    }
  }

	  // Get current thread id and format it into a string.
    sprintf( ThreadIdStr,"%d", ::rvThreadHash(::rvThreadId()));

    //Get current time and format it into a string.
#ifndef __TV_UNIX__
    ftime(&timebuf);
    currTime = localtime( &timebuf.time ); /* Convert to local time. */
    MillSec = (int)timebuf.millitm ;
#else
    time_t secs_now;

    tzset();
    time(&secs_now);
#ifndef HAVE_LOCALTIME_R
    currTime = localtime(&secs_now);
#else
    struct tm tn;
    localtime_r(&secs_now,&tn); /* get dst information */
    currTime = &tn;
#endif

    MillSec = 0;
#endif
    sprintf( TimeStr,"%d/%d-%02d:%02d:%02d.%03d", currTime->tm_mday, currTime->tm_mon+1,
                                          currTime->tm_hour, currTime->tm_min, currTime->tm_sec, MillSec );
    sprintf( LineNoStr,"%d", m_LineNo );


    if (!m_ToTraceFile) {
      TraceMsgHeader +=  m_ProgramName ;
      TraceMsgHeader += HeadSeparator ;
    } ;
    TraceMsgHeader += m_ModuleName ;
    TraceMsgHeader += "/" ;
    TraceMsgHeader += ShortFileName ;
    TraceMsgHeader += "(" ;
    TraceMsgHeader += LineNoStr ;
    TraceMsgHeader += ")" ;
    TraceMsgHeader += HeadSeparator ;
    TraceMsgHeader += TimeStr ;
    TraceMsgHeader += HeadSeparator ;
    TraceMsgHeader += ThreadIdStr ;
    TraceMsgHeader += HeadSeparator ;
    TraceMsgHeader += Separator ;
    TraceMsgHeader += "> " ;

    //Adding user message and arguments
    va_list ap;
    va_start(ap, pMsg);
    try {
      vsprintf(TempTraceMsg, pMsg, ap);
    } catch (...) {
         strcpy (TempTraceMsg,"*#*#*#*# Error in creating trace message  for format string: \" ");
         strcat (TempTraceMsg, pMsg) ;
         strcat (TempTraceMsg, "\" *#*#*#*#");
        }
    va_end(ap);

    //From this point the procedure is MTSafe.
    //The guard is needed for the increasing of the LineNo and for
    //writing to the trace file
    RVCriticalSection::LockGuard Lock(m_Guard);

    sprintf ( OutLineNoStr, "%d", ++m_OutLineNo) ;
    TraceMsg += OutLineNoStr ;
    TraceMsg += HeadSeparator ;
    TraceMsg += TraceMsgHeader ;
    TraceMsg += TempTraceMsg ;

    //Wherever there is a CR or \n or CRLF add the message header (i.e. file name, module name..)
		char	curr, next; 
		int		headLen			= strlen(TraceMsgHeader);
		int		msgLen			= TraceMsg.length(); 
    int		i						= headLen; 
		bool	newLine			= false;
		int		loopCounter = 0;

		while (i < msgLen ) {

			loopCounter++;

			curr = TraceMsg.data()[i]; 

			if (i == msgLen-1) {
				next = curr ; 
			}
			else
			{
				next = TraceMsg.data()[i+1]; 
			}
	  
      if (curr == '\x0d') {
				newLine = true;
				if (next == '\x0a') {
					//CR + LF, remove the CR
					TraceMsg.remove(i,1);
			  }
				else {
		  	  //Only CR, replace with LF
					TraceMsg.replace(i, 1,'\n'); 
				}
			}
			else if (curr == '\x0a') {
				//Only LF
				newLine = true ; 
			}

			else if (loopCounter == m_LineSizeLimit ) {
				TraceMsg.insert (i, '\n');
				newLine = true;
			}

			if (newLine) {
				//Add new line headers
				loopCounter = 0;
      i ++ ;
      sprintf ( OutLineNoStr, "%d", ++m_OutLineNo) ;
      strcat ( OutLineNoStr , HeadSeparator ) ;
      TraceMsg.insert ( i ,  OutLineNoStr + TraceMsgHeader) ;
				i += headLen + strlen(OutLineNoStr)  ;
				msgLen = TraceMsg.length() ;
				newLine = false ; 
		  }
		  else
			{
				i++;
			}
    }


    //**********************************************************
    //Sending the trace message to the trace window/trace file
    //**********************************************************
      if (m_ToTraceFile && m_pTraceFile) {
         //Write to file
         fprintf (m_pTraceFile, "%s\n", TraceMsg.data()) ;
         fflush ( m_pTraceFile ) ;
	     m_recordSizeCount = m_recordSizeCount + ((strlen(TraceMsg.data()) + 2)*sizeof(char));
	     if (m_recordSizeCount >= m_FileSizeLimit && m_FileSizeLimit > 0){
		     fclose(m_pTraceFile);
		     m_pTraceFile=NULL;
		     OpenTraceFile () ;
	     }
      }
      else
      {
      #ifdef  __TV_WINDOWS__
        //Write to trace window (only for WINDOWS platform)
        static HWND traceWindow = (HWND)GetProfileInt( "WebLoad-TraceApp", "m_hWnd", 0 );
	      if( !traceWindow ) {
		      return 0;
        }
        // pack the Data into global-memory.
        HCLIENTDATA h = NVSPackData((void*)TraceMsg.data(), TraceMsg.length(), TRACE_OUTPUT_MSG, 0);
        BOOL ret = NVSSendMsg(h, traceWindow, 0);
        NVSFreeData(h);
      #else
        //for UNIX write to STD error
        fprintf(stderr,"%s\n", TraceMsg.data()) ;
        fflush(stderr);
      #endif
      }
  } catch (...) {}

  return 0 ;
}

int CTraceMsg::SendErr (const char *pMsg, // Format string for user message
                        ...)              // Arguments for user message
{
  char TempTraceMsg[102400];
  struct timeb timebuf;
  struct tm *currTime;
  char TimeStr[30] ;
  char ThreadIdStr[30] ;
  int  MillSec ;
  RVCString TraceMsg = "" ;
  RVCString TraceMsgHeader = "";
  RVCString SearchModuleName = CHAR_SEPARATOR;
  RVCString SearchFileName = CHAR_SEPARATOR;
  RVCString SearchProgramName = CHAR_SEPARATOR;
  char LineNoStr[10] ;
  char ShortFileName[MAX_PATH];
  char HeadSeparator[5] = " | " ;
  char Separator[5] = "#" ;
  char OutLineNoStr[20] ;

  try {
    char *SlashP;
    #ifdef  __TV_WINDOWS__
      SlashP = __STRRCHR ( m_FileName, '\\' ) ;
    #else
      SlashP = __STRRCHR ( m_FileName, '/' ) ;
    #endif
    if (SlashP == NULL ) SlashP = m_FileName-1 ;
    strcpy( ShortFileName, SlashP+1 );

    SearchModuleName += m_ModuleName ;
    SearchModuleName += CHAR_SEPARATOR ;
    SearchFileName += ShortFileName ;
    SearchFileName += CHAR_SEPARATOR ;
    SearchProgramName += m_ProgramName ;
    SearchProgramName += CHAR_SEPARATOR ;


    //***************************
    //Format the Trace message
    //***************************

    
	//Get current thread id and format it into a string.
    sprintf( ThreadIdStr,"%d", ::rvThreadHash(::rvThreadId()));

    //Get current time and format it into a string.
#ifndef __TV_UNIX__
    ftime(&timebuf);
    currTime = localtime( &timebuf.time ); /* Convert to local time. */
    MillSec = (int)timebuf.millitm ;
#else
    time_t secs_now;

    tzset();
    time(&secs_now);
#ifndef HAVE_LOCALTIME_R
    currTime = localtime(&secs_now);
#else
    struct tm tn;
    localtime_r(&secs_now,&tn); /* get dst information */
    currTime = &tn;
#endif

    MillSec = 0;
#endif
    sprintf( TimeStr,"%d/%d-%02d:%02d:%02d.%03d", currTime->tm_mday, currTime->tm_mon+1,
                                          currTime->tm_hour, currTime->tm_min, currTime->tm_sec, MillSec );
    sprintf( LineNoStr,"%d", m_LineNo );

    TraceMsgHeader += m_ModuleName ;
    TraceMsgHeader += "/" ;
    TraceMsgHeader += ShortFileName ;
    TraceMsgHeader += "(" ;
    TraceMsgHeader += LineNoStr ;
    TraceMsgHeader += ")" ;
    TraceMsgHeader += HeadSeparator ;
    TraceMsgHeader += TimeStr ;
    TraceMsgHeader += HeadSeparator ;
    TraceMsgHeader += ThreadIdStr ;
    TraceMsgHeader += HeadSeparator ;
    TraceMsgHeader += Separator ;
    TraceMsgHeader += "> " ;

    //Adding user message and arguments
    va_list ap;
    va_start(ap, pMsg);
    try {
      vsprintf(TempTraceMsg, pMsg, ap);
    } catch (...) {
         strcpy (TempTraceMsg,"*#*#*#*# Error in creating Error message  for format string: \" ");
         strcat (TempTraceMsg, pMsg) ;
         strcat (TempTraceMsg, "\" *#*#*#*#");
        }
    va_end(ap);

    //From this point the procedure is MTSafe.
    //The guard is needed for the increasing of the LineNo and for
    //writing to the trace file
#ifndef __TV_UNIX__
    RVCriticalSection::LockGuard Lock(m_Guard_err);
#else
    RVTRecursiveLock<RVMutexLock>::LockGuard lock(_m_Guard_err);
#endif
     

    sprintf ( OutLineNoStr, "%d", ++m_OutLineNoErr) ;
    TraceMsg += OutLineNoStr ;
    TraceMsg += HeadSeparator ;
    TraceMsg += TraceMsgHeader ;
    TraceMsg += TempTraceMsg ;

    //Wherever there is a \n add the message header (i.e. file name, module name..)
    int i = TraceMsg.index ( "\n" ) ;
    while (i != RV_NPOS) {
      i ++ ;
      sprintf ( OutLineNoStr, "%d", ++m_OutLineNoErr) ;
      strcat ( OutLineNoStr , HeadSeparator ) ;
      TraceMsg.insert ( i ,  OutLineNoStr + TraceMsgHeader) ;
      i = TraceMsg.index ( "\n", i ) ;
    } ;

    //**********************************************************
    //Sending the trace message to the trace window/trace file
    //**********************************************************
   if (!m_pErrorFile)
	   OpenErrorFile();

   if ( m_pErrorFile )
   {
     if (m_FirstWriteToErr){
       m_FirstWriteToErr = false;
       // Go over all thread and agenda data
       RVHashTableIterator agendaIter(m_AgendaNameHashTab);
       RVCollectable* nextAgenda;
       while ((nextAgenda = agendaIter())){
         WlAssert(dynamic_cast<RVCollectableString*>(nextAgenda));
         RVCollectableString* agendaName = static_cast<RVCollectableString*>(nextAgenda);
         SendErr("Agenda %s", agendaName->data());
       }

       RVHashDictionaryIterator threadIter(m_ThreadInfoHashDict);
       RVCollectable* nextThread;
       while ((nextThread = threadIter())){
         WlAssert(dynamic_cast<RVCollectableInt*>(nextThread));
         RVCollectableInt* threadId = static_cast<RVCollectableInt*>(nextThread);
         RVCollectable* temp = threadIter.value();
         WlAssert(dynamic_cast<RVCollectableString*>(temp));
         RVCollectableString* threadInfo = static_cast<RVCollectableString*>(temp);
         SendErr("Thread %d (%x), %s", threadId->value(), threadId->value(), threadInfo->data());
       }
     }

	   fprintf (m_pErrorFile, "%s\n", TraceMsg.data()) ;
	   fflush ( m_pErrorFile ) ;

     if (CTraceMsg::m_ErrFileCyclic){
	   m_ErrRecordSizeCount = m_ErrRecordSizeCount + ((strlen(TraceMsg.data()) + 2)*sizeof(char));
	   if (m_ErrRecordSizeCount >= m_ErrFileSizeLimit && m_ErrFileSizeLimit > 0){
			   fclose(m_pErrorFile);
			   m_pErrorFile=NULL;
			   OpenErrorFile () ;
		   }
     }

   }
	  if(WlTrace || WlSessionTrace)	
		SendMsg(TempTraceMsg);		
       
  } catch (...) {}

  return 0 ;
}

int CTraceMsg::SendErrAssert (int assertInd,const char *pMsg, // Format string for user message
                        ...)              // Arguments for user message
{

 char TempTraceMsg[102400];

	try{
		SendErr("Assert occurred!");
		if(!assertInd){
			va_list ap;
			va_start(ap, pMsg);
			try {
				vsprintf(TempTraceMsg, pMsg, ap);
			} catch (...) {
				strcpy (TempTraceMsg,"*#*#*#*# Error in creating Error message  for format string: \" ");
				strcat (TempTraceMsg, pMsg) ;
				strcat (TempTraceMsg, "\" *#*#*#*#");
			}
			va_end(ap);
			SendErr(TempTraceMsg);
			assert(0);
		}
	}catch (...) {
		SendErr("Error in assert");
	}

	return 0;
}

void CTraceMsg::RegisterThread (int virtualClientNum)
{
	if ((WlTraceSelective) &&
		  ((virtualClientNum < 0) || 
			 (((m_ThreadStart < 1) || (virtualClientNum >= m_ThreadStart)) && 
			 ((m_ThreadLimit < 1) || (virtualClientNum <= m_ThreadLimit))))) {
		RVCollectableInt* x = new RVCollectableInt(::rvThreadHash(::rvThreadId()));
		if(!m_ThreadHashTab.find(x)){
				m_ThreadHashTab.insert(x);
		}
	}
}

void CTraceMsg::RegisterThreadInfo (const char* threadInfo)
{
	RVCollectableInt* x = new RVCollectableInt(::rvThreadHash(::rvThreadId()));
  RVCollectableString* info = new RVCollectableString(threadInfo);
  if (!m_ThreadInfoHashDict.find(x)){
    m_ThreadInfoHashDict.insertKeyAndValue(x, info);
  }
}

void CTraceMsg::RegisterAgenda (const char* agendaName){
  RVCollectableString* name = new RVCollectableString(agendaName);
  if (!m_AgendaNameHashTab.find(name)){
    m_AgendaNameHashTab.insert(name);
  }
}

void CTraceMsg::UnregisterAgenda (const char* agendaName){
  RVCollectableString* name = new RVCollectableString(agendaName);
  if (m_AgendaNameHashTab.find(name)){
    m_AgendaNameHashTab.remove(name);
  }
}

int CTraceMsg::SendErrWithCallStack (const char *pMsg, // Format string for user message
                        ...)              // Arguments for user message
{

 char TempTraceMsg[102400];

	try{
			va_list ap;
			va_start(ap, pMsg);
			try {
				vsprintf(TempTraceMsg, pMsg, ap);
			} catch (...) {
				strcpy (TempTraceMsg,"*#*#*#*# Error in creating Error message  for format string: \" ");
				strcat (TempTraceMsg, pMsg) ;
				strcat (TempTraceMsg, "\" *#*#*#*#");
			}
			va_end(ap);
			SendErr(TempTraceMsg);

		  // Stack trace
		  SendErr("Trace for assert:");
		  WLErrorExceptionWithStackTrace *temp = new WLErrorExceptionWithStackTrace("stam", WLLogMessage::Note);
		  RVCString traceBuf;
		  temp->StackTrace(traceBuf);
		  SendErr(traceBuf.data());
		  delete temp;
			
	}catch (...) {
		SendErr("Error in assert");
	}

	return 0;
}

int CTraceMsg::SendErrAssert (const char *pMsg, // Format string for user message
                        ...)              // Arguments for user message
{

	char TempTraceMsg[102400];

	try{
			SendErr("Assert occurred!");
			va_list ap;
			va_start(ap, pMsg);
			try {
				vsprintf(TempTraceMsg, pMsg, ap);
			} catch (...) {
				strcpy (TempTraceMsg,"*#*#*#*# Error in creating Error message  for format string: \" ");
				strcat (TempTraceMsg, pMsg) ;
				strcat (TempTraceMsg, "\" *#*#*#*#");
			}
			va_end(ap);
			SendErr(TempTraceMsg);
			assert(0);
	}catch (...) {
		SendErr("Error in assert");
	}

	return 0;
}


extern "C" DllExport void RvErrorMsgPosition( const char* module, const char* file, long line, const char* s )
{
	WlErrorMsgPosition( module, file, line )( s );
}

extern "C" DllExport void RvTraceMsgPosition( const char* module, const char* file, long line, const char* s )
{
	WlTraceMsgPosition( module, file, line )( s );
}

// PurifyLog stuff

#ifdef _USE_PURIFY_
// calls PurifyNewLeaks
void ReportNewPurifyLeaks(bool bRound,const char *msg)
{
	if((!s_bPurifyInstrumentation) || (bRound && !s_bPurifyRound))
		return;
#ifdef __TV_WINDOWS__
	if(msg) PurifyPrintf(msg);
	PurifyNewLeaks();
#endif // __TV_WINDOWS__
#ifdef __TV_solaris__
	if(msg) purify_printf(msg);
	purify_new_leaks();
#endif // __TV_solaris__
}

// calls PurifyAllLeaks
void ReportAllPurifyLeaks(bool bRound,const char *msg)
{
	if((!s_bPurifyInstrumentation) || (bRound && !s_bPurifyRound))
		return;
#ifdef __TV_WINDOWS__
	if(msg) PurifyPrintf(msg);
	PurifyAllLeaks();
#endif // __TV_WINDOWS__
#ifdef __TV_solaris__
	if(msg) purify_printf(msg);
  	purify_all_leaks();
#endif // __TV_solaris__
}

// calls PurifyClearLeaks
void ClearPurifyLeaks(bool bRound,const char *msg)
{
	if((!s_bPurifyInstrumentation) || (bRound && !s_bPurifyRound))
		return;
#ifdef __TV_WINDOWS__
	if(msg) PurifyPrintf(msg);
	PurifyClearLeaks();
#endif // __TV_WINDOWS__
#ifdef __TV_solaris__
	if(msg) purify_printf(msg);
  	purify_clear_leaks();
#endif // __TV_solaris__
}

//calls PurifyAllInUse
void ReportPurifyMemory(bool bRound,const char *msg)
{
	if(!s_bPurifyInstrumentInuseMemory)
		return;
	if((!s_bPurifyInstrumentation) || (bRound && !s_bPurifyRound))
		return;
#ifdef __TV_WINDOWS__
	if(msg) PurifyPrintf(msg);
	PurifyAllInuse();
#endif // __TV_WINDOWS__
#ifdef __TV_solaris__
	if(msg) purify_printf(msg);
  	purify_all_inuse();
#endif // __TV_solaris__
}

#endif // _USE_PURIFY_
