/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include <assert.h>
#include <RVCString.h>

#include "tvenv.h"
#ifdef __TV_WINDOWS__
#include "tvver.h"
#endif // __TV_WINDOWS__

static char buf[80] = "";
char datastr[60];

int getint(const char* text)
{
  cout << text;
  cin.getline(buf,sizeof(buf));
  return atoi(buf);
}

char* getstr(const char* text)
{
  cout << text;
  cin.getline(buf,sizeof(buf));
  return buf;
}

int main(int argc, char *argv[])
{
	
  int choice = 0;
	
  TVEnvContainerClass tvpc;
#ifdef __TV_UNIX__
  char thisFileName[MAX_PATH];
  if( !TVEInit(tvpc.TVPGetPathDirectory( (const char *)realpath(argv[0],thisFileName ))))
  {
    TVPFatalErrorLog("tvenv.dll: Init Terminated with error: (%s)",TVEError());
    return FALSE;
  }
#endif
	
  while (1) {
    switch (choice)
		{
		case 0:
			cout << endl
				<< "1. Check TVPSplitPath" << endl
				<< "2. Check TVPGetTempName" << endl
				<< "3. Check TVPCopyFile" << endl
				<< "4. Check TVPGetFullName & Make/Split path" << endl
				<< "5. Check TVPResolveFilePath" << endl
				<< "6. Check TVPSearchPath" << endl
				<< "7. Check TVPReplacePathExt" << endl
				<< "8. Check TVPFindfile" << endl
				<< "9. Print CWD, User's dir " << endl
				<< "10.Chdir " << endl
				<< "11.TVPAdjustPathLen" << endl
				<< "12.check TVEGetEnv"<<endl
				<< "13.check TVESetEnv"<<endl
				<< "14.check TVERuntimeResolve "<<endl
#ifdef __TV_WINDOWS__
				<< "15.Tvver functions"<<endl
#endif // __TV_WINDOWS__
				<< "99. Exit" << endl;
			break;
			
		case 1: // check TVPSplitPath
			{
				char *d,*e,*f;
				const char *s = getstr("Enter filename to split: ");
				tvpc.TVPSplitPath(s,&d,&f,&e);
				printf("splitpath of '%s' ->\n\t(%s,%s,%s)\n",s,d,f,e);
			}
			break;
			
		case 2: // check TVPGetTempName
			{
				RVCString pathvar = getstr("Please enter the path variable: ");
				int n = getint("How many tmp-names you want ? : ");
				cout << n << " temporary names: " << endl;
				for (int i=1;i<=n;i++)
					cout << i << ": " << tvpc.TVPGetTempName(pathvar) << endl;
			}
			break;
			
		case 3: // Copy-file
			{
				RVCString from = getstr("Enter from file: ");
				RVCString to   = getstr("Enter to file: ");
				BOOL b = TVPCopyFile(from, to);
				cout << "The copy has " << (b ? "Succeeded" : "failed") << endl;
			}
			break;
			
			
		case 4: // check TVPGetFullName
			{
				char *d,*e,*f;
				char New[MAX_PATH] = "";
				const char *s = getstr("TVPGetFullName: Enter filename: ");
				tvpc.TVPSplitPath(s,&d,&f,&e);
				printf("splitpath of '%s' ->\n\t(%s,%s,%s)\n",s,d,f,e);
				const char* b = tvpc.TVPGetFullName(s,(char*) New, MAX_PATH);
				if(strlen(b))
				{
					tvpc.TVPSplitPath(b,&d,&f,&e);
					printf("fullname = '%s' ->\n\t(%s,%s,%s)\n\tMakepath = (%s)\n",s,d,f,e,tvpc.TVPMakePath(d,f,e));
				}
				else
					printf("fullname of '%s' not found\n",s);
				printf("TVPIsClosedFileADirectory(%s) returned %d\n",s,TVPIsClosedFileADirectory(s));
				printf("TVPIsClosedFileADirectory(%s) returned %d\n",d,TVPIsClosedFileADirectory(d));
				printf("TVPIsClosedFileADirectory(%s) returned %d\n",f,TVPIsClosedFileADirectory(f));
				printf("TVPIsClosedFileADirectory(%s) returned %d\n",e,TVPIsClosedFileADirectory(e));
				printf("TVPIsClosedFileADirectory(%s) returned %d\n",b,TVPIsClosedFileADirectory(b));
			}
			break;
			
		case 5: // TVPResolveFilePath
			{
				RVCString name = getstr("Enter file name: ");
				RVCString pathvar = getstr("Enter path var: ");
				const char* s = tvpc.TVPResolveFilePath(name,pathvar);
				printf("TVPResolveFilePath(%s,%s) = '%s'\n",(const char*)name,(const char*)pathvar,(s ? s : "Not FOUND"));
			}
			break;
			
		case 6: // TVPSearchPath
			{
				RVCString name = getstr("Enter file name to search: ");
				const char* s = tvpc.TVPSearchPath(name);
				printf("TVPSearchPath(%s) = '%s'\n",(const char*)name,(s ? s : "Not FOUND"));
			}
			break;
			
		case 7: // replace file-extcTVPReplacePathExt
			{
				RVCString name = getstr("Enter file name: ");
				RVCString newext = getstr("Enter new extension: ");
				const char* s = tvpc.TVPReplacePathExt(name,newext);
				printf("TVPReplacePathExt(%s,%s) = '%s'\n",(const char*)name,(const char*)newext,s);
			}
			break;
			
		case 8: // TVPFindfile
			{
				RVCString pv[10];
				const char* ppv[10];
				RVCString name = getstr("Enter file name: ");
				int i=0;
				while((i==0 || strlen(pv[i-1])>0) && i<10)
					pv[i++] = getstr("Enter path var: ");
				
				for (int j=0;j<10;j++)
					ppv[j] = (const char*)(pv[j]);
				
				printf("TVPFindfile(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)\n",(const char*)name,ppv[0],ppv[1],ppv[2],ppv[3],ppv[4],ppv[5],ppv[6],ppv[7],ppv[8],ppv[9]);
				const char* s = tvpc.TVPFindfile(name,ppv[0],ppv[1],ppv[2],ppv[3],ppv[4],ppv[5],ppv[6],ppv[7],ppv[8],ppv[9],NULL);
				printf("TVPFindfile(%s,...) = '%s'\n",(const char*)name,(s ? s : "Not FOUND"));
			}
			break;
			
		case 9: // file-information
			{
				printf("current directory is: '%s'\n",tvpc.TVPGetCwd());
				printf("User's directory is: '%s'\n",tvpc.TVPGetUserDir());
			}
			break;
		case 10: // check TVPChdir
			{
				const char *s = getstr("Enter new path: ");
				int st = TVPChdir(s);
				printf("chdir(%s) == %d, current dir = '%s'\n",s,st,tvpc.TVPGetCwd());
			}
			break;
			
		case 11: // check TVPAdjustPathLen
			{
				char temp[MAX_PATH];
				RVCString s = getstr("Enter path: ");
				int len = getint("Enter new len: ");
				const char* st = tvpc.TVPAdjustPathLen((const char*)s,temp,len);
				printf("TVPAdjustPathLen(%s) == '%s'\n",(const char*)s,(st?st:"null"));
			}
			break;
			
			
		case 12:
			{
				char key[1024];
				int count = 0;
				strcpy(key,getstr("enter a table key : "));
				count =  getint("enter num calls: ");
				for (int ii=1;ii<=count;ii++)
					if (tvpc.TVEGetEnv(key))
						cout<<"retrieved value : "<<tvpc.TVEGetEnv(key)<<endl;
					else
						cout<<"error while retrieving"<<endl;
			}
			break;
			
		case 13:
			{
				char key[1024];
				char value[1024];
				bool bNewKey = false;
				while ( !bNewKey )
				{
					strcpy( key, getstr( "enter a table key: " ) );
					if ( tvpc.TVEGetEnvValue( key ) )
						cout << "key already exists, please ";
					else
						bNewKey = true;
				}
				strcpy( value, getstr( "enter a table value: " ) );
				tvpc.TVESetEnvValue( key, value );
				if ( tvpc.TVEGetEnvValue( key ) )
					cout << "retrieved value: " << tvpc.TVEGetEnvValue( key ) << endl;
				else
					cout << "error while retrieving" << endl;
			}
			break;

		case 14:
			{
				char value[1024];
				strcpy(value,getstr("enter a value to resolve: "));
				cout<<tvpc.TVERuntimeResolve(value)<<endl;
			}
			break;
			
#ifdef __TV_WINDOWS__
		case 15: // tvver functions
			{
				cout<<"name : "<<TVVProductName()<<endl
          <<"version : "<<TVVProductVersion()<<endl
					<<"version name : "<<TVVProductVersionName()<<endl
          <<"error message : "<<TVVProductErrorMessage()<<endl
          <<"query : "<<TVVProductQuery()<<endl
          <<"title : "<<TVVEditorTitle()<<endl
          <<"ini file : "<<TVVProductIniFile()<<endl
          <<"backup file : "<<TVVProductIniBackup()<<endl
          <<"help file : "<<TVVProductHelpFile()<<endl
          <<"frm file : "<<TVVProductFrmFile()<<endl
          <<"Default session name : "<<TVVersionInfo("DEFAULT_SESSION_NAME")<<endl;
				
				TVV_ABOUT_AATWIN(NULL);
				//TVV_ABOUT_CAPTURE( NULL);
				//TVV_ABOUT_CUSTOM( NULL);
				//TVV_ABOUT_DCVIEW( NULL);
				//TVV_ABOUT_DCWIZARD( NULL);
				//TVV_ABOUT_EDITOR( NULL);
				//TVV_ABOUT_EXITNET( NULL);
				TVV_ABOUT_MONITOR( NULL);
				//TVV_ABOUT_POSTMORT( NULL);
				TVV_ABOUT_STARTNET( NULL);
				//TVV_ABOUT_UARBUILD( NULL);
				//TVV_ABOUT_VIEW( NULL );
				TVV_ABOUT_WEBFT( NULL );
			}
#endif // __TV_WINDOWS__
			
		case 99:
			return 1;
			
		default:
			cout << "Illegal option" << endl << endl;
			break;
      } // end switch
			
			choice = getint("Please enter selection: [0]");
			//getstr("hit <Enter> to continue");
			
  } // end while
  return 1;
}

