/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/*************************************************************/
/*                                                           */
/*                A PARSER FOR THE .INI FILES                */
/*               =============================               */
/*                                                           */
/*   Writen By Iris Kraus                                 */
/*  Date Of Creation:   02-09-93             */
/*                                                           */


/*************************************************************/
#ifdef __TV_UNIX__
#include <unistd.h>
#include <ctype.h>
#endif
#include "platform.h"
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <errno.h>
#include "tvtables.h"

#include "strform.h"
#include "severr.h"
#include "lsevenv.h"

#ifndef _ANSI_ONLY //Galtech
#include "gt_unicode.h"
#else
extern char* yytext;
extern FILE* yyin;
extern int yyparse();
extern void yyrestart(FILE*);
extern unsigned int lineNumber;
#endif //_ANSI_ONLY //Galtech

TableHandle Handle;
ParseMode mode;
static int ParseFlag = 0;
char IniTmpName[MAX_PATH];
char InputFileName[MAX_PATH];
extern int ParseError;
filterfunction filterfunc;

const char* TVSDummyfilter(const char* key,const char* value);

#ifndef _ANSI_ONLY //Galtech
/*************************************************************/
/* The  function adds key and value to the HashTable.        */
/*************************************************************/

void AddMemberToTable(TableHandle h, const char* key, const char* value)
{
  if (!strcmp(key,"INCLUDE_PATH")) {
    const char *v;
		v = TVSGetValue(h,key);
    if (v){
      char buf[1024];
      sprintf(buf,"%s;%s",value,v);
      TVSAssignValue(h,key,buf);
      return;
    }
   }
//   printf("Key = %s, Value = %s\n",key,value);
   TVSAssignValue(h,key,value);
}

int Parse(FILE *fp, TableHandle h, char *tmpName)
{
  char buffer[1024*3], value[1024*3], key[1024*3];
  char ANSI_value[1024], ANSI_key[1024];
  char tmp[1024];
  BOOL bSave = (tmpName	!= NULL);
  FILE *of;

  if (bSave)
	of = fopen(tmpName,"w");

  while (fgets(tmp, sizeof(tmp), fp))
  {
	  //convert string to unicode
	  ANSIToUTF8(CP_ACP, tmp, -1, buffer, sizeof(buffer)); 
	  char *p = buffer;

	  //skip spaces
	  while (isspace(*p))
		  p++;

	  //empty line
	  if (!*p)
	  {
		  if (bSave)
			  fprintf(of, tmp);
		  continue;
	  }

	  //comments
	  if (!strncmp(p, "//", 2))
	  {
		  if (bSave)
			  fprintf(of, tmp);
		  continue;
	  }

	  //get key
	  int i = 0;
	  while (*p && !isspace(*p) && *p != '=')
		  key[i++] = *(p++);
	  key[i] = '\0';

	  //skip spaces
	  while (isspace(*p))
		  p++;

	  //check for equal
	  if (*p != '=')
		  return TVSINTERR_PARSE_ERROR;
	  p++;

	  //skip spaces
	  while (isspace(*p))
		  p++;

	  //get value
	  i = 0;
	  if (*p == '\'' || *p == '\"')
	  {
		  char delimeter = *p;
		  while (*p && *p != '\n' && *p != '\r' &&
			  !(i > 1 && *(p-1) == delimeter && *(p-2) != '\\'))
			value[i++] = *(p++);
	  }
	  else
		  while (*p && !isspace(*p) )
			value[i++] = *(p++);

	  value[i] = '\0';

	  //convert to ANSI

	  UTF8ToANSI(CP_ACP, key, -1, ANSI_key, sizeof(ANSI_key));
	  UTF8ToANSI(CP_ACP, value, -1, ANSI_value, sizeof(ANSI_value));


	  //filter key and value and add them to the table
	  TVEFormatString strKey, strValue; 
	  strKey = ANSI_key;
	  strValue = ANSI_value;

	  const char *filter_value;
	  if ((filter_value=filterfunc(strKey.unformat(),strValue.unformat()))!=NULL)
	  {
		strValue = filter_value; //ANSI

		if (bSave)
		{
		    const char *v = TVSGetValue(h,strKey.unformat());
			//Galtech: fixed 24.10.2002
			if (v /*&& strcmp(v, strValue)*/)
			{
				TVEFormatString str;
				str = v;
				fprintf(of, "%s=%s\n", (const char *)strKey,
					(const char *)str.format());
			}
			TVSUpdateSaveDone(h,strKey.unformat(),1);
		}
		else
			AddMemberToTable(h, strKey.unformat(),strValue.unformat());
	  }
	  else
		return TVSINTERR_ILEGAL_CONF_WORD;
  }

  if (bSave)
  {
	  int type = FIRST_ENTRY;
	  int saveDone;
	  char *key, *value;  

	  //get from table in ansi
	  while(TVSGetNextEntry(type,h,&key,&value,&saveDone)){
		type = NEXT_ENTRY;
		if (!saveDone)
		if (filterfunc(key,value)){
     		TVEFormatString Key, Value;
			Key = key;
			Value = value;

			fprintf(of, "%s=%s\n",(const char *)Key, 
				(const char *)Value.format());
		}
      }
	  fclose(of);
  }

  return TVS_SUCCESS;
}

#endif// _ANSI_ONLY //Galtech



/*************************************************************/
/* This funcion load onto a given HashTable handle a given   */
/* .ini file. It distroys the privious value that was stored */
/* under a key that allready existed. The only exeption is   */
/* when the key is 'INCLUDE_PATH' then, it conctanates the   */
/* new value to the old one.                                 */
/*************************************************************/

BOOL WINAPI TVSLoad(const char* FileName,TableHandle h, filterfunction func)
{
  if (func)
      filterfunc = func;
  else
      filterfunc = TVSDummyfilter;
  //  printf("Inside TVSLoad\n");

#ifndef _ANSI_ONLY //Galtech
  FILE *fp = fopen(FileName,"r");

  //fix: 28.04.2002
  if (!fp)
	  return  FALSE;

  if ((ParseError = Parse(fp, h, NULL)) != TVS_SUCCESS)
  {
    fclose(fp);
    return FALSE;
  }

  fclose(fp);
  //Galtech
#else
  strcpy(InputFileName,FileName);
  yyin = fopen(FileName,"r");
  Handle = h;
  mode = Load;
  if(yyin) {
    if (ParseFlag) {
      ParseFlag = 0;
      yyrestart(yyin);
    }
    lineNumber = 1;
    ParseFlag = 1;
    yyparse();
    fclose(yyin);
  }
  if (ParseError)
    return FALSE;
#endif //_ANSI_ONLY
  return TRUE;
}

/*************************************************************/
/* This function saves a given HashTable on a given .ini file*/
/* If the file does noe exist it is created, else, it is     */
/* updated.                                                  */
/*************************************************************/

void TVSSaveOrAppend(const char* FileName, TableHandle h,filterfunction func,int setAppendMode)
{
	if (func)
		filterfunc = func;
	else
		filterfunc = TVSDummyfilter;
		//  printf("Inside TVSSave\n");
#ifndef _ANSI_ONLY
//galtech
	if (TVSIsTableEmpty(h))
		return;

	FILE *fp = fopen(FileName,"r");

	if(fp && !setAppendMode) {

		char IniTmpName[MAX_PATH];
		strcpy(IniTmpName, FileName);
#ifdef __TV_WINDOWS__
		char * p = __STRRCHR(IniTmpName,'\\');//Galtech
		*p = '\0';
	    strcat(IniTmpName,"\\tmp.ini");
#else
		char * p = __STRRCHR(IniTmpName,'/');//Galtech
		*p = '\0';
	    strcat(IniTmpName,"/tmp.ini");
#endif
		Parse(fp, h, IniTmpName);
		fclose(fp);

#ifndef __TV_UNIX__
		remove(FileName);
		rename(IniTmpName,FileName);
#else
		rename(IniTmpName,FileName);
#endif
	} else {
    FILE* f =  setAppendMode ? fopen(FileName,"a") : fopen(FileName,"w");
    if(!f) {
      ParseError = TVSINTERR_CANNOT_OPEN_FILE_FOR_WRITE;
      return;
		}
		TVEFormatString Key,Value;
		char *key;
		char *value;
		int type;
		int saveDone;

		type = FIRST_ENTRY;
		while(TVSGetNextEntry(type,h,&key,&value,&saveDone)) {
			type = NEXT_ENTRY;
			if (filterfunc(key,value)){ //if key and value are accepted
					Key = key;
					Value = value;
					fprintf(f,"%s=%s\n", (const char *)Key,
						(const char *)Value.format());
			}
		}
		fclose(f);
	}
#else //_ANSI_ONLY
  if (func)
    filterfunc = func;
	else
		filterfunc = TVSDummyfilter;
		//  printf("Inside TVSSave\n");
	Handle = h;

	if (TVSIsTableEmpty(h))
		return;

	yyin = fopen(FileName,"r");

	if(yyin && !setAppendMode) {
		mode = Save;

		if (ParseFlag) {
			lineNumber = 1;
			ParseFlag = 0;
			yyrestart(yyin);
		}

 
		char dir[MAX_PATH];
		strcpy(InputFileName,FileName);

#ifdef __TV_WINDOWS__
		char * p = __STRRCHR(InputFileName,'\\');
    *p = '\0';
    strcpy(dir,InputFileName);
		strcpy(IniTmpName,dir);
    strcat(IniTmpName,"\\tmp.ini");
#else
		char * p = __STRRCHR(InputFileName,'/');
    *p = '\0';
    strcpy(dir,InputFileName);
		strcpy(IniTmpName,dir);
    strcat(IniTmpName,"/tmp.ini");
#endif
		yyparse();
		fclose(yyin);
		ParseFlag = 1;
#ifndef __TV_UNIX__
		remove(FileName);
		rename(IniTmpName,FileName);
#else
		rename(IniTmpName,FileName);
#endif
	} else {
    FILE* f =  setAppendMode ? fopen(FileName,"a") : fopen(FileName,"w");
    if(!f) {
      ParseError = TVSINTERR_CANNOT_OPEN_FILE_FOR_WRITE;
      return;
		}
		TVEFormatString Key,Value;
		char *key;
		char *value;
		int type;
		int saveDone;

		type = FIRST_ENTRY;
		while(TVSGetNextEntry(type,Handle,&key,&value,&saveDone)) {
			type = NEXT_ENTRY;
			if (filterfunc(key,value)){ //if key and value are accepted
					Key = key;
					Value = value;
					fprintf(f,"%s=%s\n",(const char *) Key.format(),
					(const char *) Value.format());
			}
		}
		fclose(f);
	}
#endif //_ANSI_ONLY
}

void WINAPI TVSAppend(const char* FileName, TableHandle h, filterfunction func)
{
  TVSSaveOrAppend(FileName, h, func,TRUE);

}

void WINAPI TVSSave(const char* FileName, TableHandle h, filterfunction func)
{
  TVSSaveOrAppend(FileName, h, func,FALSE);
}
//------------------------------------------------------------------------------
const char* TVSDummyfilter(const char* key,const char* value)
{
 (void) key;
 return value;
}

