/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
///////////////////////////////////////////////////////////////////////////////
//
// (c) 2000 GalTech Soft. All rights reserved.
//
// unidef.h
// general code page definitions
// Unicode Library. 
///////////////////////////////////////////////////////////////////////////////
#include <portable.h>
#ifndef _UNIDEF_H
#define _UNIDEF_H
#define CP_DEFAULT					0
#define CP_MS_THAI					874
#define CP_MS_JAPANESE				932
#define CP_MS_CHINESE_SIMPLIFIED	936 
#define CP_MS_KOREAN				949
#define CP_MS_CHINESE_TRADITIONAL	950
#define CP_MS_EBCDIC				1026  
#define CP_UCS2						1200
#define CP_UTF7						65000
#define CP_UTF8						65001
#define CP_MS_LATIN2				1250
#define CP_MS_CYRILLIC				1251
#define CP_MS_LATIN1				1252
#define CP_MS_GREEK					1253 
#define CP_MS_TURKISH				1254 
#define CP_MS_HEBREW				1255
#define CP_MS_ARABIC				1256
#define CP_MS_BALTIC				1257
#define CP_MS_LATIN9				28605
#define CP_MS_LATIN3				28593

#define CP_ISO_8859_1				88591
#define CP_ISO_8859_2				88592
#define CP_ISO_8859_3				88593
#define CP_ISO_8859_4				88594
#define CP_ISO_8859_5				88595
#define CP_ISO_8859_6				88596
#define CP_ISO_8859_7				88597
#define CP_ISO_8859_8				88598
#define CP_ISO_8859_9				88599

#define CP_MAC_ROMAN				10000
#define CP_MAC_JAPANESE				10001
#define CP_MAC_GREEK				10006
#define CP_MAC_CYRILIC				10007
#define CP_MAC_HEBREW				10008
#define CP_MAC_LATIN2				10029
#define CP_MAC_ICELANDIC			10079
#define CP_MAC_TURKISH				10081

#define CP_EUCJP					51932
#define CP_EUCCN					51936
#define CP_EUCKR					CP_MS_KOREAN


#define CP_JIS						2022
#define CP_JAPANESEAUTODETECT		2023

#define CP_KOI_8					20866

#define CP_IBM_850					40850
#define CP_IBM_852					40852
#define CP_IBM_861					40861
#define CP_IBM_862					40862
#define CP_IBM_866					40866
#define CP_IBM_869					40869

#define CP_LAST						0xffffffff



#ifndef MAX_CSNAME
#define MAX_CSNAME	20
#endif


#define MAX_FRIENDLY_NAME 50

typedef struct _friendlyname2id_t {
	char	friendly_name[MAX_FRIENDLY_NAME];
	UINT	cs_id;
} friendlyname2id_t;

static friendlyname2id_t friendly2id_tbl[] = {
	{"Arabic(ISO)",CP_MS_ARABIC},
	{"Baltic(ISO)",CP_MS_BALTIC},
	{"Central European(DOS)",CP_IBM_852},
	{"Central European(ISO)",CP_MS_LATIN2},
	{"Chinese Simplified(GB2312)",CP_MS_CHINESE_SIMPLIFIED},
	{"Chinese Traditional(Big5)",CP_MS_CHINESE_TRADITIONAL},
	{"Cyrillic(DOS)",CP_IBM_866},
	{"Cyrillic(ISO)",CP_MS_CYRILLIC},
	{"Cyrillic(KOI8-R)",CP_KOI_8},
	{"Greek(ISO)",CP_MS_GREEK},
	{"Greek, Modern(DOS)",CP_IBM_869},
	{"Hebrew(DOS)",CP_IBM_862},
	{"Hebrew(ISO-Visual)",CP_MS_HEBREW},
	{"Hebrew(ISO-Logical)",CP_MS_HEBREW},
	{"Hebrew(Mac)",CP_MAC_HEBREW},
	{"Icelandic(DOS)",CP_IBM_861},
	{"Japanese(EUC)",CP_EUCJP},
	{"Japanese(JIS)",CP_JIS},
	{"Japanese(Shift-JIS)",CP_MS_JAPANESE},
	{"Korean(ISO)",CP_MS_KOREAN},
//	{"Korean(EUC)",CP_EUCKR},
	{"Latin 3 (ISO)",CP_ISO_8859_3},
//	{"Thai(Windows)",CP_MS_THAI},
	{"Turkish(ISO)",CP_MS_TURKISH},
	{"Unicode",CP_UCS2},
	{"Unicode(UTF-7)",CP_UTF7},
	{"Unicode(UTF-8)",CP_UTF8},
	{"US-ASCII",CP_MS_LATIN1},
	{"Western European(DOS)",CP_IBM_850},
	{"Western European(ISO)",CP_MS_LATIN1},
	{"Western European(Mac)",CP_MAC_ROMAN},
	{"", CP_LAST}
};



typedef struct _csname2id_t {
	char	cs_name[MAX_CSNAME];
	UINT	cs_id;
} csname2id_t;

/* Charset names and aliases from RFC 1700. Also encloded equivelend Java encoding names. Names are case
 * insenstive. Currently searches table linearly, so keep commonly used names at the beginning.
 */
static csname2id_t csname2id_tbl[] = {
											/* default if not specified */
			{"x-default", CP_DEFAULT},			/* or unknown charset		*/
			{"iso-8859-1", CP_MS_LATIN1},
			{"iso-2022-jp",CP_JIS},
			{"Shift_JIS",  CP_MS_JAPANESE},
			{"euc-jp", CP_EUCJP},
			{"macintosh", CP_MAC_ROMAN}, 
			{"iso-8859-2", CP_MS_LATIN2},
			{"iso-8859-3", CP_MS_LATIN3},
			{"iso-8859-4", CP_MS_BALTIC},
			{"iso-8859-5", CP_MS_CYRILLIC},
			{"iso-8859-6", CP_MS_ARABIC},
			{"iso-8859-7", CP_MS_GREEK},
			{"iso-8859-8", CP_MS_HEBREW},
			{"iso-8859-9", CP_MS_TURKISH},
			{"iso-8859-15", CP_MS_LATIN9},
			{"iso8859-15", CP_MS_LATIN9},
			{"EUC-KR", CP_EUCKR},	
			{"ks_c_5601-1987", CP_MS_KOREAN},
			{"gb2312", CP_MS_CHINESE_SIMPLIFIED},
			{"big5", CP_MS_CHINESE_TRADITIONAL},
			{"koi8-r",  CP_KOI_8},
			{"unicode", CP_UCS2},
			{"utf-8", CP_UTF8},
			{"utf-7", CP_UTF7},
			{"x-mac-ce", CP_MAC_LATIN2},
			{"us-ascii", CP_MS_LATIN1},
			{"iso-2022-jp-2", CP_JIS},		/* treat same as iso-2022-jp*/
			{"jis_x0208-1983", CP_JIS},
			{"x-jisx0208-11",  CP_JIS},
			{"jis_x0201",  CP_JIS},
			{"jis_x0212-1990", CP_JIS},
			{"x-ksc5601-11", CP_MS_KOREAN},
			{"gb_2312-80", CP_MS_CHINESE_SIMPLIFIED},
			{"x-gb2312-11",CP_MS_CHINESE_SIMPLIFIED},
			{"x-mac-roman", CP_MAC_ROMAN}, 
//			{"x-euc-tw", "CNS11643", CS_CNS_8BIT},
//			{"x-cns11643-1", "CNS11643", CS_CNS11643_1},
//			{"x-cns11643-2", "CNS11643", CS_CNS11643_2},
//			{"x-cns11643-1110", "CNS11643", CS_CNS11643_1110},
			{"iso-2022-kr", CP_MS_KOREAN},
			{"x-x-big5", CP_MS_CHINESE_TRADITIONAL},
//			{"x-tis620", "TIS620", CS_TIS620},
//			{"adobe-symbol-encoding", "Symbol", CS_SYMBOL},
//			{"x-dingbats", "DingBats", CS_DINGBATS},
//			{"x-dectech", "DECTECH", CS_DECTECH},
			{"x-mac-cyrillic", CP_MAC_CYRILIC}, 
			{"x-mac-greek", CP_MAC_GREEK}, 
			{"x-mac-turkish",  CP_MAC_TURKISH}, 
			{"windows-1250", CP_MS_LATIN2},
			{"windows-1251", CP_MS_CYRILLIC}, /* cyrillic */
			{"windows-1253", CP_MS_GREEK},  /* greek */
			{"windows-1255", CP_MS_HEBREW},  /* Hebrew */
			{"windows-1256", CP_MS_ARABIC},  /* Arabic */
			{"windows-1257", CP_MS_BALTIC},  /* baltic */
			{"UTF-8", CP_UTF8},
			{"UTF-7", CP_UTF7},
			{"ISO-10646-UCS-2", CP_UCS2},
//			{"ISO-10646-UCS-4", "UCS4", CS_UCS4},
//			{"x-imap4-modified-utf7", "", CS_IMAP4_UTF7},
//			{"T.61-8bit", "", CS_T61},
//			{"HZ-GB-2312", "", CS_HZ},

	/* cP866 support for ibm */
//			{"ibm866", "Cp866", CS_CP_866},         /* PC Russian */
//			{"cp866", "Cp866", CS_CP_866},          /* PC Russian */


	/* aliases for us-ascii: */
			{"ansi_x3.4-1968", CP_MS_LATIN1},
			{"iso-ir-6", CP_MS_LATIN1},
			{"ansi_x3.4-1986", CP_MS_LATIN1},
			{"iso_646.irv:1991",CP_MS_LATIN1},
			{"ascii", CP_MS_LATIN1},
			{"iso646-us", CP_MS_LATIN1},
			{"us", CP_MS_LATIN1},
			{"ibm367", CP_MS_LATIN1},
			{"cp367", CP_MS_LATIN1},
			{"csASCII",CP_MS_LATIN1},

	/* aliases for iso_8859-1:	*/
			{"latin1", CP_MS_LATIN1},
			{"iso_8859-1", CP_MS_LATIN1},
			{"iso_8859-1:1987", CP_MS_LATIN1},
			{"iso-ir-100", CP_MS_LATIN1},
			{"l1", CP_MS_LATIN1},
			{"ibm819", CP_MS_LATIN1},
			{"cp819",  CP_MS_LATIN1},
//			{"ISO-8859-1-Windows-3.0-Latin-1", CP_MS_LATIN1},
//			{"ISO-8859-1-Windows-3.1-Latin-1", CP_MS_LATIN1},
			{"windows-1252", CP_MS_LATIN1},
			{"iso8859-1", CP_MS_LATIN1},
			
	/* aliases for ISO_8859-2:	*/
			{"latin2", CP_MS_LATIN2},			
			{"iso_8859-2", CP_MS_LATIN2},
			{"iso_8859-2:1987", CP_MS_LATIN2},
			{"iso-ir-101", CP_MS_LATIN2},
			{"l2", CP_MS_LATIN2},											
//			{"ISO-8859-2-Windows-Latin-2", CP_MS_LATIN2},

	/* aliases for KS_C_5601-1987:	*/
			{"ks_c_5601-1987", CP_MS_KOREAN},
			{"iso-ir-149", CP_MS_KOREAN},
			{"ks_c_5601-1989", CP_MS_KOREAN},
			{"ksc_5601", CP_MS_KOREAN},
			{"ks_c_5601", CP_MS_KOREAN},
			{"korean", CP_MS_KOREAN},
			{"csKSC56011987", CP_MS_KOREAN},

	/* aliases for iso-2022-kr:	*/
			{"csISO2022KR", CP_MS_KOREAN},

	/* aliases for euc-kr:	*/
//			{"csEUCKR", "", CS_KSC_8BIT},
//			{"ksc5601", "", CS_KSC_8BIT},

	/* aliases for iso-2022-jp:	*/
			{"csISO2022JP", CP_JIS},

	/* aliases for iso-2022-jp-2:	*/
			{"csISO2022JP2", CP_JIS},

	/* aliases for GB_2312-80:	*/
			{"iso-ir-58", CP_MS_CHINESE_SIMPLIFIED},
			{"chinese", CP_MS_CHINESE_SIMPLIFIED},
			{"csISO58GB231280", CP_MS_CHINESE_SIMPLIFIED},

	/* aliases for gb2312:	*/
			{"csGB2312", CP_MS_CHINESE_SIMPLIFIED},
			{"CN-GB", CP_MS_CHINESE_SIMPLIFIED},  /* Simplified Chinese */
			{"CN-GB-ISOIR165", CP_MS_CHINESE_SIMPLIFIED},  /* Simplified Chinese */

	/* aliases for big5:	*/
			{"csBig5", CP_MS_CHINESE_TRADITIONAL},
			{"CN-Big5", CP_MS_CHINESE_TRADITIONAL},  /* Traditional Chinese */

	/* aliases for iso-8859-7:	*/
			{"iso-ir-126", CP_MS_GREEK},
			{"iso_8859-7", CP_MS_GREEK},
			{"iso_8859-7:1987", CP_MS_GREEK},
			{"elot_928", CP_MS_GREEK},
			{"ecma-118", CP_MS_GREEK},
			{"greek", CP_MS_GREEK},
			{"greek8", CP_MS_GREEK},
			{"csISOLatinGreek", CP_MS_GREEK},

	/* aliases for iso-8859-5:	*/
			{"iso-ir-144", CP_MS_CYRILLIC},
			{"iso_8859-5", CP_MS_CYRILLIC},
			{"iso_8859-5:1988", CP_MS_CYRILLIC},
			{"cyrillic", CP_MS_CYRILLIC},
			{"csISOLatinCyrillic", CP_MS_CYRILLIC},

	/* aliases for iso-8859-4:	*/
			{"iso-ir-110", CP_MS_BALTIC},
			{"iso_8859-4", CP_MS_BALTIC},
			{"iso_8859-4:1988", CP_MS_BALTIC},
			{"latin4", CP_MS_BALTIC},
			{"l4", CP_MS_BALTIC},
			{"csISOLatin4", CP_MS_BALTIC},

	/* aliases for jis_x0212-1990:	*/
			{"x0212", CP_JIS},
			{"iso-ir-159", CP_JIS},
//			{"csISO159JISX02121990", CP_JIS},

	/* aliases for jis_x0201:	*/
			{"x0201", CP_JIS},
			{"csHalfWidthKatakana", CP_JIS},

	/* aliases for koi8-r:	*/
			{"csKOI8R", CP_KOI_8},

	/* aliases for Shift_JIS:	*/
			{"x-sjis", CP_MS_JAPANESE},
			{"ms_Kanji", CP_MS_JAPANESE},
			{"csShiftJIS", CP_MS_JAPANESE},
			{"Windows-31J", CP_MS_JAPANESE},

	/* aliases for x-euc-jp:	*/
//			{"Extended_UNIX_Code_Packed_Format_for_Japanese", CP_EUCJP},
			{"csEUCPkdFmtJapanese", CP_EUCJP},
			{"x-euc-jp", CP_EUCJP},

	/* aliases for adobe-symbol-encoding:	*/
//			{"csHPPSMath", "", CS_SYMBOL},

	/* aliases for iso-8859-5-windows-latin-5:	*/
			{"csWindows31Latin5",  CP_MS_CYRILLIC},
//			{"iso-8859-5-windows-latin-5", CP_MS_CYRILLIC},

	/* aliases for windows-1250:	*/
			{"x-cp1250",  CP_MS_LATIN2},

	/* aliases for windows-1253:	*/
			{"x-cp1253", CP_MS_GREEK}, 

	/* aliases for windows-1254:	*/
			{"windows-1254", CP_MS_TURKISH},  /* turkish */

	/* aliases for UNICODE-1-1:	*/
			{"csUnicode11", CP_UCS2},
			{"ISO-10646-UCS-BASIC", CP_UCS2},
			{"csUnicodeASCII",  CP_UCS2},
//			{"ISO-10646-Unicode-Latin1", CP_UCS2},
			{"csUnicodeLatin1", CP_UCS2},
			{"ISO-10646", CP_UCS2},
			{"ISO-10646-J-1", CP_UCS2},

	/* aliases for UTF-7:	*/
			{"x-UNICODE-2-0-UTF-7", CP_UTF7},
			{"UNICODE-1-1-UTF-7", CP_UTF7},
			{"UNICODE-2-0-UTF-7", CP_UTF7},	/* Appeared in UTF-7 RFC Draft */
			{"csUnicode11UTF7", CP_UTF7},

	/* aliases for T.61-8bit:	*/
//			{"T.61", "", CS_T61},
//			{"iso-ir-103", "", CS_T61},
//			{"csISO103T618bit", "", CS_T61},

	/* aliases for UNICODE-1-1-UTF-8:	*/
			{"UNICODE-1-1-UTF-8", CP_UTF8},

//			{"x-user-defined", "", CS_USER_DEFINED_ENCODING},											
//			{"x-user-defined", "", CS_USRDEF2},											

//			{"RESERVED", "", CS_DEFAULT},			/* or unknown charset		*/

			{"", CP_LAST}
};
#ifdef __TV_UNIX__
#define MAX_LCNAME	40

typedef struct _lcname2id_t {
	char	lc_name[MAX_LCNAME];
	UINT	lc_id;
} lcname2id_t;

/* locale name as define in Unix (see man locale, iconv.5, iconv_unicode.5 etc)
 */
static lcname2id_t lcname2id_tbl[] = {
											/* default if not specified */
			{"C", CP_MS_LATIN1},			/* or unknown charset		*/
			{"common", CP_MS_LATIN1},
			{"POSIX",CP_MS_LATIN1},
			{"UTF-8",  CP_UTF8},
			{"ISO8859-1", CP_MS_LATIN1}, 
			{"ISO8859-2", CP_MS_LATIN2},
			{"ISO8859-3", CP_MS_LATIN3},
			{"ISO8859-4", CP_MS_BALTIC},
			{"ISO8859-5", CP_MS_CYRILLIC},
			{"ISO8859-6", CP_MS_ARABIC},
			{"ISO8859-7", CP_MS_GREEK},
			{"ISO8859-8", CP_MS_HEBREW},
			{"ISO8859-9", CP_MS_TURKISH},
			{"ISO8859-15", CP_MS_LATIN9},
			{"ja", CP_EUCJP},
			{"eucJP", CP_EUCJP},
			{"japanese", CP_EUCJP},
			{"PCK", CP_EUCJP},
			{"UTF-8", CP_UTF8},
			{"", CP_LAST}
};
#endif //UNIX
#endif // _UNIDEF_H
