/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifdef WIN32
#include <windows.h>
#include <assert.h>
#include <limits.h>
#endif
#include <stdio.h> 
#include <malloc.h>

#include "wlibrary.h"

#ifdef __TV_UNIX__
//typedef char* LPSTR;
//typedef const char* LPCSTR;
typedef gt_wchar_t* LPWSTR;
typedef gt_wchar_t* LPWSTR;
typedef const gt_wchar_t* LPCWSTR;
#include <sys/ddi.h>
typedef unsigned short gt_wchar_t;
#else
typedef wchar_t gt_wchar_t;
#endif UNIX

#include "cwstring.h"
#include "unicode.h"

#ifdef AFX_CORE1_SEG
#pragma code_seg(AFX_CORE1_SEG)
#endif

/////////////////////////////////////////////////////////////////////////////
// static class data, special inlines

// afxWChNil is left for backward compatibility
gt_wchar_t afxWChNil = '\0';

// For an empty string, m_pchData will point here
// (note: avoids special case of checking for NULL m_pchData)
// empty string data (and locked)
static int rgInitData[] = { -1, 0, 0, 0 };
static CWStringData* afxDataNil = (CWStringData*)&rgInitData;
static LPCWSTR afxPWChNil = (LPCWSTR)(((BYTE*)&rgInitData)+sizeof(CWStringData));
// special function to make afxEmptyString work even during initialization
const CWString& GetEmptyWideString()
	{ return *(CWString*)&afxPWChNil; }


//////////////////////////////////////////////////////////////////////////////
// Construction/Destruction

CWString::CWString()
{
	Init();
}

CWString::CWString(const CWString& stringSrc)
{
#ifdef WIN32
	assert(stringSrc.GetData()->nRefs != 0);
	if (stringSrc.GetData()->nRefs >= 0)
	{
		assert(stringSrc.GetData() != afxDataNil);
		m_pchData = stringSrc.m_pchData;
		InterlockedIncrement(&GetData()->nRefs);
	}
	else
	{
		Init();
		*this = stringSrc.m_pchData;
	}
#else //UNIX
		Init();
		*this = stringSrc.m_pchData;
#endif //WIN32
}

CWString::CWString(gt_wchar_t ch, int nLength)
{
	Init();
	if (nLength >= 1)
	{
		AllocBuffer(nLength);
		for (int i = 0; i < nLength; i++)
			m_pchData[i] = ch;
	}
}

CWString::CWString(LPCWSTR lpch, int nLength)
{
	Init();
	if (nLength != 0)
	{
		AllocBuffer(nLength);
		memcpy(m_pchData, lpch, nLength*sizeof(gt_wchar_t));
	}
}


void CWString::AllocBuffer(int nLen)
// always allocate one extra character for '\0' termination
// assumes [optimistically] that data length will equal allocation length
{
	assert(nLen >= 0);
	assert(nLen <= INT_MAX-1);    // max size (enough room for 1 extra)

	if (nLen == 0)
		Init();
	else
	{
		CWStringData* pData =
			(CWStringData*)new BYTE[sizeof(CWStringData) + (nLen+1)*sizeof(gt_wchar_t)];
		pData->nRefs = 1;
		pData->data()[nLen] = '\0';
		pData->nDataLength = nLen;
		pData->nAllocLength = nLen;
		m_pchData = pData->data();
	}
}
CWStringData* CWString::GetData() const
	{ assert(m_pchData != NULL); return ((CWStringData*)m_pchData)-1; }
void CWString::Init()
	{ m_pchData = EmptyWideString.m_pchData; }
int CWString::GetLength() const
	{ return GetData()->nDataLength; }
int CWString::GetAllocLength() const
	{ return GetData()->nAllocLength; }
BOOL CWString::IsEmpty() const
	{ return GetData()->nDataLength == 0; }
CWString::operator LPCWSTR() const
{ return m_pchData; } 
int PASCAL CWString::SafeStrlen(LPCWSTR lpsz)
	{ return (lpsz == NULL) ? 0 : wcslen(lpsz); }
int CWString::Compare(LPCWSTR lpsz) const
{
	int nStr2Len = lpsz != NULL ? wcslen(lpsz) : 0;
	int nStr1Len = !IsEmpty() ? GetLength() : 0;

	for(int i = 0; i < nStr1Len && i < nStr2Len; i++)
	{
		if(lpsz[i] != m_pchData[i])
			return (m_pchData[i] - lpsz[i]);
	}
	return nStr1Len - nStr2Len;
}
//CompareNoCase funcrion is calling the Compare, and doesn't support
//for now no case
int CWString::CompareNoCase(LPCWSTR lpsz) const
{ 
	return w_stricmp( LPCWSTR(*this), lpsz );
}   

gt_wchar_t CWString::GetAt(int nIndex) const
{
	assert(nIndex >= 0);
	assert(nIndex < GetData()->nDataLength);
	return m_pchData[nIndex];
}
gt_wchar_t CWString::operator[](int nIndex) const
{
	// same as GetAt
//	assert(nIndex >= 0);
//	assert(nIndex < GetData()->nDataLength);
	return m_pchData[nIndex];
}

void CWString::Release()
{
	if (GetData() != afxDataNil)
	{
		assert(GetData()->nRefs != 0);
#ifdef WIN32
		if (InterlockedDecrement(&GetData()->nRefs) <= 0)
#endif //WIN32
			delete[] (BYTE*)GetData();
		Init();
	}
}

void PASCAL CWString::Release(CWStringData* pData)
{
	if (pData != afxDataNil)
	{
		assert(pData->nRefs != 0);
#ifdef WIN32
		if (InterlockedDecrement(&pData->nRefs) <= 0)
#endif //WIN32
			delete[] (BYTE*)pData;
	}
}

void CWString::Empty()
{
	if (GetData()->nDataLength == 0)
		return;
	if (GetData()->nRefs >= 0)
		Release();
	else
		*this = &afxWChNil;
	assert(GetData()->nDataLength == 0);
	assert(GetData()->nRefs < 0 || GetData()->nAllocLength == 0);
}

void CWString::CopyBeforeWrite()
{
	if (GetData()->nRefs > 1)
	{
		CWStringData* pData = GetData();
		Release();
		AllocBuffer(pData->nDataLength);
		memcpy(m_pchData, pData->data(), (pData->nDataLength+1)*sizeof(gt_wchar_t));
	}
	assert(GetData()->nRefs <= 1);
}

////////////////////////////////////////
int CWString::Delete(int nIndex, int nCount /* = 1 */)
{
	if (nIndex < 0)
		nIndex = 0;
	int nNewLength = GetData()->nDataLength;
	if (nCount > 0 && nIndex < nNewLength)
	{
		CopyBeforeWrite();
		int nBytesToCopy = nNewLength - (nIndex + nCount) + 1;

		memcpy(m_pchData + nIndex,
			m_pchData + nIndex + nCount, nBytesToCopy * sizeof(gt_wchar_t));
		GetData()->nDataLength = nNewLength - nCount;
	}

	return nNewLength;
}
/////////////////////////////////////////////////////////////


void CWString::AllocBeforeWrite(int nLen)
{
	if (GetData()->nRefs > 1 || nLen > GetData()->nAllocLength)
	{
		Release();
		AllocBuffer(nLen);
	}
	assert(GetData()->nRefs <= 1);
}

CWString::~CWString()
//  free any attached data
{
	if (GetData() != afxDataNil)
	{
#ifdef WIN32
		if (InterlockedDecrement(&GetData()->nRefs) <= 0)
#endif //WIN32
			delete[] (BYTE*)GetData();
	}
}

//////////////////////////////////////////////////////////////////////////////
// Helpers for the rest of the implementation

void CWString::AllocCopy(CWString& dest, int nCopyLen, int nCopyIndex,
	 int nExtraLen) const
{
	// will clone the data attached to this string
	// allocating 'nExtraLen' characters
	// Places results in uninitialized string 'dest'
	// Will copy the part or all of original data to start of new string

	int nNewLen = nCopyLen + nExtraLen;
	if (nNewLen == 0)
	{
		dest.Init();
	}
	else
	{
		dest.AllocBuffer(nNewLen);
		memcpy(dest.m_pchData, m_pchData+nCopyIndex, nCopyLen * sizeof(gt_wchar_t));
	}
}

//////////////////////////////////////////////////////////////////////////////
// More sophisticated construction

CWString::CWString(LPCWSTR lpsz)
{
	Init();
	int nLen = SafeStrlen(lpsz);
	if (nLen != 0)
	{
		AllocBuffer(nLen);
		memcpy(m_pchData, lpsz, nLen*sizeof(gt_wchar_t));
	}
}

/////////////////////////////////////////////////////////////////////////////
// Special conversion constructor, converts from ANSI string

CWString::CWString(LPCSTR lpsz)
{
	Init();
	int nSrcLen = 0;
	if(lpsz != NULL)
	{
		nSrcLen =  UL_StrLength( CP_ACP, lpsz, -1);

	}
	if (nSrcLen != 0)
	{
		AllocBuffer(nSrcLen);
		UL_MultiByteToWideChar(CP_ACP, 0 /*MB_PRECOMPOSED*/,
			lpsz, nSrcLen, m_pchData, nSrcLen);
		ReleaseBuffer();
	}
}

//////////////////////////////////////////////////////////////////////////////
// Assignment operators
//  All assign a new value to the string
//      (a) first see if the buffer is big enough
//      (b) if enough room, copy on top of old buffer, set size and type
//      (c) otherwise free old string data, and create a new one
//
//  All routines return the new string (but as a 'const CWString&' so that
//      assigning it again will cause a copy, eg: s1 = s2 = "hi there".
//

void CWString::AssignCopy(int nSrcLen, LPCWSTR lpszSrcData)
{
	AllocBeforeWrite(nSrcLen);
	memcpy(m_pchData, lpszSrcData, nSrcLen*sizeof(gt_wchar_t));
	GetData()->nDataLength = nSrcLen;
	m_pchData[nSrcLen] = '\0';
}

const CWString& CWString::operator=(const CWString& stringSrc)
{
	if (m_pchData != stringSrc.m_pchData)
	{
		if ((GetData()->nRefs < 0 && GetData() != afxDataNil) ||
			stringSrc.GetData()->nRefs < 0)
		{
			// actual copy necessary since one of the strings is locked
			AssignCopy(stringSrc.GetData()->nDataLength, stringSrc.m_pchData);
		}
		else
		{
			// can just copy references around
			Release();
			assert(stringSrc.GetData() != afxDataNil);
			m_pchData = stringSrc.m_pchData;
#ifdef WIN32
			InterlockedIncrement(&GetData()->nRefs);
#endif //WIN32
		}
	}
	return *this;
}

const CWString& CWString::operator=(LPCSTR lpsz)
{
	int nSrcLen = 0;
	if(lpsz != NULL)
	{
		nSrcLen =  UL_StrLength( CP_ACP, lpsz, -1);

	}
	AllocBuffer(nSrcLen);
	UL_MultiByteToWideChar(CP_ACP, 0 /*MB_PRECOMPOSED*/,
		lpsz, nSrcLen, m_pchData, nSrcLen);
	ReleaseBuffer();
	return *this;
}

const CWString& CWString::operator=(LPCWSTR lpsz)
{
	AssignCopy(SafeStrlen(lpsz), lpsz);
	return *this;
}

const CWString& CWString::operator=(gt_wchar_t ch)
{
	AssignCopy(1, &ch);
	return *this;
}


//////////////////////////////////////////////////////////////////////////////
// Very simple sub-string extraction

CWString CWString::Mid(int nFirst) const
{
	return Mid(nFirst, GetData()->nDataLength - nFirst);
}

CWString CWString::Mid(int nFirst, int nCount) const
{
	// out-of-bounds requests return sensible things
	if (nFirst < 0)
		nFirst = 0;
	if (nCount < 0)
		nCount = 0;

	if (nFirst + nCount > GetData()->nDataLength)
		nCount = GetData()->nDataLength - nFirst;
	if (nFirst > GetData()->nDataLength)
		nCount = 0;

	CWString dest;
	AllocCopy(dest, nCount, nFirst, 0);
	return dest;
}

CWString CWString::Right(int nCount) const
{
	if (nCount < 0)
		nCount = 0;
	else if (nCount > GetData()->nDataLength)
		nCount = GetData()->nDataLength;

	CWString dest;
	AllocCopy(dest, nCount, GetData()->nDataLength-nCount, 0);
	return dest;
}

CWString CWString::Left(int nCount) const
{
	if (nCount < 0)
		nCount = 0;
	else if (nCount > GetData()->nDataLength)
		nCount = GetData()->nDataLength;

	CWString dest;
	AllocCopy(dest, nCount, 0, 0);
	return dest;
}


// strspn equivalent
CWString CWString::SpanIncluding(LPCWSTR lpszCharSet) const
{
	return Left(wcscspn(m_pchData, lpszCharSet));
}

// strcspn equivalent
CWString CWString::SpanExcluding(LPCWSTR lpszCharSet) const
{
	return Left(wcscspn(m_pchData, lpszCharSet));
}


int CWString::CopyAsUTF8(LPSTR lpString, int nMaxCount)
{
	return UL_WideCharToUTF8(LPCWSTR(*this), -1, lpString, nMaxCount);
}
int CWString::CopyAsUTF7(LPSTR lpString, int nMaxCount)
{
	return UL_WideCharToUTF7(LPCWSTR(*this), -1, lpString, nMaxCount);
}

int CWString::CopyAsNative(LPSTR lpString, int nMaxCount) const
{
#ifdef WIN32
	return WideCharToMultiByte(CP_ACP, 0, LPCWSTR(*this),	-1, lpString, nMaxCount, NULL, NULL);
#else //UNIX
	return UL_WideCharToMultiByte(CP_ACP, 0, LPCWSTR(*this),	-1, lpString, nMaxCount, NULL, NULL);
#endif //WIN32
}

BOOL CWString::GetFromUTF7(LPCSTR lpszString)
{
	int nSrcLen =  UL_StrLength( CP_UTF7, lpszString, -1);
	gt_wchar_t* pTarget = GetBuffer(nSrcLen );
	UL_MultiByteToWideChar(CP_UTF7, 0 /*MB_PRECOMPOSED*/,
		lpszString, -1, pTarget, nSrcLen);
	ReleaseBuffer();
	return TRUE;

}

BOOL CWString::GetFromNative(LPCSTR lpszString)
{
	int nSrcLen =  UL_StrLength( CP_ACP, lpszString, -1);
	gt_wchar_t* pTarget = GetBuffer(nSrcLen );
	UL_MultiByteToWideChar(CP_ACP, 0/*MB_PRECOMPOSED*/,
		lpszString, -1, pTarget, nSrcLen);
	ReleaseBuffer();
	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// concatenation

// NOTE: "operator+" is done as friend functions for simplicity
//      There are three variants:
//          CWString + CWString
// and for ? = gt_wchar_t, LPCWSTR
//          CWString + ?
//          ? + CWString

void CWString::ConcatCopy(int nSrc1Len, LPCWSTR lpszSrc1Data,
	int nSrc2Len, LPCWSTR lpszSrc2Data)
{
  // -- master concatenation routine
  // Concatenate two sources
  // -- assume that 'this' is a new CWString object

	int nNewLen = nSrc1Len + nSrc2Len;
	if (nNewLen != 0)
	{
		AllocBuffer(nNewLen);
		memcpy(m_pchData, lpszSrc1Data, nSrc1Len*sizeof(gt_wchar_t));
		memcpy(m_pchData+nSrc1Len, lpszSrc2Data, nSrc2Len*sizeof(gt_wchar_t));
	}
}

CWString  operator+(const CWString& string1, const CWString& string2)
{
	CWString s;
	s.ConcatCopy(string1.GetData()->nDataLength, string1.m_pchData,
		string2.GetData()->nDataLength, string2.m_pchData);
	return s;
}

CWString  operator+(const CWString& string, LPCWSTR lpsz)
{
	CWString s;
	s.ConcatCopy(string.GetData()->nDataLength, string.m_pchData,
		CWString::SafeStrlen(lpsz), lpsz);
	return s;
}

CWString  operator+(const CWString& string, gt_wchar_t wch)
{
	CWString s;
	s.ConcatCopy(string.GetData()->nDataLength, string.m_pchData,
		1/*sizeof(gt_wchar_t)*/, &wch);
	return s;
}

CWString  operator+(LPCWSTR lpsz, const CWString& string)
{
	CWString s;
	s.ConcatCopy(CWString::SafeStrlen(lpsz), lpsz, string.GetData()->nDataLength,
		string.m_pchData);
	return s;
}

//////////////////////////////////////////////////////////////////////////////
// concatenate in place

void CWString::ConcatInPlace(int nSrcLen, LPCWSTR lpszSrcData)
{
	//  -- the main routine for += operators

	// concatenating an empty string is a no-op!
	if (nSrcLen == 0)
		return;

	// if the buffer is too small, or we have a width mis-match, just
	//   allocate a new buffer (slow but sure)
	if (GetData()->nRefs > 1 || GetData()->nDataLength + nSrcLen > GetData()->nAllocLength)
	{
		// we have to grow the buffer, use the ConcatCopy routine
		CWStringData* pOldData = GetData();
		ConcatCopy(GetData()->nDataLength, m_pchData, nSrcLen, lpszSrcData);
		assert(pOldData != NULL);
		CWString::Release(pOldData);
	}
	else
	{
		// fast concatenation when buffer big enough
		memcpy(m_pchData+GetData()->nDataLength, lpszSrcData, nSrcLen*sizeof(gt_wchar_t));
		GetData()->nDataLength += nSrcLen;
		assert(GetData()->nDataLength <= GetData()->nAllocLength);
		m_pchData[GetData()->nDataLength] = '\0';
	}
}

const CWString& CWString::operator+=(LPCWSTR lpsz)
{
	ConcatInPlace(SafeStrlen(lpsz), lpsz);
	return *this;
}

const CWString& CWString::operator+=(gt_wchar_t ch)
{
	ConcatInPlace(1, &ch);
	return *this;
}

const CWString& CWString::operator+=(const CWString& string)
{
	ConcatInPlace(string.GetData()->nDataLength, string.m_pchData);
	return *this;
}

///////////////////////////////////////////////////////////////////////////////
// Advanced direct buffer access

LPWSTR CWString::GetBuffer(int nMinBufLength)
{
	assert(nMinBufLength >= 0);

	if (GetData()->nRefs > 1 || nMinBufLength > GetData()->nAllocLength)
	{
		// we have to grow the buffer
		CWStringData* pOldData = GetData();
		int nOldLen = GetData()->nDataLength;   // AllocBuffer will tromp it
		if (nMinBufLength < nOldLen)
			nMinBufLength = nOldLen;
		AllocBuffer(nMinBufLength);
		memcpy(m_pchData, pOldData->data(), (nOldLen+1)*sizeof(gt_wchar_t));
		GetData()->nDataLength = nOldLen;
		CWString::Release(pOldData);
	}
	assert(GetData()->nRefs <= 1);

	// return a pointer to the character storage for this string
	assert(m_pchData != NULL);
	return m_pchData;
}

void CWString::ReleaseBuffer(int nNewLength)
{
	CopyBeforeWrite();  // just in case GetBuffer was not called

	if (nNewLength == -1)
		nNewLength = wcslen(m_pchData); // zero terminated

	assert(nNewLength <= GetData()->nAllocLength);
	GetData()->nDataLength = nNewLength;
	m_pchData[nNewLength] = '\0';
}

LPWSTR CWString::GetBufferSetLength(int nNewLength)
{
	assert(nNewLength >= 0);

	GetBuffer(nNewLength);
	GetData()->nDataLength = nNewLength;
	m_pchData[nNewLength] = '\0';
	return m_pchData;
}

void CWString::FreeExtra()
{
	assert(GetData()->nDataLength <= GetData()->nAllocLength);
	if (GetData()->nDataLength != GetData()->nAllocLength)
	{
		CWStringData* pOldData = GetData();
		AllocBuffer(GetData()->nDataLength);
		memcpy(m_pchData, pOldData->data(), pOldData->nDataLength*sizeof(gt_wchar_t));
		assert(m_pchData[GetData()->nDataLength] == '\0');
		CWString::Release(pOldData);
	}
	assert(GetData() != NULL);
}

LPWSTR CWString::LockBuffer()
{
	LPWSTR lpsz = GetBuffer(0);
	GetData()->nRefs = -1;
	return lpsz;
}

void CWString::UnlockBuffer()
{
	assert(GetData()->nRefs == -1);
	if (GetData() != afxDataNil)
		GetData()->nRefs = 1;
}

///////////////////////////////////////////////////////////////////////////////
// Commonly used routines (rarely used routines in STREX.CPP)

int CWString::Find(gt_wchar_t ch) const
{
	// find first single character
	LPWSTR lpsz = wcschr(m_pchData, ch);

	// return -1 if not found and index otherwise
	return (lpsz == NULL) ? -1 : (int)(lpsz - m_pchData);
}

int CWString::FindOneOf(LPCWSTR lpszCharSet) const
{
	LPWSTR lpsz = wcspbrk(m_pchData, lpszCharSet);
	return (lpsz == NULL) ? -1 : (int)(lpsz - m_pchData);
}

int CWString::ReverseFind(gt_wchar_t ch) const
{
	// find last single character
	LPWSTR lpsz = wcsrchr(m_pchData, ch);

	// return -1 if not found, distance from beginning otherwise
	return (lpsz == NULL) ? -1 : (int)(lpsz - m_pchData);
}

// find a sub-string (like strstr)
int CWString::Find(LPCWSTR lpszSub) const
{
	// find first matching substring
	LPWSTR lpsz = wcsstr(m_pchData, lpszSub);

	// return -1 for not found, distance from beginning otherwise
	return (lpsz == NULL) ? -1 : (int)(lpsz - m_pchData);
}


void CWString::MakeUpper()
{
	CopyBeforeWrite();
	w_strupr(m_pchData);
}

int CWString::FindAt(LPCWSTR lpszSub, int nIndex) const
{
	assert(nIndex >= 0);
	assert(nIndex < GetData()->nDataLength);

	if(nIndex < 0 || nIndex >= GetData()->nDataLength)
		return -1;

	// find first matching substring since nIndex
	LPWSTR lpsz = wcsstr(m_pchData+nIndex, lpszSub);

	// return -1 for not found, distance from beginning otherwise
	return (lpsz == NULL) ? -1 : (int)(lpsz - m_pchData);
}


void CWString::MakeLower()
{
	CopyBeforeWrite();
	w_strlwr(m_pchData);
}

#ifdef WIN32
void CWString::MakeReverse()
{
	CopyBeforeWrite();
	wcsrev(m_pchData);
}
#endif //WIN32

void CWString::SetAt(int nIndex, gt_wchar_t ch)
{
	assert(nIndex >= 0);
	assert(nIndex < GetData()->nDataLength);

	CopyBeforeWrite();
	m_pchData[nIndex] = ch;
}


void CWString::TrimRight()
{
	CopyBeforeWrite();

	// find beginning of trailing spaces by starting at beginning (DBCS aware)
	LPWSTR lpsz = m_pchData;
	LPWSTR lpszLast = NULL;

	while (*lpsz != '\0')
	{
		if (w_isspace(*lpsz))
		{
			if (lpszLast == NULL)
				lpszLast = lpsz;
		}
		else
			lpszLast = NULL;
		lpsz = lpsz++;
	}

	if (lpszLast != NULL)
	{
		// truncate at trailing space start
		*lpszLast = '\0';
		GetData()->nDataLength = lpszLast - m_pchData;
	}
}

void CWString::TrimLeft()
{
	CopyBeforeWrite();

	// find first non-space character
	LPCWSTR lpsz = m_pchData;
	while ( w_isspace(*lpsz))
		lpsz++;

	// fix up data and length
	int nDataLength = GetData()->nDataLength - (lpsz - m_pchData);
	memmove(m_pchData, lpsz, (nDataLength+1) * sizeof(gt_wchar_t));
	GetData()->nDataLength = nDataLength;
}

#define FORCE_ANSI      0x10000
#define FORCE_UNICODE   0x20000
#define FORCE_INT64     0x40000


void CWString::FormatV(LPCWSTR lpszFormat, va_list argList)
{
	va_list argListSave = argList;

	// make a guess at the maximum length of the resulting string
	int nMaxLen = 0;
	for (LPCWSTR lpsz = lpszFormat; *lpsz != '\0'; lpsz = lpsz + 1)
	{
		// handle '%' character, but watch out for '%%'
		if (*lpsz != '%' || *(lpsz = lpsz + 1) == '%')
		{
			nMaxLen ++;
			continue;
		}
		int nItemLen = 0;

		// handle '%' character with format
		int nWidth = 0;
		for (; *lpsz != '\0';  lpsz = lpsz + 1)
		{
			// check for valid flags
			if (*lpsz == '#')
				nMaxLen += 2;   // for '0x'
			else if (*lpsz == '*')
				nWidth = va_arg(argList, int);
			else if (*lpsz == '-' || *lpsz == '+' || *lpsz == '0' ||
				*lpsz == ' ')
				;
			else // hit non-flag character
				break;
		}
		// get width and skip it
		if (nWidth == 0)
		{
			// width indicated by
			nWidth = _wtoi(lpsz);
			for (; *lpsz != '\0' && isdigit(*lpsz); lpsz = lpsz + 1)
				;
		}
		assert(nWidth >= 0);
		int nPrecision = 0;
		if (*lpsz == '.')
		{
			// skip past '.' separator (width.precision)
			lpsz = lpsz + 1;

			// get precision and skip it
			if (*lpsz == '*')
			{
				nPrecision = va_arg(argList, int);
				lpsz = lpsz + 1;
			}
			else
			{
				nPrecision = _wtoi(lpsz);
				for (; *lpsz != '\0' && isdigit(*lpsz); lpsz = lpsz + 1)
					;
			}
			assert(nPrecision >= 0);
		}

		// should be on type modifier or specifier
		int nModifier = 0;
		if (wcsncmp(lpsz, L"I64", 3) == 0)
		{
			lpsz += 3;
			nModifier = FORCE_INT64;
#if !defined(_X86_) && !defined(_ALPHA_)
			// __int64 is only available on X86 and ALPHA platforms
			assert(FALSE);
#endif
		}
		else
		{
			switch (*lpsz)
			{
			// modifiers that affect size
			case 'h':
				nModifier = FORCE_ANSI;
				lpsz = lpsz + 1;
				break;
			case 'l':
				nModifier = FORCE_UNICODE;
				lpsz = lpsz + 1;
				break;

			// modifiers that do not affect size
			case 'F':
			case 'N':
			case 'L':
				lpsz = lpsz + 1;
				break;
			}
		}

		// now should be on specifier
		switch (*lpsz | nModifier)
		{
		// single characters
		case 'c':
		case 'C':
			nItemLen = 2;
			va_arg(argList, gt_wchar_t);
			break;
		case 'c'|FORCE_ANSI:
		case 'C'|FORCE_ANSI:
			nItemLen = 2;
			va_arg(argList, char);
			break;
		case 'c'|FORCE_UNICODE:
		case 'C'|FORCE_UNICODE:
			nItemLen = 2;
			va_arg(argList, gt_wchar_t);
			break;

		// strings
		case 's':
			{
				LPCWSTR pstrNextArg = va_arg(argList, LPCWSTR);
				if (pstrNextArg == NULL)
				   nItemLen = 6;  // "(null)"
				else
				{
#ifdef WIN32
				   nItemLen = lstrlenW(pstrNextArg);
#else //UNIX
				   nItemLen = wcslen(pstrNextArg);
#endif //WIN32
				   nItemLen = max(1, nItemLen);
				}
			}
			break;

		case 'S':
			{
#ifndef _UNICODE
				LPWSTR pstrNextArg = va_arg(argList, LPWSTR);
				if (pstrNextArg == NULL)
				   nItemLen = 6;  // "(null)"
				else
				{
				   nItemLen = wcslen(pstrNextArg);
				   nItemLen = max(1, nItemLen);
				}
#else
				LPCSTR pstrNextArg = va_arg(argList, LPCSTR);
				if (pstrNextArg == NULL)
				   nItemLen = 6; // "(null)"
				else
				{
				   nItemLen = lstrlenA(pstrNextArg);
				   nItemLen = max(1, nItemLen);
				}
#endif
			}
			break;

		case 's'|FORCE_ANSI:
		case 'S'|FORCE_ANSI:
			{
				LPCSTR pstrNextArg = va_arg(argList, LPCSTR);
				if (pstrNextArg == NULL)
				   nItemLen = 6; // "(null)"
				else
				{
#ifdef WIN32
				   nItemLen = lstrlenA(pstrNextArg);
#else //UNIX
				   nItemLen = strlen(pstrNextArg);
#endif //WIN32
				   nItemLen = max(1, nItemLen);
				}
			}
			break;

		case 's'|FORCE_UNICODE:
		case 'S'|FORCE_UNICODE:
			{
				LPWSTR pstrNextArg = va_arg(argList, LPWSTR);
				if (pstrNextArg == NULL)
				   nItemLen = 6; // "(null)"
				else
				{
				   nItemLen = wcslen(pstrNextArg);
				   nItemLen = max(1, nItemLen);
				}
			}
			break;
		}

		// adjust nItemLen for strings
		if (nItemLen != 0)
		{
			if (nPrecision != 0)
				nItemLen = min(nItemLen, nPrecision);
			nItemLen = max(nItemLen, nWidth);
		}
		else
		{
			switch (*lpsz)
			{
			// integers
			case 'd':
			case 'i':
			case 'u':
			case 'x':
			case 'X':
			case 'o':
#ifdef WIN32
				if (nModifier & FORCE_INT64)
					va_arg(argList, __int64);
				else
#endif WIN32
					va_arg(argList, int);
				nItemLen = 32;
				nItemLen = max(nItemLen, nWidth+nPrecision);
				break;

			case 'e':
			case 'g':
			case 'G':
				va_arg(argList, double);
				nItemLen = 128;
				nItemLen = max(nItemLen, nWidth+nPrecision);
				break;

			case 'f':
				{
					double f;
					LPWSTR pszTemp;

					// 312 == strlen("-1+(309 zeroes).")
					// 309 zeroes == max precision of a double
					// 6 == adjustment in case precision is not specified,
					//   which means that the precision defaults to 6
#ifdef __TV_UNIX__
					pszTemp = (LPWSTR)malloc(max(nWidth, 312+nPrecision+6));
#else //WIN32
					pszTemp = (LPWSTR)_alloca(max(nWidth,312+nPrecision+6));
#endif //UNIX

					f = va_arg(argList, double);
					SWPRINTF( pszTemp, L"%*.*f", nWidth, nPrecision+6, f );
				nItemLen = wcslen(pszTemp);
#ifdef __TV_UNIX__
					free(pszTemp);
#endif //UNIX
				}
				break;

			case 'p':
				va_arg(argList, void*);
				nItemLen = 32;
				nItemLen = max(nItemLen, nWidth+nPrecision);
				break;

			// no output
			case 'n':
				va_arg(argList, int*);
				break;

			default:
					assert(FALSE);  // unknown formatting option
			}
		}

		// adjust nMaxLen for output nItemLen
		nMaxLen += nItemLen;
	}

	GetBuffer(nMaxLen);
#ifdef WIN32
	vswprintf(m_pchData, lpszFormat, argListSave);
#else //UNIX
	vswprintf(m_pchData,nMaxLen,lpszFormat, argListSave);
#endif //WIN32
	ReleaseBuffer();

	va_end(argListSave);
}

// formatting (using wsprintf style formatting)
void CWString::Format(LPCWSTR lpszFormat, ...)
{


	va_list argList;
	va_start(argList, lpszFormat);
	FormatV(lpszFormat, argList);
	va_end(argList);
}

///////////////////////////////////////////////////////////////////////////////
// OLE BSTR support
#ifdef WIN32
BSTR CWString::AllocSysString() const
{
	BSTR bstr = ::SysAllocStringLen(m_pchData, GetData()->nDataLength);
	return bstr;
}
#endif //WIN32

bool  operator==(const CWString& s1, const CWString& s2)
	{ return s1.Compare(s2) == 0; }
bool  operator==(const CWString& s1, LPCWSTR s2)
	{ return s1.Compare(s2) == 0; }
bool  operator==(LPCWSTR s1, const CWString& s2)
	{ return s2.Compare(s1) == 0; }
bool  operator!=(const CWString& s1, const CWString& s2)
	{ return s1.Compare(s2) != 0; }
bool  operator!=(const CWString& s1, LPCWSTR s2)
	{ return s1.Compare(s2) != 0; }
bool  operator!=(LPCWSTR s1, const CWString& s2)
	{ return s2.Compare(s1) != 0; }
bool  operator<(const CWString& s1, const CWString& s2)
	{ return s1.Compare(s2) < 0; }
bool  operator<(const CWString& s1, LPCWSTR s2)
	{ return s1.Compare(s2) < 0; }
bool  operator<(LPCWSTR s1, const CWString& s2)
	{ return s2.Compare(s1) > 0; }
bool  operator>(const CWString& s1, const CWString& s2)
	{ return s1.Compare(s2) > 0; }
bool  operator>(const CWString& s1, LPCWSTR s2)
	{ return s1.Compare(s2) > 0; }
bool  operator>(LPCWSTR s1, const CWString& s2)
	{ return s2.Compare(s1) < 0; }
bool  operator<=(const CWString& s1, const CWString& s2)
	{ return s1.Compare(s2) <= 0; }
bool  operator<=(const CWString& s1, LPCWSTR s2)
	{ return s1.Compare(s2) <= 0; }
bool  operator<=(LPCWSTR s1, const CWString& s2)
	{ return s2.Compare(s1) >= 0; }
bool  operator>=(const CWString& s1, const CWString& s2)
	{ return s1.Compare(s2) >= 0; }
bool  operator>=(const CWString& s1, LPCWSTR s2)
	{ return s1.Compare(s2) >= 0; }
bool  operator>=(LPCWSTR s1, const CWString& s2)
	{ return s2.Compare(s1) <= 0; }



