/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/*///////////////////////////////////////////////////////////////////////////////
 * dbcode.cp -- double byte encoding implementation.
 * Copyright (C) GalTech Soft Ltd. 1998-2000. All rights reserved.
 *
///////////////////////////////////////////////////////////////////////////////
*/
#ifdef WIN32
#include "windows.h"
#endif
#include "gt_portable.h"
#include "dbcode.h"
#include "gt_unidef.h"
#include "uniserv.h"
#include "gt_uniservapis.h"

// DBCS escape sequences
static char escape_JIS1[] = "(B";
static char escape_JIS2[] = "$B";
static char escape_KS1[] = "(B";
static char escape_KS2[] = "$(C";
static char escape_GB1[] = "(?";
static char escape_GB2[] = "$(A";

int SkipESCSeq(LPSTR lpJis, unsigned char cTemp, BOOL *intwobyte);

int UL_EscapeSeqstrlen(LPSTR lpJis, UINT CodePage, int iLength, int *pnBytesUsed)
{
	if(!IS_ESCAPE_SEQUENCE_CODEPAGE(CodePage))
		return 0;
	unsigned char HiJis, LoJis, cTemp;
	int iChars = 0, iJis =0;
	BOOL intwobyte = FALSE;
	BOOL bNullTermination = iLength == -1 ? TRUE : FALSE;

	for (iJis = 0; iJis < iLength || 
			(bNullTermination && lpJis[iJis] != '\0'); iJis++)
	{
		HiJis = lpJis[iJis];
		if(HiJis == ESCJ)
		{
			int iJisSave = iJis;

			iJis++;
			if ((bNullTermination && lpJis[iJis] == '\0') ||
				(!bNullTermination && iJis >= iLength))
			{   // truncated sequence
				iJis = iJisSave;
				break;
			}
			cTemp = lpJis[iJis];
			iJis += SkipESCSeq((char*)lpJis, cTemp, &intwobyte);
			if ((bNullTermination && lpJis[iJis] == '\0') ||
				(!bNullTermination && iJis >= iLength))
			{   // truncated sequence
				iJis = iJisSave;
				break;
			}
		}
		else if (HiJis == NLJ || HiJis == CRJ)
		{
			if(intwobyte)
				intwobyte = FALSE;
			iChars++;
		}
		else
		{
			if(intwobyte)
			{
				if ((bNullTermination && lpJis[iJis+1]=='\0') ||
					(!bNullTermination && iJis+1>=iLength))
				{   // truncated char
					break;
				}
				iJis++;
				LoJis = lpJis[iJis];
				iChars++;
			}
			else
			{
				iChars++;
			}
		}
	}
	if(bNullTermination)
		iChars++;
	if (pnBytesUsed)
		*pnBytesUsed = iJis;
	return iChars;

}
int UL_DBCSstrlen(LPSTR str, UINT CodePage, int iLength, int *pnBytesUsed)
{
    int len = 0;
    int i;
	BOOL bNullTermination = iLength == -1 ? TRUE : FALSE;

    for (i = 0; i < iLength || 
			(bNullTermination && str[i] != '\0'); i++)
    {
        if (UL_IsLeadByte(str[i], CodePage))
        /* Double Byte character */
        {
			if (! ((bNullTermination && str[i+1]=='\0') || 
					(!bNullTermination && (i+1 >= iLength))))
			{  // double byte char not truncated
				len++;
				i++;        // skip over lower byte of character
			}
			else
			{
				break;
			}
        }
        else   /* Single Byte character */
            len++;
    }
	if(bNullTermination)
		len++;
	if (pnBytesUsed)
		*pnBytesUsed = i;
    return len;
}

/* Determine if two character string is a SJIS code or not */
BOOL UL_isSjis(unsigned char *str)
{
    unsigned char	uByte, lByte;

    uByte = str[0];
    lByte = str[1];

    if (((uByte >= 0x81 && uByte <= 0x9F) ||
         (uByte >= 0xE0 && uByte <= 0xFC)) &&
        ((lByte >= 0x40 && lByte <= 0x7E) ||
         (lByte >= 0x80 && lByte <= 0xFC)))
		return(TRUE);
    else
		return(FALSE);

}


/* Determine if two character string is an EUC code or not */
BOOL UL_isEUC(unsigned char *str)
{
    unsigned char	uByte, lByte;

    uByte = str[0];
    lByte = str[1];

	// in EUC the lead byte and the second byte have the same range so we use IsLeadByte() 
	// for both
    if (UL_IsLeadByte(uByte, CP_EUCJP) &&
        UL_IsLeadByte(lByte, CP_EUCJP))
		return(TRUE);
    else
		return(FALSE);

}

/////////////////////////////////////////////////////////////////
// THE FOLLOWING CODE IS TAKEN FROM KEN LUNDEN BOOK PAGE 163 - 164
//       AND REWRITEN BY ALEX
/////////////////////////////////////////////////////////////////
void UL_Jis2SJisChar(unsigned char HiJis,unsigned char LoJis,
	unsigned char *HiSJis, unsigned char *LoSJis)
{
	unsigned char c1 = HiJis;
	unsigned char c2 = LoJis;
	int rowOffset = c1 < 95 ? 112 : 176;
	int cellOffset = c1 % 2 ? (c2 > 95 ? 32 :31) : 126;
	*HiSJis = ((c1 + 1) >> 1) + rowOffset;
	*LoSJis = LoJis + cellOffset;
}

void UL_SJis2JisChar(unsigned char HiSJis,unsigned char LoSJis,
	unsigned char *HiJis, unsigned char *LoJis)
{
	unsigned char c1 = HiSJis;
	unsigned char c2 = LoSJis;
	int adjust = c2 < 159;
	int rowOffset = c1 < 160 ? 112 : 176;
	int cellOffset = adjust ? (c2 > 127 ? 32 :31) : 126;
	*HiJis = ((c1 - rowOffset) << 1) - adjust;
	*LoJis = LoSJis - cellOffset;
}

/////////////////////////////////////////////////////////////////
// 	 UL_SJis2Jis recieves string in Shift-Jis and buffer for string
//   converted to Jis. The return value is the number of bytes copied 
//   to the buffer. iCharMode is return value - the last mode of Jis
//   string in buffer 
/////////////////////////////////////////////////////////////////////
int UL_SJis2Jis(LPSTR lpSJis, int ccSJis,
	LPSTR lpJis, int ccJis, int *iCharMode)
{

	unsigned char HiSJis, LoSJis;
	unsigned char HiJis, LoJis;
	int iSJis = 0, iJis =0;
	int intwobyte = FALSE;
	BOOL bNullTermination = ccSJis == -1 ? TRUE : FALSE;

	while((iJis < ccJis - 1) && (iSJis < ccSJis || 
		(bNullTermination && lpSJis[iSJis] != 0 )))
	{
		HiSJis = lpSJis[iSJis];
		if(SJIS1(HiSJis))
		{
			iSJis++;
			LoSJis = lpSJis[iSJis];
			if(SJIS2(LoSJis))
			{
				UL_SJis2JisChar( HiSJis, LoSJis, &HiJis, &LoJis);
				if(!intwobyte)
				{
					intwobyte = TRUE;
					lpJis[iJis++] = ESCJ;
					strcpy(&lpJis[iJis], escape_JIS2);
					iJis += strlen(escape_JIS2);
				}
			}
			lpJis[iJis++] = HiJis; 
			lpJis[iJis++] = LoJis; 
		}
		else
		{
			if(intwobyte)
			{
				intwobyte = FALSE;
				lpJis[iJis++] = ESCJ;
				strcpy(&lpJis[iJis], escape_JIS1);
				iJis += strlen(escape_JIS1);
			}
			lpJis[iJis++] = HiSJis; 
		}
		iSJis++;
	}

	if(bNullTermination && lpSJis[iSJis] == 0 )
	{
		lpJis[iJis] = '\0';
		iJis++;
	}
	*iCharMode = intwobyte ? 2 : 1;
	return iJis;
}
/////////////////////////////////////////////////////////////////
// 	 SkipESCSeq recieves string in Jis current character in the 
//   string and character mode. The return value is the number of 
//   bytes in escape sequence and current character mode
//   string 
/////////////////////////////////////////////////////////////////////
int SkipESCSeq(LPSTR lpJis, unsigned char cTemp, BOOL *intwobyte)
{
	int iResult = 0;
	if(cTemp == '$' || cTemp == '(')
		iResult = 1;
	if(cTemp == 'K' || cTemp == '$')
	{
		*intwobyte = TRUE;
	}
	else
	{
		*intwobyte = FALSE;
	}
	return iResult;
}
/////////////////////////////////////////////////////////////////
// 	 UL_Jis2SJis recieves string in Jis and buffer for string
//   converted to Shift-Jis. The return value is the number of bytes 
//   copied  to the buffer. iCharMode is the value of star Jis
//   string 
/////////////////////////////////////////////////////////////////////
int UL_Jis2SJis(LPCSTR lpJis, int ccJis,
	LPSTR lpSJis, int ccSJis, int *iCharMode)
{

	unsigned char HiSJis, LoSJis;
	unsigned char HiJis, LoJis, cTemp;
	int iSJis = 0, iJis =0;
	int intwobyte = (*iCharMode == 2 ) ? TRUE : FALSE;
	BOOL bNullTermination = ccJis == -1 ? TRUE : FALSE;

	for(;(iSJis < ccSJis - 1) && 
		(iJis < ccJis || 
		(bNullTermination && lpJis[iJis] != 0 )); iJis++)
	{
		HiJis = lpJis[iJis];
		if(HiJis == ESCJ)
		{
			iJis++;
			if ((bNullTermination && lpJis[iJis]=='\0') ||
				(!bNullTermination && iJis >= ccJis))
				break;
			cTemp = lpJis[iJis];
			iJis += SkipESCSeq((char*)lpJis, cTemp, &intwobyte);
			if ((bNullTermination && cTemp=='\0') ||
				(!bNullTermination && iJis >= ccJis))
				break;
		}
		else if (HiJis == NLJ || HiJis == CRJ)
		{
			if(intwobyte)
				intwobyte = FALSE;
			lpSJis[iSJis] = HiJis;
			iSJis++;
		}
		else
		{
			if(intwobyte)
			{
				iJis++;
				if ((bNullTermination && lpJis[iJis]=='\0') ||
					(!bNullTermination && iJis >= ccJis))
					break;
				LoJis = lpJis[iJis];
				UL_Jis2SJisChar(HiJis, LoJis, &HiSJis, &LoSJis);
				lpSJis[iSJis] = HiSJis;
				iSJis++;
				lpSJis[iSJis] = LoSJis;
				iSJis++;
			}
			else
			{
				lpSJis[iSJis] = HiJis;
				iSJis++;
			}
		}
		
	}
	if(bNullTermination && lpJis[iJis] == 0 )
	{
		lpSJis[iSJis] = '\0';
		iSJis++;
	}
	*iCharMode = intwobyte ? 2 : 1;
	return iSJis;
}


/////////////////////////////////////////////////////////////////
// 	 UL_Euc2SJis recieves string in EUC and buffer for string
//   converted to Shift-Jis. The return value is the number of bytes 
//   copied  to the buffer. 
/////////////////////////////////////////////////////////////////////
int UL_Euc2SJis(LPCSTR lpEuc, int ccEuc,
	LPSTR lpSJis, int ccSJis)
{
	unsigned char HiSJis, LoSJis;
	unsigned char HiEuc, LoEuc;
	int iSJis = 0, iEuc =0;
	BOOL bNullTermination = ccEuc == -1 ? TRUE : FALSE;

	while((iSJis < ccSJis - 1) && (iEuc < ccEuc || 
		(bNullTermination && lpEuc[iEuc] != 0 )))
	{
		HiEuc = lpEuc[iEuc];
		if(ISEUC(HiEuc))
		{
			iEuc++;
			if ((bNullTermination && lpEuc[iEuc]=='\0') || 
							(!bNullTermination && iEuc >= ccEuc))
				break;
			LoEuc = lpEuc[iEuc];
			if(ISEUC(LoEuc))
			{
				HiEuc -= 128;
				LoEuc -= 128;
				UL_Jis2SJisChar(HiEuc, LoEuc, &HiSJis, &LoSJis);
			}
			else
			{
				HiSJis = HiEuc;
				LoSJis = LoEuc;
			}
			lpSJis[iSJis] = HiSJis;
			iSJis ++;
			lpSJis[iSJis] = LoSJis;
			iSJis ++;
			iEuc++;
		}
		else if (HiEuc == 0x8E) // class 2 characters
		{
			iEuc++;
			if ((bNullTermination && lpEuc[iEuc]=='\0') || 
							(!bNullTermination && iEuc >= ccEuc))
				break;
			LoEuc = lpEuc[iEuc];
			lpSJis[iSJis] = LoEuc;
			iSJis ++;
			iEuc++;
		}
		else if (HiEuc == 0x8F)
		{
			// kludge - don't know how to convert class 3 characters
			iEuc++;
			if ((bNullTermination && lpEuc[iEuc]=='\0') || 
							(!bNullTermination && iEuc >= ccEuc))
				break;
			iEuc++;
			if ((bNullTermination && lpEuc[iEuc]=='\0') || 
							(!bNullTermination && iEuc >= ccEuc))
				break;
			iEuc++;
		}
		else
		{
			lpSJis[iSJis] = HiEuc;
			iSJis ++;
			iEuc++;
		}
	}
	if(bNullTermination && lpEuc[iEuc] == 0 )
	{
		lpSJis[iSJis] = '\0';
		iSJis++;
	}
	return iSJis;
}

/////////////////////////////////////////////////////////////////
// 	 UL_SJis2Euc recieves string in Shift-Jis and buffer for string
//   converted to EUC. The return value is the number of bytes 
//   copied  to the buffer. 
/////////////////////////////////////////////////////////////////////
int UL_SJis2Euc(LPSTR lpSJis, int ccSJis, 
	LPSTR lpEuc, int ccEuc)
{
	unsigned char HiSJis, LoSJis;
	unsigned char HiEuc, LoEuc;
	int iSJis = 0, iEuc =0;
	BOOL bNullTermination = ccSJis == -1 ? TRUE : FALSE;

	while((iEuc < ccEuc - 1) && (iSJis < ccSJis || 
		(bNullTermination && lpSJis[iSJis] != 0 )))
	{
		HiSJis = lpEuc[iSJis];
		if(SJIS1(HiSJis))
		{
			iSJis++;
			LoSJis = lpSJis[iSJis];
			if(SJIS2(LoSJis))
			{
				UL_SJis2JisChar(HiSJis, LoSJis, &HiEuc, &LoEuc);
				HiEuc += 128;
				LoEuc += 128;

			}
			else
			{
				HiEuc = HiSJis;
				LoEuc = LoSJis;
			}
			lpEuc[iEuc] = HiEuc;
			iEuc ++;
			lpEuc[iEuc] = LoEuc;
			iEuc ++;
		}
		else
		{
			lpEuc[iEuc] = HiSJis;
			iEuc ++;
		}
		iSJis++;
	}

	if(bNullTermination && lpSJis[iSJis] == 0 )
	{
		lpEuc[iEuc] = '\0';
		iEuc++;
	}
	return iEuc;
}


void UL_sjis2jis(unsigned char *sjis, unsigned char *jis)
{
    unsigned char hByte, lByte;
    unsigned char newlByte, newhByte;
    int i;

    for (i = 0; sjis[i] != '\0'; i += 2)
    {
        hByte = sjis[i];
        lByte = sjis[i+1];

/* Convert double byte character from sjis to jis */

		newhByte = hByte;
		if (hByte > 0x9F)
			newhByte = (unsigned char)(newhByte - (0xE0 - 0xA0));
		newhByte = (unsigned char)((newhByte - 0x71) * 2 + 1);
		newlByte = (unsigned char)(lByte - 0x1F);
		if (lByte > 0x7F)
			newlByte--;
		if (newlByte > 0x7E)
		{
			newlByte = (unsigned char)(newlByte - 0x5E);
			newhByte++;
		}

		jis[i] = newhByte;
		jis[i+1] = newlByte;
    }
    jis[i] = '\0';
    return;
}




// Determine if uByte is the start of a double-byte character.
BOOL UL_IsLeadByte(BYTE uByte, UINT CodePage)
{
	// Chaim 07.01.02 - For Jap version, we want to check jap codepage first.
	if (CodePage==CP_MS_JAPANESE)
	{
	    // Don't use IsDBCSLeadByte, because it is dependent on the language
	    // specified in the Windows setup.
	    if (   ((129 <= uByte) && (uByte <= 159))
	      || ((224 <= uByte) && (uByte <= 239)))
	        return TRUE;
	    else
	        return FALSE;
	}
	else if (CodePage == CP_EUCJP)
	{
		if (uByte >= 0xA1 && uByte <= 0xFE)
			return TRUE;
		else
			return FALSE;
	}
	else if (CodePage == CP_MS_CHINESE_TRADITIONAL)
	{
		if (uByte >= 0xA1 && uByte <= 0xFE)
			return TRUE;
		else
			return FALSE;
	}
	else if (CodePage == CP_MS_KOREAN)
	{
		if (uByte >= 0xA1 && uByte <= 0xFE)
			return TRUE;
		else
			return FALSE;
	}
	else if (CodePage == CP_MS_CHINESE_SIMPLIFIED)
	{
		if (uByte >= 0xA1 && uByte <= 0xFE)
			return TRUE;
		else
			return FALSE;
	}
/*	else if (CodePage==CP_MS_JAPANESE)
	{
	    // Don't use IsDBCSLeadByte, because it is dependent on the language
	    // specified in the Windows setup.
	    if (   ((129 <= uByte) && (uByte <= 159))
	      || ((224 <= uByte) && (uByte <= 239)))
	        return TRUE;
	    else
	        return FALSE;
	}
*/
	else  // unknown encoding
		return FALSE;
}


UINT UL_JapaneseAutoDetect(LPCSTR str, int ccStr, BOOL *pbDetected)
{
	UINT CodePage = CP_JIS;
	BOOL bSjis = FALSE;
	BOOL bEUC = FALSE;
	int i;
	BOOL bNullTermination = ccStr == -1 ? TRUE : FALSE;
	*pbDetected = FALSE;

	for (i = 0; i < ccStr || 
			(bNullTermination && str[i] != '\0'); i++)
	{
		if ( (unsigned char)str[i]==0x1B)
		{
			*pbDetected = TRUE;
			break;  // JIS is the only Japanese codepage with escape sequense
		}
		if ((unsigned char)str[i] > 0x80)
		{
			if (UL_isSjis((unsigned char* )( str+i)))
				bSjis = TRUE;
			if (UL_isEUC((unsigned char *) (str+i)))
				bEUC = TRUE;
			if (bEUC != bSjis)
			{
				if (bSjis)
				{
					CodePage = CP_MS_JAPANESE;
					*pbDetected = TRUE;
				}
				else
				{
					CodePage = CP_EUCJP;
					*pbDetected = TRUE;
				}
				break;
			}
			else if (bEUC)
				CodePage = CP_EUCJP;  // can't be JIS, put a resonable default
			i++;  // skip the second byte
		}
	}
	return CodePage;
}

