/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "gt_strwrapper.h"
#include <stdio.h>

#ifndef __TV_UNIX__
#include "mbstring.h"
#endif

StrTokenizer::StrTokenizer(const char*  str)
{
	buffer = new char[strlen(str) + 1];
	strcpy(buffer, str);
	pointer = buffer;
}

StrTokenizer::~StrTokenizer()
{
	delete [] buffer;
}

#ifndef __TV_UNIX__
char* StrTokenizer::operator()(const char* delimiter)
{
	if (pointer)
	{
		char *ret = pointer;
		pointer = (char*)_mbsstr((unsigned  char*)pointer, (const unsigned char*)delimiter);
		if (pointer)
		{
			*pointer = '\0';
			pointer += strlen(delimiter);
		}
		return ret;
	}
	else
		return "";
}

#endif

StrWrapper EscapeSlashes(const char *sIn)
{
	if (!sIn)
        {
		StrWrapper t(NULL);
		return t;
        }

	char *sOut = new char[strlen(sIn)*2 + 1];
	char *sBegin = sOut;

	for ( ; *sIn; sIn++)
	{
		*sOut++ = *sIn;
		if (*sIn == '\\')
			*sOut++ = '\\';
	}
	*sOut = '\0';
	StrWrapper t(sBegin);
	return t;
}

//On CJK Windows wide numbers may be applied to controls where only narrow numbers
//are allowed , we are getting multibyte string and convert it to UCS2, after that shifting all wide numbers to 
//ASCI numbers range and returning ANSI string, length default value is -1, it means that 
//str is closed by 0
StrWrapper ConvertWideNumbers(const char *str)
{
	//Calling with buffer NULL, in order to know real size
	int countW = ANSIToUCS2(GetACP(), str, -1, NULL, 0);
	gt_wchar_t *charsW = new gt_wchar_t[countW];

	//Real conversion
	ANSIToUCS2(GetACP(), str, -1, charsW, countW);

	int i = 0;
	while(i < countW)
	{

		if(charsW[i] > 65295 && charsW[i] < 65306)
			charsW[i] -= 65248;
		else
			break;
		
		i++;
	}

	int countA = UCS2ToANSI(CP_DEFAULT, charsW, -1,  NULL,  0);
	char *charA = new char[countA];

	UCS2ToANSI(CP_DEFAULT, charsW,	-1, charA, countA);
	delete [] charsW;

	StrWrapper t(charA);
	return t;
}

StrWrapper ToUTF8(const char *strANSI, int nCharset)
{
	int nLen = ANSIToUTF8(nCharset, strANSI, -1, NULL, 0);
	char *szStr = new char[nLen];
	ANSIToUTF8(nCharset, strANSI, -1, szStr, nLen);

	StrWrapper t(szStr);
	return t;
}

StrWrapper ToANSI(const char *strUTF, int nCharset)
{
	int nLen = UTF8ToANSI(nCharset, strUTF, -1, NULL, 0);
	char *szStr = new char[nLen];
	UTF8ToANSI(nCharset, strUTF, -1, szStr, nLen);

	StrWrapper t(szStr);
	return t;
}

const size_t RW_NPOS            = ~(size_t)0;


#ifndef __TV_UNIX__
int StrFirst(const char* str, char chr)
{
	const char *strFirst = (const char*)_mbschr((unsigned const char*)str, (unsigned char)chr);
	if (strFirst)
		return (strFirst - str);
	else
		return RW_NPOS;
}

int StrLast(const char* str, char chr)
{
	const char *strLast = (const char*)_mbsrchr((unsigned const char*)str, (unsigned char)chr);
	if (strLast)
		return (strLast - str);
	else
		return RW_NPOS;
}

bool StrContains(const char* str, const char* substr)
{
	return (_mbsstr((unsigned const char*)str, (unsigned const char*)substr) != NULL);
}
#endif
