/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/* Copyright  1993-1995, Radview Software LTD. All Rights Reserved. */
/***********************************************************************
 * Name : tvdebug
 * Purpose : header file for the debug module of testview.
 *
 * Description :
 *
 * Creation: Yahali, 6-Nov-94
 * Updates :  Who          When         What
 *
 ***********************************************************************/
#ifndef __TVDEBUG_H
#define __TVDEBUG_H

#include <stdio.h>
#include <stdarg.h>
#include <assert.h>

#if (defined_cplusplus) || (defined __cplusplus)
#include <RVCString.h>
#endif


#ifndef __MODULENAME__
#define __MODULENAME__ "ALL"
#endif
#ifdef CHECK
#undef CHECK
#endif
#ifdef PRECONDITION
#undef PRECONDITION
#endif

#ifndef __TV_UNIX__
#define debug(module,level,f) { \
	if (isOutput(module,level))   \
		debugOutput f;             \
}

typedef void (*debugOutputFunc)(const char*);

typedef unsigned short debugmask;

/* external function type declarations */

#ifdef __cplusplus
extern "C" {
#endif
int isOutput(const char* modulename, debugmask n);
void setDebugOutputFunction(debugOutputFunc dof);
void defOutputFunc(const char* outbuf);
int  debugOutput(const char* format,...);
const char* debugString(const char* format,...);
void debugNewLine();

void enterprocedure(const char *modulename, const char* procname);
void warn(char *s);
void panic(char *s);
#ifdef __cplusplus
}
#endif
#endif

#ifdef __TV_DEBUG__

#define CHECK assert
#define PRECONDITION assert

#ifndef __TV_UNIX__
#define debugPrint(level,f) debug(__MODULENAME__, level, f)

#ifdef __cplusplus
void tvdIndent();
void tvdUnIndent();
class EnterProc {
public:
	EnterProc(const char* n) : name(n)
	{
		debugPrint(1,(">> Entering '%s'\n",(const char*)name));
		tvdIndent();
	}
	~EnterProc()
	{
    tvdUnIndent();
    debugPrint(1,("<< Exiting '%s'\n",(const char*)name));
	}
private:
	RVCString name;
};
#define EnterProcedure(p) EnterProc enterProc(p)
#else
#define EnterProcedure(p) enterprocedure(__MODULENAME__,p)
#endif

#else // TV_UNIX
#define EnterProcedure(p) NULL
#define _CrtSetDbgFlag(p) NULL
#endif

#else // __TV_DEBUG__ is not defined...

#define debugPrint(level,f) NULL //DbgVMessage f
#define EnterProcedure(p) NULL //DbgMessage(p)
#define CHECK assert
#define PRECONDITION assert
#ifndef __TV_UNIX__
#define _CrtSetDbgFlag(p) NULL
#endif
//#define setDebugOutputFunction(p) NULL

#endif /* __TV_DEBUG__ */

#endif /* __TVDEBUG_H */

