/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "domitem.h"
#include <string.h>
#include <stdio.h>

CWLItemFrame::CWLItemFrame(const char *source, const char* url, const char *loc, LocationPosColl *p_pFramesOffsetLength /* = NULL */)
 :  frameSource(source) ,
    frameUrl(url) ,
    frameLocation(loc) ,
    pFramesOffsetLength(p_pFramesOffsetLength)
{
}
 
CWLItemFrame::~CWLItemFrame()
{
  if (pFramesOffsetLength != NULL)
  {
    pFramesOffsetLength->clearAndDestroy();
    delete pFramesOffsetLength;
    pFramesOffsetLength = NULL;
  }
}


CWLDomItem::~CWLDomItem()
{
  if( root )
    delete root;
  
  byLocation.clearAndDestroy();
  
}

void CWLDomItem::add(const char *url, const char *location, const char *data, bool isRoot /* = false */, LocationPosColl *coll /* = NULL */)
{
	RVCollectable*	result	=  NULL;
  CWLItemFrame *fr = new CWLItemFrame(data,url,location,coll);

  
  RVCollectableString *locKey = new RVCollectableString(location);
	result = byLocation.insertKeyAndValue((RVCollectable *)locKey,
																					(RVCollectable *)fr);

  if( isRoot ) //! if it's the root, mark it
    root = new RVCollectableString(url);

	return;
}


//! first look according to Location (if it is set) otherwise
//! look according to URL
CWLItemFrame *CWLDomItem::get(const char *url, const char *location)
{
  CWLItemFrame *fi = NULL;
  RVCollectableString *locKey = NULL;

  locKey = new RVCollectableString(location);

  fi = (CWLItemFrame *)byLocation.findValue((RVCollectable *)locKey);

  delete locKey;

  return fi;
}



const char *CWLDomItem::getRootUrl()
{
  if( root )
    return root->data();
  else
    return NULL;
}


void CWLDomItem::dump(CWLDomRepository& rep )
{
  LocationPosColl* pLocationPairColl = NULL;
  LocationPos* LPitem = NULL;
  
  
  long lVer = rep.getCurrnetVersion();

  CWLItemFrame *fi;

	//! first dump the number of frames in the item
	rep.dumpInt32(byLocation.entries());

  //! Dump the root if exist or NONE
  if( root )
    rep.dumpBuffer(root->data());
  else
    rep.dumpBuffer("NONE");

	//! iterate through all the frames and dump the key and then the frame
	RVHashDictionaryIterator  iter(byLocation);
	while( iter() != NULL )
	{
    fi = (CWLItemFrame *)(iter.value());

    //! dump the url
		rep.dumpBuffer(fi->getUrl());

    //! dump the location
		rep.dumpBuffer(fi->getLocation());

		//! dump the source
		rep.dumpBuffer(fi->getSource());


  //*******************************************************
  //    added frame/iframe offset and length in page src
  //*******************************************************

    if (rep. getCurrnetVersion() < EXR_SECOND_VERSION )
      continue;

    //! dump offset and length of frame/iframe src.
    pLocationPairColl = NULL;
    pLocationPairColl = fi->getFramesLocationInSrc();
    long lLocPairLength = (pLocationPairColl) ? pLocationPairColl->entries() : 0;
    
    //! dump coll length
    rep.dumpInt32(lLocPairLength);

    if (!pLocationPairColl || pLocationPairColl->entries() == 0)
      continue;

    LPitem = NULL;
    LocationPosCollIterator FSiter(*pLocationPairColl);
    while ( LPitem = (LocationPos*)FSiter() )
    {
      //! dump offset and length
      LPitem->dump(rep);
    }
	}
	return;

}

void CWLDomItem::unDump(CWLDomRepository& rep)
{
  CWLItemFrame *fi;
	RVCollectableString *urlKey;
	RVCollectableString *locKey;
	int numEntries;
  const char *temp,*url,*location,*source;

	//! first get the number of entries in the DomItem
	numEntries = rep.unDumpInt32();

  //! get the root
  temp = rep.unDumpBuffer();
  if( strcmp(temp,"NONE" ) )  //! if it's NOT NONE
  {
    if( root )
      delete root;
    root = new RVCollectableString(temp);
    free((void *)temp);
  }

  long lLocationCollLength        = 0;
  LocationPos* LPitem             = NULL;
  LocationPosColl *pLocationColl  = NULL;

	//! iterate on all entries
	for(int i = 0 ; i < numEntries ; ++i)
	{
		//! get the url
    url = rep.unDumpBuffer();
		urlKey = new RVCollectableString(url);

    //! get the location
    location = rep.unDumpBuffer();

		//! get the source
    source = rep.unDumpBuffer();

   
  //*******************************************************
  //    added frame/iframe offset and length in page src
  //*******************************************************
            
    if (rep.getCurrnetVersion() >= EXR_SECOND_VERSION )
    {
      //! get locationColl - collection of LocationPaire which are long offset and long length.
      lLocationCollLength = rep.unDumpInt32();
      pLocationColl = NULL;
      if (lLocationCollLength > 0)
      {
        pLocationColl = new LocationPosColl();
        for (;lLocationCollLength > 0 ; lLocationCollLength--)
        {
          LPitem = new LocationPos();
          LPitem->undump(rep);
          pLocationColl->append(LPitem);
          LPitem = NULL;
        }
      }
    }

    fi = new CWLItemFrame(source,url,location,pLocationColl);

		//! put them in
    locKey = new RVCollectableString(location);
    byLocation.insertKeyAndValue((RVCollectable *)locKey, (RVCollectable *)fi);

    free((void *)url);
    free((void *)location);
    free((void *)source);
	}

	return;
}


void CWLDomItem::printAll()
{
	RVCollectableString *frame;
	RVCollectableString *key;

  cout << "===========================================\n";

	//! first print the number of frames in the item
	cout << "#entries: " << byLocation.entries() << endl;

  if( root )
    cout << "Root item is: " << *root << endl;

	//! iterate through all the frames and print the key and then the frame
	RVHashDictionaryIterator  iter(byLocation);
	while( iter() != NULL )
	{
		//! print the key
		key = (RVCollectableString *)iter.key();
		cout << "Key: " << *key << endl;

		//! print the frame
		frame = (RVCollectableString *)iter.value();
		cout << "Frame: " << *frame << endl;
	}

	return;
}



void CWLDomRepository::dumpInt32(unsigned int i32)
{
	char c;
	for(int i = 0 ; i < 8 ; ++i)  //! loop on the 8 quadlets of the int32
	{
		c = ((i32>>(i*4)) & 0x000F) + '0'; //! get the quad and make it printable (by adding '0')
		dumpedStorage.append(c,1);  //! dump it
	}

  if( dumpedStorage.length() > STORAGE_LIMIT )
    flush();

	return;
}


void CWLDomRepository::dumpBuffer(const char *data)
{
	size_t len = strlen(data);
	//! dump the buffer length
	dumpInt32(len);

	//! dump the buffer
	dumpedStorage.append(data, len);

  if( dumpedStorage.length() > STORAGE_LIMIT )
    flush();

  return;
}


bool CWLDomRepository::testFill(int size)
{
  int len;
	len = dumpedStorage.length();
	while( int(len - storagePointer) < size )
	{
    if( ! fill() )
      return false;
		len = dumpedStorage.length();
	}
  return true;
}

void CWLDomRepository::fillStorage(const char *data)
{
  //! first shrink the storage
  dumpedStorage.remove(0,storagePointer);

  //! add the new data
  dumpedStorage.append(data);

  initUnDump();
}


unsigned int CWLDomRepository::unDumpInt32()
{
	unsigned int i32 = 0;
	unsigned char c;

  if( !testFill(8) )
    return -1;
	for(int i = 0 ; i < 8 ; ++i)  //! loop on the 8 quadlets of the int32
	{
		c = (char) dumpedStorage[(size_t)storagePointer++] - '0';
		i32 |= c<<(i*4);
	}

	return i32;
}

const char *CWLDomRepository::unDumpBuffer()
{
	//! get the buffer length
	int len;
  if( (len = unDumpInt32()) < 0 )  //! no data
    return NULL;

	if( !testFill(len))  //! make sure there are enoght bytes
    return NULL;

  //! get the buffer
	//! first allocate space
	char *buff = (char *)malloc(len+1);
	memcpy(buff,dumpedStorage.data()+storagePointer,len);
	buff[len] = '\0';
	storagePointer += len;

  return buff;
}



CWLExpectedDomRepository::CWLExpectedDomRepository(int p_nCurrVer/* = EXR_FIRST_VERSION */)
  : CWLDomRepository(p_nCurrVer)
{
  fp = NULL;
	m_bNewVersion = false;
}


int CWLExpectedDomRepository::add(CWLDomItem *item,int domId)
{
	RVCollectable *res;
	int i = itemList.entries()+1;

	if(domId == -1) //the default parameter
	{
	  RVCollectableInt rwCInt(i);
		while(itemList.findValue(&rwCInt)) //key already exists
		{
			i++;
		}
		RVCollectableInt *key = new RVCollectableInt(i);
		res = itemList.insertKeyAndValue(key,item);
		if(!res)
			return -1;
		else
			return i;
	}
	else
	{
		RVCollectableInt *key = new RVCollectableInt(domId);
		res = itemList.insertKeyAndValue(key,item);
		if(!res)
			return -1;
		else
			return domId;
	}
	

	//itemList.append((RVCollectable *)item);
	//return itemList.entries();
}


CWLDomItem *CWLExpectedDomRepository::get(int itemId)
{
	CWLDomItem *item;
	if(itemId >= 0)
	{
		RVCollectableInt *key = new RVCollectableInt(itemId);
		item = (CWLDomItem *)(itemList.findValue(key));
		delete key;
	}
	else
		item = NULL;
 
	return item;
	/*if( itemId <= int(itemList.entries()) && itemId > 0)
		return (CWLDomItem *)itemList[itemId-1];
	else
		return NULL;*/
}

bool CWLExpectedDomRepository::replace(int itemId, CWLDomItem *item)
{
	//CWLDomItem *old;
	RVCollectable *res;

	RVCollectableInt *tempKey = new RVCollectableInt(itemId);
	if(itemList.occurrencesOf(tempKey) > 0)
	//if( (old = get(itemId)) != NULL ) //! found
	{
		//itemList[itemId-1] = (RVCollectable *)item; //! replace
		itemList.removeAndDestroy(tempKey); //removes and deletes both the key and the value
		delete tempKey;
		RVCollectableInt *key = new RVCollectableInt(itemId);
		res = itemList.insertKeyAndValue(key,item);
		//delete old;  //! and despose the old one
		if(!res)
			return false;
		else
			return true;
	}
	else
	{
		delete tempKey;
		return false;
	}
}

bool CWLExpectedDomRepository::save(const char *name)
{
	CWLDomItem *item;
	RVHashDictionaryIterator* iterator;
	int itemId;
	unsigned int bigValue = getCurrnetVersion();


	if( (fp = fopen(name,"wb")) == NULL )
		return false;

	initDump();

	//save an integer bigger then 10000, so we'll know this is a new exr
	dumpInt32(bigValue); 

	size_t _entries = itemList.entries();
	//! save the number of entries in the repository
	dumpInt32(_entries);
	iterator = new RVHashDictionaryIterator(itemList);
	while(iterator->operator()())
	{
		RVCollectableInt* key = (RVCollectableInt*)iterator->key();
    itemId = (int)*key;
		dumpInt32(itemId);
		item = (CWLDomItem *)(iterator->value());
		item->dump(*this);  //! save the item
	}

  delete iterator;
	/*for(size_t i = 0 ; i < _entries; ++i)
	{
		item = (CWLDomItem *)itemList[i];
		item->dump(*this);  //! save the item
	}*/

  flush();
  fclose(fp);

  return true;

}


bool CWLExpectedDomRepository::open(const char *name)
{
	CWLDomItem *item;
  int numItems;
	int itemId;
	RVCollectable *res;

	if( (fp = fopen(name,"rb")) == NULL )
		return false;

	initUnDump();
	
	fillVersion(); //read just the first int from the file to the buffer
	//! save the number of entries in the repository
	numItems = unDumpInt32();
  
  // set Current Vession
  setCurrentVersion(numItems);

  if(numItems >= EXR_FIRST_VERSION)
	{
		m_bNewVersion = true;
		numItems = unDumpInt32();
	}
	else //open the file textually
	{
		if(fclose(fp))
			return false;
		if( (fp = fopen(name,"r")) == NULL )
			return false;
		if(fseek(fp,8L,SEEK_SET)) //we already read the first int
			return false;
	}

	for(int i = 0 ; i < numItems; ++i)
	{
    item = new CWLDomItem();
		if(m_bNewVersion)
			itemId = unDumpInt32();
		else
			itemId = i+1;
		if(itemId < 0 || itemId >= EXR_FIRST_VERSION) //error - wrong itemId
			return false;
    item->unDump(*this);
    //itemList.insert((RVCollectable *)item); //!!!!!!needs to be changed - the domId should  be read from the file
		RVCollectableInt *key = new RVCollectableInt(itemId);
		res = itemList.insertKeyAndValue(key,item);
		if(!res)
			return false;
	}

  fclose(fp);

  return true;
}

static int unix_fread(char *buff, FILE *fp)
{
  int len=0;
  char c, prev_c = '\r';

  while((len < STORAGE_LIMIT) && ((c = getc(fp))!=EOF)) {
    if ( c == '\n' && prev_c == '\r' && len > 0 )
      buff[len-1] = '\n';
    else {
      buff[len++] = c;
      prev_c = c;
    }
  }
  return len;
}

bool CWLExpectedDomRepository::fill()
{
  char buff[STORAGE_LIMIT+1];  //! temp
  int  len;

  if( fp == NULL )
    return false;

	
	if(m_bNewVersion) //the file was opened and written binary
		len = fread(buff, 1, STORAGE_LIMIT, fp);
	else
	{
		#ifndef __TV_UNIX__
			len = fread(buff, 1, STORAGE_LIMIT, fp);  //! not all requested bytes were read
		#else
			len = unix_fread(buff, fp);  //ignores one /r in sequences of /r/r/n caused by non-binary writting.
		#endif
	}

	if(len < STORAGE_LIMIT)  //! not all requested bytes were read
    if( feof(fp) ) //! if its EOF, close and return true
    {
      //! fclose(fp);
      buff[len] = '\0';
      fillStorage(buff);
      return true;
    }
    else //! error. close and return false
    {
      //! fclose(fp);
      return false;
    }
  else
  {
    buff[len] = '\0';
    fillStorage(buff);
    return true;
  }
}



bool CWLExpectedDomRepository::fillVersion()
{
  char buff[9];  //! temp
  int  len;

  if( fp == NULL )
    return false;

	len = fread(buff, 1, 8, fp);
	
	if(len < 8)  //! not all requested bytes were read
    if( feof(fp) ) //! if its EOF, close and return true
    {
      //! fclose(fp);
      buff[len] = '\0';
      fillStorage(buff);
      return true;
    }
    else //! error. close and return false
    {
      //! fclose(fp);
      return false;
    }
  else
  {
    buff[len] = '\0';
    fillStorage(buff);
    return true;
  }
}


void CWLExpectedDomRepository::flush()
{
  if( fp )
  {
    //fprintf(fp,"%s",storage());
		fwrite(storage(),1, buffLength(),fp);

    initDump();
  }

  return;
}


void CWLExpectedDomRepository::printAll()
{
	CWLDomItem *item;
	RVHashDictionaryIterator* iterator;

	size_t _entries = itemList.entries();
  //! print the number of entries in the repository
	cout << "#Entries in the repository: " << _entries << endl;

	iterator = new RVHashDictionaryIterator(itemList);
	
	while(iterator->operator()())
	{
		item = (CWLDomItem *)(iterator->value());
		item->printAll();  //! print the item
	}

	delete iterator;

	/*for(size_t i = 0 ; i < _entries; ++i)
	{
    cout << "-------------------------------------------\n";
    cout << " Item #" << i << endl;
    cout << "-------------------------------------------\n";
		item = (CWLDomItem *)itemList[i];
		item->printAll();  //! print the item
	}*/
}
