/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  


#include "rvjsgeneral.h"
#include "tracedll.h"

namespace nJs {


cContextThreadAcquire::cContextThreadAcquire( JSContext* rCx )
:
	mCx(0)
{
	if( !JS_GetContextThread(rCx) ) {
		mCx = rCx;
		JS_SetContextThread(mCx);
	}
/*
	{
		intN aX0 = JS_GetContextThread(mCx);
		if( aX0 ) {
			WlAssert(0);
		}
	}
//	WlAssert( !JS_GetContextThread(mCx) );
	JS_SetContextThread(mCx);
	{
		intN aX0 = JS_GetContextThread(mCx);
		if( !aX0 ) {
			WlAssert(0);
		}
	}
//	WlAssert( JS_GetContextThread(mCx) );
*/
}

cContextThreadAcquire::~cContextThreadAcquire()
{
	if(mCx) {
		JS_ClearContextThread(mCx);
	}
/*
	{
		intN aX0 = JS_GetContextThread(mCx);
		if( !aX0 ) {
			WlAssert(0);
		}
	}
//	WlAssert( JS_GetContextThread(mCx) );
	JS_ClearContextThread(mCx);
	{
		intN aX0 = JS_GetContextThread(mCx);
		if( aX0 ) {
			WlAssert(0);
		}
	}
//	WlAssert( !JS_GetContextThread(mCx) );
*/
}



cContextThreadRelease::cContextThreadRelease( JSContext* rCx )
:
	mCx(0)
{
	if( JS_GetContextThread(rCx) ) {
		mCx = rCx;
		JS_ClearContextThread(mCx);
	}
}

cContextThreadRelease::~cContextThreadRelease()
{
	if(mCx) {
		JS_SetContextThread(mCx);
	}
}







const char nZTSEncription::msCodeBytes[3] = { '~', '~', '^' };
const int nZTSEncription::msCodeLength = (sizeof(msCodeBytes)/sizeof(char));

int nZTSEncription::fsPutCode(
	char* ZTS ) 
{
	for( int I = 0; I<(sizeof(msCodeLength)/sizeof(char)); ++I ) {
		ZTS[I] = msCodeBytes[I];
	}
	return msCodeLength;
}

int nZTSEncription::fsCheckCode(
	const char* ZTS ) 
{
	int I = msCodeLength;
	if( ZTS ) {
		while( I-- ) {
			if( ZTS[I] != msCodeBytes[I] ) {
				return 0;
			}	
		}	
	} else {
		return 0;
	}
	return msCodeLength;
}


const int nZTSEncription::msPointerEncriptionMaxLength = 6 + nZTSEncription::msCodeLength;


void nZTSEncription::fsPointerToZTS(
	const void* Pointer,
	char* ZTS ) {

	int I = fsPutCode( ZTS );
	sprintf( &ZTS[I], "%p", Pointer );
	}


bool nZTSEncription::fsZTSToPointer(
	const char* ZTS,
	void*& Pointer ) {

	int I = fsCheckCode( ZTS );
	if(I) {
		sscanf( &ZTS[I], "%p", &Pointer );
		}
	return (bool)(I!=0);
	}


cFileRef::cFileRef()
:
	mFile(NULL)
{}


cFileRef::cFileRef(
	iFile& rFile )
:
	mFile( &rFile )
{
	rFile.fIncRef();
}

cFileRef::cFileRef(
	cFileRef& rFileRef )
:
	mFile( &rFileRef.fGet() )
{
	mFile->fIncRef();
}


cFileRef::~cFileRef()
{
	if(mFile) {
	mFile->fDecRef();
}
}

cFileRef& cFileRef::operator=(
	const cFileRef& r )
{
	iFile* aNew = r.mFile;
	iFile* aOld = mFile;

	if( aNew != aOld ) {
		if( aOld ) {
			aOld->fDecRef();
		}
		if( aNew ) {
			aNew->fIncRef();
		}
		mFile = aNew;
	}
	return *this;
}

iFile& cFileRef::fGet() const
{
	return *mFile;
}


iFile* ZTSEncriptionToFile(
	const char* rZTS ) 
{
	iFile* rtn = 0;
	if( rZTS ) {
		rtn = ecfsZTSToPointer< iFile >::_( rZTS );
	}
	return rtn;
}


iFile::iFile()
:
	mRefN(-1)
{}

iFile::~iFile()
{
//	WlAssert( ( 0 == mRefN ) || ( -1 == mRefN ) );
}

void iFile::fIncRef()
{
		if( 0 >= mRefN ) {
		vPick();
		mRefN = 1;
	} else {
		++mRefN;
	}
	WlAssert( mRefN>0 );
}

bool iFile::fDecRef()
{
	bool rtn = false;
	WlAssert( mRefN>0 );
	--mRefN;
	if( 0 == mRefN ) {
		rtn = vDrop();
	}
	return( rtn );
}

/*
void iFile::vPick()
{}

bool iFile::vDrop()
{
	delete this;
	return false;
}
*/

RVCString fsJSCodeString( const char* rCodeBytes, int rCodeLength )
{
	RVCString rtn;

	for( int aI = 0; aI < rCodeLength; ++aI ) {
		char aC = rCodeBytes[aI];
		switch( aC ) {
		case '\\':
			rtn += "\\\\";
			break;
		case '\n':
			rtn += "\\n";
			break;
		case '\r':
			rtn += "\\r";
			break;
		case '\"':
			rtn += "\\\"";
			break;
		case '\'':
			rtn += "\\\'";
			break;
		//case '\t':
		default:
			rtn += aC;
			break;
		}
	}

	return rtn;
}



void fsJSNewScriptHook(
	JSContext  *cx,
	const char *filename,
	uintN      lineno,     
	JSScript   *script,
	JSFunction *fun,
	void       *callerdata )
{
	cRuntime::fsNewScriptHook( cx, filename, lineno, script, fun );
}

void fsJSDestroyScriptHook(
	JSContext *cx, 
	JSScript  *script, 
	void      *callerdata)
{
	cRuntime::fsDestroyScriptHook( cx, script );
}


cRuntime::cRuntime():
mJSRuntime(NULL)
{
}

cRuntime::~cRuntime()
{}

void cRuntime::fInit(
	int rSize )
{
	WlAssert( !mJSRuntime );
	
	JSRuntime* aRuntime = JS_NewRuntime(rSize);

	JS_SetNewScriptHook( aRuntime, fsJSNewScriptHook, NULL );
	JS_SetDestroyScriptHook( aRuntime, fsJSDestroyScriptHook, NULL );

	mJSRuntime = aRuntime;
}

void cRuntime::fFinal()
{
	WlAssert( mJSRuntime );
	JS_DestroyRuntime( mJSRuntime );
}

JSRuntime* cRuntime::fGet()
{
	WlAssert( mJSRuntime );
	return mJSRuntime;
}


void cRuntime::fsNewScriptHook(
	JSContext* rCx,
	const char* rFilename,
	uintN rLineno,
	JSScript* rScript,
	JSFunction* rFun)
{
	const char* aFilename = JS_GetScriptFilename( rCx, rScript );
	iFile* aFile = ZTSEncriptionToFile( aFilename );

	if( aFile ) {
		aFile->fIncRef();
	}
}

void cRuntime::fsDestroyScriptHook(
	JSContext* rCx,
	JSScript* rScript)
{
	const char* aFilename = JS_GetScriptFilename( rCx, rScript );
	iFile* aFile = ZTSEncriptionToFile( aFilename );

	if( aFile ) {
		aFile->fDecRef();
	}
}




 	} // nJs


