/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "stdafx.h"
#include "HashDictionaries.h"
#include "cache.h"
#include <crtdbg.h>

RVDEFINE_COLLECTABLE(CObjectsHashDictionary, 102)
RVDEFINE_COLLECTABLE(CMethodsHashDictionary, 103)
RVDEFINE_COLLECTABLE(CMethodInfo, 100)
RVDEFINE_COLLECTABLE(CParamterInfo, 101)
RVDEFINE_COLLECTABLE(CObjectGUID, 102)
RVDEFINE_COLLECTABLE(CVARIANTArrayPointer, 104)


CObjectsHashDictionary *iJSComPrivateData::m_pObjectsCache  = new CObjectsHashDictionary();

CObjectsHashDictionary :: CObjectsHashDictionary() : RVHashDictionary(110){
}

CObjectsHashDictionary :: CObjectsHashDictionary(const CObjectsHashDictionary &obj) : RVHashDictionary(110){

}

CObjectsHashDictionary ::   ~CObjectsHashDictionary(){

	 clearAndDestroy();
}

//This function reads / writes the TypeInformation in multithreading mode
//First it checks weather the TypeInfo exists in the cache. 
//In case that it doesn't exist, it tries to write down it
//in to the cache.

//There are 2 Synchronization Events / Locks in the mechanisme of
//write/read cache.
//First  - 	RVReadersWriterLock m_CacheLock;
//Second - 	TVPManualEvent m_FillCacheEvent;

//m_CacheLock is resonsible to lock the external layer of the cache
//(GUID <--> TypeInfo pointer) when it is updated. It can function 
//in 2 states : 
// 1) multiple threads read at the same time
// 2) single thread write 
//
//m_FillCacheEvent doesn't permit to access the cache entry of the
//specific object when is's updated


void CObjectsHashDictionary :: GetTypeInfo(iJSComPrivateData *pCJSComObject){

	CMethodsHashDictionary *methods_cache_RW = NULL;
	IDispatch *ptrIDispatch_cache;

	//ptrIDispatch_cache = pCJSComObject->GetptrIDispatch_WithoutAddref();
	ptrIDispatch_cache = pCJSComObject->GetptrIDispatch();
	ptrIDispatch_cache->AddRef();


	int ret = iJSComPrivateData::m_pObjectsCache->ReadCache(ptrIDispatch_cache,
																										 &methods_cache_RW);
        if (2 == ret) {
          pCJSComObject->SetTypeInfoExists(FALSE);  //no TypeInfo. maybe we can get it at runtime.
        }

	if (ret != 0)
		ret = iJSComPrivateData::m_pObjectsCache->WriteCache(ptrIDispatch_cache,
																										&methods_cache_RW);
	
	if (ret == 0){
		pCJSComObject->Set_ptr_methods_cache(methods_cache_RW);
		pCJSComObject->SetTypeInfoExists(TRUE);
		methods_cache_RW->SetFillCacheEventWait();
	}

  pCJSComObject->TypeInfoWarning();
	ptrIDispatch_cache->Release();

}


//when return value is "0" - the appropriate object was found in the cache
//when return value is "1" - the appropriate object wasn't found in the cache
//when return value is "2" - we coundn't open the TypeInfo of the "searched" 
//object, i.e. the object doesn't exist in the cache and there is no chance
//in the future to get its TypeInfo (we will not try to fill the cache with it)

int CObjectsHashDictionary :: ReadCache(
																IDispatch *ptrIDispatch_cache,
																CMethodsHashDictionary **methods_cache_RW){

	RVCollectable *methods_cache_RWColl = NULL;

	// READ Lock start
	RVReadersWriterLock::ReadLockGuard reader(m_CacheLock);
//methods_cache_RWColl : WILL GET INFORMATION ABOUT THE GUID
	int res = FindObjectInCache(ptrIDispatch_cache,&methods_cache_RWColl);
	*methods_cache_RW = (CMethodsHashDictionary*)methods_cache_RWColl;
	
	return res;
}

int CObjectsHashDictionary :: WriteCache(
																 IDispatch *ptrIDispatch_cache,
																 CMethodsHashDictionary **methods_cache_RW)
{
	RVCollectable *methods_cache_RWColl = NULL;

	// WRITE Lock start - Set the lock for cache write only
	RVReadersWriterLock::WriteLockGuard writer(m_CacheLock);

	//checks wheather the cache wasn't updated (by other thread) 
	// since we last time tried to find our entry there
	int res = FindObjectInCache(ptrIDispatch_cache,&methods_cache_RWColl);
	
	if (res == 0){
		//there is no need anymore to wtite in the cache
		*methods_cache_RW = (CMethodsHashDictionary*)methods_cache_RWColl;
		return res;
	}

	CMethodsHashDictionary *pMethodsCache = new CMethodsHashDictionary;

	//Sets CacheEventReset (TVPManualEvent) so that nobody could read / write
	//to the inner cache contents of the specific object while it's updated
	pMethodsCache->SetFillCacheEventReset();

	//Fills only the external entry of the cache in order that the other
	//thread wouldn't try to fill the cache with the same object
	res = CreateCacheEntry(ptrIDispatch_cache,pMethodsCache);
	if (res != 0) {
		//there was a failure in writing the external cache entry
		pMethodsCache->SetFillCacheEventSet();
		return res;
	}

	//The Global CacheLock is released  -  all the cache are accessible  
	//"hutz me"  the entry of the updated object
	// 1-12-2003 changes by Yahali - fix to release writer and not the m_CacheLock
	//								 to avoid Assert when the writer guard is released.
	//m_CacheLock.release();
	writer.release();


	//Fills the cache entry with a TypeInfo
	res = FillCacheEntry(ptrIDispatch_cache,pMethodsCache);

	//Cleans the CacheEvent  - now all the cache entries are acessible
	pMethodsCache->SetFillCacheEventSet();

	if (res == 0){
		*methods_cache_RW = pMethodsCache;
		return 0;
	}
	else
		return 1;
}

int CObjectsHashDictionary :: CreateCacheEntry(
 																  IDispatch *ptrIDispatch_cache,
																	CMethodsHashDictionary *pMethodsCache){

	UINT pCountTypeInfo;
	HRESULT hr;
	ITypeInfo *pITypeInfo;
	TYPEATTR *pTypeAttr;

	hr = ptrIDispatch_cache->GetTypeInfoCount(&pCountTypeInfo);

 	if (FAILED(hr) || (pCountTypeInfo == 0))
  		return 1;


	ITypeInfo *pITypeInfo_old;

	hr = ptrIDispatch_cache->GetTypeInfo(0,0,&pITypeInfo_old);

 	if (FAILED(hr))
  		return 1;

	ITypeLib *ppTLib;
	unsigned int pIndex;

	hr = pITypeInfo_old->GetContainingTypeLib(&ppTLib,&pIndex);
	_ASSERT(hr == 0);

 	if (FAILED(hr))
  		return 1;


	hr = ppTLib->GetTypeInfo(pIndex,&pITypeInfo);
	_ASSERT(hr == 0);

	if (FAILED(hr))
  	return 1;


  hr = pITypeInfo->GetTypeAttr(&pTypeAttr);

 	if (FAILED(hr)){
  		return 1;
  }

  CObjectGUID default_interface_GUID = pTypeAttr->guid;

 //creates "dictionary" for the methods of this object

  CObjectGUID *tmp_obj_GUID = new  CObjectGUID(pTypeAttr->guid);

  RVCollectable*  retVal = insertKeyAndValue(tmp_obj_GUID,pMethodsCache);
	
	pITypeInfo->ReleaseTypeAttr(pTypeAttr);

	if (!retVal)
		return 1;
	else
		return 0;

}

int CObjectsHashDictionary :: FillCacheEntry(
																	 IDispatch *ptrIDispatch_cache,
																	 CMethodsHashDictionary *pmethods_cache_RW){

	UINT pCountTypeInfo;
	HRESULT hr;
	ITypeInfo *pITypeInfo;
	FUNCDESC *pfuncdesc;

  RVCollectableString* method_name_RW;
  CMethodInfo* method_params_RW;
  CParamterInfo *parameter_info_RW;


	hr = ptrIDispatch_cache->GetTypeInfoCount(&pCountTypeInfo);

 	if (FAILED(hr) || (pCountTypeInfo == 0))
  		return 1;

//************ patch begin 15.05.2001 ************************

//	hr = ptrIDispatch_cache->GetTypeInfo(0,0,&pITypeInfo);
//
// 	if (FAILED(hr))
//  		return 1;

	ITypeInfo *pITypeInfo_old;

	hr = ptrIDispatch_cache->GetTypeInfo(0,0,&pITypeInfo_old);

 	if (FAILED(hr))
  		return 1;

	ITypeLib *ppTLib;
	unsigned int pIndex;

	hr = pITypeInfo_old->GetContainingTypeLib(&ppTLib,&pIndex);
	_ASSERT(hr == 0);

	if (FAILED(hr))
  	return 1;


	hr = ppTLib->GetTypeInfo(pIndex,&pITypeInfo);
	_ASSERT(hr == 0);

 	if (FAILED(hr))
  		return 1;

//************ patch end 15.05.2001************************


/*

	HREFTYPE href = NULL ;
	ITypeInfo *pITypeInfo_inher;

	hr = pITypeInfo->GetRefTypeOfImplType(0,&href);

	hr = pITypeInfo->GetRefTypeInfo(href,&pITypeInfo_inher);
*/

	int count = 0;
	BOOL fill_prop_in_cache;
	int previous_invoke_type = 0; //for read / write properties
	
	while (SUCCEEDED(pITypeInfo->GetFuncDesc(count, &pfuncdesc))){ 

		bool methodStuffInsertedToCache = false;

    //We don't want to take standard IDispatch Methods, which 
    //have very "large" memid number
    if (((pfuncdesc->memid >= 1610612736) && (pfuncdesc->memid <= 1610612738)) ||
				((pfuncdesc->memid >= 1610678272) && (pfuncdesc->memid <= 1610678275))) {

			BSTR Method_Name_BSTR[50];
			UINT readNames;
			LPOLESTR temp_str_w;

			if (SUCCEEDED(pITypeInfo->GetNames( pfuncdesc->memid,Method_Name_BSTR, 9, &readNames))) {

			temp_str_w = (LPOLESTR)Method_Name_BSTR[0];
			USES_CONVERSION;

			//all the methods names are saved in low-case format
			char *temp_str = OLE2A(temp_str_w);
			    // Ofir Memory leaks
				for (int i=0; i < (int) readNames; i++)
			        SysFreeString(Method_Name_BSTR[i]);
			}

      count++;
	  // Ofir Memory leaks
	  pITypeInfo->ReleaseFuncDesc(pfuncdesc);
      continue;
    }



	BSTR Method_Name_BSTR[50];
	UINT readNames;
  LPOLESTR temp_str_w;

	fill_prop_in_cache = false;

//		array_param[pfuncdesc->memid].length = 0;

	if (SUCCEEDED(pITypeInfo->GetNames( pfuncdesc->memid,Method_Name_BSTR, 9, &readNames))){

		temp_str_w = (LPOLESTR)Method_Name_BSTR[0];
		USES_CONVERSION;

		//all the methods names are saved in low-case format
		char *temp_str = OLE2A(temp_str_w);
		char *temp_str_lcase = _strlwr(_strdup(temp_str));

		//Creates RW String with name of the method
		method_name_RW = new RVCollectableString(temp_str_lcase);
		free(temp_str_lcase);

		//constructor with only DISPID of the method
		method_params_RW = new CMethodInfo(pfuncdesc->memid);


		if (pfuncdesc->invkind != INVOKE_FUNC){

			FUNCDESC *pfuncdesc_next;

			if (SUCCEEDED(pITypeInfo->GetFuncDesc(count+1, &pfuncdesc_next))){

				//checks the weather the next memid is the same
				if (pfuncdesc->memid == pfuncdesc_next->memid) { 

					previous_invoke_type = pfuncdesc->invkind;

					//we don't need to fill this "get" memid  - 
					//we will fill in the next property (the "put" one)
					fill_prop_in_cache = false; 									
				}

				else {

					//previous_invoke_type != 0 in case when there was "get" memid
					//in the previous one
					method_params_RW->SetInvokeType(pfuncdesc->invkind + previous_invoke_type);
					fill_prop_in_cache = true;
				}
				// Ofir Memory Leak					
				pITypeInfo->ReleaseFuncDesc(pfuncdesc_next);
			}

			else {

				method_params_RW->SetInvokeType(pfuncdesc->invkind + previous_invoke_type);
				fill_prop_in_cache = true; // the next memid doesn't exist
			}

		}

		else 
			method_params_RW->SetInvokeType(pfuncdesc->invkind);


// As an IvokeType the following entries could exist in the cache :
// 1. The MEMID is function -> IvokeType == INVOKE_FUNC
// 2. There are Get and Put(or by ref Put) Methods for the same MEMID ->
//       IvokeType == INVOKE_PROPERTYPUT + INVOKE_PROPERTYGET
// 3. The MEMID is INVOKE_PROPERTYPUT -> IvokeType == INVOKE_PROPERTYPUT
// 4. The MEMID is INVOKE_PROPERTYGET -> IvokeType == INVOKE_PROPERTYGET


//	INVOKE_FUNC	= 1,
//	INVOKE_PROPERTYGET	= 2,
//	INVOKE_PROPERTYPUT	= 4,
//	INVOKE_PROPERTYPUTREF	= 8


		//Sets return type of the Method
		method_params_RW->SetTypeReturn(pfuncdesc->elemdescFunc.tdesc.vt);

      
  		if(pfuncdesc->cParams > 0){      

        int i=0;

  		  for(;;){

          //creating with default constructor CParamterInfo object
           parameter_info_RW = new CParamterInfo;

					 VARTYPE vt = pfuncdesc->lprgelemdescParam[i].tdesc.vt;

					 if(vt == VT_PTR){
             //the parameter is pointer
             parameter_info_RW->SetTypeIsPointer(true);
                			 
						 //We need to find out what type is being pointed to...
						 TYPEDESC * pPointedAt =  pfuncdesc->lprgelemdescParam[i].tdesc.lptdesc;

						 vt = pPointedAt->vt;

						 if (vt == VT_SAFEARRAY) {

							 TYPEDESC * pPointedAt1 =  pfuncdesc->lprgelemdescParam[i].tdesc.lptdesc->lptdesc;
							 //int vt111 = pPointedAt1->vt | vt;

							 vt = VT_ARRAY | pPointedAt1->vt;
						 }

//             parameter_info->SetType(VARENUM_types[vt]);						
           }

//					 else if (vt == VT_SAFEARRAY) {
					 
//            TYPEDESC * pPointedAt =  pfuncdesc->lprgelemdescParam[i].tdesc.lptdesc;
//            vt = pPointedAt->vt;

				

//              return 1;
//           }

					 else {
               parameter_info_RW->SetTypeIsPointer(false);
//               parameter_info->SetType(VARENUM_types[vt]);						
           }


           //int vt__ = FilterParamType(vt);
           parameter_info_RW->SetType(vt);		
//           parameter_info_RW->SetType(vt);		
           
           //sets wheather the paramter is optional
  				 if (pfuncdesc->cParams - i > pfuncdesc->cParamsOpt )
							parameter_info_RW->SetOptional(false);
					 else
							parameter_info_RW->SetOptional(true);

           method_params_RW->InsertParam(parameter_info_RW);
					 i++;

					 if(i >= pfuncdesc->cParams){

 						 method_params_RW->SetParamNumber(i);
             method_params_RW->SetParamNumberMinimum(i - pfuncdesc->cParamsOpt);
						 break;
					 }

        }//for(;;)

      }//if(pfuncdesc->cParams > 0)

      else {
        method_params_RW->SetParamNumber(0);
        method_params_RW->SetParamNumberMinimum(0);
      }

    }//if(SUCCEEDED...)

    for (int i=0; i < (int) readNames; i++)
       SysFreeString(Method_Name_BSTR[i]);

    
    //in case of a property there are 2 entries in pfuncdesc structure
    //in the first entery there are no params, so we don't insert
    //this entery as far as in RVHashDictionary the key is unique.
    //So, we omit this entery and inssert only the second one wich 
    //contains the param info.

//    if ((pfuncdesc->cParams > 0) || ((pfuncdesc->cParams == 0) && (pfuncdesc->invkind == INVOKE_FUNC))){
//    if ((last_memid == pfuncdesc->memid) || (pfuncdesc->invkind == INVOKE_FUNC)){
	  if (fill_prop_in_cache || (pfuncdesc->invkind == INVOKE_FUNC)){

      //RVCollectable *ret_val; //debug needs
      RVCollectable *ret_val = pmethods_cache_RW->insertKeyAndValue(method_name_RW, method_params_RW);
			methodStuffInsertedToCache = true;

			previous_invoke_type  = 0;

      if (pfuncdesc->memid == 0)
        pmethods_cache_RW->SetDefaultPropertyParam(method_params_RW);
      //int enter = m_pMethodsCache->entries();//debug needs
    }

    count++;

		if (!methodStuffInsertedToCache){
			delete method_name_RW;
			delete method_params_RW;
		}
 // Ofir Memory leaks
   pITypeInfo->ReleaseFuncDesc(pfuncdesc);
  }//while((SUCCEEDED...)


  //pmethods_cache_RW->applyToKeyAndValue(printKeyAndValue, rwnil); //debug needs
  //applyToKeyAndValue(printKeyAndValue1, rwnil); //debug needs

  return 0;
}

//IF TYPE INFO EXISTS. TAKE OUT THE GUID FROM TYPE INFO, AND PUT IT'S INFORMATION (IN THE FindValue() )IN *methods_cache_RWColl 
int CObjectsHashDictionary :: FindObjectInCache(
																	IDispatch *ptrIDispatch_cache,
																	RVCollectable **methods_cache_RWColl){

	UINT pCountTypeInfo;
	HRESULT hr;
	ITypeInfo *pITypeInfo;
	TYPEATTR *pTypeAttr;
	CMethodsHashDictionary *methods_cache_RW = NULL;

//	RVCollectable **methods_cache_RWColl_1;
	
	hr = ptrIDispatch_cache->GetTypeInfoCount(&pCountTypeInfo);

 	if (FAILED(hr) || (pCountTypeInfo == 0))
		return 2;
  
//************ patch begin 15.05.2001 ************************

//	hr = ptrIDispatch_cache->GetTypeInfo(0,0,&pITypeInfo);
//
// 	if (FAILED(hr))
//  		return 2;

	ITypeInfo *pITypeInfo_old;

	hr = ptrIDispatch_cache->GetTypeInfo(0,0,&pITypeInfo_old);

 	if (FAILED(hr))
  		return 2;

	ITypeLib *ppTLib;
	unsigned int pIndex;

	hr = pITypeInfo_old->GetContainingTypeLib(&ppTLib,&pIndex);
	_ASSERT(hr == 0);

	if (FAILED(hr))
  		return 2;


	hr = ppTLib->GetTypeInfo(pIndex,&pITypeInfo);
	_ASSERT(hr == 0);

	if (FAILED(hr))
  	return 2;

//************ patch end 15.05.2001************************
  
	hr = pITypeInfo->GetTypeAttr(&pTypeAttr);

 	if (FAILED(hr))
		return 2;
  
  CObjectGUID tmp_obj_GUID(pTypeAttr->guid);
	pITypeInfo->ReleaseTypeAttr(pTypeAttr);


	*methods_cache_RWColl = findValue(&tmp_obj_GUID);

	if (*methods_cache_RWColl == NULL){
		//cout<<"Not Found in Cache"<<endl;
		return 1;
	}
	else{
		//cout<<"Found in Cache"<<endl;
		return 0;
	}


/*  
  RVHashDictionaryIterator  iter(*this);

  iter.reset();
  const CObjectGUID* v;

  while( (v = (CObjectGUID*)iter()) != 0 ){

    if (v->isEqual(&tmp_obj_GUID)){
      //methods_cache_RW = (CMethodsHashDictionary*)iter.value();
			*methods_cache_RWColl_1 = iter.value();	
			return(0);
    }
  }
  return 1;
*/
}


CMethodsHashDictionary :: CMethodsHashDictionary() : RVHashDictionary(110){
  m_pDefaultPropertyParam = NULL;
}

CMethodsHashDictionary ::   ~CMethodsHashDictionary(){

	clearAndDestroy();
}

void CMethodsHashDictionary ::	SetFillCacheEventReset(){
	m_FillCacheEvent.Reset();
}

void CMethodsHashDictionary ::	SetFillCacheEventSet(){
	m_FillCacheEvent.Set();
}

void CMethodsHashDictionary ::	SetFillCacheEventWait(){
	m_FillCacheEvent.Wait();
}

void CMethodsHashDictionary :: SetDefaultPropertyParam
                               (CMethodInfo* pDefaultPropertyParam){

  m_pDefaultPropertyParam = pDefaultPropertyParam;
}

CMethodInfo* CMethodsHashDictionary :: GetDefaultPropertyParam(){

  return (m_pDefaultPropertyParam);
}


