/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLGZip.cpp: implementation of the CWLUnZip class.
//
//////////////////////////////////////////////////////////////////////


//#include "zipstream.hpp"
//#include <iostream>
//#include <string>
//#include <RVCString.h>
//#include <sstream>
//#include <fstream>
//#include <cmath>
//#include <cstdlib>
//


#include "tracedll.h"
#include "wlgzip.h"
#include "zlib.h"
#define BUFLEN      16384


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWLGZip::CWLGZip()
{

}

CWLGZip::~CWLGZip()
{

}


int CWLGZip::UNGzip(const RVCString &sInputBuff, RVCString &sOutBuff)
{

  gzFile in;
  const char *bf = sInputBuff.data();
  int len = sInputBuff.length();
  char buf[BUFLEN];
  sOutBuff.remove(0);
  int count =0;

  WlTraceMsg("CWLGZip::UNGzip Started");
  in=wlgz_prep_stream(bf,len);

      for (;;) {
          len = wlgzread(in, buf, sizeof(buf));
          if (len < 0)
          {
			  sOutBuff.remove(0);
                 WlTraceMsg("CWLGZip::UNGzip wlgzread FAILED");
                return -1;
	  }
          if (len == 0) break;

          sOutBuff.append((char*)buf, len);
      }

    if (wlgzclose(in) != Z_OK)
    {
		sOutBuff.remove(0);
              WlTraceMsg("CWLGZip::UNGzip wlgzclose FAILED");
                return -1;
    }

    WlTraceMsg("CWLGZip::UNGzip Ended - Out-buf size = %d", sOutBuff.length());

    return 0;

}
int CWLGZip::Inflate(const RVCString &sInputBuff, RVCString &sOutBuff)
{
      const char* ibuff = sInputBuff.data();
      long end = sInputBuff.length();
      char buf[BUFLEN];
      sOutBuff.remove(0);


      WlTraceMsg("CWLGZip::Inflate Started  -inSize = %d",end);
      
      z_stream d_stream;

      memset(&d_stream, 0, sizeof (d_stream));

    if (inflateInit(&d_stream) != Z_OK)
		     {
	                  WlTraceMsg("Inflate Init Problem ");
			     return (-1);
		     }

                   d_stream.next_in =(Bytef*) ibuff;
		   d_stream.avail_in = end;


		   bool mDummyStreamInitialised = false;

        for (;;)
       {
		d_stream.next_out =(Bytef*) buf;
		d_stream.avail_out = BUFLEN;

		int code = inflate(&d_stream, Z_NO_FLUSH);
		unsigned bytesWritten = BUFLEN - d_stream.avail_out;


              if (code == Z_STREAM_END)
	      {
		if (bytesWritten)
		{
			sOutBuff.append(buf,bytesWritten);
		}
		inflateEnd(&d_stream);
		break;
	      }
	      else if (code == Z_OK)
	      {
	         	if (bytesWritten)
		         {
			    sOutBuff.append(buf,bytesWritten);
			 }
	      }
	      else if (code == Z_BUF_ERROR)
	      {
	         	if (bytesWritten)
		         {
			    sOutBuff.append(buf,bytesWritten);
	                  WlTraceMsg("Inflate Z_BUF_ERROR");
			 }
			break;
	      }
	      else if (code == Z_DATA_ERROR)
	     {
		     static char dummy_head[2] =
			{
				0x8 + 0x7 * 0x10,
				(((0x8 + 0x7 * 0x10) * 0x100 + 30) / 31 * 31) & 0xFF,
			};



		     inflateReset(&d_stream);
		     d_stream.next_in = (Bytef*) dummy_head;
		     d_stream.avail_in = sizeof(dummy_head);

		     code = inflate(&d_stream, Z_NO_FLUSH);
		     if (code != Z_OK)
		     {
	                  WlTraceMsg("Inflate Dummy Header Problem");
	                  //fprintf(stderr,"Header Problem Loop ");
			     return (-1);
		     }

		     if (mDummyStreamInitialised) {
			     //endless loop
	                  WlTraceMsg("Inflate Dummy Header Endless Loop");
	                  //fprintf(stderr,"Endless Loop ");
			     return (-1);
		     }
		      
		     mDummyStreamInitialised = true;

		     d_stream.next_in = (Bytef*)ibuff;
		     d_stream.avail_in = end;

             }
       }//for

        WlTraceMsg("CWLGZip::Inflate Ended - Outbuf size = %d", sOutBuff.length());
    return 0;


}

