/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadreportinfo.h"
#include "wlxmlfuncs.h"
#include "wlxmlprivate.h"
#include "wldomerrorhandler.h"
#include <xercesc/util/XMLUniDefs.hpp>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/dom/DOMWriter.hpp>

#define CLASSNAME DOMNode

DOMNode* node_GetThisObject(JSContext *p_pCX, JSObject *p_pObj);

XERCES_CPP_NAMESPACE_USE

JSBool WLXMLNodeEnumerate(JSContext *cx, JSObject *obj );


static JSClass wl_XMLNodeClass = {
	"XMLNodeObject",
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	WLXMLNodeEnumerate,//JS_EnumerateStub
	JS_ResolveStub,
	JS_ConvertStub,
	WLXmlGenericFinalize
};




JSBool WLXMLNodeEnumerate(JSContext *cx, JSObject *obj )
{
  //enumerate class is implemented for: (for a in xmlparse), it can be implemented later.
  DOMNode* pNode = node_GetThisObject(cx, obj);
  
  if (pNode==NULL)
    return JS_TRUE;;

	return JS_TRUE;
}


static JSBool
node_getNodeType(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {
    short nNodeType = pNode->getNodeType();
    WLXmlSetIntValue(cx,obj,nNodeType, rval);
  }
	return JS_TRUE;
}

METHOD_RET_XMLCHAR_NO_ARGS(node, getNodeName)
METHOD_RET_XMLCHAR_NO_ARGS(node, getNodeValue);
METHOD_RET_OBJECT_NO_ARGS(node, getParentNode, Node, false);
METHOD_RET_OBJECT_NO_ARGS(node, getFirstChild, Node, false);
METHOD_RET_OBJECT_NO_ARGS(node, getLastChild, Node, false);
METHOD_RET_OBJECT_NO_ARGS(node, getPreviousSibling, Node, false);
METHOD_RET_OBJECT_NO_ARGS(node, getNextSibling, Node, false);
METHOD_RET_OBJECT_NO_ARGS(node, getChildNodes, NodeList, false);
METHOD_RET_OBJECT_NO_ARGS(node, getAttributes, NamedNodeMap, false);
METHOD_RET_OBJECT_NO_ARGS(node, getOwnerDocument, Document, false);
METHOD_RET_XMLCHAR_NO_ARGS(node, getNamespaceURI);
METHOD_RET_XMLCHAR_NO_ARGS(node, getPrefix);
METHOD_RET_XMLCHAR_NO_ARGS(node, getLocalName);

static JSBool
node_hasChildNodes(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {
    bool bHas = pNode->hasChildNodes();
    WLXmlSetBoolValue(cx,obj,bHas, rval);
  }
	return JS_TRUE;
}

static JSBool
node_hasAttributes(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {
    bool bHas = pNode->hasAttributes();
    WLXmlSetBoolValue(cx,obj,bHas, rval);
  }
	return JS_TRUE;
}

METHOD_NO_RET_NO_ARGS(node,normalize)
METHOD_NO_RET_NO_ARGS(node,release)

static JSBool
node_removeChild(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {
    XML_GET_OBJECT_ARGUMENT_CHECK(oldNode, argv, 0, DOMNode, "node_removeChild");
    DOMNode* pRemovedNode = 0;
    TRY_XML_PARSER_STANDARD
      pRemovedNode = pNode->removeChild(oldNode);//should be equal to Node1
    CATCH_XML_PARSER_STANDARD(node,removeChild)
    *rval = WLCreateNodeObject(cx, obj, pRemovedNode, true);
  }
	return JS_TRUE;
}

static JSBool
node_appendChild(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNode* pNode = node_GetThisObject(cx, obj);
  DOMNode* appendedNode = 0;
  if (pNode)
  {
    XML_GET_OBJECT_ARGUMENT_CHECK(newNode, argv, 0, DOMNode, "node_appendChild");

    TRY_XML_PARSER_STANDARD
      appendedNode = pNode->appendChild(newNode);//should be equal to Node1
      //CWLXmlParserPrivate::setPrivateToDelete(cx, JSVAL_TO_OBJECT(argv[0]), false);
    CATCH_XML_PARSER_STANDARD(node,appendChild)
      
    *rval = WLCreateNodeObject(cx, obj, appendedNode, false);
  }
	return JS_TRUE;
}

static JSBool
node_insertBefore(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {
    XML_GET_OBJECT_ARGUMENT_CHECK(newChild, argv, 0, DOMNode, "node_insertBefore");
    XML_GET_OBJECT_ARGUMENT_CHECK(oldChild, argv, 1, DOMNode, "node_insertBefore");
    DOMNode* insertedChild =0;

    TRY_XML_PARSER_STANDARD
      insertedChild = pNode->insertBefore(newChild, oldChild);
      //CWLXmlParserPrivate::setPrivateToDelete(cx, JSVAL_TO_OBJECT(argv[0]), false);
    CATCH_XML_PARSER_STANDARD(node,insertBefore)
    *rval = WLCreateNodeObject(cx, obj, insertedChild, false);
  }
	return JS_TRUE;
}

static JSBool
node_replaceChild(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  //xerces have memory leak in clone node function.
  //they do not enable us to delete the node
  //we return a pointer to the same node
  *rval = OBJECT_TO_JSVAL(obj);

 	WLErrorException e( XMLPARSER_ERROR_FILE, 40, "replaceChild" );
	return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );

  /*
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {
    XML_GET_OBJECT_ARGUMENT_CHECK(newChild, argv, 0, DOMNode, "node_replaceChild");
    XML_GET_OBJECT_ARGUMENT_CHECK(oldChild, argv, 1, DOMNode, "node_replaceChild");
    DOMNode* replacedChild =0;
    TRY_XML_PARSER_STANDARD
      replacedChild = pNode->replaceChild(newChild, oldChild);
      //CWLXmlParserPrivate::setPrivateToDelete(cx, JSVAL_TO_OBJECT(argv[0]), false);
    CATCH_XML_PARSER_STANDARD(node,replaceChild)
    *rval = WLCreateNodeObject(cx, obj, replacedChild, true);
  }
	return JS_TRUE;
  */
}

static JSBool
node_cloneNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  //xerces have memory leak in clone node function.
  //they do not enable us to delete the node
  //we return a pointer to the same node
  *rval = OBJECT_TO_JSVAL(obj);

 	WLErrorException e( XMLPARSER_ERROR_FILE, 40, "cloneNode" );
	return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );

  /*
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {
    XML_GET_BOOL_ARGUMENT_CHECK(bDeep, argv, 0, "node_cloneNode");
    DOMNode* clonedNode =0;
    TRY_XML_PARSER_STANDARD
       clonedNode = pNode->cloneNode(bDeep);//should be equal to Node1
       clonedNode->normalize();
    CATCH_XML_PARSER_STANDARD(node,cloneNode)
    *rval = WLCreateNodeObject(cx, obj, clonedNode, true);
  }
	return JS_TRUE;
  */
}

static JSBool
node_setNodeValue(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {  
    XML_GET_XMLCHAR_ARGUMENT_CHECK(nodeValue, argv, 0, "node_setNodeValue")
    TRY_XML_PARSER_STANDARD
      if (nodeValue)
       pNode->setNodeValue(nodeValue);
    CATCH_XML_PARSER_STANDARD(node,setNodeValue)
  }  
	return JS_TRUE;
}

static JSBool
node_setPrefix(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {  
    XML_GET_XMLCHAR_ARGUMENT_CHECK(prefix, argv, 0, "node_setPrefix")
    TRY_XML_PARSER_STANDARD
      if (prefix)
        pNode->setPrefix(prefix);
    CATCH_XML_PARSER_STANDARD(node,setPrefix)
  }  
	return JS_TRUE;
}

static JSBool
node_isSupported(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {  
    XML_GET_XMLCHAR_ARGUMENT_CHECK(feature, argv, 0, "node_isSupported");
    XML_GET_XMLCHAR_ARGUMENT_CHECK(version, argv, 1, "node_isSupported");
    bool bIsSupported = pNode->isSupported(feature, version);
    WLXmlSetBoolValue(cx, obj, bIsSupported, rval);
  }  
	return JS_TRUE;
}

static JSBool
node_getNodeTypeString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {  
    XMLCh* ntStr = XMLString::transcode(GetNodeTypeString(pNode));
    WLXmlSetXMLChValue(cx,obj,ntStr,rval);
    XMLString::release(&ntStr);
  }
  return JS_TRUE;
  
}



static JSFunctionSpec xmlnode_methods[] = {
  {"getNodeName",   node_getNodeName, 0},
  {"getNodeValue",  node_getNodeValue, 0},
  {"getNodeType",   node_getNodeType, 0},
  {"getParentNode", node_getParentNode, 0},
  {"getFirstChild", node_getFirstChild, 0},
  {"getLastChild",  node_getLastChild, 0},
  {"getPreviousSibling", node_getPreviousSibling, 0},
  {"getNextSibling",  node_getNextSibling, 0},
  {"getChildNodes", node_getChildNodes, 0},
  {"getAttributes",   node_getAttributes, 0},
  {"getOwnerDocument",node_getOwnerDocument, 0},
  {"getNamespaceURI", node_getNamespaceURI, 0},
  {"getPrefix",     node_getPrefix, 0},
  {"getLocalName",  node_getLocalName, 0},
  {"hasChildNodes", node_hasChildNodes, 0},
  {"hasAttributes", node_hasAttributes, 0},
  {"normalize",     node_normalize, 0},
  {"release",       node_release, 0},
  {"removeChild",   node_removeChild, 1},
  {"appendChild",   node_appendChild, 1},
  {"insertBefore",  node_insertBefore, 2},
  {"replaceChild",  node_replaceChild, 2},
  {"cloneNode",     node_cloneNode, 1},
  {"setNodeValue",  node_setNodeValue, 1},
  {"setPrefix",     node_setPrefix, 1},
  {"isSupported",   node_isSupported, 2},
	{0}
};


static JSBool
node_prop_getNodeTypeString(JSContext *cx, JSObject *obj, jsval id, jsval *rval)
{
  
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {  
    XMLCh* ntStr = XMLString::transcode(GetNodeTypeString(pNode));
    WLXmlSetXMLChValue(cx,obj,ntStr,rval);
    XMLString::release(&ntStr);
  }
  return JS_TRUE;
  
}

static JSBool
node_prop_getXML(JSContext *cx, JSObject *obj, jsval id, jsval *rval)
{
  
  DOMNode* pNode = node_GetThisObject(cx, obj);
  if (pNode)
  {  
    static const XMLCh gLS[] = { chLatin_L, chLatin_S, chNull };
    DOMImplementation *impl = DOMImplementationRegistry::getDOMImplementation(gLS);
    DOMWriter* writer = ((DOMImplementationLS*)impl)->createDOMWriter();
    
    WLXMLDOMErrorHandler *errHandler = new WLXMLDOMErrorHandler();
    writer->setErrorHandler(errHandler);
    
    XMLCh* xmlStr = writer->writeToString(*pNode);
    WLXmlSetXMLChValue(cx,obj,xmlStr,rval);
    XMLString::release(&xmlStr); 
    writer->release();
  }
  return JS_TRUE;
}

PROPERTY_SETTER_FOR_STRING(node,nodeValue,setNodeValue)

PROPERTY_GETTER_RET_XMLCHAR(node,nodeName,getNodeName)
PROPERTY_GETTER_RET_XMLCHAR(node,namespaceURI,getNamespaceURI)
PROPERTY_GETTER_RET_XMLCHAR(node,nodeValue,getNodeValue)
PROPERTY_GETTER_RET_XMLCHAR(node,prefix,getPrefix)
PROPERTY_GETTER_RET_INT(node,nodeType,getNodeType)
PROPERTY_GETTER_RET_OBJECT(node,attributes,getAttributes,NamedNodeMap,false)
PROPERTY_GETTER_RET_OBJECT(node,childNodes,getChildNodes,NodeList,false)
PROPERTY_GETTER_RET_OBJECT(node,firstChild,getFirstChild,Node,false)
PROPERTY_GETTER_RET_OBJECT(node,lastChild,getLastChild,Node,false)
PROPERTY_GETTER_RET_OBJECT(node,nextSibling,getNextSibling,Node,false)
PROPERTY_GETTER_RET_OBJECT(node,ownerDocument,getOwnerDocument,Node,false)
PROPERTY_GETTER_RET_OBJECT(node,parentNode,getParentNode,Node,false)
PROPERTY_GETTER_RET_OBJECT(node,previousSibling,getPreviousSibling,Node,false)


static JSPropertySpec xmlnode_properties[] = {
  { "nodeName"       , 0, READONLY_PROPERTY_FLAGS         , node_prop_getNodeName             , JS_PropertyStub },
  { "attributes"     , 0, READONLY_PROPERTY_FLAGS         , node_prop_getAttributes           , JS_PropertyStub },
  { "childNodes"     , 0, READONLY_PROPERTY_FLAGS         , node_prop_getChildNodes           , JS_PropertyStub },
  { "firstChild"     , 0, READONLY_PROPERTY_FLAGS         , node_prop_getFirstChild           , JS_PropertyStub },
  { "lastChild"      , 0, READONLY_PROPERTY_FLAGS         , node_prop_getLastChild            , JS_PropertyStub },
  { "namespaceURI"   , 0, READONLY_PROPERTY_FLAGS         , node_prop_getNamespaceURI         , JS_PropertyStub },
  { "nextSibling"    , 0, READONLY_PROPERTY_FLAGS         , node_prop_getNextSibling          , JS_PropertyStub },
  { "nodeType"       , 0, READONLY_PROPERTY_FLAGS         , node_prop_getNodeType             , JS_PropertyStub },
  { "nodeValue"      , 0, PROPERTY_FLAGS                  , node_prop_getNodeValue            , node_prop_setNodeValue    },
  { "ownerDocument"  , 0, READONLY_PROPERTY_FLAGS         , node_prop_getOwnerDocument        , JS_PropertyStub },
  { "parentNode"     , 0, READONLY_PROPERTY_FLAGS         , node_prop_getParentNode           , JS_PropertyStub },
  { "prefix"         , 0, READONLY_PROPERTY_FLAGS         , node_prop_getPrefix               , JS_PropertyStub },
  { "previousSibling", 0, READONLY_PROPERTY_FLAGS         , node_prop_getPreviousSibling      , JS_PropertyStub },
  { "nodeTypeString" , 0, READONLY_PROPERTY_FLAGS         , node_prop_getNodeTypeString       , JS_PropertyStub },
  { "xml"            , 0, READONLY_PROPERTY_FLAGS         , node_prop_getXML                  , JS_PropertyStub },
	{0}
};



//create new Node object
jsval WLCreateNodeObject(JSContext* cx, JSObject* obj, void* value, bool toDelete)
{
  if (!value)
    return JSVAL_NULL;

  CJSGCRequestBeginEnd GCGuard(cx); //lock GC for this action
	CWLXmlParserPrivate* prv = new CWLXmlParserPrivate((void*)value, CWLXmlParserPrivate::NODE, toDelete);
  JSObject* newObj = JS_NewObject(cx, &wl_XMLNodeClass, NULL, NULL);
	JS_SetPrivate(cx, newObj, (void*)prv);
  return OBJECT_TO_JSVAL(newObj);
}

//when we'll define all classes
JSObject* rvAddXMLNodeClass(JSContext *cx, JSObject *obj, JSObject* parent_proto)
{
  return JS_InitClass(cx, obj, parent_proto, &wl_XMLNodeClass, NULL, 0, xmlnode_properties, xmlnode_methods, NULL, NULL);
}

DOMNode* node_GetThisObject(JSContext *p_pCX, JSObject *p_pObj)
{
  CJSGCRequestBeginEnd GCGuard(p_pCX); //lock GC for this action
	CWLXmlParserPrivate* pPrv = (CWLXmlParserPrivate*)JS_GetPrivate( p_pCX, p_pObj );
	if ( !pPrv )
		return NULL;

  switch (pPrv->m_eClass)
  {
  case CWLXmlParserPrivate::DOCUMENT: return (DOMDocument*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::DOCUMENT_FRAGMENT:return (DOMDocumentFragment*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::NODE:return (DOMNode*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::CHARACTERDATA:return (DOMCharacterData*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::ATTRIBUTE:return (DOMAttr*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::ELEMENT:return (DOMElement*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::TEXT:return (DOMText*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::COMMENT:return (DOMComment*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::CDATA_SECTION:return (DOMCDATASection*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::DOCUMENT_TYPE:return (DOMDocumentType*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::NOTATION:return (DOMNotation*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::ENTITY:return (DOMEntity*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::ENTITY_REFERENCE:return (DOMEntityReference*)pPrv->GetCPPObj();
  case CWLXmlParserPrivate::PROCESSING_INSTRUCTION:return (DOMProcessingInstruction*)pPrv->GetCPPObj();
  default:WlTraceMsg("node_GetThisObject default class, type = %d",pPrv->m_eClass); assert(0);
  };
  return NULL;
  
  //PARSER_OBJECT,
  //NODELIST, 
  //NAMEDNODEMAP, 
}



