/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadreportinfo.h"
#include "wlxmlfuncs.h"
#include "wlxmlprivate.h"
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOMElement.hpp>

XERCES_CPP_NAMESPACE_USE

#define CLASSNAME DOMNodeList
DEFAULT_GET_THIS_OBJECT(nodelist)

JSBool WLXMLNodeListEnumerate(JSContext *cx, JSObject *obj );
static JSBool WLXMLNodeList_vcall(JSContext *cx, JSObject *rOwner, uintN argc, jsval *argv, jsval *rval);
static JSBool WLXMLNodeList_GetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *rval);


static JSClass wl_XMLNodeListClass = {
	"XMLNodeListObject",
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	WLXMLNodeList_GetProperty,
	JS_PropertyStub,
	WLXMLNodeListEnumerate,//JS_EnumerateStub
	JS_ResolveStub,
	JS_ConvertStub,
	WLXmlGenericFinalize,
  NULL,
  NULL,
  WLXMLNodeList_vcall,
  NULL,
  NULL,
  NULL,
  NULL,
  0
};




JSBool WLXMLNodeListEnumerate(JSContext *cx, JSObject *obj )
{
  //enumerate class is implemented for: (for a in xmlparse), it can be implemented later.
  DOMNodeList* pNodeList = nodelist_GetThisObject(cx, obj);
  
  if (pNodeList==NULL)
    return JS_TRUE;;

	return JS_TRUE;
}


static JSBool
nodelist_item(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNodeList* pNodeList = nodelist_GetThisObject(cx, obj);
  
  if (pNodeList)
  {
    XML_GET_XMLSIZE_ARGUMENT_CHECK(index, argv, 0, "nodelist_item")
    DOMNode* pNode =0;
    TRY_XML_PARSER_STANDARD
      pNode = pNodeList->item(index);
    CATCH_XML_PARSER_STANDARD(nodelist,item)
    CreateNodeByType(cx, obj, pNode, rval, false);  
  }
	return JS_TRUE;
}

JSBool WLXMLNodeList_GetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *rval)
{
  if (JSVAL_IS_INT(id)){
    return nodelist_item(cx , obj, 1, &id, rval);
  }
  else {
    return JS_TRUE;
  }
}

JSBool WLXMLNodeList_vcall(JSContext *cx, JSObject *rOwner, uintN argc, jsval *argv, jsval *rval) 
{
  if (argc != 1 && argc != 2)
  {
    return JS_TRUE;
  }
  JSObject *obj = JSVAL_TO_OBJECT( argv[-2] );
  return nodelist_item(cx , obj, argc, argv, rval);
}

static JSBool
nodelist_getLength(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNodeList* pNodeList = nodelist_GetThisObject(cx, obj);
  if (pNodeList)
  {
    XMLSize_t uLength = pNodeList->getLength();
    WLXmlSetIntValue(cx,obj,uLength, rval);
  }
	return JS_TRUE;
}

static JSFunctionSpec xmlnodelist_methods[] = {
  {"item",   nodelist_item, 1},
  {"getLength",   nodelist_getLength, 0},
	{0}
};

PROPERTY_GETTER_RET_INT(nodelist,length,getLength)

static JSPropertySpec xmlnodelist_properties[] = {
  { "length" , 0, READONLY_PROPERTY_FLAGS , nodelist_prop_getLength , JS_PropertyStub },
	{0}
};



//create new NodeList object
jsval WLCreateNodeListObject(JSContext* cx, JSObject* obj, void* value, bool toDelete)
{
  if (!value)
    return JSVAL_NULL;

  CJSGCRequestBeginEnd GCGuard(cx); //lock GC for this action
	CWLXmlParserPrivate* prv = new CWLXmlParserPrivate((void*)value, CWLXmlParserPrivate::NODELIST, toDelete);
  JSObject* newObj = JS_NewObject(cx, &wl_XMLNodeListClass, NULL, NULL);
	JS_SetPrivate(cx, newObj, (void*)prv);
  return OBJECT_TO_JSVAL(newObj);
}

//when we'll define all classes
JSObject* rvAddXMLNodeListClass(JSContext *cx, JSObject *obj, JSObject* parent_proto)
{
  return JS_InitClass(cx, obj, parent_proto, &wl_XMLNodeListClass, NULL, 0, xmlnodelist_properties, xmlnodelist_methods, NULL, NULL);
}

