/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "rvjsapi.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadreportinfo.h"
#include "wlxmlfuncs.h"
#include "wlxmlprivate.h"
#include "wldomerrorhandler.h"
//#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/util/XMLUniDefs.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/sax/ErrorHandler.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/dom/DOMBuilder.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>
#include <xercesc/framework/Wrapper4InputSource.hpp>


XERCES_CPP_NAMESPACE_USE

#define CLASSNAME DOMBuilder
DEFAULT_GET_THIS_OBJECT(parser);

JSBool WLXMLParserEnumerate(JSContext *cx, JSObject *obj );


static JSClass wl_XMLParserClass = {
	"XMLParserObject",
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	WLXMLParserEnumerate,//JS_EnumerateStub
	JS_ResolveStub,
	JS_ConvertStub,
	WLXmlGenericFinalize
};




JSBool WLXMLParserEnumerate(JSContext *cx, JSObject *obj )
{
  //enumerate class is implemented for: (for a in xmlparse), it can be implemented later.
  //change the class name to the xmlparser's class
  //xmlparserclass* ptr = (xmlparserclass*)CWLXmlParserPrivate::GetThisObject(cx, obj);

	return JS_TRUE;
}



//example function
static JSBool
parser_parseURI(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

 	if (argc != 1 || !JSVAL_IS_STRING(argv[0]))
  {
    WlTraceMsg("parser_parseURI failed - argc=%d",argc);
   	WLErrorException e( XMLPARSER_ERROR_FILE, 22, "parseURI", argc +1 );
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  DOMBuilder* parser = parser_GetThisObject(cx, obj);

  DOMDocument* doc = 0;

  if (parser)
  {
    //@@ yossi - check this argument
    const char* pcURI = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
    TRY_XML_PARSER_STANDARD
      doc = parser->parseURI(pcURI);
    CATCH_XML_PARSER_STANDARD(parser,parseURI)

    *rval = WLCreateDocumentObject(cx, obj, doc, true);//ptr_doc
  }
	return JS_TRUE;
}

static JSBool
parser_parse(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

 	if (argc != 1 || !JSVAL_IS_STRING(argv[0]))
  {
    WlTraceMsg("parser_parse failed - argc=%d",argc);
   	WLErrorException e( XMLPARSER_ERROR_FILE, 22, "parse", argc +1 );
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  DOMBuilder* parser = parser_GetThisObject(cx, obj);

  if (NULL==parser)
    return JS_TRUE;

  static const char*  gMemBufId = "xmlDoc";
  const char* gXMLInMemBuf = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));

  MemBufInputSource* memBufIS = new MemBufInputSource
    (
        (const XMLByte*)gXMLInMemBuf
        , strlen(gXMLInMemBuf)
        , gMemBufId
        , false
    );
  DOMInputSource* domIn = new Wrapper4InputSource(memBufIS);
  DOMDocument* doc = 0;

  TRY_XML_PARSER_STANDARD
    
    doc = parser->parse(*domIn);
  
  CATCH_XML_PARSER_STANDARD(parser,parse)
 

  *rval = WLCreateDocumentObject(cx, obj, doc, true);

  delete domIn;

	return JS_TRUE;
}


METHOD_NO_RET_NO_ARGS(parser, resetDocumentPool)
METHOD_NO_RET_NO_ARGS(parser, release)
/*
virtual void DOMBuilder::setFeature  (  const XMLCh *const   name,  
  const bool  state
  */

static JSBool
parser_setFeature(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

  DOMBuilder* parser = parser_GetThisObject(cx, obj);
  if (parser)
  {
      XML_GET_XMLCHAR_ARGUMENT_CHECK( fName, argv, 0, "parser_setFeature")
      XML_GET_BOOL_ARGUMENT_CHECK( state, argv, 1, "parser_setFeature");
      TRY_XML_PARSER_STANDARD
        parser->setFeature(fName,state);
      CATCH_XML_PARSER_STANDARD(parser,setFeature)
  }

 return JS_TRUE;
}

static JSBool
parser_getFeature(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

  DOMBuilder* parser = parser_GetThisObject(cx, obj);
  if (parser)
  {
      XML_GET_XMLCHAR_ARGUMENT_CHECK( fName, argv, 0, "parser_getFeature")
      bool fState = false;
      TRY_XML_PARSER_STANDARD
        fState = parser->getFeature(fName);
      CATCH_XML_PARSER_STANDARD(parser,getFeature)
      WLXmlSetBoolValue(cx, obj, fState, rval);
  }

 return JS_TRUE;
}

static JSBool
parser_canSetFeature(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

  DOMBuilder* parser = parser_GetThisObject(cx, obj);
  if (parser)
  {
      XML_GET_XMLCHAR_ARGUMENT_CHECK( fName, argv, 0, "parser_canSetFeature")
      XML_GET_BOOL_ARGUMENT_CHECK( state, argv, 1, "parser_canSetFeature");
      bool fState = false;
      TRY_XML_PARSER_STANDARD
        fState = parser->canSetFeature(fName, state);
      CATCH_XML_PARSER_STANDARD(parser,canSetFeature)
      WLXmlSetBoolValue(cx, obj, fState, rval);
  }

 return JS_TRUE;
}


static JSFunctionSpec xmlparser_methods[] = {
  {"parseURI",		parser_parseURI,		1},
  {"parse",		parser_parse,		1},
  {"resetDocumentPool", parser_resetDocumentPool,0},
  {"release", parser_release,0},
  {"setFeature", parser_setFeature,2},
  {"getFeature", parser_getFeature ,1},
  {"canSetFeature", parser_canSetFeature,2},
  {"load",		parser_parseURI,		1},
  {"loadXML",		parser_parse,		1},
  
	{0}
};


static JSPropertySpec xmlparser_properties[] = {
	{0}
};


jsval WLCreateParserObject(JSContext* cx, JSObject* obj, void* value, bool toDelete)
{
  if (!value)
    return JSVAL_NULL;

  CJSGCRequestBeginEnd GCGuard(cx); //lock GC for this action
	CWLXmlParserPrivate* prv = new CWLXmlParserPrivate((void*)value, CWLXmlParserPrivate::PARSER_OBJECT, toDelete);
  JSObject* newObj = JS_NewObject(cx, &wl_XMLParserClass, NULL, NULL);
	JS_SetPrivate(cx, newObj, (void*)prv);
  return OBJECT_TO_JSVAL(newObj);
}


static JSBool
WLXMLParserCtor( JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval )
{
  static const XMLCh gLS[] = { chLatin_L, chLatin_S, chNull };
  DOMImplementation *impl = DOMImplementationRegistry::getDOMImplementation(gLS);
  DOMBuilder* parser = ((DOMImplementationLS*)impl)->createDOMBuilder(DOMImplementationLS::MODE_SYNCHRONOUS, 0);

  WLXMLDOMErrorHandler *errHandler = new WLXMLDOMErrorHandler();
  parser->setErrorHandler(errHandler);
  parser->setFeature(XMLUni::fgXercesUserAdoptsDOMDocument, true);
  parser->setFeature(XMLUni::fgDOMWhitespaceInElementContent, false);
  parser->setProperty(XMLUni::fgXercesScannerName,(void*)XMLUni::fgWFXMLScanner);

  parser->setFeature(XMLUni::fgDOMIgnoreEmptyTextNodes, true);//our flag

  *rval = WLCreateParserObject(cx, obj, parser, true);

  return JS_TRUE;
}


//when we'll define all classes
JSObject* rvAddXMLParserClass(JSContext *cx, JSObject *obj, JSObject* parent_proto)
{
  return JS_InitClass(cx, obj, parent_proto, &wl_XMLParserClass, WLXMLParserCtor, 0, xmlparser_properties, xmlparser_methods, NULL, NULL);
}




