/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "wlxmlprivate.h"
#include "wljsobjects.h"
#include "tracedll.h"
#include <xercesc/dom/DOM.hpp>
#include <xercesc/dom/impl/DOMNodeImpl.hpp>
#include <xercesc/dom/impl/DOMDocumentImpl.hpp>
#include <xercesc/dom/impl/DOMCasts.hpp>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/dom/DOMBuilder.hpp>

XERCES_CPP_NAMESPACE_USE

/**********************************************************************************
    Function : CWLXmlParserPrivate C'tor
    Input    : pRealObject - pointer to CPP DOM object, 
               eClass - class type of DOM object
               eBase - base type of DOMobject
               bDelete - Delete the DOM on the D'tor or not. if the object was created outside 
               of the DOM (new Node), we are responsible to delete it, until it is appended to 
               the DOM, and then the parser is responsible to delete it.
    Output   : 
**********************************************************************************/
//ptr is an interface wrapper
CWLXmlParserPrivate::CWLXmlParserPrivate(void* pRealObject, CWLXmlParserPrivate::eClass eClass, bool bDelete)
{
	m_CPP_RealObject  = pRealObject;
  m_eClass          = eClass;
  m_bDelete         = bDelete;
}


CWLXmlParserPrivate::~CWLXmlParserPrivate()
{
  //deleting nodes that Xerces is not responsible for.
  if (m_bDelete)
  {
    switch (m_eClass)
    {
    //delete all objects that inherit from DOMNode class
    case NODE:
    case DOCUMENT_FRAGMENT:
    case CHARACTERDATA:
    case ATTRIBUTE:
    case ELEMENT:
    case TEXT:
    case COMMENT:
    case CDATA_SECTION:
    case DOCUMENT_TYPE:
    case NOTATION: 
    case ENTITY:
    case ENTITY_REFERENCE:
    case PROCESSING_INSTRUCTION:
    {
      DOMNode* node_ptr = (DOMNode*)( m_CPP_RealObject);
      WlTraceMsg("**Delete Node, Type: %d",m_eClass);
      DOMDocumentImpl* d = (DOMDocumentImpl*)node_ptr->getOwnerDocument();
      node_ptr->release();
//      d->clearRecyclePools();
  //    d->deleteFromHeap(m_CPP_RealObject);
      

//      delete node_ptr;
      //DOMNodeImpl *p = castToNodeImpl(node_ptr);
      //p->~DOMNodeImpl();
      //delete p;
      //d->clearRecyclePools();
      //delete node_ptr;

      break;
    };

    case DOCUMENT:
      {
        DOMDocument* node_ptr = (DOMDocument*)m_CPP_RealObject;
        //node_ptr->release();
        WlTraceMsg("*Delete document");
        delete node_ptr;
        break;
      };

    

    case NODELIST:
    {
      delete (DOMNodeList*)m_CPP_RealObject;
      break;
    }

    case NAMEDNODEMAP:
    {
      delete (DOMNamedNodeMap*)m_CPP_RealObject;
      break;
    }

    case PARSER_OBJECT:
    {
      //@@yossi - check what we have to add in D'tor
      DOMBuilder* pParser = (DOMBuilder*)m_CPP_RealObject;
      WlTraceMsg("***Delete Parser");
      pParser->resetDocumentPool();
      //pParser->release();
      delete pParser;    
    }
    default:
      {
        WlTraceMsg("***Default");
      }

    };
  }
}

void* CWLXmlParserPrivate::GetCPPObj()
{
	return m_CPP_RealObject;
}


void CWLXmlParserPrivate::setToDelete(bool bDelete)
{
  m_bDelete = bDelete;
}


void CWLXmlParserPrivate::setPrivateToDelete(JSContext *p_pCX, JSObject *p_pObj, bool toDelete)
{
  CJSGCRequestBeginEnd GCGuard(p_pCX); //lock GC for this action
	CWLXmlParserPrivate* pPrv = (CWLXmlParserPrivate*)JS_GetPrivate( p_pCX, p_pObj );
	
  if (pPrv)
    pPrv->setToDelete(toDelete);
}

//static function return CPPobject from JSObject
void* CWLXmlParserPrivate::GetThisObject(JSContext *p_pCX, JSObject *p_pObj)
{
  CJSGCRequestBeginEnd GCGuard(p_pCX); //lock GC for this action
	CWLXmlParserPrivate* pPrv = (CWLXmlParserPrivate*)JS_GetPrivate( p_pCX, p_pObj );
	if ( !pPrv )
		return NULL;

	return pPrv->GetCPPObj();
}

CWLXmlParserPrivate::eClass GetObjectEClass(JSContext *p_pCX, JSObject *p_pObj)
{
  CJSGCRequestBeginEnd GCGuard(p_pCX); //lock GC for this action
	CWLXmlParserPrivate* pPrv = (CWLXmlParserPrivate*)JS_GetPrivate( p_pCX, p_pObj );
	if ( !pPrv )
    return CWLXmlParserPrivate::NULL_OBJECT;

	return pPrv->m_eClass;
}

const char* CWLXmlParserPrivate::GetObjectEClassStr(JSContext *p_pCX, JSObject *p_pObj)
{
  CJSGCRequestBeginEnd GCGuard(p_pCX); //lock GC for this action
	CWLXmlParserPrivate* pPrv = (CWLXmlParserPrivate*)JS_GetPrivate( p_pCX, p_pObj );
	if ( !pPrv )
    return "UndefinedObject";

  switch(pPrv->m_eClass)
  {
  case CWLXmlParserPrivate::PARSER_OBJECT:
    return "Parser";
    break;
  case CWLXmlParserPrivate::DOCUMENT:
    return "Document";
    break;
  case CWLXmlParserPrivate::DOCUMENT_FRAGMENT:
    return "DocumentFragmentNode";
    break;
  case CWLXmlParserPrivate::NODE:
    return "Node";
    break;
  case CWLXmlParserPrivate::NODELIST:
    return "NodeList";
    break;
  case CWLXmlParserPrivate::NAMEDNODEMAP:
    return "NamedNodeMap";
    break;
  case CWLXmlParserPrivate::CHARACTERDATA:
    return "CharacterData";
    break;
  case CWLXmlParserPrivate::ATTRIBUTE:
    return "Attribute";
    break;
  case CWLXmlParserPrivate::ELEMENT:
    return "Element";
    break;
  case CWLXmlParserPrivate::TEXT:
    return "TextNode";
    break;
  case CWLXmlParserPrivate::COMMENT:
    return "CommentNode";
    break;
  case CWLXmlParserPrivate::CDATA_SECTION:
    return "CDataSection";
    break;
  case CWLXmlParserPrivate::DOCUMENT_TYPE:
    return "DocumentType";
    break;
  case CWLXmlParserPrivate::NOTATION:
    return "NotationNode";
    break;
  case CWLXmlParserPrivate::ENTITY:
    return "EntityNode";
    break;
  case CWLXmlParserPrivate::ENTITY_REFERENCE:
    return "EntityReference";
    break;
  case CWLXmlParserPrivate::PROCESSING_INSTRUCTION:
    return "ProcessingInstruction";
    break;
  case CWLXmlParserPrivate::NULL_OBJECT:
  default:
    return "Undefined";
  };
}


